Package: spicy
Title: Descriptive Statistics, Summary Tables, and Data Management
        Tools
Version: 0.12.0
Authors@R: 
    person("Amal", "Tawfik", , "amal.tawfik@hesav.ch", role = c("aut", "cre", "cph"), comment = c(ORCID = "0009-0006-2422-1555", ROR = "04j47fz63"))
Description: Provides tabulation, descriptive-summary, and
    variable-inspection tools for applied data analysis. Frequency
    tables and cross-tabulations with contingency-table association
    measures (Cramer's V, Phi, Goodman-Kruskal Gamma, Kendall's
    Tau-b, Somers' D, and others); categorical and continuous
    summary tables; regression coefficient tables for one or more
    'lm' or 'glm' fits side by side; and outcome-by-group
    comparison tables from linear models with optional additive
    covariate adjustment. All table outputs follow APA conventions
    and expose 'broom'-compatible 'tidy()' / 'glance()' methods
    for downstream pipelines. Helpers cover interactive codebooks,
    variable-label extraction, clipboard export, and row-wise
    descriptive summaries.
License: MIT + file LICENSE
URL: https://github.com/amaltawfik/spicy/,
        https://amaltawfik.github.io/spicy/
BugReports: https://github.com/amaltawfik/spicy/issues
Encoding: UTF-8
Language: en-US
Imports: crayon, dplyr, labelled, rlang (>= 1.1.0), sandwich, stats,
        stringr, tibble, tidyselect, utils
Suggests: broom, clipr, clubSandwich, DT, effectsize, emmeans,
        flextable, gt, haven, htmltools, knitr, marginaleffects,
        officer, openxlsx2, parameters, performance, rmarkdown,
        spelling, testthat (>= 3.0.0), tinytable, withr
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
Config/testthat/edition: 3
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-19 14:24:28 UTC; at
Author: Amal Tawfik [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0006-2422-1555>, ROR:
    <https://ror.org/04j47fz63>)
Maintainer: Amal Tawfik <amal.tawfik@hesav.ch>
Repository: CRAN
Date/Publication: 2026-05-19 21:40:02 UTC
Built: R 4.5.2; ; 2026-05-20 00:00:36 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(base::as.data.frame,spicy_categorical_table)
S3method(base::as.data.frame,spicy_continuous_lm_table)
S3method(base::as.data.frame,spicy_continuous_table)
S3method(base::as.data.frame,spicy_regression_table)
S3method(broom::glance,spicy_categorical_table)
S3method(broom::glance,spicy_continuous_lm_table)
S3method(broom::glance,spicy_continuous_table)
S3method(broom::glance,spicy_regression_table)
S3method(broom::tidy,spicy_categorical_table)
S3method(broom::tidy,spicy_continuous_lm_table)
S3method(broom::tidy,spicy_continuous_table)
S3method(broom::tidy,spicy_regression_table)
S3method(knitr::knit_print,spicy_flextable)
S3method(knitr::knit_print,spicy_gt)
S3method(print,spicy_assoc_detail)
S3method(print,spicy_assoc_table)
S3method(print,spicy_categorical_table)
S3method(print,spicy_continuous_lm_table)
S3method(print,spicy_continuous_table)
S3method(print,spicy_cross_table)
S3method(print,spicy_cross_table_list)
S3method(print,spicy_flextable)
S3method(print,spicy_freq_table)
S3method(print,spicy_gt)
S3method(print,spicy_regression_table)
S3method(tibble::as_tibble,spicy_categorical_table)
S3method(tibble::as_tibble,spicy_continuous_lm_table)
S3method(tibble::as_tibble,spicy_continuous_table)
S3method(tibble::as_tibble,spicy_regression_table)
export(as_structured)
export(assoc_measures)
export(build_ascii_table)
export(code_book)
export(contingency_coef)
export(copy_clipboard)
export(count_n)
export(cramer_v)
export(cross_tab)
export(freq)
export(gamma_gk)
export(goodman_kruskal_tau)
export(kendall_tau_b)
export(kendall_tau_c)
export(label_from_names)
export(lambda_gk)
export(mean_n)
export(phi)
export(somers_d)
export(spicy_print_table)
export(sum_n)
export(table_categorical)
export(table_continuous)
export(table_continuous_lm)
export(table_regression)
export(uncertainty_coef)
export(varlist)
export(vl)
export(yule_q)
importFrom(rlang,`%||%`)
importFrom(stats,complete.cases)
importFrom(stats,setNames)

# spicy: frequency tables, cross-tabulations, and summary tables in R <a href="https://amaltawfik.github.io/spicy/"><img src="man/figures/logo.png" align="left" height="139" alt="spicy website" /></a>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version-ago/spicy)](https://CRAN.R-project.org/package=spicy)
[![r-universe](https://amaltawfik.r-universe.dev/badges/spicy)](https://amaltawfik.r-universe.dev/spicy)
[![R-CMD-check](https://github.com/amaltawfik/spicy/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/amaltawfik/spicy/actions/workflows/R-CMD-check.yaml)
[![R-hub](https://github.com/amaltawfik/spicy/actions/workflows/rhub.yaml/badge.svg)](https://github.com/amaltawfik/spicy/actions/workflows/rhub.yaml)
[![Codecov](https://codecov.io/gh/amaltawfik/spicy/branch/main/graph/badge.svg)](https://app.codecov.io/gh/amaltawfik/spicy)
[![Project Status:
Active](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![MIT
License](https://img.shields.io/badge/license-MIT-blue.svg?style=flat)](https://opensource.org/licenses/MIT)
[![DOI](https://img.shields.io/badge/DOI-10.32614%2FCRAN.package.spicy-blue)](https://doi.org/10.32614/CRAN.package.spicy)
[![CRAN
downloads](https://cranlogs.r-pkg.org/badges/grand-total/spicy)](https://cranlogs.r-pkg.org/badges/grand-total/spicy)
<!-- badges: end -->

spicy is an R package for frequency tables, cross-tabulations,
association measures, summary tables, and labelled survey data
workflows.

## What is spicy?

spicy helps you explore categorical, continuous, and labelled survey
data in R. It provides readable, console-first outputs for survey
research, descriptive statistics, and reporting workflows, including
frequency tables, cross-tabulations with chi-squared tests and effect
sizes, categorical and continuous summary tables, variable inspection,
and codebooks.

With spicy, you can:

- **Inspect variables** with `varlist()` and `vl()` for names, labels,
  values, classes, and missing data.
- **Create frequency tables in R** with `freq()`.
- **Create cross-tabulations in R** with `cross_tab()`, including
  percentages, chi-squared tests, and effect sizes.
- **Measure associations** with `cramer_v()`, `phi()`, `gamma_gk()`,
  `kendall_tau_b()`, `somers_d()` and 6 other coefficients; use
  `assoc_measures()` to compute the full set at once.
- **Build categorical summary tables in R** with `table_categorical()`
  for console, gt, tinytable, flextable, Excel, Word, or clipboard
  output.
- **Build continuous summary tables in R** with `table_continuous()` for
  console, gt, tinytable, flextable, Excel, Word, or clipboard output.
- **Build model-based continuous summary tables in R** with
  `table_continuous_lm()` for linear regression reporting: classical /
  HC\* / cluster-robust / bootstrap / jackknife variance, four
  effect-size families (f², Cohen’s d, Hedges’ g, Hays’ omega²) with
  noncentral CIs, optional **additive covariate adjustment** with
  G-computation (Stata `margins` style) or equal-weight (`emmeans`
  style) marginal means, weighted comparisons, and console, gt,
  tinytable, flextable, Excel, Word, or clipboard output.
- **Build regression coefficient tables in R** with `table_regression()`
  for one or more `lm` or `glm` fits side by side: classical / HC\* /
  cluster-robust / bootstrap / jackknife variance, four standardisation
  methods (plus `pseudo` for `glm`), family-aware `exponentiate` (OR /
  IRR / HR / RR / MR), Wald or profile-likelihood CIs, average marginal
  effects on the response scale, partial *f²* / *η²* / *ω²* / *χ²*
  effect sizes, pseudo-*R²* family, hierarchical model comparisons,
  multiple-comparison adjustment, and the same console / gt / tinytable
  / flextable / Excel / Word / clipboard output.
- **Generate interactive and exportable codebooks** with `code_book()`
  for labelled and survey-style datasets.
- **Extract variable labels** with `label_from_names()`, including
  LimeSurvey-style headers.

Works with `labelled`, `factor`, `ordered`, `Date`, `POSIXct`, and other
common variable types. For a full introduction, see [Getting started
with spicy](https://amaltawfik.github.io/spicy/articles/spicy.html).

## Installation

Install the current CRAN release, recommended for most users:

``` r
install.packages("spicy")
```

Install the latest [r-universe](https://amaltawfik.r-universe.dev/spicy)
build:

``` r
install.packages(
  "spicy",
  repos = c(
    "https://amaltawfik.r-universe.dev",
    "https://cloud.r-project.org"
  )
)
```

This installs `spicy` from r-universe when available; CRAN is included
only as a fallback for dependencies. The r-universe build may be newer
than the current CRAN release.

Install the development version from GitHub with `pak`:

``` r
# install.packages("pak")
pak::pak("amaltawfik/spicy")
```

------------------------------------------------------------------------

## Quick tour

The examples below use the bundled `sochealth` dataset.

### Inspect variables

<img src="man/figures/animation_varlist.gif" alt="varlist demo with labelled data" width="100%">

``` r
varlist(sochealth, tbl = TRUE)
#> # A tibble: 24 × 7
#>    Variable          Label                 Values Class N_distinct N_valid   NAs
#>    <chr>             <chr>                 <chr>  <chr>      <int>   <int> <int>
#>  1 sex               Sex                   Femal… fact…          2    1200     0
#>  2 age               Age (years)           25, 2… nume…         51    1200     0
#>  3 age_group         Age group             25-34… orde…          4    1200     0
#>  4 education         Highest education le… Lower… orde…          3    1200     0
#>  5 social_class      Subjective social cl… Lower… orde…          5    1200     0
#>  6 region            Region of residence   Centr… fact…          6    1200     0
#>  7 employment_status Employment status     Emplo… fact…          4    1200     0
#>  8 income_group      Household income gro… Low, … orde…          4    1182    18
#>  9 income            Monthly household in… 1000,… nume…       1052    1200     0
#> 10 smoking           Current smoker        No, Y… fact…          2    1175    25
#> # ℹ 14 more rows
```

``` r
code_book(
  sochealth,
  starts_with("bmi"),
  values = TRUE,
  include_na = TRUE
)
```

See [Explore variables and build codebooks in
R](https://amaltawfik.github.io/spicy/articles/variable-exploration.html)
for more on `varlist()`, `vl()`, and `code_book()`.

### Frequency tables and cross-tabulations

``` r
freq(sochealth, income_group)
#> Frequency table: income_group
#> 
#>  Category   │ Values            Freq.    Percent    Valid Percent 
#> ────────────┼─────────────────────────────────────────────────────
#>  Valid      │ Low                 247       20.6             20.9 
#>             │ Lower middle        388       32.3             32.8 
#>             │ Upper middle        328       27.3             27.7 
#>             │ High                219       18.2             18.5 
#>  Missing    │ NA                   18        1.5                  
#> ────────────┼─────────────────────────────────────────────────────
#>  Total      │                    1200      100.0            100.0 
#> 
#> Label: Household income group
#> Class: ordered, factor
#> Data: sochealth

cross_tab(sochealth, smoking, education, percent = "col")
#> Crosstable: smoking x education (Column %)
#> 
#>  Values   │   Lower secondary    Upper secondary    Tertiary │   Total 
#> ──────────┼──────────────────────────────────────────────────┼─────────
#>  No       │              69.6               78.7        84.9 │    78.8 
#>  Yes      │              30.4               21.3        15.1 │    21.2 
#> ──────────┼──────────────────────────────────────────────────┼─────────
#>  Total    │             100.0              100.0       100.0 │   100.0 
#>  N        │               257                527         391 │    1175 
#> 
#> Chi-2(2) = 21.6, p <.001
#> Cramer's V = 0.14
```

See [Frequency tables and cross-tabulations in
R](https://amaltawfik.github.io/spicy/articles/frequency-tables.html)
for `freq()`, `cross_tab()`, percentages, weights, and tests.

### Association measures

``` r
tbl <- xtabs(~ self_rated_health + education, data = sochealth)

# Quick scalar estimate
cramer_v(tbl)
#> [1] 0.1761697

# Detailed result with CI and p-value
cramer_v(tbl, detail = TRUE)
#> Estimate  CI lower  CI upper      p
#>    0.176     0.120     0.231  <.001
```

See [Cramer’s V, Phi, and association measures in
R](https://amaltawfik.github.io/spicy/articles/association-measures.html)
for a guide on choosing the right measure.

### Summary tables

``` r
table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  labels = c(
    smoking           = "Current smoker",
    physical_activity = "Physical activity"
  )
)
#> Categorical table
#> 
#>  Variable            │   n      %    
#> ─────────────────────┼───────────────
#>  Current smoker      │               
#>    No                │  926    78.8  
#>    Yes               │  249    21.2  
#> ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
#>  Physical activity   │               
#>    No                │  650    54.2  
#>    Yes               │  550    45.8
```

``` r
table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  by = education,
  labels = c(
    smoking           = "Current smoker",
    physical_activity = "Physical activity"
  )
)
#> Categorical table by education
#> 
#>  Variable          │ Lower secondary n  Lower secondary %  Upper secondary n 
#> ───────────────────┼─────────────────────────────────────────────────────────
#>  Current smoker    │                                                         
#>    No              │        179               69.6                415        
#>    Yes             │         78               30.4                112        
#> ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
#>  Physical activity │                                                         
#>    No              │        177               67.8                310        
#>    Yes             │         84               32.2                229        
#> 
#>  Variable          │ Upper secondary %  Tertiary n  Tertiary %  Total n 
#> ───────────────────┼────────────────────────────────────────────────────
#>  Current smoker    │                                                    
#>    No              │       78.7            332         84.9       926   
#>    Yes             │       21.3             59         15.1       249   
#> ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
#>  Physical activity │                                                    
#>    No              │       57.5            163         40.8       650   
#>    Yes             │       42.5            237         59.2       550   
#> 
#>  Variable          │ Total %    p    Cramer's V 
#> ───────────────────┼────────────────────────────
#>  Current smoker    │          <.001     .14     
#>    No              │  78.8                      
#>    Yes             │  21.2                      
#> ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
#>  Physical activity │          <.001     .21     
#>    No              │  54.2                      
#>    Yes             │  45.8
```

``` r
table_continuous(
  sochealth,
  select = c(bmi, life_sat_health)
)
#> Descriptive statistics
#> 
#>  Variable                       │   M     SD    Min    Max   95% CI LL 
#> ────────────────────────────────┼──────────────────────────────────────
#>  Body mass index                │ 25.93  3.72  16.00  38.90    25.72   
#>  Satisfaction with health (1-5) │  3.55  1.25   1.00   5.00     3.48   
#> 
#>  Variable                       │ 95% CI UL   n   
#> ────────────────────────────────┼─────────────────
#>  Body mass index                │   26.14    1188 
#>  Satisfaction with health (1-5) │    3.62    1192
```

``` r
table_continuous(
  sochealth,
  select = c(bmi, life_sat_health),
  by = education
)
#> Descriptive statistics
#> 
#>  Variable                       │ Group              M     SD    Min    Max  
#> ────────────────────────────────┼────────────────────────────────────────────
#>  Body mass index                │ Lower secondary  28.09  3.47  18.20  38.90 
#>                                 │ Upper secondary  26.02  3.43  16.00  37.10 
#>                                 │ Tertiary         24.39  3.52  16.00  33.00 
#> ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
#>  Satisfaction with health (1-5) │ Lower secondary   2.71  1.20   1.00   5.00 
#>                                 │ Upper secondary   3.53  1.19   1.00   5.00 
#>                                 │ Tertiary          4.11  1.04   1.00   5.00 
#> 
#>  Variable                       │ Group            95% CI LL  95% CI UL   n  
#> ────────────────────────────────┼────────────────────────────────────────────
#>  Body mass index                │ Lower secondary    27.66      28.51    260 
#>                                 │ Upper secondary    25.73      26.31    534 
#>                                 │ Tertiary           24.04      24.74    394 
#> ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
#>  Satisfaction with health (1-5) │ Lower secondary     2.57       2.86    259 
#>                                 │ Upper secondary     3.43       3.63    534 
#>                                 │ Tertiary            4.01       4.21    399 
#> 
#>  Variable                       │ Group              p   
#> ────────────────────────────────┼────────────────────────
#>  Body mass index                │ Lower secondary  <.001 
#>                                 │ Upper secondary        
#>                                 │ Tertiary               
#> ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
#>  Satisfaction with health (1-5) │ Lower secondary  <.001 
#>                                 │ Upper secondary        
#>                                 │ Tertiary
```

``` r
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  vcov = "HC3",
  output = "data.frame"
)
#>                                      Variable M (Female) M (Male)
#> wellbeing_score WHO-5 wellbeing index (0-100)   67.16194 71.04879
#> bmi                           Body mass index   25.68506 26.19685
#>                 Δ (Male - Female)  95% CI LL 95% CI UL            p          R²
#> wellbeing_score         3.8868576 2.12265210 5.6510631 1.670572e-05 0.015475137
#> bmi                     0.5117882 0.08904596 0.9345305 1.769614e-02 0.004728908
#>                    n
#> wellbeing_score 1200
#> bmi             1188
```

``` r
fit <- lm(wellbeing_score ~ age + sex + smoking, data = sochealth)
table_regression(fit)
#> Linear regression: wellbeing_score
#> 
#>  Variable        │    B      SE       95% CI        p   
#> ─────────────────┼──────────────────────────────────────
#>  (Intercept)     │   65.20  1.66  [61.95, 68.45]  <.001 
#>  age             │    0.05  0.03  [-0.01,  0.11]   .130 
#>  sex:            │                                      
#>    Female (ref.) │     —     —          —          —    
#>    Male          │    3.86  0.91  [ 2.08,  5.63]  <.001 
#>  smoking:        │                                      
#>    No (ref.)     │     —     —          —          —    
#>    Yes           │   -1.72  1.11  [-3.89,  0.45]   .121 
#> ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
#>  n               │ 1175                                 
#>  R²              │    0.02                              
#>  Adj.R²          │    0.02                              
#> 
#> Note. Linear regression.
#> Std. errors: classical (OLS).
```

See [Categorical summary tables in
R](https://amaltawfik.github.io/spicy/articles/table-categorical.html)
for categorical summaries, [Continuous summary tables in
R](https://amaltawfik.github.io/spicy/articles/table-continuous.html)
for continuous summaries and group comparisons, [Model-based continuous
summary tables in
R](https://amaltawfik.github.io/spicy/articles/table-continuous-lm.html)
for weighted or robust linear-model reporting, [Regression coefficient
tables in
R](https://amaltawfik.github.io/spicy/articles/table-regression.html)
for `lm` and `glm` coefficient tables, and [Summary tables for APA-style
reporting](https://amaltawfik.github.io/spicy/articles/summary-tables-reporting.html)
for an overview of summary tables.

### Row-wise summaries

``` r
df <- data.frame(
  x1 = c(10, NA, 30, 40, 50),
  x2 = c(5, NA, 15, NA, 25),
  x3 = c(NA, 30, 20, 50, 10)
)

mean_n(df)
#> [1]       NA       NA 21.66667       NA 28.33333
sum_n(df, min_valid = 2)
#> [1] 15 NA 65 90 85
count_n(df, special = "NA")
#> [1] 1 2 0 1 0
```

See [Getting started with
spicy](https://amaltawfik.github.io/spicy/articles/spicy.html) for a
longer workflow using `mean_n()`, `sum_n()`, and `count_n()`.

### Label extraction

``` r
# LimeSurvey-style headers: "code. label"
df <- tibble::tibble(
  "age. Age of respondent" = c(25, 30),
  "score. Total score" = c(12, 14)
)
out <- label_from_names(df)
labelled::var_label(out)
#> $age
#> [1] "Age of respondent"
#> 
#> $score
#> [1] "Total score"
```

See [Explore variables and build codebooks in
R](https://amaltawfik.github.io/spicy/articles/variable-exploration.html)
for more on `label_from_names()`, `varlist()`, and `code_book()`.

------------------------------------------------------------------------

## Learn by task

If you are looking for a specific workflow, start with these vignettes:

- [Getting started with
  spicy](https://amaltawfik.github.io/spicy/articles/spicy.html)
- [Explore variables and build codebooks in
  R](https://amaltawfik.github.io/spicy/articles/variable-exploration.html)
- [Frequency tables and cross-tabulations in
  R](https://amaltawfik.github.io/spicy/articles/frequency-tables.html)
- [Cramer’s V, Phi, and association measures in
  R](https://amaltawfik.github.io/spicy/articles/association-measures.html)
- [Categorical summary tables in
  R](https://amaltawfik.github.io/spicy/articles/table-categorical.html)
- [Continuous summary tables in
  R](https://amaltawfik.github.io/spicy/articles/table-continuous.html)
- [Model-based continuous summary tables in
  R](https://amaltawfik.github.io/spicy/articles/table-continuous-lm.html)
- [Regression coefficient tables in
  R](https://amaltawfik.github.io/spicy/articles/table-regression.html)
- [Summary tables for APA-style
  reporting](https://amaltawfik.github.io/spicy/articles/summary-tables-reporting.html)

Key reference pages:

- [Reference for
  `varlist()`](https://amaltawfik.github.io/spicy/reference/varlist.html)
- [Reference for
  `code_book()`](https://amaltawfik.github.io/spicy/reference/code_book.html)
- [Reference for
  `label_from_names()`](https://amaltawfik.github.io/spicy/reference/label_from_names.html)
- [Reference for
  `freq()`](https://amaltawfik.github.io/spicy/reference/freq.html)
- [Reference for
  `cross_tab()`](https://amaltawfik.github.io/spicy/reference/cross_tab.html)
- [Reference for
  `cramer_v()`](https://amaltawfik.github.io/spicy/reference/cramer_v.html)
- [Reference for
  `table_categorical()`](https://amaltawfik.github.io/spicy/reference/table_categorical.html)
- [Reference for
  `table_continuous()`](https://amaltawfik.github.io/spicy/reference/table_continuous.html)
- [Reference for
  `table_continuous_lm()`](https://amaltawfik.github.io/spicy/reference/table_continuous_lm.html)
- [Reference for
  `table_regression()`](https://amaltawfik.github.io/spicy/reference/table_regression.html)
- [Reference for
  `mean_n()`](https://amaltawfik.github.io/spicy/reference/mean_n.html)
- [Reference for
  `sum_n()`](https://amaltawfik.github.io/spicy/reference/sum_n.html)
- [Reference for
  `count_n()`](https://amaltawfik.github.io/spicy/reference/count_n.html)

------------------------------------------------------------------------

## Citation

To cite spicy in a publication or teaching material:

- Use `citation("spicy")` to generate the current BibTeX entry.
- Package DOI: <https://doi.org/10.32614/CRAN.package.spicy>.
- Source citation file:
  <https://github.com/amaltawfik/spicy/blob/main/inst/CITATION>

------------------------------------------------------------------------

## License

MIT. See [`LICENSE`](LICENSE) for details.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

build_rich_tables <- identical(Sys.getenv("IN_PKGDOWN"), "true")

source("_pkgdown-helpers.R")

## ----setup--------------------------------------------------------------------
library(spicy)

## ----basic--------------------------------------------------------------------
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health)
)

## ----default-select-----------------------------------------------------------
table_continuous(sochealth)

## ----grouped------------------------------------------------------------------
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education
)

## ----lm-companion-------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education,
  vcov = "HC3"
)

## ----pvalue-effect------------------------------------------------------------
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education,
  statistic = TRUE,
  effect_size = "auto",
  effect_size_ci = TRUE
)

## ----nonparametric------------------------------------------------------------
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score),
  by = education,
  test = "nonparametric",
  statistic = TRUE,
  effect_size = TRUE
)

## ----effect-size-explicit-----------------------------------------------------
table_continuous(
  sochealth,
  select = wellbeing_score,
  by = sex,
  effect_size = "hedges_g",
  effect_size_ci = TRUE
)

## ----raw-output---------------------------------------------------------------
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score),
  by = education,
  statistic = TRUE,
  effect_size = TRUE,
  output = "data.frame"
)

## ----tidyselect---------------------------------------------------------------
table_continuous(
  sochealth,
  select = starts_with("life_sat"),
  by = sex
)

## ----regex--------------------------------------------------------------------
table_continuous(
  sochealth,
  select = "^life_sat",
  regex = TRUE,
  by = education,
  output = "data.frame"
)

## ----exclude------------------------------------------------------------------
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health, life_sat_work),
  exclude = "life_sat_work",
  by = sex
)

## ----labels, eval = build_rich_tables-----------------------------------------
# pkgdown_dark_gt(
#   table_continuous(
#     sochealth,
#     select = c(bmi, wellbeing_score, life_sat_health),
#     by = education,
#     labels = c(
#       bmi = "Body mass index",
#       wellbeing_score = "Well-being score",
#       life_sat_health = "Satisfaction with health"
#     ),
#     output = "gt"
#   )
# )

## ----tinytable, eval = build_rich_tables--------------------------------------
# table_continuous(
#   sochealth,
#   select = c(bmi, wellbeing_score, life_sat_health),
#   by = education,
#   output = "tinytable"
# )

## ----export, eval = FALSE-----------------------------------------------------
# table_continuous(
#   sochealth,
#   select = c(bmi, wellbeing_score, life_sat_health),
#   by = education,
#   output = "excel",
#   excel_path = "table_continuous.xlsx"
# )
# 
# table_continuous(
#   sochealth,
#   select = c(bmi, wellbeing_score, life_sat_health),
#   by = education,
#   output = "word",
#   word_path = "table_continuous.docx"
# )

## ----display-opts-------------------------------------------------------------
table_continuous(
  sochealth,
  select = wellbeing_score,
  by = sex,
  ci = FALSE,
  show_n = FALSE,
  p_digits = 4
)

## ----tidy-glance--------------------------------------------------------------
out <- table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score),
  by = sex
)

# Long descriptive rows: one per (variable x group) with broom-style
# columns (outcome, label, group, estimate = mean, std.error,
# conf.low / conf.high, n, min, max, sd).
broom::tidy(out)

# One row per outcome with the omnibus test + effect-size summary
# (test_type, statistic, df, df.residual, p.value, es_type, es_value,
# es_ci_lower / es_ci_upper, n_total).
broom::glance(out)

# Or just unbox to a plain data.frame (long-format underlying data)
head(as.data.frame(out))

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

build_rich_tables <- identical(Sys.getenv("IN_PKGDOWN"), "true")

source("_pkgdown-helpers.R")

## ----setup--------------------------------------------------------------------
library(spicy)

## ----basic--------------------------------------------------------------------
table_categorical(
  sochealth,
  select = c(smoking, physical_activity, dentist_12m),
  by = education
)

## ----oneway-------------------------------------------------------------------
table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  output = "default"
)

## ----gt, eval = build_rich_tables---------------------------------------------
# pkgdown_dark_gt(
#   table_categorical(
#     sochealth,
#     select = c(smoking, physical_activity, dentist_12m),
#     by = education,
#     output = "gt"
#   )
# )

## ----tinytable, eval = build_rich_tables--------------------------------------
# table_categorical(
#   sochealth,
#   select = c(smoking, physical_activity),
#   by = sex,
#   output = "tinytable"
# )

## ----data-frame---------------------------------------------------------------
table_categorical(
  sochealth,
  select = smoking,
  by = education,
  output = "data.frame"
)

## ----labels, eval = build_rich_tables-----------------------------------------
# pkgdown_dark_gt(
#   table_categorical(
#     sochealth,
#     select = c(smoking, physical_activity),
#     by = education,
#     labels = c(
#       smoking           = "Smoking status",
#       physical_activity = "Regular physical activity"
#     ),
#     output = "gt"
#   )
# )

## ----assoc-measure, eval = build_rich_tables----------------------------------
# # Uniform: same measure for every row variable
# table_categorical(
#   sochealth,
#   select = smoking,
#   by = education,
#   assoc_measure = "lambda",
#   output = "tinytable"
# )

## ----assoc-measure-named, eval = build_rich_tables----------------------------
# # Per-row: pick the right measure for each variable.
# # `smoking` x `education` is 2x3 (binary x ordered) -> Cramer's V;
# # `self_rated_health` x `education` is ordered x ordered -> Tau-b.
# # The mixed result collapses the header to "Effect size" and adds an
# # APA `Note.` line documenting the per-row measure.
# table_categorical(
#   sochealth,
#   select = c(smoking, self_rated_health),
#   by = education,
#   assoc_measure = c(
#     smoking           = "cramer_v",
#     self_rated_health = "tau_b"
#   ),
#   output = "tinytable"
# )

## ----ci-rendered, eval = build_rich_tables------------------------------------
# pkgdown_dark_gt(
#   table_categorical(
#     sochealth,
#     select = c(smoking, physical_activity),
#     by = education,
#     assoc_ci = TRUE,
#     output = "gt"
#   )
# )

## ----ci-data------------------------------------------------------------------
table_categorical(
  sochealth,
  select = smoking,
  by = education,
  assoc_ci = TRUE,
  output = "data.frame"
)

## ----weighted, eval = build_rich_tables---------------------------------------
# pkgdown_dark_gt(
#   table_categorical(
#     sochealth,
#     select = c(smoking, physical_activity),
#     by = education,
#     weights = "weight",
#     rescale = TRUE,
#     output = "gt"
#   )
# )

## ----missing, eval = build_rich_tables----------------------------------------
# pkgdown_dark_gt(
#   table_categorical(
#     sochealth,
#     select = income_group,
#     by = education,
#     drop_na = FALSE,
#     output = "gt"
#   )
# )

## ----levels-keep, eval = build_rich_tables------------------------------------
# pkgdown_dark_gt(
#   table_categorical(
#     sochealth,
#     select = income_group,
#     by = education,
#     drop_na = FALSE,
#     levels_keep = c("(Missing)", "Low", "High"),
#     output = "gt"
#   )
# )

## ----formatting, eval = build_rich_tables-------------------------------------
# pkgdown_dark_gt(
#   table_categorical(
#     sochealth,
#     select = smoking,
#     by = education,
#     percent_digits = 2,
#     p_digits = 4,
#     v_digits = 3,
#     output = "gt"
#   )
# )

## ----align--------------------------------------------------------------------
table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  by = sex,
  align = "auto"
)

## ----tidy-glance--------------------------------------------------------------
out <- table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  by = sex
)

# One row per (variable x level x group) with broom-style columns
# (outcome, level, group, n, proportion). The synthetic Total
# margin is excluded so each observation is counted once.
broom::tidy(out)

# One row per outcome with the omnibus chi-squared test and the
# chosen association measure (test_type, statistic, df, p.value,
# assoc_type, assoc_value, assoc_ci_lower / assoc_ci_upper, n_total).
broom::glance(out)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

build_rich_tables <- identical(Sys.getenv("IN_PKGDOWN"), "true")

source("_pkgdown-helpers.R")

## ----setup--------------------------------------------------------------------
library(spicy)

## ----basic--------------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi, life_sat_health),
  by = sex
)

## ----robust-------------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  vcov = "HC3",
  statistic = TRUE
)

## ----cluster, eval = requireNamespace("clubSandwich", quietly = TRUE)---------
table_continuous_lm(
  sleep,
  select = extra,
  by = group,
  vcov = "CR2",
  cluster = ID,
  statistic = TRUE
)

## ----bootstrap, eval = FALSE--------------------------------------------------
# table_continuous_lm(
#   sochealth,
#   select = wellbeing_score,
#   by = sex,
#   vcov = "bootstrap",
#   boot_n = 1000  # default
# )

## ----weights------------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = education,
  weights = weight,
  show_weighted_n = TRUE
)

## ----numeric-by---------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = age,
  vcov = "HC3"
)

## ----adjustment-basic---------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  covariates = c(age, education),
  vcov = "HC3"
)

## ----adjustment-balanced------------------------------------------------------
# Same model, balanced (emmeans-style) marginal means:
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  covariates = c(age, education),
  adjustment = "balanced",
  vcov = "HC3"
)

## ----es-d---------------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = smoking,
  effect_size = "d"
)

## ----es-g---------------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = smoking,
  effect_size = "g"
)

## ----es-omega2----------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = education,
  effect_size = "omega2"
)

## ----es-f2--------------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = age,
  effect_size = "f2"
)

## ----es-ci--------------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = smoking,
  effect_size = "g",
  effect_size_ci = TRUE
)

## ----es-ci-raw----------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = wellbeing_score,
  by = smoking,
  effect_size = "g",
  effect_size_ci = TRUE,
  output = "data.frame"
)

## ----es-ci-long---------------------------------------------------------------
out <- table_continuous_lm(
  sochealth,
  select = wellbeing_score,
  by = smoking,
  effect_size = "g",
  effect_size_ci = TRUE,
  output = "long"
)
out[, c("variable", "es_type", "es_value", "es_ci_lower", "es_ci_upper")]

## ----polish-------------------------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  labels = c(
    wellbeing_score = "WHO-5 wellbeing (0-100)",
    bmi = "Body-mass index (kg/m²)"
  ),
  effect_size = "g",
  effect_size_ci = TRUE,
  decimal_mark = ","
)

## ----gt-output, eval = build_rich_tables--------------------------------------
# pkgdown_dark_gt(
#   table_continuous_lm(
#     sochealth,
#     select = c(wellbeing_score, bmi, life_sat_health),
#     by = sex,
#     vcov = "HC3",
#     statistic = TRUE,
#     output = "gt"
#   )
# )

## ----tidy-glance--------------------------------------------------------------
out <- table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  effect_size = "g",
  effect_size_ci = TRUE
)

# One row per estimated parameter: emmean per level, contrast for
# binary predictors, slope for numeric predictors.
broom::tidy(out)

# One row per outcome with model-level statistics: r.squared,
# adj.r.squared, F / t, df, p.value, nobs, weighted_n, plus the
# effect-size summary.
broom::glance(out)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

build_rich_tables <- identical(Sys.getenv("IN_PKGDOWN"), "true")

source("_pkgdown-helpers.R")

## ----setup--------------------------------------------------------------------
library(spicy)

## ----grammar-categorical------------------------------------------------------
table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  by = education,
  labels = c("Smoking status", "Regular physical activity"),
  output = "tinytable"
)

## ----grammar-continuous-------------------------------------------------------
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education,
  labels = c(
    bmi = "Body mass index",
    wellbeing_score = "Well-being score",
    life_sat_health = "Satisfaction with health"
  ),
  output = "tinytable"
)

## ----grammar-continuous-lm----------------------------------------------------
table_continuous_lm(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education,
  weights = weight
)

## ----report-categorical, eval = build_rich_tables-----------------------------
# pkgdown_dark_gt(
#   table_categorical(
#     sochealth,
#     select = c(smoking, physical_activity, dentist_12m),
#     by = education,
#     labels = c(
#       "Smoking status",
#       "Regular physical activity",
#       "Visited a dentist in the last 12 months"
#     ),
#     output = "gt"
#   )
# )

## ----report-continuous, eval = build_rich_tables------------------------------
# pkgdown_dark_gt(
#   table_continuous(
#     sochealth,
#     select = c(bmi, wellbeing_score, life_sat_health),
#     by = education,
#     labels = c(
#       bmi = "Body mass index",
#       wellbeing_score = "Well-being score",
#       life_sat_health = "Satisfaction with health"
#     ),
#     p_value = TRUE,
#     effect_size = TRUE,
#     output = "gt"
#   )
# )

## ----report-continuous-lm, eval = build_rich_tables---------------------------
# pkgdown_dark_gt(
#   table_continuous_lm(
#     sochealth,
#     select = c(bmi, wellbeing_score, life_sat_health),
#     by = sex,
#     vcov = "HC3",
#     statistic = TRUE,
#     output = "gt"
#   )
# )

## ----output-flextable, eval = FALSE-------------------------------------------
# if (requireNamespace("flextable", quietly = TRUE)) {
#   table_continuous(
#     sochealth,
#     select = c(bmi, wellbeing_score, life_sat_health),
#     by = education,
#     output = "flextable"
#   )
# }

## ----postprocess-gt, eval = build_rich_tables---------------------------------
# tab <- pkgdown_dark_gt(table_categorical(
#   sochealth,
#   select = c(smoking, physical_activity),
#   by = education,
#   labels = c("Smoking status", "Regular physical activity"),
#   output = "gt"
# ))
# 
# tab |>
#   gt::tab_header(
#     title = "Health behaviors by education",
#     subtitle = "Categorical summary table"
#   ) |>
#   gt::tab_source_note(
#     gt::md("*Percentages are computed within each education group.*")
#   )

## ----postprocess-tinytable, eval = build_rich_tables--------------------------
# tab <- table_categorical(
#   sochealth,
#   select = c(smoking, physical_activity),
#   by = education,
#   labels = c("Smoking status", "Regular physical activity"),
#   output = "tinytable"
# )
# 
# tab |>
#   tinytable::style_tt(
#     i = 2:3,
#     j = 2:5,
#     background = "red",
#     color = "white",
#     bold = TRUE
#   )

## ----postprocess-flextable, eval = FALSE--------------------------------------
# if (requireNamespace("flextable", quietly = TRUE)) {
#   tab <- table_continuous(
#     sochealth,
#     select = c(bmi, wellbeing_score),
#     by = education,
#     output = "flextable"
#   )
# 
#   tab |>
#     flextable::theme_booktabs() |>
#     flextable::autofit() |>
#     flextable::fontsize(size = 10, part = "all")
# }

#' Styled print method for `freq()` tables
#'
#' @description
#' Internal print method used by [freq()] to display a styled, spicy-formatted
#' frequency table in the console.
#' It formats valid, missing, and total rows; handles cumulative and valid
#' percentages; and appends a labeled footer including metadata such as
#' variable label, class, dataset name, and weighting information.
#'
#' @details
#' This function is part of the *spicy table rendering engine*.
#' It is automatically called when printing the result of [freq()] with
#' `styled = TRUE`.
#' The output uses [spicy_print_table()] internally to render a colorized ASCII
#' table with consistent alignment and separators.
#'
#' The printed table includes:
#' * Valid and missing value sections (if applicable)
#' * Optional cumulative and valid percentages
#' * A final 'Total' row shown in the **Category** column
#' * A footer summarizing metadata (variable label, data source, weights)
#'
#' @param x A `data.frame` returned by [freq()] with attached attributes:
#'   - `"digits"`: number of decimal digits to display
#'   - `"data_name"`: name of the source dataset
#'   - `"var_name"`: name of the variable
#'   - `"var_label"`: variable label, if defined
#'   - `"class_name"`: original class of the variable
#'   - `"weighted"`, `"rescaled"`, `"weight_var"`: weighting metadata
#'
#' @param ... Additional arguments (ignored, required for S3 method compatibility)
#'
#' @return Invisibly returns `x` after printing the formatted table.
#'
#' @section Output structure:
#' The printed table includes the following columns:
#' \itemize{
#'   \item \strong{Category}: Sections such as "Valid", "Missing", and "Total"
#'   \item \strong{Values}: Observed categories or levels
#'   \item \strong{Freq.}: Frequency count (weighted if applicable)
#'   \item \strong{Percent}: Percentage of total
#'   \item \strong{Valid Percent}: Percentage among valid values (optional)
#'   \item \strong{Cum. Percent}: Cumulative percentage (optional)
#'   \item \strong{Cum. Valid Percent}: Cumulative valid percentage (optional)
#' }
#'
#' @examples
#' # Example using labelled data
#' library(labelled)
#' x <- labelled(
#'   c(1, 2, 3, 1, 2, 3, 1, 2, NA),
#'   labels = c("Low" = 1, "Medium" = 2, "High" = 3)
#' )
#' var_label(x) <- "Satisfaction level"
#' # Capture result without printing, then print explicitly
#' df <- spicy::freq(x, styled = FALSE)
#' print(df) # dispatches to print.spicy_freq_table()
#'
#' @seealso
#' [freq()] for the main frequency table generator.
#' [spicy_print_table()] for the generic ASCII table renderer.
#'
#' @importFrom stats na.omit
#'
#' @keywords internal
#' @export
print.spicy_freq_table <- function(x, ...) {
  df <- x
  digits <- attr(df, "digits")
  data_name <- attr(df, "data_name")
  var_name <- attr(df, "var_name")
  var_label <- attr(df, "var_label")
  class_name <- attr(df, "class_name")
  weighted <- isTRUE(attr(df, "weighted"))
  rescaled <- isTRUE(attr(df, "rescaled"))
  weight_var <- attr(df, "weight_var")
  has_cum <- "cum_prop" %in% names(df)

  var_name_clean <- sub("^.*\\$", "", var_name)
  data_name_clean <- sub("\\$.*$", "", data_name)

  valid_block <- df[!is.na(df$value), , drop = FALSE]
  missing_block <- df[is.na(df$value), , drop = FALSE]

  show_valid_col <- nrow(missing_block) > 0

  fmt_pct <- function(p) {
    ifelse(
      is.na(p),
      "NA",
      format(round(100 * p, digits), nsmall = digits, trim = TRUE)
    )
  }

  fmt_int <- function(v) format(round(v, 0), trim = TRUE)

  build_rows <- function(block, category, show_valid_col_block) {
    if (!nrow(block)) {
      return(NULL)
    }
    # Each block is homogeneous: `valid_block` has no NA in `value`, and
    # `missing_block` has only NA. Branch once on the whole block instead
    # of running a vectorized `ifelse` that is dead code on the valid path.
    values <- if (anyNA(block$value)) {
      rep("NA", nrow(block))
    } else {
      block$value
    }
    out <- data.frame(
      Category = c(category, rep("", nrow(block) - 1L)),
      Values = values,
      `Freq.` = fmt_int(block$n),
      Percent = fmt_pct(block$prop),
      stringsAsFactors = FALSE,
      check.names = FALSE
    )

    if (show_valid_col_block) {
      out$`Valid Percent` <- fmt_pct(block$valid_prop)
    }

    if (has_cum) {
      out$`Cum. Percent` <- fmt_pct(block$cum_prop)
      if (show_valid_col_block) {
        out$`Cum. Valid Percent` <- fmt_pct(block$cum_valid_prop)
      }
    }
    out
  }

  rows_valid <- build_rows(valid_block, "Valid", show_valid_col)
  rows_missing <- build_rows(missing_block, "Missing", FALSE)

  total_row <- data.frame(
    Category = "Total",
    Values = "",
    `Freq.` = fmt_int(sum(df$n)),
    Percent = format(round(100, digits), nsmall = digits, trim = TRUE),
    stringsAsFactors = FALSE,
    check.names = FALSE
  )

  if (show_valid_col) {
    total_row$`Valid Percent` <- format(
      round(100, digits),
      nsmall = digits,
      trim = TRUE
    )
  }

  if (has_cum) {
    total_row$`Cum. Percent` <- format(
      round(100, digits),
      nsmall = digits,
      trim = TRUE
    )
    if (show_valid_col) {
      total_row$`Cum. Valid Percent` <- format(
        round(100, digits),
        nsmall = digits,
        trim = TRUE
      )
    }
  }

  all_cols <- unique(c(
    names(rows_valid),
    names(rows_missing),
    names(total_row)
  ))
  fix_cols <- function(df_part) {
    if (is.null(df_part)) {
      return(NULL)
    }
    missing <- setdiff(all_cols, names(df_part))
    for (m in missing) {
      df_part[[m]] <- ""
    }
    df_part[all_cols]
  }

  disp <- do.call(
    rbind,
    lapply(list(rows_valid, rows_missing, total_row), fix_cols)
  )

  footer_lines <- c()

  # Defensive: only emit a Label line when var_label is a non-empty
  # single string. Skips NULL / NA / numeric / multi-element values
  # silently. Notably, `nzchar(NA_character_)` returns NA, which would
  # otherwise crash the surrounding `if` with "missing value where
  # TRUE/FALSE needed".
  if (
    is.character(var_label) &&
      length(var_label) == 1L &&
      !is.na(var_label) &&
      nzchar(var_label)
  ) {
    footer_lines <- c(footer_lines, paste0("Label: ", var_label))
  }

  footer_lines <- c(
    footer_lines,
    paste("Class:", class_name),
    paste("Data:", data_name_clean)
  )

  if (weighted) {
    weight_line <- if (!is.null(weight_var) && nzchar(weight_var)) {
      paste("Weight:", weight_var)
    } else {
      "Weight: (applied)"
    }

    if (isTRUE(rescaled)) {
      weight_line <- paste(weight_line, "(rescaled)")
    }

    footer_lines <- c(footer_lines, weight_line)
  }

  note_text <- paste(footer_lines, collapse = "\n")

  spicy_print_table(
    disp,
    title = paste("Frequency table:", var_name_clean),
    note = note_text,
    align_left_cols = c(1L, 2L),
    bottom_line = FALSE
  )
}

test_that("print.spicy_freq_table prints correctly and invisibly", {
  library(labelled)

  # Create a simple labelled vector
  x <- labelled(
    c(1, 2, 2, 3, 3, 3, NA),
    labels = c("Low" = 1, "Medium" = 2, "High" = 3)
  )
  var_label(x) <- "Satisfaction level"

  # Generate the styled invisible return (spicy_freq_table carrying the
  # print-method metadata). Swallow the printed output from the freq()
  # call itself via capture.output.
  capture.output(df <- freq(x))

  # Test invisibility of print method
  expect_invisible(print.spicy_freq_table(df))

  # Capture console output for inspection
  output <- capture.output(print.spicy_freq_table(df))

  # --- General checks
  expect_true(any(grepl("Frequency table:", output)))
  expect_true(any(grepl("Valid", output)))
  expect_true(any(grepl("Total", output)))

  # --- Footer checks
  expect_true(any(grepl("Label: Satisfaction level", output)))
  expect_true(any(grepl("Class:", output)))
  expect_true(any(grepl("Data:", output)))

  # --- Structural checks
  expect_true(any(grepl("\u2500", output))) # has horizontal lines
  expect_true(any(grepl("\u2502", output))) # has vertical bars
})


test_that("print.spicy_freq_table handles weighted tables", {
  df <- data.frame(
    sexe = factor(c("Male", "Female", "Female", "Male", NA)),
    poids = c(1.2, 0.8, 1.5, 1.0, 0.7)
  )

  # Weighted frequency table — keep the styled invisible return so that
  # the metadata attributes needed by print.spicy_freq_table are present.
  capture.output(ftab <- freq(df, sexe, weights = poids))

  output <- capture.output(print.spicy_freq_table(ftab))

  expect_true(any(grepl("Weight:", output)))
  expect_true(any(grepl("rescaled", output)))
  expect_true(any(grepl("Total", output)))
})


test_that("print.spicy_freq_table shows generic weight label when weight_var is empty", {
  df <- data.frame(x = c("A", "B", "C"), w = c(2, 3, 5))
  capture.output(ftab <- freq(df, x, weights = w))
  attr(ftab, "weight_var") <- NULL
  output <- capture.output(print.spicy_freq_table(ftab))
  expect_true(any(grepl("Weight: \\(applied\\)", output)))
})

test_that("print.spicy_freq_table handles variables without labels or missing values", {
  # Vector without label and no missing values
  x <- factor(c("A", "B", "B", "A", "C", "A"))

  capture.output(df <- freq(x))
  output <- capture.output(print.spicy_freq_table(df))

  # Expect no Label line, but expect Data/Class
  expect_false(any(grepl("^Label:", output)))
  expect_true(any(grepl("^Class:", output)))
  expect_true(any(grepl("^Data:", output)))
})

test_that("print.spicy_freq_table tolerates pathological var_label values", {
  # Defensive: NA / non-character / multi-element labels must not
  # produce a Label line and must not crash. NA_character_ would
  # previously trigger "missing value where TRUE/FALSE needed" via
  # nzchar(NA_character_) returning NA inside the surrounding `if`.
  df <- data.frame(x = c("A", "B", "B"))
  capture.output(ftab <- freq(df, x))

  attr(ftab, "var_label") <- NA_character_
  out <- capture.output(print.spicy_freq_table(ftab))
  expect_false(any(grepl("^Label:", out)))

  attr(ftab, "var_label") <- 42L
  out <- capture.output(print.spicy_freq_table(ftab))
  expect_false(any(grepl("^Label:", out)))

  attr(ftab, "var_label") <- c("first", "second")
  out <- capture.output(print.spicy_freq_table(ftab))
  expect_false(any(grepl("^Label:", out)))

  attr(ftab, "var_label") <- ""
  out <- capture.output(print.spicy_freq_table(ftab))
  expect_false(any(grepl("^Label:", out)))
})

#' Derive variable labels from column names \verb{name<sep>label}
#'
#' Splits each column name at the **first** occurrence of `sep`, renames
#' the column to the part before `sep` (the *name*), and assigns the part
#' after `sep` as a [`labelled::var_label()`]. This works even if the label
#' itself contains the separator.
#'
#' This function is especially useful for **LimeSurvey CSV exports** when using
#' *Export results* → *Export format: CSV* → *Headings: Question code & question text*,
#' where column names look like `"code. question text"`. In this case the
#' default separator is `". "`.
#'
#' @param df A `data.frame` or tibble with column names of the form
#'   \verb{"name<sep>label"} (e.g. "name. label"). (by default from LimeSurvey).
#' @param sep Character string used as separator between name and label.
#'   Default is `". "` (LimeSurvey's default), but any literal string can be used.
#'
#' @return A base `tibble` with column names equal to the *names* (before `sep`)
#'   and `var_label` attributes equal to the *labels* (after `sep`).
#'
#' @examples
#' # Example with LimeSurvey-style column names
#' df <- data.frame(
#'   "age. Age of respondent" = c(25, 30),
#'   "score. Total score. Manually computed." = c(12, 14),
#'   check.names = FALSE
#' )
#'
#' # sep = ". " by default (LimeSurvey)
#' out <- label_from_names(df)
#' labelled::var_label(out)
#'
#' # Example with a custom separator ("|")
#' df2 <- data.frame(
#'   "id|Identifier" = 1:3,
#'   "score|Total score" = c(10, 20, 30),
#'   check.names = FALSE
#' )
#' out2 <- label_from_names(df2, sep = "|")
#' labelled::var_label(out2)
#'
#' @export

label_from_names <- function(df, sep = ". ") {
  if (!is.data.frame(df)) {
    stop("`df` must be a data.frame or tibble.", call. = FALSE)
  }
  if (!is.character(sep) || length(sep) != 1L || is.na(sep) || !nzchar(sep)) {
    stop("`sep` must be a single non-empty character string.", call. = FALSE)
  }

  split_pos <- regexpr(sep, names(df), fixed = TRUE)

  new_names <- ifelse(
    split_pos > 0,
    substr(names(df), 1, split_pos - 1),
    names(df)
  )

  raw_labels <- ifelse(
    split_pos > 0,
    substring(names(df), split_pos + nchar(sep)),
    NA_character_
  )

  labels <- ifelse(
    is.na(raw_labels) | trimws(raw_labels) == "",
    NA_character_,
    raw_labels
  )

  df <- Map(
    function(col, lab) {
      labelled::var_label(col) <- lab
      col
    },
    df,
    labels
  )

  tibble::as_tibble(setNames(df, new_names))
}

test_that("label_from_names splits at first sep and assigns labels", {
  df <- data.frame(a = 1:2, b = 3:4)
  names(df) <- c("age. Age", "score. Score total. Computed")

  out <- label_from_names(df)

  expect_equal(names(out), c("age", "score"))

  labs <- labelled::var_label(out)

  expect_equal(labs[["age"]], "Age")
  expect_equal(labs[["score"]], "Score total. Computed")
})

test_that("empty or missing labels are skipped", {
  df <- data.frame(x = 1:2, y = 3:4)
  names(df) <- c("x", "y. ")

  out <- label_from_names(df)
  labs <- labelled::var_label(out)

  expect_true(
    is.null(labs[["x"]]) ||
      length(labs[["x"]]) == 0 ||
      is.na(labs[["x"]]) ||
      identical(labs[["x"]], "NA") ||
      labs[["x"]] == ""
  )

  expect_true(
    is.null(labs[["y"]]) ||
      length(labs[["y"]]) == 0 ||
      labs[["y"]] == "" ||
      is.na(labs[["y"]]) ||
      identical(labs[["y"]], "NA")
  )
})

test_that("label_from_names errors on non-data.frame input", {
  expect_error(
    label_from_names(1:3),
    "`df` must be a data.frame or tibble.",
    fixed = TRUE
  )
})

test_that("label_from_names validates sep parameter", {
  df <- data.frame(x = 1)
  expect_error(label_from_names(df, sep = ""), "sep")
  expect_error(label_from_names(df, sep = NA_character_), "sep")
  expect_error(label_from_names(df, sep = 1), "sep")
  expect_error(label_from_names(df, sep = c(".", ",")), "sep")
})

# Build a flextable/officer-compatible border object without forcing
# a hard dependency on officer for plain flextable output.
spicy_fp_border <- function(color = "black", width = 1, style = "solid") {
  structure(
    list(
      width = width,
      color = color,
      style = style
    ),
    class = "fp_border"
  )
}

test_that("spicy_fp_border returns an fp_border-compatible object", {
  border <- spicy:::spicy_fp_border(color = "black", width = 1)

  expect_s3_class(border, "fp_border")
  expect_equal(border$color, "black")
  expect_equal(border$width, 1)
  expect_equal(border$style, "solid")
})

#' Print method for bivariate linear-model tables
#'
#' @description
#' Formats and prints a `spicy_continuous_lm_table` object as a styled
#' ASCII table using [spicy_print_table()].
#'
#' @param x A `data.frame` of class `"spicy_continuous_lm_table"` as
#'   returned by [table_continuous_lm()].
#' @param ... Additional arguments (currently ignored).
#'
#' @return Invisibly returns `x`.
#'
#' @seealso [table_continuous_lm()], [spicy_print_table()]
#' @keywords internal
#' @export
print.spicy_continuous_lm_table <- function(x, ...) {
  digits <- attr(x, "digits") %||% 2L
  fit_digits <- attr(x, "fit_digits") %||% 2L
  effect_size_digits <- attr(x, "effect_size_digits") %||% 2L
  decimal_mark <- attr(x, "decimal_mark") %||% "."
  ci_level <- attr(x, "ci_level") %||% 0.95
  by_label <- attr(x, "by_label") %||% "Predictor"
  show_statistic <- attr(x, "show_statistic") %||% FALSE
  show_p_value <- attr(x, "show_p_value") %||% TRUE
  show_n <- attr(x, "show_n") %||% TRUE
  show_weighted_n <- attr(x, "show_weighted_n") %||% FALSE
  effect_size <- attr(x, "effect_size") %||% "none"
  r2_type <- attr(x, "r2_type") %||% "r2"
  show_ci <- attr(x, "show_ci") %||% TRUE

  display_df <- build_wide_display_df_continuous_lm(
    x,
    digits = digits,
    fit_digits = fit_digits,
    effect_size_digits = effect_size_digits,
    decimal_mark = decimal_mark,
    ci_level = ci_level,
    show_statistic = show_statistic,
    show_p_value = show_p_value,
    show_n = show_n,
    show_weighted_n = show_weighted_n,
    effect_size = effect_size,
    r2_type = r2_type,
    ci = show_ci
  )

  align_left <- 1L
  right_cols <- which(names(display_df) %in% c("n", "Weighted n", "p"))
  align_center <- setdiff(seq_len(ncol(display_df)), c(align_left, right_cols))

  padding <- "normal"
  col_widths <- vapply(
    seq_along(display_df),
    function(i) {
      max(nchar(c(names(display_df)[i], as.character(display_df[[i]]))))
    },
    numeric(1)
  )
  console_w <- getOption("width", 80L)
  normal_width <- sum(col_widths + 5L + 2L) + 1L
  if (normal_width > console_w) {
    padding <- "compact"
  }

  spicy_print_table(
    display_df,
    title = paste0("Continuous outcomes by ", by_label),
    note = NULL,
    padding = padding,
    first_column_line = TRUE,
    row_total_line = FALSE,
    column_total_line = FALSE,
    bottom_line = FALSE,
    align_left_cols = align_left,
    align_center_cols = align_center,
    group_sep_rows = integer(0)
  )

  invisible(x)
}

#' Copy data to the clipboard
#'
#' `copy_clipboard()` copies a data frame, matrix, array (2D or higher), table or vector to the clipboard.
#' You can paste the result into a text editor (e.g. Notepad++, Sublime Text), a spreadsheet (e.g. Excel, LibreOffice Calc), or a word processor (e.g. Word).
#'
#' Note: Objects that are not data frames or 2D matrices (e.g. atomic vectors, arrays, tables) are automatically converted to character
#' when copied to the clipboard, as required by [clipr::write_clip()]. The original object in R remains unchanged.
#'
#' For multidimensional arrays (e.g. 3D arrays), the entire array is flattened into a 1D character vector, with each element on a new line.
#' To preserve a tabular structure, you should extract a 2D slice before copying. For example: \code{copy_clipboard(my_array[, , 1])}.
#'
#' @param x A data frame, matrix, 2D array, 3D array, table, or atomic vector to be copied.
#' @param row.names.as.col Logical or character. If `FALSE` (the default), row names are not added as a column. If `TRUE`, a column named `"rownames"` is prepended. If a character string is supplied, it is used as the column name for row names.
#' @param row.names Logical. If `TRUE` (the default), includes row names in the clipboard output. If `FALSE`, row names are omitted.
#' @param col.names Logical. If `TRUE` (the default), includes column names in the clipboard output. If `FALSE`, column names are omitted.
#' @param show_message Logical. If `TRUE` (the default), displays a success message after copying. If `FALSE`, no success message is printed.
#' @param quiet Logical. If `FALSE` (the default), messages are shown. If `TRUE`, suppresses all messages, including success, coercion notices, and warnings.
#' @param ... Additional arguments passed to [clipr::write_clip()].
#'
#' @returns Invisibly returns the object `x`. The main purpose is the side effect of copying data to the clipboard.
#'
#' @importFrom tibble rownames_to_column
#' @export
#'
#' @examples
#' \donttest{
#' if (clipr::clipr_available()) {
#'   # Data frame
#'   copy_clipboard(sochealth)
#'
#'   # Data frame with row names as column
#'   copy_clipboard(head(sochealth), row.names.as.col = "id")
#'
#'   # Matrix
#'   mat <- matrix(1:6, nrow = 2)
#'   copy_clipboard(mat)
#'
#'   # Table
#'   tbl <- table(sochealth$education)
#'   copy_clipboard(tbl)
#'
#'   # Array (3D) — flattened to character
#'   arr <- array(1:8, dim = c(2, 2, 2))
#'   copy_clipboard(arr)
#'
#'   # Recommended: copy 2D slice for tabular layout
#'   copy_clipboard(arr[, , 1])
#'
#'   # Numeric vector
#'   copy_clipboard(c(3.14, 2.71, 1.618))
#'
#'   # Character vector
#'   copy_clipboard(c("apple", "banana", "cherry"))
#'
#'   # Quiet mode (no messages shown)
#'   copy_clipboard(sochealth, quiet = TRUE)
#' }
#' }
copy_clipboard <- function(
  x,
  row.names.as.col = FALSE,
  row.names = TRUE,
  col.names = TRUE,
  show_message = TRUE,
  quiet = FALSE,
  ...
) {
  if (!requireNamespace("clipr", quietly = TRUE)) {
    stop(
      "Package 'clipr' is required for copy_clipboard(). Please install it.",
      call. = FALSE
    )
  }

  if (!clipr::clipr_available()) {
    stop("Clipboard is not available on this system.", call. = FALSE)
  }

  is_df <- is.data.frame(x)
  is_strict_matrix <- is.matrix(x) && !inherits(x, "table")

  needs_warning <- FALSE
  warn_msg <- NULL

  if (is.logical(row.names.as.col)) {
    if (isTRUE(row.names.as.col)) {
      if (is_df) {
        x <- tibble::rownames_to_column(x, var = "rownames")
      } else if (is_strict_matrix) {
        x <- tibble::rownames_to_column(as.data.frame(x), var = "rownames")
      } else {
        needs_warning <- TRUE
        warn_msg <- "`row.names.as.col = TRUE` has no effect when `x` is not a data frame or matrix."
      }
    }
  } else if (is.character(row.names.as.col)) {
    if (is_df) {
      x <- tibble::rownames_to_column(x, var = row.names.as.col[1L])
    } else if (is_strict_matrix) {
      x <- tibble::rownames_to_column(
        as.data.frame(x),
        var = row.names.as.col[1L]
      )
    } else {
      needs_warning <- TRUE
      warn_msg <- "`row.names.as.col` is ignored because `x` is not a data frame or matrix."
    }
  } else if (!identical(row.names.as.col, FALSE)) {
    stop(
      "`row.names.as.col` must be either FALSE, TRUE, or a character string.",
      call. = FALSE
    )
  }

  msg_captured <- NULL
  warn_captured <- NULL
  withCallingHandlers(
    clipr::write_clip(
      x,
      row.names = row.names,
      col.names = col.names,
      ...
    ),
    message = function(m) {
      if (!quiet) {
        msg_captured <<- conditionMessage(m)
      }
      invokeRestart("muffleMessage")
    },
    warning = function(w) {
      if (!quiet) {
        warn_captured <<- conditionMessage(w)
      }
      invokeRestart("muffleWarning")
    }
  )

  use_color <- crayon::has_color()
  green <- if (use_color) crayon::make_style("green") else identity
  yellow <- if (use_color) crayon::make_style("yellow") else identity

  if (!quiet && isTRUE(show_message)) {
    cat(green("Data successfully copied to clipboard!"), "\n")
  }

  if (!quiet && !is.null(msg_captured)) {
    cat(yellow(paste0("Message: ", msg_captured)), "\n")
  }

  if (!quiet && !is.null(warn_captured)) {
    cat(yellow(paste0("Warning: ", warn_captured)), "\n")
  }

  if (!quiet && needs_warning) {
    cat(yellow(warn_msg), "\n")
  }

  invisible(x)
}

with_mocked_clipr <- function(
  code,
  clipr_available = function() TRUE,
  write_clip = function(...) NULL
) {
  ns <- asNamespace("clipr")
  old_available <- get("clipr_available", envir = ns)
  old_write <- get("write_clip", envir = ns)

  unlockBinding("clipr_available", ns)
  unlockBinding("write_clip", ns)
  assign("clipr_available", clipr_available, envir = ns)
  assign("write_clip", write_clip, envir = ns)
  lockBinding("clipr_available", ns)
  lockBinding("write_clip", ns)

  on.exit(
    {
      unlockBinding("clipr_available", ns)
      unlockBinding("write_clip", ns)
      assign("clipr_available", old_available, envir = ns)
      assign("write_clip", old_write, envir = ns)
      lockBinding("clipr_available", ns)
      lockBinding("write_clip", ns)
    },
    add = TRUE
  )

  eval(substitute(code), envir = parent.frame())
}

test_that("copy_clipboard() works silently for different structures", {
  skip_if_not_installed("clipr")
  skip_if_not(clipr::clipr_available())

  df <- data.frame(
    col1 = c("A", "B", "C"),
    col2 = c(1, 2, 3)
  )

  mat <- matrix(
    1:9,
    nrow = 3,
    byrow = TRUE,
    dimnames = list(
      c("Row1", "Row2", "Row3"),
      c("Col1", "Col2", "Col3")
    )
  )

  tab <- table(
    gender = c("Male", "Female", "Female", "Male", "Male", "Female"),
    city = c("Paris", "London", "Paris", "London", "Paris", "London")
  )

  arr <- array(1:8, dim = c(2, 2, 2))

  vec_num <- c(5.5, 6.6, 7.7)
  vec_chr <- c("alpha", "beta", "gamma")

  # Silent execution
  expect_silent(copy_clipboard(df, quiet = TRUE))
  expect_silent(copy_clipboard(df, row.names.as.col = TRUE, quiet = TRUE))
  expect_silent(copy_clipboard(df, col.names = FALSE, quiet = TRUE))

  expect_silent(copy_clipboard(mat, quiet = TRUE))
  expect_silent(copy_clipboard(mat, row.names.as.col = TRUE, quiet = TRUE))
  expect_silent(copy_clipboard(mat, col.names = FALSE, quiet = TRUE))

  expect_silent(copy_clipboard(tab, quiet = TRUE))
  expect_silent(copy_clipboard(tab, row.names.as.col = TRUE, quiet = TRUE))

  expect_silent(copy_clipboard(arr, quiet = TRUE))

  expect_silent(copy_clipboard(vec_num, quiet = TRUE))
  expect_silent(copy_clipboard(vec_chr, quiet = TRUE))
})

test_that("copy_clipboard() copies expected content", {
  skip_if_not_installed("clipr")
  skip_if_not(clipr::clipr_available())

  # Data frame
  df <- data.frame(
    name = c("Alice", "Bob"),
    score = c(10, 15)
  )

  copy_clipboard(df, quiet = TRUE)
  clip <- clipr::read_clip()

  expect_length(clip, 3)
  expect_true(grepl("name\\tscore", clip[1]))
  expect_true(grepl("Alice\\t10", clip[2]))
  expect_true(grepl("Bob\\t15", clip[3]))

  # Matrix
  mat <- matrix(
    1:4,
    nrow = 2,
    byrow = TRUE,
    dimnames = list(c("r1", "r2"), c("c1", "c2"))
  )

  copy_clipboard(mat, quiet = TRUE)
  mat_clip <- clipr::read_clip()
  expect_length(mat_clip, 3)
  expect_true(grepl("c1\\tc2", mat_clip[1]))

  # Numeric vector
  vec_num <- c(3.14, 2.71, 1.618)
  copy_clipboard(vec_num, quiet = TRUE)
  num_clip <- clipr::read_clip()
  expect_equal(num_clip, as.character(vec_num))

  # Character vector
  vec_chr <- c("apple", "banana", "cherry")
  copy_clipboard(vec_chr, quiet = TRUE)
  chr_clip <- clipr::read_clip()
  expect_equal(chr_clip, vec_chr)
})

test_that("copy_clipboard validates availability and row.names.as.col", {
  skip_if_not_installed("clipr")

  local_mocked_bindings(
    requireNamespace = function(...) TRUE,
    .package = "base"
  )

  with_mocked_clipr(
    clipr_available = function() FALSE,
    {
      expect_error(
        copy_clipboard(data.frame(x = 1)),
        "Clipboard is not available on this system."
      )
    }
  )

  with_mocked_clipr({
    expect_error(
      copy_clipboard(data.frame(x = 1), row.names.as.col = 1),
      "row.names.as.col"
    )
  })
})

test_that("copy_clipboard errors when clipr is unavailable", {
  local_mocked_bindings(
    requireNamespace = function(...) FALSE,
    .package = "base"
  )

  expect_error(
    copy_clipboard(data.frame(x = 1)),
    "Package 'clipr' is required"
  )
})

test_that("copy_clipboard captures write_clip messages and warnings", {
  skip_if_not_installed("clipr")

  local_mocked_bindings(
    requireNamespace = function(...) TRUE,
    .package = "base"
  )

  clip_payload <- NULL
  out <- capture.output(
    ret <- with_mocked_clipr(
      {
        copy_clipboard(
          data.frame(x = 1:2),
          show_message = TRUE,
          quiet = FALSE
        )
      },
      write_clip = function(x, ...) {
        clip_payload <<- x
        message("mock message")
        warning("mock warning")
        invisible(NULL)
      }
    )
  )

  expect_s3_class(ret, "data.frame")
  expect_true(any(grepl("Data successfully copied to clipboard!", out)))
  expect_true(any(grepl("Message: mock message", out)))
  expect_true(any(grepl("Warning: mock warning", out)))
  expect_equal(clip_payload$x, 1:2)
})

test_that("copy_clipboard warns when row.names.as.col is irrelevant", {
  skip_if_not_installed("clipr")

  local_mocked_bindings(
    requireNamespace = function(...) TRUE,
    .package = "base"
  )

  out <- capture.output(
    with_mocked_clipr({
      copy_clipboard(
        c("a", "b"),
        row.names.as.col = TRUE,
        quiet = FALSE,
        show_message = FALSE
      )
    })
  )

  expect_true(any(grepl("has no effect", out)))

  out_chr <- capture.output(
    with_mocked_clipr({
      copy_clipboard(
        table(c("a", "b")),
        row.names.as.col = "id",
        quiet = FALSE,
        show_message = FALSE
      )
    })
  )

  expect_true(any(grepl("is ignored because", out_chr)))
})

test_that("copy_clipboard adds row names column for data frames and matrices", {
  skip_if_not_installed("clipr")

  local_mocked_bindings(
    requireNamespace = function(...) TRUE,
    .package = "base"
  )

  captured_df <- NULL
  with_mocked_clipr(
    {
      df <- data.frame(value = c(10, 20), row.names = c("a", "b"))
      copy_clipboard(df, row.names.as.col = TRUE, quiet = TRUE)
    },
    write_clip = function(x, ...) {
      captured_df <<- x
      invisible(NULL)
    }
  )

  expect_equal(names(captured_df)[1], "rownames")
  expect_equal(captured_df$rownames, c("a", "b"))

  captured_mat <- NULL
  with_mocked_clipr(
    {
      mat <- matrix(
        1:4,
        nrow = 2,
        dimnames = list(c("r1", "r2"), c("c1", "c2"))
      )
      copy_clipboard(mat, row.names.as.col = "id", quiet = TRUE)
    },
    write_clip = function(x, ...) {
      captured_mat <<- x
      invisible(NULL)
    }
  )

  expect_equal(names(captured_mat)[1], "id")
  expect_equal(captured_mat$id, c("r1", "r2"))
})

#!/usr/bin/env Rscript

# Benchmark old (HEAD) vs current cross_tab() implementation.
# Run from package root:
#   Rscript tools/benchmark_cross_tab.R

suppressPackageStartupMessages({
  library(pkgload)
})

pkgload::load_all(".", quiet = TRUE)
new_cross_tab <- get("cross_tab", envir = parent.frame())

tmp_old <- tempfile(fileext = ".R")
old_src <- system2(
  "git",
  c("show", "HEAD:R/cross_tab.R"),
  stdout = TRUE,
  stderr = FALSE
)
if (length(old_src) == 0) {
  stop("Could not read HEAD version of R/cross_tab.R", call. = FALSE)
}
writeLines(old_src, tmp_old, useBytes = TRUE)

old_env <- new.env(parent = globalenv())
source(tmp_old, local = old_env, encoding = "UTF-8")
old_cross_tab <- old_env$cross_tab
unlink(tmp_old)

set.seed(20260306)
n <- 100000L
df <- data.frame(
  x = sample(c("A", "B", "C", "D"), n, replace = TRUE),
  y = sample(c("Yes", "No", "Maybe"), n, replace = TRUE),
  g = sample(c("G1", "G2", "G3"), n, replace = TRUE),
  w = runif(n, min = 0.1, max = 3),
  stringsAsFactors = FALSE
)

time_expr <- function(expr, n_rep = 7L) {
  out <- numeric(n_rep)
  for (i in seq_len(n_rep)) {
    gc(FALSE)
    out[i] <- system.time(eval(expr))[["elapsed"]]
  }
  out
}

bench_case <- function(name, expr_old, expr_new) {
  old_t <- time_expr(expr_old)
  new_t <- time_expr(expr_new)

  old_med <- stats::median(old_t)
  new_med <- stats::median(new_t)
  speedup <- if (new_med > 0) old_med / new_med else NA_real_

  data.frame(
    case = name,
    old_median_s = round(old_med, 4),
    new_median_s = round(new_med, 4),
    speedup_x = round(speedup, 3),
    stringsAsFactors = FALSE
  )
}

results <- do.call(
  rbind,
  list(
    bench_case(
      "df_no_group_stats",
      quote(old_cross_tab(
        df,
        x,
        y,
        percent = "c",
        include_stats = TRUE,
        styled = FALSE
      )),
      quote(new_cross_tab(
        df,
        x,
        y,
        percent = "c",
        include_stats = TRUE,
        styled = FALSE
      ))
    ),
    bench_case(
      "df_grouped_stats",
      quote(old_cross_tab(
        df,
        x,
        y,
        by = g,
        weights = w,
        percent = "c",
        include_stats = TRUE,
        styled = FALSE
      )),
      quote(new_cross_tab(
        df,
        x,
        y,
        by = g,
        weights = w,
        percent = "c",
        include_stats = TRUE,
        styled = FALSE
      ))
    ),
    bench_case(
      "vector_mode",
      quote(old_cross_tab(
        df$x,
        df$y,
        weights = df$w,
        percent = "c",
        include_stats = TRUE,
        styled = FALSE
      )),
      quote(new_cross_tab(
        df$x,
        df$y,
        weights = df$w,
        percent = "c",
        include_stats = TRUE,
        styled = FALSE
      ))
    )
  )
)

print(results, row.names = FALSE)

#' Categorical summary table
#'
#' @description
#' Builds a publication-ready frequency or cross-tabulation table for one
#' or many categorical variables selected with tidyselect syntax.
#'
#' With `by`, produces grouped cross-tabulation summaries (using
#' [cross_tab()] internally) with Chi-squared *p*-values and optional
#' association measures.
#' Without `by`, produces one-way frequency-style summaries.
#'
#' Multiple output formats are available via `output`: a printed ASCII
#' table (`"default"`), a wide or long numeric `data.frame`
#' (`"data.frame"`, `"long"`), or publication-ready tables
#' (`"tinytable"`, `"gt"`, `"flextable"`, `"excel"`, `"clipboard"`,
#' `"word"`).
#'
#' @param data A data frame.
#' @param select Columns to include as row variables. Supports tidyselect
#'   syntax and character vectors of column names.
#' @param by Optional grouping column used for columns/groups. Accepts an
#'   unquoted column name or a single character column name.
#' @param labels An optional character vector of display labels for the
#'   variables named in `select` (must be the same length and in the same
#'   order). When `NULL` (the default), column names are used as-is.
#' @param levels_keep Optional character vector of levels to keep/order for row
#'   modalities. If `NULL`, all observed levels are kept.
#' @param include_total Logical. If `TRUE` (the default), includes a `Total` group
#'   when available.
#' @param drop_na Logical. If `TRUE` (the default), removes rows with `NA` in the
#'   row/group variable before each cross-tabulation. If `FALSE`, missing values
#'   are displayed as a dedicated `"(Missing)"` level.
#' @param weights Optional weights. Either `NULL` (the default), a numeric vector
#'   of length `nrow(data)`, or a single column in `data` supplied as an
#'   unquoted name or a character string.
#' @param rescale Logical. If `FALSE` (the default), weights are used as-is.
#'   If `TRUE`, rescales weights so total weighted N matches raw N.
#'   Passed to `spicy::cross_tab()`.
#' @param correct Logical. If `FALSE` (the default), no continuity correction is
#'   applied. If `TRUE`, applies Yates correction in 2x2 chi-squared contexts.
#'   Passed to `spicy::cross_tab()`.
#' @param simulate_p Logical. If `FALSE` (the default), uses asymptotic p-values.
#'   If `TRUE`, uses Monte Carlo simulation. Passed to `spicy::cross_tab()`.
#' @param simulate_B Integer. Number of Monte Carlo replicates when
#'   `simulate_p = TRUE`. Defaults to `2000`.
#' @param percent_digits Number of digits for percentages in report outputs.
#'   Defaults to `1`.
#' @param p_digits Number of digits for p-values (except `< .001`).
#'   Defaults to `3`.
#' @param v_digits Number of digits for the association measure. Defaults
#'   to `2`.
#' @param assoc_measure Passed to [cross_tab()]. Which association measure
#'   to report (`"auto"`, `"cramer_v"`, `"phi"`, `"gamma"`, `"tau_b"`,
#'   `"tau_c"`, `"somers_d"`, `"lambda"`, `"none"`). Defaults to `"auto"`.
#' @param assoc_ci Passed to [cross_tab()]. If `TRUE`, includes the
#'   confidence interval of the association measure. In data formats
#'   (`"data.frame"`, `"long"`, `"excel"`, `"clipboard"`), two extra
#'   columns `CI lower` and `CI upper` are added.
#'   In rendered formats (`"gt"`, `"tinytable"`, `"flextable"`, `"word"`),
#'   the CI is shown inline (e.g., `.14 [.08, .19]`).
#'   Defaults to `FALSE`.
#' @param decimal_mark Decimal separator (`"."` or `","`). Defaults to `"."`.
#' @param output Output format. One of:
#'   - `"default"` (a printed ASCII table, returned invisibly)
#'   - `"data.frame"` (a wide numeric `data.frame`)
#'   - `"long"` (a long numeric `data.frame`)
#'   - `"tinytable"` (requires `tinytable`)
#'   - `"gt"` (requires `gt`)
#'   - `"flextable"` (requires `flextable`)
#'   - `"excel"` (requires `openxlsx2`)
#'   - `"clipboard"` (requires `clipr`)
#'   - `"word"` (requires `flextable` and `officer`)
#' @param indent_text Prefix used for modality labels in report table building.
#'   Defaults to `"  "` (two spaces).
#' @param indent_text_excel_clipboard Stronger indentation used in Excel and
#'   clipboard exports. Defaults to six non-breaking spaces.
#' @param add_multilevel_header Logical. If `TRUE` (the default), merges top
#'   headers in Excel export.
#' @param blank_na_wide Logical. If `FALSE` (the default), `NA` values are kept
#'   as-is in wide raw output. If `TRUE`, replaces them with empty strings.
#' @param excel_path Path for `output = "excel"`. Defaults to `NULL`.
#' @param excel_sheet Sheet name for Excel export. Defaults to `"Categorical"`.
#' @param clipboard_delim Delimiter for clipboard text export. Defaults to `"\t"`.
#' @param word_path Path for `output = "word"` or optional save path when
#'   `output = "flextable"`. Defaults to `NULL`.
#'
#' @return Depends on `output`:
#' \itemize{
#'   \item `"default"`: prints a styled ASCII table and returns the
#'     underlying `data.frame` invisibly (S3 class
#'     `"spicy_categorical_table"`).
#'   \item `"data.frame"`: a wide `data.frame` with one row per
#'     variable--level combination.
#'     When `by` is used, the columns are `Variable`, `Level`, and one
#'     pair of `n` / `\%` columns per group level (plus `Total` when
#'     `include_total = TRUE`), followed by `Chi2`, `df`, `p`, and the
#'     association measure column.
#'     When `by = NULL`, the columns are `Variable`, `Level`, `n`, `\%`.
#'   \item `"long"`: a long `data.frame` with columns `variable`,
#'     `level`, `group`, `n`, `percent` (and `chi2`, `df`, `p`,
#'     association measure columns when `by` is used).
#'   \item `"tinytable"`: a `tinytable` object.
#'   \item `"gt"`: a `gt_tbl` object.
#'   \item `"flextable"`: a `flextable` object.
#'   \item `"excel"` / `"word"`: writes to disk and returns the file
#'     path invisibly.
#'   \item `"clipboard"`: copies the table and returns the display
#'     `data.frame` invisibly.
#' }
#'
#' @details
#' When `by` is used, each selected variable is cross-tabulated against
#' the grouping variable with [cross_tab()]. Chi-squared statistics,
#' *p*-values, and the chosen association measure are reported for each
#' variable.
#'
#' Optional output engines require suggested packages:
#' \itemize{
#'   \item \pkg{tinytable} for `output = "tinytable"`
#'   \item \pkg{gt} for `output = "gt"`
#'   \item \pkg{flextable} for `output = "flextable"`
#'   \item \pkg{flextable} + \pkg{officer} for `output = "word"`
#'   \item \pkg{openxlsx2} for `output = "excel"`
#'   \item \pkg{clipr} for `output = "clipboard"`
#' }
#'
#' @seealso [table_continuous()] for continuous variables;
#'   [cross_tab()] for two-way cross-tabulations; [freq()] for one-way
#'   frequency tables.
#'
#' @examples
#' # Long numeric output
#' table_categorical(
#'   data = sochealth,
#'   select = c(smoking, physical_activity),
#'   by = education,
#'   labels = c("Current smoker", "Physical activity"),
#'   output = "long"
#' )
#'
#' # ASCII console output (default)
#' table_categorical(
#'   data = sochealth,
#'   select = c(smoking, physical_activity),
#'   by = sex
#' )
#'
#' # One-way frequency-style table
#' table_categorical(
#'   data = sochealth,
#'   select = c(smoking, physical_activity)
#' )
#'
#' # Wide numeric data.frame
#' table_categorical(
#'   data = sochealth,
#'   select = c(smoking, physical_activity),
#'   by = education,
#'   labels = c("Current smoker", "Physical activity"),
#'   output = "data.frame"
#' )
#'
#' # Weighted example
#' table_categorical(
#'   data = sochealth,
#'   select = c(smoking, physical_activity),
#'   by = education,
#'   labels = c("Current smoker", "Physical activity"),
#'   weights = "weight",
#'   rescale = TRUE,
#'   simulate_p = FALSE,
#'   output = "long"
#' )
#'
#' \donttest{
#' # Optional output: tinytable
#' if (requireNamespace("tinytable", quietly = TRUE)) {
#'   table_categorical(
#'     data = sochealth,
#'     select = c(smoking, physical_activity),
#'     by = sex,
#'     labels = c("Current smoker", "Physical activity"),
#'     output = "tinytable"
#'   )
#' }
#'
#' # Optional output: Excel
#' if (requireNamespace("openxlsx2", quietly = TRUE)) {
#'   table_categorical(
#'     data = sochealth,
#'     select = c(smoking, physical_activity),
#'     by = education,
#'     labels = c("Current smoker", "Physical activity"),
#'     output = "excel",
#'     excel_path = tempfile(fileext = ".xlsx")
#'   )
#' }
#' }
#'
#' @importFrom rlang enquo eval_tidy quo_get_env quo_is_null
#' @export
table_categorical <- function(
  data,
  select,
  by = NULL,
  labels = NULL,
  levels_keep = NULL,
  include_total = TRUE,
  drop_na = TRUE,
  weights = NULL,
  rescale = FALSE,
  correct = FALSE,
  simulate_p = FALSE,
  simulate_B = 2000,
  percent_digits = 1,
  p_digits = 3,
  v_digits = 2,
  assoc_measure = "auto",
  assoc_ci = FALSE,
  decimal_mark = ".",
  output = c(
    "default",
    "data.frame",
    "long",
    "tinytable",
    "gt",
    "flextable",
    "excel",
    "clipboard",
    "word"
  ),
  indent_text = "  ",
  indent_text_excel_clipboard = strrep("\u00A0", 6),
  add_multilevel_header = TRUE,
  blank_na_wide = FALSE,
  excel_path = NULL,
  excel_sheet = "Categorical",
  clipboard_delim = "\t",
  word_path = NULL
) {
  output <- match.arg(output)

  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }
  by_quo <- rlang::enquo(by)
  has_group <- !rlang::quo_is_null(by_quo)
  by_name <- NULL
  if (has_group) {
    by_name <- resolve_single_column_selection(by_quo, data, "by")
  }

  select_quo <- rlang::enquo(select)
  select_val <- tryCatch(
    rlang::eval_tidy(select_quo, env = rlang::quo_get_env(select_quo)),
    error = function(e) NULL
  )
  select_names <- if (is.character(select_val)) {
    select_val
  } else {
    tryCatch(
      names(tidyselect::eval_select(select_quo, data)),
      error = function(e) {
        stop(
          "`select` must select at least one column in `data`.",
          call. = FALSE
        )
      }
    )
  }
  if (length(select_names) == 0) {
    stop("`select` must select at least one column in `data`.", call. = FALSE)
  }
  if (!all(select_names %in% names(data))) {
    stop("Some `select` columns are missing in `data`.", call. = FALSE)
  }
  if (is.null(labels)) {
    labels <- select_names
  }
  if (length(labels) != length(select_names)) {
    stop("`labels` must have same length as `select`.", call. = FALSE)
  }
  labels <- as.character(labels)

  if (
    !is.logical(include_total) ||
      length(include_total) != 1 ||
      is.na(include_total)
  ) {
    stop("`include_total` must be TRUE/FALSE.", call. = FALSE)
  }
  if (!is.logical(drop_na) || length(drop_na) != 1 || is.na(drop_na)) {
    stop("`drop_na` must be TRUE/FALSE.", call. = FALSE)
  }
  if (!is.logical(rescale) || length(rescale) != 1 || is.na(rescale)) {
    stop("`rescale` must be TRUE/FALSE.", call. = FALSE)
  }
  if (!is.logical(correct) || length(correct) != 1 || is.na(correct)) {
    stop("`correct` must be TRUE/FALSE.", call. = FALSE)
  }
  if (!is.logical(simulate_p) || length(simulate_p) != 1 || is.na(simulate_p)) {
    stop("`simulate_p` must be TRUE/FALSE.", call. = FALSE)
  }
  if (
    !is.numeric(simulate_B) ||
      length(simulate_B) != 1 ||
      is.na(simulate_B) ||
      simulate_B < 1
  ) {
    stop("`simulate_B` must be a positive integer.", call. = FALSE)
  }
  simulate_B <- as.integer(simulate_B)
  if (
    !is.logical(add_multilevel_header) ||
      length(add_multilevel_header) != 1 ||
      is.na(add_multilevel_header)
  ) {
    stop("`add_multilevel_header` must be TRUE/FALSE.", call. = FALSE)
  }
  if (
    !is.logical(blank_na_wide) ||
      length(blank_na_wide) != 1 ||
      is.na(blank_na_wide)
  ) {
    stop("`blank_na_wide` must be TRUE/FALSE.", call. = FALSE)
  }
  if (!identical(decimal_mark, ".") && !identical(decimal_mark, ",")) {
    stop("`decimal_mark` must be either '.' or ','.", call. = FALSE)
  }
  for (.dname in c("percent_digits", "p_digits", "v_digits")) {
    .dval <- get(.dname)
    if (
      !is.numeric(.dval) || length(.dval) != 1L || is.na(.dval) || .dval < 0
    ) {
      stop(
        paste0("`", .dname, "` must be a single non-negative number."),
        call. = FALSE
      )
    }
  }
  percent_digits <- as.integer(percent_digits)
  p_digits <- as.integer(p_digits)
  v_digits <- as.integer(v_digits)

  if (!has_group) {
    if (!include_total) {
      warning(
        "`include_total` is ignored when `by` is not used.",
        call. = FALSE
      )
    }
    if (correct) {
      warning("`correct` is ignored when `by` is not used.", call. = FALSE)
    }
    if (simulate_p) {
      warning("`simulate_p` is ignored when `by` is not used.", call. = FALSE)
    }
    if (!identical(assoc_measure, "auto")) {
      warning(
        "`assoc_measure` is ignored when `by` is not used.",
        call. = FALSE
      )
    }
    if (assoc_ci) {
      warning("`assoc_ci` is ignored when `by` is not used.", call. = FALSE)
    }
    include_total <- TRUE
  }

  weights_quo <- rlang::enquo(weights)
  weights_vec <- resolve_weights_argument(weights_quo, data, "weights")

  if (isTRUE(rescale) && is.null(weights_vec)) {
    warning(
      "`rescale = TRUE` has no effect without `weights`; using `rescale = FALSE`.",
      call. = FALSE
    )
    rescale <- FALSE
  }

  all_values <- unique(unlist(
    lapply(c(select_names, by_name), function(nm) as.character(data[[nm]])),
    use.names = FALSE
  ))
  missing_label <- "(Missing)"
  if (missing_label %in% all_values) {
    idx <- 1L
    repeat {
      candidate <- paste0("(Missing_", idx, ")")
      if (!(candidate %in% all_values)) {
        missing_label <- candidate
        break
      }
      idx <- idx + 1L
    }
  }

  parse_stats <- function(ct_obj) {
    # Read numeric attributes set by cross_tab()
    p_val <- attr(ct_obj, "p_value")
    v_val <- attr(ct_obj, "assoc_value")
    m_name <- attr(ct_obj, "assoc_measure")
    ar <- attr(ct_obj, "assoc_result")
    ci_lo <- if (!is.null(ar)) ar[["ci_lower"]] else NA_real_
    ci_hi <- if (!is.null(ar)) ar[["ci_upper"]] else NA_real_

    if (!is.null(p_val) && !is.null(v_val)) {
      p_op <- if (!is.na(p_val) && p_val < 0.001) "<" else "="
      return(list(
        p = p_val,
        p_op = p_op,
        v = v_val,
        measure = m_name %||% "Cramer's V",
        ci_lower = ci_lo,
        ci_upper = ci_hi
      ))
    }

    # Fallback: parse note text
    note_txt <- attr(ct_obj, "note")
    txt <- paste(note_txt %||% "", collapse = " ")

    pm <- regmatches(
      txt,
      regexec("p\\s*([<=>])\\s*([0-9.]+(?:e[-+]?\\d+)?)", txt, perl = TRUE)
    )[[1]]
    p_op <- if (length(pm) >= 2) pm[2] else NA_character_
    p_val <- if (length(pm) >= 3) {
      suppressWarnings(as.numeric(pm[3]))
    } else {
      NA_real_
    }

    # Try to match any "Measure = value" pattern
    vm <- regmatches(
      txt,
      regexec(
        "(?:Cramer's V|Phi|Goodman-Kruskal(?:'s)? (?:Gamma|Tau)|Kendall's Tau-b|Kendall's Tau-c|Somers' D|Lambda)\\s*=\\s*([0-9.eE+-]+)",
        txt,
        perl = TRUE
      )
    )[[1]]
    v_val <- if (length(vm) >= 2) {
      suppressWarnings(as.numeric(vm[2]))
    } else {
      NA_real_
    }

    list(
      p = p_val,
      p_op = p_op,
      v = v_val,
      measure = "Cramer's V",
      ci_lower = NA_real_,
      ci_upper = NA_real_
    )
  }

  fmt_num <- function(x, digits = 1, na = "") {
    out <- ifelse(is.na(x), na, formatC(x, format = "f", digits = digits))
    if (decimal_mark != ".") {
      out <- sub("\\.", decimal_mark, out)
    }
    out
  }

  fmt_n <- function(x, na = "") {
    out <- rep(na, length(x))
    ok <- !is.na(x)
    if (any(ok)) {
      xi <- x[ok]
      int_like <- abs(xi - round(xi)) < 1e-8
      tmp <- character(length(xi))
      tmp[int_like] <- as.character(as.integer(round(xi[int_like])))
      tmp[!int_like] <- formatC(xi[!int_like], format = "f", digits = 1)
      if (decimal_mark != ".") {
        tmp <- sub("\\.", decimal_mark, tmp)
      }
      out[ok] <- tmp
    }
    out
  }

  fmt_p <- function(p, op = NA_character_) {
    if (is.na(p)) {
      return("")
    }
    if ((!is.na(op) && op == "<" && p <= 0.001) || p < 0.001) {
      return(if (decimal_mark == ".") "<\u00A0.001" else "<\u00A0,001")
    }
    s <- formatC(p, format = "f", digits = p_digits)
    s <- sub("^0\\.", ".", s)
    if (decimal_mark != ".") {
      s <- sub("\\.", decimal_mark, s)
    }
    s
  }

  fmt_v <- function(v) {
    if (is.na(v)) {
      return("")
    }
    s <- formatC(v, format = "f", digits = v_digits)
    s <- sub("^0\\.", ".", s)
    if (decimal_mark != ".") {
      s <- sub("\\.", decimal_mark, s)
    }
    s
  }

  make_stronger_indent <- function(x, base_indent, strong_indent) {
    is_mod <- startsWith(x, base_indent)
    if (any(is_mod)) {
      suffix <- substring(x[is_mod], nchar(base_indent) + 1L)
      x[is_mod] <- paste0(strong_indent, suffix)
    }
    x
  }

  if (!has_group) {
    rows <- list()
    rr <- 1L
    all_level_order <- character(0)

    for (i in seq_along(select_names)) {
      x <- data[[select_names[i]]]
      w <- weights_vec

      if (is.factor(x)) {
        var_level_order <- levels(x)
      } else {
        var_level_order <- unique(as.character(x[!is.na(x)]))
      }

      keep <- if (drop_na) !is.na(x) else rep(TRUE, length(x))
      x <- x[keep]
      if (!is.null(w)) {
        w <- w[keep]
      }
      if (!length(x)) {
        next
      }
      if (!drop_na) {
        x <- as.character(x)
        x[is.na(x)] <- missing_label
      }

      ft <- if (is.null(w)) {
        spicy::freq(
          x,
          rescale = rescale,
          valid = FALSE,
          styled = FALSE
        )
      } else {
        spicy::freq(
          x,
          weights = w,
          rescale = rescale,
          valid = FALSE,
          styled = FALSE
        )
      }
      vals <- as.character(ft$value)
      raw_levels <- vals[!is.na(vals)]

      lv_use <- if (is.null(levels_keep)) {
        known <- intersect(var_level_order, raw_levels)
        extra <- setdiff(raw_levels, c(var_level_order, missing_label))
        missing_end <- intersect(raw_levels, missing_label)
        c(known, extra, missing_end)
      } else {
        intersect(as.character(levels_keep), raw_levels)
      }
      all_level_order <- c(all_level_order, lv_use)

      for (lv in lv_use) {
        idx <- match(lv, vals)
        if (is.na(idx)) {
          next
        }
        rows[[rr]] <- data.frame(
          variable = labels[i],
          level = lv,
          n = suppressWarnings(as.numeric(ft$n[idx])),
          pct = 100 * suppressWarnings(as.numeric(ft$prop[idx])),
          stringsAsFactors = FALSE,
          check.names = FALSE
        )
        rr <- rr + 1L
      }
    }

    if (length(rows) == 0) {
      long_raw <- data.frame(
        variable = character(0),
        level = character(0),
        n = numeric(0),
        pct = numeric(0),
        stringsAsFactors = FALSE,
        check.names = FALSE
      )
    } else {
      long_raw <- do.call(rbind, rows)
    }

    if (nrow(long_raw) > 0) {
      long_raw$variable <- factor(long_raw$variable, levels = labels)
      if (!is.null(levels_keep)) {
        long_raw$level <- factor(
          long_raw$level,
          levels = as.character(levels_keep)
        )
      } else {
        long_raw$level <- factor(
          long_raw$level,
          levels = unique(all_level_order)
        )
      }
      long_raw <- long_raw[
        order(long_raw$variable, long_raw$level),
        ,
        drop = FALSE
      ]
      long_raw$variable <- as.character(long_raw$variable)
      long_raw$level <- as.character(long_raw$level)
      rownames(long_raw) <- NULL
    }

    if (output == "long") {
      return(long_raw)
    }

    wide_raw <- data.frame(
      Variable = long_raw$variable,
      Level = long_raw$level,
      n = long_raw$n,
      check.names = FALSE
    )
    wide_raw[["%"]] <- long_raw$pct

    if (blank_na_wide && nrow(wide_raw) > 0) {
      for (j in seq_len(ncol(wide_raw))) {
        if (j > 2) {
          wide_raw[[j]] <- ifelse(
            is.na(wide_raw[[j]]),
            "",
            as.character(wide_raw[[j]])
          )
        }
      }
    }

    if (output == "data.frame") {
      return(wide_raw)
    }

    report_cols <- c("Variable", "n", "%")
    make_report_wide_oneway <- function(mode = c("char", "excel")) {
      mode <- match.arg(mode)

      if (nrow(long_raw) == 0) {
        if (mode == "char") {
          return(as.data.frame(
            setNames(
              replicate(length(report_cols), character(0), simplify = FALSE),
              report_cols
            ),
            check.names = FALSE
          ))
        }
        out <- as.data.frame(
          setNames(
            replicate(length(report_cols), numeric(0), simplify = FALSE),
            report_cols
          ),
          check.names = FALSE
        )
        out$Variable <- character(0)
        return(out[, report_cols, drop = FALSE])
      }

      out <- list()
      z <- 1L
      for (lab in labels) {
        sv <- long_raw[long_raw$variable == lab, , drop = FALSE]
        if (nrow(sv) == 0) {
          next
        }

        lv_use <- if (is.null(levels_keep)) {
          unique(sv$level)
        } else {
          intersect(as.character(levels_keep), unique(sv$level))
        }

        if (mode == "char") {
          r0 <- as.list(setNames(rep("", length(report_cols)), report_cols))
        } else {
          r0 <- as.list(setNames(rep(NA, length(report_cols)), report_cols))
        }
        r0$Variable <- lab
        out[[z]] <- as.data.frame(
          r0,
          stringsAsFactors = FALSE,
          check.names = FALSE
        )
        z <- z + 1L

        for (lv in lv_use) {
          sl <- sv[sv$level == lv, , drop = FALSE]
          if (mode == "char") {
            r1 <- as.list(setNames(rep("", length(report_cols)), report_cols))
            r1$n <- fmt_n(sl$n[1])
            r1[["%"]] <- fmt_num(sl$pct[1], percent_digits)
          } else {
            r1 <- as.list(setNames(rep(NA, length(report_cols)), report_cols))
            r1$n <- sl$n[1]
            r1[["%"]] <- sl$pct[1]
          }
          r1$Variable <- paste0(indent_text, lv)
          out[[z]] <- as.data.frame(
            r1,
            stringsAsFactors = FALSE,
            check.names = FALSE
          )
          z <- z + 1L
        }
      }

      do.call(rbind, out)
    }

    report_wide_char <- make_report_wide_oneway("char")
    report_wide_excel <- make_report_wide_oneway("excel")

    if (output == "default") {
      out <- wide_raw
      attr(out, "display_df") <- report_wide_char
      attr(out, "group_var") <- NULL
      attr(out, "indent_text") <- indent_text
      class(out) <- c("spicy_categorical_table", "spicy_table", "data.frame")
      print(out)
      return(invisible(out))
    }

    if (output == "tinytable") {
      if (!requireNamespace("tinytable", quietly = TRUE)) {
        stop("Install package 'tinytable'.", call. = FALSE)
      }
      old_tt_opt <- getOption("tinytable_print_output")
      options(tinytable_print_output = "html")
      on.exit(options(tinytable_print_output = old_tt_opt), add = TRUE)

      dat_tt <- report_wide_char
      mod_rows <- which(startsWith(dat_tt[[1]], indent_text))
      if (length(mod_rows)) {
        dat_tt[[1]][mod_rows] <- paste0(
          strrep("\u00A0", 4),
          substring(dat_tt[[1]][mod_rows], nchar(indent_text) + 1L)
        )
      }
      names(dat_tt) <- c("", "n", "%")

      tt <- tinytable::tt(dat_tt, escape = FALSE)
      tt <- tinytable::theme_empty(tt)
      tt <- tinytable::style_tt(tt, j = 1, align = "l")
      tt <- tinytable::style_tt(tt, j = 2:ncol(dat_tt), align = "r")
      tt <- tinytable::style_tt(tt, i = 0, j = 2:ncol(dat_tt), align = "c")
      if (length(mod_rows)) {
        tt <- tinytable::style_tt(tt, i = mod_rows, j = 1, indent = 1)
        tt <- tinytable::style_tt(
          tt,
          i = mod_rows,
          j = 1,
          html_css = "padding-left: 0.8em;"
        )
      }
      tt <- tinytable::style_tt(
        tt,
        i = 0,
        j = seq_len(ncol(dat_tt)),
        line = "t",
        line_width = 0.06
      )
      tt <- tinytable::style_tt(
        tt,
        i = 0,
        j = seq_len(ncol(dat_tt)),
        line = "b",
        line_width = 0.06
      )
      tt <- tinytable::style_tt(
        tt,
        i = nrow(dat_tt),
        j = seq_len(ncol(dat_tt)),
        line = "b",
        line_width = 0.06
      )
      return(tt)
    }

    if (output == "gt") {
      if (!requireNamespace("gt", quietly = TRUE)) {
        stop("Install package 'gt'.", call. = FALSE)
      }
      dat_gt <- report_wide_char
      mod_rows <- which(startsWith(dat_gt[[1]], indent_text))
      if (length(mod_rows)) {
        dat_gt[[1]][mod_rows] <- paste0(
          strrep("\u00A0", 4),
          substring(dat_gt[[1]][mod_rows], nchar(indent_text) + 1L)
        )
      }
      names(dat_gt) <- c("Variable", "n", "pct")
      tbl <- gt::gt(dat_gt)
      tbl <- gt::cols_label(tbl, Variable = "", n = "n", pct = "%")
      tbl <- gt::cols_align(tbl, align = "left", columns = "Variable")
      tbl <- gt::cols_align(tbl, align = "right", columns = c("n", "pct"))
      rule <- gt::cell_borders(
        sides = "bottom",
        color = "currentColor",
        weight = gt::px(1)
      )
      rule_top <- gt::cell_borders(
        sides = "top",
        color = "currentColor",
        weight = gt::px(1)
      )
      tbl <- gt::tab_options(
        tbl,
        table.border.top.width = gt::px(0),
        table.border.bottom.width = gt::px(0),
        table_body.border.top.width = gt::px(0),
        table_body.border.bottom.width = gt::px(0),
        table_body.hlines.color = "transparent",
        column_labels.border.top.width = gt::px(0),
        column_labels.border.bottom.width = gt::px(0),
        column_labels.border.lr.color = "transparent"
      )
      tbl <- gt::tab_style(
        tbl,
        style = rule_top,
        locations = gt::cells_column_labels()
      )
      tbl <- gt::tab_style(
        tbl,
        style = rule,
        locations = gt::cells_column_labels()
      )
      tbl <- gt::tab_style(
        tbl,
        style = rule,
        locations = gt::cells_body(rows = nrow(dat_gt))
      )
      return(tbl)
    }

    build_flextable_oneway <- function(df) {
      if (!requireNamespace("flextable", quietly = TRUE)) {
        stop("Install package 'flextable'.", call. = FALSE)
      }
      ft <- flextable::flextable(df)
      map <- data.frame(
        col_keys = names(df),
        label = c("Variable", "n", "%"),
        stringsAsFactors = FALSE,
        check.names = FALSE
      )
      ft <- flextable::set_header_df(ft, mapping = map, key = "col_keys")
      bd <- spicy_fp_border(color = "black", width = 1)
      ft <- flextable::align(ft, j = 1, part = "all", align = "left")
      ft <- flextable::align(ft, j = 2:ncol(df), part = "all", align = "right")
      ft <- flextable::hline_top(ft, part = "header", border = bd)
      ft <- flextable::hline_bottom(ft, part = "header", border = bd)
      ft <- flextable::hline_bottom(ft, part = "body", border = bd)
      id_mod <- which(startsWith(df[[1]], indent_text))
      if (length(id_mod)) {
        ft <- flextable::padding(
          ft,
          i = id_mod,
          j = 1,
          part = "body",
          padding.left = 14
        )
      }
      flextable::autofit(ft)
    }

    if (output == "flextable") {
      ft <- build_flextable_oneway(report_wide_char)
      if (!is.null(word_path) && nzchar(word_path)) {
        if (!requireNamespace("officer", quietly = TRUE)) {
          stop("Install package 'officer'.", call. = FALSE)
        }
        flextable::save_as_docx(ft, path = word_path)
      }
      return(ft)
    }

    if (output == "word") {
      if (is.null(word_path) || !nzchar(word_path)) {
        stop("Provide `word_path` for output = 'word'.", call. = FALSE)
      }
      ft <- build_flextable_oneway(report_wide_char)
      flextable::save_as_docx(ft, path = word_path)
      return(invisible(word_path))
    }

    clip_body <- report_wide_char
    clip_body$Variable <- make_stronger_indent(
      clip_body$Variable,
      indent_text,
      indent_text_excel_clipboard
    )
    clip_mat <- rbind(matrix(names(clip_body), nrow = 1), as.matrix(clip_body))

    if (output == "excel") {
      if (is.null(excel_path) || !nzchar(excel_path)) {
        stop("Provide `excel_path` for output = 'excel'.", call. = FALSE)
      }
      if (!requireNamespace("openxlsx2", quietly = TRUE)) {
        stop("Install package 'openxlsx2'.", call. = FALSE)
      }

      body_xl <- report_wide_excel
      body_xl$Variable <- make_stronger_indent(
        body_xl$Variable,
        indent_text,
        indent_text_excel_clipboard
      )

      wb <- openxlsx2::wb_workbook()
      wb <- openxlsx2::wb_add_worksheet(wb, excel_sheet)
      wb <- openxlsx2::wb_add_data(
        wb,
        x = body_xl,
        start_row = 1,
        col_names = TRUE,
        row_names = FALSE
      )

      nc <- ncol(body_xl)
      last_row <- nrow(body_xl) + 1
      pct_fmt <- paste0("0.", paste(rep("0", percent_digits), collapse = ""))

      # Header borders (top + bottom on row 1)
      wb <- openxlsx2::wb_add_border(
        wb,
        dims = openxlsx2::wb_dims(rows = 1, cols = 1:nc),
        top_border = "thin",
        bottom_border = "thin"
      )
      if (nrow(body_xl) > 0) {
        # Body alignment
        wb <- openxlsx2::wb_add_cell_style(
          wb,
          dims = openxlsx2::wb_dims(rows = 2:last_row, cols = 1),
          horizontal = "left"
        )
        wb <- openxlsx2::wb_add_cell_style(
          wb,
          dims = openxlsx2::wb_dims(rows = 2:last_row, cols = 2:nc),
          horizontal = "right"
        )
        # Number formats: integers (col 2), percentages (col 3)
        wb <- openxlsx2::wb_add_numfmt(
          wb,
          dims = openxlsx2::wb_dims(rows = 2:last_row, cols = 2),
          numfmt = "0"
        )
        wb <- openxlsx2::wb_add_numfmt(
          wb,
          dims = openxlsx2::wb_dims(rows = 2:last_row, cols = 3),
          numfmt = pct_fmt
        )
        # Bottom border on last row
        wb <- openxlsx2::wb_add_border(
          wb,
          dims = openxlsx2::wb_dims(rows = last_row, cols = 1:nc),
          bottom_border = "thin"
        )
      }

      openxlsx2::wb_save(wb, excel_path, overwrite = TRUE)
      return(invisible(excel_path))
    }

    if (output == "clipboard") {
      if (!requireNamespace("clipr", quietly = TRUE)) {
        stop("Install package 'clipr'.", call. = FALSE)
      }
      lines <- apply(clip_mat, 1, function(x) {
        paste(x, collapse = clipboard_delim)
      })
      txt <- paste(lines, collapse = "\n")
      clipr::write_clip(txt)
      message("Categorical table copied to clipboard.")
      return(invisible(txt))
    }
  }

  g0 <- data[[by_name]]
  show_assoc <- !identical(assoc_measure, "none")
  if (!show_assoc) {
    assoc_ci <- FALSE
  }
  group_levels <- if (is.factor(g0)) {
    levels(g0)
  } else {
    unique(as.character(g0[!is.na(g0)]))
  }
  group_levels <- as.character(group_levels)
  if (!drop_na && any(is.na(g0))) {
    group_levels <- unique(c(group_levels, missing_label))
  }
  if (include_total) {
    group_levels <- unique(c(group_levels, "Total"))
  }

  # ---------------- LONG RAW ----------------
  rows <- list()
  rr <- 1L
  measure_col <- NULL
  all_level_order <- character(0)

  for (i in seq_along(select_names)) {
    x <- data[[select_names[i]]]
    g <- data[[by_name]]
    w <- weights_vec

    # Capture original level order before any filtering/conversion
    if (is.factor(x)) {
      var_level_order <- levels(x)
    } else {
      var_level_order <- unique(as.character(x[!is.na(x)]))
    }

    keep <- rep(TRUE, length(x))
    if (drop_na) {
      keep <- !is.na(x) & !is.na(g)
    }

    x <- x[keep]
    g <- g[keep]
    if (!is.null(w)) {
      w <- w[keep]
    }
    if (!length(x)) {
      next
    }
    if (!drop_na) {
      x <- as.character(x)
      g <- as.character(g)
      x[is.na(x)] <- missing_label
      g[is.na(g)] <- missing_label
    }

    ct_pct <- spicy::cross_tab(
      x,
      g,
      percent = "c",
      weights = w,
      rescale = rescale,
      correct = correct,
      simulate_p = simulate_p,
      simulate_B = simulate_B,
      assoc_measure = assoc_measure,
      assoc_ci = assoc_ci
    )
    ct_n <- spicy::cross_tab(
      x,
      g,
      weights = w,
      rescale = rescale,
      correct = correct,
      simulate_p = simulate_p,
      simulate_B = simulate_B,
      assoc_measure = "none"
    )
    st <- parse_stats(ct_pct)
    if (show_assoc && is.null(measure_col)) {
      measure_col <- st$measure %||% "Cramer's V"
    }

    groups_present <- setdiff(names(ct_n), "Values")
    groups_use <- intersect(group_levels, groups_present)
    if (!include_total) {
      groups_use <- setdiff(groups_use, "Total")
    }

    vals_n <- as.character(ct_n$Values)
    vals_p <- as.character(ct_pct$Values)

    lv_use <- if (is.null(levels_keep)) {
      raw_levels <- setdiff(unique(vals_n), c("Total", "N"))
      # Reorder to match original factor/occurrence order
      known <- intersect(var_level_order, raw_levels)
      extra <- setdiff(raw_levels, c(var_level_order, missing_label))
      missing_end <- intersect(raw_levels, missing_label)
      c(known, extra, missing_end)
    } else {
      intersect(as.character(levels_keep), vals_n)
    }
    all_level_order <- c(all_level_order, lv_use)

    for (lv in lv_use) {
      in_n <- match(lv, vals_n)
      in_p <- match(lv, vals_p)
      if (is.na(in_n) || is.na(in_p)) {
        next
      }

      for (gr in groups_use) {
        row_df <- data.frame(
          variable = labels[i],
          level = lv,
          group = gr,
          n = suppressWarnings(as.numeric(ct_n[in_n, gr])),
          pct = suppressWarnings(as.numeric(ct_pct[in_p, gr])),
          p = st$p,
          p_op = st$p_op,
          ci_lower = st$ci_lower,
          ci_upper = st$ci_upper,
          stringsAsFactors = FALSE,
          check.names = FALSE
        )
        if (show_assoc) {
          row_df$.assoc <- st$v
          names(row_df)[names(row_df) == ".assoc"] <- measure_col
        }
        rows[[rr]] <- row_df
        rr <- rr + 1L
      }
    }
  }

  if (show_assoc && is.null(measure_col)) {
    measure_col <- "Cramer's V"
  }

  if (length(rows) == 0) {
    long_raw <- data.frame(
      variable = character(0),
      level = character(0),
      group = character(0),
      n = numeric(0),
      pct = numeric(0),
      p = numeric(0),
      p_op = character(0),
      .assoc = numeric(0),
      ci_lower = numeric(0),
      ci_upper = numeric(0),
      stringsAsFactors = FALSE,
      check.names = FALSE
    )
    if (show_assoc) {
      names(long_raw)[names(long_raw) == ".assoc"] <- measure_col
    } else {
      long_raw$.assoc <- NULL
    }
  } else {
    long_raw <- do.call(rbind, rows)
  }

  if (nrow(long_raw) > 0) {
    long_raw$variable <- factor(long_raw$variable, levels = labels)
    if (!is.null(levels_keep)) {
      long_raw$level <- factor(
        long_raw$level,
        levels = as.character(levels_keep)
      )
    } else {
      long_raw$level <- factor(
        long_raw$level,
        levels = unique(all_level_order)
      )
    }
    long_raw$group <- factor(long_raw$group, levels = group_levels)
    long_raw <- long_raw[
      order(long_raw$variable, long_raw$level, long_raw$group),
      ,
      drop = FALSE
    ]
    long_raw$variable <- as.character(long_raw$variable)
    long_raw$level <- as.character(long_raw$level)
    long_raw$group <- as.character(long_raw$group)
    rownames(long_raw) <- NULL
  }

  if (output == "long") {
    out <- long_raw
    out$p_op <- NULL
    if (!show_assoc || !assoc_ci) {
      out$ci_lower <- NULL
      out$ci_upper <- NULL
    }
    return(out)
  }

  # ---------------- WIDE RAW ----------------
  make_wide_raw <- function(ldf) {
    cols <- c(
      "Variable",
      "Level",
      as.vector(rbind(paste0(group_levels, " n"), paste0(group_levels, " %"))),
      "p"
    )
    if (show_assoc) {
      cols <- c(cols, measure_col)
    }
    if (show_assoc && assoc_ci) {
      cols <- c(cols, "CI lower", "CI upper")
    }
    if (nrow(ldf) == 0) {
      return(as.data.frame(
        setNames(replicate(length(cols), character(0), simplify = FALSE), cols),
        check.names = FALSE
      ))
    }

    key <- unique(ldf[, c("variable", "level"), drop = FALSE])
    out <- vector("list", nrow(key))

    for (k in seq_len(nrow(key))) {
      sv <- ldf[
        ldf$variable == key$variable[k] & ldf$level == key$level[k],
        ,
        drop = FALSE
      ]
      r <- as.list(setNames(rep(NA, length(cols)), cols))
      r$Variable <- key$variable[k]
      r$Level <- key$level[k]

      for (gr in group_levels) {
        s <- sv[sv$group == gr, , drop = FALSE]
        r[[paste0(gr, " n")]] <- if (nrow(s)) s$n[1] else NA_real_
        r[[paste0(gr, " %")]] <- if (nrow(s)) s$pct[1] else NA_real_
      }

      r$p <- if (nrow(sv)) sv$p[1] else NA_real_
      if (show_assoc) {
        r[[measure_col]] <- if (nrow(sv)) sv[[measure_col]][1] else NA_real_
      }
      if (show_assoc && assoc_ci) {
        r[["CI lower"]] <- if (nrow(sv)) sv$ci_lower[1] else NA_real_
        r[["CI upper"]] <- if (nrow(sv)) sv$ci_upper[1] else NA_real_
      }

      out[[k]] <- as.data.frame(
        r,
        stringsAsFactors = FALSE,
        check.names = FALSE
      )
    }

    w <- do.call(rbind, out)

    if (blank_na_wide) {
      for (j in seq_len(ncol(w))) {
        if (j > 2) w[[j]] <- ifelse(is.na(w[[j]]), "", as.character(w[[j]]))
      }
    }
    w
  }

  wide_raw <- make_wide_raw(long_raw)
  if (output == "data.frame") {
    return(wide_raw)
  }

  # ---------------- REPORT WIDE ----------------
  report_cols <- c(
    "Variable",
    as.vector(rbind(paste0(group_levels, " n"), paste0(group_levels, " %"))),
    "p"
  )
  if (show_assoc) {
    report_cols <- c(report_cols, measure_col)
  }
  if (show_assoc && assoc_ci) {
    report_cols <- c(report_cols, "CI lower", "CI upper")
  }

  make_report_wide <- function(ldf, mode = c("char", "excel")) {
    mode <- match.arg(mode)

    if (nrow(ldf) == 0) {
      if (mode == "char") {
        return(as.data.frame(
          setNames(
            replicate(length(report_cols), character(0), simplify = FALSE),
            report_cols
          ),
          check.names = FALSE
        ))
      } else {
        out <- as.data.frame(
          setNames(
            replicate(length(report_cols), numeric(0), simplify = FALSE),
            report_cols
          ),
          check.names = FALSE
        )
        out$Variable <- character(0)
        out$p <- character(0)
        return(out[, report_cols, drop = FALSE])
      }
    }

    out <- list()
    z <- 1L

    for (lab in labels) {
      sv <- ldf[ldf$variable == lab, , drop = FALSE]
      if (nrow(sv) == 0) {
        next
      }

      lv_use <- if (is.null(levels_keep)) {
        unique(sv$level)
      } else {
        intersect(as.character(levels_keep), unique(sv$level))
      }

      # variable row
      if (mode == "char") {
        r0 <- as.list(setNames(rep("", length(report_cols)), report_cols))
      } else {
        r0 <- as.list(setNames(rep(NA, length(report_cols)), report_cols))
      }
      r0$Variable <- lab
      r0$p <- fmt_p(sv$p[1], sv$p_op[1])
      if (show_assoc) {
        r0[[measure_col]] <- fmt_v(sv[[measure_col]][1])
      }
      if (show_assoc && assoc_ci) {
        if (mode == "char") {
          r0[["CI lower"]] <- fmt_v(sv$ci_lower[1])
          r0[["CI upper"]] <- fmt_v(sv$ci_upper[1])
        } else {
          r0[["CI lower"]] <- sv$ci_lower[1]
          r0[["CI upper"]] <- sv$ci_upper[1]
        }
      }
      out[[z]] <- as.data.frame(
        r0,
        stringsAsFactors = FALSE,
        check.names = FALSE
      )
      z <- z + 1L

      # modality rows
      for (lv in lv_use) {
        sl <- sv[sv$level == lv, , drop = FALSE]
        if (mode == "char") {
          r1 <- as.list(setNames(rep("", length(report_cols)), report_cols))
        } else {
          r1 <- as.list(setNames(rep(NA, length(report_cols)), report_cols))
        }
        r1$Variable <- paste0(indent_text, lv)

        for (gr in group_levels) {
          sx <- sl[sl$group == gr, , drop = FALSE]
          n_val <- if (nrow(sx)) sx$n[1] else NA_real_
          p_val <- if (nrow(sx)) sx$pct[1] else NA_real_

          if (mode == "char") {
            r1[[paste0(gr, " n")]] <- fmt_n(n_val)
            r1[[paste0(gr, " %")]] <- fmt_num(p_val, percent_digits)
          } else {
            r1[[paste0(gr, " n")]] <- n_val
            r1[[paste0(gr, " %")]] <- p_val
          }
        }

        r1$p <- ""
        if (show_assoc) {
          r1[[measure_col]] <- ""
        }
        out[[z]] <- as.data.frame(
          r1,
          stringsAsFactors = FALSE,
          check.names = FALSE
        )
        z <- z + 1L
      }
    }

    do.call(rbind, out)
  }

  report_wide_char <- make_report_wide(long_raw, mode = "char")
  report_wide_excel <- make_report_wide(long_raw, mode = "excel")
  if (output == "default") {
    out <- wide_raw
    attr(out, "display_df") <- report_wide_char
    attr(out, "group_var") <- by_name
    attr(out, "indent_text") <- indent_text
    class(out) <- c("spicy_categorical_table", "spicy_table", "data.frame")
    print(out)
    return(invisible(out))
  }

  # For rendered formats: merge CI inline into measure column, drop CI cols
  merge_ci_inline <- function(df) {
    if (!show_assoc || !assoc_ci || !("CI lower" %in% names(df))) {
      return(df)
    }
    has_val <- nzchar(df[[measure_col]]) & nzchar(df[["CI lower"]])
    df[[measure_col]][has_val] <- paste0(
      df[[measure_col]][has_val],
      " [",
      df[["CI lower"]][has_val],
      ", ",
      df[["CI upper"]][has_val],
      "]"
    )
    df[["CI lower"]] <- NULL
    df[["CI upper"]] <- NULL
    df
  }

  # Headers (base: without CI; used by rendered formats)
  top_header_span <- c(
    "Variable",
    rep(group_levels, each = 2),
    "p"
  )
  top_header_flat <- c(
    "Variable",
    as.vector(rbind(group_levels, rep("", length(group_levels)))),
    "p"
  )
  bot_header <- c("", rep(c("n", "%"), times = length(group_levels)), "")
  if (show_assoc) {
    top_header_span <- c(top_header_span, measure_col)
    top_header_flat <- c(top_header_flat, measure_col)
    bot_header <- c(bot_header, "")
  }
  grp_j <- 2:(1 + 2 * length(group_levels))

  # ---------------- tinytable ----------------
  if (output == "tinytable") {
    if (!requireNamespace("tinytable", quietly = TRUE)) {
      stop("Install package 'tinytable'.", call. = FALSE)
    }

    old_tt_opt <- getOption("tinytable_print_output")
    options(tinytable_print_output = "html") # RStudio Viewer
    on.exit(options(tinytable_print_output = old_tt_opt), add = TRUE)

    dat_tt <- merge_ci_inline(report_wide_char)

    # Detect modality rows before header rename
    mod_rows <- which(startsWith(dat_tt[[1]], indent_text))
    if (length(mod_rows)) {
      dat_tt[[1]][mod_rows] <- paste0(
        strrep("\u00A0", 4),
        substring(dat_tt[[1]][mod_rows], nchar(indent_text) + 1L)
      )
    }

    colnames(dat_tt) <- c(
      "",
      rep(c("n", "%"), times = length(group_levels)),
      rep("", 1L + as.integer(show_assoc))
    )

    # Spanners
    gspec <- c(
      list("Variable" = 1),
      setNames(
        lapply(seq_along(group_levels), function(i) c(2 * i, 2 * i + 1)),
        group_levels
      ),
      list("p" = ncol(dat_tt) - if (show_assoc) 1L else 0L)
    )
    if (show_assoc) {
      gspec[[measure_col]] <- ncol(dat_tt)
    }

    tt <- tinytable::tt(dat_tt, escape = FALSE)
    tt <- tinytable::group_tt(tt, j = gspec)
    tt <- tinytable::theme_empty(tt)

    # Alignment
    tt <- tinytable::style_tt(tt, j = 1, align = "l")
    data_j <- 2:(1 + 2 * length(group_levels))
    stat_j <- if (show_assoc) {
      (ncol(dat_tt) - 1):ncol(dat_tt)
    } else {
      ncol(dat_tt)
    }
    tt <- tinytable::style_tt(tt, j = c(data_j, stat_j), align = "r")
    # Centre n/% labels (row 0 = column labels row)
    tt <- tinytable::style_tt(tt, i = 0, j = data_j, align = "c")
    # Centre spanner labels (row -1 = spanner row)
    tt <- tinytable::style_tt(tt, i = -1, j = 2:ncol(dat_tt), align = "c")
    if (length(mod_rows)) {
      tt <- tinytable::style_tt(tt, i = mod_rows, j = 1, indent = 1)
      tt <- tinytable::style_tt(
        tt,
        i = mod_rows,
        j = 1,
        html_css = "padding-left: 0.8em;"
      )
    }

    # Lines
    grp_j <- 2:(1 + 2 * length(group_levels))

    # Top of table
    tt <- tinytable::style_tt(
      tt,
      i = -1,
      j = seq_len(ncol(dat_tt)),
      line = "t",
      line_width = 0.06
    )
    # Intermediate line under spanner: group columns only
    tt <- tinytable::style_tt(
      tt,
      i = -1,
      j = grp_j,
      line = "b",
      line_width = 0.06
    )
    # Line under n/% header: full width
    tt <- tinytable::style_tt(
      tt,
      i = 0,
      j = seq_len(ncol(dat_tt)),
      line = "b",
      line_width = 0.06
    )
    # Bottom closing line
    tt <- tinytable::style_tt(
      tt,
      i = nrow(dat_tt),
      j = seq_len(ncol(dat_tt)),
      line = "b",
      line_width = 0.06
    )
    # Prevent p-value and measure columns from wrapping
    tt <- tinytable::style_tt(
      tt,
      j = stat_j,
      html_css = "white-space: nowrap;"
    )

    return(tt)
  }

  # ---------------- gt ----------------
  if (output == "gt") {
    if (!requireNamespace("gt", quietly = TRUE)) {
      stop("Install package 'gt'.", call. = FALSE)
    }

    dat_gt <- merge_ci_inline(report_wide_char)

    # Indent modality rows with non-breaking spaces
    mod_rows <- which(startsWith(dat_gt[[1]], indent_text))
    if (length(mod_rows)) {
      dat_gt[[1]][mod_rows] <- paste0(
        strrep("\u00A0", 4),
        substring(dat_gt[[1]][mod_rows], nchar(indent_text) + 1L)
      )
    }

    # Rename n/% columns to unique names for gt, then relabel
    col_ids <- character(ncol(dat_gt))
    col_ids[1] <- "Variable"
    for (gi in seq_along(group_levels)) {
      col_ids[2 * gi] <- paste0(group_levels[gi], "_n")
      col_ids[2 * gi + 1] <- paste0(group_levels[gi], "_pct")
    }
    p_col_pos <- ncol(dat_gt) - if (show_assoc) 1L else 0L
    col_ids[p_col_pos] <- "p"
    if (show_assoc) {
      col_ids[ncol(dat_gt)] <- "assoc_col"
    }
    names(dat_gt) <- col_ids

    tbl <- gt::gt(dat_gt)

    # Column labels: n / % under each group; empty for single-col spanners
    label_list <- list()
    label_list[["Variable"]] <- ""
    for (gi in seq_along(group_levels)) {
      label_list[[paste0(group_levels[gi], "_n")]] <- "n"
      label_list[[paste0(group_levels[gi], "_pct")]] <- "%"
    }
    label_list[["p"]] <- ""
    if (show_assoc) {
      label_list[["assoc_col"]] <- ""
    }
    tbl <- gt::cols_label(tbl, .list = label_list)

    # Spanners: group names over n/% pairs, single-col for Variable/p/V
    tbl <- gt::tab_spanner(
      tbl,
      label = "Variable",
      columns = "Variable",
      id = "spn_variable"
    )
    for (gi in seq_along(group_levels)) {
      tbl <- gt::tab_spanner(
        tbl,
        label = group_levels[gi],
        columns = c(
          paste0(group_levels[gi], "_n"),
          paste0(group_levels[gi], "_pct")
        )
      )
    }
    tbl <- gt::tab_spanner(
      tbl,
      label = "p",
      columns = "p",
      id = "spn_p"
    )
    if (show_assoc) {
      tbl <- gt::tab_spanner(
        tbl,
        label = measure_col,
        columns = "assoc_col",
        id = "spn_v"
      )
    }

    # Alignment
    tbl <- gt::cols_align(tbl, align = "left", columns = "Variable")
    grp_cols <- unlist(lapply(group_levels, function(g) {
      c(paste0(g, "_n"), paste0(g, "_pct"))
    }))
    tbl <- gt::cols_align(tbl, align = "center", columns = grp_cols)
    right_cols <- "p"
    if (show_assoc) {
      right_cols <- c(right_cols, "assoc_col")
    }
    tbl <- gt::cols_align(tbl, align = "right", columns = right_cols)
    # Left-align the Variable spanner label
    tbl <- gt::tab_style(
      tbl,
      style = gt::cell_text(align = "left"),
      locations = gt::cells_column_spanners(spanners = "spn_variable")
    )

    # APA-style borders ------------------------------------------------
    # gt emits "border-bottom-style: hidden" on the spanner <tr>,
    # which wins in border-collapse:collapse and blocks tab_style().
    # We use opt_css(!important) for full control, plus tab_style()
    # so inline-CSS renderers (as_raw_html) also get the rules.
    rule <- gt::cell_borders(
      sides = "bottom",
      color = "currentColor",
      weight = gt::px(1)
    )
    rule_top <- gt::cell_borders(
      sides = "top",
      color = "currentColor",
      weight = gt::px(1)
    )

    # 1) Silence every default border.  Setting width to 0 is
    #    critical: gt defaults to 2px, and in border-collapse the
    #    wider border wins regardless of colour.
    tbl <- gt::tab_options(
      tbl,
      table.border.top.width = gt::px(0),
      table.border.bottom.width = gt::px(0),
      table_body.border.top.width = gt::px(0),
      table_body.border.bottom.width = gt::px(0),
      table_body.hlines.color = "transparent",
      column_labels.border.top.width = gt::px(0),
      column_labels.border.bottom.width = gt::px(0),
      column_labels.border.lr.color = "transparent"
    )

    # 2) tab_style rules (work in inline-CSS renderers)
    # Rule 1: top of spanners (full width)
    tbl <- gt::tab_style(
      tbl,
      style = rule_top,
      locations = gt::cells_column_spanners()
    )
    # Rule 2: intermediate line below spanners (group columns only)
    tbl <- gt::tab_style(
      tbl,
      style = rule_top,
      locations = gt::cells_column_labels(columns = grp_cols)
    )
    # Rule 3: below column labels (full width)
    tbl <- gt::tab_style(
      tbl,
      style = rule,
      locations = gt::cells_column_labels()
    )
    # Rule 4: bottom of last body row
    tbl <- gt::tab_style(
      tbl,
      style = rule,
      locations = gt::cells_body(rows = nrow(dat_gt))
    )

    # 3) opt_css rules (override gt's hidden borders in normal
    #    renderers: RStudio viewer, Quarto, pkgdown)
    # Build CSS selector for group-column <th> elements
    grp_css_sel <- paste(
      vapply(
        grp_cols,
        function(id) {
          sprintf('.gt_table thead tr:last-child th[id="%s"]', id)
        },
        character(1)
      ),
      collapse = ",\n"
    )
    apa_css <- paste(
      ".gt_table thead tr:first-child {",
      "  border-top: 1px solid currentColor !important;",
      "}",
      ".gt_table thead tr.gt_spanner_row {",
      "  border-bottom-style: none !important;",
      "}",
      ".gt_table thead th, .gt_table thead td {",
      "  background-color: transparent !important;",
      "}",
      # Intermediate line: only group columns
      paste0(grp_css_sel, " {"),
      "  border-top: 1px solid currentColor !important;",
      "}",
      ".gt_table thead tr:last-child {",
      "  border-bottom: 1px solid currentColor !important;",
      "}",
      ".gt_table tbody tr:last-child {",
      "  border-bottom: 1px solid currentColor !important;",
      "}",
      ".gt_table tbody tr {",
      "  border-top-style: none !important;",
      "  border-bottom-style: none !important;",
      "}",
      sep = "\n"
    )
    tbl <- gt::opt_css(tbl, css = apa_css)

    return(tbl)
  }

  # ---------------- flextable / word ----------------
  build_flextable <- function(df) {
    if (!requireNamespace("flextable", quietly = TRUE)) {
      stop("Install package 'flextable'.", call. = FALSE)
    }
    ft <- flextable::flextable(df)

    map <- data.frame(
      col_keys = names(df),
      top = top_header_span,
      bottom = bot_header,
      stringsAsFactors = FALSE,
      check.names = FALSE
    )
    ft <- flextable::set_header_df(ft, mapping = map, key = "col_keys")
    ft <- flextable::merge_h(ft, part = "header")

    bd <- spicy_fp_border(color = "black", width = 1)

    ft <- flextable::align(ft, j = 1, part = "all", align = "left")
    ft <- flextable::align(ft, j = 2:ncol(df), part = "body", align = "right")
    # Centre n/% labels and spanner labels in header
    ft <- flextable::align(ft, j = grp_j, part = "header", align = "center")
    # Right-align p and association measure in header
    stat_j <- if (show_assoc) (ncol(df) - 1):ncol(df) else ncol(df)
    ft <- flextable::align(ft, j = stat_j, part = "header", align = "right")

    ft <- flextable::hline_top(ft, part = "header", border = bd)
    ft <- flextable::hline(ft, i = 1, j = grp_j, part = "header", border = bd)
    ft <- flextable::hline_bottom(ft, part = "header", border = bd)
    ft <- flextable::hline_bottom(ft, part = "body", border = bd)

    id_mod <- which(startsWith(df[[1]], indent_text))
    if (length(id_mod)) {
      ft <- flextable::padding(
        ft,
        i = id_mod,
        j = 1,
        part = "body",
        padding.left = 14
      )
    }

    flextable::autofit(ft)
  }

  if (output == "flextable") {
    ft <- build_flextable(merge_ci_inline(report_wide_char))
    if (!is.null(word_path) && nzchar(word_path)) {
      if (!requireNamespace("officer", quietly = TRUE)) {
        stop("Install package 'officer'.", call. = FALSE)
      }
      flextable::save_as_docx(ft, path = word_path)
    }
    return(ft)
  }

  if (output == "word") {
    if (is.null(word_path) || !nzchar(word_path)) {
      stop("Provide `word_path` for output = 'word'.", call. = FALSE)
    }
    ft <- build_flextable(merge_ci_inline(report_wide_char))
    flextable::save_as_docx(ft, path = word_path)
    return(invisible(word_path))
  }

  # Extend headers with CI columns for data/export formats
  if (assoc_ci) {
    top_header_flat_ex <- c(top_header_flat, "CI lower", "CI upper")
    bot_header_ex <- c(bot_header, "", "")
    top_header_span_ex <- c(top_header_span, "CI lower", "CI upper")
  } else {
    top_header_flat_ex <- top_header_flat
    bot_header_ex <- bot_header
    top_header_span_ex <- top_header_span
  }

  # ---------------- clipboard matrix ----------------
  clip_body <- report_wide_char
  clip_body$Variable <- make_stronger_indent(
    clip_body$Variable,
    indent_text,
    indent_text_excel_clipboard
  )
  to_excel_text <- function(x) ifelse(x == "", "", paste0("=\"", x, "\""))
  clip_body$p <- to_excel_text(clip_body$p)
  if (show_assoc) {
    clip_body[[measure_col]] <- to_excel_text(clip_body[[measure_col]])
  }
  if (show_assoc && assoc_ci) {
    clip_body[["CI lower"]] <- to_excel_text(clip_body[["CI lower"]])
    clip_body[["CI upper"]] <- to_excel_text(clip_body[["CI upper"]])
  }

  clip_mat <- rbind(top_header_flat_ex, bot_header_ex, as.matrix(clip_body))

  # ---------------- excel ----------------
  if (output == "excel") {
    if (is.null(excel_path) || !nzchar(excel_path)) {
      stop("Provide `excel_path` for output = 'excel'.", call. = FALSE)
    }
    if (!requireNamespace("openxlsx2", quietly = TRUE)) {
      stop("Install package 'openxlsx2'.", call. = FALSE)
    }

    wb <- openxlsx2::wb_workbook()
    wb <- openxlsx2::wb_add_worksheet(wb, excel_sheet)

    wb <- openxlsx2::wb_add_data(
      wb,
      x = as.data.frame(t(top_header_flat_ex), stringsAsFactors = FALSE),
      start_row = 1,
      col_names = FALSE
    )
    wb <- openxlsx2::wb_add_data(
      wb,
      x = as.data.frame(t(bot_header_ex), stringsAsFactors = FALSE),
      start_row = 2,
      col_names = FALSE
    )

    body_xl <- report_wide_excel
    body_xl$Variable <- make_stronger_indent(
      body_xl$Variable,
      indent_text,
      indent_text_excel_clipboard
    )
    body_xl$p <- report_wide_char$p
    if (show_assoc) {
      body_xl[[measure_col]] <- report_wide_char[[measure_col]]
    }
    if (show_assoc && assoc_ci) {
      body_xl[["CI lower"]] <- report_wide_char[["CI lower"]]
      body_xl[["CI upper"]] <- report_wide_char[["CI upper"]]
    }

    wb <- openxlsx2::wb_add_data(
      wb,
      x = body_xl,
      start_row = 3,
      col_names = FALSE,
      row_names = FALSE
    )

    nc <- ncol(body_xl)
    last_row <- 2 + nrow(body_xl)
    pct_fmt <- paste0("0.", paste(rep("0", percent_digits), collapse = ""))

    if (add_multilevel_header) {
      for (i in seq_along(group_levels)) {
        c1 <- 2 + (i - 1) * 2
        wb <- openxlsx2::wb_merge_cells(
          wb,
          dims = openxlsx2::wb_dims(rows = 1, cols = c1:(c1 + 1))
        )
      }
    }

    # Header alignment (center, vertically centered)
    wb <- openxlsx2::wb_add_cell_style(
      wb,
      dims = openxlsx2::wb_dims(rows = 1:2, cols = 1:nc),
      horizontal = "center",
      vertical = "center"
    )
    if (nrow(body_xl) > 0) {
      # Body alignment
      wb <- openxlsx2::wb_add_cell_style(
        wb,
        dims = openxlsx2::wb_dims(rows = 3:last_row, cols = 1),
        horizontal = "left"
      )
      wb <- openxlsx2::wb_add_cell_style(
        wb,
        dims = openxlsx2::wb_dims(rows = 3:last_row, cols = 2:nc),
        horizontal = "right"
      )
      # Text columns (p, assoc, CI) — force text format
      text_cols <- if (show_assoc && assoc_ci) {
        (nc - 3):nc
      } else if (show_assoc) {
        c(nc - 1, nc)
      } else {
        nc
      }
      wb <- openxlsx2::wb_add_numfmt(
        wb,
        dims = openxlsx2::wb_dims(rows = 3:last_row, cols = text_cols),
        numfmt = "@"
      )
    }

    # APA borders
    wb <- openxlsx2::wb_add_border(
      wb,
      dims = openxlsx2::wb_dims(rows = 1, cols = 1:nc),
      top_border = "thin"
    )
    wb <- openxlsx2::wb_add_border(
      wb,
      dims = openxlsx2::wb_dims(rows = 1, cols = grp_j),
      bottom_border = "thin"
    )
    wb <- openxlsx2::wb_add_border(
      wb,
      dims = openxlsx2::wb_dims(rows = 2, cols = 1:nc),
      bottom_border = "thin"
    )
    if (nrow(body_xl) > 0) {
      wb <- openxlsx2::wb_add_border(
        wb,
        dims = openxlsx2::wb_dims(rows = last_row, cols = 1:nc),
        bottom_border = "thin"
      )
    }

    # Number formats for n / % columns
    n_cols <- seq(2, 1 + 2 * length(group_levels), by = 2)
    p_cols <- n_cols + 1

    if (nrow(body_xl) > 0) {
      wb <- openxlsx2::wb_add_numfmt(
        wb,
        dims = openxlsx2::wb_dims(rows = 3:last_row, cols = n_cols),
        numfmt = "0"
      )
      wb <- openxlsx2::wb_add_numfmt(
        wb,
        dims = openxlsx2::wb_dims(rows = 3:last_row, cols = p_cols),
        numfmt = pct_fmt
      )
    }

    openxlsx2::wb_save(wb, excel_path, overwrite = TRUE)
    return(invisible(excel_path))
  }

  # ---------------- clipboard ----------------
  if (output == "clipboard") {
    if (!requireNamespace("clipr", quietly = TRUE)) {
      stop("Install package 'clipr'.", call. = FALSE)
    }
    lines <- apply(clip_mat, 1, function(x) {
      paste(x, collapse = clipboard_delim)
    })
    txt <- paste(lines, collapse = "\n")
    clipr::write_clip(txt)
    message("Categorical table copied to clipboard.")
    return(invisible(txt))
  }
}

collect_warnings <- function(expr) {
  warnings <- character()
  value <- withCallingHandlers(
    expr,
    warning = function(w) {
      warnings <<- c(warnings, conditionMessage(w))
      invokeRestart("muffleWarning")
    }
  )
  list(value = value, warnings = warnings)
}

test_that("table_categorical returns expected long raw structure", {
  df <- data.frame(
    grp = factor(c("A", "A", "B", "B", "A", "B")),
    v1 = c("Oui", "Non", "Oui", "Non", "Oui", "Non"),
    v2 = c("Oui", "Oui", "Non", "Non", "Oui", "Non")
  )

  out <- table_categorical(
    data = df,
    select = c(v1, v2),
    by = grp,
    labels = c("Var 1", "Var 2"),
    include_total = TRUE,
    simulate_p = FALSE,
    output = "long"
  )

  expect_s3_class(out, "data.frame")
  expect_true(all(
    c("variable", "level", "group", "n", "pct", "p", "Cramer's V") %in%
      names(out)
  ))
  expect_true(nrow(out) > 0)
})

test_that("table_categorical accepts weights as column name or numeric vector", {
  df <- data.frame(
    grp = c("A", "A", "B", "B", "A", "B"),
    v1 = c("Oui", "Non", "Oui", "Non", "Oui", "Non"),
    w = c(1, 2, 1, 3, 2, 1)
  )

  out_col <- table_categorical(
    data = df,
    select = "v1",
    by = "grp",
    labels = "Var 1",
    weights = "w",
    simulate_p = FALSE,
    output = "long"
  )

  out_vec <- table_categorical(
    data = df,
    select = "v1",
    by = "grp",
    labels = "Var 1",
    weights = df$w,
    simulate_p = FALSE,
    output = "long"
  )

  expect_equal(out_col$n, out_vec$n)
  expect_equal(out_col$pct, out_vec$pct)
})

test_that("table_categorical accepts weights as an unquoted column name", {
  df <- data.frame(
    grp = c("A", "A", "B", "B", "A", "B"),
    v1 = c("Oui", "Non", "Oui", "Non", "Oui", "Non"),
    w = c(1, 2, 1, 3, 2, 1)
  )

  expect_no_warning(
    out_bare <- table_categorical(
      data = df,
      select = "v1",
      by = "grp",
      labels = "Var 1",
      weights = w,
      simulate_p = FALSE,
      output = "long"
    )
  )

  out_char <- table_categorical(
    data = df,
    select = "v1",
    by = "grp",
    labels = "Var 1",
    weights = "w",
    simulate_p = FALSE,
    output = "long"
  )

  expect_equal(out_bare$n, out_char$n)
  expect_equal(out_bare$pct, out_char$pct)
})

test_that("table_categorical accepts tidyselect-style select and unquoted by", {
  df <- data.frame(
    grp = c("A", "A", "B", "B"),
    v1 = c("Oui", "Non", "Oui", "Non"),
    v2 = c("Oui", "Oui", "Non", "Non")
  )

  out <- table_categorical(
    data = df,
    select = tidyselect::starts_with("v"),
    by = grp,
    output = "data.frame"
  )

  expect_true(all(c("v1", "v2") %in% out$Variable))
})

test_that("table_categorical accepts by as a character object", {
  df <- data.frame(
    grp = c("A", "A", "B", "B"),
    v1 = c("Oui", "Non", "Oui", "Non")
  )

  by_col <- "grp"
  expect_no_warning(
    out <- table_categorical(
      data = df,
      select = "v1",
      by = by_col,
      output = "data.frame"
    )
  )

  expect_true("A n" %in% names(out))
  expect_true("B %" %in% names(out))
})

test_that("table_categorical validates by and select branches", {
  df <- data.frame(
    grp = c("A", "A", "B", "B"),
    v1 = c("Oui", "Non", "Oui", "Non"),
    v2 = c("x", "y", "x", "y")
  )

  expect_error(
    table_categorical(df, select = "v1", by = c(grp, v2)),
    "by"
  )

  expect_error(
    table_categorical(df, select = tidyselect::starts_with("zzz"), by = grp),
    "select"
  )
})

test_that("table_categorical works without by in long raw output", {
  df <- data.frame(
    v1 = c("Oui", "Non", "Oui", NA),
    v2 = c("A", "A", "B", "B")
  )

  out <- table_categorical(
    data = df,
    select = c(v1, v2),
    drop_na = FALSE,
    output = "long"
  )

  expect_true(all(c("variable", "level", "n", "pct") %in% names(out)))
  expect_false("group" %in% names(out))
  expect_true(any(grepl("Missing", out$level)))
})

test_that("table_categorical renames generated missing labels when needed", {
  df <- data.frame(v1 = c("(Missing)", NA, "Yes"))

  out <- table_categorical(
    data = df,
    select = v1,
    drop_na = FALSE,
    output = "long"
  )

  expect_true("(Missing)" %in% out$level)
  expect_true("(Missing_1)" %in% out$level)
})

test_that("table_categorical handles one-way empty results after dropping missing", {
  df <- data.frame(v1 = c(NA, NA))

  out_long <- table_categorical(
    data = df,
    select = v1,
    drop_na = TRUE,
    output = "long"
  )
  out_wide <- table_categorical(
    data = df,
    select = v1,
    drop_na = TRUE,
    output = "data.frame"
  )

  expect_equal(nrow(out_long), 0L)
  expect_equal(nrow(out_wide), 0L)
  expect_named(out_wide, c("Variable", "Level", "n", "%"))
})

test_that("table_categorical handles grouped empty results after dropping missing", {
  df <- data.frame(
    grp = c("A", "B"),
    v1 = c(NA, NA)
  )

  out_long <- table_categorical(
    data = df,
    select = v1,
    by = grp,
    drop_na = TRUE,
    output = "long"
  )
  out_wide <- table_categorical(
    data = df,
    select = v1,
    by = grp,
    drop_na = TRUE,
    output = "data.frame"
  )
  out_default <- table_categorical(
    data = df,
    select = v1,
    by = grp,
    drop_na = TRUE,
    output = "data.frame"
  )

  expect_equal(nrow(out_long), 0L)
  expect_equal(nrow(out_wide), 0L)
  expect_equal(nrow(out_default), 0L)
  expect_true("Variable" %in% names(out_wide))
  expect_true("Variable" %in% names(out_default))
})

test_that("table_categorical warns about ignored grouped options without by", {
  df <- data.frame(v1 = c("Oui", "Non", "Oui"))

  res <- collect_warnings(
    table_categorical(
      data = df,
      select = "v1",
      include_total = FALSE,
      correct = TRUE,
      simulate_p = TRUE,
      assoc_measure = "phi",
      assoc_ci = TRUE,
      output = "long"
    )
  )

  expect_true(any(grepl("include_total", res$warnings)))
  expect_true(any(grepl("correct", res$warnings)))
  expect_true(any(grepl("simulate_p", res$warnings)))
  expect_true(any(grepl("assoc_measure", res$warnings)))
  expect_true(any(grepl("assoc_ci", res$warnings)))
  expect_s3_class(res$value, "data.frame")
})

test_that("table_categorical default output prints ASCII and returns styled object", {
  printed <- capture.output(
    out <- table_categorical(
      sochealth,
      select = smoking,
      output = "default"
    )
  )

  expect_true(length(printed) > 0)
  expect_s3_class(out, "spicy_categorical_table")
})

test_that("table_categorical default output with output = 'data.frame' returns wide raw data", {
  out <- table_categorical(
    sochealth,
    select = smoking,
    output = "data.frame"
  )

  expect_s3_class(out, "data.frame")
  expect_true(all(c("Variable", "Level", "n", "%") %in% names(out)))
})

test_that("table_categorical validates weights and simulate_B", {
  df <- data.frame(
    grp = c("A", "A", "B", "B"),
    v1 = c("Oui", "Non", "Oui", "Non")
  )

  expect_error(
    table_categorical(
      data = df,
      select = "v1",
      by = "grp",
      labels = "Var 1",
      weights = c(1, 2),
      output = "long"
    ),
    "Numeric `weights` must have length `nrow(data)`.",
    fixed = TRUE
  )

  expect_error(
    table_categorical(
      data = df,
      select = "v1",
      by = "grp",
      labels = "Var 1",
      simulate_B = 0,
      output = "long"
    ),
    "`simulate_B` must be a positive integer.",
    fixed = TRUE
  )
})

test_that("table_categorical keeps missing values as explicit levels when drop_na is FALSE", {
  df <- data.frame(
    grp = c("A", "A", "B", NA),
    v1 = c("Oui", NA, "Non", "Oui"),
    stringsAsFactors = FALSE
  )

  out_keep <- table_categorical(
    data = df,
    select = "v1",
    by = "grp",
    labels = "Var 1",
    drop_na = FALSE,
    simulate_p = FALSE,
    output = "long"
  )

  out_drop <- table_categorical(
    data = df,
    select = "v1",
    by = "grp",
    labels = "Var 1",
    drop_na = TRUE,
    simulate_p = FALSE,
    output = "long"
  )

  expect_true(any(grepl("^\\(Missing", out_keep$level)))
  expect_true(any(grepl("^\\(Missing", out_keep$group)))
  expect_false(any(grepl("^\\(Missing", out_drop$level)))
  expect_false(any(grepl("^\\(Missing", out_drop$group)))
})

test_that("table_categorical returns tinytable object when requested", {
  skip_if_not_installed("tinytable")

  df <- data.frame(
    grp = c("A", "A", "B", "B", "A", "B"),
    v1 = c("Oui", "Non", "Oui", "Non", "Oui", "Non")
  )

  tt <- table_categorical(
    data = df,
    select = "v1",
    by = "grp",
    labels = "Var 1",
    simulate_p = FALSE,
    output = "tinytable"
  )

  expect_true(methods::is(tt, "tinytable"))
})

test_that("table_categorical returns one-way rendered objects when requested", {
  skip_if_not_installed("tinytable")
  skip_if_not_installed("gt")
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")
  skip_if_not_installed("openxlsx2")
  skip_if_not_installed("clipr")

  tt <- table_categorical(
    sochealth,
    select = smoking,
    output = "tinytable"
  )
  expect_true(methods::is(tt, "tinytable"))

  gt_tbl <- table_categorical(
    sochealth,
    select = smoking,
    output = "gt"
  )
  expect_s3_class(gt_tbl, "gt_tbl")

  ft <- table_categorical(
    sochealth,
    select = smoking,
    output = "flextable"
  )
  expect_s3_class(ft, "flextable")

  tmp_xlsx <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp_xlsx), add = TRUE)
  expect_invisible(
    table_categorical(
      sochealth,
      select = smoking,
      output = "excel",
      excel_path = tmp_xlsx
    )
  )
  expect_true(file.exists(tmp_xlsx))

  tmp_docx <- tempfile(fileext = ".docx")
  on.exit(unlink(tmp_docx), add = TRUE)
  expect_identical(
    table_categorical(
      sochealth,
      select = smoking,
      output = "word",
      word_path = tmp_docx
    ),
    invisible(tmp_docx)
  )
  expect_true(file.exists(tmp_docx))

  clip_text <- NULL
  ns <- asNamespace("clipr")
  old_write <- get("write_clip", envir = ns)
  unlockBinding("write_clip", ns)
  assign(
    "write_clip",
    function(x, ...) {
      clip_text <<- x
      invisible(NULL)
    },
    envir = ns
  )
  lockBinding("write_clip", ns)
  on.exit(
    {
      unlockBinding("write_clip", ns)
      assign("write_clip", old_write, envir = ns)
      lockBinding("write_clip", ns)
    },
    add = TRUE
  )

  txt <- table_categorical(
    sochealth,
    select = smoking,
    output = "clipboard"
  )
  expect_type(txt, "character")
  expect_identical(txt, invisible(txt))
  expect_match(clip_text, "Variable")
})

test_that("table_categorical returns gt object when requested", {
  skip_if_not_installed("gt")

  df <- data.frame(
    grp = c("A", "A", "B", "B", "A", "B"),
    v1 = c("Oui", "Non", "Oui", "Non", "Oui", "Non")
  )

  gt_tbl <- table_categorical(
    data = df,
    select = "v1",
    by = "grp",
    labels = "Var 1",
    simulate_p = FALSE,
    output = "gt"
  )

  expect_s3_class(gt_tbl, "gt_tbl")
})

# Ã¢â€â‚¬Ã¢â€â‚¬ Dynamic association measure column Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬

test_that("table_categorical default column is Cramer's V", {
  df <- data.frame(
    grp = factor(c("A", "A", "B", "B", "A", "B")),
    v1 = c("Oui", "Non", "Oui", "Non", "Oui", "Non")
  )
  out <- table_categorical(
    df,
    "v1",
    "grp",
    labels = "Var 1",
    output = "long"
  )
  expect_true("Cramer's V" %in% names(out))
})

test_that("table_categorical drops association column when assoc_measure is none", {
  out_long <- table_categorical(
    sochealth,
    select = smoking,
    by = education,
    assoc_measure = "none",
    output = "long"
  )

  out_wide <- table_categorical(
    sochealth,
    select = smoking,
    by = education,
    assoc_measure = "none",
    output = "data.frame"
  )

  expect_false("Cramer's V" %in% names(out_long))
  expect_false(any(grepl("Cramer's V", names(out_wide), fixed = TRUE)))
  expect_true("p" %in% names(out_long))
  expect_true("p" %in% names(out_wide))
})

test_that("table_categorical uses dynamic column name with assoc_measure = 'gamma'", {
  df <- data.frame(
    grp = factor(c("A", "A", "B", "B", "A", "B")),
    v1 = c("Oui", "Non", "Oui", "Non", "Oui", "Non")
  )
  out <- table_categorical(
    df,
    "v1",
    "grp",
    labels = "Var 1",
    assoc_measure = "gamma",
    output = "long"
  )
  expect_true("Goodman-Kruskal Gamma" %in% names(out))
  expect_false("Cramer's V" %in% names(out))
})

test_that("assoc_ci adds CI columns in wide raw output", {
  out <- table_categorical(
    sochealth,
    "smoking",
    "education",
    output = "data.frame",
    assoc_ci = TRUE
  )
  expect_true("CI lower" %in% names(out))
  expect_true("CI upper" %in% names(out))
  expect_true(is.numeric(out[["CI lower"]]))
  expect_true(all(!is.na(out[["CI lower"]])))
})

test_that("assoc_ci = FALSE omits CI columns in wide raw output", {
  out <- table_categorical(
    sochealth,
    "smoking",
    "education",
    output = "data.frame",
    assoc_ci = FALSE
  )
  expect_false("CI lower" %in% names(out))
  expect_false("CI upper" %in% names(out))
})

test_that("assoc_ci adds CI columns in long raw output", {
  out <- table_categorical(
    sochealth,
    "smoking",
    "education",
    output = "long",
    assoc_ci = TRUE
  )
  expect_true("ci_lower" %in% names(out))
  expect_true("ci_upper" %in% names(out))
  expect_true(is.numeric(out$ci_lower))
})

test_that("assoc_ci shows inline CI in rendered formats", {
  skip_if_not_installed("gt")
  gt_out <- table_categorical(
    sochealth,
    "smoking",
    "education",
    output = "gt",
    assoc_ci = TRUE
  )
  dat <- gt_out[["_data"]]
  expect_match(dat$assoc_col[1], "\\[")
  expect_false("CI lower" %in% names(dat))
})
# Ã¢â€â‚¬Ã¢â€â‚¬ levels_keep Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬

test_that("table_categorical levels_keep filters and reorders levels", {
  out <- table_categorical(
    sochealth,
    "smoking",
    "education",
    levels_keep = c("Yes"),
    output = "data.frame"
  )
  expect_true(all(out$Level == "Yes", na.rm = TRUE))
})

test_that("table_categorical levels_keep with (Missing)", {
  out <- table_categorical(
    sochealth,
    "income_group",
    "education",
    drop_na = FALSE,
    levels_keep = c("Low", "High", "(Missing)"),
    output = "data.frame"
  )
  lvls <- out$Level[!is.na(out$Level) & out$Level != ""]
  expect_equal(lvls, c("Low", "High", "(Missing)"))
})
# Ã¢â€â‚¬Ã¢â€â‚¬ blank_na_wide Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬

test_that("table_categorical blank_na_wide replaces NA with empty strings", {
  out <- table_categorical(
    sochealth,
    "smoking",
    "education",
    output = "data.frame",
    blank_na_wide = TRUE
  )
  chr_cols <- vapply(out, is.character, logical(1))
  if (any(chr_cols)) {
    expect_false(any(is.na(out[chr_cols])))
  }
})

# Ã¢â€â‚¬Ã¢â€â‚¬ Validation errors Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬

test_that("table_categorical validates data argument", {
  expect_error(
    table_categorical("not_df", "v1", "grp"),
    "`data` must be a data.frame"
  )
})

test_that("table_categorical validates select", {
  df <- data.frame(g = 1, v = 1)
  expect_error(
    table_categorical(df, character(0), "g"),
    "`select` must select at least one column"
  )
  expect_error(
    table_categorical(df, "missing", "g"),
    "Some `select` columns are missing"
  )
})

test_that("table_categorical validates by", {
  df <- data.frame(g = 1, v = 1)
  expect_error(
    table_categorical(df, "v", "missing"),
    "`by` must select exactly one column"
  )
})

test_that("table_categorical validates labels length", {
  df <- data.frame(g = c("A", "B"), v = c("x", "y"))
  expect_error(
    table_categorical(df, "v", "g", labels = c("a", "b")),
    "`labels` must have same length"
  )
})

test_that("table_categorical validates boolean arguments", {
  df <- data.frame(g = c("A", "B"), v = c("x", "y"))
  expect_error(
    table_categorical(df, "v", "g", include_total = NA),
    "`include_total` must be"
  )
  expect_error(
    table_categorical(df, "v", "g", drop_na = "yes"),
    "`drop_na` must be"
  )
  expect_error(
    table_categorical(df, "v", "g", rescale = NA),
    "`rescale` must be"
  )
  expect_error(
    table_categorical(df, "v", "g", correct = NA),
    "`correct` must be"
  )
  expect_error(
    table_categorical(df, "v", "g", simulate_p = NA),
    "`simulate_p` must be"
  )
  expect_error(
    table_categorical(df, "v", "g", blank_na_wide = NA),
    "`blank_na_wide` must be"
  )
  expect_error(
    table_categorical(df, "v", "g", add_multilevel_header = NA),
    "`add_multilevel_header` must be"
  )
})

test_that("table_categorical validates decimal_mark", {
  df <- data.frame(g = c("A", "B"), v = c("x", "y"))
  expect_error(
    table_categorical(df, "v", "g", decimal_mark = ";"),
    "`decimal_mark` must be"
  )
})

test_that("table_categorical validates weights type", {
  df <- data.frame(g = c("A", "B"), v = c("x", "y"))
  expect_error(
    table_categorical(df, "v", "g", weights = TRUE),
    "`weights` must be NULL"
  )
})

test_that("table_categorical validates weights column name", {
  df <- data.frame(g = c("A", "B"), v = c("x", "y"))
  expect_error(
    table_categorical(df, "v", "g", weights = "nonexistent"),
    "column name in `data`"
  )
})

test_that("table_categorical warns when rescale = TRUE without weights", {
  df <- data.frame(g = c("A", "B"), v = c("x", "y"))
  expect_warning(
    table_categorical(df, "v", "g", rescale = TRUE, output = "data.frame"),
    "rescale = TRUE.*no effect"
  )
})

# Ã¢â€â‚¬Ã¢â€â‚¬ Multiple select Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬

test_that("table_categorical handles multiple select in wide output", {
  out <- table_categorical(
    sochealth,
    c(smoking, physical_activity),
    education,
    output = "data.frame"
  )
  expect_true(all(c("smoking", "physical_activity") %in% out$Variable))
})

# Ã¢â€â‚¬Ã¢â€â‚¬ include_total = FALSE Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬

test_that("table_categorical include_total = FALSE omits Total column", {
  out <- table_categorical(
    sochealth,
    "smoking",
    "education",
    include_total = FALSE,
    output = "data.frame"
  )
  expect_false(any(grepl("^Total", names(out))))
})

# Ã¢â€â‚¬Ã¢â€â‚¬ Flextable output Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬

test_that("table_categorical returns flextable object when requested", {
  skip_if_not_installed("flextable")
  ft <- table_categorical(
    sochealth,
    "smoking",
    "education",
    output = "flextable"
  )
  expect_s3_class(ft, "flextable")
})

test_that("table_categorical grouped word and clipboard outputs work", {
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")
  skip_if_not_installed("clipr")

  tmp_docx <- tempfile(fileext = ".docx")
  on.exit(unlink(tmp_docx), add = TRUE)
  path <- table_categorical(
    sochealth,
    "smoking",
    "education",
    output = "word",
    word_path = tmp_docx
  )
  expect_identical(path, invisible(tmp_docx))
  expect_true(file.exists(tmp_docx))

  clip_text <- NULL
  ns <- asNamespace("clipr")
  old_write <- get("write_clip", envir = ns)
  unlockBinding("write_clip", ns)
  assign(
    "write_clip",
    function(x, ...) {
      clip_text <<- x
      invisible(NULL)
    },
    envir = ns
  )
  lockBinding("write_clip", ns)
  on.exit(
    {
      unlockBinding("write_clip", ns)
      assign("write_clip", old_write, envir = ns)
      lockBinding("write_clip", ns)
    },
    add = TRUE
  )

  txt <- table_categorical(
    sochealth,
    "smoking",
    "education",
    output = "clipboard",
    assoc_ci = TRUE
  )
  expect_type(txt, "character")
  expect_match(clip_text, "Cramer's V")
  expect_match(clip_text, "CI lower")
})

test_that("table_categorical requires file paths for word and excel outputs", {
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")
  skip_if_not_installed("openxlsx2")

  expect_error(
    table_categorical(sochealth, "smoking", output = "word"),
    "word_path"
  )
  expect_error(
    table_categorical(sochealth, "smoking", output = "excel"),
    "excel_path"
  )
})

# Ã¢â€â‚¬Ã¢â€â‚¬ Excel output Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬

test_that("table_categorical writes excel file", {
  skip_if_not_installed("openxlsx2")
  tmp <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp), add = TRUE)
  table_categorical(
    sochealth,
    "smoking",
    "education",
    output = "excel",
    excel_path = tmp
  )
  expect_true(file.exists(tmp))
})

# Ã¢â€â‚¬Ã¢â€â‚¬ assoc_measure = "none" Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬

test_that("table_categorical assoc_measure = 'none' omits association column", {
  out <- table_categorical(
    sochealth,
    "smoking",
    "education",
    assoc_measure = "none",
    output = "long"
  )
  expect_false("Cramer's V" %in% names(out))
})
test_that("table_categorical with assoc_ci includes CI columns in raw long", {
  out <- table_categorical(
    sochealth,
    "smoking",
    "education",
    assoc_ci = TRUE,
    output = "long"
  )
  expect_true("CI lower" %in% names(out) || "ci_lower" %in% names(out))
})

test_that("table_categorical simulate_p works in long output", {
  out <- table_categorical(
    sochealth,
    "smoking",
    "education",
    simulate_p = TRUE,
    output = "long"
  )
  expect_s3_class(out, "data.frame")
  expect_true(nrow(out) > 0)
})

test_that("table_categorical with drop_na = FALSE includes Missing level", {
  df <- sochealth
  df$smoking[1:5] <- NA
  out <- table_categorical(
    df,
    "smoking",
    "education",
    drop_na = FALSE,
    output = "long"
  )
  expect_true(any(grepl("Missing", out$level)))
})

# Ã¢â€â‚¬Ã¢â€â‚¬ Digit validation Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬

test_that("table_categorical rejects invalid digit arguments", {
  df <- data.frame(
    grp = c("A", "B", "A", "B"),
    v1 = c("x", "y", "x", "y")
  )

  expect_error(
    table_categorical(df, "v1", "grp", percent_digits = -1, output = "long"),
    "percent_digits"
  )
  expect_error(
    table_categorical(df, "v1", "grp", p_digits = "a", output = "long"),
    "p_digits"
  )
  expect_error(
    table_categorical(df, "v1", "grp", v_digits = NA, output = "long"),
    "v_digits"
  )
})

# Ã¢â€â‚¬Ã¢â€â‚¬ Level ordering Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬Ã¢â€â‚¬

test_that("table_categorical preserves factor level order in row variables", {
  df <- data.frame(
    grp = c("A", "A", "B", "B", "A", "B"),
    v1 = factor(
      c("Low", "High", "Medium", "Low", "High", "Medium"),
      levels = c("Low", "Medium", "High")
    )
  )
  out <- table_categorical(
    df,
    "v1",
    "grp",
    include_total = FALSE,
    output = "long"
  )
  lvs <- unique(out$level)
  expect_equal(lvs, c("Low", "Medium", "High"))
})

test_that("table_categorical places (Missing) at end when drop_na = FALSE", {
  df <- data.frame(
    grp = c("A", "A", "B", "B", "A", "B"),
    v1 = factor(
      c("Yes", NA, "No", "Yes", NA, "No"),
      levels = c("Yes", "No")
    )
  )
  out <- table_categorical(
    df,
    "v1",
    "grp",
    drop_na = FALSE,
    include_total = FALSE,
    output = "long"
  )
  lvs <- unique(out$level)
  expect_equal(lvs, c("Yes", "No", "(Missing)"))
})

test_that("table_categorical rescale warning includes call. = FALSE", {
  df <- data.frame(
    grp = c("A", "B", "A", "B"),
    v1 = c("x", "y", "x", "y")
  )
  w <- tryCatch(
    table_categorical(
      df,
      "v1",
      "grp",
      rescale = TRUE,
      output = "long"
    ),
    warning = function(w) w
  )
  expect_s3_class(w, "simpleWarning")
  expect_null(w$call)
})

# ---- grouped empty data returns character columns, not logical ----

test_that("grouped table with empty data returns character(0) columns", {
  df <- data.frame(
    x = factor(levels = c("a", "b")),
    g = factor(levels = c("A", "B")),
    stringsAsFactors = FALSE
  )
  out <- table_categorical(df, select = "x", by = "g", output = "data.frame")
  # All columns should be character, not logical
  col_types <- vapply(out, typeof, character(1))
  expect_true(all(col_types == "character"))
})

test_that("table_categorical gt output omits association header when assoc_measure = 'none'", {
  skip_if_not_installed("gt")

  gt_tbl <- table_categorical(
    sochealth,
    "smoking",
    "education",
    assoc_measure = "none",
    output = "gt"
  )

  boxhead <- gt_tbl[["_boxhead"]]
  spanners <- gt_tbl[["_spanners"]]

  expect_false(any(boxhead$column_label == "Cramer's V"))
  expect_false(any(boxhead$var == "assoc_col"))
  expect_false(any(spanners$spanner_id == "spn_assoc"))
  expect_false(any(spanners$spanner_label == "Cramer's V"))
})

# --- Coverage tests: uncovered paths ---

test_that("table_categorical errors when select matches no columns", {
  df <- data.frame(a = 1:3, b = 4:6)
  expect_error(
    table_categorical(df, select = c(nonexistent_col)),
    "select"
  )
})

test_that("table_categorical one-way with weights", {
  df <- data.frame(
    x = factor(c("A", "B", "A", "B", "A")),
    w = c(2, 1, 3, 1, 2)
  )
  out <- table_categorical(df, select = x, weights = w, output = "data.frame")
  expect_s3_class(out, "data.frame")
  expect_true(all(c("Variable", "Level", "n", "%") %in% names(out)))
  # Weighted n should reflect weights
  expect_equal(sum(out$n), sum(df$w))
})

test_that("table_categorical one-way with levels_keep filters levels", {
  df <- data.frame(
    x = factor(c("A", "B", "C", "A", "B"), levels = c("A", "B", "C"))
  )
  out <- table_categorical(
    df,
    select = x,
    levels_keep = c("B", "A"),
    output = "data.frame"
  )
  expect_equal(as.character(out$Level), c("B", "A"))
})

test_that("table_categorical one-way with levels_keep and long output", {
  df <- data.frame(
    x = factor(c("A", "B", "C", "A", "B"), levels = c("A", "B", "C"))
  )
  out <- table_categorical(
    df,
    select = x,
    levels_keep = c("B", "C"),
    output = "long"
  )
  expect_true(all(out$level %in% c("B", "C")))
})

test_that("table_categorical one-way with decimal_mark comma", {
  df <- data.frame(x = factor(c("A", "B", "A", "B", "A")))
  out <- table_categorical(
    df,
    select = x,
    decimal_mark = ",",
    output = "default"
  )
  disp <- attr(out, "display_df")
  # Percentages should use comma as decimal separator
  pct_col <- disp[["%"]]
  expect_true(any(grepl(",", pct_col)))
})

test_that("table_categorical one-way with blank_na_wide", {
  df <- data.frame(
    x = factor(c("A", NA, "B", NA)),
    y = factor(c("C", "D", NA, NA))
  )
  out <- table_categorical(
    df,
    select = c(x, y),
    drop_na = TRUE,
    blank_na_wide = TRUE,
    output = "data.frame"
  )
  expect_s3_class(out, "data.frame")
})

test_that("table_categorical one-way empty after dropping NA produces 0-row data.frame", {
  df <- data.frame(x = factor(c(NA, NA, NA)))
  out_wide <- table_categorical(
    df,
    select = x,
    drop_na = TRUE,
    output = "data.frame"
  )
  expect_equal(nrow(out_wide), 0L)
  out_long <- table_categorical(df, select = x, drop_na = TRUE, output = "long")
  expect_equal(nrow(out_long), 0L)
})

test_that("table_categorical grouped default output prints and returns invisibly", {
  out <- table_categorical(
    data = sochealth,
    select = smoking,
    by = sex
  )
  expect_s3_class(out, "spicy_categorical_table")
  expect_equal(attr(out, "group_var"), "sex")
})

test_that("table_categorical grouped with levels_keep filters levels", {
  out <- table_categorical(
    data = sochealth,
    select = smoking,
    by = sex,
    levels_keep = c("Yes"),
    output = "data.frame"
  )
  out_long <- table_categorical(
    data = sochealth,
    select = smoking,
    by = sex,
    levels_keep = c("Yes"),
    output = "long"
  )
  expect_true(all(out_long$level == "Yes"))
})

test_that("table_categorical grouped with assoc_measure = none", {
  out <- table_categorical(
    data = sochealth,
    select = smoking,
    by = sex,
    assoc_measure = "none",
    output = "long"
  )
  # No association measure column
  expect_false("Cramer's V" %in% names(out))
})

test_that("table_categorical grouped empty after dropping NA", {
  df <- data.frame(
    grp = factor(c("A", "B")),
    v = factor(c(NA, NA))
  )
  out_wide <- table_categorical(
    df,
    select = v,
    by = grp,
    drop_na = TRUE,
    output = "data.frame"
  )
  expect_equal(nrow(out_wide), 0L)
  out_long <- table_categorical(
    df,
    select = v,
    by = grp,
    drop_na = TRUE,
    output = "long"
  )
  expect_equal(nrow(out_long), 0L)
})

test_that("table_categorical one-way weighted with rescale", {
  df <- data.frame(
    x = factor(c("A", "B", "A", "B", "A")),
    w = c(10, 5, 10, 5, 10)
  )
  out <- table_categorical(
    df,
    select = x,
    weights = w,
    rescale = TRUE,
    output = "data.frame"
  )
  # After rescaling, total n should equal nrow(df)
  expect_equal(sum(out$n), nrow(df), tolerance = 0.01)
})

test_that("table_categorical handles Missing_ label collision", {
  df <- data.frame(
    x = factor(c("(Missing)", "(Missing_1)", NA, "B")),
    stringsAsFactors = FALSE
  )
  out <- table_categorical(df, select = x, drop_na = FALSE, output = "long")
  expect_true(any(grepl("Missing", out$level)))
  # Should not have duplicate level names
  expect_equal(length(unique(out$level)), nrow(out))
})

test_that("table_categorical grouped with levels_keep and default output", {
  out <- table_categorical(
    data = sochealth,
    select = smoking,
    by = sex,
    levels_keep = c("Yes")
  )
  expect_s3_class(out, "spicy_categorical_table")
  disp <- attr(out, "display_df")
  # Only "Yes" level should appear in indented rows
  indented <- disp$Variable[startsWith(disp$Variable, "  ")]
  expect_true(all(trimws(indented) == "Yes"))
})

test_that("table_categorical grouped empty via levels_keep with non-matching levels", {
  df <- data.frame(
    grp = factor(c("A", "B", "A", "B")),
    v = factor(c("x", "y", "x", "y"))
  )
  out <- table_categorical(
    df,
    select = v,
    by = grp,
    levels_keep = c("nonexistent"),
    output = "data.frame"
  )
  expect_equal(nrow(out), 0L)
  out_long <- table_categorical(
    df,
    select = v,
    by = grp,
    levels_keep = c("nonexistent"),
    output = "long"
  )
  expect_equal(nrow(out_long), 0L)
  # Also with assoc_measure = "none" to cover L1196
  out_none <- table_categorical(
    df,
    select = v,
    by = grp,
    levels_keep = c("nonexistent"),
    assoc_measure = "none",
    output = "long"
  )
  expect_equal(nrow(out_none), 0L)
})

test_that("table_categorical grouped with assoc_measure = none and default output", {
  out <- table_categorical(
    data = sochealth,
    select = smoking,
    by = sex,
    assoc_measure = "none"
  )
  expect_s3_class(out, "spicy_categorical_table")
})

test_that("table_categorical one-way with levels_keep that filters some levels", {
  df <- data.frame(
    x = factor(c("A", "B", "C", "D"), levels = c("A", "B", "C", "D")),
    y = factor(c("P", "Q", "P", "Q"))
  )
  out <- table_categorical(
    df,
    select = c(x, y),
    levels_keep = c("A", "C", "P"),
    output = "data.frame"
  )
  # Should only have matching levels
  expect_true(all(out$Level %in% c("A", "C", "P")))
})

test_that("table_categorical grouped with decimal_mark comma and default output", {
  out <- table_categorical(
    data = sochealth,
    select = smoking,
    by = sex,
    decimal_mark = ","
  )
  expect_s3_class(out, "spicy_categorical_table")
  disp <- attr(out, "display_df")
  # p-value and percentage columns should use comma
  p_col <- disp$p
  non_empty_p <- p_col[nzchar(p_col)]
  if (length(non_empty_p) > 0) {
    expect_true(any(grepl(",", non_empty_p)) || any(grepl("<", non_empty_p)))
  }
})

test_that("table_categorical grouped tinytable with assoc_measure = none", {
  skip_if_not_installed("tinytable")
  tt <- table_categorical(
    data = sochealth,
    select = smoking,
    by = sex,
    assoc_measure = "none",
    output = "tinytable"
  )
  expect_true(inherits(tt, "tinytable"))
})

test_that("table_categorical grouped excel with assoc_ci", {
  skip_if_not_installed("openxlsx2")
  tmp <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp), add = TRUE)
  path <- table_categorical(
    data = sochealth,
    select = smoking,
    by = sex,
    assoc_ci = TRUE,
    output = "excel",
    excel_path = tmp
  )
  expect_true(file.exists(path))
})

test_that("table_categorical grouped excel with assoc_measure = none", {
  skip_if_not_installed("openxlsx2")
  tmp <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp), add = TRUE)
  path <- table_categorical(
    data = sochealth,
    select = smoking,
    by = sex,
    assoc_measure = "none",
    output = "excel",
    excel_path = tmp
  )
  expect_true(file.exists(path))
})

test_that("table_categorical one-way all-NA renders empty default table", {
  df <- data.frame(x = factor(c(NA, NA, NA)))
  out <- table_categorical(df, select = x, drop_na = TRUE, output = "default")
  expect_s3_class(out, "spicy_categorical_table")
  expect_equal(nrow(out), 0L)
})

test_that("table_categorical grouped all-NA renders empty default table", {
  df <- data.frame(
    grp = factor(c("A", "B", "A")),
    v = factor(c(NA, NA, NA))
  )
  out <- table_categorical(
    df,
    select = v,
    by = grp,
    drop_na = TRUE,
    output = "default"
  )
  expect_s3_class(out, "spicy_categorical_table")
})

test_that("table_categorical one-way levels_keep with no match returns empty", {
  df <- data.frame(x = factor(c("A", "B"), levels = c("A", "B", "C")))
  out <- table_categorical(
    df,
    select = x,
    levels_keep = c("nonexistent"),
    output = "data.frame"
  )
  expect_equal(nrow(out), 0L)
  # levels_keep includes "C" which exists in factor levels but has 0 obs
  # → covers the `next` at match(lv, vals) returning NA
  out2 <- table_categorical(
    df,
    select = x,
    levels_keep = c("A", "C"),
    output = "data.frame"
  )
  expect_equal(nrow(out2), 1L)
  expect_equal(as.character(out2$Level), "A")
  # Also test default output path (covers make_report_wide_oneway empty path)
  out3 <- table_categorical(
    df,
    select = x,
    levels_keep = c("nonexistent"),
    output = "default"
  )
  expect_s3_class(out3, "spicy_categorical_table")
})

test_that("table_categorical errors for missing tinytable package", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) {
      if (pkg == "tinytable") {
        return(FALSE)
      }
      base::requireNamespace(pkg, ...)
    },
    .package = "base"
  )
  expect_error(
    table_categorical(sochealth, select = smoking, output = "tinytable"),
    "tinytable"
  )
})

test_that("table_categorical errors for missing gt package", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) {
      if (pkg == "gt") {
        return(FALSE)
      }
      base::requireNamespace(pkg, ...)
    },
    .package = "base"
  )
  expect_error(
    table_categorical(sochealth, select = smoking, output = "gt"),
    "gt"
  )
})

test_that("table_categorical errors for missing flextable package", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) {
      if (pkg == "flextable") {
        return(FALSE)
      }
      base::requireNamespace(pkg, ...)
    },
    .package = "base"
  )
  expect_error(
    table_categorical(sochealth, select = smoking, output = "flextable"),
    "flextable"
  )
})

test_that("table_categorical errors for missing openxlsx2 package", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) {
      if (pkg == "openxlsx2") {
        return(FALSE)
      }
      base::requireNamespace(pkg, ...)
    },
    .package = "base"
  )
  expect_error(
    table_categorical(
      sochealth,
      select = smoking,
      output = "excel",
      excel_path = tempfile(fileext = ".xlsx")
    ),
    "openxlsx2"
  )
})

test_that("table_categorical errors for missing clipr package", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) {
      if (pkg == "clipr") {
        return(FALSE)
      }
      base::requireNamespace(pkg, ...)
    },
    .package = "base"
  )
  expect_error(
    table_categorical(sochealth, select = smoking, output = "clipboard"),
    "clipr"
  )
})

test_that("table_categorical one-way word output errors for missing officer package", {
  skip("Cannot mock officer requireNamespace without recursion")
})

test_that("table_categorical word output errors when word_path is missing", {
  expect_error(
    table_categorical(sochealth, select = smoking, by = sex, output = "word"),
    "word_path"
  )
})

test_that("table_categorical excel output errors when excel_path is missing", {
  expect_error(
    table_categorical(
      sochealth,
      select = smoking,
      by = sex,
      output = "excel"
    ),
    "excel_path"
  )
})

test_that("table_categorical grouped errors for missing tinytable", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) {
      if (pkg == "tinytable") {
        return(FALSE)
      }
      base::requireNamespace(pkg, ...)
    },
    .package = "base"
  )
  expect_error(
    table_categorical(
      sochealth,
      select = smoking,
      by = sex,
      output = "tinytable"
    ),
    "tinytable"
  )
})

test_that("table_categorical grouped errors for missing gt", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) {
      if (pkg == "gt") {
        return(FALSE)
      }
      base::requireNamespace(pkg, ...)
    },
    .package = "base"
  )
  expect_error(
    table_categorical(sochealth, select = smoking, by = sex, output = "gt"),
    "gt"
  )
})

test_that("table_categorical grouped errors for missing flextable", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) {
      if (pkg == "flextable") {
        return(FALSE)
      }
      base::requireNamespace(pkg, ...)
    },
    .package = "base"
  )
  expect_error(
    table_categorical(
      sochealth,
      select = smoking,
      by = sex,
      output = "flextable"
    ),
    "flextable"
  )
})

test_that("table_categorical grouped errors for missing openxlsx2", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) {
      if (pkg == "openxlsx2") {
        return(FALSE)
      }
      base::requireNamespace(pkg, ...)
    },
    .package = "base"
  )
  expect_error(
    table_categorical(
      sochealth,
      select = smoking,
      by = sex,
      output = "excel",
      excel_path = tempfile(fileext = ".xlsx")
    ),
    "openxlsx2"
  )
})

test_that("table_categorical grouped errors for missing clipr", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) {
      if (pkg == "clipr") {
        return(FALSE)
      }
      base::requireNamespace(pkg, ...)
    },
    .package = "base"
  )
  expect_error(
    table_categorical(
      sochealth,
      select = smoking,
      by = sex,
      output = "clipboard"
    ),
    "clipr"
  )
})

test_that("table_categorical grouped word errors for missing officer", {
  skip("Cannot mock officer requireNamespace without recursion")
})

# Association measures for contingency tables.
#
# Formulas for asymptotic standard errors follow the implementations in
# DescTools (Signorell et al.), which in turn implement the formulas from
# Agresti (2002) and Liebetrau (1983).

# ── Internal helpers ──────────────────────────────────────────────────────────

.validate_table <- function(x, min_dim = c(2L, 2L)) {
  if (!inherits(x, "table")) {
    stop("`x` must be a contingency table (class `table`).", call. = FALSE)
  }
  if (length(dim(x)) != 2L) {
    stop("`x` must be a two-dimensional table.", call. = FALSE)
  }
  if (nrow(x) < min_dim[1] || ncol(x) < min_dim[2]) {
    stop(
      sprintf("`x` must be at least %dx%d.", min_dim[1], min_dim[2]),
      call. = FALSE
    )
  }
  invisible(NULL)
}


.assoc_result <- function(
  estimate,
  se = NA_real_,
  conf_level = 0.95,
  p_value = NA_real_,
  lower_bound = -Inf,
  upper_bound = Inf,
  ci_lower = NULL,
  ci_upper = NULL,
  .include_se = FALSE,
  digits = 3L
) {
  if (is.null(conf_level) || is.na(conf_level)) {
    out <- if (.include_se) {
      c(estimate = estimate, se = se, p_value = p_value)
    } else {
      c(estimate = estimate, p_value = p_value)
    }
    class(out) <- "spicy_assoc_detail"
    attr(out, "digits") <- digits
    return(out)
  }
  if (is.null(ci_lower)) {
    z <- stats::qnorm(1 - (1 - conf_level) / 2)
    ci_lower <- max(lower_bound, estimate - z * se)
    ci_upper <- min(upper_bound, estimate + z * se)
  }
  out <- if (.include_se) {
    c(
      estimate = estimate,
      se = se,
      ci_lower = ci_lower,
      ci_upper = ci_upper,
      p_value = p_value
    )
  } else {
    c(
      estimate = estimate,
      ci_lower = ci_lower,
      ci_upper = ci_upper,
      p_value = p_value
    )
  }
  class(out) <- "spicy_assoc_detail"
  attr(out, "digits") <- digits
  out
}


#' Print a detailed association measure result
#'
#' Formats a `spicy_assoc_detail` vector (returned by association
#' functions with `detail = TRUE`) with fixed decimal places and
#' `< 0.001` notation for small p-values.
#'
#' @param x A `spicy_assoc_detail` object.
#' @param digits Number of decimal places for the estimate, SE, and
#'   confidence interval. Defaults to 3. The p-value is always
#'   formatted separately (`< 0.001` or three decimal places).
#' @param ... Ignored.
#' @return `x`, invisibly.
#'
#' @seealso [cramer_v()], [assoc_measures()]
#'
#' @keywords internal
#' @export
print.spicy_assoc_detail <- function(
  x,
  digits = attr(x, "digits") %||% 3L,
  ...
) {
  nms <- names(x)
  labels <- c(
    estimate = "Estimate",
    se = "SE",
    ci_lower = "CI lower",
    ci_upper = "CI upper",
    p_value = "p"
  )
  hdr <- labels[nms]
  vals <- vapply(
    nms,
    function(nm) {
      v <- x[[nm]]
      if (is.na(v)) {
        return("--")
      }
      if (nm == "p_value") {
        if (v < 0.001) "< 0.001" else formatC(v, format = "f", digits = 3)
      } else {
        formatC(v, format = "f", digits = digits)
      }
    },
    character(1)
  )
  widths <- pmax(nchar(hdr), nchar(vals))
  hdr_parts <- mapply(formatC, hdr, width = widths)
  val_parts <- mapply(formatC, vals, width = widths)
  cat(
    paste(hdr_parts, collapse = "  "),
    "\n",
    paste(val_parts, collapse = "  "),
    "\n",
    sep = ""
  )
  invisible(x)
}


.concordance_counts <- function(x) {
  r <- nrow(x)
  k <- ncol(x)
  pi_c <- matrix(0, r, k)
  pi_d <- matrix(0, r, k)

  for (i in seq_len(r)) {
    for (j in seq_len(k)) {
      if (i < r && j < k) {
        pi_c[i, j] <- sum(x[(i + 1):r, (j + 1):k, drop = FALSE])
      }
      if (i < r && j > 1L) {
        pi_d[i, j] <- sum(x[(i + 1):r, 1:(j - 1), drop = FALSE])
      }
    }
  }

  # Also count from other direction for symmetry
  pi_c2 <- matrix(0, r, k)
  pi_d2 <- matrix(0, r, k)
  for (i in seq_len(r)) {
    for (j in seq_len(k)) {
      if (i > 1L && j > 1L) {
        pi_c2[i, j] <- sum(x[1:(i - 1), 1:(j - 1), drop = FALSE])
      }
      if (i > 1L && j < k) {
        pi_d2[i, j] <- sum(x[1:(i - 1), (j + 1):k, drop = FALSE])
      }
    }
  }

  C <- sum(x * pi_c)
  D <- sum(x * pi_d)

  list(
    C = C,
    D = D,
    pi_c = pi_c + pi_c2,
    pi_d = pi_d + pi_d2
  )
}


# ── Nominal measures ─────────────────────────────────────────────────────────

#' Cramer's V
#'
#' `cramer_v()` computes Cramer's V for a two-way contingency table,
#' measuring the strength of association between two categorical variables.
#'
#' @param x A contingency table (of class `table`).
#' @param detail Logical. If `FALSE` (default), return the estimate
#'   as a numeric scalar. If `TRUE`, return a named numeric vector
#'   including confidence interval and p-value.
#' @param conf_level A number between 0 and 1 giving the confidence
#'   level (default `0.95`). Only used when `detail = TRUE`. Set
#'   to `NULL` to omit the confidence interval.
#' @param digits Number of decimal places used when printing the
#'   result (default `3`). Only affects the `detail = TRUE` output.
#' @param .include_se Internal parameter; do not use.
#'
#' @return When `detail = FALSE`: a single numeric value (the
#'   estimate).
#'   When `detail = TRUE` and `conf_level` is non-`NULL`:
#'   `c(estimate, ci_lower, ci_upper, p_value)`.
#'   When `detail = TRUE` and `conf_level = NULL`:
#'   `c(estimate, p_value)`.
#'   The p-value tests the null hypothesis of no association
#'   (Pearson chi-squared test).
#'
#' @details
#' Cramer's V is computed as
#' \eqn{V = \sqrt{\chi^2 / (n \cdot (k - 1))}}, where \eqn{\chi^2}
#' is the Pearson chi-squared statistic, \eqn{n} is the total count,
#' and \eqn{k = \min(r, c)}.
#' The confidence interval uses the Fisher z-transformation.
#' Standard error formulas follow the DescTools implementations
#' (Signorell et al., 2024).
#'
#' @references
#' Agresti, A. (2002). *Categorical Data Analysis* (2nd ed.). Wiley.
#'
#' Liebetrau, A. M. (1983). *Measures of Association*. Sage.
#'
#' Signorell, A. et al. (2024). *DescTools: Tools for Descriptive
#' Statistics*. R package.
#'
#' @examples
#' tab <- table(sochealth$smoking, sochealth$education)
#' cramer_v(tab)
#' cramer_v(tab, detail = TRUE)
#' cramer_v(tab, detail = TRUE, conf_level = NULL)
#'
#' @seealso [phi()], [contingency_coef()], [assoc_measures()]
#' @export
cramer_v <- function(
  x,
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
) {
  .validate_table(x)
  n <- sum(x)
  k <- min(nrow(x), ncol(x)) - 1L
  if (n <= 0 || k <= 0) {
    warning(
      "Cramer's V is undefined for this table; returning NA.",
      call. = FALSE
    )
    return(c(estimate = NA_real_))
  }
  chi <- suppressWarnings(stats::chisq.test(x, correct = FALSE))
  chi2 <- as.numeric(chi$statistic)
  V <- sqrt(chi2 / (n * k))

  if (!detail) {
    return(V)
  }

  p_value <- as.numeric(chi$p.value)

  ci_lower <- NULL
  ci_upper <- NULL
  if (!is.null(conf_level) && !is.na(conf_level) && n > 3 && V > 0) {
    z_alpha <- stats::qnorm(1 - (1 - conf_level) / 2)
    ci_lower <- max(0, tanh(atanh(V) - z_alpha / sqrt(n - 3)))
    ci_upper <- min(1, tanh(atanh(V) + z_alpha / sqrt(n - 3)))
  }

  .assoc_result(
    V,
    conf_level = conf_level,
    p_value = p_value,
    ci_lower = ci_lower,
    ci_upper = ci_upper,
    .include_se = .include_se,
    digits = digits
  )
}


#' Phi coefficient
#'
#' `phi()` computes the phi coefficient for a 2x2 contingency table.
#'
#' @inheritParams cramer_v
#'
#' @return Same structure as [cramer_v()]: a scalar when
#'   `detail = FALSE`, a named vector when `detail = TRUE`.
#'   The p-value tests the null hypothesis of no association
#'   (Pearson chi-squared test).
#'
#' @details
#' The phi coefficient is \eqn{\phi = \sqrt{\chi^2 / n}}.
#' It is equivalent to Cramer's V for 2x2 tables and equals the
#' Pearson correlation between the two binary variables.
#' The confidence interval uses the Fisher z-transformation.
#' Standard error formulas follow the DescTools implementations
#' (Signorell et al., 2024); see [cramer_v()] for full references.
#'
#' @examples
#' tab <- table(sochealth$smoking, sochealth$sex)
#' phi(tab)
#' phi(tab, detail = TRUE)
#'
#' @seealso [cramer_v()], [yule_q()], [assoc_measures()]
#' @export
phi <- function(
  x,
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
) {
  .validate_table(x, min_dim = c(2L, 2L))
  if (nrow(x) != 2L || ncol(x) != 2L) {
    stop("`x` must be a 2x2 table for the phi coefficient.", call. = FALSE)
  }
  n <- sum(x)
  chi <- suppressWarnings(stats::chisq.test(x, correct = FALSE))
  chi2 <- as.numeric(chi$statistic)
  ph <- sqrt(chi2 / n)

  if (!detail) {
    return(ph)
  }

  p_value <- as.numeric(chi$p.value)

  ci_lower <- NULL
  ci_upper <- NULL
  if (!is.null(conf_level) && !is.na(conf_level) && n > 3 && ph > 0) {
    z_alpha <- stats::qnorm(1 - (1 - conf_level) / 2)
    ci_lower <- max(0, tanh(atanh(ph) - z_alpha / sqrt(n - 3)))
    ci_upper <- min(1, tanh(atanh(ph) + z_alpha / sqrt(n - 3)))
  }

  .assoc_result(
    ph,
    conf_level = conf_level,
    p_value = p_value,
    ci_lower = ci_lower,
    ci_upper = ci_upper,
    .include_se = .include_se,
    digits = digits
  )
}


#' Pearson's contingency coefficient
#'
#' `contingency_coef()` computes Pearson's contingency coefficient C
#' for a two-way contingency table.
#'
#' @inheritParams cramer_v
#'
#' @return Same structure as [cramer_v()]: a scalar when
#'   `detail = FALSE`, a named vector when `detail = TRUE`.
#'   The p-value tests the null hypothesis of no association
#'   (Pearson chi-squared test). CI values are `NA` because no
#'   standard asymptotic SE exists for C.
#'
#' @details
#' The contingency coefficient is
#' \eqn{C = \sqrt{\chi^2 / (\chi^2 + n)}}.
#' It ranges from 0 (independence) to a maximum that depends on
#' the table dimensions. No standard asymptotic standard error exists,
#' so the confidence interval is not computed.
#'
#' @examples
#' tab <- table(sochealth$smoking, sochealth$education)
#' contingency_coef(tab)
#'
#' @seealso [cramer_v()], [assoc_measures()]
#' @export
contingency_coef <- function(
  x,
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
) {
  .validate_table(x)
  n <- sum(x)
  chi <- suppressWarnings(stats::chisq.test(x, correct = FALSE))
  chi2 <- as.numeric(chi$statistic)
  C_val <- sqrt(chi2 / (chi2 + n))

  if (!detail) {
    return(C_val)
  }

  p_value <- as.numeric(chi$p.value)

  .assoc_result(
    C_val,
    conf_level = conf_level,
    p_value = p_value,
    ci_lower = NA_real_,
    ci_upper = NA_real_,
    .include_se = .include_se,
    digits = digits
  )
}


#' Yule's Q
#'
#' `yule_q()` computes Yule's Q coefficient of association for a 2x2
#' contingency table.
#'
#' @inheritParams cramer_v
#'
#' @return Same structure as [cramer_v()]: a scalar when
#'   `detail = FALSE`, a named vector when `detail = TRUE`.
#'   The p-value tests H0: Q = 0 (Wald z-test).
#'
#' @details
#' For a 2x2 table with cells \eqn{a, b, c, d}, Yule's Q is
#' \eqn{Q = (ad - bc) / (ad + bc)}.
#' It is equivalent to the Goodman-Kruskal Gamma for 2x2 tables.
#' The asymptotic standard error is
#' \eqn{SE = 0.5 (1 - Q^2) \sqrt{1/a + 1/b + 1/c + 1/d}}.
#' Standard error formulas follow the DescTools implementations
#' (Signorell et al., 2024); see [cramer_v()] for full references.
#'
#' @examples
#' tab <- table(sochealth$smoking, sochealth$sex)
#' yule_q(tab)
#'
#' @seealso [phi()], [gamma_gk()], [assoc_measures()]
#' @export
yule_q <- function(
  x,
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
) {
  .validate_table(x, min_dim = c(2L, 2L))
  if (nrow(x) != 2L || ncol(x) != 2L) {
    stop("`x` must be a 2x2 table for Yule's Q.", call. = FALSE)
  }
  a <- x[1, 1]
  b <- x[1, 2]
  c_ <- x[2, 1]
  d <- x[2, 2]
  ad <- a * d
  bc <- b * c_

  if (ad + bc == 0) {
    warning(
      "Yule's Q is undefined when ad + bc = 0; returning NA.",
      call. = FALSE
    )
    return(c(estimate = NA_real_))
  }

  Q <- (ad - bc) / (ad + bc)

  if (!detail) {
    return(Q)
  }

  if (any(c(a, b, c_, d) == 0)) {
    se <- NA_real_
    p_value <- NA_real_
  } else {
    se <- 0.5 * (1 - Q^2) * sqrt(1 / a + 1 / b + 1 / c_ + 1 / d)
    p_value <- 2 * stats::pnorm(-abs(Q / se))
  }

  .assoc_result(
    Q,
    se,
    conf_level = conf_level,
    p_value = p_value,
    lower_bound = -1,
    upper_bound = 1,
    .include_se = .include_se,
    digits = digits
  )
}


#' Goodman-Kruskal's Lambda
#'
#' `lambda_gk()` computes Goodman-Kruskal's Lambda, a proportional
#' reduction in error (PRE) measure for nominal variables.
#'
#' @inheritParams cramer_v
#' @param direction Direction of prediction:
#'   `"symmetric"` (default), `"row"` (column predicts row),
#'   or `"column"` (row predicts column).
#'
#' @return Same structure as [cramer_v()]: a scalar when
#'   `detail = FALSE`, a named vector when `detail = TRUE`.
#'   The p-value tests H0: lambda = 0 (Wald z-test).
#'
#' @details
#' Lambda measures how much prediction error is reduced when
#' the independent variable is used to predict the dependent
#' variable. It ranges from 0 (no reduction) to 1 (perfect
#' prediction). Lambda can equal zero even when variables
#' are associated if the modal category dominates in every
#' column (or row).
#' Standard error formulas follow the DescTools implementations
#' (Signorell et al., 2024); see [cramer_v()] for full references.
#'
#' @examples
#' tab <- table(sochealth$smoking, sochealth$education)
#' lambda_gk(tab)
#' lambda_gk(tab, direction = "row")
#' lambda_gk(tab, direction = "column", detail = TRUE)
#'
#' @seealso [goodman_kruskal_tau()], [uncertainty_coef()],
#'   [assoc_measures()]
#' @export
lambda_gk <- function(
  x,
  direction = c("symmetric", "row", "column"),
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
) {
  .validate_table(x)
  direction <- match.arg(direction)
  n <- sum(x)

  nr <- nrow(x)
  nc <- ncol(x)
  rsum <- rowSums(x)
  csum <- colSums(x)
  rmax <- apply(x, 1, max)
  cmax <- apply(x, 2, max)
  max_rsum <- max(rsum)
  max_csum <- max(csum)

  est <- switch(
    direction,
    symmetric = {
      0.5 *
        (sum(rmax, cmax) - (max_csum + max_rsum)) /
        (n - 0.5 * (max_csum + max_rsum))
    },
    column = (sum(rmax) - max_csum) / (n - max_csum),
    row = (sum(cmax) - max_rsum) / (n - max_rsum)
  )

  if (!detail) {
    return(est)
  }

  # ASE (DescTools approach)
  sigma2 <- switch(
    direction,
    row = {
      L_row_max <- min(which(rsum == max_rsum))
      L_row <- integer(nc)
      for (i in seq_len(nc)) {
        if (x[L_row_max, i] == max_rsum) {
          L_row[i] <- L_row_max
        } else {
          L_row[i] <- min(which(x[, i] == cmax[i]))
        }
      }
      (n - sum(cmax)) *
        (sum(cmax) + max_rsum - 2 * sum(cmax[which(L_row == L_row_max)])) /
        (n - max_rsum)^3
    },
    column = {
      L_col_max <- min(which(csum == max_csum))
      L_col <- integer(nr)
      for (i in seq_len(nr)) {
        if (x[i, L_col_max] == max_csum) {
          L_col[i] <- L_col_max
        } else {
          L_col[i] <- min(which(x[i, ] == rmax[i]))
        }
      }
      (n - sum(rmax)) *
        (sum(rmax) + max_csum - 2 * sum(rmax[which(L_col == L_col_max)])) /
        (n - max_csum)^3
    },
    symmetric = {
      l <- min(which(csum == max_csum))
      k <- min(which(rsum == max_rsum))
      li <- apply(x, 1, which.max)
      ki <- apply(x, 2, which.max)
      w <- 2 * n - max_csum - max_rsum
      v <- 2 * n - sum(rmax, cmax)
      xx <- sum(rmax[li == l], cmax[ki == k], rmax[k], cmax[l])
      y <- 8 * n - w - v - 2 * xx
      t_vec <- vapply(
        seq_along(li),
        \(i) {
          ki[li[i]] == i && li[ki[li[i]]] == li[i]
        },
        logical(1)
      )
      (w * v * y - 2 * w^2 * (n - sum(rmax[t_vec])) - 2 * v^2 * (n - x[k, l])) /
        w^4
    }
  )

  se <- sqrt(max(0, sigma2))
  p_value <- if (se > 0) {
    2 * stats::pnorm(-abs(est / se))
  } else {
    NA_real_
  }

  .assoc_result(
    est,
    se,
    conf_level = conf_level,
    p_value = p_value,
    lower_bound = 0,
    upper_bound = 1,
    .include_se = .include_se,
    digits = digits
  )
}


#' Goodman-Kruskal's Tau
#'
#' `goodman_kruskal_tau()` computes Goodman-Kruskal's Tau, a
#' proportional reduction in error (PRE) measure for nominal
#' variables.
#'
#' @inheritParams cramer_v
#' @param direction Direction of prediction:
#'   `"row"` (default, column predicts row) or
#'   `"column"` (row predicts column).
#'
#' @return Same structure as [cramer_v()]: a scalar when
#'   `detail = FALSE`, a named vector when `detail = TRUE`.
#'   The p-value tests H0: tau = 0 (Wald z-test).
#'
#' @details
#' Unlike [lambda_gk()], Goodman-Kruskal's Tau uses all cell
#' frequencies rather than only the modal categories, making it
#' more sensitive to association patterns where lambda may be zero.
#' Standard error formulas follow the DescTools implementations
#' (Signorell et al., 2024); see [cramer_v()] for full references.
#'
#' @examples
#' tab <- table(sochealth$smoking, sochealth$education)
#' goodman_kruskal_tau(tab)
#' goodman_kruskal_tau(tab, direction = "column", detail = TRUE)
#'
#' @seealso [lambda_gk()], [uncertainty_coef()], [assoc_measures()]
#' @export
goodman_kruskal_tau <- function(
  x,
  direction = c("row", "column"),
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
) {
  .validate_table(x)
  direction <- match.arg(direction)
  n <- sum(x)
  rsum <- rowSums(x)
  csum <- colSums(x)

  if (direction == "row") {
    # Column predicts row
    v <- sum(rsum^2) / n
    d <- 0
    for (j in seq_len(ncol(x))) {
      if (csum[j] > 0) {
        d <- d + sum(x[, j]^2) / csum[j]
      }
    }
    tau <- (d - v) / (n - v)
  } else {
    # Row predicts column
    v <- sum(csum^2) / n
    d <- 0
    for (i in seq_len(nrow(x))) {
      if (rsum[i] > 0) {
        d <- d + sum(x[i, ]^2) / rsum[i]
      }
    }
    tau <- (d - v) / (n - v)
  }

  if (!detail) {
    return(unname(tau))
  }

  # ASE (DescTools gradient approach)
  se <- .ase_gk_tau(x, direction)
  p_value <- if (!is.na(se) && se > 0) {
    2 * stats::pnorm(-abs(tau / se))
  } else {
    NA_real_
  }

  .assoc_result(
    unname(tau),
    se,
    conf_level = conf_level,
    p_value = unname(p_value),
    lower_bound = 0,
    upper_bound = 1,
    .include_se = .include_se,
    digits = digits
  )
}


.ase_gk_tau <- function(x, direction) {
  n <- sum(x)
  rsum <- rowSums(x)
  csum <- colSums(x)
  r <- nrow(x)
  k <- ncol(x)

  # DescTools formula exactly
  n_err_uncond <- n^2
  if (direction == "row") {
    for (j in seq_len(k)) {
      if (csum[j] > 0) {
        n_err_uncond <- n_err_uncond - n * sum(x[, j]^2 / csum[j])
      }
    }
    n_err_cond <- n^2 - sum(rsum^2)
    v <- n_err_uncond / n^2
    d <- n_err_cond / n^2
    f <- d * (v + 1) - 2 * v
    var_tau <- 0
    for (i in seq_len(r)) {
      for (j in seq_len(k)) {
        if (csum[j] > 0) {
          term <- -2 *
            v *
            (rsum[i] / n) +
            d *
              ((2 * x[i, j] / csum[j]) -
                sum((x[, j] / csum[j])^2)) -
            f
          var_tau <- var_tau + x[i, j] * term^2 / (n^2 * d^4)
        }
      }
    }
  } else {
    for (i in seq_len(r)) {
      if (rsum[i] > 0) {
        n_err_uncond <- n_err_uncond - n * sum(x[i, ]^2 / rsum[i])
      }
    }
    n_err_cond <- n^2 - sum(csum^2)
    v <- n_err_uncond / n^2
    d <- n_err_cond / n^2
    f <- d * (v + 1) - 2 * v
    var_tau <- 0
    for (i in seq_len(r)) {
      for (j in seq_len(k)) {
        if (rsum[i] > 0) {
          term <- -2 *
            v *
            (csum[j] / n) +
            d *
              ((2 * x[i, j] / rsum[i]) -
                sum((x[i, ] / rsum[i])^2)) -
            f
          var_tau <- var_tau + x[i, j] * term^2 / (n^2 * d^4)
        }
      }
    }
  }
  sqrt(max(0, var_tau))
}


#' Uncertainty Coefficient
#'
#' `uncertainty_coef()` computes the Uncertainty Coefficient
#' (Theil's U) for a two-way contingency table, based on
#' information entropy.
#'
#' @inheritParams cramer_v
#' @param direction Direction of prediction:
#'   `"symmetric"` (default), `"row"` (column predicts row),
#'   or `"column"` (row predicts column).
#'
#' @return Same structure as [cramer_v()]: a scalar when
#'   `detail = FALSE`, a named vector when `detail = TRUE`.
#'   The p-value tests H0: U = 0 (Wald z-test).
#'
#' @details
#' The uncertainty coefficient measures association using
#' Shannon entropy.
#' For `direction = "row"`:
#' \eqn{U = (H_X + H_Y - H_{XY}) / H_X}, where \eqn{H_X},
#' \eqn{H_Y} are the marginal entropies and \eqn{H_{XY}} is
#' the joint entropy.
#' The symmetric version is
#' \eqn{U = 2 (H_X + H_Y - H_{XY}) / (H_X + H_Y)}.
#' Standard error formulas follow the DescTools implementations
#' (Signorell et al., 2024); see [cramer_v()] for full references.
#'
#' @examples
#' tab <- table(sochealth$smoking, sochealth$education)
#' uncertainty_coef(tab)
#' uncertainty_coef(tab, direction = "row", detail = TRUE)
#'
#' @seealso [lambda_gk()], [goodman_kruskal_tau()], [assoc_measures()]
#' @export
uncertainty_coef <- function(
  x,
  direction = c("symmetric", "row", "column"),
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
) {
  .validate_table(x)
  direction <- match.arg(direction)
  n <- sum(x)

  rsum <- rowSums(x)
  csum <- colSums(x)

  # DescTools uses natural log on counts/n
  H_x <- -sum((rsum * log(rsum / n)) / n)
  H_y <- -sum((csum * log(csum / n)) / n)
  H_xy <- -sum(x[x > 0] * log(x[x > 0] / n) / n)

  mi <- H_x + H_y - H_xy # mutual information

  U <- switch(
    direction,
    row = if (H_x > 0) mi / H_x else 0,
    column = if (H_y > 0) mi / H_y else 0,
    symmetric = if (H_x + H_y > 0) 2 * mi / (H_x + H_y) else 0
  )

  if (!detail) {
    return(U)
  }

  # ASE (DescTools approach)
  se <- .ase_uncert_coef(x, direction, H_x, H_y, H_xy)
  p_value <- if (!is.na(se) && se > 0) {
    2 * stats::pnorm(-abs(U / se))
  } else {
    NA_real_
  }

  .assoc_result(
    U,
    se,
    conf_level = conf_level,
    p_value = p_value,
    lower_bound = 0,
    upper_bound = 1,
    .include_se = .include_se,
    digits = digits
  )
}


.ase_uncert_coef <- function(x, direction, H_x, H_y, H_xy) {
  n <- sum(x)
  rsum <- rowSums(x)
  csum <- colSums(x)
  r <- nrow(x)
  k <- ncol(x)

  # DescTools formula exactly
  if (direction == "symmetric") {
    log_outer <- log(rsum %o% csum / n^2)
    log_xn <- matrix(0, r, k)
    nz <- x > 0
    log_xn[nz] <- log(x[nz] / n)
    terms <- x * (H_xy * log_outer - (H_x + H_y) * log_xn)^2
    terms[!nz] <- 0
    sigma2 <- 4 * sum(terms) / (n^2 * (H_x + H_y)^4)
  } else {
    var_rc <- 0
    var_cr <- 0
    for (i in seq_len(r)) {
      for (j in seq_len(k)) {
        if (x[i, j] > 0) {
          var_rc <- var_rc +
            x[i, j] *
              (H_x *
                log(x[i, j] / csum[j]) +
                (H_y - H_xy) * log(rsum[i] / n))^2 /
              (n^2 * H_x^4)
          var_cr <- var_cr +
            x[i, j] *
              (H_y *
                log(x[i, j] / rsum[i]) +
                (H_x - H_xy) * log(csum[j] / n))^2 /
              (n^2 * H_y^4)
        }
      }
    }
    sigma2 <- if (direction == "row") var_rc else var_cr
  }
  sqrt(max(0, sigma2))
}


# ── Ordinal measures ─────────────────────────────────────────────────────────

#' Goodman-Kruskal Gamma
#'
#' `gamma_gk()` computes the Goodman-Kruskal Gamma statistic for a
#' two-way contingency table of ordinal variables.
#'
#' @inheritParams cramer_v
#'
#' @return Same structure as [cramer_v()]: a scalar when
#'   `detail = FALSE`, a named vector when `detail = TRUE`.
#'   The p-value tests H0: gamma = 0 (Wald z-test).
#'
#' @details
#' Gamma is computed as \eqn{\gamma = (C - D) / (C + D)}, where
#' \eqn{C} and \eqn{D} are the numbers of concordant and
#' discordant pairs. It ignores tied pairs, making it appropriate
#' for ordinal variables with many ties.
#' Standard error formulas follow the DescTools implementations
#' (Signorell et al., 2024); see [cramer_v()] for full references.
#'
#' @examples
#' tab <- table(sochealth$education, sochealth$self_rated_health)
#' gamma_gk(tab)
#' gamma_gk(tab, detail = TRUE)
#'
#' @seealso [kendall_tau_b()], [kendall_tau_c()], [somers_d()],
#'   [assoc_measures()]
#' @export
gamma_gk <- function(
  x,
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
) {
  .validate_table(x)
  cd <- .concordance_counts(x)
  C <- cd$C
  D <- cd$D

  if (C + D == 0) {
    warning("No concordant or discordant pairs; returning NA.", call. = FALSE)
    return(c(estimate = NA_real_))
  }

  G <- (C - D) / (C + D)

  if (!detail) {
    return(G)
  }

  # ASE (Agresti, 2002)
  psi <- 2 * (D * cd$pi_c - C * cd$pi_d) / (C + D)^2
  se <- sqrt(sum(x * psi^2) - (sum(x * psi))^2)

  p_value <- 2 * stats::pnorm(-abs(G / se))

  .assoc_result(
    G,
    se,
    conf_level = conf_level,
    p_value = p_value,
    lower_bound = -1,
    upper_bound = 1,
    .include_se = .include_se,
    digits = digits
  )
}


#' Kendall's Tau-b
#'
#' `kendall_tau_b()` computes Kendall's Tau-b for a two-way
#' contingency table of ordinal variables.
#'
#' @inheritParams cramer_v
#'
#' @return Same structure as [cramer_v()]: a scalar when
#'   `detail = FALSE`, a named vector when `detail = TRUE`.
#'   The p-value tests H0: tau-b = 0 (Wald z-test).
#'
#' @details
#' Kendall's Tau-b is computed as
#' \eqn{\tau_b = (C - D) / \sqrt{(n_0 - n_1)(n_0 - n_2)}},
#' where \eqn{n_0 = n(n-1)/2}, \eqn{n_1} is the number of
#' pairs tied on the row variable, and \eqn{n_2} is the number
#' tied on the column variable. Tau-b corrects for ties and is
#' appropriate for square tables.
#' Standard error formulas follow the DescTools implementations
#' (Signorell et al., 2024); see [cramer_v()] for full references.
#'
#' @examples
#' tab <- table(sochealth$education, sochealth$self_rated_health)
#' kendall_tau_b(tab)
#'
#' @seealso [kendall_tau_c()], [gamma_gk()], [somers_d()],
#'   [assoc_measures()]
#' @export
kendall_tau_b <- function(
  x,
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
) {
  .validate_table(x)
  n <- sum(x)
  cd <- .concordance_counts(x)
  C <- cd$C
  D <- cd$D

  rsum <- rowSums(x)
  csum <- colSums(x)

  n0 <- n * (n - 1) / 2
  n1 <- sum(rsum * (rsum - 1)) / 2 # row ties
  n2 <- sum(csum * (csum - 1)) / 2 # column ties

  denom <- sqrt((n0 - n1) * (n0 - n2))
  if (denom == 0) {
    warning("Tau-b is undefined for this table; returning NA.", call. = FALSE)
    return(c(estimate = NA_real_))
  }

  tau_b <- (C - D) / denom

  if (!detail) {
    return(tau_b)
  }

  # ASE (Brown & Benedetti, via DescTools)
  pi <- x / n
  rowsum <- rowSums(pi)
  colsum <- colSums(pi)
  pdiff <- (cd$pi_c - cd$pi_d) / n
  Pdiff <- 2 * (C - D) / n^2
  rowmat <- matrix(rep(rowsum, ncol(x)), ncol = ncol(x))
  colmat <- matrix(rep(colsum, nrow(x)), nrow = nrow(x), byrow = TRUE)
  delta1 <- sqrt(1 - sum(rowsum^2))
  delta2 <- sqrt(1 - sum(colsum^2))

  tauphi <- (2 * pdiff + Pdiff * colmat) *
    delta2 *
    delta1 +
    (Pdiff * rowmat * delta2) / delta1

  se_sq <- (sum(pi * tauphi^2) - sum(pi * tauphi)^2) /
    (delta1 * delta2)^4 /
    n
  if (se_sq < .Machine$double.eps * 10) {
    se_sq <- 0
  }
  se <- sqrt(se_sq)

  p_value <- 2 * stats::pnorm(-abs(tau_b / se))

  .assoc_result(
    tau_b,
    se,
    conf_level = conf_level,
    p_value = p_value,
    lower_bound = -1,
    upper_bound = 1,
    .include_se = .include_se,
    digits = digits
  )
}


#' Kendall's Tau-c (Stuart's Tau-c)
#'
#' `kendall_tau_c()` computes Stuart's Tau-c (also known as
#' Kendall's Tau-c) for a two-way contingency table of ordinal
#' variables.
#'
#' @inheritParams cramer_v
#'
#' @return Same structure as [cramer_v()]: a scalar when
#'   `detail = FALSE`, a named vector when `detail = TRUE`.
#'   The p-value tests H0: tau-c = 0 (Wald z-test).
#'
#' @details
#' Stuart's Tau-c is computed as
#' \eqn{\tau_c = 2m(C - D) / (n^2(m - 1))}, where
#' \eqn{m = \min(r, c)}. It is appropriate for rectangular
#' tables and is not restricted to the range \eqn{[-1, 1]} only for
#' square tables.
#' Standard error formulas follow the DescTools implementations
#' (Signorell et al., 2024); see [cramer_v()] for full references.
#'
#' @examples
#' tab <- table(sochealth$education, sochealth$self_rated_health)
#' kendall_tau_c(tab)
#'
#' @seealso [kendall_tau_b()], [gamma_gk()], [somers_d()],
#'   [assoc_measures()]
#' @export
kendall_tau_c <- function(
  x,
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
) {
  .validate_table(x)
  n <- sum(x)
  m <- min(nrow(x), ncol(x))
  cd <- .concordance_counts(x)
  C <- cd$C
  D <- cd$D

  tau_c <- 2 * m * (C - D) / (n^2 * (m - 1))

  if (!detail) {
    return(tau_c)
  }

  # ASE (DescTools approach)
  sigma2 <- 4 *
    m^2 /
    ((m - 1)^2 * n^4) *
    (sum(x * (cd$pi_c - cd$pi_d)^2) - 4 * (C - D)^2 / n)
  se <- sqrt(max(0, sigma2))

  p_value <- 2 * stats::pnorm(-abs(tau_c / se))

  .assoc_result(
    tau_c,
    se,
    conf_level = conf_level,
    p_value = p_value,
    lower_bound = -1,
    upper_bound = 1,
    .include_se = .include_se,
    digits = digits
  )
}


#' Somers' D
#'
#' `somers_d()` computes Somers' D for a two-way contingency
#' table of ordinal variables.
#'
#' @inheritParams cramer_v
#' @param direction Direction of prediction:
#'   `"row"` (default, column predicts row),
#'   `"column"` (row predicts column),
#'   or `"symmetric"` (average of both directions).
#'
#' @return Same structure as [cramer_v()]: a scalar when
#'   `detail = FALSE`, a named vector when `detail = TRUE`.
#'   The p-value tests H0: D = 0 (Wald z-test).
#'
#' @details
#' Somers' D is an asymmetric ordinal measure defined as
#' \eqn{d = (C - D) / (C + D + T)}, where \eqn{T} is the
#' number of pairs tied on the independent variable.
#' The symmetric version is the harmonic mean of the two
#' asymmetric values.
#' Standard error formulas follow the DescTools implementations
#' (Signorell et al., 2024); see [cramer_v()] for full references.
#'
#' @examples
#' tab <- table(sochealth$education, sochealth$self_rated_health)
#' somers_d(tab, direction = "row")
#' somers_d(tab, direction = "column", detail = TRUE)
#'
#' @seealso [kendall_tau_b()], [gamma_gk()], [assoc_measures()]
#' @export
somers_d <- function(
  x,
  direction = c("row", "column", "symmetric"),
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
) {
  .validate_table(x)
  direction <- match.arg(direction)
  n <- sum(x)
  cd <- .concordance_counts(x)
  C <- cd$C
  D <- cd$D

  rsum <- rowSums(x)
  csum <- colSums(x)

  n0 <- n * (n - 1) / 2
  n1 <- sum(rsum * (rsum - 1)) / 2 # row ties
  n2 <- sum(csum * (csum - 1)) / 2 # column ties

  # DescTools: ni. = colSums for "row", rowSums for "column"
  ni <- switch(direction, row = csum, column = rsum, symmetric = NULL)

  if (direction == "symmetric") {
    # Symmetric Somers' d equals tau-b
    return(kendall_tau_b(
      x,
      detail = detail,
      conf_level = conf_level,
      .include_se = .include_se
    ))
  }

  denom <- n0 - switch(direction, row = n2, column = n1)
  if (denom == 0) {
    warning(
      "Somers' d is undefined for this table; returning NA.",
      call. = FALSE
    )
    return(c(estimate = NA_real_))
  }
  d_val <- (C - D) / denom

  if (!detail) {
    return(d_val)
  }

  # ASE (DescTools formula)
  wt <- n^2 - sum(ni^2)
  if (direction == "row") {
    n_minus_ni <- matrix(
      rep(n - csum, each = nrow(x)),
      nrow = nrow(x),
      ncol = ncol(x)
    )
  } else {
    n_minus_ni <- matrix(rep(n - rsum, ncol(x)), nrow = nrow(x), ncol = ncol(x))
  }
  sigma2 <- 4 /
    wt^4 *
    sum(x * (wt * (cd$pi_c - cd$pi_d) - 2 * (C - D) * n_minus_ni)^2)
  se <- sqrt(max(0, sigma2))

  p_value <- if (se > 0) {
    2 * stats::pnorm(-abs(d_val / se))
  } else {
    NA_real_
  }

  .assoc_result(
    d_val,
    se,
    conf_level = conf_level,
    p_value = p_value,
    lower_bound = -1,
    upper_bound = 1,
    .include_se = .include_se,
    digits = digits
  )
}


# ── Summary table ────────────────────────────────────────────────────────────

#' Association measures summary table
#'
#' `assoc_measures()` computes a range of association measures for a
#' two-way contingency table and returns them in a tidy data frame.
#'
#' @param x A contingency table (of class `table`).
#' @param type Which family of measures to compute:
#'   `"all"` (default), `"nominal"`, or `"ordinal"`.
#' @param conf_level A number between 0 and 1 giving the confidence
#'   level (default `0.95`). Set to `NULL` to omit the confidence
#'   interval.
#' @param digits Number of decimal places used when printing the
#'   result (default `3`).
#'
#' @return A data frame with columns `measure`, `estimate`, `se`,
#'   `ci_lower`, `ci_upper`, and `p_value`. For nominal measures
#'   (Cramer's V, Phi, Contingency Coef.), the p-value comes from
#'   the Pearson chi-squared test of independence. For all other
#'   measures, it is a Wald z-test of H0: measure = 0.
#'
#' @details
#' `type = "all"` (the default) returns all nominal and ordinal
#' measures. Use `type = "nominal"` or `type = "ordinal"` to
#' restrict the output to a single family.
#'
#' The nominal family includes [cramer_v()], [contingency_coef()],
#' [lambda_gk()], [goodman_kruskal_tau()], [uncertainty_coef()],
#' and (for 2x2 tables) [phi()] and [yule_q()].
#'
#' The ordinal family includes [gamma_gk()], [kendall_tau_b()],
#' [kendall_tau_c()], and [somers_d()].
#'
#' Standard error formulas follow the DescTools implementations
#' (Signorell et al., 2024).
#'
#' @examples
#' tab <- table(sochealth$smoking, sochealth$education)
#' assoc_measures(tab)
#' assoc_measures(tab, type = "nominal")
#' assoc_measures(tab, type = "ordinal")
#'
#' @references
#' Agresti, A. (2002). *Categorical Data Analysis* (2nd ed.). Wiley.
#'
#' Liebetrau, A. M. (1983). *Measures of Association*. Sage.
#'
#' Signorell, A. et al. (2024). *DescTools: Tools for Descriptive
#' Statistics*. R package.
#'
#' @seealso [cramer_v()], [gamma_gk()], [kendall_tau_b()]
#' @export
assoc_measures <- function(
  x,
  type = c("all", "nominal", "ordinal"),
  conf_level = 0.95,
  digits = 3L
) {
  .validate_table(x)
  type <- match.arg(type)
  is_2x2 <- nrow(x) == 2L && ncol(x) == 2L

  nominal_fns <- list()
  if (is_2x2) {
    nominal_fns[["Phi"]] <- \(t) {
      phi(t, detail = TRUE, conf_level = conf_level, .include_se = TRUE)
    }
    nominal_fns[["Yule's Q"]] <- \(t) {
      yule_q(t, detail = TRUE, conf_level = conf_level, .include_se = TRUE)
    }
  }
  nominal_fns[["Cramer's V"]] <- \(t) {
    cramer_v(t, detail = TRUE, conf_level = conf_level, .include_se = TRUE)
  }
  nominal_fns[["Contingency Coefficient"]] <- \(t) {
    contingency_coef(
      t,
      detail = TRUE,
      conf_level = conf_level,
      .include_se = TRUE
    )
  }
  nominal_fns[["Lambda symmetric"]] <- \(t) {
    lambda_gk(
      t,
      "symmetric",
      detail = TRUE,
      conf_level = conf_level,
      .include_se = TRUE
    )
  }
  nominal_fns[["Lambda R|C"]] <- \(t) {
    lambda_gk(
      t,
      "row",
      detail = TRUE,
      conf_level = conf_level,
      .include_se = TRUE
    )
  }
  nominal_fns[["Lambda C|R"]] <- \(t) {
    lambda_gk(
      t,
      "column",
      detail = TRUE,
      conf_level = conf_level,
      .include_se = TRUE
    )
  }
  nominal_fns[["Goodman-Kruskal's Tau R|C"]] <- \(t) {
    goodman_kruskal_tau(
      t,
      "row",
      detail = TRUE,
      conf_level = conf_level,
      .include_se = TRUE
    )
  }
  nominal_fns[["Goodman-Kruskal's Tau C|R"]] <- \(t) {
    goodman_kruskal_tau(
      t,
      "column",
      detail = TRUE,
      conf_level = conf_level,
      .include_se = TRUE
    )
  }
  nominal_fns[["Uncertainty Coefficient symmetric"]] <- \(t) {
    uncertainty_coef(
      t,
      "symmetric",
      detail = TRUE,
      conf_level = conf_level,
      .include_se = TRUE
    )
  }
  nominal_fns[["Uncertainty Coefficient R|C"]] <- \(t) {
    uncertainty_coef(
      t,
      "row",
      detail = TRUE,
      conf_level = conf_level,
      .include_se = TRUE
    )
  }
  nominal_fns[["Uncertainty Coefficient C|R"]] <- \(t) {
    uncertainty_coef(
      t,
      "column",
      detail = TRUE,
      conf_level = conf_level,
      .include_se = TRUE
    )
  }

  ordinal_fns <- list(
    "Goodman-Kruskal Gamma" = \(t) {
      gamma_gk(t, detail = TRUE, conf_level = conf_level, .include_se = TRUE)
    },
    "Kendall's Tau-b" = \(t) {
      kendall_tau_b(
        t,
        detail = TRUE,
        conf_level = conf_level,
        .include_se = TRUE
      )
    },
    "Kendall's Tau-c" = \(t) {
      kendall_tau_c(
        t,
        detail = TRUE,
        conf_level = conf_level,
        .include_se = TRUE
      )
    },
    "Somers' D R|C" = \(t) {
      somers_d(
        t,
        "row",
        detail = TRUE,
        conf_level = conf_level,
        .include_se = TRUE
      )
    },
    "Somers' D C|R" = \(t) {
      somers_d(
        t,
        "column",
        detail = TRUE,
        conf_level = conf_level,
        .include_se = TRUE
      )
    }
  )

  fns <- switch(
    type,
    nominal = nominal_fns,
    ordinal = ordinal_fns,
    all = c(nominal_fns, ordinal_fns)
  )

  .extract <- function(res, field) {
    if (field %in% names(res)) unname(res[[field]]) else NA_real_
  }

  rows <- lapply(names(fns), \(nm) {
    res <- tryCatch(
      suppressWarnings(fns[[nm]](x)),
      error = \(e) c(estimate = NA_real_, p_value = NA_real_)
    )
    data.frame(
      measure = nm,
      estimate = .extract(res, "estimate"),
      se = .extract(res, "se"),
      ci_lower = .extract(res, "ci_lower"),
      ci_upper = .extract(res, "ci_upper"),
      p_value = .extract(res, "p_value"),
      stringsAsFactors = FALSE
    )
  })
  out <- do.call(rbind, rows)
  class(out) <- c("spicy_assoc_table", "data.frame")
  attr(out, "digits") <- digits
  out
}


#' Print an association measures summary table
#'
#' Formats a `spicy_assoc_table` data frame (returned by
#' [assoc_measures()]) with fixed decimal places, aligned columns,
#' and `< 0.001` notation for small p-values.
#'
#' @param x A `spicy_assoc_table` object.
#' @param digits Number of decimal places for estimates, SE, and
#'   confidence intervals. Defaults to 3. The p-value is always
#'   formatted separately (`< 0.001` or three decimal places).
#' @param ... Ignored.
#' @return `x`, invisibly.
#'
#' @seealso [assoc_measures()]
#'
#' @keywords internal
#' @export
print.spicy_assoc_table <- function(
  x,
  digits = attr(x, "digits") %||% 3L,
  ...
) {
  fmt_num <- function(v) {
    ifelse(is.na(v), "--", formatC(v, format = "f", digits = digits))
  }
  fmt_p <- function(v) {
    ifelse(
      is.na(v),
      "--",
      ifelse(v < 0.001, "< 0.001", formatC(v, format = "f", digits = 3))
    )
  }
  cols <- list(
    Measure = format(x$measure, justify = "left"),
    Estimate = fmt_num(x$estimate),
    SE = fmt_num(x$se),
    `CI lower` = fmt_num(x$ci_lower),
    `CI upper` = fmt_num(x$ci_upper),
    p = fmt_p(x$p_value)
  )
  widths <- vapply(
    names(cols),
    \(nm) max(nchar(nm), max(nchar(cols[[nm]]))),
    integer(1)
  )
  hdr <- paste(
    mapply(
      \(nm, w) formatC(nm, width = w, flag = if (nm == "Measure") "-" else ""),
      names(cols),
      widths
    ),
    collapse = "  "
  )
  cat(hdr, "\n")
  for (i in seq_len(nrow(x))) {
    row_str <- paste(
      mapply(
        \(nm, w) {
          v <- cols[[nm]][i]
          formatC(v, width = w, flag = if (nm == "Measure") "-" else "")
        },
        names(cols),
        widths
      ),
      collapse = "  "
    )
    cat(row_str, "\n")
  }
  invisible(x)
}

test_that("cramer_v returns scalar by default", {
  data(mtcars)

  mtcars$gear <- as.factor(mtcars$gear)
  mtcars$cyl <- as.factor(mtcars$cyl)

  tab <- table(mtcars$gear, mtcars$cyl)

  result <- suppressWarnings(cramer_v(tab))

  expect_type(result, "double")
  expect_length(result, 1)
  expect_null(names(result))
  expect_gte(result, 0)
  expect_lte(result, 1)
})

test_that("cramer_v with detail = TRUE returns 4-element vector", {
  data(mtcars)
  tab <- table(factor(mtcars$gear), factor(mtcars$cyl))
  result <- suppressWarnings(cramer_v(tab, detail = TRUE))
  expect_length(result, 4)
  expect_named(result, c("estimate", "ci_lower", "ci_upper", "p_value"))
})

test_that("cramer_v with detail = TRUE, conf_level = NULL returns 2 elements", {
  data(mtcars)
  tab <- table(factor(mtcars$gear), factor(mtcars$cyl))
  result <- suppressWarnings(cramer_v(tab, detail = TRUE, conf_level = NULL))
  expect_length(result, 2)
  expect_named(result, c("estimate", "p_value"))
})

test_that("cramer_v fails gracefully with non-table input", {
  expect_error(
    cramer_v(iris),
    "`x` must be a contingency table \\(class `table`\\)\\."
  )

  expect_error(
    cramer_v(1:10),
    "`x` must be a contingency table \\(class `table`\\)\\."
  )
})

test_that("cramer_v rejects degenerate 1-row tables", {
  tab <- table(factor(c("A", "A")), factor(c("X", "Y")))
  expect_error(cramer_v(tab), "at least 2x2")
})

# Reference values from DescTools on mtcars
# tab  = table(factor(gear), factor(cyl))  [3x3]
# tab2 = table(factor(vs), factor(am))     [2x2]

tab_3x3 <- function() {
  data(mtcars, envir = environment())
  table(factor(mtcars$gear), factor(mtcars$cyl))
}

tab_2x2 <- function() {
  data(mtcars, envir = environment())
  table(factor(mtcars$vs), factor(mtcars$am))
}

# ── Default scalar return ──────────────────────────────────────────────────

test_that("all functions return scalar by default", {
  tab <- tab_3x3()
  tab2 <- tab_2x2()

  expect_length(suppressWarnings(cramer_v(tab)), 1)
  expect_length(phi(tab2), 1)
  expect_length(suppressWarnings(contingency_coef(tab)), 1)
  expect_length(yule_q(tab2), 1)
  expect_length(suppressWarnings(lambda_gk(tab)), 1)
  expect_length(suppressWarnings(goodman_kruskal_tau(tab)), 1)
  expect_length(suppressWarnings(uncertainty_coef(tab)), 1)
  expect_length(suppressWarnings(gamma_gk(tab)), 1)
  expect_length(suppressWarnings(kendall_tau_b(tab)), 1)
  expect_length(suppressWarnings(kendall_tau_c(tab)), 1)
  expect_length(suppressWarnings(somers_d(tab, "row")), 1)
})

# ── cramer_v ─────────────────────────────────────────────────────────────────

test_that("cramer_v returns correct estimate and 4-element vector", {
  res <- suppressWarnings(cramer_v(tab_3x3(), detail = TRUE))
  expect_type(res, "double")
  expect_length(res, 4)
  expect_named(res, c("estimate", "ci_lower", "ci_upper", "p_value"))
  expect_equal(res[["estimate"]], 0.5308655, tolerance = 1e-5)
  expect_true(res[["p_value"]] < 0.01)
})

test_that("cramer_v with detail = TRUE, conf_level = NULL returns 2 elements", {
  res <- suppressWarnings(cramer_v(tab_3x3(), detail = TRUE, conf_level = NULL))
  expect_length(res, 2)
  expect_named(res, c("estimate", "p_value"))
})

test_that("cramer_v rejects non-table input", {
  expect_error(cramer_v(iris), "contingency table")
})

# ── phi ──────────────────────────────────────────────────────────────────────

test_that("phi returns correct value for 2x2 table", {
  res <- phi(tab_2x2(), detail = TRUE)
  expect_equal(res[["estimate"]], 0.1683451, tolerance = 1e-5)
})

test_that("phi rejects non-2x2 tables", {
  expect_error(phi(tab_3x3()), "2x2")
})

# ── contingency_coef ─────────────────────────────────────────────────────────

test_that("contingency_coef matches DescTools", {
  res <- suppressWarnings(contingency_coef(tab_3x3(), detail = TRUE))
  expect_equal(res[["estimate"]], 0.6003875, tolerance = 1e-5)
  expect_true(is.na(res[["ci_lower"]]))
})

# ── yule_q ───────────────────────────────────────────────────────────────────

test_that("yule_q matches DescTools for 2x2", {
  res <- yule_q(tab_2x2(), detail = TRUE)
  expect_equal(res[["estimate"]], 0.3333333, tolerance = 1e-5)
})

test_that("yule_q rejects non-2x2 tables", {
  expect_error(yule_q(tab_3x3()), "2x2")
})

# ── lambda_gk ────────────────────────────────────────────────────────────────

test_that("lambda_gk matches DescTools for all directions", {
  tab <- tab_3x3()
  sym <- suppressWarnings(lambda_gk(tab, "symmetric", detail = TRUE))
  row <- suppressWarnings(lambda_gk(tab, "row", detail = TRUE))
  col <- suppressWarnings(lambda_gk(tab, "column", detail = TRUE))

  expect_equal(sym[["estimate"]], 0.4857143, tolerance = 1e-5)
  expect_equal(row[["estimate"]], 0.5294118, tolerance = 1e-5)
  expect_equal(col[["estimate"]], 0.4444444, tolerance = 1e-5)

  # CI matches DescTools
  expect_equal(sym[["ci_lower"]], 0.2324839, tolerance = 1e-4)
  expect_equal(sym[["ci_upper"]], 0.7389447, tolerance = 1e-4)
})

# ── goodman_kruskal_tau ──────────────────────────────────────────────────────

test_that("goodman_kruskal_tau matches DescTools", {
  tab <- tab_3x3()
  row <- suppressWarnings(goodman_kruskal_tau(tab, "row", detail = TRUE))
  col <- suppressWarnings(goodman_kruskal_tau(tab, "column", detail = TRUE))

  expect_equal(row[["estimate"]], 0.3825603, tolerance = 1e-5)
  expect_equal(col[["estimate"]], 0.3386018, tolerance = 1e-5)
  expect_equal(row[["ci_lower"]], 0.1520724, tolerance = 1e-4)
  expect_equal(row[["ci_upper"]], 0.6130482, tolerance = 1e-4)
})

# ── uncertainty_coef ─────────────────────────────────────────────────────────

test_that("uncertainty_coef returns sensible values", {
  tab <- tab_3x3()
  sym <- suppressWarnings(uncertainty_coef(tab, "symmetric", detail = TRUE))
  row <- suppressWarnings(uncertainty_coef(tab, "row", detail = TRUE))
  col <- suppressWarnings(uncertainty_coef(tab, "column", detail = TRUE))

  expect_true(sym[["estimate"]] > 0 && sym[["estimate"]] < 1)
  expect_true(row[["estimate"]] > 0 && row[["estimate"]] < 1)
  expect_true(col[["estimate"]] > 0 && col[["estimate"]] < 1)
  expect_length(sym, 4)
})

# ── gamma_gk ─────────────────────────────────────────────────────────────────

test_that("gamma_gk matches DescTools", {
  res <- suppressWarnings(gamma_gk(tab_3x3(), detail = TRUE))
  expect_equal(res[["estimate"]], -0.6573705, tolerance = 1e-5)
  expect_equal(res[["ci_lower"]], -0.9960223, tolerance = 1e-4)
  expect_equal(res[["ci_upper"]], -0.3187187, tolerance = 1e-4)
})

# ── kendall_tau_b ────────────────────────────────────────────────────────────

test_that("kendall_tau_b matches DescTools", {
  res <- suppressWarnings(kendall_tau_b(tab_3x3(), detail = TRUE))
  expect_equal(res[["estimate"]], -0.5125435, tolerance = 1e-5)
  expect_equal(res[["ci_lower"]], -0.8043717, tolerance = 1e-4)
  expect_equal(res[["ci_upper"]], -0.2207153, tolerance = 1e-4)
})

# ── kendall_tau_c ────────────────────────────────────────────────────────────

test_that("kendall_tau_c matches DescTools", {
  res <- suppressWarnings(kendall_tau_c(tab_3x3(), detail = TRUE))
  expect_equal(res[["estimate"]], -0.4833984, tolerance = 1e-5)
  expect_equal(res[["ci_lower"]], -0.7422941, tolerance = 1e-4)
  expect_equal(res[["ci_upper"]], -0.2245028, tolerance = 1e-4)
})

# ── somers_d ─────────────────────────────────────────────────────────────────

test_that("somers_d matches DescTools for row and column", {
  tab <- tab_3x3()
  row <- suppressWarnings(somers_d(tab, "row", detail = TRUE))
  col <- suppressWarnings(somers_d(tab, "column", detail = TRUE))

  expect_equal(row[["estimate"]], -0.5015198, tolerance = 1e-5)
  expect_equal(col[["estimate"]], -0.5238095, tolerance = 1e-5)
  expect_equal(col[["ci_lower"]], -0.8383951, tolerance = 1e-4)
  expect_equal(col[["ci_upper"]], -0.2092240, tolerance = 1e-4)
})

test_that("somers_d symmetric returns tau-b", {
  tab <- tab_3x3()
  sym <- suppressWarnings(somers_d(tab, "symmetric", detail = TRUE))
  tau <- suppressWarnings(kendall_tau_b(tab, detail = TRUE))
  expect_equal(sym[["estimate"]], tau[["estimate"]], tolerance = 1e-10)
})

# ── assoc_measures ───────────────────────────────────────────────────────────

test_that("assoc_measures returns data.frame with correct structure", {
  tab <- tab_3x3()
  res <- suppressWarnings(assoc_measures(tab))
  expect_s3_class(res, "data.frame")
  expect_true(all(
    c("measure", "estimate", "se", "ci_lower", "ci_upper", "p_value") %in%
      names(res)
  ))
  expect_true(nrow(res) > 0)
})

test_that("assoc_measures auto returns all measures", {
  tab <- tab_3x3()
  res <- suppressWarnings(assoc_measures(tab))
  expect_true("Cramer's V" %in% res$measure)
  expect_true("Goodman-Kruskal Gamma" %in% res$measure)
  expect_true("Kendall's Tau-b" %in% res$measure)
})

test_that("assoc_measures type ordinal returns ordinal measures only", {
  tab <- tab_3x3()
  res <- suppressWarnings(assoc_measures(tab, type = "ordinal"))
  expect_true("Goodman-Kruskal Gamma" %in% res$measure)
  expect_true("Kendall's Tau-b" %in% res$measure)
  expect_false("Cramer's V" %in% res$measure)
})

test_that("assoc_measures type nominal returns nominal measures only", {
  tab <- tab_3x3()
  res <- suppressWarnings(assoc_measures(tab, type = "nominal"))
  expect_true("Cramer's V" %in% res$measure)
  expect_false("Goodman-Kruskal Gamma" %in% res$measure)
})

test_that("assoc_measures includes phi and yule_q for 2x2", {
  tab <- tab_2x2()
  res <- suppressWarnings(assoc_measures(tab))
  expect_true("Phi" %in% res$measure)
  expect_true("Yule's Q" %in% res$measure)
})

# ── Input validation ─────────────────────────────────────────────────────────

test_that("all functions reject non-table input", {
  expect_error(gamma_gk(matrix(1:4, 2)), "contingency table")
  expect_error(kendall_tau_b(data.frame(a = 1)), "contingency table")
  expect_error(somers_d(1:10), "contingency table")
})

test_that("detail = TRUE returns spicy_assoc_detail class", {
  tab <- table(c("A", "B", "A", "B"), c("X", "X", "Y", "Y"))
  res <- cramer_v(tab, detail = TRUE)
  expect_s3_class(res, "spicy_assoc_detail")
  expect_named(res, c("estimate", "ci_lower", "ci_upper", "p_value"))
  res2 <- cramer_v(tab, detail = TRUE, conf_level = NULL)
  expect_s3_class(res2, "spicy_assoc_detail")
  expect_named(res2, c("estimate", "p_value"))
})

test_that("print.spicy_assoc_detail formats output", {
  tab <- table(c("A", "B", "A", "B"), c("X", "X", "Y", "Y"))
  res <- cramer_v(tab, detail = TRUE)
  out <- capture.output(print(res))
  expect_length(out, 2)
  expect_match(out[1], "Estimate")
  expect_match(out[1], "\\bp\\b")
})

test_that("print.spicy_assoc_detail respects digits argument", {
  tab <- table(c("A", "B", "A", "B"), c("X", "X", "Y", "Y"))
  out4 <- capture.output(print(cramer_v(tab, detail = TRUE), digits = 4))
  expect_match(out4[2], "\\.[0-9]{4}")
})

test_that("digits argument in function propagates to print", {
  tab <- table(c("A", "B", "A", "B"), c("X", "X", "Y", "Y"))
  res <- cramer_v(tab, detail = TRUE, digits = 4)
  expect_equal(attr(res, "digits"), 4)
  out <- capture.output(print(res))
  expect_match(out[2], "\\.[0-9]{4}")
})

test_that("assoc_measures returns spicy_assoc_table class", {
  tab <- table(c("A", "B", "A", "B"), c("X", "X", "Y", "Y"))
  res <- assoc_measures(tab)
  expect_s3_class(res, "spicy_assoc_table")
  expect_s3_class(res, "data.frame")
  expect_true("measure" %in% names(res))
})

test_that("print.spicy_assoc_table formats output", {
  tab <- table(c("A", "B", "A", "B"), c("X", "X", "Y", "Y"))
  res <- assoc_measures(tab)
  out <- capture.output(print(res))
  expect_match(out[1], "Measure")
  expect_match(out[1], "\\bp\\b")
  expect_true(length(out) > 1)
})

test_that("print.spicy_assoc_table respects digits argument", {
  tab <- table(c("A", "B", "A", "B"), c("X", "X", "Y", "Y"))
  out4 <- capture.output(print(assoc_measures(tab), digits = 4))
  expect_match(out4[2], "\\.[0-9]{4}")
})

test_that("digits argument in assoc_measures propagates to print", {
  tab <- table(c("A", "B", "A", "B"), c("X", "X", "Y", "Y"))
  res <- assoc_measures(tab, digits = 2)
  expect_equal(attr(res, "digits"), 2)
  out <- capture.output(print(res))
  expect_match(out[2], "\\.[0-9]{2}\\b")
})

# ── Validation errors ──────────────────────────────────────────────────────

test_that("validation rejects non-2D table", {
  t3d <- array(1:8, dim = c(2, 2, 2))
  class(t3d) <- "table"
  expect_error(cramer_v(t3d), "two-dimensional")
})

# ── conf_level = NULL returns without CI ───────────────────────────────────

test_that("cramer_v detail with conf_level = NULL omits CI", {
  tab <- tab_3x3()
  res <- cramer_v(tab, detail = TRUE, conf_level = NULL)
  expect_s3_class(res, "spicy_assoc_detail")
  expect_true(!"ci_lower" %in% names(res))
  expect_true("p_value" %in% names(res))
})

test_that("phi detail with conf_level = NULL omits CI", {
  tab <- tab_2x2()
  res <- phi(tab, detail = TRUE, conf_level = NULL)
  expect_s3_class(res, "spicy_assoc_detail")
  expect_true(!"ci_lower" %in% names(res))
})

test_that("contingency_coef detail with conf_level = NULL omits CI", {
  tab <- tab_3x3()
  res <- contingency_coef(tab, detail = TRUE, conf_level = NULL)
  expect_s3_class(res, "spicy_assoc_detail")
  expect_true(!"ci_lower" %in% names(res))
})

# ── Degenerate tables ─────────────────────────────────────────────────────

test_that("cramer_v warns on degenerate table", {
  tab <- matrix(0L, 2, 2)
  class(tab) <- "table"
  expect_warning(cramer_v(tab, detail = TRUE), "undefined")
})

test_that("yule_q warns when ad + bc = 0", {
  tab <- matrix(c(0L, 0L, 0L, 5L), 2, 2)
  class(tab) <- "table"
  expect_warning(yule_q(tab), "undefined")
})

test_that("yule_q handles zero cells in detail mode", {
  tab <- matrix(c(5L, 0L, 3L, 2L), 2, 2)
  class(tab) <- "table"
  res <- yule_q(tab, detail = TRUE)
  expect_true(is.na(res[["p_value"]]))
})

test_that("gamma_gk warns when C + D = 0", {
  tab <- matrix(c(5L, 0L, 3L, 0L), 2, 2)
  class(tab) <- "table"
  expect_warning(gamma_gk(tab), "No concordant")
})

test_that("kendall_tau_b warns on degenerate table", {
  tab <- matrix(c(5L, 5L, 5L, 5L), 2, 2)
  class(tab) <- "table"
  res <- kendall_tau_b(tab)
  expect_true(is.numeric(res))
})

test_that("kendall_tau_c warns on 1-row table", {
  tab <- matrix(c(5L, 3L), nrow = 1)
  class(tab) <- "table"
  expect_error(kendall_tau_c(tab), "at least")
})

test_that("somers_d warns when denom = 0", {
  tab <- matrix(c(5L, 0L, 3L, 0L), 2, 2)
  class(tab) <- "table"
  expect_warning(somers_d(tab, "column"), "undefined")
})

test_that("print.spicy_assoc_detail formats NA as --", {
  tab <- matrix(c(5L, 0L, 3L, 2L), 2, 2)
  class(tab) <- "table"
  res <- yule_q(tab, detail = TRUE)
  out <- capture.output(print(res))
  expect_true(any(grepl("--", out)))
})

test_that("print.spicy_assoc_detail formats p < 0.001", {
  tab <- matrix(c(50L, 0L, 0L, 50L), 2, 2)
  class(tab) <- "table"
  res <- cramer_v(tab, detail = TRUE)
  out <- capture.output(print(res))
  expect_true(any(grepl("< 0.001", out)))
})

test_that(".assoc_result detail = FALSE returns scalar", {
  tab <- tab_3x3()
  res <- gamma_gk(tab, detail = FALSE)
  expect_length(res, 1)
  expect_false(inherits(res, "spicy_assoc_detail"))
})

test_that(".assoc_result conf_level = NULL with .include_se = TRUE", {
  tab <- tab_3x3()
  res <- gamma_gk(tab, detail = TRUE, conf_level = NULL, .include_se = TRUE)
  expect_s3_class(res, "spicy_assoc_detail")
  expect_true("se" %in% names(res))
  expect_true("p_value" %in% names(res))
  expect_false("ci_lower" %in% names(res))
})

test_that("cramer_v conf_level = NULL with .include_se = TRUE", {
  tab <- tab_3x3()
  res <- cramer_v(tab, detail = TRUE, conf_level = NULL, .include_se = TRUE)
  expect_s3_class(res, "spicy_assoc_detail")
  expect_true("se" %in% names(res))
})

test_that("phi conf_level = NULL with .include_se = TRUE", {
  tab <- tab_2x2()
  res <- phi(tab, detail = TRUE, conf_level = NULL, .include_se = TRUE)
  expect_s3_class(res, "spicy_assoc_detail")
  expect_true("se" %in% names(res))
})

test_that("yule_q conf_level = NULL omits CI", {
  tab <- tab_2x2()
  res <- yule_q(tab, detail = TRUE, conf_level = NULL)
  expect_s3_class(res, "spicy_assoc_detail")
  expect_true("p_value" %in% names(res))
  expect_false("ci_lower" %in% names(res))
})

test_that("contingency_coef conf_level = NULL with .include_se = TRUE", {
  tab <- tab_3x3()
  res <- contingency_coef(
    tab,
    detail = TRUE,
    conf_level = NULL,
    .include_se = TRUE
  )
  expect_s3_class(res, "spicy_assoc_detail")
  expect_true("se" %in% names(res))
})

test_that("kendall_tau_b warns when denom = 0", {
  tab <- matrix(c(0L, 0L, 3L, 7L), 2, 2)
  class(tab) <- "table"
  expect_warning(kendall_tau_b(tab, detail = TRUE), "undefined")
})

test_that("kendall_tau_b se_sq clamped to 0 when tiny negative", {
  tab <- matrix(c(10L, 0L, 0L, 10L), 2, 2)
  class(tab) <- "table"
  res <- kendall_tau_b(tab, detail = TRUE)
  expect_s3_class(res, "spicy_assoc_detail")
})

test_that("lambda_gk row detail covers L_row_max branch", {
  tab <- matrix(c(10L, 0L, 0L, 5L), 2, 2)
  class(tab) <- "table"
  res <- lambda_gk(tab, direction = "row", detail = TRUE)
  expect_s3_class(res, "spicy_assoc_detail")
})

test_that("lambda_gk column detail covers L_col_max branch", {
  tab <- matrix(c(10L, 0L, 0L, 5L), 2, 2)
  class(tab) <- "table"
  res <- lambda_gk(tab, direction = "column", detail = TRUE)
  expect_s3_class(res, "spicy_assoc_detail")
})

code_book_filename <- function(title, filename = NULL) {
  if (!is.null(filename)) {
    return(code_book_sanitize_filename(
      filename,
      arg = "filename",
      fallback = NULL
    ))
  }

  code_book_sanitize_filename(
    title,
    arg = "title",
    fallback = "Codebook"
  )
}


code_book_sanitize_filename <- function(filename, arg, fallback = NULL) {
  if (is.null(filename)) {
    return(fallback)
  }

  filename <- code_book_ascii_filename(trimws(filename))

  if (is.na(filename)) {
    filename <- ""
  }

  filename <- gsub("[^A-Za-z0-9_-]+", "_", filename, perl = TRUE)
  filename <- gsub("_+", "_", filename, perl = TRUE)
  filename <- gsub("^_+|_+$", "", filename, perl = TRUE)

  if (!nzchar(filename)) {
    if (!is.null(fallback)) {
      return(fallback)
    }

    stop(
      "`",
      arg,
      "` must contain at least one letter, number, ",
      "underscore, or hyphen after sanitization.",
      call. = FALSE
    )
  }

  max_length <- 120L
  if (nchar(filename, type = "chars") > max_length) {
    filename <- substr(filename, 1L, max_length)
    filename <- gsub("_+$", "", filename, perl = TRUE)
  }

  if (!nzchar(filename)) {
    if (!is.null(fallback)) {
      return(fallback)
    }

    stop(
      "`",
      arg,
      "` must contain at least one letter, number, ",
      "underscore, or hyphen after sanitization.",
      call. = FALSE
    )
  }

  filename
}


code_book_ascii_filename <- function(filename) {
  filename <- enc2utf8(filename)
  filename <- gsub("\\p{M}+", "", filename, perl = TRUE)
  filename <- suppressWarnings(
    iconv(filename, from = "UTF-8", to = "ASCII//TRANSLIT", sub = "")
  )

  if (is.na(filename)) {
    return(filename)
  }

  # Some iconv implementations transliterate accents as ASCII marks.
  filename <- gsub("\\p{M}+", "", filename, perl = TRUE)
  gsub("[`'\"^~]+", "", filename, perl = TRUE)
}

data("sochealth", package = "spicy", envir = environment())

long_cat <- table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  statistic = TRUE,
  effect_size = "f2",
  output = "long"
)
long_cat_no_ci <- table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  statistic = FALSE,
  p_value = FALSE,
  show_n = FALSE,
  effect_size = "none",
  r2 = "none",
  ci = FALSE,
  output = "long"
)
long_num <- table_continuous_lm(
  sochealth,
  select = wellbeing_score,
  by = age,
  statistic = TRUE,
  effect_size = "f2",
  r2 = "adj_r2",
  output = "long"
)
long_3lvl <- table_continuous_lm(
  sochealth,
  select = wellbeing_score,
  by = education,
  statistic = TRUE,
  output = "long"
)
invisible(table_continuous_lm(
  sochealth,
  select = wellbeing_score,
  by = sex,
  labels = c(wellbeing_score = "Named wellbeing"),
  output = "long"
))

invisible(build_wide_raw_continuous_lm(
  long_cat,
  show_statistic = TRUE,
  show_p_value = TRUE,
  show_n = TRUE,
  effect_size = "f2",
  r2_type = "r2",
  ci = TRUE
))
invisible(build_wide_raw_continuous_lm(
  long_cat_no_ci,
  show_statistic = FALSE,
  show_p_value = FALSE,
  show_n = FALSE,
  effect_size = "none",
  r2_type = "none",
  ci = FALSE
))
invisible(build_wide_raw_continuous_lm(
  long_num,
  show_statistic = TRUE,
  show_p_value = TRUE,
  show_n = TRUE,
  effect_size = "f2",
  r2_type = "adj_r2",
  ci = TRUE
))

wide_cat <- build_wide_display_df_continuous_lm(
  long_cat,
  digits = 2L,
  decimal_mark = ".",
  ci_level = 0.95,
  show_statistic = TRUE,
  show_p_value = TRUE,
  show_n = TRUE,
  effect_size = "f2",
  r2_type = "r2",
  ci = TRUE
)
invisible(build_wide_display_df_continuous_lm(
  long_cat_no_ci,
  digits = 2L,
  decimal_mark = ",",
  ci_level = 0.95,
  show_statistic = FALSE,
  show_p_value = FALSE,
  show_n = FALSE,
  effect_size = "none",
  r2_type = "none",
  ci = FALSE
))
invisible(build_wide_display_df_continuous_lm(
  long_num,
  digits = 2L,
  decimal_mark = ".",
  ci_level = 0.95,
  show_statistic = TRUE,
  show_p_value = TRUE,
  show_n = TRUE,
  effect_size = "f2",
  r2_type = "adj_r2",
  ci = TRUE
))
invisible(build_wide_display_df_continuous_lm(
  make_empty_lm_rows("y", "Y", "continuous"),
  digits = 2L,
  decimal_mark = ".",
  ci_level = 0.95,
  show_statistic = TRUE,
  show_p_value = TRUE,
  show_n = TRUE,
  effect_size = "none",
  r2_type = "none",
  ci = TRUE
))

tmp_xlsx <- tempfile(fileext = ".xlsx")
tmp_docx <- tempfile(fileext = ".docx")

invisible(export_continuous_lm_table(
  wide_cat,
  output = "tinytable",
  ci_level = 0.95,
  excel_path = NULL,
  excel_sheet = "Sheet1",
  clipboard_delim = "\t",
  word_path = NULL
))
invisible(export_continuous_lm_table(
  wide_cat,
  output = "gt",
  ci_level = 0.95,
  excel_path = NULL,
  excel_sheet = "Sheet1",
  clipboard_delim = "\t",
  word_path = NULL
))
invisible(export_continuous_lm_table(
  wide_cat[,
    setdiff(names(wide_cat), c("95% CI LL", "95% CI UL")),
    drop = FALSE
  ],
  output = "gt",
  ci_level = 0.95,
  excel_path = NULL,
  excel_sheet = "Sheet1",
  clipboard_delim = "\t",
  word_path = NULL
))
invisible(export_continuous_lm_table(
  wide_cat,
  output = "flextable",
  ci_level = 0.95,
  excel_path = NULL,
  excel_sheet = "Sheet1",
  clipboard_delim = "\t",
  word_path = NULL
))
local({
  testthat::local_mocked_bindings(
    wb_save = function(...) invisible(NULL),
    .package = "openxlsx2"
  )
  invisible(export_continuous_lm_table(
    wide_cat,
    output = "excel",
    ci_level = 0.95,
    excel_path = tmp_xlsx,
    excel_sheet = "Sheet1",
    clipboard_delim = "\t",
    word_path = NULL
  ))
})
local({
  testthat::local_mocked_bindings(
    save_as_docx = function(..., path) path,
    .package = "flextable"
  )
  invisible(export_continuous_lm_table(
    wide_cat,
    output = "word",
    ci_level = 0.95,
    excel_path = NULL,
    excel_sheet = "Sheet1",
    clipboard_delim = "\t",
    word_path = tmp_docx
  ))
})

rename_ci_cols_lm(
  setNames(data.frame(1, 2, check.names = FALSE), c("95% CI LL", "95% CI UL")),
  "95% CI LL",
  "95% CI UL"
)
build_header_rows_lm(c("Variable", "LL", "UL"), "95%")
get_delta_label_lm(long_cat[
  long_cat$variable == "wellbeing_score",
  ,
  drop = FALSE
])
get_test_row_index_lm(long_cat[
  long_cat$variable == "wellbeing_score",
  ,
  drop = FALSE
])
get_test_header_lm(
  long_cat[long_cat$variable == "wellbeing_score", , drop = FALSE],
  TRUE,
  FALSE
)
get_test_header_lm(
  long_3lvl[long_3lvl$variable == "wellbeing_score", , drop = FALSE],
  TRUE,
  TRUE
)
get_test_header_lm(
  long_3lvl[long_3lvl$variable == "wellbeing_score", , drop = FALSE],
  TRUE,
  FALSE
)
format_effect_size_header_lm("f2")
format_effect_size_header_lm("none")
format_r2_header_lm("r2")
format_r2_header_lm("adj_r2")
get_r2_value_lm(
  long_num[long_num$variable == "wellbeing_score", , drop = FALSE],
  "adj_r2"
)
format_number_lm(c(1.23, NA_real_), 2L, ",")
format_p_value_lm(NA_real_, ".")
format_p_value_lm(0.0005, ".")
format_p_value_lm(0.045, ",")

make_empty_lm_rows("y", "Y", "continuous")
fit_numeric_predictor_lm_rows(
  y = c(1, 2),
  x = c(1, 1),
  weights = NULL,
  outcome_name = "y",
  outcome_label = "Y",
  predictor_label = "X",
  vcov_type = "classical",
  ci_level = 0.95
)
fit_numeric_predictor_lm_rows(
  y = c(1, 2, 3),
  x = c(1, 2, 3),
  weights = c(1, 2, 3),
  outcome_name = "y",
  outcome_label = "Y",
  predictor_label = "X",
  vcov_type = "HC3",
  ci_level = 0.95
)
fit_numeric_predictor_lm_rows(
  y = c(1, 2),
  x = c(1, 2),
  weights = NULL,
  outcome_name = "y",
  outcome_label = "Y",
  predictor_label = "X",
  vcov_type = "classical",
  ci_level = 0.95
)
fit_categorical_predictor_lm_rows(
  y = c(1, 2),
  x = factor(c("A", "A")),
  weights = NULL,
  outcome_name = "y",
  outcome_label = "Y",
  predictor_label = "G",
  vcov_type = "classical",
  contrast = "auto",
  ci_level = 0.95
)
fit_categorical_predictor_lm_rows(
  y = c(1, 2),
  x = factor(c("A", "B")),
  weights = NULL,
  outcome_name = "y",
  outcome_label = "Y",
  predictor_label = "G",
  vcov_type = "classical",
  contrast = "auto",
  ci_level = 0.95
)
is_supported_lm_predictor(TRUE)
coerce_lm_factor(c("a", "b"))
detect_weights_column_name(rlang::quo(NULL), sochealth)
detect_weights_column_name(rlang::quo(survey_weight), sochealth)
detect_weights_column_name(rlang::quo("survey_weight"), sochealth)
detect_weights_column_name(rlang::quo("w"), data.frame(y = 1:3, w = 1:3))

fit <- lm(mpg ~ wt, data = mtcars)
compute_lm_model_stats(fit)
compute_lm_vcov(fit, "classical")
compute_lm_vcov(fit, "HC0")
compute_lm_vcov(fit, "HC1")
compute_lm_vcov(fit, "HC2")
compute_lm_vcov(fit, "HC3")
compute_lm_vcov(fit, "HC4")
compute_lm_vcov(fit, "HC4m")
compute_lm_vcov(fit, "HC5")

local({
  testthat::local_mocked_bindings(
    requireNamespace = function(package, quietly = TRUE) FALSE,
    .package = "base"
  )
  try(
    export_continuous_lm_table(
      wide_cat,
      "tinytable",
      0.95,
      NULL,
      "Sheet1",
      "\t",
      NULL
    ),
    silent = TRUE
  )
})
local({
  testthat::local_mocked_bindings(
    requireNamespace = function(package, quietly = TRUE) package != "gt",
    .package = "base"
  )
  try(
    export_continuous_lm_table(
      wide_cat,
      "gt",
      0.95,
      NULL,
      "Sheet1",
      "\t",
      NULL
    ),
    silent = TRUE
  )
})
local({
  testthat::local_mocked_bindings(
    requireNamespace = function(package, quietly = TRUE) package != "flextable",
    .package = "base"
  )
  try(
    export_continuous_lm_table(
      wide_cat,
      "flextable",
      0.95,
      NULL,
      "Sheet1",
      "\t",
      NULL
    ),
    silent = TRUE
  )
})
local({
  testthat::local_mocked_bindings(
    requireNamespace = function(package, quietly = TRUE) package != "officer",
    .package = "base"
  )
  try(
    export_continuous_lm_table(
      wide_cat,
      "word",
      0.95,
      NULL,
      "Sheet1",
      "\t",
      tmp_docx
    ),
    silent = TRUE
  )
})
local({
  testthat::local_mocked_bindings(
    requireNamespace = function(package, quietly = TRUE) package != "openxlsx2",
    .package = "base"
  )
  try(
    export_continuous_lm_table(
      wide_cat,
      "excel",
      0.95,
      tmp_xlsx,
      "Sheet1",
      "\t",
      NULL
    ),
    silent = TRUE
  )
})
try(
  export_continuous_lm_table(
    wide_cat,
    "word",
    0.95,
    NULL,
    "Sheet1",
    "\t",
    NULL
  ),
  silent = TRUE
)
try(
  export_continuous_lm_table(
    wide_cat,
    "excel",
    0.95,
    NULL,
    "Sheet1",
    "\t",
    NULL
  ),
  silent = TRUE
)
local({
  testthat::local_mocked_bindings(
    requireNamespace = function(package, quietly = TRUE) package != "clipr",
    .package = "base"
  )
  try(
    export_continuous_lm_table(
      wide_cat,
      "clipboard",
      0.95,
      NULL,
      "Sheet1",
      "\t",
      NULL
    ),
    silent = TRUE
  )
})
try(
  export_continuous_lm_table(
    wide_cat,
    "bogus",
    0.95,
    NULL,
    "Sheet1",
    "\t",
    NULL
  ),
  silent = TRUE
)

unlink(c(tmp_xlsx, tmp_docx), force = TRUE)

#' Cross-tabulation
#'
#' @description
#' Computes a two-way cross-tabulation with optional weights, grouping
#' (including combinations of multiple variables), percentage displays,
#' and inferential statistics.
#'
#' `cross_tab()` produces weighted or unweighted contingency tables with
#' row or column percentages, optional grouping via `by`, and associated
#' Chi-squared tests with an association measure and diagnostic information.
#'
#' Both `x` and `y` variables are required. For one-way frequency tables,
#' use [freq()] instead.
#'
#' @param data A data frame. Alternatively, a vector when using the
#'   vector-based interface.
#' @param x Row variable (unquoted).
#' @param y Column variable (unquoted). Mandatory; for one-way tables, use [freq()].
#' @param by Optional grouping variable or expression. Can be a single variable
#'   or a combination of multiple variables (e.g. `interaction(vs, am)`).
#' @param weights Optional numeric weights.
#' @param rescale Logical. If `FALSE` (the default), weights are used as-is.
#'   If `TRUE`, rescales weights so total weighted N matches raw N.
#' @param percent One of `"none"` (the default), `"row"`, `"column"`.
#'   Unique abbreviations are accepted (e.g. `"n"`, `"r"`, `"c"`).
#' @param include_stats Logical. If `TRUE` (the default), computes Chi-squared
#'   and an association measure (see `assoc_measure`).
#' @param assoc_measure Character. Which association measure to report.
#'   `"auto"` (default) selects Kendall's Tau-b when both variables are
#'   ordered factors and Cramer's V otherwise. Other choices:
#'   `"cramer_v"`, `"phi"`, `"gamma"`, `"tau_b"`, `"tau_c"`,
#'   `"somers_d"`, `"lambda"`, `"none"`.
#' @param assoc_ci Logical. If `TRUE`, includes the 95 percent confidence
#'   interval of the association measure in the note. Defaults to `FALSE`.
#' @param correct Logical. If `FALSE` (the default), no continuity correction is
#'   applied. If `TRUE`, applies Yates correction (only for 2x2 tables).
#' @param simulate_p Logical. If `FALSE` (the default), uses asymptotic p-values.
#'   If `TRUE`, uses Monte Carlo simulation.
#' @param simulate_B Integer. Number of replicates for Monte Carlo simulation.
#'   Defaults to `2000`.
#' @param digits Number of decimals. Defaults to `1` for percentages, `0` for counts.
#' @param styled Logical. If `TRUE` (the default), returns a `spicy_cross_table` object
#'   (for formatted printing). If `FALSE`, returns a plain `data.frame`.
#' @param show_n Logical. If `TRUE` (the default), adds marginal N totals when
#'   `percent != "none"`.
#'
#' @return
#' A `data.frame`, list of data.frames, or `spicy_cross_table` object.
#' When `by` is used, returns a `spicy_cross_table_list`.
#'
#' @section Global Options:
#'
#' The function recognizes the following global options that modify its default behavior:
#'
#' * **`options(spicy.percent = "column")`**
#'   Sets the default percentage mode for all calls to `cross_tab()`.
#'   Valid values are `"none"`, `"row"`, and `"column"`.
#'   Equivalent to setting `percent = "column"` (or another choice) in each call.
#'
#' * **`options(spicy.simulate_p = TRUE)`**
#'   Enables Monte Carlo simulation for all Chi-squared tests by default.
#'   Equivalent to setting `simulate_p = TRUE` in every call.
#'
#' * **`options(spicy.rescale = TRUE)`**
#'   Automatically rescales weights so that total weighted N equals the raw N.
#'   Equivalent to setting `rescale = TRUE` in each call.
#'
#' These options are convenient for users who wish to enforce consistent behavior
#' across multiple calls to `cross_tab()` and other spicy table functions.
#' They can be disabled or reset by setting them to `NULL`:
#' `options(spicy.percent = NULL, spicy.simulate_p = NULL, spicy.rescale = NULL)`.
#'
#' Example:
#' ```r
#' options(spicy.simulate_p = TRUE, spicy.rescale = TRUE)
#' cross_tab(sochealth, smoking, education, weights = weight)
#' ```
#' @examples
#' # Basic crosstab
#' cross_tab(sochealth, smoking, education)
#'
#' # Column percentages
#' cross_tab(sochealth, smoking, education, percent = "column")
#'
#' # Weighted (rescaled)
#' cross_tab(sochealth, smoking, education, weights = weight, rescale = TRUE)
#'
#' # Grouped by sex
#' cross_tab(sochealth, smoking, education, by = sex)
#'
#' # Grouped by combination of variables
#' cross_tab(sochealth, smoking, education, by = interaction(sex, age_group))
#'
#' # Ordinal variables: auto-selects Kendall's Tau-b
#' cross_tab(sochealth, education, self_rated_health)
#'
#' # 2x2 table with Yates correction
#' cross_tab(sochealth, smoking, physical_activity, correct = TRUE)
#'
#' @export
cross_tab <- function(
  data,
  x,
  y = NULL,
  by = NULL,
  weights = NULL,
  rescale = FALSE,
  percent = c("none", "column", "row"),
  include_stats = TRUE,
  assoc_measure = c(
    "auto",
    "cramer_v",
    "phi",
    "gamma",
    "tau_b",
    "tau_c",
    "somers_d",
    "lambda",
    "none"
  ),
  assoc_ci = FALSE,
  correct = FALSE,
  simulate_p = FALSE,
  simulate_B = 2000,
  digits = NULL,
  styled = TRUE,
  show_n = TRUE
) {
  if (missing(data)) {
    stop("You must provide a dataset or a vector for `data`.", call. = FALSE)
  }

  call_x <- substitute(x)
  call_y <- substitute(y)
  call_data <- substitute(data)
  call_by <- substitute(by)
  call_weights <- substitute(weights)

  is_vector_input <- !is.data.frame(data) &&
    is.null(dim(data)) &&
    (is.atomic(data) || is.factor(data))

  if (is.data.frame(data)) {
    if (missing(x)) {
      stop(
        "You must specify at least one variable name for `x` (e.g., cross_tab(data, x, y)).",
        call. = FALSE
      )
    }
    if (missing(y) || identical(call_y, quote(NULL))) {
      stop(
        "You must specify a `y` variable (e.g., cross_tab(data, x, y)).",
        call. = FALSE
      )
    }
  }

  if (is_vector_input) {
    if (missing(x) || identical(call_x, quote(NULL))) {
      stop(
        "When using vector input, you must provide both x and y vectors of the same length (e.g., cross_tab(data$x, data$y)).",
        call. = FALSE
      )
    }
    if (length(data) != length(x)) {
      stop("Vectors `x` and `y` must have the same length.", call. = FALSE)
    }
  }

  # Global options
  if (missing(simulate_p)) {
    simulate_p <- getOption("spicy.simulate_p", FALSE)
  }
  if (missing(rescale)) {
    rescale <- getOption("spicy.rescale", FALSE)
  }
  if (missing(percent)) {
    percent <- getOption("spicy.percent", "none")
  }

  percent <- match.arg(percent)
  assoc_measure <- match.arg(assoc_measure)
  if (is.null(digits)) {
    digits <- if (percent == "none") 0 else 1
  }

  # Capture original expressions to retrieve variable names
  get_var_name <- function(expr) {
    if (is.symbol(expr)) {
      return(as.character(expr))
    }

    if (is.call(expr)) {
      fn <- expr[[1]]

      if (identical(fn, as.name("$")) && length(expr) >= 3) {
        return(as.character(expr[[3]]))
      }

      if (identical(fn, as.name("[[")) && length(expr) >= 3) {
        idx <- expr[[3]]
        if (is.character(idx)) {
          return(idx)
        }
        if (is.symbol(idx)) {
          return(as.character(idx))
        }
      }

      args <- as.list(expr)[-1]
      if (length(args) > 0) {
        for (arg in rev(args)) {
          nm <- get_var_name(arg)
          if (!is.null(nm) && nzchar(nm)) {
            return(nm)
          }
        }
      }
    }

    rlang::as_label(expr)
  }

  parse_by_name <- function(expr_txt, fallback_expr = NULL) {
    expr_txt <- gsub("^~", "", expr_txt)
    expr_txt <- gsub("\\s+", "", expr_txt)
    if (grepl("^interaction\\(", expr_txt)) {
      inside <- gsub("^interaction\\(|\\)$", "", expr_txt)
      parts <- unlist(strsplit(inside, ","))
      parts <- trimws(gsub(".*\\$", "", parts))
      paste(parts, collapse = " x ")
    } else if (!is.null(fallback_expr)) {
      get_var_name(fallback_expr)
    } else {
      trimws(gsub(".*\\$", "", expr_txt))
    }
  }

  make_levels <- function(v) {
    vals <- unique(v[!is.na(v)])
    if (length(vals) == 0) {
      return(vals)
    }
    tryCatch(sort(vals), error = function(e) vals)
  }

  # Call mode detection
  is_vector_mode <- is_vector_input

  if (is_vector_mode) {
    # Vector mode : cross_tab(df$x, df$y, ...)
    x_vals <- data
    y_vals <- x # 2nd argument becomes y

    # Weight
    if (!is.null(weights)) {
      if (!is.numeric(weights)) {
        stop(
          "When using vector input, `weights` must be a numeric vector.",
          call. = FALSE
        )
      }
      if (length(weights) != length(x_vals)) {
        stop(
          "`weights` must have the same length as `x` and `y` in vector mode.",
          call. = FALSE
        )
      }
      w_vals <- weights
    } else {
      w_vals <- rep(1, length(x_vals))
    }

    # Management of by
    if (!is.null(by)) {
      by_vals <- by
      if (length(by_vals) != length(x_vals)) {
        stop(
          "`by` must be the same length as `x` when using vector input.",
          call. = FALSE
        )
      }
    } else {
      by_vals <- rep(NA, length(x_vals))
    }

    # Building a complete data.frame
    data <- data.frame(
      x_tmp = x_vals,
      y_tmp = y_vals,
      by_tmp = by_vals,
      w_tmp = w_vals,
      stringsAsFactors = FALSE
    )

    # Create quosures for the rest of the code
    x_expr <- rlang::new_quosure(rlang::sym("x_tmp"))
    y_expr <- rlang::new_quosure(rlang::sym("y_tmp"))
    by_expr <- if (all(is.na(by_vals))) {
      rlang::quo(NULL)
    } else {
      rlang::new_quosure(rlang::sym("by_tmp"))
    }
    w_expr <- rlang::new_quosure(rlang::sym("w_tmp"))

    x_name <- get_var_name(call_data)
    y_name <- get_var_name(call_x)

    if (!missing(by) && !identical(call_by, quote(NULL))) {
      by_name <- parse_by_name(deparse(call_by), fallback_expr = call_by)
    } else {
      by_name <- NULL
    }
  } else {
    x_expr <- rlang::enquo(x)
    y_expr <- rlang::enquo(y)
    by_expr <- rlang::enquo(by)
    w_expr <- rlang::enquo(weights)

    x_name <- tryCatch(rlang::as_name(x_expr), error = function(e) {
      get_var_name(rlang::get_expr(x_expr))
    })
    y_name <- tryCatch(rlang::as_name(y_expr), error = function(e) {
      get_var_name(rlang::get_expr(y_expr))
    })

    if (!rlang::quo_is_null(by_expr)) {
      by_name <- parse_by_name(rlang::expr_text(by_expr))
    } else {
      by_name <- NULL
    }
  }

  if (!rlang::quo_is_null(w_expr)) {
    w <- rlang::eval_tidy(w_expr, data)
    if (!is.numeric(w)) {
      stop("`weights` must be numeric.", call. = FALSE)
    }
    if (length(w) != nrow(data)) {
      stop(
        "`weights` must have the same length as the number of rows.",
        call. = FALSE
      )
    }
    if (any(!is.finite(w[!is.na(w)]))) {
      stop("`weights` must contain only finite numeric values.", call. = FALSE)
    }
    if (any(w < 0, na.rm = TRUE)) {
      stop("`weights` must be non-negative.", call. = FALSE)
    }
    w[is.na(w)] <- 0
  } else {
    w <- rep(1, nrow(data))
  }

  if (rescale && rlang::quo_is_null(w_expr)) {
    warning(
      "`rescale = TRUE` has no effect since no weights provided.",
      call. = FALSE
    )
  }

  data$`..spicy_w` <- w
  full_x_levels <- make_levels(rlang::eval_tidy(x_expr, data))
  full_y_levels <- make_levels(rlang::eval_tidy(y_expr, data))

  make_named_row <- function(template_df, values) {
    row <- as.list(rep(NA, ncol(template_df)))
    names(row) <- names(template_df)

    nm <- names(values)
    for (i in seq_along(values)) {
      key <- nm[[i]]
      if (nzchar(key) && key %in% names(row)) {
        row[[key]] <- values[[i]]
      }
    }

    out <- as.data.frame(row, stringsAsFactors = FALSE, check.names = FALSE)
    rownames(out) <- NULL
    out
  }

  append_rows <- function(df, rows) {
    if (inherits(rows, "data.frame")) {
      rows <- list(rows)
    }
    rows <- rows[!vapply(rows, is.null, logical(1))]
    out <- do.call(rbind, c(list(df), rows))
    rownames(out) <- NULL
    out
  }

  compute_ctab <- function(df, group_label = NULL) {
    x_val <- rlang::eval_tidy(x_expr, df)
    y_val <- rlang::eval_tidy(y_expr, df)
    w_val <- df$`..spicy_w`

    df_sub <- data.frame(
      x_val = factor(x_val, levels = full_x_levels),
      y_val = factor(y_val, levels = full_y_levels),
      w_val = w_val,
      stringsAsFactors = FALSE
    )

    # Rescale weights on complete cases only (NA rows are dropped by xtabs)
    if (rescale && !rlang::quo_is_null(w_expr)) {
      complete <- complete.cases(df_sub[, c("x_val", "y_val", "w_val")])
      n_complete <- sum(complete)
      w_sum_complete <- sum(df_sub$w_val[complete], na.rm = TRUE)
      if (!is.finite(w_sum_complete) || w_sum_complete <= 0) {
        stop(
          "`rescale = TRUE` requires a strictly positive sum of weights.",
          call. = FALSE
        )
      }
      df_sub$w_val <- df_sub$w_val * n_complete / w_sum_complete
    }

    tab_full <- stats::xtabs(w_val ~ x_val + y_val, data = df_sub)

    total_n <- sum(tab_full, na.rm = TRUE)

    tab_perc <- switch(
      percent,
      "row" = prop.table(tab_full, 1) * 100,
      "column" = prop.table(tab_full, 2) * 100,
      "none" = tab_full
    )
    tab_perc[is.nan(tab_perc)] <- 0

    df_out <- as.data.frame.matrix(
      round(tab_perc, digits),
      stringsAsFactors = FALSE
    )
    df_out <- data.frame(
      Values = rownames(df_out),
      df_out,
      row.names = NULL,
      check.names = FALSE,
      stringsAsFactors = FALSE
    )

    if (styled) {
      if (percent == "column") {
        total_values <- colSums(tab_perc, na.rm = TRUE)
        n_values <- colSums(tab_full, na.rm = TRUE)

        df_out$Total <- round(
          rowSums(tab_full, na.rm = TRUE) / sum(tab_full) * 100,
          digits
        )

        total_row <- make_named_row(
          df_out,
          c(
            list(Values = "Total"),
            as.list(round(total_values, digits)),
            list(Total = 100)
          )
        )
        n_row <- if (show_n) {
          make_named_row(
            df_out,
            c(
              list(Values = "N"),
              as.list(round(n_values, 0)),
              list(Total = sum(tab_full))
            )
          )
        } else {
          NULL
        }

        df_out <- append_rows(df_out, list(total_row, n_row))
      } else if (percent == "row") {
        df_out$Total <- round(rowSums(tab_perc, na.rm = TRUE), digits)
        if (show_n) {
          df_out$N <- as.numeric(rowSums(tab_full, na.rm = TRUE))
        }

        col_tot <- colSums(tab_full, na.rm = TRUE)
        col_perc <- round(col_tot / sum(col_tot) * 100, digits)
        names(col_perc) <- names(col_tot)

        total_values <- c(
          list(Values = "Total"),
          as.list(col_perc),
          list(Total = 100)
        )
        if (show_n) {
          total_values <- c(total_values, list(N = sum(tab_full)))
        }
        total_row <- make_named_row(df_out, total_values)

        df_out <- append_rows(df_out, total_row)
      } else {
        df_out$Total <- as.numeric(rowSums(tab_full, na.rm = TRUE))
        grand_total <- make_named_row(
          df_out,
          c(
            list(Values = "Total"),
            as.list(colSums(df_out[, -1, drop = FALSE], na.rm = TRUE))
          )
        )
        df_out <- append_rows(df_out, grand_total)
      }
    }

    note <- NULL
    tab_stats <- tab_full
    tab_stats <- tab_stats[
      rowSums(tab_stats) > 0,
      colSums(tab_stats) > 0,
      drop = FALSE
    ]
    if (include_stats && all(dim(tab_stats) > 1)) {
      # Yates correction only meaningful for 2x2 tables
      correct_used <- isTRUE(correct) && all(dim(tab_stats) == c(2, 2))
      chi <- suppressWarnings(stats::chisq.test(
        tab_stats,
        correct = correct_used,
        simulate.p.value = simulate_p,
        B = simulate_B
      ))

      chi2 <- as.numeric(chi$statistic)
      df_ <- as.numeric(chi$parameter)
      pval <- as.numeric(chi$p.value)

      # Resolve association measure
      assoc_choice <- assoc_measure
      if (assoc_choice == "auto") {
        both_ordered <- is.ordered(x_val) && is.ordered(y_val)
        assoc_choice <- if (both_ordered) "tau_b" else "cramer_v"
      }

      # Compute association measure
      assoc_result <- NULL
      assoc_name <- NULL
      if (assoc_choice != "none") {
        assoc_labels <- c(
          cramer_v = "Cramer's V",
          phi = "Phi",
          gamma = "Goodman-Kruskal Gamma",
          tau_b = "Kendall's Tau-b",
          tau_c = "Kendall's Tau-c",
          somers_d = "Somers' D",
          lambda = "Lambda"
        )
        assoc_out <- tryCatch(
          suppressWarnings(switch(
            assoc_choice,
            cramer_v = cramer_v(tab_stats, detail = TRUE),
            phi = phi(tab_stats, detail = TRUE),
            gamma = gamma_gk(tab_stats, detail = TRUE),
            tau_b = kendall_tau_b(tab_stats, detail = TRUE),
            tau_c = kendall_tau_c(tab_stats, detail = TRUE),
            somers_d = somers_d(tab_stats, "symmetric", detail = TRUE),
            lambda = lambda_gk(tab_stats, "symmetric", detail = TRUE)
          )),
          error = function(e) NULL
        )
        if (!is.null(assoc_out)) {
          assoc_result <- assoc_out
          assoc_name <- assoc_labels[[assoc_choice]]
        }
      }

      estimate <- if (!is.null(assoc_result)) {
        assoc_result[["estimate"]]
      } else {
        NA_real_
      }

      p_str <- if (pval < 0.001) {
        "< 0.001"
      } else {
        paste0("= ", formatC(pval, format = "f", digits = 3))
      }

      chi2_str <- ifelse(
        is.nan(chi2) | is.na(chi2),
        "NA",
        formatC(chi2, format = "f", digits = 1)
      )
      note <- paste0(
        "Chi-2(",
        df_,
        ") = ",
        chi2_str,
        ", p ",
        p_str,
        if (simulate_p) " (simulated)"
      )

      if (!is.null(assoc_name) && !is.na(estimate)) {
        est_str <- formatC(estimate, format = "f", digits = 2)
        assoc_line <- paste0(assoc_name, " = ", est_str)
        if (isTRUE(assoc_ci) && !is.null(assoc_result)) {
          ci_lo <- assoc_result[["ci_lower"]]
          ci_hi <- assoc_result[["ci_upper"]]
          if (!is.na(ci_lo) && !is.na(ci_hi)) {
            assoc_line <- paste0(
              assoc_line,
              ", 95% CI [",
              formatC(ci_lo, format = "f", digits = 2),
              ", ",
              formatC(ci_hi, format = "f", digits = 2),
              "]"
            )
          }
        }
        note <- paste0(note, "\n", assoc_line)
      }

      if (isTRUE(correct_used)) {
        note <- paste0(note, "\nYates continuity correction applied.")
      }

      # Store numeric attributes
      attr(df_out, "chi2") <- chi2
      attr(df_out, "df") <- df_
      attr(df_out, "p_value") <- pval
      attr(df_out, "assoc_measure") <- assoc_name
      attr(df_out, "assoc_value") <- estimate
      attr(df_out, "assoc_result") <- assoc_result

      expected <- chi$expected
      small5 <- sum(expected < 5, na.rm = TRUE)
      small1 <- sum(expected < 1, na.rm = TRUE)
      prop5 <- small5 / length(expected)
      if ((prop5 > 0.20 || small1 > 0) && !simulate_p) {
        min_exp <- round(min(expected, na.rm = TRUE), 2)
        note <- paste0(
          note,
          "\nWarning: ",
          small5,
          " expected cell",
          if (small5 > 1) "s" else "",
          " < 5 (",
          round(prop5 * 100, 1),
          "%).",
          if (small1 > 0) {
            paste0(
              " ",
              small1,
              " expected cell",
              if (small1 > 1) "s" else "",
              " < 1."
            )
          },
          " Minimum expected = ",
          min_exp,
          ". Consider `simulate_p = TRUE` or set globally via `options(spicy.simulate_p = TRUE)`."
        )
      }
    }

    perc_label <- switch(
      percent,
      "row" = " (Row %)",
      "column" = " (Column %)",
      "none" = " (N)"
    )
    title <- paste0(
      "Crosstable: ",
      x_name,
      if (!is.null(y_name)) paste0(" x ", y_name),
      perc_label
    )
    if (!is.null(group_label)) {
      title <- paste0(title, " | ", by_name, " = ", group_label)
    }

    # Add weighting information to the note when applicable
    if (!rlang::quo_is_null(w_expr) && !isTRUE(all(w == 1))) {
      w_name <- get_var_name(call_weights)
      w_text <- paste0("Weight: ", w_name)
      if (isTRUE(rescale)) {
        w_text <- paste0(w_text, " (rescaled)")
      }

      # Append to the existing note or create a new one
      if (is.null(note) || note == "") {
        note <- w_text
      } else {
        note <- paste0(note, "\n", w_text)
      }
    }

    attr(df_out, "title") <- title
    attr(df_out, "note") <- note
    attr(df_out, "n_total") <- total_n
    attr(df_out, "digits") <- digits

    num_cols <- vapply(df_out, is.numeric, logical(1))
    if (any(num_cols)) {
      df_out[num_cols] <- lapply(df_out[num_cols], function(col) {
        col[is.nan(col)] <- NA_real_
        col
      })
    }
    df_out
  }

  if (!rlang::quo_is_null(by_expr)) {
    by_vals <- rlang::eval_tidy(by_expr, data)

    if (is.factor(by_vals)) {
      f <- droplevels(by_vals)
    } else {
      unique_levels <- sort(unique(by_vals), na.last = TRUE)
      f <- factor(by_vals, levels = unique_levels)
    }

    split_data <- split(data, f, drop = TRUE)[levels(f)]
    split_data <- split_data[!vapply(split_data, is.null, logical(1))]

    level_names <- names(split_data)
    tables <- Map(
      function(df, lvl) compute_ctab(df, lvl),
      split_data,
      level_names
    )
    names(tables) <- level_names

    if (styled) {
      tables <- lapply(tables, function(tt) {
        class(tt) <- c("spicy_cross_table", "spicy_table", class(tt))
        tt
      })
      class(tables) <- c("spicy_cross_table_list", class(tables))
    }
    return(tables)
  } else {
    out <- compute_ctab(data)
  }

  if (styled) {
    class(out) <- c("spicy_cross_table", "spicy_table", class(out))
    out
  } else {
    as.data.frame(out, stringsAsFactors = FALSE)
  }
}


#' Internal print method for lists of cross-tab tables
#'
#' @description
#' Prints each element of a `spicy_cross_table_list` object on its own,
#' inserting a blank line between tables.
#'
#' @name print.spicy_cross_table_list
#'
#' @param x A `spicy_cross_table_list` object.
#' @param ... Additional arguments passed to individual print methods.
#'
#' @return Invisibly returns `x`.
#'
#' @keywords internal
#' @export
print.spicy_cross_table_list <- function(x, ...) {
  n <- length(x)
  for (i in seq_len(n)) {
    print(x[[i]], ...)
    if (i < n) cat("\n")
  }
  invisible(x)
}


#' @title Print method for spicy_cross_table objects
#' @description
#' Prints a formatted SPSS-like crosstable created by [cross_tab()].
#'
#' @param x A `spicy_cross_table` object.
#' @param digits Optional integer; number of decimal places to display.
#'   Defaults to the value stored in the object.
#' @param ... Additional arguments passed to internal formatting functions.
#'
#' @keywords internal
#' @export
print.spicy_cross_table <- function(x, digits = NULL, ...) {
  title <- attr(x, "title")
  digits_attr <- attr(x, "digits")

  if (is.null(digits)) {
    digits <- if (!is.null(digits_attr)) {
      digits_attr
    } else if (grepl("%", title)) {
      1
    } else {
      0
    }
  }

  df_display <- x

  is_n_row <- if ("Values" %in% names(df_display)) {
    df_display$Values == "N"
  } else {
    rep(FALSE, nrow(df_display))
  }
  has_n_col <- "N" %in% names(df_display)

  df_display[] <- Map(
    function(col, name) {
      if (is.numeric(col)) {
        formatted <- if (has_n_col && name == "N") {
          # N column: always integers
          sprintf("%.0f", col)
        } else if (any(is_n_row)) {
          # N row: integers for that row, decimals elsewhere
          ifelse(
            is_n_row,
            sprintf("%.0f", col),
            sprintf(paste0("%.", digits, "f"), col)
          )
        } else {
          sprintf(paste0("%.", digits, "f"), col)
        }
        ifelse(is.na(col), NA, formatted)
      } else {
        col
      }
    },
    df_display,
    names(df_display)
  )

  spicy_print_table(
    df_display,
    padding = "normal",
    first_column_line = TRUE,
    row_total_line = TRUE,
    column_total_line = TRUE,
    bottom_line = FALSE,
    ...
  )

  invisible(x)
}

# Ensure global spicy options don't trigger unintended behavior
old_opts <- options(spicy.rescale = FALSE, spicy.simulate_p = FALSE)
on.exit(options(old_opts)) # restore previous user options after tests

test_that("cross_tab basic two-way table works", {
  data <- mtcars
  res <- cross_tab(data, cyl, gear, styled = FALSE)

  expect_s3_class(res, "data.frame")
  expect_true("Values" %in% names(res))

  if ("Total" %in% names(res)) {
    expect_equal(sum(res$Total, na.rm = TRUE), nrow(data))
  } else {
    expect_equal(attr(res, "n_total"), nrow(data))
  }
})


test_that("cross_tab supports grouping with by", {
  data <- mtcars
  res <- cross_tab(data, cyl, gear, by = am, styled = FALSE)

  expect_type(res, "list")
  expect_length(res, length(unique(data$am)))
  expect_true(all(vapply(res, inherits, logical(1), "data.frame")))
})

test_that("cross_tab supports interaction() in by", {
  data <- mtcars
  res <- cross_tab(data, cyl, gear, by = interaction(vs, am), styled = FALSE)

  expect_type(res, "list")
  expect_length(res, length(unique(interaction(data$vs, data$am))))
})

test_that("cross_tab handles weights and rescale properly", {
  data <- mtcars

  # Without rescale: sum(weights) ≠ N
  res1 <- cross_tab(
    data,
    cyl,
    gear,
    weights = mpg,
    rescale = FALSE,
    styled = FALSE
  )
  total1 <- attr(res1, "n_total")

  # With rescale: sum(weights) == N
  res2 <- cross_tab(
    data,
    cyl,
    gear,
    weights = mpg,
    rescale = TRUE,
    styled = FALSE
  )
  total2 <- attr(res2, "n_total")

  expect_false(isTRUE(all.equal(total1, nrow(data))))
  expect_true(isTRUE(all.equal(round(total2), nrow(data))))
})

test_that("cross_tab automatically ignores NA values", {
  df <- data.frame(
    x = c("A", "B", NA, "A", "B", NA),
    y = c("Yes", "No", "Yes", "No", "Yes", NA)
  )

  # xtabs() ignores missing values automatically
  res <- cross_tab(df, x, y, styled = FALSE)

  complete_n <- sum(stats::complete.cases(df[, c("x", "y")]))
  total_tab <- attr(res, "n_total")

  expect_equal(total_tab, complete_n)
})


test_that("cross_tab respects global options spicy.simulate_p and spicy.rescale", {
  data <- mtcars

  # Backup current options
  old_opts <- options()

  options(spicy.simulate_p = TRUE, spicy.rescale = TRUE)
  res <- cross_tab(data, cyl, gear, weights = mpg, styled = FALSE)

  # Verify attributes and global option effect
  expect_true(grepl("Chi-2\\(", attr(res, "note")))
  expect_true(isTRUE(all.equal(round(attr(res, "n_total")), nrow(data))))

  # Restore options
  options(old_opts)
})

test_that("cross_tab returns spicy_cross_table or list when styled = TRUE", {
  data <- mtcars
  res1 <- cross_tab(data, cyl, gear)
  res2 <- cross_tab(data, cyl, gear, by = am)

  expect_s3_class(res1, "spicy_cross_table")
  expect_s3_class(res2, "spicy_cross_table_list")
})

test_that("cross_tab accepts labelled vectors in vector mode", {
  x <- haven::labelled(
    c(1, 2, 1, 2, 1, 2),
    labels = c(Non = 1, Oui = 2)
  )
  y <- factor(c("BFH", "BFH", "HESAV", "HESAV", "ZHAW", "ZHAW"))

  res <- cross_tab(x, y, percent = "c", styled = FALSE)

  expect_s3_class(res, "data.frame")
  expect_true("Values" %in% names(res))
  expect_match(attr(res, "title"), "x x y")
})

test_that("cross_tab keeps column names with $ vector calls", {
  d <- data.frame(
    pasemploiraison_1 = c("Non", "Oui", "Non", "Oui"),
    hes = c("BFH", "BFH", "HESAV", "HESAV")
  )

  res <- cross_tab(d$pasemploiraison_1, d$hes, percent = "c", styled = FALSE)

  expect_match(attr(res, "title"), "pasemploiraison_1 x hes", fixed = TRUE)
})

test_that("cross_tab validates weights length in data.frame and vector modes", {
  df <- data.frame(
    x = c("A", "B", "A", "B"),
    y = c("Yes", "No", "Yes", "No")
  )

  expect_error(
    cross_tab(df, x, y, weights = c(1, 2), styled = FALSE),
    "`weights` must have the same length as the number of rows.",
    fixed = TRUE
  )

  expect_error(
    cross_tab(df$x, df$y, weights = c(1, 2), styled = FALSE),
    "`weights` must have the same length as `x` and `y` in vector mode.",
    fixed = TRUE
  )
})

test_that("cross_tab rejects rescale when weight sum is zero", {
  df <- data.frame(
    x = c("A", "B"),
    y = c("Yes", "No"),
    w = c(0, 0)
  )

  expect_error(
    cross_tab(df, x, y, weights = w, rescale = TRUE, styled = FALSE),
    "`rescale = TRUE` requires a strictly positive sum of weights.",
    fixed = TRUE
  )
})

test_that("cross_tab fails early when y is explicitly NULL", {
  expect_error(
    cross_tab(mtcars, cyl, y = NULL, styled = FALSE),
    "You must specify a `y` variable",
    fixed = TRUE
  )
})

test_that("cross_tab computes by-group stats on non-empty margins", {
  df <- data.frame(
    g = c(rep("A", 6), rep("B", 8)),
    x = c("a", "a", "b", "b", "c", "c", "u", "u", "u", "u", "v", "v", "v", "v"),
    y = c("k", "l", "k", "l", "k", "l", "k", "k", "l", "l", "k", "k", "l", "l")
  )

  out <- cross_tab(df, x, y, by = g, correct = TRUE, styled = TRUE)
  note_b <- attr(out[["B"]], "note")

  expect_true(grepl(
    "Yates continuity correction applied.",
    note_b,
    fixed = TRUE
  ))
  expect_false(grepl("p = NA", note_b, fixed = TRUE))
})

# ── Association measure features ───────────────────────────────────────────

test_that("cross_tab stores numeric attributes", {
  res <- cross_tab(mtcars, cyl, gear, styled = FALSE)
  expect_true(is.numeric(attr(res, "chi2")))
  expect_true(is.numeric(attr(res, "df")))
  expect_true(is.numeric(attr(res, "p_value")))
  expect_equal(attr(res, "assoc_measure"), "Cramer's V")
  expect_true(is.numeric(attr(res, "assoc_value")))
  expect_true(length(attr(res, "assoc_result")) == 4)
})

test_that("cross_tab auto-detects ordinal variables", {
  mt <- mtcars
  mt$cyl <- ordered(mt$cyl)
  mt$gear <- ordered(mt$gear)
  res <- cross_tab(mt, cyl, gear, styled = FALSE)
  expect_equal(attr(res, "assoc_measure"), "Kendall's Tau-b")
})

test_that("cross_tab assoc_ci adds CI to note", {
  res <- cross_tab(mtcars, cyl, gear, assoc_ci = TRUE, styled = FALSE)
  expect_true(grepl("95% CI", attr(res, "note")))
})

test_that("cross_tab assoc_measure = 'none' omits coefficient line", {
  res <- cross_tab(mtcars, cyl, gear, assoc_measure = "none", styled = FALSE)
  expect_false(grepl("Cramer", attr(res, "note")))
  expect_true(grepl("Chi-2", attr(res, "note")))
})

test_that("cross_tab note uses new Chi-2(df) format", {
  res <- cross_tab(mtcars, cyl, gear, styled = FALSE)
  expect_true(grepl("Chi-2\\(\\d+\\) =", attr(res, "note")))
})

# ── Percentage modes ─────────────────────────────────────────────────────

test_that("cross_tab row percent shows row percentages", {
  res <- cross_tab(mtcars, cyl, gear, percent = "row", styled = FALSE)
  expect_true("Values" %in% names(res))
  expect_match(attr(res, "title"), "Row %")
})

test_that("cross_tab column percent shows column percentages", {
  res <- cross_tab(mtcars, cyl, gear, percent = "column", styled = FALSE)
  expect_match(attr(res, "title"), "Column %")
})

test_that("cross_tab styled row percent includes Total and N rows", {
  res <- cross_tab(mtcars, cyl, gear, percent = "row", styled = TRUE)
  expect_true("N" %in% names(res))
  vals <- res$Values
  expect_true("Total" %in% vals)
})

test_that("cross_tab styled column percent includes Total and N rows", {
  res <- cross_tab(mtcars, cyl, gear, percent = "column", styled = TRUE)
  vals <- res$Values
  expect_true("Total" %in% vals)
  expect_true("N" %in% vals)
})

test_that("cross_tab show_n = FALSE omits N row/column", {
  res <- cross_tab(
    mtcars,
    cyl,
    gear,
    percent = "row",
    show_n = FALSE,
    styled = TRUE
  )
  expect_false("N" %in% names(res))
  res2 <- cross_tab(
    mtcars,
    cyl,
    gear,
    percent = "column",
    show_n = FALSE,
    styled = TRUE
  )
  expect_false("N" %in% res2$Values)
})

# ── Association measures ────────────────────────────────────────────────

test_that("cross_tab assoc_measure = 'phi' works", {
  res <- cross_tab(mtcars, am, vs, assoc_measure = "phi", styled = FALSE)
  expect_equal(attr(res, "assoc_measure"), "Phi")
})

test_that("cross_tab assoc_measure = 'gamma' works", {
  mt <- mtcars
  mt$cyl <- ordered(mt$cyl)
  mt$gear <- ordered(mt$gear)
  res <- cross_tab(mt, cyl, gear, assoc_measure = "gamma", styled = FALSE)
  expect_equal(attr(res, "assoc_measure"), "Goodman-Kruskal Gamma")
})

test_that("cross_tab assoc_measure = 'tau_c' works", {
  mt <- mtcars
  mt$cyl <- ordered(mt$cyl)
  mt$gear <- ordered(mt$gear)
  res <- cross_tab(mt, cyl, gear, assoc_measure = "tau_c", styled = FALSE)
  expect_equal(attr(res, "assoc_measure"), "Kendall's Tau-c")
})

test_that("cross_tab assoc_measure = 'somers_d' works", {
  mt <- mtcars
  mt$cyl <- ordered(mt$cyl)
  mt$gear <- ordered(mt$gear)
  res <- cross_tab(mt, cyl, gear, assoc_measure = "somers_d", styled = FALSE)
  expect_equal(attr(res, "assoc_measure"), "Somers' D")
})

test_that("cross_tab assoc_measure = 'lambda' works", {
  res <- cross_tab(mtcars, cyl, gear, assoc_measure = "lambda", styled = FALSE)
  expect_equal(attr(res, "assoc_measure"), "Lambda")
})

# ── Weight note ──────────────────────────────────────────────────────────

test_that("cross_tab adds weight note", {
  res <- cross_tab(mtcars, cyl, gear, weights = mpg, styled = FALSE)
  expect_match(attr(res, "note"), "Weight: mpg")
})

test_that("cross_tab adds rescaled note", {
  res <- cross_tab(
    mtcars,
    cyl,
    gear,
    weights = mpg,
    rescale = TRUE,
    styled = FALSE
  )
  expect_match(attr(res, "note"), "rescaled")
})

# ── Small expected cells warning ─────────────────────────────────────────

test_that("cross_tab warns about small expected cells", {
  df <- data.frame(
    x = c("A", "A", "A", "B", "B", "C"),
    y = c("X", "X", "Y", "X", "Y", "Y")
  )
  res <- cross_tab(df, x, y, styled = FALSE)
  expect_match(attr(res, "note"), "expected cell")
})

# ── Simulate p ───────────────────────────────────────────────────────────

test_that("cross_tab simulate_p adds (simulated) to note", {
  res <- cross_tab(mtcars, cyl, gear, simulate_p = TRUE, styled = FALSE)
  expect_match(attr(res, "note"), "simulated")
})

# ── include_stats = FALSE ────────────────────────────────────────────────

test_that("cross_tab include_stats = FALSE omits note", {
  res <- cross_tab(mtcars, cyl, gear, include_stats = FALSE, styled = FALSE)
  expect_null(attr(res, "note"))
})

# ── Validation ───────────────────────────────────────────────────────────

test_that("cross_tab errors when data is missing", {
  expect_error(cross_tab(), "must provide a dataset")
})

test_that("cross_tab errors when x is missing for data.frame", {
  expect_error(cross_tab(mtcars), "must specify at least one variable")
})

test_that("cross_tab errors with non-numeric weights", {
  expect_error(
    cross_tab(mtcars, cyl, gear, weights = "a"),
    "`weights` must be numeric"
  )
})

test_that("cross_tab errors with negative weights", {
  df <- data.frame(x = c("A", "B"), y = c("C", "D"), w = c(-1, 1))
  expect_error(cross_tab(df, x, y, weights = w), "non-negative")
})

test_that("cross_tab errors with infinite weights", {
  df <- data.frame(x = c("A", "B"), y = c("C", "D"), w = c(Inf, 1))
  expect_error(cross_tab(df, x, y, weights = w), "finite")
})

# ── Print methods ────────────────────────────────────────────────────────

test_that("print.spicy_cross_table produces output", {
  res <- cross_tab(mtcars, cyl, gear)
  expect_output(print(res))
})

test_that("print.spicy_cross_table_list produces output", {
  res <- cross_tab(mtcars, cyl, gear, by = am)
  expect_output(print(res))
})

# ── Vector input paths ──────────────────────────────────────────────────

test_that("cross_tab vector input with by works", {
  x <- c("A", "B", "A", "B")
  y <- c("X", "Y", "X", "Y")
  by <- c("G1", "G1", "G2", "G2")
  res <- cross_tab(x, y, by = by, styled = FALSE)
  expect_type(res, "list")
  expect_length(res, 2)
})

test_that("cross_tab vector input by length mismatch errors", {
  expect_error(
    cross_tab(c("A", "B"), c("X", "Y"), by = c("G1"), styled = FALSE),
    "same length"
  )
})

test_that("cross_tab vector input x/y length mismatch errors", {
  expect_error(
    cross_tab(c("A", "B", "C"), c("X", "Y"), styled = FALSE),
    "same length"
  )
})

test_that("cross_tab rescale without weights warns", {
  expect_warning(
    cross_tab(mtcars, cyl, gear, rescale = TRUE, styled = FALSE),
    "no effect"
  )
})

test_that("cross_tab 1x1 table omits chi-2 note", {
  df <- data.frame(x = c("A", "A"), y = c("X", "X"))
  res <- cross_tab(df, x, y, styled = FALSE)
  expect_null(attr(res, "note"))
})

test_that("cross_tab percent = 'none' shows N in title", {
  res <- cross_tab(mtcars, cyl, gear, percent = "none", styled = FALSE)
  expect_match(attr(res, "title"), "(N)", fixed = TRUE)
})

test_that("cross_tab styled percent = 'none' with digits", {
  res <- cross_tab(mtcars, cyl, gear, percent = "none", styled = TRUE)
  expect_output(print(res))
})

test_that("print.spicy_cross_table formats N row in column percent", {
  res <- cross_tab(mtcars, cyl, gear, percent = "column", styled = TRUE)
  out <- capture.output(print(res))
  expect_true(any(grepl("N", out)))
})

test_that("print.spicy_cross_table formats N column in row percent", {
  res <- cross_tab(mtcars, cyl, gear, percent = "row", styled = TRUE)
  out <- capture.output(print(res))
  expect_true(any(grepl("N", out)))
})

test_that("cross_tab vector mode errors when only one vector given", {
  expect_error(
    cross_tab(c("A", "B")),
    "must provide both"
  )
})

test_that("cross_tab vector mode with interaction by", {
  x <- c("A", "B", "A", "B", "A", "B")
  y <- c("X", "Y", "X", "Y", "X", "Y")
  g1 <- c("M", "M", "F", "F", "M", "F")
  g2 <- c("Y", "Y", "Y", "O", "O", "O")
  res <- cross_tab(x, y, by = interaction(g1, g2), styled = FALSE)
  expect_type(res, "list")
})

test_that("cross_tab with [[ extraction preserves var name", {
  d <- data.frame(a = c("X", "Y", "X"), b = c("M", "F", "M"))
  res <- cross_tab(d[["a"]], d[["b"]], styled = FALSE)
  expect_s3_class(res, "data.frame")
})

test_that("cross_tab vector mode rejects non-numeric weights", {
  x <- c("A", "B", "A")
  y <- c("X", "Y", "X")
  expect_error(cross_tab(x, y, weights = c("a", "b", "c")), "numeric")
})

test_that("cross_tab vector mode rejects mismatched by length", {
  x <- c("A", "B", "A")
  y <- c("X", "Y", "X")
  expect_error(cross_tab(x, y, by = c("G1", "G2")), "same length")
})

test_that("cross_tab with $ accessor extracts var name", {
  d <- data.frame(aa = c("X", "Y", "X"), bb = c("M", "F", "M"))
  res <- cross_tab(d$aa, d$bb, styled = FALSE)
  expect_s3_class(res, "data.frame")
})

test_that("cross_tab skips stats when single non-empty column", {
  d <- data.frame(
    x = c("A", "A", "B", "B"),
    y = c("X", "X", "X", "X")
  )
  out <- capture.output(cross_tab(d, x, y))
  expect_false(any(grepl("Chi-2", out)))
})

test_that("cross_tab weighted without stats shows weight note alone", {
  d <- data.frame(
    x = c("A", "B", "A", "B"),
    y = c("X", "Y", "X", "Y"),
    w = c(2, 3, 1, 4)
  )
  out <- capture.output(cross_tab(d, x, y, weights = w, include_stats = FALSE))
  expect_true(any(grepl("Weight:", out)))
})

test_that("cross_tab percent column styled output", {
  out <- capture.output(cross_tab(mtcars, cyl, gear, percent = "column"))
  expect_true(any(grepl("%", out)))
})

test_that("cross_tab percent row styled output", {
  out <- capture.output(cross_tab(mtcars, cyl, gear, percent = "row"))
  expect_true(any(grepl("%", out)))
})

test_that("cross_tab invalid assoc_measure errors", {
  expect_error(cross_tab(mtcars, cyl, gear, assoc_measure = "invalid"))
})

test_that("cross_tab tryCatch fallback for complex x/y expressions", {
  d <- data.frame(a = c("X", "Y", "X"), b = c("M", "F", "M"))
  res <- cross_tab(d, a, b, styled = FALSE)
  expect_s3_class(res, "data.frame")
})

test_that("cross_tab print without Values column", {
  d <- data.frame(x = c("A", "B"), y = c("X", "Y"))
  res <- cross_tab(d, x, y, styled = FALSE)
  names(res)[names(res) == "Values"] <- "Category"
  class(res) <- c("spicy_cross_table", "spicy_table", "data.frame")
  attr(res, "title") <- "Test (N)"
  out <- capture.output(print(res))
  expect_true(length(out) > 0)
})

test_that("cross_tab print uses digits=1 for percent titles", {
  d <- data.frame(x = c("A", "B", "A"), y = c("X", "Y", "X"))
  res <- cross_tab(d, x, y, percent = "row", styled = FALSE)
  class(res) <- c("spicy_cross_table", "spicy_table", class(res))
  attr(res, "title") <- "Table (Row %)"
  attr(res, "digits") <- NULL
  out <- capture.output(print(res))
  expect_true(length(out) > 0)
})

test_that("cross_tab vector mode with [[ symbol index extracts name", {
  d <- data.frame(aa = c("X", "Y", "X"), bb = c("M", "F", "M"))
  col <- "aa"
  res <- cross_tab(d[[col]], d[["bb"]], styled = FALSE)
  expect_s3_class(res, "data.frame")
})

test_that("cross_tab vector mode with function-wrapped expression", {
  d <- data.frame(aa = c("x", "y", "x"), bb = c("M", "F", "M"))
  res <- cross_tab(toupper(d$aa), d$bb, styled = FALSE)
  expect_s3_class(res, "data.frame")
})

test_that("cross_tab handles all-NA vector in make_levels", {
  res <- cross_tab(c(NA, NA), c("A", "B"), styled = FALSE)
  expect_s3_class(res, "data.frame")
})

test_that("cross_tab DF mode with complex x/y expressions triggers tryCatch fallback", {
  d <- data.frame(
    a = c("X", "Y", "X", "Y"),
    b = c("M", "F", "M", "F"),
    c = c("P", "Q", "P", "Q")
  )
  res <- cross_tab(d, interaction(a, b), c, styled = FALSE)
  expect_s3_class(res, "data.frame")
  res2 <- cross_tab(d, c, interaction(a, b), styled = FALSE)
  expect_s3_class(res2, "data.frame")
})

test_that("cross_tab print uses digits=0 for count titles", {
  d <- data.frame(x = c("A", "B", "A"), y = c("X", "Y", "X"))
  res <- cross_tab(d, x, y, styled = FALSE)
  class(res) <- c("spicy_cross_table", "spicy_table", class(res))
  attr(res, "title") <- "Table (N)"
  attr(res, "digits") <- NULL
  out <- capture.output(print(res))
  expect_true(length(out) > 0)
})

#' Frequency Table
#'
#' @description
#' Creates a frequency table for a vector or variable from a data frame, with
#' options for weighting, sorting, handling *labelled* data, defining custom
#' missing values, and displaying cumulative percentages.
#'
#' When `styled = TRUE`, the function prints a spicy-formatted ASCII table
#' using [print.spicy_freq_table()] and [spicy_print_table()]; otherwise, it
#' returns a `data.frame` containing frequencies and proportions.
#'
#' @details
#' This function is designed to mimic common frequency procedures from
#' statistical software such as SPSS or Stata, while integrating the
#' flexibility of R's data structures.
#'
#' It automatically detects the type of input (`vector`, `factor`, or
#' `labelled`) and applies appropriate transformations, including:
#'
#' * Handling of labelled variables via **labelled** or **haven**
#' * Optional recoding of specific values as missing (`na_val`)
#' * Optional weighting with a rescaling mechanism
#' * Support for cumulative percentages (`cum = TRUE`)
#' * Multiple display modes for labels via `labelled_levels`
#' * Schema-vs-observed level display via `factor_levels`
#'
#' For factor and labelled inputs, the `factor_levels` argument
#' controls whether declared-but-unobserved levels appear in the
#' output. The default `"observed"` drops them (Stata `tab` behavior);
#' `"all"` keeps them with `n = 0`, matching SPSS `FREQUENCIES` and
#' [code_book()]'s default. For schema-level inspection without
#' computing frequencies, use [varlist()] or [code_book()] with
#' `factor_levels = "all"`.
#'
#' When weighting is applied (`weights`), the frequencies and percentages are
#' computed proportionally to the weights. The argument `rescale = TRUE`
#' normalizes weights so their sum equals the unweighted sample size
#' (`length(weights)`).
#'
#' Missing values in `weights` are treated as zero (with a warning), so
#' the corresponding rows contribute nothing to any cell. With
#' `rescale = TRUE`, the remaining weights are normalized so the total
#' weighted N still equals `length(weights)` — the implicit share of the
#' zeroed rows is redistributed over the others, mirroring Stata's
#' `pweight` semantics. With `rescale = FALSE`, the total weighted N is
#' the actual sum of non-`NA` weights.
#'
#' @param data A `data.frame`, vector, or factor. If a data frame is provided,
#'   specify the target variable `x`. If both `data` and `x` are supplied as
#'   vectors, `data` is ignored with a warning.
#' @param x A variable from `data` (unquoted).
#' @param weights Optional numeric vector of weights (same length as `x`).
#'   The variable may be referenced as a bare name when it belongs to `data`,
#'   or as a qualified expression like `other$w` (evaluated in the calling
#'   environment), which always takes precedence over `data` lookup. `NA`
#'   weights are treated as zero with a warning; see `Details`.
#' @param digits Number of decimal digits to display for percentages (default: `1`).
#' @param valid Logical. If `TRUE` (default), display valid percentages
#'   (excluding missing values).
#' @param cum Logical. If `FALSE` (the default), cumulative percentages are omitted.
#'   If `TRUE`, adds cumulative percentages.
#' @param sort Sorting method for values:
#'   * `""` - no sorting (default)
#'   * `"+"` - increasing frequency
#'   * `"-"` - decreasing frequency
#'   * `"name+"` - alphabetical A-Z
#'   * `"name-"` - alphabetical Z-A
#' @param na_val Atomic vector of numeric or character values to be treated as missing (`NA`).
#'
#' For *labelled* variables (from **haven** or **labelled**), this argument
#' must refer to the underlying coded values, not the visible labels.
#'
#' Example:
#' ```
#' x <- labelled(c(1, 2, 3, 1, 2, 3), c("Low" = 1, "Medium" = 2, "High" = 3))
#' freq(x, na_val = 1) # Treat all "Low" as missing
#' ```
#'
#' @param labelled_levels For `labelled` variables, defines how labels and
#'   values are displayed:
#'   * `"prefixed"` or `"p"` - show labels as `[value] label` (default)
#'   * `"labels"` or `"l"` - show only labels
#'   * `"values"` or `"v"` - show only numeric codes
#' @param factor_levels Character. Controls how factor and labelled values
#'   are displayed in the frequency table. `"observed"` (the default;
#'   matches Stata's `tab`) shows only levels present in the data.
#'   `"all"` (matches SPSS `FREQUENCIES` and [code_book()]'s default)
#'   keeps every declared level, including unused ones, which appear
#'   with `n = 0`.
#' @param rescale Logical. If `TRUE` (default), rescale weights so that their
#'   total equals the unweighted sample size (`length(weights)`). See
#'   `Details` for the interaction with `NA` weights.
#' @param styled Logical. If `TRUE` (default), print the formatted spicy table.
#'   If `FALSE`, return a plain `data.frame` with frequency values.
#' @param ... Additional arguments passed to [print.spicy_freq_table()].
#'
#' @return
#' With `styled = FALSE`, a plain `data.frame` with no extra attributes
#' and columns:
#' \itemize{
#'   \item \code{value} - unique values or factor levels
#'   \item \code{n} - frequency count (weighted if applicable)
#'   \item \code{prop} - proportion of total
#'   \item \code{valid_prop} - proportion of valid responses (if `valid = TRUE`)
#'   \item \code{cum_prop}, \code{cum_valid_prop} - cumulative percentages (if `cum = TRUE`)
#' }
#'
#' With `styled = TRUE` (default), prints the formatted table to the
#' console and invisibly returns a `spicy_freq_table` object: the same
#' `data.frame` carrying rendering metadata as attributes (`digits`,
#' `data_name`, `var_name`, `var_label`, `class_name`, `n_total`,
#' `n_valid`, `weighted`, `rescaled`, `weight_var`) used by
#' [print.spicy_freq_table()].
#'
#' @examples
#' # Frequency table with labelled ordered factor
#' freq(sochealth, education)
#' freq(sochealth, self_rated_health, sort = "-")
#'
#' library(labelled)
#'
#' # Simple numeric vector
#' x <- c(1, 2, 2, 3, 3, 3, NA)
#' freq(x)
#'
#' # Plain vector with a sentinel value recoded as missing
#' freq(c(1, 2, 3, 99, 99), na_val = 99)
#'
#' # Labelled variable (haven-style)
#' x_lbl <- labelled(
#'   c(1, 2, 3, 1, 2, 3, 1, 2, NA),
#'   labels = c("Low" = 1, "Medium" = 2, "High" = 3)
#' )
#' var_label(x_lbl) <- "Satisfaction level"
#'
#' # Treat value 1 ("Low") as missing
#' freq(x_lbl, na_val = 1)
#'
#' # Display only labels, add cumulative %
#' freq(x_lbl, labelled_levels = "labels", cum = TRUE)
#'
#' # Display values only, sorted descending
#' freq(x_lbl, labelled_levels = "values", sort = "-")
#'
#' # Show all declared factor levels, including unused ones (SPSS-style).
#' # The default "observed" mirrors Stata's `tab` and drops unused levels.
#' f <- factor(c("Yes", "No", "Yes"), levels = c("Yes", "No", "Maybe"))
#' freq(f, factor_levels = "all")
#'
#' # With weighting
#' df <- data.frame(
#'   sex = factor(c("Male", "Female", "Female", "Male", NA, "Female")),
#'   weight = c(12, 8, 10, 15, 7, 9)
#' )
#'
#' # Weighted frequencies (normalized)
#' freq(df, sex, weights = weight, rescale = TRUE)
#'
#' # Weighted frequencies (without rescaling)
#' freq(df, sex, weights = weight, rescale = FALSE)
#'
#' # Base R style, with weights and cumulative percentages
#' freq(df$sex, weights = df$weight, cum = TRUE)
#'
#' # Piped version (tidy syntax) and sort alphabetically descending ("name-")
#' df |> freq(sex, sort = "name-")
#'
#' # Non-styled return (for programmatic use)
#' f <- freq(df, sex, styled = FALSE)
#' head(f)
#'
#' @seealso
#' [print.spicy_freq_table()] for formatted printing.
#' [spicy_print_table()] for the underlying ASCII rendering engine.
#'
#' @importFrom labelled is.labelled to_factor var_label
#'
#' @export

freq <- function(
  data,
  x = NULL,
  weights = NULL,
  digits = 1,
  valid = TRUE,
  cum = FALSE,
  sort = "",
  na_val = NULL,
  labelled_levels = c("prefixed", "labels", "values"),
  factor_levels = c("observed", "all"),
  rescale = TRUE,
  styled = TRUE,
  ...
) {
  labelled_levels <- match.arg(labelled_levels)
  factor_levels <- match_varlist_factor_levels(factor_levels)

  if (
    !is.numeric(digits) ||
      length(digits) != 1L ||
      !is.finite(digits) ||
      digits < 0
  ) {
    stop("`digits` must be a single non-negative number.", call. = FALSE)
  }

  if (
    !is.character(sort) ||
      length(sort) != 1L ||
      is.na(sort) ||
      !sort %in% c("", "+", "-", "name+", "name-")
  ) {
    stop(
      "Invalid value for 'sort'. Use '+', '-', 'name+', or 'name-'.",
      call. = FALSE
    )
  }

  validate_varlist_logical(valid, "valid")
  validate_varlist_logical(cum, "cum")
  validate_varlist_logical(rescale, "rescale")
  validate_varlist_logical(styled, "styled")

  is_df <- is.data.frame(data)
  if (is_df && missing(x)) {
    stop(
      "When `data` is a data frame, you must supply `x` (e.g., freq(data, x)).",
      call. = FALSE
    )
  }

  if (is_df && !missing(x)) {
    var_name <- deparse(substitute(x))
    data_name <- deparse(substitute(data))
    x <- dplyr::pull(data, {{ x }})
  } else if (!is_df && missing(x)) {
    var_name <- deparse(substitute(data))
    data_name <- var_name
    x <- data
  } else {
    warning(
      "Both `data` and `x` are vectors; `data` is ignored.",
      call. = FALSE
    )
    # `x` is what gets analyzed here — mirror the `!is_df && missing(x)`
    # branch above so the printed footer (`Data: ...`) does not surface
    # the name of the vector that was just declared "ignored".
    var_name <- deparse(substitute(x))
    data_name <- var_name
  }

  x_original <- x

  weight_name <- NULL

  if (!missing(weights)) {
    weight_expr <- substitute(weights)

    # Any `weights` expression that evaluates to NULL is treated as
    # "no weighting": literal `weights = NULL`, parameterized patterns
    # like `weights = if (use_w) w else NULL`, or a variable holding
    # NULL. Only an expression that *fails to resolve* (e.g., the
    # typo `weights = nonexistent_var`) is rejected — caught via
    # the sentinel below to distinguish it from a legitimate NULL.
    if (!is.null(weight_expr)) {
      weight_name <- deparse(weight_expr, backtick = FALSE)

      # Evaluate via `eval_tidy` with `data` as a data mask: bare
      # column names resolve from `data` first; qualified expressions
      # like `df2$w` skip the mask and resolve in the caller's
      # environment, preserving the precedence tested by "weights from
      # a qualified expression win over data lookup". Compound
      # expressions (`if (cond) col else NULL`) also see column names
      # through the mask, which the earlier bare-name shortcut could
      # not handle.
      not_found <- new.env(parent = emptyenv())
      weights <- tryCatch(
        rlang::eval_tidy(
          rlang::new_quosure(weight_expr, env = parent.frame()),
          data = if (is_df) data else NULL
        ),
        error = function(e) not_found
      )

      if (identical(weights, not_found)) {
        stop(
          paste0(
            "The weighting variable '",
            weight_name,
            "' was not found either in the data frame or in the global environment."
          ),
          call. = FALSE
        )
      }

      # Resolved to NULL — drop the name so the printed footer does
      # not claim a weighting that was never applied.
      if (is.null(weights)) {
        weight_name <- NULL
      }
    }
  }

  if (!is.null(weights)) {
    # Type guard up front: without it, a character weight vector
    # passes the comparisons via lexicographic coercion and only
    # crashes later at the `is.finite` check, with a misleading
    # "finite numeric" message. Logical is accepted because
    # TRUE/FALSE coerce naturally to 1/0 — a common shorthand for
    # "include / exclude" weighting.
    if (!is.numeric(weights) && !is.logical(weights)) {
      stop("`weights` must be a numeric or logical vector.", call. = FALSE)
    }
    if (length(weights) != length(x)) {
      stop("`weights` must have the same length as `x`.", call. = FALSE)
    }
    if (any(weights < 0, na.rm = TRUE)) {
      stop("`weights` must be non-negative.", call. = FALSE)
    }
    if (any(!is.finite(weights[!is.na(weights)]))) {
      stop("`weights` must contain only finite numeric values.", call. = FALSE)
    }
    if (any(is.na(weights))) {
      warning("NA values in `weights` are treated as zero.", call. = FALSE)
      weights[is.na(weights)] <- 0
    }

    if (rescale) {
      w_sum <- sum(weights, na.rm = TRUE)
      if (!is.finite(w_sum) || w_sum <= 0) {
        stop(
          "`rescale = TRUE` requires a strictly positive sum of weights.",
          call. = FALSE
        )
      }
      weights <- weights * length(weights) / w_sum
    }
  }

  if (labelled::is.labelled(x)) {
    if (!is.null(na_val) && !is.numeric(na_val)) {
      warning(
        "For labelled variables, 'na_val' should match the underlying numeric value (e.g., 1), not the label.",
        call. = FALSE
      )
    }

    if (!is.null(na_val)) {
      x_values <- unclass(x)
      x[x_values %in% na_val] <- NA
    }

    x <- labelled::to_factor(x, levels = labelled_levels, nolabel_to_na = FALSE)
  } else {
    if (!is.null(na_val)) x[x %in% na_val] <- NA
  }

  if (is.factor(x)) {
    # Keep all declared levels with `factor_levels = "all"` so the
    # output table includes unused levels with n = 0; otherwise drop
    # them, matching Stata `tab` and the SPSS `FREQUENCIES` default
    # behavior controlled by this argument.
    if (factor_levels == "observed") {
      x <- droplevels(x)
    }
  } else {
    x <- factor(x)
  }

  n_total <- if (is.null(weights)) length(x) else sum(weights)
  n_missing <- if (is.null(weights)) sum(is.na(x)) else sum(weights[is.na(x)])
  n_valid <- n_total - n_missing

  if (n_total == 0) {
    stop("Total frequency is zero; cannot compute proportions.", call. = FALSE)
  }

  if (is.null(weights)) {
    tab <- table(x, useNA = "ifany")
  } else {
    f <- addNA(x, ifany = TRUE)
    tab <- tapply(weights, f, sum)
    # `tapply` returns NA for groups with no observations, which only
    # happens with `factor_levels = "all"` for declared-but-unobserved
    # levels. Coerce those to 0 so the frequency table shows them as
    # `n = 0` rather than `n = NA`.
    tab[is.na(tab)] <- 0
  }

  df <- data.frame(
    value = names(tab),
    n = as.numeric(tab),
    stringsAsFactors = FALSE
  )

  df$prop <- df$n / n_total

  if (valid && n_valid > 0) {
    df$valid_prop <- df$n / n_valid
    df$valid_prop[is.na(df$value)] <- NA
  } else {
    df$valid_prop <- NA
  }

  # --- Sort
  if (sort != "") {
    decreasing <- sort %in% c("-", "name-")
    sort_col <- if (sort %in% c("+", "-")) "n" else "value"
    df <- df[order(df[[sort_col]], decreasing = decreasing), ]
  }

  # Move missing-value rows to the end so cumulative columns match the
  # printed layout (valid rows, then missing rows). Without this, a
  # user-supplied `sort` can place the NA row between valid rows and make
  # the displayed cum_prop look non-monotonic.
  na_rows <- is.na(df$value)
  if (any(na_rows) && !all(na_rows)) {
    df <- rbind(
      df[!na_rows, , drop = FALSE],
      df[na_rows, , drop = FALSE]
    )
    rownames(df) <- NULL
  }

  if (cum) {
    df$cum_prop <- cumsum(df$prop)
    if (valid) {
      # cumsum over the non-NA subset only, leaving NAs in place at
      # the missing-value row(s). A plain `cumsum(df$valid_prop)`
      # would propagate the trailing NA forward and corrupt every
      # subsequent cumulative value.
      valid_idx <- !is.na(df$valid_prop)
      df$cum_valid_prop <- df$valid_prop
      df$cum_valid_prop[valid_idx] <- cumsum(df$valid_prop[valid_idx])
    } else {
      df$cum_valid_prop <- NA
    }
  }

  if (!styled) {
    # Return a genuinely plain data.frame: no spicy print-method attributes
    # clinging to it. Users who want the metadata can keep `styled = TRUE`
    # (default) and inspect the invisibly returned `spicy_freq_table`.
    return(df)
  }

  attr(df, "digits") <- digits
  attr(df, "data_name") <- data_name
  attr(df, "var_name") <- var_name
  attr(df, "var_label") <- attr(x_original, "label", exact = TRUE)
  attr(df, "class_name") <- paste(class(x_original), collapse = ", ")
  attr(df, "n_total") <- n_total
  attr(df, "n_valid") <- n_valid
  attr(df, "weighted") <- !is.null(weights)
  attr(df, "rescaled") <- rescale
  attr(df, "weight_var") <- weight_name

  class(df) <- c("spicy_freq_table", "spicy_table", class(df))

  print(df, ...)
  invisible(df)
}

test_that("freq() works with a simple numeric vector", {
  x <- c(1, 2, 2, 3, 3, 3, NA)
  df <- freq(x, styled = FALSE)

  expect_s3_class(df, "data.frame")
  expect_true(all(c("value", "n", "prop") %in% names(df)))
  expect_equal(sum(df$n, na.rm = TRUE), length(x))
  expect_equal(round(sum(df$prop, na.rm = TRUE), 1), 1)
})


test_that("freq() works with a data.frame column", {
  df <- data.frame(cat = c("A", "B", "A", "C", "A", "B", "B", "C", "C", "C"))
  res <- freq(df, cat, styled = FALSE)

  expect_s3_class(res, "data.frame")
  expect_equal(sum(res$n, na.rm = TRUE), nrow(df))
  expect_true(any(grepl("C", res$value)))
})

test_that("freq() requires x when data is a data.frame", {
  expect_error(
    freq(mtcars, styled = FALSE),
    "must supply `x`",
    fixed = TRUE
  )
})


test_that("freq() handles labelled variables correctly", {
  library(labelled)

  x <- labelled(
    c(1, 2, 3, 1, 2, 3, 1, 2, NA),
    labels = c("Low" = 1, "Medium" = 2, "High" = 3)
  )
  var_label(x) <- "Satisfaction level"

  # Prefixed (default)
  f1 <- freq(x, labelled_levels = "prefixed", styled = FALSE)
  expect_true(any(grepl("\\[1\\]", f1$value)))

  # Labels only
  f2 <- freq(x, labelled_levels = "labels", styled = FALSE)
  expect_true(all(!grepl("\\[", f2$value)))

  # Underlying values only
  f1 <- freq(x, styled = FALSE)
  f2 <- freq(x, na_val = 1, styled = FALSE)

  # After recoding, there should be one fewer distinct category
  expect_true(length(unique(f2$value)) <= length(unique(f1$value)))
})


test_that("freq() handles weights and rescaling", {
  df <- data.frame(
    sexe = factor(c("Male", "Female", "Female", "Male", NA, "Female")),
    poids = c(12, 8, 10, 15, 7, 9)
  )

  # Weighted, rescaled
  f_rescaled <- freq(df, sexe, weights = poids, rescale = TRUE, styled = FALSE)
  total_weighted <- sum(f_rescaled$n)
  expect_true(abs(total_weighted - nrow(df)) < 1e-6)

  # Weighted, not rescaled
  f_unscaled <- freq(df, sexe, weights = poids, rescale = FALSE, styled = FALSE)
  expect_true(sum(f_unscaled$n) > nrow(df))
})

test_that("freq() rejects rescale when sum of weights is zero", {
  df <- data.frame(x = c("A", "B"), w = c(0, 0))
  expect_error(
    freq(df, x, weights = w, rescale = TRUE, styled = FALSE),
    "strictly positive sum of weights",
    fixed = TRUE
  )
})


test_that("freq() handles missing value recoding", {
  x <- labelled(
    c(1, 2, 3, 1, 2, 3, 1),
    labels = c("Low" = 1, "Medium" = 2, "High" = 3)
  )

  f1 <- freq(x, styled = FALSE)
  f2 <- freq(x, na_val = 1, styled = FALSE)

  # Compare NA frequencies in the output table
  na_count_f1 <- f1$n[f1$value == "<NA>"]
  na_count_f2 <- f2$n[f2$value == "<NA>"]

  # Handle case where <NA> row is missing
  na_count_f1 <- if (length(na_count_f1)) na_count_f1 else 0
  na_count_f2 <- if (length(na_count_f2)) na_count_f2 else 0

  # Expect the 'Low' category to be removed after recoding
  expect_false(any(grepl("Low", f2$value)))

  # Optionally, confirm total frequency unchanged (only recoded)
  expect_equal(
    round(sum(f1$n, na.rm = TRUE), 5),
    round(sum(f2$n, na.rm = TRUE), 5)
  )
})

test_that("freq() correctly sorts by frequency and name", {
  x <- c("Banana", "Apple", "Cherry", "Banana", "Apple", "Cherry", "Apple")

  f_plus <- freq(x, sort = "+", styled = FALSE)
  f_minus <- freq(x, sort = "-", styled = FALSE)
  f_name_plus <- freq(x, sort = "name+", styled = FALSE)
  f_name_minus <- freq(x, sort = "name-", styled = FALSE)

  # Frequency ascending/descending
  expect_true(f_plus$n[1] <= f_plus$n[length(f_plus$n)])
  expect_true(f_minus$n[1] >= f_minus$n[length(f_minus$n)])

  # Alphabetical order
  expect_equal(sort(f_name_plus$value), f_name_plus$value)
  expect_equal(sort(f_name_minus$value, decreasing = TRUE), f_name_minus$value)
})


test_that("freq() handles multiple data types correctly", {
  df <- data.frame(
    logical_col = c(TRUE, FALSE, TRUE, NA),
    date_col = as.Date(c("2023-01-01", "2023-01-02", "2023-01-01", NA)),
    posix_col = as.POSIXct(c(
      "2023-01-01 12:00",
      "2023-01-02 12:00",
      NA,
      "2023-01-02 12:00"
    )),
    char_col = c("a", "a", "b", NA),
    num_col = c(1, 1, 2, NA)
  )

  expect_s3_class(freq(df, logical_col, styled = FALSE), "data.frame")
  expect_s3_class(freq(df, date_col, styled = FALSE), "data.frame")
  expect_s3_class(freq(df, posix_col, styled = FALSE), "data.frame")
  expect_s3_class(freq(df, char_col, styled = FALSE), "data.frame")
  expect_s3_class(freq(df, num_col, styled = FALSE), "data.frame")
})


test_that("freq() handles invalid weight and sort arguments", {
  x <- c(1, 2, 3)
  expect_error(freq(x, weights = c(-1, 0, 1)), "must be non-negative")
  expect_error(freq(x, weights = c(1, 2)), "same length")
  expect_error(freq(x, sort = "wrong"), "Invalid value for 'sort'")
})


test_that("freq() prints styled table invisibly", {
  x <- c("A", "B", "B", "C")
  expect_invisible(freq(x, styled = TRUE))
})

test_that("freq() cum = TRUE adds cumulative columns", {
  x <- c("A", "B", "B", "C")
  res <- freq(x, cum = TRUE, styled = FALSE)
  expect_true("cum_prop" %in% names(res))
  cum_vals <- res$cum_prop[!is.na(res$cum_prop)]
  expect_equal(cum_vals[length(cum_vals)], 1)
})

test_that("freq() valid = FALSE keeps valid_prop as NA", {
  x <- c(1, 2, 2, NA)
  res <- freq(x, valid = FALSE, styled = FALSE)
  expect_true("valid_prop" %in% names(res))
  expect_true(all(is.na(res$valid_prop)))
})

test_that("freq() valid = TRUE includes valid_prop", {
  x <- c(1, 2, 2, NA)
  res <- freq(x, valid = TRUE, styled = FALSE)
  expect_true("valid_prop" %in% names(res))
})

test_that("freq() labelled_levels = 'values' shows raw values", {
  skip_if_not_installed("labelled")
  x <- labelled::labelled(c(1, 2, 3), labels = c(A = 1, B = 2, C = 3))
  res <- freq(x, labelled_levels = "values", styled = FALSE)
  expect_true(any(res$value %in% c("1", "2", "3")))
})

test_that("freq() with factor input works directly", {
  f <- factor(c("x", "y", "x", "z"))
  res <- freq(f, styled = FALSE)
  expect_s3_class(res, "data.frame")
  expect_equal(sum(res$n, na.rm = TRUE), 4)
})

test_that("freq() works with a tibble input", {
  tib <- tibble::tibble(
    x = c("A", "B", "B", "C", "A", "B"),
    w = c(1, 2, 3, 4, 5, 6)
  )

  # Bare-name column extraction
  res_unweighted <- freq(tib, x, styled = FALSE)
  expect_s3_class(res_unweighted, "data.frame")
  expect_equal(sum(res_unweighted$n), 6L)
  expect_setequal(res_unweighted$value, c("A", "B", "C"))

  # Bare-name weight resolves through the data mask (tidy-eval)
  res_weighted <- freq(tib, x, weights = w, rescale = FALSE, styled = FALSE)
  expect_equal(sum(res_weighted$n), sum(tib$w))
})

test_that("freq() works with a tibble containing a labelled column", {
  skip_if_not_installed("labelled")

  tib <- tibble::tibble(
    sat = labelled::labelled(
      c(1, 2, 3, 1, 2),
      labels = c(Low = 1, Mid = 2, High = 3)
    )
  )

  res <- freq(tib, sat, styled = FALSE)
  expect_s3_class(res, "data.frame")
  expect_equal(sum(res$n), 5L)
  # Default labelled_levels = "prefixed"
  expect_true(any(grepl("\\[1\\] Low", res$value)))
})

test_that("freq() preserves footer metadata when input is a tibble", {
  tib <- tibble::tibble(
    x = c("A", "B", "B"),
    w = c(2, 3, 5)
  )
  capture.output(res <- freq(tib, x, weights = w))

  expect_equal(attr(res, "var_name"), "x")
  expect_equal(attr(res, "data_name"), "tib")
  expect_true(isTRUE(attr(res, "weighted")))
  expect_equal(attr(res, "weight_var"), "w")
})

test_that("freq() cum + weighted works", {
  df <- data.frame(x = c("A", "B", "C"), w = c(2, 3, 5))
  res <- freq(df, x, weights = w, cum = TRUE, styled = FALSE)
  expect_true("cum_prop" %in% names(res))
})

test_that("freq() na_val with plain vector", {
  x <- c(1, 2, 3, 99, 99)
  res <- freq(x, na_val = 99, styled = FALSE)
  n_na <- res$n[is.na(res$value)]
  expect_equal(n_na, 2)
})

test_that("freq() errors with non-finite weights", {
  expect_error(freq(c(1, 2), weights = c(1, Inf)), "finite")
})

test_that("freq() rejects non-numeric weights with a clear type error", {
  # Without the upfront type guard, character weights pass the
  # `< 0` comparison via lexicographic coercion and only crash at
  # `is.finite` with a misleading "finite numeric values" message.
  expect_error(
    freq(c(1, 2, 3), weights = c("a", "b", "c")),
    "must be a numeric or logical vector",
    fixed = TRUE
  )
  expect_error(
    freq(c(1, 2, 3), weights = factor(c("a", "b", "c"))),
    "must be a numeric or logical vector",
    fixed = TRUE
  )
  expect_error(
    freq(c(1, 2, 3), weights = list(1, 2, 3)),
    "must be a numeric or logical vector",
    fixed = TRUE
  )
})

test_that("freq() accepts logical weights via implicit coercion", {
  # TRUE = 1, FALSE = 0 — common shorthand for "include / exclude"
  # weighting in survey contexts. Locks in the supported type set.
  res <- freq(
    c("A", "B", "C"),
    weights = c(TRUE, FALSE, TRUE),
    rescale = FALSE,
    styled = FALSE
  )
  expect_equal(sum(res$n), 2)
  expect_equal(res$n[res$value == "B"], 0)
})

test_that("freq() styled output has class spicy_freq_table", {
  res <- freq(c("A", "B", "A"))
  expect_s3_class(res, "spicy_freq_table")
})

test_that("freq() cum + valid shows cumulative valid column", {
  x <- c("A", "B", "B", NA)
  res <- freq(x, cum = TRUE, valid = TRUE, styled = FALSE)
  expect_true("cum_valid_prop" %in% names(res))
})

test_that("freq() styled weighted output prints invisibly", {
  df <- data.frame(x = c("A", "B", "C"), w = c(2, 3, 5))
  expect_invisible(freq(df, x, weights = w, styled = TRUE))
})

test_that("freq() labelled with non-numeric na_val warns", {
  skip_if_not_installed("labelled")
  x <- labelled::labelled(c(1, 2, 3), labels = c(A = 1, B = 2, C = 3))
  expect_warning(
    freq(x, na_val = "A", styled = FALSE),
    "underlying numeric value"
  )
})

test_that("freq() styled cum output prints invisibly", {
  expect_invisible(freq(c("A", "B", "B"), cum = TRUE, styled = TRUE))
})

test_that("freq() errors when weight variable not found", {
  df <- data.frame(x = c("A", "B", "C"))
  expect_error(
    freq(df, x, weights = nonexistent_var, styled = FALSE),
    "not found"
  )
})

test_that("freq() accepts literal `weights = NULL` as no weighting", {
  df <- data.frame(x = c("A", "B", "C"))

  expect_silent({
    res <- freq(df, x, weights = NULL, styled = FALSE)
  })
  expect_equal(sum(res$n), 3L)
})

test_that("freq() supports the `weights = if (cond) wts else NULL` pattern", {
  df <- data.frame(x = c("A", "B", "C"))
  my_w <- c(2, 3, 5)

  use_w <- FALSE
  res_off <- freq(
    df,
    x,
    weights = if (use_w) my_w else NULL,
    rescale = FALSE,
    styled = FALSE
  )
  expect_equal(sum(res_off$n), 3L)

  use_w <- TRUE
  res_on <- freq(
    df,
    x,
    weights = if (use_w) my_w else NULL,
    rescale = FALSE,
    styled = FALSE
  )
  expect_equal(sum(res_on$n), 10L)
})

test_that("freq() resolves column refs inside compound weight expressions", {
  # Tidy-eval makes `data` available as a mask, so column references
  # nested inside an `if/else` (or any other expression) work.
  df <- data.frame(x = c("A", "B", "C"), w = c(2, 3, 5))

  use_w <- TRUE
  res_on <- freq(
    df,
    x,
    weights = if (use_w) w else NULL,
    rescale = FALSE,
    styled = FALSE
  )
  expect_equal(sum(res_on$n), 10L)

  use_w <- FALSE
  res_off <- freq(
    df,
    x,
    weights = if (use_w) w else NULL,
    rescale = FALSE,
    styled = FALSE
  )
  expect_equal(sum(res_off$n), 3L)
})

test_that("freq() treats a variable holding NULL as no weighting", {
  df <- data.frame(x = c("A", "B", "C"))
  my_w <- NULL

  expect_silent({
    res <- freq(df, x, weights = my_w, styled = FALSE)
  })
  expect_equal(sum(res$n), 3L)
})

test_that("freq() with `weights = NULL` carries no weighting metadata", {
  df <- data.frame(x = c("A", "B", "C"))
  capture.output(res <- freq(df, x, weights = NULL))

  expect_false(isTRUE(attr(res, "weighted")))
  expect_null(attr(res, "weight_var"))
})

test_that("freq() cum + valid styled prints invisibly", {
  x <- c("A", "B", NA, "A")
  expect_invisible(freq(x, cum = TRUE, valid = TRUE, styled = TRUE))
})

test_that("freq() warns when data is vector and x is given", {
  expect_warning(
    res <- freq(c(1, 2, 3), x = c(4, 5, 6), styled = FALSE),
    "ignored"
  )
  expect_s3_class(res, "data.frame")
})

test_that("freq() aligns data_name on x when both data and x are vectors", {
  capture.output(
    res <- suppressWarnings(freq(c(1, 2, 3), x = c(4, 5, 6)))
  )

  expect_equal(attr(res, "var_name"), "c(4, 5, 6)")
  expect_equal(attr(res, "data_name"), attr(res, "var_name"))
})

test_that("freq() footer does not reveal the ignored `data` vector name", {
  out <- capture.output(
    suppressWarnings(freq(c(1, 2, 3), x = c(4, 5, 6)))
  )

  # The analyzed vector (x) appears in the title and the footer;
  # the ignored `data` vector must not appear anywhere.
  expect_false(any(grepl("c\\(1, 2, 3\\)", out)))
  expect_true(any(grepl("c\\(4, 5, 6\\)", out)))
})

test_that("freq() errors with invalid digits", {
  expect_error(freq(c(1, 2), digits = -1), "non-negative")
  expect_error(freq(c(1, 2), digits = "a"), "non-negative")
})

test_that("freq() rejects non-finite digits with the same friendly message", {
  # is.finite() catches NA, NaN, Inf and -Inf in one check. Without
  # this guard, `digits = NA` would crash inside the validation `if`
  # with "missing value where TRUE/FALSE needed", and `digits = Inf`
  # would crash later in `format(..., nsmall = Inf)`.
  for (bad in list(NA, NA_real_, NA_integer_, NaN, Inf, -Inf)) {
    expect_error(
      freq(c(1, 2), digits = bad),
      "non-negative number",
      fixed = TRUE,
      info = paste("digits =", deparse(bad))
    )
  }
})

test_that("freq() rejects pathological sort values with the friendly message", {
  # Without `is.character` / `length == 1L` / `!is.na` guards, `sort = NULL`
  # crashed with "argument is of length zero" (logical(0) inside `if`),
  # `sort = NA` crashed with "missing value where TRUE/FALSE needed",
  # and `sort = c("+", "-")` warned about length and silently took the
  # first element.
  for (bad in list(NULL, NA_character_, NA, c("+", "-"), 1L, "")) {
    if (identical(bad, "")) {
      next # "" is a valid sort value, skip
    }
    expect_error(
      freq(c(1, 2), sort = bad),
      "Invalid value for 'sort'",
      fixed = TRUE,
      info = paste("sort =", deparse(bad))
    )
  }
})

test_that("freq() validates logical arguments up front", {
  # Reuses validate_varlist_logical() — same error message format
  # as varlist() / code_book(), so users get a consistent diagnostic
  # across the package.
  expect_error(freq(c(1, 2), valid = "yes"), "`valid` must be TRUE or FALSE", fixed = TRUE)
  expect_error(freq(c(1, 2), valid = NA), "`valid` must be TRUE or FALSE", fixed = TRUE)
  expect_error(freq(c(1, 2), valid = c(TRUE, FALSE)), "`valid` must be TRUE or FALSE", fixed = TRUE)

  expect_error(freq(c(1, 2), cum = "yes"), "`cum` must be TRUE or FALSE", fixed = TRUE)
  expect_error(freq(c(1, 2), cum = NA), "`cum` must be TRUE or FALSE", fixed = TRUE)

  expect_error(freq(c(1, 2), rescale = "no"), "`rescale` must be TRUE or FALSE", fixed = TRUE)
  expect_error(freq(c(1, 2), rescale = NA), "`rescale` must be TRUE or FALSE", fixed = TRUE)

  expect_error(freq(c(1, 2), styled = "yes"), "`styled` must be TRUE or FALSE", fixed = TRUE)
  expect_error(freq(c(1, 2), styled = NA), "`styled` must be TRUE or FALSE", fixed = TRUE)
})

test_that("freq() validates sort early", {
  expect_error(freq(c(1, 2), sort = "bad"), "Invalid value for 'sort'")
})

test_that("freq() warns with NA weights", {
  df <- data.frame(x = c("A", "B", "C"), w = c(1, NA, 3))
  expect_warning(
    res <- freq(df, x, weights = w, styled = FALSE),
    "NA values in `weights`"
  )
})

test_that("freq() rescaled total stays at length(weights) when some are NA", {
  # Locks in the documented Stata-pweight behavior: NA weights become
  # zero (the row contributes nothing), but with rescale = TRUE the
  # remaining weights are normalized so the total weighted N still
  # equals length(weights). With rescale = FALSE the total reflects
  # the actual sum of non-NA weights.
  df <- data.frame(x = c("A", "B", "C", "D"), w = c(1, 2, NA, NA))

  res_rescaled <- suppressWarnings(
    freq(df, x, weights = w, rescale = TRUE, styled = FALSE)
  )
  res_unrescaled <- suppressWarnings(
    freq(df, x, weights = w, rescale = FALSE, styled = FALSE)
  )

  expect_equal(sum(res_rescaled$n), 4)
  expect_equal(sum(res_unrescaled$n), 3)

  # The NA-weighted rows contribute nothing in either mode.
  expect_equal(res_unrescaled$n[res_unrescaled$value == "C"], 0)
  expect_equal(res_unrescaled$n[res_unrescaled$value == "D"], 0)
  expect_equal(res_rescaled$n[res_rescaled$value == "C"], 0)
  expect_equal(res_rescaled$n[res_rescaled$value == "D"], 0)
})

test_that("freq() errors when total frequency is zero", {
  expect_error(
    freq(character(0), styled = FALSE),
    "Total frequency is zero"
  )
  expect_error(
    freq(c("A", "B"), weights = c(0, 0), rescale = FALSE, styled = FALSE),
    "Total frequency is zero"
  )
})

test_that("freq() NA representation is consistent with and without weights", {
  x <- c("A", "B", NA)
  res_plain <- freq(x, styled = FALSE)
  df <- data.frame(x = x, w = c(1, 2, 3))
  res_weighted <- freq(df, x, weights = w, styled = FALSE)

  # Both should use true NA, not "<NA>" string
  expect_true(any(is.na(res_plain$value)))
  expect_true(any(is.na(res_weighted$value)))
  expect_false(any(res_plain$value == "<NA>", na.rm = TRUE))
  expect_false(any(res_weighted$value == "<NA>", na.rm = TRUE))
})

test_that("freq() cum_valid_prop is NA for missing rows", {
  x <- c("A", "B", NA, "A")
  res <- freq(x, cum = TRUE, valid = TRUE, styled = FALSE)
  na_row <- res[is.na(res$value), ]
  expect_true(is.na(na_row$cum_valid_prop))
})

test_that("freq() styled = FALSE returns plain data.frame", {
  res <- freq(c("A", "B"), styled = FALSE)
  expect_equal(class(res), "data.frame")
  expect_false(inherits(res, "spicy_freq_table"))
})

test_that("freq() styled = FALSE strips spicy metadata attributes", {
  df <- data.frame(
    x = c("A", "B", "C"),
    w = c(1, 2, 3)
  )
  res <- freq(df, x, weights = w, cum = TRUE, styled = FALSE)
  spicy_attrs <- c(
    "digits",
    "data_name",
    "var_name",
    "var_label",
    "class_name",
    "n_total",
    "n_valid",
    "weighted",
    "rescaled",
    "weight_var"
  )
  for (a in spicy_attrs) {
    expect_null(attr(res, a), info = paste("attribute", a))
  }
  expect_setequal(
    names(attributes(res)),
    c("names", "row.names", "class")
  )
})

test_that("freq() styled = TRUE invisibly returns an object carrying metadata", {
  df <- data.frame(x = c("A", "B", "C"), w = c(1, 2, 3))
  res <- withVisible(freq(df, x, weights = w))
  expect_false(res$visible)
  expect_s3_class(res$value, "spicy_freq_table")
  expect_equal(attr(res$value, "digits"), 1)
  expect_true(isTRUE(attr(res$value, "weighted")))
  expect_equal(attr(res$value, "weight_var"), "w")
})

test_that("freq() weights from a qualified expression win over data lookup", {
  # Two data frames share a column name `w` but hold different weights.
  # freq(df1, x, weights = df2$w) must use df2$w, not df1$w.
  df1 <- data.frame(x = c("A", "B", "C"), w = c(1, 1, 1))
  df2 <- data.frame(w = c(10, 20, 30))

  res_qualified <- freq(
    df1,
    x,
    weights = df2$w,
    rescale = FALSE,
    styled = FALSE
  )
  res_bare <- freq(df1, x, weights = w, rescale = FALSE, styled = FALSE)

  expect_equal(sum(res_qualified$n), 60)
  expect_equal(sum(res_bare$n), 3)

  # Qualified expression also works with [[ ]]
  res_brackets <- freq(
    df1,
    x,
    weights = df2[["w"]],
    rescale = FALSE,
    styled = FALSE
  )
  expect_equal(sum(res_brackets$n), 60)
})

test_that("freq() cum_prop stays monotonic with sort and missing values", {
  # When sort pushes frequent categories to the top, the NA row must still
  # be placed at the end so the valid block's cum_prop rises monotonically
  # from 0 toward the total-valid share.
  x <- c("A", "A", "A", "B", "B", "C", NA, NA)
  res <- freq(x, sort = "-", cum = TRUE, styled = FALSE)

  na_pos <- which(is.na(res$value))
  expect_equal(na_pos, nrow(res))

  valid_cum <- res$cum_prop[seq_len(nrow(res) - 1L)]
  expect_true(all(diff(valid_cum) >= 0))
  expect_equal(res$cum_prop[nrow(res)], 1)

  valid_cv <- res$cum_valid_prop[seq_len(nrow(res) - 1L)]
  expect_true(all(diff(valid_cv) >= 0))
  expect_equal(valid_cv[length(valid_cv)], 1)
  expect_true(is.na(res$cum_valid_prop[nrow(res)]))
})

test_that("freq() drops unused factor levels from the output", {
  # Documented Stata-style behavior: declared-but-unobserved levels
  # are not shown. Schema-level inspection lives in varlist() /
  # code_book(factor_levels = "all").
  x <- factor(c("a", "b", "a"), levels = c("a", "b", "c"))
  res <- freq(x, styled = FALSE)

  expect_equal(nrow(res), 2L)
  expect_setequal(res$value, c("a", "b"))
  expect_false("c" %in% res$value)
})

test_that("freq() factor_levels = 'all' keeps unused factor levels with n = 0", {
  x <- factor(c("a", "b", "a"), levels = c("a", "b", "c"))
  res <- freq(x, factor_levels = "all", styled = FALSE)

  expect_equal(nrow(res), 3L)
  expect_setequal(res$value, c("a", "b", "c"))
  expect_equal(res$n[res$value == "c"], 0)
  expect_equal(res$prop[res$value == "c"], 0)
  # Sum still equals length(x): the n = 0 row contributes nothing.
  expect_equal(sum(res$n), 3L)
})

test_that("freq() factor_levels = 'all' keeps unused labelled levels", {
  skip_if_not_installed("labelled")
  x <- labelled::labelled(
    c(1, 2, 1),
    labels = c(Low = 1, Mid = 2, High = 3)
  )
  res <- freq(x, factor_levels = "all", styled = FALSE)

  expect_equal(nrow(res), 3L)
  expect_true(any(grepl("High", res$value)))
  expect_equal(res$n[grepl("High", res$value)], 0)
})

test_that("freq() factor_levels = 'all' keeps weighted unused levels at n = 0", {
  # tapply() returns NA for empty groups; freq() must coerce those
  # to 0 so the table shows the unused level with `n = 0` rather
  # than a stray `n = NA` row.
  df <- data.frame(
    x = factor(c("a", "b", "a"), levels = c("a", "b", "c")),
    w = c(2, 3, 5)
  )
  res <- freq(
    df,
    x,
    weights = w,
    factor_levels = "all",
    rescale = FALSE,
    styled = FALSE
  )

  expect_equal(nrow(res), 3L)
  expect_equal(res$n[res$value == "c"], 0)
  expect_equal(sum(res$n), sum(df$w))
})

test_that("freq() factor_levels = 'all' interacts cleanly with sort and cum", {
  x <- factor(c("a", "b", "a"), levels = c("a", "b", "c"))

  # sort = "+" (freq ascending): unused level (n = 0) comes first
  res_asc <- freq(x, factor_levels = "all", sort = "+", styled = FALSE)
  expect_equal(res_asc$value[1], "c")

  # sort = "-" (freq descending): unused level lands at the end
  res_desc <- freq(x, factor_levels = "all", sort = "-", styled = FALSE)
  expect_equal(res_desc$value[nrow(res_desc)], "c")

  # cum_prop monotone, reaches 1, stays at 1 across the n = 0 row
  res_cum <- freq(x, factor_levels = "all", cum = TRUE, styled = FALSE)
  expect_true(all(diff(res_cum$cum_prop) >= 0))
  expect_equal(res_cum$cum_prop[nrow(res_cum)], 1)
})

test_that("freq() factor_levels = 'all' interacts with na_val", {
  # na_val recodes some values as NA, but the original levels stay
  # in the factor's `levels` attribute — so factor_levels = "all"
  # still shows them with n = 0 alongside the new NA row.
  skip_if_not_installed("labelled")
  x <- labelled::labelled(
    c(1, 2, 1),
    labels = c(Low = 1, Mid = 2, High = 3)
  )
  res <- freq(x, na_val = 1, factor_levels = "all", styled = FALSE)

  # Low (recoded to NA, n = 0), Mid (1), High (unused, n = 0), NA (2)
  expect_equal(nrow(res), 4L)
  expect_true(any(is.na(res$value)))
  expect_equal(sum(res$n[!is.na(res$value)]), 1)
  expect_equal(res$n[is.na(res$value)], 2)
})

test_that("freq() factor_levels = 'all' is a no-op for plain character vectors", {
  # Numeric / character / logical have no declared levels — the
  # argument has nothing to do, output identical to "observed".
  res_obs <- freq(c("a", "b", "a"), factor_levels = "observed", styled = FALSE)
  res_all <- freq(c("a", "b", "a"), factor_levels = "all", styled = FALSE)

  expect_equal(res_obs, res_all)
})

test_that("freq() validates factor_levels", {
  expect_error(
    freq(c(1, 2), factor_levels = "bogus"),
    'must be "observed" or "all"',
    fixed = TRUE
  )
  expect_error(
    freq(c(1, 2), factor_levels = NA_character_),
    'must be "observed" or "all"',
    fixed = TRUE
  )
  expect_error(
    freq(c(1, 2), factor_levels = c("observed", "bad")),
    'must be "observed" or "all"',
    fixed = TRUE
  )
})

test_that("freq() default factor_levels preserves current behavior", {
  # Regression guard: omitting the argument equals "observed".
  x <- factor(c("a", "b", "a"), levels = c("a", "b", "c"))
  expect_equal(
    freq(x, styled = FALSE),
    freq(x, factor_levels = "observed", styled = FALSE)
  )
})

test_that("freq() drops unused labelled values from the output", {
  skip_if_not_installed("labelled")

  x <- labelled::labelled(
    c(1, 2, 1),
    labels = c(Low = 1, Mid = 2, High = 3)
  )
  res <- freq(x, styled = FALSE)

  expect_equal(nrow(res), 2L)
  expect_false(any(grepl("High", res$value)))
})

test_that("freq() handles a single-level factor", {
  f <- factor(c("only", "only", "only"))
  res <- freq(f, styled = FALSE)
  expect_equal(nrow(res), 1L)
  expect_equal(res$n, 3)
  expect_equal(res$prop, 1)
})

test_that("freq() handles an all-NA input", {
  x <- c(NA, NA, NA)
  res <- freq(x, styled = FALSE)
  expect_true(all(is.na(res$value)))
  expect_equal(sum(res$n), 3)
  expect_equal(res$prop, 1)
  expect_true(all(is.na(res$valid_prop)))
})

test_that("freq() prints integer counts even with fractional weights", {
  # Matches SPSS/Stata/SAS convention: the Freq. column is always rendered
  # as integers, regardless of `digits` (which controls percentages only).
  df <- data.frame(x = c("A", "B", "C"), w = c(1.1, 2.2, 3.3))
  out1 <- capture.output(freq(df, x, weights = w, digits = 1, rescale = FALSE))
  out3 <- capture.output(freq(df, x, weights = w, digits = 3, rescale = FALSE))
  expect_false(any(grepl("1\\.1\\b", out1)))
  expect_false(any(grepl("1\\.100", out3)))
  expect_true(any(grepl("\\b1\\b", out1)))
  expect_true(any(grepl("\\b2\\b", out1)))
  expect_true(any(grepl("\\b3\\b", out1)))
})

test_that("freq() labelled_levels accepts p/l/v shortcuts via partial matching", {
  skip_if_not_installed("labelled")
  x <- labelled::labelled(c(1, 2, 3), labels = c(Low = 1, Mid = 2, High = 3))
  expect_true(any(grepl(
    "\\[1\\]",
    freq(x, labelled_levels = "p", styled = FALSE)$value
  )))
  expect_true(all(
    !grepl("\\[", freq(x, labelled_levels = "l", styled = FALSE)$value)
  ))
  expect_true(any(
    freq(x, labelled_levels = "v", styled = FALSE)$value %in% c("1", "2", "3")
  ))
})

validate_varlist_logical <- function(x, arg) {
  if (!is.logical(x) || length(x) != 1L || is.na(x)) {
    stop("`", arg, "` must be TRUE or FALSE.", call. = FALSE)
  }

  invisible(x)
}


validate_varlist_names <- function(x) {
  nms <- names(x)

  if (is.null(nms)) {
    stop("`x` must have column names.", call. = FALSE)
  }

  if (anyNA(nms) || any(!nzchar(nms))) {
    stop("`x` must have non-empty column names.", call. = FALSE)
  }

  if (anyDuplicated(nms)) {
    stop("`x` must have unique column names.", call. = FALSE)
  }

  invisible(x)
}


validate_varlist_selectors <- function(selectors, x) {
  selected_names <- names(x)[unname(selectors)]

  if (!identical(names(selectors), selected_names)) {
    stop(
      "`...` can select columns but cannot rename them in varlist().",
      call. = FALSE
    )
  }

  invisible(selectors)
}


match_varlist_factor_levels <- function(factor_levels) {
  choices <- c("observed", "all")

  if (
    !is.character(factor_levels) ||
      length(factor_levels) < 1L ||
      anyNA(factor_levels)
  ) {
    stop('`factor_levels` must be "observed" or "all".', call. = FALSE)
  }

  if (length(factor_levels) > 1L) {
    if (
      length(factor_levels) == length(choices) &&
        setequal(factor_levels, choices)
    ) {
      factor_levels <- factor_levels[[1L]]
    } else {
      stop('`factor_levels` must be "observed" or "all".', call. = FALSE)
    }
  }

  tryCatch(
    match.arg(factor_levels, choices = choices),
    error = function(e) {
      stop('`factor_levels` must be "observed" or "all".', call. = FALSE)
    }
  )
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom dplyr pick
#' @importFrom dplyr pull
#' @importFrom dplyr select
#' @importFrom haven labelled
#' @importFrom labelled is.labelled
#' @importFrom labelled to_factor
#' @importFrom labelled var_label
#' @importFrom labelled var_label<-
#' @importFrom rlang enquo
#' @importFrom rlang eval_tidy
#' @importFrom rlang inform
#' @importFrom rlang quo_is_null
#' @importFrom stats complete.cases
#' @importFrom stats na.omit
#' @importFrom stats setNames
#' @importFrom stats xtabs
#' @importFrom stringr str_pad
#' @importFrom tibble as_tibble
#' @importFrom tibble rownames_to_column
#' @importFrom tibble view
#' @importFrom tidyselect all_of
#' @importFrom tidyselect any_of
#' @importFrom tidyselect eval_select
#' @importFrom tidyselect everything
#' @importFrom tidyselect where
#' @importFrom utils capture.output
#' @importFrom utils head
#' @importFrom utils tail
## usethis namespace: end
NULL

#' Build a formatted ASCII table
#'
#' @description
#' Low-level internal function that constructs a visually aligned ASCII table
#' from a `data.frame`.
#' It supports Unicode characters, ANSI colors, dynamic width adjustment,
#' left/right alignment, and spacing control.
#' This function is primarily used internally by higher-level wrappers such as
#' [spicy_print_table()] or [print.spicy_freq_table()].
#'
#' @details
#' `build_ascii_table()` is the rendering engine that produces the aligned text
#' layout of **spicy-formatted tables**.
#' It automatically detects cell widths (including colored text), inserts Unicode
#' separators, and applies padding for different display modes (`"compact"`,
#' `"normal"`, `"wide"`).
#'
#' For most users, this function should not be called directly. Instead, use
#' [spicy_print_table()] which adds headers, notes, and alignment logic
#' automatically.
#'
#' @param x A `data.frame` or `spicy_table` object containing the table to format.
#'   Typically, this includes columns such as *Category*, *Values*, *Freq.*, *Percent*, etc.
#' @param padding Character string controlling horizontal spacing between columns:
#'   * `"compact"` — minimal spacing
#'   * `"normal"` — moderate spacing (default)
#'   * `"wide"` — extra spacing (for large displays or wide content)
#' @param first_column_line Logical. If `TRUE` (the default), a vertical separator
#'   is drawn after the first column (useful for separating categories from data).
#' @param row_total_line,column_total_line Logical. Control horizontal rules
#'   before total rows or columns. Both default to `TRUE`.
#' @param bottom_line Logical. If `FALSE` (the default), no closing line is drawn.
#'   If `TRUE`, draws a closing line at the bottom of the table.
#' @param lines_color Character. Color used for table separators. Defaults to `"darkgrey"`.
#'   The color is applied only when ANSI color support is available
#'   (see [crayon::has_color()]).
#' @param align_left_cols Integer vector of column indices to left-align.
#'   Defaults to `c(1, 2)` for frequency tables (Category + Values).
#' @param align_center_cols Integer vector of column indices to
#'   center-align. Defaults to `integer(0)` (no centered columns).
#'   Columns not in `align_left_cols` or `align_center_cols` are
#'   right-aligned.
#' @param group_sep_rows Integer vector of row indices before which a
#'   light dashed separator line is drawn. Defaults to `integer(0)`.
#' @param ... Additional arguments (currently ignored).
#'
#' @return
#' A single character string containing the full ASCII-formatted table,
#' suitable for direct printing with `cat()`.
#'
#' @examples
#' # Internal usage example (for developers)
#' df <- data.frame(
#'   Category = c("Valid", "", "Missing", "Total"),
#'   Values = c("Yes", "No", "NA", ""),
#'   Freq. = c(12, 8, 1, 21),
#'   Percent = c(57.1, 38.1, 4.8, 100.0)
#' )
#'
#' cat(build_ascii_table(df, padding = "compact"))
#'
#' @seealso
#' [spicy_print_table()] for a user-facing wrapper that adds titles and notes.
#'
#' @importFrom crayon has_color make_style col_nchar
#' @importFrom stringr str_pad
#'
#' @keywords internal
#' @export

build_ascii_table <- function(
  x,
  padding = c("compact", "normal", "wide"),
  first_column_line = TRUE,
  row_total_line = TRUE,
  column_total_line = TRUE,
  bottom_line = FALSE,
  lines_color = "darkgrey",
  align_left_cols = c(1L, 2L),
  align_center_cols = integer(0),
  group_sep_rows = integer(0),
  ...
) {
  stopifnot(is.data.frame(x))
  padding <- match.arg(padding)

  df <- as.data.frame(x, check.names = FALSE)
  df[] <- lapply(df, as.character)

  w <- ascii_table_widths(df, padding)

  # Helper for cell alignment
  pad_cell <- function(txt, width, align = "right") {
    side <- switch(align, left = "right", center = "both", "left")
    stringr::str_pad(txt, width, side = side)
  }

  sep_after <- ascii_table_separators(df, first_column_line, row_total_line)

  # Build line for header or data row
  build_line <- function(values, widths) {
    stopifnot(length(values) == length(widths))
    pieces <- character(0)
    bars <- integer(0)
    pos <- 0L
    for (i in seq_along(values)) {
      pieces <- c(pieces, " ")
      pos <- pos + 1L

      # Alignment: left, center, or right (default)
      col_align <- if (i %in% align_left_cols) {
        "left"
      } else if (i %in% align_center_cols) {
        "center"
      } else {
        "right"
      }
      cell <- pad_cell(values[i], widths[i], align = col_align)

      pieces <- c(pieces, cell)
      pos <- pos + nchar(cell, type = "width")
      pieces <- c(pieces, " ")
      pos <- pos + 1L

      if (i %in% sep_after) {
        pieces <- c(pieces, "\u2502")
        pos <- pos + 1L
        bars <- c(bars, pos)
      }
    }
    list(text = paste0(pieces, collapse = ""), bars = bars, width = pos)
  }

  header_line <- build_line(colnames(df), w)
  data_lines <- lapply(seq_len(nrow(df)), function(i) build_line(df[i, ], w))

  full_width <- max(c(
    header_line$width,
    vapply(data_lines, `[[`, integer(1), "width")
  ))
  normalize <- function(s) stringr::str_pad(s, full_width, side = "right")
  header_txt <- normalize(header_line$text)
  rows_txt <- vapply(data_lines, function(z) normalize(z$text), character(1))

  # Determine bar positions for horizontal rules
  bar_positions <- sort(unique(c(
    header_line$bars,
    unlist(lapply(data_lines, `[[`, "bars"))
  )))
  bar_positions <- bar_positions[
    bar_positions >= 1 & bar_positions <= full_width
  ]

  make_rule <- function(width, bars, junction = "\u253c") {
    chars <- rep("\u2500", width)
    if (length(bars)) {
      chars[bars] <- junction
    }
    paste0(chars, collapse = "")
  }

  style <- if (crayon::has_color()) {
    crayon::make_style(lines_color)
  } else {
    identity
  }
  header_rule <- style(make_rule(full_width, bar_positions, "\u253c"))
  total_rule <- style(make_rule(full_width, bar_positions, "\u253c")) # line before Total
  bottom_rule <- style(make_rule(full_width, bar_positions, "\u2534"))

  # Light dashed rule for group separators (U+254C = BOX DRAWINGS LIGHT DOUBLE DASH HORIZONTAL)
  make_light_rule <- function(width, bars) {
    chars <- rep("\u254c", width)
    if (length(bars)) {
      chars[bars] <- "\u253c"
    }
    paste0(chars, collapse = "")
  }
  light_rule <- style(make_light_rule(full_width, bar_positions))

  # --- Colorize vertical bars if supported
  if (crayon::has_color()) {
    header_txt <- gsub("\u2502", style("\u2502"), header_txt, fixed = TRUE)
    rows_txt <- gsub("\u2502", style("\u2502"), rows_txt, fixed = TRUE)
  }

  out <- character(0)

  # --- Add header
  out <- c(out, header_txt, header_rule)

  # --- Add rows, with horizontal line before Total and light separators
  total_idx <- grep("\\b(Total|Column_Total)\\b", rows_txt, perl = TRUE)
  sep_set <- as.integer(group_sep_rows)
  sep_set <- sep_set[sep_set >= 2L & sep_set <= length(rows_txt)]

  if (length(total_idx) == 1 && total_idx > 1) {
    # Insert light separators before total rule, then total
    body_rows <- seq_len(total_idx - 1)
    for (r in body_rows) {
      if (r %in% sep_set) {
        out <- c(out, light_rule)
      }
      out <- c(out, rows_txt[r])
    }
    out <- c(out, total_rule, rows_txt[total_idx:length(rows_txt)])
  } else if (length(sep_set) > 0L && length(rows_txt) > 0L) {
    for (r in seq_along(rows_txt)) {
      if (r %in% sep_set) {
        out <- c(out, light_rule)
      }
      out <- c(out, rows_txt[r])
    }
  } else {
    if (length(rows_txt)) out <- c(out, rows_txt)
  }

  # --- Bottom rule
  if (isTRUE(bottom_line)) {
    out <- c(out, bottom_rule)
  }

  paste(out, collapse = "\n")
}

ascii_table_widths <- function(df, padding) {
  widths <- vapply(
    seq_along(df),
    function(i) {
      max(
        crayon::col_nchar(c(df[[i]], colnames(df)[i]), type = "width"),
        na.rm = TRUE
      )
    },
    numeric(1)
  )

  if (identical(padding, "normal")) {
    widths <- widths + 5L
  }
  if (identical(padding, "wide")) {
    widths <- widths + 9L
  }

  widths
}

ascii_table_separators <- function(df, first_column_line, row_total_line) {
  sep_after <- integer(0)

  if (isTRUE(first_column_line) && ncol(df) > 1L) {
    sep_after <- c(sep_after, 1L)
  }

  if (isTRUE(row_total_line) && any(c("Row_Total", "Total") %in% names(df))) {
    idx <- which(names(df) %in% c("Row_Total", "Total"))[1]
    sep_after <- c(sep_after, idx - 1L)
  }

  sort(unique(sep_after[sep_after >= 1L & sep_after <= ncol(df)]))
}

ascii_table_total_width <- function(
  df,
  padding,
  first_column_line,
  row_total_line
) {
  widths <- ascii_table_widths(df, padding)
  separators <- ascii_table_separators(df, first_column_line, row_total_line)

  sum(widths + 2L) + length(separators)
}

ascii_table_panels <- function(
  df,
  console_width,
  padding,
  first_column_line,
  row_total_line,
  sticky_cols
) {
  cols <- seq_len(ncol(df))
  if (length(cols) <= 1L) {
    return(list(cols))
  }

  sticky_cols <- sort(unique(as.integer(sticky_cols)))
  sticky_cols <- sticky_cols[sticky_cols %in% cols]
  if (!length(sticky_cols)) {
    sticky_cols <- 1L
  }

  if (
    ascii_table_total_width(
      df,
      padding = padding,
      first_column_line = first_column_line,
      row_total_line = row_total_line
    ) <=
      console_width
  ) {
    return(list(cols))
  }

  remaining_cols <- setdiff(cols, sticky_cols)
  if (!length(remaining_cols)) {
    return(list(cols))
  }

  panels <- list()
  current <- sticky_cols

  for (col in remaining_cols) {
    candidate <- sort(c(current, col))
    candidate_width <- ascii_table_total_width(
      df[, candidate, drop = FALSE],
      padding = padding,
      first_column_line = first_column_line,
      row_total_line = row_total_line
    )

    if (
      candidate_width <= console_width ||
        identical(sort(current), sort(sticky_cols))
    ) {
      current <- candidate
      next
    }

    panels[[length(panels) + 1L]] <- current
    current <- sort(c(sticky_cols, col))
  }

  panels[[length(panels) + 1L]] <- current
  panels
}


#' Print a spicy-formatted ASCII table
#'
#' @description
#' User-facing helper that prints a visually aligned, spicy-styled ASCII table
#' created by functions such as [freq()] or [cross_tab()].
#' It automatically adjusts column alignment, spacing, and separators for
#' improved readability in console outputs.
#'
#' This function wraps the internal renderer [build_ascii_table()], adding
#' optional titles, notes, and automatic alignment rules depending on the type
#' of table.
#'
#' @details
#' `spicy_print_table()` detects whether the table represents frequencies
#' (`freq`-style) or cross-tabulations (`cross`-style) and adjusts formatting
#' accordingly:
#' * For **frequency tables**, the first two columns (*Category* and *Values*)
#'   are left-aligned.
#' * For **cross tables**, only the first column (row variable) is left-aligned.
#'
#' The function supports Unicode line-drawing characters and colored separators
#' using the **crayon** package, with graceful fallback to monochrome output when
#' color is not supported.
#' If the table exceeds the console width, it is split into stacked horizontal
#' panels while repeating the left-most identifier columns.
#'
#' @param x A `spicy_table` or `data.frame` to be printed.
#' @param title Optional title displayed above the table. Defaults to the
#'   `"title"` attribute of `x` if present.
#' @param note Optional note displayed below the table. Defaults to the `"note"`
#'   attribute of `x` if present.
#' @param padding Character string controlling horizontal spacing between columns:
#'   * `"compact"` - minimal spacing
#'   * `"normal"` - moderate spacing (default)
#'   * `"wide"` - extra spacing (for wide displays)
#' @param first_column_line Logical. If `TRUE` (the default), adds a vertical separator
#'   after the first column.
#' @param row_total_line,column_total_line,bottom_line Logical flags controlling
#'   the presence of horizontal lines before total rows/columns or at the bottom
#'   of the table.
#'   Both `row_total_line` and `column_total_line` default to `TRUE`;
#'   `bottom_line` defaults to `FALSE`.
#' @param lines_color Character. Color for table separators. Defaults to `"darkgrey"`.
#'   Only applied if the output supports ANSI colors (see [crayon::has_color()]).
#' @param align_left_cols Integer vector of column indices to left-align.
#'   If `NULL` (the default), alignment is auto-detected based on `x`:
#'   * For `freq` tables -> `c(1, 2)`
#'   * For `cross` tables -> `1`
#' @param align_center_cols Integer vector of column indices to
#'   center-align. Defaults to `integer(0)`.
#' @param group_sep_rows Integer vector of row indices before which a
#'   light dashed separator line is drawn. Defaults to `integer(0)`.
#' @param ... Additional arguments passed to [build_ascii_table()].
#'
#' @return
#' Invisibly returns `x`, after printing the formatted ASCII table to the console.
#'
#' @examples
#' # Simple demonstration
#' df <- data.frame(
#'   Category = c("Valid", "", "Missing", "Total"),
#'   Values = c("Yes", "No", "NA", ""),
#'   Freq. = c(12, 8, 1, 21),
#'   Percent = c(57.1, 38.1, 4.8, 100.0)
#' )
#'
#' spicy_print_table(df,
#'   title = "Frequency table: Example",
#'   note = "Class: data.frame\nData: demo"
#' )
#'
#' @seealso
#' [build_ascii_table()] for the underlying text rendering engine.
#' [print.spicy_freq_table()] for the specialized printing method used by [freq()].
#'
#' @importFrom crayon has_color make_style
#'
#' @export

spicy_print_table <- function(
  x,
  title = attr(x, "title"),
  note = attr(x, "note"),
  padding = c("compact", "normal", "wide"),
  first_column_line = TRUE,
  row_total_line = TRUE,
  column_total_line = TRUE,
  bottom_line = FALSE,
  lines_color = "darkgrey",
  align_left_cols = NULL,
  align_center_cols = integer(0),
  group_sep_rows = integer(0),
  ...
) {
  stopifnot(is.data.frame(x))
  padding <- match.arg(padding)

  table_type <- if (any(grepl("^Category$", names(x)))) "freq" else "cross"

  if (is.null(align_left_cols)) {
    align_left_cols <- if (table_type == "freq") c(1L, 2L) else 1L
  }

  if (!is.null(title)) {
    attr(x, "title") <- title
  }
  if (!is.null(note)) {
    attr(x, "note") <- note
  }

  panel_cols <- ascii_table_panels(
    x,
    console_width = getOption("width", 80L),
    padding = padding,
    first_column_line = first_column_line,
    row_total_line = row_total_line,
    sticky_cols = align_left_cols
  )

  txt <- vapply(
    panel_cols,
    function(cols) {
      build_ascii_table(
        x[, cols, drop = FALSE],
        padding = padding,
        first_column_line = first_column_line,
        row_total_line = row_total_line,
        column_total_line = column_total_line,
        bottom_line = bottom_line,
        lines_color = lines_color,
        align_left_cols = which(cols %in% align_left_cols),
        align_center_cols = which(cols %in% align_center_cols),
        group_sep_rows = group_sep_rows,
        ...
      )
    },
    character(1)
  )

  style_grey <- if (crayon::has_color()) {
    crayon::make_style("darkgrey")
  } else {
    identity
  }
  if (!is.null(title)) {
    cat(style_grey(title), "\n\n", sep = "")
  }
  cat(txt[1], "\n", sep = "")
  if (length(txt) > 1L) {
    for (i in 2:length(txt)) {
      cat("\n", txt[i], "\n", sep = "")
    }
  }
  if (!is.null(note)) {
    cat("\n", style_grey(note), "\n", sep = "")
  }

  invisible(x)
}

test_that("build_ascii_table produces aligned ASCII output", {
  df <- data.frame(A = 1:2, B = c("x", "yy"))
  txt <- build_ascii_table(df)

  expect_type(txt, "character")
  expect_true(any(grepl("\u2502", txt))) # has vertical bars
  expect_true(any(grepl("\u2500", txt))) # has horizontal lines
  expect_no_error(build_ascii_table(df))
})


test_that("spicy_print_table prints and returns invisibly", {
  df <- data.frame(Category = "Valid", Values = "A", Freq. = 1)

  # Test invisibility directly
  expect_invisible(spicy_print_table(df, title = "Test Title"))

  # Test capture of printed output (for robustness)
  output <- capture.output(spicy_print_table(df, title = "Test Title"))
  expect_true(any(grepl("Test Title", output)))
})


test_that("spicy_print_table aligns Category and Values left", {
  df <- data.frame(
    Category = c("Valid", "Total"),
    Values = c("A", "B"),
    Freq. = c(1, 2)
  )
  output <- capture.output(spicy_print_table(df))

  # Rough alignment test (spaces after "Valid")
  expect_true(any(grepl("^ Valid", output)))
})

test_that("build_ascii_table supports wide padding", {
  df <- data.frame(A = 1:2, B = c("x", "y"))
  txt <- build_ascii_table(df, padding = "wide")
  expect_type(txt, "character")
})

test_that("build_ascii_table supports bottom_line", {
  df <- data.frame(A = 1:2, B = c("x", "y"))
  txt <- build_ascii_table(df, bottom_line = TRUE)
  expect_true(grepl("\u2534", txt))
})

test_that("print.spicy_categorical_table falls back to x when display_df is absent", {
  x <- data.frame(
    Variable = c("Smoking", "  Yes"),
    n = c("10", "10"),
    check.names = FALSE
  )
  class(x) <- c("spicy_categorical_table", "spicy_table", "data.frame")
  attr(x, "data_name") <- "demo"
  attr(x, "indent_text") <- "  "

  output <- capture.output(print(x))

  expect_true(any(grepl("Categorical table", output, fixed = TRUE)))
})

test_that("print.spicy_categorical_table uses grouped title and compact padding", {
  withr::local_options(list(width = 18))

  x <- data.frame(
    Variable = c("Var 1", "  Yes", "Var 2"),
    n = c("12", "8", "9"),
    check.names = FALSE
  )
  class(x) <- c("spicy_categorical_table", "spicy_table", "data.frame")
  attr(x, "display_df") <- x
  attr(x, "data_name") <- "demo"
  attr(x, "group_var") <- "education"
  attr(x, "indent_text") <- "  "

  output <- capture.output(print(x))

  expect_true(any(grepl(
    "Categorical table by education",
    output,
    fixed = TRUE
  )))
})

test_that("spicy_print_table splits wide tables into stacked panels", {
  withr::local_options(list(width = 26))

  df <- data.frame(
    Variable = c("Smoking", "Activity"),
    Group = c("Women", "Men"),
    Count = c("120", "98"),
    Percent = c("52.3", "47.7"),
    check.names = FALSE
  )

  output <- capture.output(
    spicy_print_table(
      df,
      title = NULL,
      padding = "compact",
      align_left_cols = c(1L, 2L)
    )
  )

  expect_gt(sum(grepl("Variable", output, fixed = TRUE)), 1L)
  expect_true(any(grepl("Count", output, fixed = TRUE)))
  expect_true(any(grepl("Percent", output, fixed = TRUE)))
})

#' Print method for categorical summary tables
#'
#' @description
#' Formats and prints a `spicy_categorical_table` object as a styled ASCII table using
#' [spicy_print_table()].
#'
#' @param x A `data.frame` of class `"spicy_categorical_table"` as returned by
#'   [table_categorical()] with `output = "default"` and `styled = TRUE`.
#' @param ... Additional arguments (currently ignored).
#'
#' @return Invisibly returns `x`.
#'
#' @seealso [table_categorical()], [spicy_print_table()]
#' @keywords internal
#' @export
print.spicy_categorical_table <- function(x, ...) {
  display_df <- attr(x, "display_df")
  group_var <- attr(x, "group_var")
  indent_text <- attr(x, "indent_text") %||% "  "

  if (is.null(display_df)) {
    display_df <- x
  }

  align_left <- 1L
  align_center <- integer(0)

  sep_rows <- integer(0)
  first_col <- display_df[[1]]
  for (i in seq_along(first_col)) {
    if (
      i > 1L && nzchar(first_col[i]) && !startsWith(first_col[i], indent_text)
    ) {
      sep_rows <- c(sep_rows, i)
    }
  }

  padding <- "normal"
  col_widths <- vapply(
    seq_along(display_df),
    function(i) {
      max(nchar(c(names(display_df)[i], as.character(display_df[[i]]))))
    },
    numeric(1)
  )
  console_w <- getOption("width", 80L)
  normal_width <- sum(col_widths + 5L + 2L) + 1L
  if (normal_width > console_w) {
    padding <- "compact"
  }

  title <- if (is.null(group_var)) {
    "Categorical table"
  } else {
    paste0("Categorical table by ", group_var)
  }

  spicy_print_table(
    display_df,
    title = title,
    note = NULL,
    padding = padding,
    first_column_line = TRUE,
    row_total_line = FALSE,
    column_total_line = FALSE,
    bottom_line = FALSE,
    align_left_cols = align_left,
    align_center_cols = align_center,
    group_sep_rows = sep_rows
  )

  invisible(x)
}

#' Generate a comprehensive summary of the variables
#'
#' `varlist()` lists the variables of a data frame and extracts essential
#' metadata, including variable names, labels, summary values, classes, number
#' of distinct values, number of valid (non-missing) observations, and number
#' of missing values.
#'
#' The function can also apply tidyselect-style variable selectors to select or
#' reorder columns dynamically.
#'
#' If used interactively (e.g. in RStudio or Positron), the summary is
#' displayed in the Viewer pane with a contextual title like `vl: sochealth`.
#' If the data frame has been transformed or subsetted, the title will display
#' an asterisk (`*`), e.g. `vl: sochealth*`. Anonymous or ambiguous calls use
#' `vl: <data>`.
#'
#' For factor variables, `varlist()` defaults to displaying only the levels
#' observed in the data (`factor_levels = "observed"`) — a reflection of what
#' is actually present. By contrast, [code_book()] defaults to `"all"` to
#' document the declared schema, including unused levels. Pass `factor_levels`
#' explicitly to override either default.
#'
#' @aliases vl
#'
#' @param x A data frame, or a transformation of one.
#' @param ... Optional tidyselect-style column selectors (e.g.
#'   `starts_with("var")`, `where(is.numeric)`, etc.). Columns can be selected
#'   or reordered, but renaming selections is not supported.
#'
#' @param values Logical. If `FALSE` (the default), displays a compact summary
#'   of the variable's values. For numeric, character, date/time, labelled, and
#'   factor variables, all unique non-missing values are shown when there are
#'   at most four; otherwise the first three values, an ellipsis (`...`), and
#'   the last value are shown. Values are sorted when appropriate (e.g.,
#'   numeric, character, date).
#'   For factors, `factor_levels` controls whether observed or all declared
#'   levels are shown; level order is preserved.
#'   For labelled variables, prefixed labels are displayed via
#'   `labelled::to_factor(levels = "prefixed")`.
#'   If `TRUE`, all unique non-missing values are displayed.
#' @param tbl Logical. If `FALSE` (the default), opens the summary in the Viewer
#'   if the session is interactive. If `TRUE`, returns a tibble.
#' @param include_na Logical. If `TRUE`, unique missing value markers
#'   (`<NA>`, `<NaN>`) are explicitly appended at the end of the `Values`
#'   summary when present in the variable. This applies to all variable types.
#'   Literal strings `"NA"`, `"NaN"`, and `""` are quoted to distinguish them
#'   from missing markers. If `FALSE` (the default), missing values are omitted
#'   from `Values` but still counted in the `NAs` column.
#' @param factor_levels Character. Controls how factor values are displayed
#'   in `Values`. `"observed"` (the default; [code_book()] uses `"all"`)
#'   shows only levels present in the data, preserving factor level order.
#'   `"all"` shows all declared levels, including unused levels.
#'
#' @returns
#' A tibble with one row per selected variable, containing the following
#' columns:
#' - `Variable`: variable names
#' - `Label`: variable labels (if available via the `label` attribute)
#' - `Values`: a summary of the variable's values, depending on the `values`
#'   and `include_na` arguments. If `values = FALSE`, a compact summary is
#'   shown: all unique values when there are at most four, otherwise
#'   3 + ... + last. If `values = TRUE`, all unique non-missing values are
#'   displayed. For labelled variables, **prefixed labels** are displayed using
#'   `labelled::to_factor(levels = "prefixed")`.
#'   For factors, levels are displayed according to `factor_levels`.
#'   Matrix and array columns are summarized by their dimensions.
#'   Missing value markers (`<NA>`, `<NaN>`) are optionally appended at the
#'   end (controlled via `include_na`). Literal strings `"NA"`, `"NaN"`, and
#'   `""` are quoted to distinguish them from missing markers.
#' - `Class`: the class of each variable (possibly multiple, e.g.
#'   `"labelled", "numeric"`)
#' - `N_distinct`: number of distinct non-missing values
#' - `N_valid`: number of non-missing observations
#' - `NAs`: number of missing observations
#'
#' If `tbl = TRUE`, the tibble is returned. If `tbl = FALSE` and the session is
#' interactive, the summary is displayed in the Viewer pane and the function
#' returns invisibly. In non-interactive sessions, a message is displayed and
#' the function returns invisibly.
#'
#' @importFrom labelled is.labelled
#' @importFrom labelled to_factor
#' @importFrom tibble as_tibble view
#' @importFrom tidyselect eval_select everything
#' @importFrom rlang expr
#' @importFrom stats na.omit
#' @importFrom utils head tail
#'
#' @export
#'
#' @examples
#' varlist(sochealth, tbl = TRUE)
#' sochealth |> varlist(tbl = TRUE)
#' varlist(sochealth, where(is.numeric), values = TRUE, tbl = TRUE)
#' varlist(
#'   sochealth,
#'   starts_with("bmi"),
#'   values = TRUE,
#'   include_na = TRUE,
#'   tbl = TRUE
#' )
#'
#' df <- data.frame(
#'   group = factor(c("A", "B", NA), levels = c("A", "B", "C"))
#' )
#' varlist(
#'   df,
#'   values = TRUE,
#'   include_na = TRUE,
#'   factor_levels = "all",
#'   tbl = TRUE
#' )
#'
varlist <- function(
  x,
  ...,
  values = FALSE,
  tbl = FALSE,
  include_na = FALSE,
  factor_levels = c("observed", "all")
) {
  varlist_impl(
    x = x,
    ...,
    values = values,
    tbl = tbl,
    include_na = include_na,
    factor_levels = factor_levels,
    raw_expr = substitute(x)
  )
}


varlist_impl <- function(
  x,
  ...,
  values = FALSE,
  tbl = FALSE,
  include_na = FALSE,
  factor_levels = c("observed", "all"),
  raw_expr = substitute(x)
) {
  if (!is.data.frame(x)) {
    stop(
      "varlist() only works with named data frames or transformations of them.",
      call. = FALSE
    )
  }

  validate_varlist_names(x)
  validate_varlist_logical(values, "values")
  validate_varlist_logical(tbl, "tbl")
  validate_varlist_logical(include_na, "include_na")
  factor_levels <- match_varlist_factor_levels(factor_levels)

  selectors <- if (missing(...)) {
    tidyselect::eval_select(rlang::expr(everything()), data = x)
  } else {
    tidyselect::eval_select(rlang::expr(c(...)), data = x)
  }
  validate_varlist_selectors(selectors, x)

  if (length(selectors) == 0) {
    warning("No columns selected.", call. = FALSE)
    res <- tibble::tibble(
      Variable = character(),
      Label = character(),
      Values = character(),
      Class = character(),
      N_distinct = integer(),
      N_valid = integer(),
      NAs = integer()
    )

    if (tbl) {
      return(res)
    }

    if (interactive()) {
      tryCatch(
        tibble::view(res, title = "vl: (no columns selected)"),
        error = function(e) {
          message("tibble::view() failed: ", e$message)
          message("Displaying result in console instead:")
          print(res)
        }
      )
    } else {
      message("No columns selected. Use `tbl = TRUE` to return result.")
    }

    return(invisible(NULL))
  }

  x <- x[selectors]

  res <- list(
    Variable = names(x),
    Label = vapply(
      x,
      function(col) {
        lbl <- attributes(col)[["label"]]

        if (is.null(lbl)) {
          NA_character_
        } else {
          as.character(lbl)
        }
      },
      character(1)
    ),
    Class = vapply(
      x,
      function(col) paste(class(col), collapse = ", "),
      character(1)
    ),
    N_distinct = vapply(
      x,
      varlist_n_distinct,
      integer(1)
    ),
    N_valid = vapply(x, varlist_n_valid, integer(1)),
    NAs = vapply(x, varlist_n_missing, integer(1))
  )

  res$Values <- vapply(
    seq_along(x),
    function(i) {
      summarize_varlist_column(
        col = x[[i]],
        name = names(x)[[i]],
        values = values,
        include_na = include_na,
        factor_levels = factor_levels
      )
    },
    character(1)
  )

  res <- tibble::as_tibble(res[c(
    "Variable",
    "Label",
    "Values",
    "Class",
    "N_distinct",
    "N_valid",
    "NAs"
  )])

  if (tbl) {
    return(res)
  } else if (interactive()) {
    title_txt <- varlist_title(expr = raw_expr, selectors_used = !missing(...))

    tryCatch(
      tibble::view(res, title = title_txt),
      error = function(e) {
        message("tibble::view() failed: ", e$message)
        message("Displaying result in console instead:")
        print(res)
      }
    )
  } else {
    message("Non-interactive session: use `tbl = TRUE` to return a tibble.")
  }

  invisible(NULL)
}


#' Alias for `varlist()`
#'
#' `vl()` is a convenient shorthand for `varlist()` that offers identical
#' functionality with a shorter name.
#'
#' @rdname varlist
#'
#' @export
#'
#' @examples
#' vl(sochealth, tbl = TRUE)
#' sochealth |> vl(tbl = TRUE)
#' vl(sochealth, starts_with("bmi"), tbl = TRUE)
#' vl(sochealth, where(is.numeric), values = TRUE, tbl = TRUE)
vl <- function(
  x,
  ...,
  values = FALSE,
  tbl = FALSE,
  include_na = FALSE,
  factor_levels = c("observed", "all")
) {
  varlist_impl(
    x = x,
    ...,
    values = values,
    tbl = tbl,
    include_na = include_na,
    factor_levels = factor_levels,
    raw_expr = substitute(x)
  )
}

empty_code_book_data <- function() {
  data.frame(
    Variable = character(),
    Label = character(),
    Values = character(),
    Class = character(),
    N_distinct = integer(),
    N_valid = integer(),
    NAs = integer()
  )
}


code_book_columns <- function() {
  c(
    "Variable",
    "Label",
    "Values",
    "Class",
    "N_distinct",
    "N_valid",
    "NAs"
  )
}


code_book_filenames <- function(cb) {
  vapply(
    cb$x$options$buttons[[3]]$buttons,
    function(button) button$filename,
    character(1)
  )
}


code_book_export_extends <- function(cb) {
  vapply(
    cb$x$options$buttons[[3]]$buttons,
    function(button) button$extend,
    character(1)
  )
}


test_that("code_book() runs without error on a simple data frame", {
  skip_if_not_installed("DT")

  df <- head(mtcars)

  expect_silent({
    cb <- suppressMessages(code_book(df))
  })

  expect_s3_class(cb, "datatables")
  expect_true(inherits(cb, "htmlwidget"))
  expect_named(cb$x$data, code_book_columns())
  expect_equal(cb$x$data$Variable, names(df))
  expect_equal(cb$x$options$dom, "Bfrtip")
  expect_equal(cb$x$options$pageLength, 10)
  expect_true(cb$x$options$colReorder)
  expect_true(cb$x$options$fixedHeader)
  expect_equal(
    unlist(cb$x$extensions, use.names = FALSE),
    c("Buttons", "ColReorder", "FixedHeader")
  )
})

test_that("code_book() works with values = TRUE", {
  skip_if_not_installed("DT")
  df <- data.frame(x = letters[1:6])

  cb <- suppressMessages(code_book(df, values = TRUE))

  expect_s3_class(cb, "datatables")
  expect_equal(cb$x$data$Values, "a, b, c, d, e, f")
})

test_that("code_book() works with include_na = TRUE", {
  skip_if_not_installed("DT")
  df <- data.frame(x = c(1, NA, 3), y = c("a", "b", NA))

  cb <- suppressMessages(code_book(df, include_na = TRUE))

  expect_s3_class(cb, "datatables")
  expect_match(cb$x$data$Values[[1]], "<NA>", fixed = TRUE)
  expect_match(cb$x$data$Values[[2]], "<NA>", fixed = TRUE)
})

test_that("code_book() selects and reorders variables like varlist()", {
  skip_if_not_installed("DT")
  df <- data.frame(a = 1:3, b = 4:6, c = letters[1:3])

  cb <- suppressMessages(code_book(df, b, a))
  expect_equal(cb$x$data$Variable, c("b", "a"))

  cb <- suppressMessages(code_book(df, where(is.character)))
  expect_equal(cb$x$data$Variable, "c")
})

test_that("code_book() returns the same data as varlist()", {
  skip_if_not_installed("DT")
  df <- data.frame(
    a = c(3, 1, NA),
    b = letters[1:3],
    c = factor(c("yes", "no", "yes"), levels = c("no", "yes", "missing"))
  )

  expected <- varlist(
    df,
    c,
    a,
    values = TRUE,
    include_na = TRUE,
    factor_levels = "all",
    tbl = TRUE
  )
  cb <- suppressMessages(code_book(
    df,
    c,
    a,
    values = TRUE,
    include_na = TRUE,
    factor_levels = "all"
  ))

  expect_equal(cb$x$data, as.data.frame(expected))
})

test_that("code_book() accepts custom title", {
  skip_if_not_installed("DT")

  cb <- suppressMessages(code_book(head(mtcars), title = "My Codebook"))

  expect_s3_class(cb, "datatables")
  expect_equal(cb$x$caption, "<caption>My Codebook</caption>")
  expect_equal(code_book_filenames(cb), rep("My_Codebook", 3))
})

test_that("code_book() configures export buttons consistently", {
  skip_if_not_installed("DT")

  cb <- suppressMessages(code_book(head(mtcars), title = "My Codebook"))
  buttons <- cb$x$options$buttons
  download_buttons <- buttons[[3]]$buttons

  expect_equal(buttons[[1]], "copy")
  expect_equal(buttons[[2]], "print")
  expect_equal(buttons[[3]]$extend, "collection")
  expect_equal(buttons[[3]]$text, "Download")
  expect_equal(code_book_export_extends(cb), c("csv", "excel", "pdf"))
  expect_true(all(vapply(
    download_buttons,
    function(button) is.null(button$title),
    logical(1)
  )))
  expect_equal(code_book_filenames(cb), rep("My_Codebook", 3))
})

test_that("code_book() accepts title = NULL", {
  skip_if_not_installed("DT")

  cb <- suppressMessages(code_book(head(mtcars), title = NULL))

  expect_s3_class(cb, "datatables")
  expect_null(cb$x$caption)
  expect_equal(cb$x$options$buttons[[3]]$buttons[[1]]$filename, "Codebook")
})

test_that("code_book() accepts explicit export filenames", {
  skip_if_not_installed("DT")

  cb <- suppressMessages(code_book(
    head(mtcars),
    title = "Codebook: BMI / smoking?",
    filename = "bmi smoking review"
  ))

  expect_equal(cb$x$caption, "<caption>Codebook: BMI / smoking?</caption>")
  expect_equal(code_book_filenames(cb), rep("bmi_smoking_review", 3))

  cb <- suppressMessages(code_book(
    head(mtcars),
    title = NULL,
    filename = "exports/final:codebook"
  ))

  expect_null(cb$x$caption)
  expect_equal(code_book_filenames(cb), rep("exports_final_codebook", 3))
})

test_that("code_book() sanitizes export filenames", {
  skip_if_not_installed("DT")

  cb <- suppressMessages(code_book(
    head(mtcars),
    title = "Codebook: BMI / smoking?"
  ))
  expect_equal(cb$x$caption, "<caption>Codebook: BMI / smoking?</caption>")
  expect_equal(code_book_filenames(cb), rep("Codebook_BMI_smoking", 3))

  cb <- suppressMessages(code_book(head(mtcars), title = "***"))
  expect_equal(code_book_filenames(cb), rep("Codebook", 3))

  cb <- suppressMessages(code_book(
    head(mtcars),
    title = "\u00c2ge & sant\u00e9"
  ))
  expect_equal(cb$x$caption, "<caption>\u00c2ge &amp; sant\u00e9</caption>")
  expect_equal(code_book_filenames(cb), rep("Age_sante", 3))

  cb <- suppressMessages(code_book(
    head(mtcars),
    title = "Cafe\u0301 creme brule\u0301e"
  ))
  expect_equal(code_book_filenames(cb), rep("Cafe_creme_brulee", 3))

  cb <- suppressMessages(code_book(
    head(mtcars),
    filename = "R\u00e9sum\u00e9 final"
  ))
  expect_equal(code_book_filenames(cb), rep("Resume_final", 3))
})

test_that("code_book() works with labelled data", {
  skip_if_not_installed("DT")
  skip_if_not_installed("labelled")
  df <- data.frame(x = labelled::labelled(1:3, labels = c(A = 1, B = 2, C = 3)))
  cb <- suppressMessages(code_book(df))
  expect_s3_class(cb, "datatables")
})

test_that("code_book() errors on non-data.frame", {
  expect_error(code_book(1:10), "`x` must be a data frame or tibble")
})

test_that("code_book() errors when DT is not available", {
  local_mocked_bindings(
    requireNamespace = function(pkg, ...) if (pkg == "DT") FALSE else TRUE,
    .package = "base"
  )
  expect_error(code_book(mtcars), "Package 'DT' is required")
})

test_that("code_book() passes arguments and selectors to varlist", {
  skip_if_not_installed("DT")
  captured <- list()

  local_mocked_bindings(
    varlist = function(x, ..., values, include_na, factor_levels, tbl) {
      captured <<- list(
        values = values,
        include_na = include_na,
        factor_levels = factor_levels,
        tbl = tbl,
        dots = as.list(substitute(list(...)))[-1]
      )
      empty_code_book_data()
    },
    .package = "spicy"
  )

  suppressMessages(code_book(
    mtcars,
    cyl,
    values = TRUE,
    include_na = TRUE,
    factor_levels = "observed"
  ))

  expect_true(captured$values)
  expect_true(captured$include_na)
  expect_equal(captured$factor_levels, "observed")
  expect_true(captured$tbl)
  expect_identical(captured$dots[[1]], quote(cyl))
})

test_that("code_book() validates factor_levels", {
  expect_error(
    code_book(mtcars, factor_levels = "bad"),
    '`factor_levels` must be "observed" or "all"'
  )
})

test_that("code_book() validates logical and title arguments", {
  expect_error(
    code_book(mtcars, values = "yes"),
    "`values` must be TRUE or FALSE"
  )
  expect_error(
    code_book(mtcars, include_na = NA),
    "`include_na` must be TRUE or FALSE"
  )
  expect_error(
    code_book(mtcars, title = NA_character_),
    "`title` must be NULL or a single non-empty character string"
  )
  expect_error(
    code_book(mtcars, title = ""),
    "`title` must be NULL or a single non-empty character string"
  )
})

test_that("code_book() validates filename arguments", {
  expect_error(
    code_book(mtcars, filename = NA_character_),
    "`filename` must be NULL or a single non-empty character string"
  )
  expect_error(
    code_book(mtcars, filename = ""),
    "`filename` must be NULL or a single non-empty character string"
  )
  expect_error(
    code_book(mtcars, filename = c("a", "b")),
    "`filename` must be NULL or a single non-empty character string"
  )
  expect_error(
    code_book(mtcars, filename = "***"),
    "`filename` must contain at least one letter"
  )
})

test_that("code_book() rejects option-like partial matches in dots", {
  expect_error(
    code_book(mtcars, value = TRUE),
    "`value` was supplied through `\\.\\.\\.`"
  )
  expect_error(
    code_book(mtcars, inc = TRUE),
    "`inc` was supplied through `\\.\\.\\.`"
  )
  expect_error(
    code_book(mtcars, tit = "x"),
    "`tit` was supplied through `\\.\\.\\.`"
  )
  expect_error(
    code_book(mtcars, fac = "observed"),
    "`fac` was supplied through `\\.\\.\\.`"
  )
  expect_error(
    code_book(mtcars, fil = "x"),
    "`fil` was supplied through `\\.\\.\\.`"
  )
})

test_that("code_book() rejects renamed selections", {
  skip_if_not_installed("DT")
  df <- data.frame(x = 1:3, y = 4:6)

  expect_error(
    code_book(df, selected = x),
    "`\\.\\.\\.` can select columns but cannot rename them"
  )
})

test_that("code_book() handles empty selections", {
  skip_if_not_installed("DT")
  df <- data.frame(x = 1:3)

  expect_warning(
    cb <- suppressMessages(code_book(df, starts_with("missing"))),
    "No columns selected"
  )

  expect_s3_class(cb, "datatables")
  expect_named(cb$x$data, code_book_columns())
  expect_equal(nrow(cb$x$data), 0L)
})

test_that("code_book() handles data frames with no columns", {
  skip_if_not_installed("DT")
  df <- data.frame()

  expect_warning(
    cb <- suppressMessages(code_book(df)),
    "No columns selected"
  )

  expect_s3_class(cb, "datatables")
  expect_named(cb$x$data, code_book_columns())
  expect_equal(nrow(cb$x$data), 0L)
  expect_equal(cb$x$caption, "<caption>Codebook</caption>")
  expect_equal(code_book_export_extends(cb), c("csv", "excel", "pdf"))
  expect_equal(code_book_filenames(cb), rep("Codebook", 3))
})

test_that("code_book() defaults to all factor levels", {
  skip_if_not_installed("DT")

  captured <- character()

  local_mocked_bindings(
    varlist = function(..., factor_levels, tbl) {
      captured <<- c(captured, factor_levels)
      empty_code_book_data()
    },
    .package = "spicy"
  )

  suppressMessages(code_book(mtcars))
  suppressMessages(code_book(mtcars, factor_levels = "observed"))

  expect_equal(captured, c("all", "observed"))
})

test_that("code_book() lets varlist() errors surface directly", {
  skip_if_not_installed("DT")

  local_mocked_bindings(
    varlist = function(...) stop("bad varlist", call. = FALSE),
    .package = "spicy"
  )

  expect_error(
    code_book(mtcars),
    "^bad varlist$"
  )
})

test_that("code_book() lets varlist() column name errors surface directly", {
  skip_if_not_installed("DT")
  df <- data.frame(x = 1:3, y = 4:6)
  names(df) <- c("x", "x")

  expect_error(
    code_book(df),
    "`x` must have unique column names"
  )
})

test_that("code_book() errors when varlist() does not return a data frame", {
  skip_if_not_installed("DT")

  local_mocked_bindings(
    varlist = function(...) list(x = 1),
    .package = "spicy"
  )

  expect_error(
    code_book(mtcars),
    "`varlist\\(\\)` did not return a data frame"
  )
})

test_that("code_book() HTML-escapes special characters in title", {
  skip_if_not_installed("DT")

  cb <- suppressMessages(code_book(
    head(mtcars),
    title = "<script>alert(1)</script>"
  ))

  expect_equal(
    cb$x$caption,
    "<caption>&lt;script&gt;alert(1)&lt;/script&gt;</caption>"
  )
  expect_false(grepl("<script>alert", cb$x$caption, fixed = TRUE))
})

test_that("varlist() returns a data.frame (tibble)", {
  result <- varlist(mtcars, tbl = TRUE)
  expect_s3_class(result, "data.frame")
  expect_s3_class(result, "tbl_df")
})

test_that("varlist()$Variable returns a character vector", {
  expect_type(varlist(mtcars, tbl = TRUE)$Variable, "character")
})

test_that("varlist() returns correct column names", {
  expect_named(
    varlist(mtcars, tbl = TRUE),
    c("Variable", "Label", "Values", "Class", "N_distinct", "N_valid", "NAs")
  )
})

test_that("varlist() works with tidyselect selectors", {
  result <- varlist(iris, starts_with("Sepal"), tbl = TRUE)
  expect_true(all(grepl("^Sepal", result$Variable)))
})

test_that("varlist() supports tidyselect helpers with local objects", {
  selected <- "Sepal.Width"
  result <- varlist(iris, all_of(selected), tbl = TRUE)
  expect_equal(result$Variable, selected)
})

test_that("varlist() errors on renamed tidyselect selections", {
  df <- data.frame(old = 1:3)

  expect_error(
    varlist(df, new = old, tbl = TRUE),
    "`...` can select columns but cannot rename them in varlist\\(\\)"
  )
})

test_that("varlist() errors on renamed tidyselect helper selections", {
  selected <- c(new = "old")
  df <- data.frame(old = 1:3)

  expect_error(
    varlist(df, all_of(selected), tbl = TRUE),
    "`...` can select columns but cannot rename them in varlist\\(\\)"
  )
})

test_that("varlist() does not expose internal raw expression argument", {
  expect_false(any(c("raw_expr", ".raw_expr") %in% names(formals(varlist))))
  expect_false(any(c("raw_expr", ".raw_expr") %in% names(formals(vl))))
})

test_that("vl() is an alias of varlist()", {
  expect_equal(vl(mtcars, tbl = TRUE), varlist(mtcars, tbl = TRUE))
})

test_that("varlist() throws error for non-data.frame input", {
  expect_error(varlist(1:10), "only works with named data frames")
})

test_that("varlist() validates missing column names", {
  df <- structure(
    list(1:3),
    class = "data.frame",
    row.names = c(NA_integer_, -3L)
  )

  expect_error(varlist(df, tbl = TRUE), "`x` must have column names")
})

test_that("varlist() validates NA column names", {
  df <- data.frame(x = 1:3)
  names(df) <- NA_character_

  expect_error(varlist(df, tbl = TRUE), "`x` must have non-empty column names")
})

test_that("varlist() validates empty column names", {
  df <- data.frame(x = 1:3)
  names(df) <- ""

  expect_error(varlist(df, tbl = TRUE), "`x` must have non-empty column names")
})

test_that("varlist() validates duplicate column names", {
  df <- data.frame(x = 1:3, y = 4:6)
  names(df) <- c("x", "x")

  expect_error(varlist(df, tbl = TRUE), "`x` must have unique column names")
})

test_that("varlist() validates logical arguments", {
  expect_error(varlist(mtcars, values = NA), "`values` must be TRUE or FALSE")
  expect_error(
    varlist(mtcars, values = c(TRUE, FALSE)),
    "`values` must be TRUE or FALSE"
  )
  expect_error(varlist(mtcars, tbl = NA), "`tbl` must be TRUE or FALSE")
  expect_error(
    varlist(mtcars, include_na = NA),
    "`include_na` must be TRUE or FALSE"
  )
  expect_error(vl(mtcars, values = NA), "`values` must be TRUE or FALSE")
})

test_that("varlist() validates factor_levels", {
  expect_error(
    varlist(mtcars, factor_levels = "bad"),
    '`factor_levels` must be "observed" or "all"'
  )
  expect_error(
    varlist(mtcars, factor_levels = NA_character_),
    '`factor_levels` must be "observed" or "all"'
  )
  expect_error(
    varlist(mtcars, factor_levels = c("observed", "bad")),
    '`factor_levels` must be "observed" or "all"'
  )
  expect_error(
    vl(mtcars, factor_levels = "bad"),
    '`factor_levels` must be "observed" or "all"'
  )
})

test_that("varlist_title() returns vl: name for simple object", {
  dummy_expr <- quote(iris)
  expect_equal(varlist_title(dummy_expr), "vl: iris")
})

test_that("varlist_title() adds * for transformed object", {
  transformed_expr <- quote(head(iris))
  expect_equal(varlist_title(transformed_expr), "vl: iris*")
})

test_that("varlist_title() adds * when selectors are used", {
  expect_equal(varlist_title(quote(iris), selectors_used = TRUE), "vl: iris*")
})

test_that("varlist_title() handles nested calls", {
  expect_equal(varlist_title(quote(dplyr::filter(df, x > 1))), "vl: df*")
})

test_that("varlist_title() extracts symbol from nested call arg", {
  expect_equal(varlist_title(quote(fun(subset(df, x > 1)))), "vl: df*")
})

test_that("varlist_title() handles extraction calls", {
  expect_equal(varlist_title(quote(df[, 1:3])), "vl: df*")
  expect_equal(varlist_title(quote(df[["x"]])), "vl: df*")
  expect_equal(varlist_title(quote(df$x)), "vl: df*")
})

test_that("varlist_title() handles literal lookup calls", {
  expect_equal(varlist_title(quote(get("df"))), "vl: df")
  expect_equal(varlist_title(quote(base::get("df"))), "vl: df")
  expect_equal(
    varlist_title(quote(get("df")), selectors_used = TRUE),
    "vl: df*"
  )
})

test_that("varlist_title() handles magrittr-style pipe calls", {
  expect_equal(varlist_title(quote(df %>% head())), "vl: df*")
})

test_that("varlist_title() avoids ambiguous lookup and call arguments", {
  expect_equal(varlist_title(quote(get(name))), "vl: <data>")
  expect_equal(varlist_title(quote(fun(x, df))), "vl: <data>")
})

test_that("varlist_title() returns fallback for non-symbol expressions", {
  expect_equal(varlist_title(1L), "vl: <data>")
})

test_that("varlist_title() returns fallback for unusual objects", {
  bad <- structure(list(), class = "weird")
  expect_equal(varlist_title(bad), "vl: <data>")
})

test_that("varlist() warns when no columns match", {
  expect_warning(
    varlist(mtcars, starts_with("zzz"), tbl = TRUE),
    "No columns selected"
  )
})

test_that("varlist() returns empty tibble when no columns match", {
  res <- suppressWarnings(varlist(mtcars, starts_with("zzz"), tbl = TRUE))
  expect_equal(nrow(res), 0L)
  expect_named(
    res,
    c("Variable", "Label", "Values", "Class", "N_distinct", "N_valid", "NAs")
  )
})

test_that("varlist() non-interactive with no tbl returns message", {
  expect_message(
    varlist(mtcars),
    "Non-interactive session"
  )
})

test_that("varlist() counts NAs correctly", {
  df <- data.frame(a = c(1, NA, 3), b = c("x", "y", NA))
  res <- varlist(df, tbl = TRUE)
  expect_equal(unname(res$NAs), c(1L, 1L))
  expect_equal(unname(res$N_valid), c(2L, 2L))
})

test_that("varlist() shows labels when available", {
  df <- data.frame(x = 1:3)
  attr(df$x, "label") <- "My label"
  res <- varlist(df, tbl = TRUE)
  expect_equal(unname(res$Label), "My label")
})

test_that("varlist() returns NA label when none set", {
  res <- varlist(data.frame(x = 1:3), tbl = TRUE)
  expect_true(is.na(res$Label))
})

test_that("varlist() values = TRUE shows all unique values", {
  df <- data.frame(x = c(1, 2, 3, 4, 5, 6, 7))
  res <- varlist(df, tbl = TRUE, values = TRUE)
  expect_equal(unname(res$Values), "1, 2, 3, 4, 5, 6, 7")
})

test_that("varlist() values = FALSE truncates >4 unique values", {
  df <- data.frame(x = c(1, 2, 3, 4, 5, 6, 7))
  res <- varlist(df, tbl = TRUE, values = FALSE)
  expect_match(res$Values, "\\.\\.\\.")
})

test_that("varlist() values = FALSE shows all when <= 4 values", {
  df <- data.frame(x = c(1, 2, 3))
  res <- varlist(df, tbl = TRUE, values = FALSE)
  expect_equal(unname(res$Values), "1, 2, 3")
})

test_that("varlist() preserves literal NA and empty string values", {
  df <- data.frame(x = c("", "NA", "b"), stringsAsFactors = FALSE)

  res <- varlist(df, tbl = TRUE)
  res_all <- varlist(df, tbl = TRUE, values = TRUE)

  expect_equal(unname(res$Values), "\"\", \"NA\", b")
  expect_equal(unname(res_all$Values), "\"\", \"NA\", b")
  expect_equal(unname(res$N_distinct), 3L)
})

test_that("varlist() include_na appends NA to values", {
  df <- data.frame(x = c(1, 2, NA))
  res <- varlist(df, tbl = TRUE, include_na = TRUE)
  expect_equal(unname(res$Values), "1, 2, <NA>")
})

test_that("varlist() include_na appends NaN for numeric", {
  df <- data.frame(x = c(1, 2, NaN))
  res <- varlist(df, tbl = TRUE, include_na = TRUE)
  expect_equal(unname(res$Values), "1, 2, <NaN>")
})

test_that("varlist() include_na distinguishes NA from NaN", {
  nan_only <- data.frame(x = c(1, NaN, 3))
  both <- data.frame(x = c(1, NA, NaN, 3))

  res_nan_only <- varlist(nan_only, tbl = TRUE, include_na = TRUE)
  res_both <- varlist(both, tbl = TRUE, include_na = TRUE)

  expect_equal(unname(res_nan_only$Values), "1, 3, <NaN>")
  expect_equal(unname(res_both$Values), "1, 3, <NA>, <NaN>")
  expect_equal(unname(res_both$NAs), 2L)
  expect_equal(unname(res_both$N_valid), 2L)
})

test_that("varlist() distinguishes literal NA strings from missing values", {
  df <- data.frame(x = c("NA", NA), stringsAsFactors = FALSE)
  res <- varlist(df, tbl = TRUE, include_na = TRUE)
  res_all <- varlist(df, tbl = TRUE, values = TRUE, include_na = TRUE)

  expect_equal(unname(res$Values), "\"NA\", <NA>")
  expect_equal(unname(res_all$Values), "\"NA\", <NA>")
})

test_that("varlist() quotes literal NaN strings", {
  df <- data.frame(x = c("NaN", NA), stringsAsFactors = FALSE)
  res <- varlist(df, tbl = TRUE, include_na = TRUE)

  expect_equal(unname(res$Values), "\"NaN\", <NA>")
})

test_that("varlist() distinguishes factor NA levels from missing values", {
  f <- factor(c("NA", NA), levels = "NA")
  res <- varlist(data.frame(x = f), tbl = TRUE, include_na = TRUE)

  expect_equal(unname(res$Values), "\"NA\", <NA>")
})

test_that("summarize_values_minmax handles factors", {
  f <- factor(c("a", "b", "c"), levels = c("c", "b", "a"))
  res <- varlist(data.frame(x = f), tbl = TRUE)
  expect_equal(unname(res$Values), "c, b, a")
})

test_that("summarize_values_minmax handles Date columns", {
  df <- data.frame(d = as.Date(c("2024-01-01", "2024-06-15", "2024-12-31")))
  res <- varlist(df, tbl = TRUE)
  expect_match(res$Values, "2024")
})

test_that("summarize_values_minmax handles POSIXct columns", {
  df <- data.frame(t = as.POSIXct(c("2024-01-01 10:00", "2024-06-15 12:00")))
  res <- varlist(df, tbl = TRUE)
  expect_match(res$Values, "2024")
})

test_that("varlist() summarizes matrix columns by rows", {
  mat <- rbind(c(1, 2), c(3, 4))
  df <- data.frame(x = I(mat))

  res <- varlist(df, tbl = TRUE)

  expect_equal(unname(res$Values), "Matrix(2 x 2)")
  expect_equal(unname(res$N_distinct), 2L)
  expect_equal(unname(res$N_valid), 2L)
  expect_equal(unname(res$NAs), 0L)
})

test_that("varlist() counts matrix rows with missing cells as missing", {
  mat <- rbind(c(1, 2), c(NA_real_, 4))
  df <- data.frame(x = I(mat))

  res <- varlist(df, tbl = TRUE, include_na = TRUE)

  expect_equal(unname(res$Values), "Matrix(2 x 2), <NA>")
  expect_equal(unname(res$N_distinct), 1L)
  expect_equal(unname(res$N_valid), 1L)
  expect_equal(unname(res$NAs), 1L)
})

test_that("varlist() counts distinct matrix rows", {
  mat <- rbind(c(1, 2), c(1, 2), c(3, 4))
  df <- data.frame(x = I(mat))

  res <- varlist(df, tbl = TRUE)

  expect_equal(unname(res$N_distinct), 2L)
  expect_equal(unname(res$N_valid), 3L)
})

test_that("varlist() values = TRUE keeps matrix columns structural", {
  mat <- rbind(c(1, 2), c(3, 4))
  df <- data.frame(x = I(mat))

  res <- varlist(df, tbl = TRUE, values = TRUE)

  expect_equal(unname(res$Values), "Matrix(2 x 2)")
})

test_that("varlist() summarizes array columns by dimensions", {
  arr <- array(1:8, dim = c(2, 2, 2))
  df <- tibble::tibble(x = I(arr))

  res <- varlist(df, tbl = TRUE)

  expect_equal(unname(res$Values), "Array(2 x 2 x 2)")
  expect_equal(unname(res$N_distinct), 2L)
  expect_equal(unname(res$N_valid), 2L)
})

test_that("summarize_values_minmax handles list columns", {
  df <- tibble::tibble(x = list(1:3, "a", TRUE))
  res <- varlist(df, tbl = TRUE)
  expect_match(res$Values, "List\\(3\\)")
})

test_that("summarize_values_all handles list columns", {
  df <- tibble::tibble(x = list(1:3, "a"))
  res <- varlist(df, tbl = TRUE, values = TRUE)
  expect_match(res$Values, "List\\(2\\)")
})

test_that("varlist() minmax list-column shows only structure", {
  df <- tibble::tibble(x = list(1L, 2L, 3L))
  res <- varlist(df, tbl = TRUE)
  expect_equal(unname(res$Values), "List(3)")
})

test_that("varlist() values=TRUE deduplicates list-column element types", {
  df <- tibble::tibble(x = list(1L, 2L, 3L, "a"))
  res <- varlist(df, tbl = TRUE, values = TRUE)
  expect_equal(unname(res$Values), "List(4): character, integer")
})

test_that("varlist() values=TRUE handles empty list-column without orphan colon", {
  df <- tibble::tibble(x = I(list()))
  res <- varlist(df, tbl = TRUE, values = TRUE)
  expect_equal(unname(res$Values), "List(0)")
})

test_that("varlist() minmax list-column appends <NA> when include_na = TRUE", {
  df <- tibble::tibble(x = list(1, NA, "x"))
  res <- varlist(df, tbl = TRUE, include_na = TRUE)
  expect_equal(unname(res$Values), "List(3), <NA>")
})

test_that("varlist() values=TRUE list-column appends <NA> when include_na = TRUE", {
  df <- tibble::tibble(x = list(1, NA, "x"))
  res <- varlist(df, tbl = TRUE, values = TRUE, include_na = TRUE)
  expect_equal(
    unname(res$Values),
    "List(3): character, double, logical, <NA>"
  )
})

test_that("varlist() does not append <NA> for list-column without NA", {
  df <- tibble::tibble(x = list(1L, 2L, 3L))
  res <- varlist(df, tbl = TRUE, include_na = TRUE)
  expect_equal(unname(res$Values), "List(3)")
})

test_that("summarize_values_all handles logical columns", {
  df <- data.frame(x = c(TRUE, FALSE, TRUE))
  res <- varlist(df, tbl = TRUE, values = TRUE)
  expect_match(res$Values, "FALSE")
  expect_match(res$Values, "TRUE")
})

test_that("summarize_values_all handles character columns", {
  df <- data.frame(x = c("b", "a", "c"), stringsAsFactors = FALSE)
  res <- varlist(df, tbl = TRUE, values = TRUE)
  expect_equal(unname(res$Values), "a, b, c")
})

test_that("summarize_values_all handles factors with values = TRUE", {
  f <- factor(c("x", "y"), levels = c("y", "x"))
  res <- varlist(data.frame(v = f), tbl = TRUE, values = TRUE)
  expect_equal(unname(res$Values), "y, x")
})

test_that("varlist() displays observed factor levels by default", {
  f <- factor(c("x", "x"), levels = c("x", "y"))
  res <- varlist(data.frame(v = f), tbl = TRUE)
  res_all <- varlist(data.frame(v = f), tbl = TRUE, values = TRUE)

  expect_equal(unname(res$Values), "x")
  expect_equal(unname(res_all$Values), "x")
  expect_equal(unname(res$N_distinct), 1L)
})

test_that("varlist() can display all factor levels", {
  f <- factor(c("x", "x"), levels = c("x", "y"))
  res <- varlist(data.frame(v = f), tbl = TRUE, factor_levels = "all")
  res_all <- varlist(
    data.frame(v = f),
    tbl = TRUE,
    values = TRUE,
    factor_levels = "all"
  )

  expect_equal(unname(res$Values), "x, y")
  expect_equal(unname(res_all$Values), "x, y")
  expect_equal(unname(res$N_distinct), 1L)
})

test_that("summarize_values_minmax handles all-NA column", {
  df <- data.frame(x = c(NA_real_, NA_real_))
  res <- varlist(df, tbl = TRUE)
  expect_equal(unname(res$Values), "")
  expect_equal(unname(res$N_distinct), 0L)
})

test_that("summarize_values_minmax include_na with empty non-NA values", {
  df <- data.frame(x = c(NA_real_, NA_real_))
  res <- varlist(df, tbl = TRUE, include_na = TRUE)
  expect_equal(unname(res$Values), "<NA>")
})

test_that("varlist() handles labelled columns", {
  skip_if_not_installed("labelled")
  x <- labelled::labelled(c(1, 2, 3), labels = c(Low = 1, Mid = 2, High = 3))
  df <- data.frame(v = x)
  res <- varlist(df, tbl = TRUE)
  expect_match(res$Values, "Low")
  expect_match(res$Class, "labelled")
})

test_that("varlist() handles labelled columns with values = TRUE", {
  skip_if_not_installed("labelled")
  x <- labelled::labelled(c(1, 2, 3), labels = c(Low = 1, Mid = 2, High = 3))
  df <- data.frame(v = x)
  res <- varlist(df, tbl = TRUE, values = TRUE)
  expect_match(res$Values, "Low")
})

test_that("varlist() uses labelled value order consistently", {
  skip_if_not_installed("labelled")
  x <- labelled::labelled(c(2, 1, 3), labels = c(Low = 1, Mid = 2, High = 3))
  df <- data.frame(v = x)

  res <- varlist(df, tbl = TRUE)
  res_all <- varlist(df, tbl = TRUE, values = TRUE)

  expect_equal(unname(res$Values), "[1] Low, [2] Mid, [3] High")
  expect_equal(unname(res_all$Values), "[1] Low, [2] Mid, [3] High")
})

test_that("N_distinct counts correctly", {
  df <- data.frame(x = c(1, 1, 2, NA, 2))
  res <- varlist(df, tbl = TRUE)
  expect_equal(unname(res$N_distinct), 2L)
})

test_that("varlist() empty selection returns empty data.frame with tbl", {
  df <- data.frame(x = 1:3, y = 4:6)
  res <- suppressWarnings(varlist(df, starts_with("z"), tbl = TRUE))
  expect_equal(nrow(res), 0L)
})

test_that("varlist() non-interactive empty selection prints message", {
  df <- data.frame(x = 1:3)
  expect_message(
    suppressWarnings(varlist(df, starts_with("z"))),
    "No columns selected"
  )
})

test_that("varlist() non-interactive prints message", {
  expect_message(
    varlist(mtcars),
    "Non-interactive"
  )
})

test_that("varlist() values=TRUE with NaN shows NaN", {
  df <- data.frame(x = c(1, NaN, 3))
  res <- varlist(df, tbl = TRUE, values = TRUE, include_na = TRUE)
  expect_equal(unname(res$Values), "1, 3, <NaN>")
})

test_that("varlist() warns and marks the cell when a column cannot be summarized", {
  local_mocked_bindings(
    summarize_values_minmax = function(...) stop("synthetic failure"),
    .package = "spicy"
  )

  df <- data.frame(x = 1:3, y = letters[1:3])

  warnings <- testthat::capture_warnings(
    res <- varlist(df, tbl = TRUE)
  )

  expect_length(warnings, 2L)
  expect_match(warnings[[1]], "Could not summarize column `x`.*synthetic failure")
  expect_match(warnings[[2]], "Could not summarize column `y`.*synthetic failure")
  expect_equal(
    unname(res$Values),
    rep("<error: synthetic failure>", 2L)
  )
})

test_that("varlist() isolates per-column failures", {
  call_count <- 0L
  local_mocked_bindings(
    summarize_values_minmax = function(col, ...) {
      call_count <<- call_count + 1L
      if (call_count == 1L) {
        stop("only x fails")
      }
      "fine"
    },
    .package = "spicy"
  )

  df <- data.frame(x = 1:3, y = letters[1:3])

  suppressWarnings(res <- varlist(df, tbl = TRUE))

  expect_equal(unname(res$Values[[1]]), "<error: only x fails>")
  expect_equal(unname(res$Values[[2]]), "fine")
})

test_that("varlist() error handling is symmetric across values modes", {
  local_mocked_bindings(
    summarize_values_all = function(...) stop("values=TRUE failure"),
    .package = "spicy"
  )

  expect_warning(
    res <- varlist(data.frame(x = 1:3), tbl = TRUE, values = TRUE),
    "Could not summarize column `x`.*values=TRUE failure"
  )
  expect_equal(unname(res$Values[[1]]), "<error: values=TRUE failure>")
})

test_that("varlist() collapses multi-line error messages onto a single cell", {
  local_mocked_bindings(
    summarize_values_minmax = function(...) stop("line one\n  line two"),
    .package = "spicy"
  )

  expect_warning(
    res <- varlist(data.frame(x = 1:3), tbl = TRUE),
    "Could not summarize column `x`"
  )
  expect_equal(unname(res$Values[[1]]), "<error: line one line two>")
})

test_that("varlist() handles zero-row data frames across column types", {
  df <- tibble::tibble(
    int = integer(0),
    chr = character(0),
    fac = factor(character(0), levels = c("x", "y", "z")),
    dat = as.Date(character(0)),
    lst = list()
  )

  res <- varlist(df, tbl = TRUE)

  expect_equal(nrow(res), 5L)
  expect_equal(res$Variable, c("int", "chr", "fac", "dat", "lst"))
  expect_equal(unname(res$N_valid), rep(0L, 5L))
  expect_equal(unname(res$NAs), rep(0L, 5L))
  expect_equal(unname(res$N_distinct), rep(0L, 5L))
  expect_equal(unname(res$Values), c("", "", "", "", "List(0)"))
})

test_that("varlist() factor_levels = 'all' shows declared levels even on zero rows", {
  df <- tibble::tibble(
    f = factor(character(0), levels = c("x", "y"))
  )

  res_obs <- varlist(df, tbl = TRUE, values = TRUE)
  res_all <- varlist(df, tbl = TRUE, values = TRUE, factor_levels = "all")

  expect_equal(unname(res_obs$Values), "")
  expect_equal(unname(res_all$Values), "x, y")
})

test_that("varlist() combines factor_levels = 'all' with include_na", {
  f <- factor(c("a", NA), levels = c("a", "b", "c"))
  df <- data.frame(v = f)

  res_minmax <- varlist(
    df,
    tbl = TRUE,
    factor_levels = "all",
    include_na = TRUE
  )
  res_all <- varlist(
    df,
    tbl = TRUE,
    values = TRUE,
    factor_levels = "all",
    include_na = TRUE
  )

  expect_equal(unname(res_minmax$Values), "a, b, c, <NA>")
  expect_equal(unname(res_all$Values), "a, b, c, <NA>")
  expect_equal(unname(res_minmax$NAs), 1L)
  expect_equal(unname(res_minmax$N_distinct), 1L)
})

test_that("vl() works end-to-end through a base pipe chain", {
  res_pipe <- mtcars |> head(5) |> vl(tbl = TRUE)
  res_direct <- vl(head(mtcars, 5), tbl = TRUE)

  expect_equal(res_pipe, res_direct)
  expect_equal(nrow(res_pipe), ncol(mtcars))
  expect_equal(unname(res_pipe$N_valid), rep(5L, ncol(mtcars)))
  expect_equal(unname(res_pipe$NAs), rep(0L, ncol(mtcars)))
})

summarize_varlist_column <- function(
  col,
  name,
  values = FALSE,
  include_na = FALSE,
  factor_levels = "observed"
) {
  tryCatch(
    if (values) {
      summarize_values_all(
        col,
        include_na = include_na,
        factor_levels = factor_levels
      )
    } else {
      summarize_values_minmax(
        col,
        include_na = include_na,
        factor_levels = factor_levels
      )
    },
    error = function(e) {
      msg <- trimws(gsub("\\s+", " ", conditionMessage(e)))
      warning(
        "Could not summarize column `",
        name,
        "`: ",
        msg,
        call. = FALSE
      )
      paste0("<error: ", msg, ">")
    }
  )
}


summarize_values_minmax <- function(
  col,
  include_na = FALSE,
  factor_levels = "observed"
) {
  has_na <- varlist_has_na(col)
  has_nan <- varlist_has_nan(col)
  max_display <- 4

  if (labelled::is.labelled(col)) {
    col <- labelled::to_factor(col, levels = "prefixed")
    unique_vals <- factor_values(col, factor_levels = "observed")
  } else if (is.factor(col)) {
    unique_vals <- factor_values(col, factor_levels = factor_levels)
  } else if (inherits(col, c("Date", "POSIXct", "POSIXlt"))) {
    col_no_na <- stats::na.omit(col)
    unique_vals <- sort(unique(col_no_na))
  } else if (varlist_is_array_column(col)) {
    return(summarize_varlist_array(col, include_na = include_na))
  } else if (is.list(col)) {
    return(summarize_varlist_list(
      col,
      values = FALSE,
      include_na = include_na
    ))
  } else {
    col_no_na <- stats::na.omit(col)
    unique_vals <- sort(unique(col_no_na))
  }

  unique_vals <- format_varlist_values(unique_vals)
  vals_chr_clean <- unique_vals[!is.na(unique_vals)]

  if (length(vals_chr_clean) == 0) {
    val_str <- ""
  } else if (length(vals_chr_clean) <= max_display) {
    val_str <- paste(vals_chr_clean, collapse = ", ")
  } else {
    val_str <- paste(
      c(vals_chr_clean[seq_len(3)], "...", utils::tail(vals_chr_clean, 1)),
      collapse = ", "
    )
  }

  extras <- format_varlist_missing_values(has_na, has_nan, include_na)

  if (length(extras)) {
    if (nzchar(val_str)) {
      return(paste(val_str, paste(extras, collapse = ", "), sep = ", "))
    } else {
      return(paste(extras, collapse = ", "))
    }
  }

  val_str
}


summarize_values_all <- function(
  col,
  include_na = FALSE,
  factor_levels = "observed"
) {
  na_omit_col <- stats::na.omit(col)
  has_na <- varlist_has_na(col)
  has_nan <- varlist_has_nan(col)

  show_vals <- function(v, sort_values = TRUE) {
    vals <- tryCatch(
      {
        vals <- unique(v)
        if (sort_values) {
          sort(vals)
        } else {
          vals
        }
      },
      error = function(e) {
        "Error: invalid values"
      }
    )

    vals_chr <- format_varlist_values(vals)

    vals_chr_clean <- vals_chr[!is.na(vals_chr)]

    extras <- format_varlist_missing_values(has_na, has_nan, include_na)

    all_vals <- c(vals_chr_clean, extras)

    paste(all_vals, collapse = ", ")
  }

  if (labelled::is.labelled(col)) {
    col <- labelled::to_factor(col, levels = "prefixed")
    return(show_vals(col))
  }

  if (is.factor(col)) {
    return(show_vals(
      factor_values(col, factor_levels = factor_levels),
      sort_values = FALSE
    ))
  }

  if (is.logical(col) || is.character(col)) {
    return(show_vals(na_omit_col))
  }

  if (varlist_is_array_column(col)) {
    return(summarize_varlist_array(col, include_na = include_na))
  }

  if (is.list(col)) {
    return(summarize_varlist_list(
      col,
      values = TRUE,
      include_na = include_na
    ))
  }

  show_vals(na_omit_col)
}


format_varlist_values <- function(x) {
  values <- as.character(x)
  quote_values <- !is.na(values) & values %in% c("", "NA", "NaN")
  values[quote_values] <- paste0("\"", values[quote_values], "\"")
  values
}


format_varlist_missing_values <- function(has_na, has_nan, include_na) {
  if (!include_na) {
    return(character())
  }

  extras <- character()
  if (has_na) {
    extras <- c(extras, "<NA>")
  }
  if (has_nan) {
    extras <- c(extras, "<NaN>")
  }

  extras
}


summarize_varlist_array <- function(col, include_na = FALSE) {
  summary <- paste0(
    if (is.matrix(col)) "Matrix" else "Array",
    "(",
    paste(dim(col), collapse = " x "),
    ")"
  )
  extras <- format_varlist_missing_values(
    varlist_has_na(col),
    varlist_has_nan(col),
    include_na
  )

  paste(c(summary, extras), collapse = ", ")
}


summarize_varlist_list <- function(col, values = FALSE, include_na = FALSE) {
  base <- paste0("List(", length(col), ")")

  if (values && length(col) > 0L) {
    types <- sort(unique(vapply(col, typeof, character(1))))
    base <- paste0(base, ": ", paste(types, collapse = ", "))
  }

  extras <- format_varlist_missing_values(
    varlist_has_na(col),
    varlist_has_nan(col),
    include_na
  )

  if (length(extras)) {
    return(paste(c(base, extras), collapse = ", "))
  }

  base
}


varlist_n_distinct <- function(col) {
  if (varlist_is_array_column(col)) {
    rows <- varlist_array_rows(col)
    valid <- !varlist_array_missing_rows(col)

    if (!any(valid)) {
      return(0L)
    }

    return(nrow(unique(rows[valid, , drop = FALSE])))
  }

  length(unique(stats::na.omit(col)))
}


varlist_n_valid <- function(col) {
  if (varlist_is_array_column(col)) {
    return(sum(!varlist_array_missing_rows(col)))
  }

  sum(!is.na(col))
}


varlist_n_missing <- function(col) {
  if (varlist_is_array_column(col)) {
    return(sum(varlist_array_missing_rows(col)))
  }

  sum(is.na(col))
}


varlist_is_array_column <- function(x) {
  is.array(x) && length(dim(x)) >= 2L
}


varlist_array_rows <- function(x) {
  matrix(x, nrow = dim(x)[[1L]])
}


varlist_array_missing_rows <- function(x) {
  rowSums(varlist_array_rows(is.na(x))) > 0L
}


varlist_has_na <- function(x) {
  missing <- is.na(x)
  any(missing & !varlist_is_nan(x))
}


varlist_has_nan <- function(x) {
  any(varlist_is_nan(x))
}


varlist_is_nan <- function(x) {
  if (!is.numeric(x)) {
    return(rep(FALSE, length(x)))
  }

  is.nan(x)
}


factor_values <- function(col, factor_levels = "observed") {
  if (identical(factor_levels, "all")) {
    return(levels(col))
  }

  observed <- stats::na.omit(col)
  levels(col)[levels(col) %in% as.character(observed)]
}

#' Spicy Table Engine: Frequency and Cross-tabulation Rendering
#'
#' @description
#' The *spicy table engine* provides a cohesive set of tools for creating and
#' printing formatted ASCII tables in R, designed for descriptive statistics.
#'
#' Functions in this family include:
#' * [freq()] — frequency tables with support for weights, labelled data, and cumulative percentages
#' * [spicy_print_table()] — general-purpose ASCII table printer
#' * [build_ascii_table()] — internal rendering engine for column alignment and formatting
#'
#' @details
#' All functions in this family share a common philosophy:
#' * Console-friendly display with Unicode box-drawing characters
#' * Consistent alignment and spacing across outputs
#' * Automatic detection of variable type (`factor`, `labelled`, `numeric`)
#' * Optional integration of variable labels and weighting information
#'
#' @section Core functions:
#' - **`freq()`** — Main entry point for generating frequency tables.
#' - **`spicy_print_table()`** — Applies formatting and optional titles or notes.
#' - **`build_ascii_table()`** — Internal engine handling padding, alignment, and box rules.
#'
#' @section Output styling:
#' The spicy table engine supports multiple padding options via `padding`:
#' `"compact"` (default), `"normal"`, and `"wide"`.
#' Horizontal and vertical rules can be customized, and colors are supported
#' when the terminal allows ANSI color output (via the **crayon** package).
#'
#' @seealso
#' [print.spicy_freq_table()] for the specialized frequency display method.
#' [labelled::to_factor()] and [dplyr::pull()] for data transformations.
#'
#' @family spicy tables
#' @keywords tables descriptive frequency spicy
#' @keywords internal
#' @name spicy_tables
NULL

#' Continuous summary table
#'
#' @description
#' Computes descriptive statistics (mean, SD, min, max, confidence interval
#' of the mean, *n*) for one or many continuous variables selected with
#' tidyselect syntax.
#'
#' With `by`, produces grouped summaries and reports a group-comparison
#' *p*-value by default (Welch test; change via `test`). Additional
#' inferential output is opt-in: test statistics (`statistic`) and
#' effect sizes (`effect_size` / `effect_size_ci`). Set `p_value = FALSE`
#' to suppress the *p*-value column. Without `by`, produces one-way
#' descriptive summaries.
#'
#' Multiple output formats are available via `output`: a printed ASCII
#' table (`"default"`), a plain numeric `data.frame` (`"data.frame"`), or
#' publication-ready tables (`"tinytable"`, `"gt"`, `"flextable"`,
#' `"excel"`, `"clipboard"`, `"word"`).
#'
#' @param data A `data.frame`.
#' @param select Columns to include. If `regex = FALSE`, use tidyselect
#'   syntax or a character vector of column names (default:
#'   `tidyselect::everything()`). If `regex = TRUE`, provide a regular
#'   expression pattern (character string).
#' @param by Optional grouping column. Accepts an unquoted column name or
#'   a single character column name. The column does not need to be
#'   numeric.
#' @param exclude Columns to exclude. Supports tidyselect syntax and
#'   character vectors of column names.
#' @param regex Logical. If `FALSE` (the default), uses tidyselect
#'   helpers. If `TRUE`, the `select` argument is treated as a regular
#'   expression.
#' @param test Character. Statistical test to use when comparing groups.
#'   One of `"welch"` (default), `"student"`, or `"nonparametric"`.
#'   - `"welch"`: Welch *t*-test (2 groups) or Welch one-way ANOVA
#'     (3+ groups). Does not assume equal variances.
#'   - `"student"`: Student *t*-test (2 groups) or classic one-way
#'     ANOVA (3+ groups). Assumes equal variances.
#'   - `"nonparametric"`: Wilcoxon rank-sum / Mann--Whitney *U*
#'     (2 groups) or Kruskal--Wallis *H* (3+ groups).
#'
#'   Used whenever `by` is supplied (since `p_value` defaults to `TRUE`
#'   in that case) or when `statistic = TRUE` / `effect_size = TRUE`.
#'   Ignored when `by` is not used, or when all three display toggles
#'   are turned off.
#' @param p_value Logical or `NULL`. If `TRUE` and `by` is used, adds a
#'   *p*-value column from the test specified by `test`. When `NULL` (the
#'   default), the *p*-value is shown automatically whenever `by` is
#'   supplied, and hidden otherwise. Pass `p_value = FALSE` to suppress
#'   the column explicitly. Ignored when `by` is not used.
#' @param statistic Logical. If `TRUE` and `by` is used, the test
#'   statistic is shown in an additional column (e.g.,
#'   `t(df) = ...`, `F(df1, df2) = ...`, `W = ...`, or `H(df) = ...`).
#'   Both `p_value` and `statistic` are independent; either or both
#'   can be enabled. Defaults to `FALSE`. Ignored when `by` is not
#'   used.
#' @param effect_size Logical. If `TRUE` and `by` is used, adds an
#'   effect-size column ("ES"). The measure is chosen automatically:
#'   - Hedges' *g* (bias-corrected) - 2 groups, parametric (CI via
#'     Hedges & Olkin approximation).
#'   - Eta-squared (\eqn{\eta^2}) - 3+ groups, parametric (CI via
#'     noncentral *F* distribution).
#'   - Rank-biserial *r* (`r_rb`) - 2 groups, nonparametric (CI via
#'     Fisher *z*-transform).
#'   - Epsilon-squared (\eqn{\varepsilon^2}) - 3+ groups,
#'     nonparametric (CI via percentile bootstrap, 2 000 replicates).
#'
#'   Defaults to `FALSE`. Ignored when `by` is not used.
#' @param effect_size_ci Logical. If `TRUE`, appends the confidence
#'   interval of the effect size in brackets (e.g.,
#'   `g = 0.45 [0.22, 0.68]`). Implies `effect_size = TRUE`.
#'   Defaults to `FALSE`.
#' @param labels An optional named character vector of variable labels.
#'   Names must match column names in `data`. When `NULL` (the default),
#'   labels are auto-detected from variable attributes (e.g., haven
#'   labels); if none are found, the column name is used.
#' @param ci_level Confidence level for the mean confidence interval
#'   (default: `0.95`). Must be between 0 and 1 exclusive.
#' @param digits Number of decimal places for descriptive values and test
#'   statistics (default: `2`).
#' @param effect_size_digits Number of decimal places for effect-size values
#'   in formatted displays (default: `2`).
#' @param decimal_mark Character used as decimal separator.
#'   Either `"."` (default) or `","`.
#' @param output Output format. One of:
#'   - `"default"` (a printed ASCII table, returned invisibly)
#'   - `"data.frame"` (a plain numeric `data.frame`)
#'   - `"tinytable"` (requires `tinytable`)
#'   - `"gt"` (requires `gt`)
#'   - `"flextable"` (requires `flextable`)
#'   - `"excel"` (requires `openxlsx2`)
#'   - `"clipboard"` (requires `clipr`)
#'   - `"word"` (requires `flextable` and `officer`)
#' @param excel_path File path for `output = "excel"`.
#' @param excel_sheet Sheet name for `output = "excel"`
#'   (default: `"Descriptives"`).
#' @param clipboard_delim Delimiter for `output = "clipboard"`
#'   (default: `"\t"`).
#' @param word_path File path for `output = "word"`.
#' @param verbose Logical. If `TRUE`, prints messages about excluded
#'   non-numeric columns (default: `FALSE`).
#'
#' @return Depends on `output`:
#' \itemize{
#'   \item `"default"`: prints a styled ASCII table and returns the
#'     underlying `data.frame` invisibly (S3 class
#'     `"spicy_continuous_table"`).
#'   \item `"data.frame"`: a plain `data.frame` with columns
#'     `variable`, `label`, `group` (when `by` is used), `mean`, `sd`,
#'     `min`, `max`, `ci_lower`, `ci_upper`, `n`.
#'     When `by` is used together with `p_value = TRUE`,
#'     `statistic = TRUE`, or `effect_size = TRUE`, additional columns
#'     are appended (populated on the first row of each variable block
#'     only):
#'     \itemize{
#'       \item `test_type` -- test identifier (e.g., `"welch_t"`,
#'         `"welch_anova"`, `"student_t"`, `"anova"`, `"wilcoxon"`,
#'         `"kruskal"`).
#'       \item `statistic`, `df1`, `df2`, `p.value` -- test results.
#'       \item `es_type` -- effect-size identifier (`"hedges_g"`,
#'         `"eta_sq"`, `"r_rb"`, or `"epsilon_sq"`), when
#'         `effect_size = TRUE`.
#'       \item `es_value`, `es_ci_lower`, `es_ci_upper` -- effect-size
#'         estimate and confidence interval bounds.
#'     }
#'   \item `"tinytable"`: a `tinytable` object.
#'   \item `"gt"`: a `gt_tbl` object.
#'   \item `"flextable"`: a `flextable` object.
#'   \item `"excel"` / `"word"`: writes to disk and returns the file
#'     path invisibly.
#'   \item `"clipboard"`: copies the table and returns the display
#'     `data.frame` invisibly.
#' }
#'
#' @details
#' Non-numeric columns are silently dropped (set `verbose = TRUE` to see
#' which columns were excluded). When a single constant column is passed,
#' SD and CI are shown as `"--"` in the ASCII table.
#'
#' Optional output engines require suggested packages:
#' \itemize{
#'   \item \pkg{tinytable} for `output = "tinytable"`
#'   \item \pkg{gt} for `output = "gt"`
#'   \item \pkg{flextable} for `output = "flextable"`
#'   \item \pkg{flextable} + \pkg{officer} for `output = "word"`
#'   \item \pkg{openxlsx2} for `output = "excel"`
#'   \item \pkg{clipr} for `output = "clipboard"`
#' }
#'
#' @seealso [table_categorical()] for categorical variables;
#'   [freq()] for one-way frequency tables; [cross_tab()] for two-way
#'   cross-tabulations.
#'
#' @examples
#' table_continuous(
#'   sochealth,
#'   select = c(bmi, wellbeing_score)
#' )
#'
#' table_continuous(
#'   sochealth,
#'   select = c(bmi, wellbeing_score),
#'   output = "data.frame"
#' )
#'
#' table_continuous(
#'   sochealth,
#'   select = c(bmi, wellbeing_score),
#'   by = education
#' )
#'
#' table_continuous(
#'   sochealth,
#'   select = c(bmi, wellbeing_score),
#'   by = education,
#'   statistic = TRUE
#' )
#'
#' table_continuous(
#'   sochealth,
#'   select = wellbeing_score,
#'   by = education,
#'   effect_size_ci = TRUE,
#'   effect_size_digits = 3
#' )
#'
#' table_continuous(
#'   sochealth,
#'   select = wellbeing_score,
#'   by = sex,
#'   effect_size_ci = TRUE
#' )
#'
#' table_continuous(
#'   sochealth,
#'   select = "^life_sat",
#'   regex = TRUE
#' )
#'
#' table_continuous(
#'   sochealth,
#'   select = c(bmi, life_sat_health),
#'   labels = c(
#'     bmi = "Body mass index",
#'     life_sat_health = "Satisfaction with health"
#'   )
#' )
#'
#' \donttest{
#' if (requireNamespace("tinytable", quietly = TRUE)) {
#'   table_continuous(
#'     sochealth,
#'     select = starts_with("life_sat"),
#'     by = education,
#'     output = "tinytable"
#'   )
#' }
#'
#' if (requireNamespace("gt", quietly = TRUE)) {
#'   table_continuous(
#'     sochealth,
#'     select = starts_with("life_sat"),
#'     by = education,
#'     output = "gt"
#'   )
#' }
#'
#' if (requireNamespace("flextable", quietly = TRUE)) {
#'   table_continuous(
#'     sochealth,
#'     select = starts_with("life_sat"),
#'     by = education,
#'     output = "flextable"
#'   )
#' }
#' }
#'
#' @importFrom stats qt qnorm qf pf sd var t.test oneway.test wilcox.test
#'   kruskal.test quantile
#' @importFrom rlang enquo eval_tidy quo_get_env inform
#' @export
table_continuous <- function(
  data,
  select = tidyselect::everything(),
  by = NULL,
  exclude = NULL,
  regex = FALSE,
  test = c("welch", "student", "nonparametric"),
  p_value = NULL,
  statistic = FALSE,
  effect_size = FALSE,
  effect_size_ci = FALSE,
  labels = NULL,
  ci_level = 0.95,
  digits = 2,
  effect_size_digits = 2,
  decimal_mark = ".",
  output = c(
    "default",
    "data.frame",
    "tinytable",
    "gt",
    "flextable",
    "excel",
    "clipboard",
    "word"
  ),
  excel_path = NULL,
  excel_sheet = "Descriptives",
  clipboard_delim = "\t",
  word_path = NULL,
  verbose = FALSE
) {
  # --- validation ---
  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }
  if (
    !is.numeric(ci_level) ||
      length(ci_level) != 1L ||
      is.na(ci_level) ||
      ci_level <= 0 ||
      ci_level >= 1
  ) {
    stop("`ci_level` must be a single number between 0 and 1.", call. = FALSE)
  }
  if (
    !is.numeric(digits) ||
      length(digits) != 1L ||
      is.na(digits) ||
      digits < 0
  ) {
    stop("`digits` must be a single non-negative number.", call. = FALSE)
  }
  digits <- as.integer(digits)
  if (
    !is.numeric(effect_size_digits) ||
      length(effect_size_digits) != 1L ||
      is.na(effect_size_digits) ||
      effect_size_digits < 0
  ) {
    stop(
      "`effect_size_digits` must be a single non-negative number.",
      call. = FALSE
    )
  }
  effect_size_digits <- as.integer(effect_size_digits)
  if (!decimal_mark %in% c(".", ",")) {
    stop('`decimal_mark` must be "." or ","', call. = FALSE)
  }
  if (!is.null(labels) && (!is.character(labels) || is.null(names(labels)))) {
    stop("`labels` must be a named character vector.", call. = FALSE)
  }
  for (.lname in c(
    "statistic",
    "effect_size",
    "effect_size_ci",
    "regex",
    "verbose"
  )) {
    .lval <- get(.lname)
    if (!is.logical(.lval) || length(.lval) != 1L || is.na(.lval)) {
      stop(sprintf("`%s` must be TRUE/FALSE.", .lname), call. = FALSE)
    }
  }
  if (
    !is.null(p_value) &&
      (!is.logical(p_value) || length(p_value) != 1L || is.na(p_value))
  ) {
    stop("`p_value` must be TRUE, FALSE, or NULL.", call. = FALSE)
  }
  output <- match.arg(output)
  test_explicit <- !missing(test)
  test <- match.arg(test)

  # --- by (grouping) handling ---
  group_quo <- rlang::enquo(by)
  has_group <- !rlang::quo_is_null(group_quo)
  group_col_name <- NULL

  if (has_group) {
    group_col_name <- tryCatch(
      resolve_single_column_selection(group_quo, data, "by"),
      error = function(e) {
        stop(
          "`by` must be a single column name in `data`.",
          call. = FALSE
        )
      }
    )
  }

  p_value_explicit <- !is.null(p_value)
  if (!p_value_explicit) {
    p_value <- has_group
  }
  if ((p_value || statistic) && !has_group) {
    if (p_value_explicit || statistic) {
      warning(
        "`p_value` and `statistic` are ignored when `by` is not used.",
        call. = FALSE
      )
    }
    p_value <- FALSE
  }
  if (
    test_explicit && !p_value && !statistic && !effect_size && !effect_size_ci
  ) {
    warning(
      "`test` is ignored when both `p_value` and `statistic` are FALSE.",
      call. = FALSE
    )
  }
  do_test <- (p_value || statistic) && has_group

  if ((effect_size || effect_size_ci) && !has_group) {
    warning(
      "`effect_size` is ignored when `by` is not used.",
      call. = FALSE
    )
  }
  if (effect_size_ci && !effect_size) {
    warning(
      "`effect_size_ci` implies `effect_size = TRUE`.",
      call. = FALSE
    )
    effect_size <- TRUE
  }
  do_es <- effect_size && has_group
  # Effect size needs test computation even if p_value/statistic are FALSE
  if (do_es && !do_test) {
    do_test <- TRUE
  }

  # --- column selection (reuse mean_n pattern) ---
  work <- data
  if (has_group) {
    work <- dplyr::select(work, -tidyselect::all_of(group_col_name))
  }

  if (regex) {
    if (missing(select)) {
      select <- ".*"
    }
    if (!is.character(select) || length(select) != 1L || is.na(select)) {
      stop(
        "When `regex = TRUE`, `select` must be a single character pattern.",
        call. = FALSE
      )
    }
    matched <- grep(select, names(work), value = TRUE)
    work <- work[, matched, drop = FALSE]
  } else {
    sel_quo <- rlang::enquo(select)
    sel_val <- tryCatch(
      rlang::eval_tidy(sel_quo, env = rlang::quo_get_env(sel_quo)),
      error = function(e) NULL
    )
    if (is.character(sel_val)) {
      work <- dplyr::select(work, tidyselect::all_of(sel_val))
    } else {
      work <- dplyr::select(work, !!sel_quo)
    }
  }

  exclude_quo <- rlang::enquo(exclude)
  exclude_names <- resolve_multi_column_selection(exclude_quo, work, "exclude")
  work <- dplyr::select(work, -tidyselect::any_of(exclude_names))

  all_cols <- names(work)
  work <- dplyr::select(work, tidyselect::where(is.numeric))
  numeric_cols <- names(work)

  ignored <- setdiff(all_cols, numeric_cols)
  if (verbose && length(ignored) > 0L) {
    rlang::inform(
      paste0(
        "table_continuous(): Ignored non-numeric columns: ",
        paste(ignored, collapse = ", ")
      )
    )
  }

  if (length(numeric_cols) == 0L) {
    warning("No numeric columns selected.", call. = FALSE)
    return(data.frame())
  }

  # --- label detection ---
  var_labels <- vapply(
    numeric_cols,
    function(nm) {
      if (!is.null(labels) && nm %in% names(labels)) {
        return(labels[[nm]])
      }
      lab <- attr(data[[nm]], "label", exact = TRUE)
      if (is.null(lab) || !nzchar(lab)) nm else lab
    },
    character(1L),
    USE.NAMES = FALSE
  )

  # --- computation ---
  compute_one <- function(x, ci_level) {
    x_valid <- x[!is.na(x)]
    n <- length(x_valid)
    if (n == 0L) {
      return(data.frame(
        mean = NA_real_,
        sd = NA_real_,
        min = NA_real_,
        max = NA_real_,
        ci_lower = NA_real_,
        ci_upper = NA_real_,
        n = 0L,
        stringsAsFactors = FALSE
      ))
    }
    m <- mean(x_valid)
    s <- if (n > 1L) stats::sd(x_valid) else NA_real_
    se <- if (n > 1L) s / sqrt(n) else NA_real_
    alpha <- 1 - ci_level
    t_crit <- if (n > 1L) stats::qt(1 - alpha / 2, df = n - 1L) else NA_real_
    data.frame(
      mean = m,
      sd = s,
      min = min(x_valid),
      max = max(x_valid),
      ci_lower = if (n > 1L) m - t_crit * se else NA_real_,
      ci_upper = if (n > 1L) m + t_crit * se else NA_real_,
      n = n,
      stringsAsFactors = FALSE
    )
  }

  if (has_group) {
    groups <- data[[group_col_name]]
    n_na_groups <- sum(is.na(groups))
    if (n_na_groups > 0L) {
      warning(
        sprintf(
          "%d observation(s) with NA in `%s` were excluded.",
          n_na_groups,
          group_col_name
        ),
        call. = FALSE
      )
    }
    group_levels <- if (is.factor(groups)) {
      levels(groups)
    } else {
      sort(unique(groups[!is.na(groups)]))
    }
    n_groups <- length(group_levels)
    rows <- list()
    for (i in seq_along(numeric_cols)) {
      nm <- numeric_cols[i]

      # --- group-comparison test ---
      test_row <- data.frame(
        test_type = NA_character_,
        statistic = NA_real_,
        df1 = NA_real_,
        df2 = NA_real_,
        p.value = NA_real_,
        stringsAsFactors = FALSE
      )
      if (do_test) {
        xvec <- work[[nm]]
        gvec <- groups
        complete <- !is.na(xvec) & !is.na(gvec)
        xvec <- xvec[complete]
        gvec <- gvec[complete]
        if (is.factor(gvec)) {
          gvec <- droplevels(gvec)
        }
        n_valid_groups <- length(unique(gvec))
        # Need at least 2 groups with >=2 obs each for a test
        grp_n <- table(gvec)
        testable <- n_valid_groups >= 2L && all(grp_n >= 2L)
        if (testable) {
          test_row <- run_group_test(xvec, gvec, n_valid_groups, test)
        }
      }

      # --- effect size ---
      es_row <- data.frame(
        es_type = NA_character_,
        es_value = NA_real_,
        es_ci_lower = NA_real_,
        es_ci_upper = NA_real_,
        stringsAsFactors = FALSE
      )
      if (do_es) {
        if (testable) {
          es_row <- compute_effect_size(
            xvec,
            gvec,
            n_valid_groups,
            test,
            ci_level
          )
        }
      }

      es_na_row <- data.frame(
        es_type = NA_character_,
        es_value = NA_real_,
        es_ci_lower = NA_real_,
        es_ci_upper = NA_real_,
        stringsAsFactors = FALSE
      )

      for (j in seq_along(group_levels)) {
        g <- group_levels[j]
        idx <- which(groups == g)
        desc <- compute_one(work[[nm]][idx], ci_level)
        desc <- cbind(
          data.frame(
            variable = nm,
            label = var_labels[i],
            group = as.character(g),
            stringsAsFactors = FALSE
          ),
          desc
        )
        if (do_test) {
          if (j == 1L) {
            desc <- cbind(desc, test_row)
          } else {
            desc <- cbind(
              desc,
              data.frame(
                test_type = NA_character_,
                statistic = NA_real_,
                df1 = NA_real_,
                df2 = NA_real_,
                p.value = NA_real_,
                stringsAsFactors = FALSE
              )
            )
          }
        }
        if (do_es) {
          desc <- cbind(desc, if (j == 1L) es_row else es_na_row)
        }
        rows[[length(rows) + 1L]] <- desc
      }
    }
    result <- do.call(rbind, rows)
  } else {
    rows <- lapply(seq_along(numeric_cols), function(i) {
      desc <- compute_one(work[[numeric_cols[i]]], ci_level)
      cbind(
        data.frame(
          variable = numeric_cols[i],
          label = var_labels[i],
          stringsAsFactors = FALSE
        ),
        desc
      )
    })
    result <- do.call(rbind, rows)
  }

  rownames(result) <- NULL

  # --- attributes & class ---
  attr(result, "ci_level") <- ci_level
  attr(result, "digits") <- digits
  attr(result, "effect_size_digits") <- effect_size_digits
  attr(result, "decimal_mark") <- decimal_mark
  attr(result, "group_var") <- group_col_name
  attr(result, "test") <- if (do_test) test else NA_character_
  attr(result, "show_p") <- p_value && has_group
  attr(result, "show_statistic") <- statistic && has_group
  attr(result, "show_effect_size") <- effect_size && has_group
  attr(result, "show_effect_size_ci") <- effect_size_ci && has_group

  # --- plain data.frame return ---
  if (output == "data.frame") {
    return(result)
  }

  # --- raw return for non-default outputs ---
  if (output != "default") {
    display_df <- build_display_df(
      result,
      digits = digits,
      effect_size_digits = effect_size_digits,
      decimal_mark = decimal_mark,
      ci_level = ci_level,
      show_p = attr(result, "show_p"),
      show_statistic = attr(result, "show_statistic"),
      show_effect_size = attr(result, "show_effect_size"),
      show_effect_size_ci = attr(result, "show_effect_size_ci")
    )
    return(
      export_desc_table(
        display_df,
        result,
        output = output,
        ci_level = ci_level,
        has_group = has_group,
        excel_path = excel_path,
        excel_sheet = excel_sheet,
        clipboard_delim = clipboard_delim,
        word_path = word_path
      )
    )
  }

  # --- return ---
  class(result) <- c("spicy_continuous_table", "spicy_table", class(result))
  print(result)
  invisible(result)
}


# --- internal: run group-comparison test ---
run_group_test <- function(xvec, gvec, n_groups, method) {
  row <- data.frame(
    test_type = NA_character_,
    statistic = NA_real_,
    df1 = NA_real_,
    df2 = NA_real_,
    p.value = NA_real_,
    stringsAsFactors = FALSE
  )

  if (method == "nonparametric") {
    if (n_groups == 2L) {
      wt <- stats::wilcox.test(xvec ~ gvec)
      row$test_type <- "wilcoxon"
      row$statistic <- unname(wt$statistic)
      row$p.value <- wt$p.value
    } else {
      kt <- stats::kruskal.test(xvec ~ gvec)
      row$test_type <- "kruskal"
      row$statistic <- unname(kt$statistic)
      row$df1 <- unname(kt$parameter)
      row$p.value <- kt$p.value
    }
  } else {
    var_equal <- (method == "student")
    if (n_groups == 2L) {
      tt <- stats::t.test(xvec ~ gvec, var.equal = var_equal)
      row$test_type <- if (var_equal) "student_t" else "welch_t"
      row$statistic <- unname(tt$statistic)
      row$df1 <- unname(tt$parameter)
      row$p.value <- tt$p.value
    } else {
      ft <- stats::oneway.test(xvec ~ gvec, var.equal = var_equal)
      row$test_type <- if (var_equal) "anova" else "welch_anova"
      row$statistic <- unname(ft$statistic)
      row$df1 <- unname(ft$parameter[1])
      row$df2 <- unname(ft$parameter[2])
      row$p.value <- ft$p.value
    }
  }

  row
}

# --- internal: compute effect size ---
compute_effect_size <- function(xvec, gvec, n_groups, method, ci_level) {
  row <- data.frame(
    es_type = NA_character_,
    es_value = NA_real_,
    es_ci_lower = NA_real_,
    es_ci_upper = NA_real_,
    stringsAsFactors = FALSE
  )
  alpha <- 1 - ci_level

  if (method == "nonparametric") {
    if (n_groups == 2L) {
      # Rank-biserial r from Wilcoxon W
      grp_levels <- if (is.factor(gvec)) levels(gvec) else sort(unique(gvec))
      n1 <- sum(gvec == grp_levels[1])
      n2 <- sum(gvec == grp_levels[2])
      wt <- stats::wilcox.test(xvec ~ gvec)
      w <- unname(wt$statistic)
      r <- 1 - (2 * w) / (n1 * n2)
      row$es_type <- "r_rb"
      row$es_value <- r
      # Fisher z-transform CI
      n_total <- n1 + n2
      if (n_total > 3L) {
        z <- atanh(r)
        se_z <- 1 / sqrt(n_total - 3)
        z_crit <- stats::qnorm(1 - alpha / 2)
        row$es_ci_lower <- tanh(z - z_crit * se_z)
        row$es_ci_upper <- tanh(z + z_crit * se_z)
      }
    } else {
      # Epsilon-squared from Kruskal-Wallis H
      kt <- stats::kruskal.test(xvec ~ gvec)
      h <- unname(kt$statistic)
      n_total <- length(xvec)
      row$es_type <- "epsilon_sq"
      row$es_value <- max(0, (h - n_groups + 1) / (n_total - n_groups))
      # Bootstrap CI for epsilon-squared
      ci <- epsilon_sq_boot_ci(xvec, gvec, n_groups, ci_level)
      row$es_ci_lower <- ci[1]
      row$es_ci_upper <- ci[2]
    }
  } else {
    if (n_groups == 2L) {
      # Hedges' g (bias-corrected standardised mean difference)
      grp_levels <- if (is.factor(gvec)) levels(gvec) else sort(unique(gvec))
      x1 <- xvec[gvec == grp_levels[1]]
      x2 <- xvec[gvec == grp_levels[2]]
      n1 <- length(x1)
      n2 <- length(x2)
      s_pooled <- sqrt(
        ((n1 - 1) * stats::var(x1) + (n2 - 1) * stats::var(x2)) / (n1 + n2 - 2)
      )
      d <- (mean(x1) - mean(x2)) / s_pooled
      # Hedges' correction factor (J)
      g <- d * (1 - 3 / (4 * (n1 + n2 - 2) - 1))
      row$es_type <- "hedges_g"
      row$es_value <- g
      # Hedges & Olkin approximation for SE
      se_g <- sqrt(1 / n1 + 1 / n2 + g^2 / (2 * (n1 + n2)))
      z_crit <- stats::qnorm(1 - alpha / 2)
      row$es_ci_lower <- g - z_crit * se_g
      row$es_ci_upper <- g + z_crit * se_g
    } else {
      # Eta-squared from one-way ANOVA (SS_between / SS_total)
      grand_mean <- mean(xvec)
      grp_levels <- if (is.factor(gvec)) levels(gvec) else sort(unique(gvec))
      ss_between <- 0
      for (g in grp_levels) {
        xg <- xvec[gvec == g]
        ss_between <- ss_between + length(xg) * (mean(xg) - grand_mean)^2
      }
      ss_total <- sum((xvec - grand_mean)^2)
      eta_sq <- ss_between / ss_total
      row$es_type <- "eta_sq"
      row$es_value <- eta_sq
      # CI via noncentral F
      n_total <- length(xvec)
      df1 <- n_groups - 1
      df2 <- n_total - n_groups
      f_obs <- (ss_between / df1) / ((ss_total - ss_between) / df2)
      ci <- eta_sq_ci(f_obs, df1, df2, ci_level)
      row$es_ci_lower <- ci[1]
      row$es_ci_upper <- ci[2]
    }
  }

  row
}

# --- internal: CI for eta-squared via noncentral F ---
eta_sq_ci <- function(f_obs, df1, df2, ci_level) {
  alpha <- 1 - ci_level

  # Suppress benign pnbeta precision warnings from noncentral F
  pf_safe <- function(...) suppressWarnings(stats::pf(...))

  # Find lower ncp
  ncp_lower <- tryCatch(
    {
      if (pf_safe(f_obs, df1, df2, ncp = 0) < 1 - alpha / 2) {
        0
      } else {
        stats::uniroot(
          function(ncp) {
            pf_safe(f_obs, df1, df2, ncp = ncp, lower.tail = FALSE) -
              alpha / 2
          },
          interval = c(0, f_obs * (df1 + df2) * 5),
          tol = 1e-8
        )$root
      }
    },
    error = function(e) NA_real_
  )

  # Find upper ncp
  ncp_upper <- tryCatch(
    {
      stats::uniroot(
        function(ncp) {
          pf_safe(f_obs, df1, df2, ncp = ncp, lower.tail = FALSE) -
            (1 - alpha / 2)
        },
        interval = c(0, f_obs * (df1 + df2) * 5),
        tol = 1e-8
      )$root
    },
    error = function(e) NA_real_
  )

  # Convert ncp to eta-squared: eta_sq = ncp / (ncp + df1 + df2 + 1)
  n_total <- df1 + df2 + 1
  lower <- if (is.na(ncp_lower)) {
    NA_real_
  } else {
    max(0, ncp_lower / (ncp_lower + n_total))
  }
  upper <- if (is.na(ncp_upper)) {
    NA_real_
  } else {
    min(1, ncp_upper / (ncp_upper + n_total))
  }

  c(lower, upper)
}

# --- internal: bootstrap CI for epsilon-squared ---
epsilon_sq_boot_ci <- function(xvec, gvec, n_groups, ci_level, n_boot = 2000L) {
  alpha <- 1 - ci_level
  n_total <- length(xvec)

  compute_eps <- function(x, g, k) {
    h <- unname(stats::kruskal.test(x ~ g)$statistic)
    max(0, (h - k + 1) / (length(x) - k))
  }

  boot_vals <- vapply(
    seq_len(n_boot),
    function(i) {
      idx <- sample.int(n_total, replace = TRUE)
      xb <- xvec[idx]
      gb <- gvec[idx]
      # Ensure all groups are represented in the resample
      if (length(unique(gb)) < n_groups) {
        return(NA_real_)
      }
      tryCatch(compute_eps(xb, gb, n_groups), error = function(e) NA_real_)
    },
    double(1)
  )

  boot_vals <- boot_vals[!is.na(boot_vals)]
  if (length(boot_vals) < 100L) {
    return(c(NA_real_, NA_real_))
  }

  unname(stats::quantile(boot_vals, probs = c(alpha / 2, 1 - alpha / 2)))
}

# --- internal: build formatted display data frame ---
build_display_df <- function(
  result,
  digits,
  decimal_mark,
  ci_level,
  show_p = FALSE,
  show_statistic = FALSE,
  show_effect_size = FALSE,
  show_effect_size_ci = FALSE,
  effect_size_digits = 2L
) {
  fmt <- function(v, d = digits) {
    out <- formatC(v, format = "f", digits = d)
    if (decimal_mark != ".") {
      out <- sub("\\.", decimal_mark, out)
    }
    ifelse(is.na(v), "--", out)
  }

  fmt_p <- function(p) {
    if (is.na(p)) {
      return("")
    }
    if (p < 0.001) {
      return(if (decimal_mark == ".") "<\u00A0.001" else "<\u00A0,001")
    }
    s <- formatC(p, format = "f", digits = 3L)
    s <- sub("^0\\.", ".", s)
    if (decimal_mark != ".") {
      s <- sub("\\.", decimal_mark, s)
    }
    s
  }

  fmt_test <- function(test_type, stat, df1, df2, decimal_mark) {
    if (is.na(stat)) {
      return("")
    }
    s <- formatC(stat, format = "f", digits = 2L)
    if (decimal_mark != ".") {
      s <- sub("\\.", decimal_mark, s)
    }
    if (test_type == "wilcoxon") {
      paste0("W = ", s)
    } else if (test_type == "kruskal") {
      d <- formatC(df1, format = "f", digits = 0L)
      paste0("H(", d, ") = ", s)
    } else if (is.na(df2)) {
      # t-test (welch or student): df can be fractional
      d <- formatC(df1, format = "f", digits = 2L)
      if (decimal_mark != ".") {
        d <- sub("\\.", decimal_mark, d)
      }
      paste0("t(", d, ") = ", s)
    } else {
      # F-test (welch_anova or anova)
      d1 <- formatC(df1, format = "f", digits = 0L)
      d2 <- formatC(df2, format = "f", digits = 2L)
      if (decimal_mark != ".") {
        d2 <- sub("\\.", decimal_mark, d2)
      }
      paste0("F(", d1, ", ", d2, ") = ", s)
    }
  }

  es_labels <- c(
    hedges_g = "g",
    eta_sq = "\u03b7\u00b2",
    r_rb = "r_rb",
    epsilon_sq = "\u03b5\u00b2"
  )

  fmt_es <- function(es_type, es_value, ci_lower, ci_upper, show_ci) {
    if (is.na(es_value)) {
      return("")
    }
    label <- es_labels[[es_type]]
    v <- formatC(es_value, format = "f", digits = effect_size_digits)
    if (decimal_mark != ".") {
      v <- sub("\\.", decimal_mark, v)
    }
    s <- paste0(label, " = ", v)
    if (show_ci && !is.na(ci_lower) && !is.na(ci_upper)) {
      lo <- formatC(ci_lower, format = "f", digits = effect_size_digits)
      hi <- formatC(ci_upper, format = "f", digits = effect_size_digits)
      if (decimal_mark != ".") {
        lo <- sub("\\.", decimal_mark, lo)
        hi <- sub("\\.", decimal_mark, hi)
      }
      s <- paste0(s, " [", lo, ", ", hi, "]")
    }
    s
  }

  ci_pct <- paste0(round(ci_level * 100), "%")

  has_group <- "group" %in% names(result)
  has_computed <- "statistic" %in% names(result)
  has_es <- "es_value" %in% names(result)

  if (has_group) {
    df <- data.frame(
      Variable = result$label,
      Group = result$group,
      M = fmt(result$mean),
      SD = fmt(result$sd),
      Min = fmt(result$min),
      Max = fmt(result$max),
      LL = fmt(result$ci_lower),
      UL = fmt(result$ci_upper),
      n = as.character(result$n),
      stringsAsFactors = FALSE,
      check.names = FALSE
    )
    names(df)[7:8] <- c(
      paste0(ci_pct, " CI LL"),
      paste0(ci_pct, " CI UL")
    )

    # Deduplicate Variable labels: show only on first row per block
    vars <- result$variable
    for (i in seq_along(vars)) {
      if (i > 1L && vars[i] == vars[i - 1L]) {
        df$Variable[i] <- ""
      }
    }

    # Add test columns if computed and requested
    if (has_computed && show_statistic) {
      df$Test <- vapply(
        seq_len(nrow(result)),
        function(i) {
          tt <- result$test_type[i]
          if (is.na(tt)) {
            tt <- "welch_t"
          }
          fmt_test(
            tt,
            result$statistic[i],
            result$df1[i],
            result$df2[i],
            decimal_mark
          )
        },
        character(1)
      )
    }
    if (has_computed && show_p) {
      df$p <- vapply(result$p.value, fmt_p, character(1))
    }
    if (has_es && show_effect_size) {
      df$ES <- vapply(
        seq_len(nrow(result)),
        function(i) {
          fmt_es(
            result$es_type[i],
            result$es_value[i],
            result$es_ci_lower[i],
            result$es_ci_upper[i],
            show_effect_size_ci
          )
        },
        character(1)
      )
    }
  } else {
    df <- data.frame(
      Variable = result$label,
      M = fmt(result$mean),
      SD = fmt(result$sd),
      Min = fmt(result$min),
      Max = fmt(result$max),
      LL = fmt(result$ci_lower),
      UL = fmt(result$ci_upper),
      n = as.character(result$n),
      stringsAsFactors = FALSE,
      check.names = FALSE
    )
    names(df)[6:7] <- c(
      paste0(ci_pct, " CI LL"),
      paste0(ci_pct, " CI UL")
    )
  }

  df
}


# --- internal: compute separator row indices (first row of each var block) ---
compute_var_sep_rows <- function(display_df) {
  if (!"Variable" %in% names(display_df)) {
    return(integer(0)) # nocov
  }
  vars <- display_df$Variable
  sep <- integer(0)
  for (i in seq_along(vars)) {
    if (i > 1L && nzchar(vars[i])) {
      sep <- c(sep, i)
    }
  }
  sep
}

# --- internal: rename CI columns for export ---
rename_ci_cols <- function(display_df, ci_ll, ci_ul) {
  names(display_df)[names(display_df) == ci_ll] <- "LL"
  names(display_df)[names(display_df) == ci_ul] <- "UL"
  display_df
}

# --- internal: build 2-row header vectors ---
build_header_rows <- function(col_keys, ci_pct) {
  nc <- length(col_keys)
  top <- col_keys
  top[col_keys == "LL"] <- paste0(ci_pct, " CI")
  top[col_keys == "UL"] <- paste0(ci_pct, " CI")
  bot <- rep("", nc)
  bot[col_keys == "LL"] <- "LL"
  bot[col_keys == "UL"] <- "UL"
  list(top = top, bottom = bot)
}

# --- internal: export to various formats ---
export_desc_table <- function(
  display_df,
  raw_result,
  output,
  ci_level,
  has_group,
  excel_path,
  excel_sheet,
  clipboard_delim,
  word_path
) {
  ci_pct <- paste0(round(ci_level * 100), "%")
  ci_ll <- paste0(ci_pct, " CI LL")
  ci_ul <- paste0(ci_pct, " CI UL")
  has_statistic <- "Test" %in% names(display_df)
  has_p <- "p" %in% names(display_df)
  has_es <- "ES" %in% names(display_df)
  sep_rows <- compute_var_sep_rows(display_df)

  # ---- tinytable ----
  if (output == "tinytable") {
    if (!requireNamespace("tinytable", quietly = TRUE)) {
      # nocov start
      stop("Install package 'tinytable'.", call. = FALSE)
    } # nocov end

    old_tt_opt <- getOption("tinytable_print_output")
    options(tinytable_print_output = "html")
    on.exit(options(tinytable_print_output = old_tt_opt), add = TRUE)

    display_df <- rename_ci_cols(display_df, ci_ll, ci_ul)
    nc <- ncol(display_df)
    ll_pos <- which(names(display_df) == "LL")
    ul_pos <- which(names(display_df) == "UL")

    # Sub-row labels: empty for single-col spanners, LL/UL for CI
    sub_labels <- rep("", nc)
    sub_labels[ll_pos] <- "LL"
    sub_labels[ul_pos] <- "UL"
    colnames(display_df) <- sub_labels

    # Build gspec: all columns as spanners
    gspec <- list()
    col_names <- c("Variable")
    if (has_group) {
      col_names <- c(col_names, "Group")
    }
    col_names <- c(col_names, "M", "SD", "Min", "Max")

    pos <- 1L
    for (nm in col_names) {
      gspec[[nm]] <- pos
      pos <- pos + 1L
    }
    gspec[[paste0(ci_pct, " CI")]] <- c(ll_pos, ul_pos)

    # n position: after UL, before Test/p
    n_pos <- which(names(display_df) == "" & seq_len(nc) > ul_pos)[1]
    if (is.na(n_pos)) {
      n_pos <- ul_pos + 1L # nocov
    }
    gspec[["n"]] <- n_pos
    next_pos <- n_pos + 1L
    if (has_statistic) {
      gspec[["Test"]] <- next_pos
      next_pos <- next_pos + 1L
    }
    if (has_p) {
      gspec[["p"]] <- next_pos
      next_pos <- next_pos + 1L
    }
    if (has_es) {
      gspec[["ES"]] <- next_pos
    }

    tt <- tinytable::tt(display_df)
    tt <- tinytable::group_tt(tt, j = gspec)
    tt <- tinytable::theme_empty(tt)

    # Alignment
    left_j <- 1L
    tt <- tinytable::style_tt(tt, j = 1, align = "l")
    if (has_group) {
      tt <- tinytable::style_tt(tt, j = 2, align = "l")
      left_j <- c(left_j, 2L)
    }
    right_j <- n_pos
    if (has_p) {
      right_j <- c(right_j, gspec[["p"]])
    }
    center_j <- setdiff(seq_len(nc), c(left_j, right_j))
    tt <- tinytable::style_tt(tt, j = center_j, align = "c")
    for (rj in right_j) {
      tt <- tinytable::style_tt(tt, j = rj, align = "r")
    }

    # Spanner alignment
    spanner_center_j <- setdiff(seq_len(nc), left_j)
    tt <- tinytable::style_tt(
      tt,
      i = -1,
      j = spanner_center_j,
      align = "c"
    )
    tt <- tinytable::style_tt(tt, i = -1, j = left_j, align = "l")

    # APA lines
    tt <- tinytable::style_tt(
      tt,
      i = -1,
      j = seq_len(nc),
      line = "t",
      line_width = 0.06
    )
    tt <- tinytable::style_tt(
      tt,
      i = -1,
      j = c(ll_pos, ul_pos),
      line = "b",
      line_width = 0.06
    )
    tt <- tinytable::style_tt(
      tt,
      i = 0,
      j = seq_len(nc),
      line = "b",
      line_width = 0.06
    )
    tt <- tinytable::style_tt(
      tt,
      i = nrow(display_df),
      j = seq_len(nc),
      line = "b",
      line_width = 0.06
    )

    # Light separators between variable blocks
    for (sr in sep_rows) {
      tt <- tinytable::style_tt(
        tt,
        i = sr - 1L,
        j = seq_len(nc),
        line = "b",
        line_width = 0.03
      )
    }

    return(tt)
  }

  # ---- gt ----
  if (output == "gt") {
    if (!requireNamespace("gt", quietly = TRUE)) {
      # nocov start
      stop("Install package 'gt'.", call. = FALSE)
    } # nocov end

    display_df <- rename_ci_cols(display_df, ci_ll, ci_ul)
    tbl <- gt::gt(display_df)

    # Sub-row labels
    label_list <- list(
      Variable = "",
      M = "",
      SD = "",
      Min = "",
      Max = "",
      LL = "LL",
      UL = "UL",
      n = ""
    )
    if (has_group) {
      label_list[["Group"]] <- ""
    }
    if (has_statistic) {
      label_list[["Test"]] <- ""
    }
    if (has_p) {
      label_list[["p"]] <- ""
    }
    if (has_es) {
      label_list[["ES"]] <- ""
    }
    tbl <- gt::cols_label(tbl, .list = label_list)

    # Spanners
    single_cols <- c("Variable")
    if (has_group) {
      single_cols <- c(single_cols, "Group")
    }
    single_cols <- c(single_cols, "M", "SD", "Min", "Max", "n")
    if (has_statistic) {
      single_cols <- c(single_cols, "Test")
    }
    if (has_p) {
      single_cols <- c(single_cols, "p")
    }
    if (has_es) {
      single_cols <- c(single_cols, "ES")
    }

    for (col in single_cols) {
      tbl <- gt::tab_spanner(
        tbl,
        label = col,
        columns = col,
        id = paste0("spn_", col)
      )
    }
    tbl <- gt::tab_spanner(
      tbl,
      label = paste0(ci_pct, " CI"),
      columns = c("LL", "UL")
    )

    # Alignment
    tbl <- gt::cols_align(tbl, align = "left", columns = "Variable")
    if (has_group) {
      tbl <- gt::cols_align(tbl, align = "left", columns = "Group")
    }
    center_cols <- c("M", "SD", "Min", "Max", "LL", "UL")
    if (has_statistic) {
      center_cols <- c(center_cols, "Test")
    }
    if (has_es) {
      center_cols <- c(center_cols, "ES")
    }
    tbl <- gt::cols_align(tbl, align = "center", columns = center_cols)
    right_cols <- "n"
    if (has_p) {
      right_cols <- c(right_cols, "p")
    }
    tbl <- gt::cols_align(tbl, align = "right", columns = right_cols)

    left_spanners <- "spn_Variable"
    if (has_group) {
      left_spanners <- c(left_spanners, "spn_Group")
    }
    tbl <- gt::tab_style(
      tbl,
      style = gt::cell_text(align = "left"),
      locations = gt::cells_column_spanners(spanners = left_spanners)
    )

    # APA borders
    rule <- gt::cell_borders(
      sides = "bottom",
      color = "currentColor",
      weight = gt::px(1)
    )
    rule_top <- gt::cell_borders(
      sides = "top",
      color = "currentColor",
      weight = gt::px(1)
    )
    light_rule <- gt::cell_borders(
      sides = "bottom",
      color = "#cccccc",
      weight = gt::px(0.5)
    )

    tbl <- gt::tab_options(
      tbl,
      table.border.top.width = gt::px(0),
      table.border.bottom.width = gt::px(0),
      table_body.border.top.width = gt::px(0),
      table_body.border.bottom.width = gt::px(0),
      table_body.hlines.color = "transparent",
      column_labels.border.top.width = gt::px(0),
      column_labels.border.bottom.width = gt::px(0),
      column_labels.border.lr.color = "transparent"
    )

    ci_cols <- c("LL", "UL")
    tbl <- gt::tab_style(
      tbl,
      style = rule_top,
      locations = gt::cells_column_spanners()
    )
    tbl <- gt::tab_style(
      tbl,
      style = rule_top,
      locations = gt::cells_column_labels(columns = ci_cols)
    )
    tbl <- gt::tab_style(
      tbl,
      style = rule,
      locations = gt::cells_column_labels()
    )
    tbl <- gt::tab_style(
      tbl,
      style = rule,
      locations = gt::cells_body(rows = nrow(display_df))
    )

    # Light separators between variable blocks
    for (sr in sep_rows) {
      tbl <- gt::tab_style(
        tbl,
        style = light_rule,
        locations = gt::cells_body(rows = sr - 1L)
      )
    }

    # CSS overrides
    ci_css_sel <- paste(
      vapply(
        ci_cols,
        function(id) {
          sprintf('.gt_table thead tr:last-child th[id="%s"]', id)
        },
        character(1)
      ),
      collapse = ",\n"
    )
    apa_css <- paste(
      ".gt_table thead tr:first-child {",
      "  border-top: 1px solid currentColor !important;",
      "}",
      ".gt_table thead tr.gt_spanner_row {",
      "  border-bottom-style: none !important;",
      "}",
      ".gt_table thead th, .gt_table thead td {",
      "  background-color: transparent !important;",
      "}",
      paste0(ci_css_sel, " {"),
      "  border-top: 1px solid currentColor !important;",
      "}",
      ".gt_table thead tr:last-child {",
      "  border-bottom: 1px solid currentColor !important;",
      "}",
      ".gt_table tbody tr:last-child {",
      "  border-bottom: 1px solid currentColor !important;",
      "}",
      ".gt_table tbody tr {",
      "  border-top-style: none !important;",
      "  border-bottom-style: none !important;",
      "}",
      sep = "\n"
    )
    tbl <- gt::opt_css(tbl, css = apa_css)

    return(tbl)
  }

  # ---- flextable / word ----
  if (output %in% c("flextable", "word")) {
    if (!requireNamespace("flextable", quietly = TRUE)) {
      # nocov start
      stop("Install package 'flextable'.", call. = FALSE)
    } # nocov end
    if (output == "word" && !requireNamespace("officer", quietly = TRUE)) {
      # nocov start
      stop("Install package 'officer'.", call. = FALSE)
    } # nocov end
    display_df <- rename_ci_cols(display_df, ci_ll, ci_ul)
    col_keys <- names(display_df)
    nc <- length(col_keys)
    hdrs <- build_header_rows(col_keys, ci_pct)

    map <- data.frame(
      col_keys = col_keys,
      top = hdrs$top,
      bottom = hdrs$bottom,
      stringsAsFactors = FALSE,
      check.names = FALSE
    )

    ft <- flextable::flextable(display_df)
    ft <- flextable::set_header_df(ft, mapping = map, key = "col_keys")
    ft <- flextable::merge_h(ft, part = "header")

    bd <- spicy_fp_border(color = "black", width = 1)
    bd_light <- spicy_fp_border(color = "#cccccc", width = 0.5)

    ci_j <- which(col_keys %in% c("LL", "UL"))
    left_j <- if (has_group) 1:2 else 1L
    right_j <- which(col_keys == "n")
    if (has_p) {
      right_j <- c(right_j, which(col_keys == "p"))
    }
    center_j <- setdiff(seq_len(nc), c(left_j, right_j))

    ft <- flextable::align(ft, j = left_j, part = "all", align = "left")
    ft <- flextable::align(ft, j = center_j, part = "all", align = "center")
    ft <- flextable::align(ft, j = right_j, part = "all", align = "right")

    # APA borders
    ft <- flextable::hline_top(ft, part = "header", border = bd)
    ft <- flextable::hline(
      ft,
      i = 1,
      j = ci_j,
      part = "header",
      border = bd
    )
    ft <- flextable::hline_bottom(ft, part = "header", border = bd)
    ft <- flextable::hline_bottom(ft, part = "body", border = bd)

    # Light separators between variable blocks
    for (sr in sep_rows) {
      ft <- flextable::hline(
        ft,
        i = sr - 1L,
        part = "body",
        border = bd_light
      )
    }

    ft <- flextable::autofit(ft)

    if (output == "word") {
      if (is.null(word_path) || !nzchar(word_path)) {
        stop("Provide `word_path` for output = 'word'.", call. = FALSE)
      }
      flextable::save_as_docx(ft, path = word_path)
      return(invisible(word_path))
    }

    return(ft)
  }

  # ---- excel ----
  if (output == "excel") {
    if (!requireNamespace("openxlsx2", quietly = TRUE)) {
      # nocov start
      stop("Install package 'openxlsx2'.", call. = FALSE)
    } # nocov end
    if (is.null(excel_path) || !nzchar(excel_path)) {
      stop("Provide `excel_path` for output = 'excel'.", call. = FALSE)
    }

    display_df <- rename_ci_cols(display_df, ci_ll, ci_ul)
    col_keys <- names(display_df)
    nc <- length(col_keys)
    hdrs <- build_header_rows(col_keys, ci_pct)
    ci_j <- which(col_keys %in% c("LL", "UL"))

    wb <- openxlsx2::wb_workbook()
    wb <- openxlsx2::wb_add_worksheet(wb, excel_sheet)

    wb <- openxlsx2::wb_add_data(
      wb,
      x = as.data.frame(t(hdrs$top), stringsAsFactors = FALSE),
      start_row = 1,
      col_names = FALSE
    )
    wb <- openxlsx2::wb_add_data(
      wb,
      x = as.data.frame(t(hdrs$bottom), stringsAsFactors = FALSE),
      start_row = 2,
      col_names = FALSE
    )
    wb <- openxlsx2::wb_add_data(
      wb,
      x = display_df,
      start_row = 3,
      col_names = FALSE,
      row_names = FALSE
    )

    wb <- openxlsx2::wb_merge_cells(
      wb,
      dims = openxlsx2::wb_dims(rows = 1, cols = ci_j)
    )
    last_row <- 2 + nrow(display_df)

    # Alignment
    left_cols <- if (has_group) 1:2 else 1L
    right_cols <- which(col_keys == "n")
    if (has_p) {
      right_cols <- c(right_cols, which(col_keys == "p"))
    }
    center_cols <- setdiff(seq_len(nc), c(left_cols, right_cols))
    all_rows <- 1:last_row

    wb <- openxlsx2::wb_add_cell_style(
      wb,
      dims = openxlsx2::wb_dims(rows = all_rows, cols = left_cols),
      horizontal = "left"
    )
    if (length(center_cols) > 0L) {
      wb <- openxlsx2::wb_add_cell_style(
        wb,
        dims = openxlsx2::wb_dims(rows = all_rows, cols = center_cols),
        horizontal = "center",
        vertical = "center"
      )
    }
    wb <- openxlsx2::wb_add_cell_style(
      wb,
      dims = openxlsx2::wb_dims(rows = all_rows, cols = right_cols),
      horizontal = "right"
    )

    # APA borders
    wb <- openxlsx2::wb_add_border(
      wb,
      dims = openxlsx2::wb_dims(rows = 1, cols = 1:nc),
      top_border = "thin"
    )
    wb <- openxlsx2::wb_add_border(
      wb,
      dims = openxlsx2::wb_dims(rows = 1, cols = ci_j),
      bottom_border = "thin"
    )
    wb <- openxlsx2::wb_add_border(
      wb,
      dims = openxlsx2::wb_dims(rows = 2, cols = 1:nc),
      bottom_border = "thin"
    )
    if (nrow(display_df) > 0) {
      wb <- openxlsx2::wb_add_border(
        wb,
        dims = openxlsx2::wb_dims(rows = last_row, cols = 1:nc),
        bottom_border = "thin"
      )
    }

    # Light separators between variable blocks
    for (sr in sep_rows) {
      wb <- openxlsx2::wb_add_border(
        wb,
        dims = openxlsx2::wb_dims(rows = sr - 1L + 2L, cols = 1:nc),
        bottom_border = "hair"
      )
    }

    openxlsx2::wb_save(wb, excel_path, overwrite = TRUE)
    return(invisible(excel_path))
  }

  # ---- clipboard ----
  if (output == "clipboard") {
    if (!requireNamespace("clipr", quietly = TRUE)) {
      # nocov start
      stop("Install package 'clipr'.", call. = FALSE)
    } # nocov end

    display_df <- rename_ci_cols(display_df, ci_ll, ci_ul)
    col_keys <- names(display_df)
    nc <- length(col_keys)
    hdrs <- build_header_rows(col_keys, ci_pct)

    clip_mat <- rbind(hdrs$top, hdrs$bottom, as.matrix(display_df))
    lines <- apply(clip_mat, 1, function(r) {
      paste(r, collapse = clipboard_delim)
    })
    txt <- paste(lines, collapse = "\n")
    clipr::write_clip(txt)
    message("Descriptive statistics copied to clipboard.")
    return(invisible(display_df))
  }

  stop("Unknown output format: ", output, call. = FALSE) # nocov
}

# ---- structure ----

test_that("table_continuous returns correct structure", {
  out <- table_continuous(iris, output = "data.frame")
  expect_s3_class(out, "data.frame")
  expect_named(
    out,
    c(
      "variable",
      "label",
      "mean",
      "sd",
      "min",
      "max",
      "ci_lower",
      "ci_upper",
      "n"
    )
  )
  expect_equal(nrow(out), 4L)
})

test_that("table_continuous returns spicy_continuous_table class when default output", {
  out <- table_continuous(iris, select = c(Sepal.Length))
  expect_s3_class(out, "spicy_continuous_table")
  expect_s3_class(out, "spicy_table")
})

test_that("table_continuous default output object carries correct attributes", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    ci_level = 0.90,
    digits = 3,
    decimal_mark = ","
  )
  expect_equal(attr(out, "ci_level"), 0.90)
  expect_equal(attr(out, "digits"), 3L)
  expect_equal(attr(out, "decimal_mark"), ",")
  expect_null(attr(out, "group_var"))
})

test_that("table_continuous default output with group carries group_var attribute", {
  out <- table_continuous(iris, select = Sepal.Length, by = Species)
  expect_equal(attr(out, "group_var"), "Species")
})

test_that("table_continuous accepts by as a character object without warnings", {
  by_col <- "Species"

  expect_no_warning(
    out <- table_continuous(
      iris,
      select = Sepal.Length,
      by = by_col,
      output = "data.frame"
    )
  )

  expect_true("group" %in% names(out))
  expect_true(all(out$group %in% unique(iris$Species)))
})

# ---- computation ----

test_that("table_continuous computes correct values", {
  df <- data.frame(x = c(1, 2, 3, 4, 5))
  out <- table_continuous(df, output = "data.frame")
  expect_equal(out$mean, 3)
  expect_equal(out$sd, sd(c(1, 2, 3, 4, 5)))
  expect_equal(out$min, 1)
  expect_equal(out$max, 5)
  expect_equal(out$n, 5L)
})

test_that("table_continuous CI is t-based", {
  df <- data.frame(x = c(10, 20, 30))
  out <- table_continuous(df, ci_level = 0.95, output = "data.frame")
  m <- mean(c(10, 20, 30))
  se <- sd(c(10, 20, 30)) / sqrt(3)
  t_crit <- qt(0.975, df = 2)
  expect_equal(out$ci_lower, m - t_crit * se, tolerance = 1e-10)
  expect_equal(out$ci_upper, m + t_crit * se, tolerance = 1e-10)
})

test_that("table_continuous ci_level affects width", {
  df <- data.frame(x = 1:100)
  out90 <- table_continuous(df, ci_level = 0.90, output = "data.frame")
  out99 <- table_continuous(df, ci_level = 0.99, output = "data.frame")
  expect_gt(
    out99$ci_upper - out99$ci_lower,
    out90$ci_upper - out90$ci_lower
  )
})

test_that("table_continuous handles multiple numeric variables", {
  out <- table_continuous(iris, output = "data.frame")
  expect_equal(nrow(out), 4L)
  expect_equal(
    out$variable,
    c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")
  )
  expect_equal(out$mean[1], mean(iris$Sepal.Length))
  expect_equal(out$mean[4], mean(iris$Petal.Width))
})

# ---- NA handling ----

test_that("table_continuous handles NAs", {
  df <- data.frame(x = c(1, NA, 3, NA, 5))
  out <- table_continuous(df, output = "data.frame")
  expect_equal(out$n, 3L)
  expect_equal(out$mean, mean(c(1, 3, 5)))
})

test_that("table_continuous handles all-NA column", {
  df <- data.frame(x = rep(NA_real_, 5))
  out <- table_continuous(df, output = "data.frame")
  expect_equal(out$n, 0L)
  expect_true(is.na(out$mean))
  expect_true(is.na(out$sd))
  expect_true(is.na(out$min))
  expect_true(is.na(out$max))
  expect_true(is.na(out$ci_lower))
  expect_true(is.na(out$ci_upper))
})

test_that("table_continuous n=1 gives NA for sd and CI", {
  df <- data.frame(x = 42)
  out <- table_continuous(df, output = "data.frame")
  expect_equal(out$n, 1L)
  expect_equal(out$mean, 42)
  expect_equal(out$min, 42)
  expect_equal(out$max, 42)
  expect_true(is.na(out$sd))
  expect_true(is.na(out$ci_lower))
  expect_true(is.na(out$ci_upper))
})

test_that("table_continuous display uses -- for NA values", {
  df <- data.frame(x = 42)
  out <- table_continuous(df)
  display <- spicy:::build_display_df(out, 2L, ".", 0.95)
  expect_equal(display$SD[1], "--")
  expect_equal(display[["95% CI LL"]][1], "--")
  expect_equal(display[["95% CI UL"]][1], "--")
})

# ---- column selection ----

test_that("table_continuous filters non-numeric columns", {
  df <- data.frame(x = 1:5, y = letters[1:5], z = 6:10)
  out <- table_continuous(df, output = "data.frame")
  expect_equal(out$variable, c("x", "z"))
})

test_that("table_continuous select works with tidyselect", {
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Petal.Width),
    output = "data.frame"
  )
  expect_equal(nrow(out), 2L)
  expect_equal(out$variable, c("Sepal.Length", "Petal.Width"))
})

test_that("table_continuous select works with character vector", {
  out <- table_continuous(
    iris,
    select = c("Sepal.Length", "Petal.Width"),
    output = "data.frame"
  )
  expect_equal(nrow(out), 2L)
  expect_equal(out$variable, c("Sepal.Length", "Petal.Width"))
})

test_that("table_continuous select works with tidyselect helpers", {
  out <- table_continuous(
    iris,
    select = starts_with("Sepal"),
    output = "data.frame"
  )
  expect_equal(nrow(out), 2L)
  expect_true(all(grepl("^Sepal", out$variable)))
})

test_that("table_continuous exclude works", {
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Sepal.Width, Petal.Length),
    exclude = "Sepal.Width",
    output = "data.frame"
  )
  expect_equal(nrow(out), 2L)
  expect_false("Sepal.Width" %in% out$variable)
})

test_that("table_continuous exclude works with an unquoted column name", {
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Sepal.Width, Petal.Length),
    exclude = Sepal.Width,
    output = "data.frame"
  )
  expect_equal(nrow(out), 2L)
  expect_false("Sepal.Width" %in% out$variable)
})

test_that("table_continuous exclude works with tidyselect syntax", {
  out <- table_continuous(
    iris,
    select = everything(),
    exclude = c(Sepal.Width, Petal.Width),
    output = "data.frame"
  )
  expect_equal(
    out$variable,
    c("Sepal.Length", "Petal.Length")
  )
})

test_that("table_continuous regex selection works", {
  out <- table_continuous(
    iris,
    select = "^Sepal",
    regex = TRUE,
    output = "data.frame"
  )
  expect_equal(nrow(out), 2L)
  expect_true(all(grepl("^Sepal", out$variable)))
})

test_that("table_continuous regex with default select matches all", {
  out <- table_continuous(iris, regex = TRUE, output = "data.frame")
  expect_equal(nrow(out), 4L)
})

test_that("table_continuous verbose reports ignored columns", {
  df <- data.frame(x = 1:5, y = letters[1:5], z = 6:10)
  expect_message(
    table_continuous(df, output = "data.frame", verbose = TRUE),
    "Ignored non-numeric"
  )
})

# ---- labels ----

test_that("table_continuous uses column names as default labels", {
  df <- data.frame(x = 1:5, y = 6:10)
  out <- table_continuous(df, output = "data.frame")
  expect_equal(out$label, c("x", "y"))
})

test_that("table_continuous accepts custom labels", {
  df <- data.frame(x = 1:5, y = 6:10)
  out <- table_continuous(
    df,
    labels = c(x = "My X", y = "My Y"),
    output = "data.frame"
  )
  expect_equal(out$label, c("My X", "My Y"))
})

test_that("table_continuous custom labels apply only to matching columns", {
  df <- data.frame(x = 1:5, y = 6:10)
  out <- table_continuous(df, labels = c(x = "My X"), output = "data.frame")
  expect_equal(out$label, c("My X", "y"))
})

test_that("table_continuous auto-detects haven labels", {
  df <- data.frame(x = 1:5)
  attr(df$x, "label") <- "A labeled var"
  out <- table_continuous(df, output = "data.frame")
  expect_equal(out$label, "A labeled var")
})

test_that("table_continuous custom labels override haven labels", {
  df <- data.frame(x = 1:5)
  attr(df$x, "label") <- "Haven label"
  out <- table_continuous(df, labels = c(x = "Custom"), output = "data.frame")
  expect_equal(out$label, "Custom")
})

# ---- grouping ----

test_that("table_continuous grouped output has group column", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    output = "data.frame"
  )
  expect_true("group" %in% names(out))
  expect_equal(nrow(out), 3L)
  expect_equal(sort(out$group), c("setosa", "versicolor", "virginica"))
})

test_that("table_continuous grouped stats are correct", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    output = "data.frame"
  )
  setosa_data <- iris$Sepal.Length[iris$Species == "setosa"]
  setosa_row <- out[out$group == "setosa", ]
  expect_equal(setosa_row$mean, mean(setosa_data))
  expect_equal(setosa_row$sd, sd(setosa_data))
  expect_equal(setosa_row$n, length(setosa_data))
})

test_that("table_continuous group_var is excluded from numeric selection", {
  df <- data.frame(g = rep(1:2, each = 5), x = 1:10, y = 11:20)
  out <- table_continuous(df, by = g, output = "data.frame")
  expect_false("g" %in% out$variable)
})

test_that("table_continuous preserves factor level order in group_var", {
  df <- data.frame(
    g = factor(
      rep(c("C", "A", "B"), each = 5),
      levels = c("B", "A", "C")
    ),
    x = 1:15
  )
  out <- table_continuous(df, by = g, output = "data.frame")
  expect_equal(out$group, c("B", "A", "C"))
})

test_that("table_continuous grouped with multiple variables", {
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Petal.Length),
    by = Species,
    output = "data.frame"
  )
  expect_equal(nrow(out), 6L)
  expect_equal(
    out$variable,
    rep(c("Sepal.Length", "Petal.Length"), each = 3L)
  )
})

test_that("table_continuous grouped with character group_var sorts levels", {
  df <- data.frame(g = c("Z", "A", "Z", "A"), x = c(1, 2, 3, 4))
  out <- table_continuous(df, by = g, output = "data.frame")
  expect_equal(out$group, c("A", "Z"))
})

# ---- p_value / statistic ----

test_that("table_continuous p_value adds p column without Test column", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = TRUE,
    output = "data.frame"
  )
  expect_true("p.value" %in% names(out))
  expect_true("statistic" %in% names(out))
  display <- spicy:::build_display_df(
    out,
    2L,
    ".",
    0.95,
    show_p = TRUE,
    show_statistic = FALSE
  )
  expect_true("p" %in% names(display))
  expect_false("Test" %in% names(display))
})

test_that("table_continuous p_value + statistic adds both columns", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = TRUE,
    statistic = TRUE,
    output = "data.frame"
  )
  display <- spicy:::build_display_df(
    out,
    2L,
    ".",
    0.95,
    show_p = TRUE,
    show_statistic = TRUE
  )
  expect_true("Test" %in% names(display))
  expect_true("p" %in% names(display))
})

test_that("table_continuous statistic alone shows Test column without p", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    statistic = TRUE,
    output = "data.frame"
  )
  expect_true("statistic" %in% names(out))
  display <- spicy:::build_display_df(
    out,
    2L,
    ".",
    0.95,
    show_p = FALSE,
    show_statistic = TRUE
  )
  expect_true("Test" %in% names(display))
  expect_false("p" %in% names(display))
})

test_that("table_continuous p_value without by warns", {
  expect_warning(
    out <- table_continuous(
      iris,
      select = Sepal.Length,
      p_value = TRUE,
      output = "data.frame"
    ),
    "ignored"
  )
  expect_false("p.value" %in% names(out))
})

test_that("table_continuous statistic without by warns", {
  expect_warning(
    out <- table_continuous(
      iris,
      select = Sepal.Length,
      statistic = TRUE,
      output = "data.frame"
    ),
    "ignored"
  )
  expect_false("statistic" %in% names(out))
})

test_that("table_continuous p_value default output carries show_p attribute", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = TRUE
  )
  expect_true(attr(out, "show_p"))
  expect_false(attr(out, "show_statistic"))
})

test_that("table_continuous p_value + statistic default output carries both attributes", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = TRUE,
    statistic = TRUE
  )
  expect_true(attr(out, "show_p"))
  expect_true(attr(out, "show_statistic"))
})

test_that("table_continuous statistic default output carries show_statistic attribute", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    statistic = TRUE,
    p_value = FALSE
  )
  expect_false(attr(out, "show_p"))
  expect_true(attr(out, "show_statistic"))
})

test_that("table_continuous p_value auto-shows when by is supplied", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    output = "data.frame"
  )
  expect_true("p.value" %in% names(out))
  expect_true(attr(out, "show_p"))
  expect_false(is.na(out$p.value[1]))
})

test_that("table_continuous p_value stays hidden when by is absent", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    output = "data.frame"
  )
  expect_false("p.value" %in% names(out))
  expect_false(isTRUE(attr(out, "show_p")))
})

test_that("table_continuous p_value = FALSE suppresses column when by is supplied", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = FALSE,
    output = "data.frame"
  )
  expect_false("p.value" %in% names(out))
  expect_false(attr(out, "show_p"))
})

test_that("table_continuous print works with statistic only", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    statistic = TRUE
  )
  expect_output(print(out))
})

test_that("table_continuous print works with p_value only", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = TRUE
  )
  expect_output(print(out))
})

test_that("table_continuous print works with p_value + statistic", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = TRUE,
    statistic = TRUE
  )
  expect_output(print(out))
})

# ---- test method ----

test_that("table_continuous test='welch' is the default (2 groups)", {
  df <- data.frame(g = rep(c("A", "B"), each = 20), x = rnorm(40))
  out <- table_continuous(df, by = g, p_value = TRUE, output = "data.frame")
  expect_equal(out$test_type[1], "welch_t")
})
test_that("table_continuous test='welch' with 3+ groups uses welch_anova", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = TRUE,
    output = "data.frame"
  )
  expect_equal(out$test_type[1], "welch_anova")
})

test_that("table_continuous test='student' uses student_t for 2 groups", {
  df <- data.frame(g = rep(c("A", "B"), each = 20), x = rnorm(40))
  out <- table_continuous(
    df,
    by = g,
    test = "student",
    p_value = TRUE,
    output = "data.frame"
  )
  expect_equal(out$test_type[1], "student_t")
})

test_that("table_continuous test='student' uses anova for 3+ groups", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    test = "student",
    p_value = TRUE,
    output = "data.frame"
  )
  expect_equal(out$test_type[1], "anova")
  expect_false(is.na(out$df2[1]))
})

test_that("table_continuous test='nonparametric' uses wilcoxon for 2 groups", {
  df <- data.frame(g = rep(c("A", "B"), each = 20), x = rnorm(40))
  out <- table_continuous(
    df,
    by = g,
    test = "nonparametric",
    p_value = TRUE,
    output = "data.frame"
  )
  expect_equal(out$test_type[1], "wilcoxon")
  expect_true(is.na(out$df1[1]))
})

test_that("table_continuous test='nonparametric' uses kruskal for 3+ groups", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    test = "nonparametric",
    p_value = TRUE,
    output = "data.frame"
  )
  expect_equal(out$test_type[1], "kruskal")
  expect_false(is.na(out$df1[1]))
})

test_that("table_continuous test attribute is stored", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    test = "student",
    p_value = TRUE
  )
  expect_equal(attr(out, "test"), "student")
})

test_that("table_continuous test attribute is NA without by", {
  expect_warning(
    out <- table_continuous(iris, select = Sepal.Length, p_value = TRUE),
    "ignored"
  )
  expect_true(is.na(attr(out, "test")))
})

test_that("table_continuous warns when test is set without p_value/statistic", {
  expect_warning(
    table_continuous(
      iris,
      select = Sepal.Length,
      by = Species,
      test = "student",
      p_value = FALSE
    ),
    "ignored"
  )
})

test_that("table_continuous nonparametric statistic display uses W and H", {
  df2 <- data.frame(g = rep(c("A", "B"), each = 20), x = rnorm(40))
  out2 <- table_continuous(
    df2,
    by = g,
    test = "nonparametric",
    statistic = TRUE,
    output = "data.frame"
  )
  display2 <- spicy:::build_display_df(
    out2,
    2L,
    ".",
    0.95,
    show_p = FALSE,
    show_statistic = TRUE
  )
  expect_match(display2$Test[1], "^W = ")

  out3 <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    test = "nonparametric",
    statistic = TRUE,
    output = "data.frame"
  )
  display3 <- spicy:::build_display_df(
    out3,
    2L,
    ".",
    0.95,
    show_p = FALSE,
    show_statistic = TRUE
  )
  expect_match(display3$Test[1], "^H\\(")
})

test_that("table_continuous student statistic display uses t and F", {
  df2 <- data.frame(g = rep(c("A", "B"), each = 20), x = rnorm(40))
  out2 <- table_continuous(
    df2,
    by = g,
    test = "student",
    statistic = TRUE,
    output = "data.frame"
  )
  display2 <- spicy:::build_display_df(
    out2,
    2L,
    ".",
    0.95,
    show_p = FALSE,
    show_statistic = TRUE
  )
  expect_match(display2$Test[1], "^t\\(")

  out3 <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    test = "student",
    statistic = TRUE,
    output = "data.frame"
  )
  display3 <- spicy:::build_display_df(
    out3,
    2L,
    ".",
    0.95,
    show_p = FALSE,
    show_statistic = TRUE
  )
  expect_match(display3$Test[1], "^F\\(")
})

test_that("table_continuous test='nonparametric' p-values match base R", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    test = "nonparametric",
    p_value = TRUE,
    output = "data.frame"
  )
  ref <- kruskal.test(Sepal.Length ~ Species, data = iris)
  expect_equal(out$p.value[1], ref$p.value)
})

# ---- formatting ----

test_that("table_continuous decimal_mark attribute is set", {
  out <- table_continuous(iris, select = Sepal.Length, decimal_mark = ",")
  expect_equal(attr(out, "decimal_mark"), ",")
})

test_that("table_continuous decimal_mark comma in display output", {
  df <- data.frame(x = c(1.5, 2.5, 3.5))
  out <- table_continuous(df, decimal_mark = ",")
  display <- spicy:::build_display_df(out, 2L, ",", 0.95)
  expect_true(any(grepl(",", display$M)))
  expect_false(any(grepl("\\.", display$M)))
})

test_that("table_continuous digits parameter controls precision", {
  df <- data.frame(x = c(1.123456, 2.654321, 3.987654))
  d0 <- spicy:::build_display_df(
    table_continuous(df, digits = 0),
    0L,
    ".",
    0.95
  )
  d4 <- spicy:::build_display_df(
    table_continuous(df, digits = 4),
    4L,
    ".",
    0.95
  )
  expect_false(grepl("\\.", d0$M[1]))
  expect_match(d4$M[1], "\\.[0-9]{4}$")
})

test_that("table_continuous uses dedicated digits for effect sizes", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    effect_size_ci = TRUE,
    digits = 1,
    effect_size_digits = 3
  )
  display <- spicy:::build_display_df(
    out,
    1L,
    ".",
    0.95,
    show_effect_size = TRUE,
    show_effect_size_ci = TRUE,
    effect_size_digits = 3L
  )

  expect_equal(attr(out, "effect_size_digits"), 3L)
  expect_match(display$M[1], "\\.[0-9]{1}$")
  expect_match(display$ES[1], "= [0-9]+\\.[0-9]{3}")
  expect_match(display$ES[1], "\\[[0-9]+\\.[0-9]{3}, [0-9]+\\.[0-9]{3}\\]")
})

# ---- printing ----

test_that("print.spicy_continuous_table produces output", {
  out <- table_continuous(iris, select = c(Sepal.Length, Sepal.Width))
  expect_output(print(out))
})

test_that("print.spicy_continuous_table works with groups", {
  out <- table_continuous(iris, select = Sepal.Length, by = Species)
  expect_output(print(out))
})

test_that("print.spicy_continuous_table returns invisible x", {
  out <- table_continuous(iris, select = Sepal.Length)
  ret <- withVisible(print(out))
  expect_false(ret$visible)
  expect_s3_class(ret$value, "spicy_continuous_table")
})

# ---- validation ----

test_that("table_continuous errors on non-data-frame", {
  expect_error(table_continuous(1:10), "data\\.frame")
})

test_that("table_continuous validates ci_level", {
  df <- data.frame(x = 1:5)
  expect_error(table_continuous(df, ci_level = 2), "ci_level")
  expect_error(table_continuous(df, ci_level = 0), "ci_level")
  expect_error(table_continuous(df, ci_level = -0.5), "ci_level")
  expect_error(table_continuous(df, ci_level = NA), "ci_level")
  expect_error(table_continuous(df, ci_level = "a"), "ci_level")
})

test_that("table_continuous validates digits", {
  df <- data.frame(x = 1:5)
  expect_error(table_continuous(df, digits = -1), "digits")
  expect_error(table_continuous(df, digits = "a"), "digits")
  expect_error(table_continuous(df, digits = NA), "digits")
  expect_error(
    table_continuous(df, effect_size_digits = -1),
    "effect_size_digits"
  )
  expect_error(
    table_continuous(df, effect_size_digits = "a"),
    "effect_size_digits"
  )
  expect_error(
    table_continuous(df, effect_size_digits = NA),
    "effect_size_digits"
  )
})

test_that("table_continuous validates decimal_mark", {
  df <- data.frame(x = 1:5)
  expect_error(table_continuous(df, decimal_mark = ";"), "decimal_mark")
})

test_that("table_continuous validates labels", {
  df <- data.frame(x = 1:5)
  expect_error(table_continuous(df, labels = c("a", "b")), "labels")
  expect_error(table_continuous(df, labels = 42), "labels")
})

test_that("table_continuous validates by", {
  df <- data.frame(x = 1:5)
  expect_error(table_continuous(df, by = nonexistent), "by")
})

test_that("table_continuous validates regex select", {
  df <- data.frame(x = 1:5)
  expect_error(
    table_continuous(df, select = c("a", "b"), regex = TRUE),
    "single character pattern"
  )
  expect_error(
    table_continuous(df, select = NA_character_, regex = TRUE),
    "single character pattern"
  )
})

test_that("table_continuous validates logical parameters", {
  df <- data.frame(x = 1:5)
  expect_error(table_continuous(df, p_value = "yes"), "p_value")
  expect_error(table_continuous(df, statistic = 1), "statistic")
  expect_error(table_continuous(df, effect_size = NA), "effect_size")
  expect_error(table_continuous(df, effect_size_ci = NULL), "effect_size_ci")
  expect_error(table_continuous(df, regex = "TRUE"), "regex")
  expect_error(table_continuous(df, verbose = NA), "verbose")
})

test_that("table_continuous warns when NA present in by column", {
  df <- data.frame(
    x = 1:6,
    g = c("A", "A", "B", "B", NA, NA)
  )
  expect_warning(
    table_continuous(df, select = "x", by = "g", output = "data.frame"),
    "2 observation.*excluded"
  )
  out <- suppressWarnings(
    table_continuous(df, select = "x", by = "g", output = "data.frame")
  )
  # Only A and B groups should be present
  expect_equal(sort(unique(out$group)), c("A", "B"))
})

test_that("fmt_p uses non-breaking space in display", {
  set.seed(1)
  df <- data.frame(
    x = c(rnorm(30, 0), rnorm(30, 10)),
    g = rep(c("A", "B"), each = 30)
  )
  out <- table_continuous(df, select = "x", by = "g", p_value = TRUE)
  # build_display_df is used by the print method; call it directly
  display <- spicy:::build_display_df(
    out,
    digits = attr(out, "digits"),
    decimal_mark = attr(out, "decimal_mark"),
    ci_level = attr(out, "ci_level"),
    show_p = TRUE,
    show_statistic = FALSE,
    show_effect_size = FALSE,
    show_effect_size_ci = FALSE
  )
  p_col <- display[["p"]][nzchar(display[["p"]])]
  # p should be very small -> "<\u00A0.001" with non-breaking space
  expect_true(any(grepl("<\u00A0", p_col)))
})

test_that("table_continuous warns on no numeric columns", {
  df <- data.frame(x = letters[1:5])
  expect_warning(table_continuous(df, output = "data.frame"), "No numeric")
})

test_that("table_continuous no-numeric warning returns empty data.frame", {
  df <- data.frame(x = letters[1:5])
  out <- suppressWarnings(table_continuous(df, output = "data.frame"))
  expect_s3_class(out, "data.frame")
  expect_equal(nrow(out), 0L)
})

# ---- optional outputs: tinytable ----

test_that("table_continuous tinytable output works", {
  skip_if_not_installed("tinytable")
  out <- table_continuous(iris, select = c(Sepal.Length), output = "tinytable")
  expect_true(methods::is(out, "tinytable"))
})

test_that("table_continuous tinytable with groups works", {
  skip_if_not_installed("tinytable")
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    output = "tinytable"
  )
  expect_true(methods::is(out, "tinytable"))
})

# ---- optional outputs: gt ----

test_that("table_continuous gt output works", {
  skip_if_not_installed("gt")
  out <- table_continuous(iris, select = c(Sepal.Length), output = "gt")
  expect_s3_class(out, "gt_tbl")
})

test_that("table_continuous gt with groups works", {
  skip_if_not_installed("gt")
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    output = "gt"
  )
  expect_s3_class(out, "gt_tbl")
})

test_that("table_continuous gt has spanners for all columns", {
  skip_if_not_installed("gt")
  out <- table_continuous(iris, select = c(Sepal.Length), output = "gt")
  spanners <- out[["_spanners"]]
  labels <- unlist(spanners$spanner_label)
  expect_true("Variable" %in% labels)
  expect_true("M" %in% labels)
  expect_true("SD" %in% labels)
  expect_true("n" %in% labels)
  expect_true(any(grepl("CI", labels)))
})

test_that("table_continuous gt with p_value only has p spanner but not Test", {
  skip_if_not_installed("gt")
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = TRUE,
    output = "gt"
  )
  spanners <- out[["_spanners"]]
  labels <- unlist(spanners$spanner_label)
  expect_true("p" %in% labels)
  expect_false("Test" %in% labels)
})

test_that("table_continuous gt with p_value + statistic has both spanners", {
  skip_if_not_installed("gt")
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = TRUE,
    statistic = TRUE,
    output = "gt"
  )
  spanners <- out[["_spanners"]]
  labels <- unlist(spanners$spanner_label)
  expect_true("p" %in% labels)
  expect_true("Test" %in% labels)
})

# ---- optional outputs: flextable ----

test_that("table_continuous flextable output works", {
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")
  out <- table_continuous(iris, select = c(Sepal.Length), output = "flextable")
  expect_s3_class(out, "flextable")
})

test_that("table_continuous flextable has 2-row header with CI spanner", {
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")
  out <- table_continuous(iris, select = c(Sepal.Length), output = "flextable")
  hdr <- out$header$dataset
  expect_equal(nrow(hdr), 2L)
  expect_true(any(grepl("CI", hdr[1, ])))
})

# ---- optional outputs: excel ----

test_that("table_continuous excel output works", {
  skip_if_not_installed("openxlsx2")
  tmp <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp), add = TRUE)
  out <- table_continuous(
    iris,
    select = c(Sepal.Length),
    output = "excel",
    excel_path = tmp
  )
  expect_true(file.exists(tmp))
})

test_that("table_continuous excel errors without path", {
  expect_error(
    table_continuous(iris, select = c(Sepal.Length), output = "excel"),
    "excel_path"
  )
})

test_that("table_continuous excel errors with empty path", {
  expect_error(
    table_continuous(
      iris,
      select = c(Sepal.Length),
      output = "excel",
      excel_path = ""
    ),
    "excel_path"
  )
})

# ---- optional outputs: word ----

test_that("table_continuous word output writes file", {
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")
  tmp <- tempfile(fileext = ".docx")
  on.exit(unlink(tmp), add = TRUE)
  out <- table_continuous(
    iris,
    select = c(Sepal.Length),
    output = "word",
    word_path = tmp
  )
  expect_true(file.exists(tmp))
})

test_that("table_continuous word errors without path", {
  expect_error(
    table_continuous(iris, select = c(Sepal.Length), output = "word"),
    "word_path"
  )
})

test_that("table_continuous word errors with empty path", {
  expect_error(
    table_continuous(
      iris,
      select = c(Sepal.Length),
      output = "word",
      word_path = ""
    ),
    "word_path"
  )
})

# ---- optional outputs: clipboard ----

test_that("table_continuous clipboard output works", {
  skip_if_not_installed("clipr")
  skip_if_not(clipr::clipr_available(), "Clipboard not available")
  expect_message(
    out <- table_continuous(
      iris,
      select = c(Sepal.Length),
      output = "clipboard"
    ),
    "copied to clipboard"
  )
  expect_s3_class(out, "data.frame")
})

test_that("table_continuous clipboard output can be exercised with a mocked writer", {
  skip_if_not_installed("clipr")

  captured <- NULL

  local_mocked_bindings(
    write_clip = function(text, ...) {
      captured <<- text
      invisible(text)
    },
    clipr_available = function(...) TRUE,
    .package = "clipr"
  )

  expect_message(
    out <- table_continuous(
      iris,
      select = c(Sepal.Length),
      output = "clipboard",
      clipboard_delim = ";"
    ),
    "copied to clipboard"
  )

  expect_s3_class(out, "data.frame")
  expect_true(is.character(captured))
  expect_match(captured, "Variable;M;SD")
  expect_match(captured, "Sepal.Length")
})

# ---- grouped optional outputs ----

test_that("table_continuous flextable with groups works", {
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    output = "flextable"
  )
  expect_s3_class(out, "flextable")
})

test_that("table_continuous excel with groups works", {
  skip_if_not_installed("openxlsx2")
  tmp <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp), add = TRUE)
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    output = "excel",
    excel_path = tmp
  )
  expect_true(file.exists(tmp))
})

test_that("table_continuous word with groups works", {
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")
  tmp <- tempfile(fileext = ".docx")
  on.exit(unlink(tmp), add = TRUE)
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    output = "word",
    word_path = tmp
  )
  expect_true(file.exists(tmp))
})

# ---- display format details ----

test_that("build_display_df ungrouped has correct column names", {
  df <- data.frame(x = 1:10)
  out <- table_continuous(df, ci_level = 0.90)
  display <- spicy:::build_display_df(out, 2L, ".", 0.90)
  expect_true("90% CI LL" %in% names(display))
  expect_true("90% CI UL" %in% names(display))
  expect_false("Group" %in% names(display))
})

test_that("build_display_df grouped has Group column", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
  )
  display <- spicy:::build_display_df(out, 2L, ".", 0.95)
  expect_true("Group" %in% names(display))
  expect_true("95% CI LL" %in% names(display))
  expect_true("95% CI UL" %in% names(display))
})

test_that("table_continuous multiple variables with groups has correct rows", {
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Sepal.Width),
    by = Species,
    output = "data.frame"
  )
  expect_equal(nrow(out), 6L)
  expect_equal(out$n[1], 50L)
})

test_that("table_continuous NAs in grouped data are handled", {
  df <- data.frame(
    g = c("A", "A", "B", "B", "B"),
    x = c(1, NA, 3, NA, 5)
  )
  out <- table_continuous(df, by = g, output = "data.frame")
  a_row <- out[out$group == "A", ]
  b_row <- out[out$group == "B", ]
  expect_equal(a_row$n, 1L)
  expect_equal(b_row$n, 2L)
  expect_equal(b_row$mean, mean(c(3, 5)))
})

test_that("table_continuous gt grouped output has Group column aligned left", {
  skip_if_not_installed("gt")
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    output = "gt"
  )
  dat <- out[["_data"]]
  expect_true("Group" %in% names(dat))
})

test_that("table_continuous tinytable grouped has correct number of rows", {
  skip_if_not_installed("tinytable")
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Sepal.Width),
    by = Species,
    output = "tinytable"
  )
  expect_true(methods::is(out, "tinytable"))
})

# ---- by selecting multiple columns ----

test_that("table_continuous errors when by selects multiple columns", {
  df <- data.frame(a = 1:6, b = rep(1:2, 3), c = rep(1:3, 2), x = 11:16)
  expect_error(table_continuous(df, by = c(b, c), output = "data.frame"))
})


# ---- grouped clipboard ----

test_that("table_continuous clipboard with groups works", {
  skip_if_not_installed("clipr")
  skip_if_not(clipr::clipr_available(), "Clipboard not available")
  expect_message(
    out <- table_continuous(
      iris,
      select = Sepal.Length,
      by = Species,
      output = "clipboard"
    ),
    "copied to clipboard"
  )
  expect_s3_class(out, "data.frame")
  expect_true("Group" %in% names(out))
})

# ---- non-default clipboard_delim ----

test_that("table_continuous clipboard with custom delimiter works", {
  skip_if_not_installed("clipr")
  skip_if_not(clipr::clipr_available(), "Clipboard not available")
  expect_message(
    out <- table_continuous(
      iris,
      select = Sepal.Length,
      output = "clipboard",
      clipboard_delim = ";"
    ),
    "copied to clipboard"
  )
  expect_s3_class(out, "data.frame")
})

# ---- print method %||% fallback branches ----

# ---- effect size ----

test_that("table_continuous effect_size=TRUE adds es columns (welch, 2 groups)", {
  df <- iris[iris$Species != "virginica", ]
  df$Species <- droplevels(df$Species)
  out <- table_continuous(
    df,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    output = "data.frame"
  )
  expect_true("es_type" %in% names(out))
  expect_true("es_value" %in% names(out))
  expect_true("es_ci_lower" %in% names(out))
  expect_true("es_ci_upper" %in% names(out))
  expect_equal(out$es_type[1], "hedges_g")
  expect_false(is.na(out$es_value[1]))
})

test_that("table_continuous Hedges' g matches manual calculation", {
  df <- iris[iris$Species != "virginica", ]
  df$Species <- droplevels(df$Species)
  out <- table_continuous(
    df,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    output = "data.frame"
  )
  x1 <- df$Sepal.Length[df$Species == "setosa"]
  x2 <- df$Sepal.Length[df$Species == "versicolor"]
  n1 <- length(x1)
  n2 <- length(x2)
  s_pooled <- sqrt(
    ((n1 - 1) * var(x1) + (n2 - 1) * var(x2)) / (n1 + n2 - 2)
  )
  d <- (mean(x1) - mean(x2)) / s_pooled
  g_manual <- d * (1 - 3 / (4 * (n1 + n2 - 2) - 1))
  expect_equal(out$es_value[1], g_manual)
})

test_that("table_continuous eta-squared for 3+ groups (welch)", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    output = "data.frame"
  )
  expect_equal(out$es_type[1], "eta_sq")
  expect_false(is.na(out$es_ci_lower[1]))
  expect_false(is.na(out$es_ci_upper[1]))
  # Manual eta-squared
  grand_mean <- mean(iris$Sepal.Length)
  ss_between <- sum(tapply(
    iris$Sepal.Length,
    iris$Species,
    \(x) length(x) * (mean(x) - grand_mean)^2
  ))
  ss_total <- sum((iris$Sepal.Length - grand_mean)^2)
  expect_equal(out$es_value[1], ss_between / ss_total)
})

test_that("table_continuous nonparametric 2 groups gives rank-biserial r", {
  df <- iris[iris$Species != "virginica", ]
  df$Species <- droplevels(df$Species)
  out <- table_continuous(
    df,
    select = Sepal.Length,
    by = Species,
    test = "nonparametric",
    effect_size = TRUE,
    output = "data.frame"
  )
  expect_equal(out$es_type[1], "r_rb")
  expect_false(is.na(out$es_ci_lower[1]))
  expect_false(is.na(out$es_ci_upper[1]))
})

test_that("table_continuous nonparametric 3+ groups gives epsilon-squared", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    test = "nonparametric",
    effect_size = TRUE,
    output = "data.frame"
  )
  expect_equal(out$es_type[1], "epsilon_sq")
  expect_false(is.na(out$es_ci_lower[1]))
  expect_false(is.na(out$es_ci_upper[1]))
})

test_that("table_continuous effect_size without by warns", {
  expect_warning(
    out <- table_continuous(
      iris,
      select = Sepal.Length,
      effect_size = TRUE,
      output = "data.frame"
    ),
    "ignored"
  )
  expect_false("es_type" %in% names(out))
})

test_that("table_continuous effect_size_ci without effect_size warns and enables it", {
  expect_warning(
    out <- table_continuous(
      iris,
      select = Sepal.Length,
      by = Species,
      effect_size_ci = TRUE,
      output = "data.frame"
    ),
    "effect_size_ci"
  )
  expect_true("es_type" %in% names(out))
})

test_that("table_continuous effect_size default output carries attributes", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    effect_size_ci = TRUE
  )
  expect_true(attr(out, "show_effect_size"))
  expect_true(attr(out, "show_effect_size_ci"))
})

test_that("table_continuous effect_size display shows ES column", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
  )
  display <- spicy:::build_display_df(
    out,
    2L,
    ".",
    0.95,
    show_effect_size = TRUE
  )
  expect_true("ES" %in% names(display))
  expect_match(display$ES[1], "=")
})

test_that("table_continuous effect_size_ci display shows brackets", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    effect_size_ci = TRUE,
  )
  display <- spicy:::build_display_df(
    out,
    2L,
    ".",
    0.95,
    show_effect_size = TRUE,
    show_effect_size_ci = TRUE
  )
  expect_true("ES" %in% names(display))
  expect_match(display$ES[1], "\\[")
})

test_that("table_continuous print works with effect_size", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    effect_size_ci = TRUE
  )
  expect_output(print(out))
})

test_that("table_continuous es values only on first row of each variable block", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    output = "data.frame"
  )
  # 3 groups x 1 var = 3 rows; es_value only on first row
  expect_false(is.na(out$es_value[1]))
  expect_true(is.na(out$es_value[2]))
  expect_true(is.na(out$es_value[3]))
})

test_that("table_continuous gt with effect_size has ES spanner", {
  skip_if_not_installed("gt")
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    output = "gt"
  )
  spanners <- out[["_spanners"]]
  labels <- unlist(spanners$spanner_label)
  expect_true("ES" %in% labels)
})

# ---- effect size: do_es without do_test (no p_value/statistic) ----

test_that("table_continuous effect_size=TRUE alone adds es but not test columns", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    output = "data.frame"
  )
  expect_true("es_type" %in% names(out))
  expect_true("es_value" %in% names(out))
  expect_true("test_type" %in% names(out))
  expect_false(is.na(out$es_value[1]))
  # display should not show Test/p when show_p/show_statistic are FALSE
  display <- spicy:::build_display_df(
    out,
    2L,
    ".",
    0.95,
    show_effect_size = TRUE
  )
  expect_true("ES" %in% names(display))
  expect_false("Test" %in% names(display))
  expect_false("p" %in% names(display))
})

test_that("table_continuous does not warn about test when effect_size uses it", {
  expect_no_warning(
    out <- table_continuous(
      iris,
      select = Sepal.Length,
      by = Species,
      test = "student",
      effect_size = TRUE,
      p_value = FALSE,
      statistic = FALSE,
      output = "data.frame"
    )
  )

  expect_equal(out$es_type[1], "eta_sq")
  expect_equal(out$test_type[1], "anova")
})

# ---- effect size: student test ----

test_that("table_continuous effect_size with test='student' 2 groups gives hedges_g", {
  df <- iris[iris$Species != "virginica", ]
  df$Species <- droplevels(df$Species)
  out <- table_continuous(
    df,
    select = Sepal.Length,
    by = Species,
    test = "student",
    effect_size = TRUE,
    output = "data.frame"
  )
  expect_equal(out$es_type[1], "hedges_g")
  expect_equal(out$test_type[1], "student_t")
})

test_that("table_continuous effect_size with test='student' 3+ groups gives eta_sq", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    test = "student",
    effect_size = TRUE,
    output = "data.frame"
  )
  expect_equal(out$es_type[1], "eta_sq")
  expect_equal(out$test_type[1], "anova")
})

# ---- effect size: decimal_mark comma ----

test_that("table_continuous effect_size display uses comma decimal_mark", {
  df <- iris[iris$Species != "virginica", ]
  df$Species <- droplevels(df$Species)
  out <- table_continuous(
    df,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    effect_size_ci = TRUE,
    decimal_mark = ",",
  )
  display <- spicy:::build_display_df(
    out,
    2L,
    ",",
    0.95,
    show_effect_size = TRUE,
    show_effect_size_ci = TRUE
  )
  es_cell <- display$ES[1]
  expect_true(grepl(",", es_cell))
})

# ---- effect size: untestable groups (n=1) ----

test_that("table_continuous effect_size with untestable group gives NA", {
  df <- data.frame(
    g = c("A", "B"),
    x = c(1, 2)
  )
  out <- table_continuous(
    df,
    by = g,
    effect_size = TRUE,
    output = "data.frame"
  )
  expect_true("es_type" %in% names(out))
  expect_true(is.na(out$es_value[1]))
})

# ---- effect size: nonparametric rank-biserial CI small n ----

test_that("table_continuous rank-biserial r_rb has CI with small n", {
  df <- data.frame(
    g = c("A", "A", "B", "B"),
    x = c(1, 2, 3, 4)
  )
  out <- table_continuous(
    df,
    by = g,
    test = "nonparametric",
    effect_size = TRUE,
    output = "data.frame"
  )
  expect_equal(out$es_type[1], "r_rb")
  # n_total = 4 > 3, so CI should be computed
  expect_false(is.na(out$es_ci_lower[1]))
})

# ---- effect size: multiple variables ----

test_that("table_continuous effect_size with multiple variables", {
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Sepal.Width),
    by = Species,
    effect_size = TRUE,
    output = "data.frame"
  )
  sl_es <- out$es_value[out$variable == "Sepal.Length" & !is.na(out$es_value)]
  sw_es <- out$es_value[out$variable == "Sepal.Width" & !is.na(out$es_value)]
  expect_length(sl_es, 1L)
  expect_length(sw_es, 1L)
  expect_false(identical(sl_es, sw_es))
})

# ---- effect size: export formats ----

test_that("table_continuous tinytable with effect_size works", {
  skip_if_not_installed("tinytable")
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    effect_size_ci = TRUE,
    output = "tinytable"
  )
  expect_true(methods::is(out, "tinytable"))
})

test_that("table_continuous flextable with effect_size works", {
  skip_if_not_installed("flextable")
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    output = "flextable"
  )
  expect_s3_class(out, "flextable")
})

test_that("table_continuous excel with effect_size works", {
  skip_if_not_installed("openxlsx2")
  tmp <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp), add = TRUE)
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    output = "excel",
    excel_path = tmp
  )
  expect_true(file.exists(tmp))
})

test_that("table_continuous word with effect_size works", {
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")
  tmp <- tempfile(fileext = ".docx")
  on.exit(unlink(tmp), add = TRUE)
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    effect_size_ci = TRUE,
    output = "word",
    word_path = tmp
  )
  expect_true(file.exists(tmp))
})

test_that("table_continuous gt with effect_size_ci has ES spanner", {
  skip_if_not_installed("gt")
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
    effect_size_ci = TRUE,
    output = "gt"
  )
  dat <- out[["_data"]]
  expect_true("ES" %in% names(dat))
})

# ---- effect size: epsilon_sq_boot_ci internals ----

test_that("epsilon_sq_boot_ci returns valid CI", {
  set.seed(1)
  x <- c(rnorm(30, 0), rnorm(30, 1), rnorm(30, 2))
  g <- rep(c("A", "B", "C"), each = 30)
  ci <- spicy:::epsilon_sq_boot_ci(x, g, 3L, 0.95)
  expect_length(ci, 2L)
  expect_true(ci[1] < ci[2])
})

test_that("epsilon_sq_boot_ci handles resamples that lose a group", {
  # Small groups: many resamples will miss a group (triggers NA branch)
  x <- c(1, 2, 10, 11, 20, 21)
  g <- rep(c("A", "B", "C"), each = 2)
  ci <- spicy:::epsilon_sq_boot_ci(x, g, 3L, 0.95)
  expect_length(ci, 2L)
  # Still enough valid resamples to produce a CI
  expect_false(is.na(ci[1]))
})

test_that("epsilon_sq_boot_ci returns NA when too few valid resamples", {
  # Many groups with 1 obs each: most resamples miss groups
  x <- 1:8
  g <- paste0("G", 1:8)
  ci <- spicy:::epsilon_sq_boot_ci(x, g, 8L, 0.95)
  expect_length(ci, 2L)
  expect_true(is.na(ci[1]))
  expect_true(is.na(ci[2]))
})

test_that("epsilon_sq_boot_ci does not alter user RNG state", {
  set.seed(123)
  before <- .Random.seed
  x <- c(rnorm(30, 0), rnorm(30, 1), rnorm(30, 2))
  g <- rep(c("A", "B", "C"), each = 30)
  set.seed(99)
  rng_before <- .Random.seed
  spicy:::epsilon_sq_boot_ci(x, g, 3L, 0.95)
  rng_after <- .Random.seed
  # RNG should have advanced (not been reset to a fixed seed)
  expect_false(identical(rng_before, rng_after))
})

test_that("epsilon squared is clamped to 0 when H is small", {
  # Create data where groups are nearly identical -> H ≈ 0 -> raw ε² < 0
  set.seed(1)
  x <- rnorm(90, mean = 5, sd = 10)
  g <- rep(c("A", "B", "C"), each = 30)
  out <- table_continuous(
    data.frame(x = x, g = g),
    select = "x",
    by = "g",
    test = "nonparametric",
    effect_size = TRUE
  )
  eps_vals <- out$es_value[!is.na(out$es_value)]
  expect_true(all(eps_vals >= 0))
})

test_that("eta_sq_ci returns lower = 0 for very small F", {
  # F close to 0: ncp_lower should be 0
  ci <- spicy:::eta_sq_ci(0.5, df1 = 2, df2 = 100, ci_level = 0.95)
  expect_equal(ci[1], 0)
  expect_false(is.na(ci[2]))
})

# ---- effect size: fmt_es empty when NA ----

test_that("table_continuous ES display is empty for subsequent group rows", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = TRUE,
  )
  display <- spicy:::build_display_df(
    out,
    2L,
    ".",
    0.95,
    show_effect_size = TRUE
  )
  expect_equal(display$ES[2], "")
  expect_equal(display$ES[3], "")
})

# ---- effect size: p_value + statistic + effect_size together ----

test_that("table_continuous all columns together: p + stat + es", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = TRUE,
    statistic = TRUE,
    effect_size = TRUE,
    effect_size_ci = TRUE,
  )
  display <- spicy:::build_display_df(
    out,
    2L,
    ".",
    0.95,
    show_p = TRUE,
    show_statistic = TRUE,
    show_effect_size = TRUE,
    show_effect_size_ci = TRUE
  )
  expect_true(all(c("Test", "p", "ES") %in% names(display)))
  expect_match(display$ES[1], "\\[")
})

test_that("table_continuous print works with all columns", {
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Sepal.Width),
    by = Species,
    p_value = TRUE,
    statistic = TRUE,
    effect_size = TRUE,
    effect_size_ci = TRUE
  )
  expect_output(print(out))
})

# ---- print: auto-compact padding ----

test_that("print auto-selects compact padding for narrow console", {
  out <- table_continuous(iris, select = Sepal.Length, by = Species)
  old_w <- getOption("width")
  on.exit(options(width = old_w), add = TRUE)
  # Normal padding should produce wider output than compact
  options(width = 80)
  expect_output(print(out))
  # Very narrow: still prints without error
  options(width = 40)
  expect_output(print(out))
})

# ---- print method %||% fallback branches ----

test_that("print.spicy_continuous_table uses defaults when attributes are missing", {
  df <- data.frame(x = 1:10)
  out <- table_continuous(df)
  # Strip attributes to test fallback branches
  attr(out, "digits") <- NULL
  attr(out, "decimal_mark") <- NULL
  attr(out, "ci_level") <- NULL
  attr(out, "data_name") <- NULL
  expect_output(print(out))
})

# ---- coverage: multi-variable exports with separator rows ----

test_that("table_continuous gt with multiple variables has separator rows", {
  skip_if_not_installed("gt")
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Sepal.Width),
    by = Species,
    p_value = TRUE,
    statistic = TRUE,
    effect_size = TRUE,
    output = "gt"
  )
  expect_s3_class(out, "gt_tbl")
})

test_that("table_continuous flextable with multiple variables has separator rows", {
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Sepal.Width),
    by = Species,
    p_value = TRUE,
    output = "flextable"
  )
  expect_s3_class(out, "flextable")
})

test_that("table_continuous excel with multiple variables has separator rows", {
  skip_if_not_installed("openxlsx2")
  tmp <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp), add = TRUE)
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Sepal.Width),
    by = Species,
    p_value = TRUE,
    output = "excel",
    excel_path = tmp
  )
  expect_true(file.exists(tmp))
})

test_that("table_continuous tinytable with p + stat + es works", {
  skip_if_not_installed("tinytable")
  out <- table_continuous(
    iris,
    select = c(Sepal.Length, Sepal.Width),
    by = Species,
    p_value = TRUE,
    statistic = TRUE,
    effect_size = TRUE,
    output = "tinytable"
  )
  expect_true(methods::is(out, "tinytable"))
})

# ---- coverage: display formatting branches ----

test_that("build_display_df formats p >= 0.001 correctly", {
  df <- data.frame(g = rep(c("A", "B"), each = 30), x = rnorm(60))
  out <- table_continuous(df, by = g, p_value = TRUE)
  display <- spicy:::build_display_df(
    out,
    2L,
    ".",
    0.95,
    show_p = TRUE
  )
  # p might be >= 0.001 for random data, but ensure column exists
  expect_true("p" %in% names(display))
})

test_that("build_display_df fmt_test with decimal comma", {
  df <- data.frame(g = rep(c("A", "B"), each = 20), x = rnorm(40))
  out <- table_continuous(
    df,
    by = g,
    statistic = TRUE,
    decimal_mark = ",",
  )
  display <- spicy:::build_display_df(
    out,
    2L,
    ",",
    0.95,
    show_statistic = TRUE
  )
  # t-test statistic should use comma
  expect_true(grepl(",", display$Test[1]))
})

test_that("build_display_df fmt_p with decimal comma", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    p_value = TRUE,
    decimal_mark = ",",
  )
  display <- spicy:::build_display_df(
    out,
    2L,
    ",",
    0.95,
    show_p = TRUE
  )
  # Very small p: "< ,001"
  expect_match(display$p[1], ",")
})

test_that("build_display_df fmt_p large p with decimal comma", {
  set.seed(123)
  df <- data.frame(
    g = rep(c("A", "B"), each = 20),
    x = rnorm(40, mean = 5, sd = 2)
  )
  out <- table_continuous(
    df,
    by = g,
    p_value = TRUE,
    decimal_mark = ",",
  )
  display <- spicy:::build_display_df(
    out,
    2L,
    ",",
    0.95,
    show_p = TRUE
  )
  # p >= 0.001 should be formatted with comma
  expect_true(grepl(",", display$p[1]))
})

test_that("build_display_df fmt_test F-test with decimal comma", {
  out <- table_continuous(
    iris,
    select = Sepal.Length,
    by = Species,
    statistic = TRUE,
    decimal_mark = ",",
  )
  display <- spicy:::build_display_df(
    out,
    2L,
    ",",
    0.95,
    show_statistic = TRUE
  )
  expect_true(grepl(",", display$Test[1]))
})

varlist_title <- function(expr, selectors_used = FALSE) {
  source <- varlist_expr_source(expr)

  if (is.null(source)) {
    return("vl: <data>")
  }

  transformed <- selectors_used || source$transformed
  paste("vl:", if (transformed) paste0(source$name, "*") else source$name)
}


varlist_expr_source <- function(expr) {
  tryCatch(
    varlist_expr_source_impl(expr),
    error = function(e) NULL
  )
}


varlist_expr_source_impl <- function(expr) {
  if (is.symbol(expr)) {
    return(list(name = as.character(expr), transformed = FALSE))
  }

  if (!is.call(expr)) {
    return(NULL)
  }

  call_name <- varlist_call_name(expr[[1]])
  args <- as.list(expr)[-1]

  if (length(args) == 0L || is.null(call_name)) {
    return(NULL)
  }

  if (call_name %in% c("get", "get0")) {
    return(varlist_lookup_source(args))
  }

  if (call_name %in% c("[", "[[", "$", "@")) {
    return(varlist_transformed_source(args[[1L]]))
  }

  if (call_name %in% c("|>", "%>%", "%T>%", "%<>%")) {
    return(varlist_transformed_source(args[[1L]]))
  }

  if (call_name %in% varlist_data_first_calls()) {
    return(varlist_transformed_source(args[[1L]]))
  }

  if (length(args) == 1L) {
    return(varlist_transformed_source(args[[1L]]))
  }

  NULL
}


varlist_lookup_source <- function(args) {
  object_name <- args[[1L]]

  if (
    !is.character(object_name) ||
      length(object_name) != 1L ||
      is.na(object_name) ||
      !nzchar(object_name)
  ) {
    return(NULL)
  }

  list(name = object_name, transformed = FALSE)
}


varlist_transformed_source <- function(expr) {
  source <- varlist_expr_source(expr)

  if (is.null(source)) {
    return(NULL)
  }

  source$transformed <- TRUE
  source
}


varlist_call_name <- function(fun) {
  if (is.symbol(fun)) {
    return(as.character(fun))
  }

  if (!is.call(fun)) {
    return(NULL)
  }

  parts <- as.character(fun)
  if (length(parts) == 0L) {
    return(NULL)
  }

  if (length(parts) >= 3L && parts[[1L]] %in% c("::", ":::")) {
    return(utils::tail(parts, 1L)[[1L]])
  }

  parts[[1L]]
}


varlist_data_first_calls <- function() {
  c(
    "as.data.frame",
    "as_tibble",
    "arrange",
    "distinct",
    "drop_na",
    "droplevels",
    "filter",
    "group_by",
    "head",
    "mutate",
    "na.omit",
    "relocate",
    "rename",
    "select",
    "slice",
    "subset",
    "summarise",
    "summarize",
    "tail",
    "transmute",
    "ungroup",
    "within"
  )
}

validate_code_book_title <- function(title) {
  if (is.null(title)) {
    return(invisible(title))
  }

  if (
    !is.character(title) ||
      length(title) != 1L ||
      is.na(title) ||
      !nzchar(trimws(title))
  ) {
    stop(
      "`title` must be NULL or a single non-empty character string.",
      call. = FALSE
    )
  }

  invisible(title)
}


validate_code_book_filename_arg <- function(filename) {
  if (is.null(filename)) {
    return(invisible(filename))
  }

  if (
    !is.character(filename) ||
      length(filename) != 1L ||
      is.na(filename) ||
      !nzchar(trimws(filename))
  ) {
    stop(
      "`filename` must be NULL or a single non-empty character string.",
      call. = FALSE
    )
  }

  invisible(filename)
}


validate_code_book_control_dots <- function(dots) {
  dot_names <- names(dots)

  if (is.null(dot_names)) {
    return(invisible(dots))
  }

  dot_names[is.na(dot_names)] <- ""
  named_idx <- which(nzchar(dot_names))

  if (length(named_idx) == 0L) {
    return(invisible(dots))
  }

  controls <- c("values", "include_na", "title", "filename", "factor_levels")
  named_dots <- dot_names[named_idx]
  partial_controls <- vapply(
    named_dots,
    function(nm) any(startsWith(controls, nm)) && !nm %in% controls,
    logical(1)
  )
  literal_values <- vapply(
    dots[named_idx],
    function(quo) {
      expr <- rlang::quo_get_expr(quo)
      is.null(expr) || (is.atomic(expr) && length(expr) == 1L)
    },
    logical(1)
  )
  suspect_idx <- which(partial_controls & literal_values)

  if (length(suspect_idx) > 0L) {
    arg <- named_dots[[suspect_idx[[1L]]]]
    option <- controls[startsWith(controls, arg)][[1L]]
    stop(
      "`",
      arg,
      "` was supplied through `...`. ",
      "Use `",
      option,
      " = ...` exactly for this `code_book()` option; ",
      "`...` is reserved for tidyselect column selectors.",
      call. = FALSE
    )
  }

  invisible(dots)
}

if (!requireNamespace("covr", quietly = TRUE)) {
  stop("Install package 'covr' to run this script.", call. = FALSE)
}
if (!requireNamespace("devtools", quietly = TRUE)) {
  stop("Install package 'devtools' to run this script.", call. = FALSE)
}
if (!requireNamespace("testthat", quietly = TRUE)) {
  stop("Install package 'testthat' to run this script.", call. = FALSE)
}

suppressPackageStartupMessages(devtools::load_all(quiet = TRUE))

source_file <- "R/table_continuous_lm.R"
test_files <- c(
  "tests/testthat/test-table_continuous_lm.R",
  "tools/coverage_table_continuous_lm_probes.R"
)

cov <- covr::file_coverage(source_file, test_files)
print(cov)

zero <- covr::zero_coverage(cov)

if (nrow(zero) == 0L) {
  cat("\nNo uncovered lines reported by covr.\n")
} else {
  zero <- unique(zero[, c("functions", "line")])
  zero <- zero[order(zero$functions, zero$line), , drop = FALSE]
  cat("\nUncovered lines:\n")
  print(zero, row.names = FALSE)
}

#' Simulated social-health survey
#'
#' A simulated dataset of 1200 respondents from a fictional
#' social-health survey, designed to illustrate the main features of
#' the spicy package: variable labels, ordered factors, survey
#' weights, association measures, and APA-style reporting.
#'
#' All variables carry labels (accessible via [labelled::var_label()]
#' and displayed by [varlist()]). Several ordered factors are included
#' so that [cross_tab()] can demonstrate automatic ordinal measure
#' selection.
#'
#' @format A tibble with 1200 rows and 24 variables:
#' \describe{
#'   \item{sex}{Factor. Sex of the respondent.}
#'   \item{age}{Numeric. Age in years (25--75).}
#'   \item{age_group}{Ordered factor. Age group
#'     (25--34, 35--49, 50--64, 65--75).}
#'   \item{education}{Ordered factor. Highest education level
#'     (Lower secondary, Upper secondary, Tertiary).}
#'   \item{social_class}{Ordered factor. Subjective social class
#'     (Lower, Working, Lower middle, Middle, Upper middle).}
#'   \item{region}{Factor. Region of residence (6 regions).}
#'   \item{employment_status}{Factor. Employment status
#'     (Employed, Student, Unemployed, Inactive).}
#'   \item{income_group}{Ordered factor. Household income group
#'     (Low, Lower middle, Upper middle, High). Contains missing
#'     values.}
#'   \item{income}{Numeric. Monthly household income in CHF.}
#'   \item{smoking}{Factor. Current smoker (No, Yes). Contains
#'     missing values.}
#'   \item{physical_activity}{Factor. Regular physical activity
#'     (No, Yes).}
#'   \item{dentist_12m}{Factor. Dentist visit in the last 12 months
#'     (No, Yes).}
#'   \item{self_rated_health}{Ordered factor. Self-rated health
#'     (Poor, Fair, Good, Very good). Contains missing values.}
#'   \item{wellbeing_score}{Numeric. WHO-5 wellbeing index
#'     (0--100).}
#'   \item{bmi}{Numeric. Body mass index. Contains missing values.}
#'   \item{bmi_category}{Ordered factor. BMI category
#'     (Normal weight, Overweight, Obesity). Contains missing values.}
#'   \item{institutional_trust}{Ordered factor. Trust in institutions
#'     (Very low, Low, High, Very high).}
#'   \item{political_position}{Numeric. Political position on a
#'     0 (left) to 10 (right) scale. Contains missing values.}
#'   \item{life_sat_health}{Integer. Satisfaction with own health
#'     (1--5 Likert scale). Contains missing values.}
#'   \item{life_sat_work}{Integer. Satisfaction with work or main
#'     activity (1--5 Likert scale). Contains missing values.}
#'   \item{life_sat_relationships}{Integer. Satisfaction with
#'     personal relationships (1--5 Likert scale). Contains missing
#'     values.}
#'   \item{life_sat_standard}{Integer. Satisfaction with standard
#'     of living (1--5 Likert scale). Contains missing values.}
#'   \item{response_date}{POSIXct. Date and time of survey response
#'     (September--November 2024).}
#'   \item{weight}{Numeric. Survey design weight.}
#' }
#'
#' @source Simulated data for illustration purposes.
#'
#' @examples
#' data(sochealth)
#' varlist(sochealth)
#' freq(sochealth, education)
#' cross_tab(sochealth, education, self_rated_health)
"sochealth"

#' Print method for continuous summary tables
#'
#' @description
#' Formats and prints a `spicy_continuous_table` object as a styled ASCII
#' table using [spicy_print_table()].
#'
#' @param x A `data.frame` of class `"spicy_continuous_table"` as returned
#'   by [table_continuous()].
#' @param ... Additional arguments (currently ignored).
#'
#' @return Invisibly returns `x`.
#'
#' @seealso [table_continuous()], [spicy_print_table()]
#' @keywords internal
#' @export
print.spicy_continuous_table <- function(x, ...) {
  digits <- attr(x, "digits") %||% 2L
  effect_size_digits <- attr(x, "effect_size_digits") %||% 2L
  decimal_mark <- attr(x, "decimal_mark") %||% "."
  ci_level <- attr(x, "ci_level") %||% 0.95
  group_var <- attr(x, "group_var")

  show_p <- isTRUE(attr(x, "show_p"))
  show_statistic <- isTRUE(attr(x, "show_statistic"))
  show_effect_size <- isTRUE(attr(x, "show_effect_size"))
  show_effect_size_ci <- isTRUE(attr(x, "show_effect_size_ci"))

  display_df <- build_display_df(
    x,
    digits = digits,
    effect_size_digits = effect_size_digits,
    decimal_mark = decimal_mark,
    ci_level = ci_level,
    show_p = show_p,
    show_statistic = show_statistic,
    show_effect_size = show_effect_size,
    show_effect_size_ci = show_effect_size_ci
  )

  has_group <- !is.null(group_var)
  has_statistic <- "Test" %in% names(display_df)
  has_p <- "p" %in% names(display_df)
  align_left <- if (has_group) c(1L, 2L) else 1L

  # Center M, SD, Min, Max, CI LL, CI UL (and Test if present);
  # right-align n (and p if present)
  nc <- ncol(display_df)
  right_cols <- which(names(display_df) == "n")
  if (has_p) {
    right_cols <- c(right_cols, which(names(display_df) == "p"))
  }
  align_center <- setdiff(seq_len(nc), c(align_left, right_cols))

  # Compute separator rows: first row of each variable block (except first)
  sep_rows <- integer(0)
  if (has_group && "Variable" %in% names(display_df)) {
    vars <- display_df$Variable
    for (i in seq_along(vars)) {
      if (i > 1L && nzchar(vars[i])) {
        sep_rows <- c(sep_rows, i)
      }
    }
  }

  title <- "Descriptive statistics"

  # Auto-select padding: use compact when normal would overflow console.
  # Each column in build_ascii_table uses: 1 (pad) + w[i] + 1 (pad) chars,

  # plus 1 char for the vertical separator after column 1.
  # "normal" adds 5 to each w[i]; "compact" uses raw w[i].
  padding <- "normal"
  col_widths <- vapply(
    seq_along(display_df),
    function(i) {
      max(nchar(c(names(display_df)[i], as.character(display_df[[i]]))))
    },
    numeric(1)
  )
  console_w <- getOption("width", 80L)
  normal_width <- sum(col_widths + 5L + 2L) + 1L
  if (normal_width > console_w) {
    padding <- "compact"
  }

  spicy_print_table(
    display_df,
    title = title,
    note = NULL,
    padding = padding,
    first_column_line = TRUE,
    row_total_line = FALSE,
    column_total_line = FALSE,
    bottom_line = FALSE,
    align_left_cols = align_left,
    align_center_cols = align_center,
    group_sep_rows = sep_rows
  )

  invisible(x)
}

#' Generate an interactive variable codebook
#'
#' @description
#' `code_book()` creates an interactive and exportable codebook summarizing
#' selected variables of a data frame. It builds upon [`varlist()`] to provide
#' an overview of variable names, labels, classes, and representative values in
#' a sortable, searchable table.
#'
#' The output is displayed as an interactive `DT::datatable()` in the Viewer pane
#' (for example in RStudio or Positron), allowing searching, sorting, and export
#' (copy, print, CSV, Excel, PDF) directly.
#'
#' @param x A data frame or tibble.
#' @param ... Optional tidyselect-style column selectors (e.g.
#'   `starts_with("var")`, `where(is.numeric)`, etc.). Columns can be selected
#'   or reordered, but renaming selections is not supported.
#' @param values Logical. If `FALSE` (the default), displays a compact
#'   summary of the variable's values. For numeric, character, date/time,
#'   labelled, and factor variables, all unique non-missing values are shown
#'   when there are at most four; otherwise the first three values, an ellipsis
#'   (`...`), and the last value are shown. Values are sorted when appropriate
#'   (e.g., numeric, character, date).
#'   For factors, `factor_levels` controls whether observed or all declared
#'   levels are shown; level order is preserved.
#'   For labelled variables, prefixed labels are displayed via
#'   `labelled::to_factor(levels = "prefixed")`.
#'   If `TRUE`, all unique non-missing values are displayed.
#' @param include_na Logical. If `TRUE`, unique missing value markers
#'   (`<NA>`, `<NaN>`) are appended at the end of the `Values` summary when
#'   present in the variable. This applies to all variable types. Literal
#'   strings `"NA"`, `"NaN"`, and `""` are quoted to distinguish them from
#'   missing markers. If `FALSE` (the default), missing values are omitted from
#'   `Values` but still counted in the `NAs` column.
#' @param title Optional character string displayed as the table caption.
#'   Defaults to `"Codebook"`. Set to `NULL` to remove the title
#'   completely. When `filename = NULL`, the title is also used as the base for
#'   export filenames after conversion to a portable ASCII name.
#' @param filename Optional character string used as the base for exported CSV,
#'   Excel, and PDF filenames. If `NULL` (the default), a portable filename is
#'   derived from `title`, falling back to `"Codebook"` when needed. File
#'   extensions are added by the browser/export engine.
#' @param factor_levels Character. Controls how factor values are displayed
#'   in `Values`. `"all"` (the default; [varlist()] uses `"observed"`) shows
#'   all declared levels, including unused levels. `"observed"` shows only
#'   levels present in the data, preserving factor level order.
#'
#' @details
#' - The interactive `datatable` supports column sorting, global searching, and
#'   client-side export to various formats.
#' - Variable selection uses the same tidyselect interface as [`varlist()`].
#' - By default, factor variables document all declared levels, including
#'   unused levels — appropriate for a schema-oriented codebook. This differs
#'   from [varlist()], which defaults to `"observed"` to summarize observed
#'   data only. Pass `factor_levels = "observed"` to mirror [varlist()]'s
#'   default.
#' - All exports occur client-side through the Viewer or Tab.
#'
#' @return
#' A `DT::datatable` object.
#'
#' @section Dependencies:
#' Requires the following package:
#' - **DT**
#'
#' @examples
#' \dontrun{
#' if (requireNamespace("DT", quietly = TRUE)) {
#'   code_book(sochealth)
#'   code_book(sochealth, starts_with("bmi"))
#'   code_book(sochealth, starts_with("bmi"), values = TRUE, include_na = TRUE)
#'
#'   factors <- data.frame(
#'     group = factor(c("A", "B", NA), levels = c("A", "B", "C"))
#'   )
#'   code_book(
#'     factors,
#'     values = TRUE,
#'     include_na = TRUE,
#'     factor_levels = "observed"
#'   )
#'
#'   code_book(
#'     sochealth,
#'     starts_with("bmi"),
#'     title = "BMI codebook",
#'     filename = "bmi_codebook"
#'   )
#' }
#' }
#'
#' @seealso
#' [varlist()] for generating the underlying variable summaries.
#'
#' @export
code_book <- function(
  x,
  ...,
  values = FALSE,
  include_na = FALSE,
  title = "Codebook",
  filename = NULL,
  factor_levels = c("all", "observed")
) {
  if (!is.data.frame(x)) {
    stop("`x` must be a data frame or tibble.", call. = FALSE)
  }

  dots <- rlang::enquos(..., .named = FALSE)
  validate_code_book_control_dots(dots)
  validate_varlist_logical(values, "values")
  validate_varlist_logical(include_na, "include_na")
  validate_code_book_title(title)
  validate_code_book_filename_arg(filename)
  filename <- code_book_filename(title, filename = filename)
  factor_levels <- match_varlist_factor_levels(factor_levels)

  if (!requireNamespace("DT", quietly = TRUE)) {
    stop(
      "Package 'DT' is required for code_book(). Please install it.",
      call. = FALSE
    )
  }

  res <- varlist(
    x,
    ...,
    values = values,
    include_na = include_na,
    factor_levels = factor_levels,
    tbl = TRUE
  )

  if (!inherits(res, "data.frame")) {
    stop(
      "`varlist()` did not return a data frame. Check your input.",
      call. = FALSE
    )
  }

  DT::datatable(
    res,
    caption = if (is.null(title)) NULL else title,
    rownames = FALSE,
    editable = FALSE,
    filter = "none",
    selection = "none",
    extensions = c("Buttons", "ColReorder", "FixedHeader"),
    options = list(
      dom = "Bfrtip",
      autoWidth = TRUE,
      pageLength = 10,
      colReorder = TRUE,
      fixedHeader = TRUE,
      searchHighlight = TRUE,
      buttons = list(
        "copy",
        "print",
        list(
          extend = "collection",
          text = "Download",
          buttons = list(
            list(
              extend = "csv",
              title = NULL,
              filename = filename
            ),
            list(
              extend = "excel",
              title = NULL,
              filename = filename
            ),
            list(
              extend = "pdf",
              title = NULL,
              filename = filename
            )
          )
        )
      )
    )
  )
}

#' Row Sums with Optional Minimum Valid Values
#'
#' `sum_n()` computes row sums from a `data.frame` or `matrix`, handling missing values (`NA`s) automatically.
#' Row-wise sums are calculated across selected numeric columns, with an optional condition on the minimum number (or proportion) of valid (non-missing) values required for a row to be included.
#' Non-numeric columns are excluded automatically and reported.
#'
#' @param data A `data.frame` or `matrix`.
#' @param select Columns to include. If `regex = FALSE`, use tidyselect syntax (default: `tidyselect::everything()`).
#' If `regex = TRUE`, provide a regular expression pattern (character string).
#' @param exclude Columns to exclude (default: `NULL`).
#' @param min_valid Minimum number of valid (non-NA) values required per row.
#'   If a proportion, it's applied to the number of selected columns.
#'   Defaults to `NULL` (all values must be valid).
#' @param digits Optional number of decimal places to round the result.
#'   Defaults to `NULL` (no rounding).
#' @param regex Logical. If `FALSE` (the default), uses tidyselect helpers.
#'   If `TRUE`, the `select` argument is treated as a regular expression.
#' @param verbose Logical. If `FALSE` (the default), messages are suppressed.
#'   If `TRUE`, prints a message about non-numeric columns excluded.
#'
#' @return A numeric vector of row-wise sums
#'
#' @importFrom rlang inform
#' @examples
#' library(dplyr)
#'
#' # Create a simple numeric data frame
#' df <- tibble(
#'   var1 = c(10, NA, 30, 40, 50),
#'   var2 = c(5, NA, 15, NA, 25),
#'   var3 = c(NA, 30, 20, 50, 10)
#' )
#'
#' # Compute row-wise sums (all values must be valid by default)
#' sum_n(df)
#'
#' # Require at least 2 valid (non-NA) values per row
#' sum_n(df, min_valid = 2)
#'
#' # Require at least 50% valid (non-NA) values per row
#' sum_n(df, min_valid = 0.5)
#'
#' # Round the results to 1 decimal
#' sum_n(df, digits = 1)
#'
#' # Select specific columns
#' sum_n(df, select = c(var1, var2))
#'
#' # Select specific columns using a pipe
#' df |>
#'   select(var1, var2) |>
#'   sum_n()
#'
#' # Exclude a column
#' sum_n(df, exclude = "var3")
#'
#' # Select columns ending with "1"
#' sum_n(df, select = ends_with("1"))
#'
#' # Use with native pipe
#' df |> sum_n(select = starts_with("var"))
#'
#' # Use inside dplyr::mutate()
#' df |> mutate(sum_score = sum_n(min_valid = 2))
#'
#' # Select columns directly inside mutate()
#' df |> mutate(sum_score = sum_n(select = c(var1, var2), min_valid = 1))
#'
#' # Select columns before mutate
#' df |>
#'   select(var1, var2) |>
#'   mutate(sum_score = sum_n(min_valid = 1))
#'
#' # Show verbose message
#' df |> mutate(sum_score = sum_n(min_valid = 2, digits = 1, verbose = TRUE))
#'
#' # Add character and grouping columns
#' df_mixed <- mutate(df,
#'   name = letters[1:5],
#'   group = c("A", "A", "B", "B", "A")
#' )
#' df_mixed
#'
#' # Non-numeric columns are ignored
#' sum_n(df_mixed)
#'
#' # Use inside mutate with mixed data
#' df_mixed |> mutate(sum_score = sum_n(select = starts_with("var")))
#'
#' # Use everything(), but exclude known non-numeric
#' sum_n(df_mixed, select = everything(), exclude = "group")
#'
#' # Select columns using regex
#' sum_n(df_mixed, select = "^var", regex = TRUE)
#' sum_n(df_mixed, select = "ar", regex = TRUE)
#'
#' # Apply to a subset of rows
#' df_mixed[1:3, ] |> sum_n(select = starts_with("var"))
#'
#' # Store the result in a new column
#' df_mixed$sum_score <- sum_n(df_mixed, select = starts_with("var"))
#' df_mixed
#'
#' # With a numeric matrix
#' mat <- matrix(c(1, 2, NA, 4, 5, NA, 7, 8, 9), nrow = 3, byrow = TRUE)
#' mat
#' mat |> sum_n(min_valid = 2)
#'
#' @export
sum_n <- function(
  data = NULL,
  select = tidyselect::everything(),
  exclude = NULL,
  min_valid = NULL,
  digits = NULL,
  regex = FALSE,
  verbose = FALSE
) {
  if (!is.null(min_valid)) {
    if (
      !is.numeric(min_valid) ||
        length(min_valid) != 1L ||
        is.na(min_valid) ||
        min_valid < 0
    ) {
      stop("`min_valid` must be a single non-negative number.", call. = FALSE)
    }
  }

  if (!is.null(digits)) {
    if (!is.numeric(digits) || length(digits) != 1L || digits < 0) {
      stop("`digits` must be a single non-negative number.", call. = FALSE)
    }
  }

  if (is.matrix(data)) {
    data <- as.data.frame(data)
  }

  if (is.null(data)) {
    data <- dplyr::pick(tidyselect::everything())
  }

  if (regex) {
    if (missing(select)) {
      select <- ".*"
    }
    if (!is.character(select) || length(select) != 1L || is.na(select)) {
      stop(
        "When `regex = TRUE`, `select` must be a single character pattern.",
        call. = FALSE
      )
    }
    col_names <- names(data)
    matched <- grep(select, col_names, value = TRUE)
    data <- data[, matched, drop = FALSE]
  } else {
    sel_quo <- rlang::enquo(select)
    sel_val <- tryCatch(
      rlang::eval_tidy(sel_quo, env = rlang::quo_get_env(sel_quo)),
      error = function(e) NULL
    )
    if (is.character(sel_val)) {
      data <- dplyr::select(data, tidyselect::all_of(sel_val))
    } else {
      data <- dplyr::select(data, !!sel_quo)
    }
  }

  data <- dplyr::select(data, -tidyselect::any_of(exclude))

  all_cols <- names(data)
  data <- dplyr::select(data, tidyselect::where(is.numeric))
  numeric_cols <- names(data)

  ignored <- setdiff(all_cols, numeric_cols)
  if (verbose && length(ignored) > 0) {
    rlang::inform(
      message = paste0(
        "sum_n(): Ignored non-numeric columns: ",
        paste(ignored, collapse = ", ")
      )
    )
  }

  if (length(numeric_cols) == 0) {
    warning(
      "sum_n(): No numeric columns selected; returning NA for all rows.",
      call. = FALSE
    )
    return(rep(NA_real_, nrow(data)))
  }

  data_mat <- as.matrix(data)

  if (is.null(min_valid)) {
    min_valid <- ncol(data_mat)
  } else if (min_valid %% 1 != 0) {
    min_valid <- round(ncol(data_mat) * min_valid)
  }

  result <- rowSums(data_mat, na.rm = TRUE)
  valid_rows <- rowSums(!is.na(data_mat)) >= min_valid
  result[!valid_rows] <- NA

  if (!is.null(digits)) {
    result <- round(result, digits)
  }

  if (verbose) {
    rlang::inform(
      message = paste0(
        "sum_n(): Row sums computed with min_valid = ",
        min_valid,
        ", regex = ",
        regex
      )
    )
  }

  result
}

#' Continuous-outcome linear-model table
#'
#' @description
#' Builds APA-style summary tables from a series of simple linear models for
#' one or many continuous outcomes selected with tidyselect syntax.
#'
#' A single predictor is supplied with `by`, and each selected numeric
#' outcome is fit as `lm(outcome ~ by, ...)`. When `by` is categorical, the
#' function returns a model-based mean-comparison table with fitted means by
#' level derived from the linear model, plus an optional single difference for
#' dichotomous predictors. When `by` is numeric, the table reports the slope
#' and its confidence interval.
#'
#' Multiple output formats are available via `output`: a printed ASCII table
#' (`"default"`), a plain wide `data.frame` (`"data.frame"`), a raw long
#' `data.frame` (`"long"`), or rendered outputs (`"tinytable"`, `"gt"`,
#' `"flextable"`, `"excel"`, `"clipboard"`, `"word"`).
#'
#' @param data A `data.frame`.
#' @param select Outcome columns to include. If `regex = FALSE`, use tidyselect
#'   syntax or a character vector of column names (default:
#'   `tidyselect::everything()`). If `regex = TRUE`, provide a regular expression
#'   pattern (character string).
#' @param by A single predictor column. Accepts an unquoted column name or a
#'   single character column name. The predictor can be numeric, logical,
#'   character, or factor.
#' @param exclude Columns to exclude from `select`. Supports tidyselect syntax
#'   and character vectors of column names.
#' @param regex Logical. If `FALSE` (the default), uses tidyselect helpers. If
#'   `TRUE`, the `select` argument is treated as a regular expression.
#' @param weights Optional case weights. Accepts `NULL`, an unquoted numeric
#'   column name, a single character column name, or a numeric vector of length
#'   `nrow(data)`.
#' @param vcov Variance estimator used for uncertainty estimates. One of
#'   `"classical"` (default), `"HC0"`, `"HC1"`, `"HC2"`, `"HC3"`, `"HC4"`,
#'   `"HC4m"`, or `"HC5"`.
#' @param contrast Contrast display for categorical predictors. One of:
#'   - `"auto"`: show a single reference contrast only when `by` has exactly
#'     two levels.
#'   - `"none"`: suppress difference columns for categorical predictors.
#' @param statistic Logical. If `TRUE`, includes a test-statistic column in the
#'   wide and rendered outputs. Defaults to `FALSE`.
#' @param p_value Logical. If `TRUE`, includes a `p` column in the wide and
#'   rendered outputs. Defaults to `TRUE`.
#' @param show_n Logical. If `TRUE`, includes an unweighted `n` column in the
#'   wide and rendered outputs. Defaults to `TRUE`.
#' @param show_weighted_n Logical. If `TRUE` and `weights` is supplied,
#'   includes a `Weighted n` column equal to the sum of case weights in the
#'   analytic sample. Defaults to `FALSE`.
#' @param effect_size Character. Effect-size column to include in the wide and
#'   rendered outputs. One of `"none"` (the default) or `"f2"`.
#' @param r2 Character. Fit statistic to include in the wide and rendered
#'   outputs. One of `"r2"` (the default), `"adj_r2"`, or `"none"`.
#' @param ci Logical. If `TRUE`, includes contrast confidence-interval columns
#'   in the wide and rendered outputs when a single contrast is shown.
#'   Defaults to `TRUE`.
#' @param labels An optional named character vector of outcome labels. Names
#'   must match column names in `data`. When `NULL` (the default), labels are
#'   auto-detected from variable attributes; if none are found, the column name
#'   is used.
#' @param ci_level Confidence level for coefficient and model-based mean
#'   intervals (default: `0.95`). Must be between 0 and 1 exclusive.
#' @param digits Number of decimal places for descriptive values, regression
#'   coefficients, and test statistics (default: `2`).
#' @param fit_digits Number of decimal places for model-fit columns (`R2` or
#'   adjusted `R2`) in wide and rendered outputs (default: `2`).
#' @param effect_size_digits Number of decimal places for effect-size columns
#'   (`f2`) in wide and rendered outputs (default: `2`).
#' @param decimal_mark Character used as decimal separator. Either `"."`
#'   (default) or `","`.
#' @param output Output format. One of:
#'   - `"default"`: a printed ASCII table, returned invisibly
#'   - `"data.frame"`: a plain wide `data.frame`
#'   - `"long"`: a raw long `data.frame`
#'   - `"tinytable"` (requires `tinytable`)
#'   - `"gt"` (requires `gt`)
#'   - `"flextable"` (requires `flextable`)
#'   - `"excel"` (requires `openxlsx2`)
#'   - `"clipboard"` (requires `clipr`)
#'   - `"word"` (requires `flextable` and `officer`)
#' @param excel_path File path for `output = "excel"`.
#' @param excel_sheet Sheet name for `output = "excel"` (default:
#'   `"Linear models"`).
#' @param clipboard_delim Delimiter for `output = "clipboard"` (default:
#'   `"\t"`).
#' @param word_path File path for `output = "word"`.
#' @param verbose Logical. If `TRUE`, prints messages about ignored
#'   non-numeric selected outcomes (default: `FALSE`).
#'
#' @return Depends on `output`:
#' \itemize{
#'   \item `"default"`: prints a styled ASCII table and returns the
#'     underlying long `data.frame` invisibly, with class
#'     `"spicy_continuous_lm_table"`.
#'   \item `"data.frame"`: a plain wide `data.frame` with one row per outcome
#'     and numeric columns for means, optional contrasts, optional test
#'     statistics, model fit, and sample size.
#'   \item `"long"`: a raw `data.frame` with one block per outcome and columns
#'     describing estimated means, contrasts, test statistics, model fit, and
#'     sample size.
#'   \item `"tinytable"`: a `tinytable` object.
#'   \item `"gt"`: a `gt_tbl` object.
#'   \item `"flextable"`: a `flextable` object.
#'   \item `"excel"` / `"word"`: writes to disk and returns the file path.
#'   \item `"clipboard"`: copies the wide table and returns it invisibly.
#' }
#'
#' If no numeric outcome columns remain after applying `select`, `exclude`,
#' and `regex`, the function emits a warning and returns an empty
#' `data.frame()` regardless of `output`.
#'
#' @details
#' `table_continuous_lm()` is designed for article-style bivariate reporting:
#' a single predictor supplied with `by`, and one simple model per selected
#' continuous outcome. The model fit is always `lm(outcome ~ by, ...)`.
#'
#' For categorical predictors, the reported means are model-based fitted means
#' for each level of `by`, and the reported contrasts are derived from the same
#' fitted linear model. For an unweighted `lm(y ~ factor)` with classical
#' variance, these fitted means coincide numerically with the empirical
#' subgroup means; the "model-based" qualifier matters because (a) under
#' `weights`, the means become weighted least-squares estimates, (b) their
#' confidence intervals are derived from the model `vcov` (classical or `HC*`)
#' rather than from a separate empirical standard error, and (c) the reported
#' tests, `p`-values, and effect sizes all come from the same fitted model,
#' so means, contrasts, and inference stay internally consistent.
#'
#' Compared with [table_continuous()], this function is the model-based
#' companion for users who want to report bivariate mean comparisons in a
#' linear-model framework. In practice, it is the better choice when you
#' want heteroskedasticity-consistent standard errors (`vcov = "HC*"`),
#' model fit statistics, or case weights via `lm(..., weights = ...)`.
#' Because the function exists to report a fitted model, its inferential
#' output is part of the default display: `p_value = TRUE` is on by default,
#' and the model `R2` is shown unless explicitly disabled with `r2 = "none"`.
#' Set `p_value = FALSE` or `r2 = "none"` to suppress them.
#'
#' Effect size is reported as Cohen's `f2`, computed from the model `R2` as
#' `R2 / (1 - R2)`. When `vcov != "classical"`, standard errors, confidence
#' intervals, and test statistics use the requested heteroskedasticity-
#' consistent estimator, while `R2` and adjusted `R2` remain the ordinary
#' least-squares fit statistics.
#'
#' When `weights` is supplied, `table_continuous_lm()` fits weighted linear
#' models using `lm(..., weights = ...)` and reports weighted model-based
#' means or slopes accordingly. This is appropriate for case-weighted
#' analyses and weighted article tables, but it is not a substitute for a
#' full complex-survey design workflow.
#'
#' In wide and rendered outputs, `n` always reports the unweighted analytic
#' sample size used for each outcome. When `show_weighted_n = TRUE`, an
#' additional `Weighted n` column reports the sum of case weights for the same
#' analytic sample.
#'
#' For dichotomous categorical predictors, the wide outputs report means in
#' reference-level order and labels the contrast column explicitly as
#' `Delta (level2 - level1)`. For categorical predictors with more than two
#' levels, no single contrast or contrast confidence interval is shown in the
#' wide outputs; instead, the table reports level-specific means plus the
#' overall `F` test when `statistic = TRUE` (or `F(df1, df2)` when the degrees
#' of freedom are constant across outcomes).
#'
#' Optional output engines require suggested packages:
#' \itemize{
#'   \item \pkg{tinytable} for `output = "tinytable"`
#'   \item \pkg{gt} for `output = "gt"`
#'   \item \pkg{flextable} for `output = "flextable"`
#'   \item \pkg{flextable} + \pkg{officer} for `output = "word"`
#'   \item \pkg{openxlsx2} for `output = "excel"`
#'   \item \pkg{clipr} for `output = "clipboard"`
#' }
#'
#' @seealso [table_continuous()], [table_categorical()]
#'
#' @examples
#' table_continuous_lm(
#'   sochealth,
#'   select = c(wellbeing_score, bmi),
#'   by = sex
#' )
#'
#' table_continuous_lm(
#'   sochealth,
#'   select = c(wellbeing_score, bmi),
#'   by = sex,
#'   output = "data.frame"
#' )
#'
#' table_continuous_lm(
#'   sochealth,
#'   select = c(wellbeing_score, bmi),
#'   by = age,
#'   vcov = "HC3",
#'   ci = FALSE
#' )
#'
#' table_continuous_lm(
#'   sochealth,
#'   select = c(wellbeing_score, bmi),
#'   by = sex,
#'   weights = weight,
#'   statistic = TRUE,
#'   effect_size = "f2",
#'   show_weighted_n = TRUE
#' )
#'
#' \donttest{
#' if (requireNamespace("tinytable", quietly = TRUE)) {
#'   table_continuous_lm(
#'     sochealth,
#'     select = c(wellbeing_score, bmi),
#'     by = sex,
#'     output = "tinytable"
#'   )
#' }
#'
#' if (requireNamespace("gt", quietly = TRUE)) {
#'   table_continuous_lm(
#'     sochealth,
#'     select = c(wellbeing_score, bmi),
#'     by = sex,
#'     output = "gt"
#'   )
#' }
#'
#' if (requireNamespace("flextable", quietly = TRUE)) {
#'   table_continuous_lm(
#'     sochealth,
#'     select = c(wellbeing_score, bmi),
#'     by = sex,
#'     output = "flextable"
#'   )
#' }
#' }
#'
#' @export
table_continuous_lm <- function(
  data,
  select = tidyselect::everything(),
  by,
  exclude = NULL,
  regex = FALSE,
  weights = NULL,
  vcov = c("classical", "HC0", "HC1", "HC2", "HC3", "HC4", "HC4m", "HC5"),
  contrast = c("auto", "none"),
  statistic = FALSE,
  p_value = TRUE,
  show_n = TRUE,
  show_weighted_n = FALSE,
  effect_size = c("none", "f2"),
  r2 = c("r2", "adj_r2", "none"),
  ci = TRUE,
  labels = NULL,
  ci_level = 0.95,
  digits = 2,
  fit_digits = 2,
  effect_size_digits = 2,
  decimal_mark = ".",
  output = c(
    "default",
    "data.frame",
    "long",
    "tinytable",
    "gt",
    "flextable",
    "excel",
    "clipboard",
    "word"
  ),
  excel_path = NULL,
  excel_sheet = "Linear models",
  clipboard_delim = "\t",
  word_path = NULL,
  verbose = FALSE
) {
  if (!is.data.frame(data)) {
    stop("`data` must be a data.frame.", call. = FALSE)
  }
  if (
    !is.numeric(ci_level) ||
      length(ci_level) != 1L ||
      is.na(ci_level) ||
      ci_level <= 0 ||
      ci_level >= 1
  ) {
    stop("`ci_level` must be a single number between 0 and 1.", call. = FALSE)
  }
  if (
    !is.numeric(digits) ||
      length(digits) != 1L ||
      is.na(digits) ||
      digits < 0
  ) {
    stop("`digits` must be a single non-negative number.", call. = FALSE)
  }
  digits <- as.integer(digits)
  if (
    !is.numeric(fit_digits) ||
      length(fit_digits) != 1L ||
      is.na(fit_digits) ||
      fit_digits < 0
  ) {
    stop("`fit_digits` must be a single non-negative number.", call. = FALSE)
  }
  fit_digits <- as.integer(fit_digits)
  if (
    !is.numeric(effect_size_digits) ||
      length(effect_size_digits) != 1L ||
      is.na(effect_size_digits) ||
      effect_size_digits < 0
  ) {
    stop(
      "`effect_size_digits` must be a single non-negative number.",
      call. = FALSE
    )
  }
  effect_size_digits <- as.integer(effect_size_digits)
  if (!decimal_mark %in% c(".", ",")) {
    stop('`decimal_mark` must be "." or ",".', call. = FALSE)
  }
  if (!is.null(labels) && (!is.character(labels) || is.null(names(labels)))) {
    stop("`labels` must be a named character vector.", call. = FALSE)
  }
  for (.arg in c(
    "regex",
    "verbose",
    "statistic",
    "p_value",
    "show_n",
    "show_weighted_n",
    "ci"
  )) {
    .val <- get(.arg)
    if (!is.logical(.val) || length(.val) != 1L || is.na(.val)) {
      stop(sprintf("`%s` must be TRUE/FALSE.", .arg), call. = FALSE)
    }
  }

  output <- match.arg(output)
  vcov <- match.arg(vcov)
  contrast <- match.arg(contrast)
  effect_size <- match.arg(effect_size)
  r2 <- match.arg(r2)

  by_quo <- rlang::enquo(by)
  by_name <- resolve_single_column_selection(by_quo, data, "by")
  by_vector <- data[[by_name]]
  if (!is_supported_lm_predictor(by_vector)) {
    stop(
      "`by` must be numeric, logical, character, or factor.",
      call. = FALSE
    )
  }

  weights_quo <- rlang::enquo(weights)
  weights_name <- detect_weights_column_name(weights_quo, data)
  weights_vec <- resolve_weights_argument(weights_quo, data, "weights")
  if (!is.null(weights_vec)) {
    if (any(!is.finite(weights_vec), na.rm = TRUE)) {
      stop("`weights` must contain only finite values.", call. = FALSE)
    }
    if (any(weights_vec < 0, na.rm = TRUE)) {
      stop("`weights` must be non-negative.", call. = FALSE)
    }
    if (all(is.na(weights_vec) | weights_vec == 0)) {
      stop("`weights` must contain at least one positive value.", call. = FALSE)
    }
  }
  if (isTRUE(show_weighted_n) && is.null(weights_vec)) {
    warning(
      "`show_weighted_n` is ignored when `weights` is not supplied.",
      call. = FALSE
    )
    show_weighted_n <- FALSE
  }

  available_names <- names(data)
  excluded_names <- resolve_multi_column_selection(
    rlang::enquo(exclude),
    data,
    "exclude"
  )
  excluded_names <- unique(c(excluded_names, by_name, weights_name))

  if (isTRUE(regex)) {
    select_val <- tryCatch(
      rlang::eval_tidy(select),
      error = function(e) NULL
    )
    if (
      !is.character(select_val) || length(select_val) != 1L || is.na(select_val)
    ) {
      stop(
        "When `regex = TRUE`, `select` must be a single regex pattern.",
        call. = FALSE
      )
    }
    selected_names <- grep(select_val, available_names, value = TRUE)
  } else {
    select_quo <- rlang::enquo(select)
    selected_pos <- tryCatch(
      tidyselect::eval_select(select_quo, data),
      error = function(e) {
        stop("`select` must select columns in `data`.", call. = FALSE)
      }
    )
    selected_names <- names(selected_pos)
  }

  selected_names <- setdiff(selected_names, excluded_names)
  numeric_outcomes <- selected_names[vapply(
    selected_names,
    function(nm) is.numeric(data[[nm]]),
    logical(1)
  )]
  ignored_names <- setdiff(selected_names, numeric_outcomes)
  if (length(ignored_names) > 0L && isTRUE(verbose)) {
    rlang::inform(
      paste0(
        "Ignoring non-numeric selected outcomes: ",
        paste(ignored_names, collapse = ", ")
      )
    )
  }

  if (length(numeric_outcomes) == 0L) {
    warning("No numeric outcome columns selected.", call. = FALSE)
    return(data.frame())
  }

  outcome_labels <- vapply(
    numeric_outcomes,
    function(nm) {
      if (!is.null(labels) && nm %in% names(labels)) {
        return(labels[[nm]])
      }
      lab <- attr(data[[nm]], "label", exact = TRUE)
      if (is.null(lab) || !nzchar(lab)) nm else lab
    },
    character(1)
  )
  by_label <- attr(data[[by_name]], "label", exact = TRUE)
  if (is.null(by_label) || !nzchar(by_label)) {
    by_label <- by_name
  }

  rows <- lapply(
    seq_along(numeric_outcomes),
    function(i) {
      fit_outcome_lm_rows(
        y = data[[numeric_outcomes[i]]],
        predictor = by_vector,
        weights = weights_vec,
        outcome_name = numeric_outcomes[i],
        outcome_label = outcome_labels[i],
        predictor_label = by_label,
        vcov_type = vcov,
        contrast = contrast,
        ci_level = ci_level
      )
    }
  )
  result <- do.call(rbind, rows)
  rownames(result) <- NULL

  attr(result, "ci_level") <- ci_level
  attr(result, "digits") <- digits
  attr(result, "fit_digits") <- fit_digits
  attr(result, "effect_size_digits") <- effect_size_digits
  attr(result, "decimal_mark") <- decimal_mark
  attr(result, "by_var") <- by_name
  attr(result, "by_label") <- by_label
  attr(result, "vcov_type") <- vcov
  attr(result, "contrast") <- contrast
  attr(result, "weights_used") <- !is.null(weights_vec)
  attr(result, "show_statistic") <- statistic
  attr(result, "show_p_value") <- p_value
  attr(result, "show_n") <- show_n
  attr(result, "show_weighted_n") <- show_weighted_n
  attr(result, "effect_size") <- effect_size
  attr(result, "r2_type") <- r2
  attr(result, "show_ci") <- ci

  if (identical(output, "long")) {
    return(result)
  }

  wide_raw <- build_wide_raw_continuous_lm(
    result,
    show_statistic = statistic,
    show_p_value = p_value,
    show_n = show_n,
    show_weighted_n = show_weighted_n,
    effect_size = effect_size,
    r2_type = r2,
    ci = ci
  )
  if (identical(output, "data.frame")) {
    return(wide_raw)
  }

  wide_df <- build_wide_display_df_continuous_lm(
    result,
    digits = digits,
    fit_digits = fit_digits,
    effect_size_digits = effect_size_digits,
    decimal_mark = decimal_mark,
    ci_level = ci_level,
    show_statistic = statistic,
    show_p_value = p_value,
    show_n = show_n,
    show_weighted_n = show_weighted_n,
    effect_size = effect_size,
    r2_type = r2,
    ci = ci
  )

  if (identical(output, "default")) {
    class(result) <- c(
      "spicy_continuous_lm_table",
      "spicy_table",
      class(result)
    )
    print(result)
    return(invisible(result))
  }

  export_continuous_lm_table(
    wide_df,
    output = output,
    ci_level = ci_level,
    excel_path = excel_path,
    excel_sheet = excel_sheet,
    clipboard_delim = clipboard_delim,
    word_path = word_path
  )
}

fit_outcome_lm_rows <- function(
  y,
  predictor,
  weights,
  outcome_name,
  outcome_label,
  predictor_label,
  vcov_type,
  contrast,
  ci_level
) {
  keep <- !is.na(y) & !is.na(predictor)
  if (!is.null(weights)) {
    keep <- keep & !is.na(weights)
  }

  y <- y[keep]
  predictor <- predictor[keep]
  weights <- if (is.null(weights)) NULL else weights[keep]

  if (is.numeric(predictor)) {
    return(
      fit_numeric_predictor_lm_rows(
        y = y,
        x = predictor,
        weights = weights,
        outcome_name = outcome_name,
        outcome_label = outcome_label,
        predictor_label = predictor_label,
        vcov_type = vcov_type,
        ci_level = ci_level
      )
    )
  }

  fit_categorical_predictor_lm_rows(
    y = y,
    x = predictor,
    weights = weights,
    outcome_name = outcome_name,
    outcome_label = outcome_label,
    predictor_label = predictor_label,
    vcov_type = vcov_type,
    contrast = contrast,
    ci_level = ci_level
  )
}

fit_numeric_predictor_lm_rows <- function(
  y,
  x,
  weights,
  outcome_name,
  outcome_label,
  predictor_label,
  vcov_type,
  ci_level
) {
  if (length(y) < 2L || stats::sd(x, na.rm = TRUE) == 0) {
    return(make_empty_lm_rows(outcome_name, outcome_label, "continuous"))
  }

  model_df <- data.frame(y = y, x = x)
  fit <- if (is.null(weights)) {
    stats::lm(y ~ x, data = model_df)
  } else {
    stats::lm(y ~ x, data = model_df, weights = weights)
  }

  vc <- compute_lm_vcov(fit, vcov_type)
  cf <- stats::coef(fit)
  se <- sqrt(diag(vc))
  model_stats <- compute_lm_model_stats(fit)

  df_resid <- stats::df.residual(fit)
  crit <- if (is.finite(df_resid) && df_resid > 0) {
    stats::qt(1 - (1 - ci_level) / 2, df = df_resid)
  } else {
    stats::qnorm(1 - (1 - ci_level) / 2)
  }

  estimate <- unname(cf[["x"]])
  estimate_se <- unname(se[["x"]])
  statistic <- estimate / estimate_se
  p_value <- 2 * stats::pt(abs(statistic), df = df_resid, lower.tail = FALSE)

  data.frame(
    variable = outcome_name,
    label = outcome_label,
    predictor_type = "continuous",
    predictor_label = predictor_label,
    level = NA_character_,
    reference = NA_character_,
    estimate_type = "slope",
    emmean = NA_real_,
    emmean_se = NA_real_,
    emmean_ci_lower = NA_real_,
    emmean_ci_upper = NA_real_,
    estimate = estimate,
    estimate_se = estimate_se,
    estimate_ci_lower = estimate - crit * estimate_se,
    estimate_ci_upper = estimate + crit * estimate_se,
    test_type = "t",
    statistic = statistic,
    df1 = 1,
    df2 = df_resid,
    p.value = p_value,
    es_type = "f2",
    es_value = model_stats$f2,
    r2 = model_stats$r2,
    adj_r2 = model_stats$adj_r2,
    n = length(y),
    sum_w = if (is.null(weights)) NA_real_ else sum(weights),
    stringsAsFactors = FALSE
  )
}

fit_categorical_predictor_lm_rows <- function(
  y,
  x,
  weights,
  outcome_name,
  outcome_label,
  predictor_label,
  vcov_type,
  contrast,
  ci_level
) {
  x <- droplevels(coerce_lm_factor(x))
  if (length(y) < 2L || nlevels(x) < 2L) {
    return(make_empty_lm_rows(outcome_name, outcome_label, "categorical"))
  }

  model_df <- data.frame(y = y, x = x)
  fit <- if (is.null(weights)) {
    stats::lm(y ~ x, data = model_df)
  } else {
    stats::lm(y ~ x, data = model_df, weights = weights)
  }

  vc <- compute_lm_vcov(fit, vcov_type)
  cf <- stats::coef(fit)
  df_resid <- stats::df.residual(fit)
  crit <- if (is.finite(df_resid) && df_resid > 0) {
    stats::qt(1 - (1 - ci_level) / 2, df = df_resid)
  } else {
    stats::qnorm(1 - (1 - ci_level) / 2)
  }
  model_stats <- compute_lm_model_stats(fit)

  levs <- levels(x)
  newdata <- data.frame(x = factor(levs, levels = levs))
  design <- stats::model.matrix(
    stats::delete.response(stats::terms(fit)),
    newdata
  )
  emmean <- as.vector(design %*% cf)
  emmean_se <- sqrt(rowSums((design %*% vc) * design))

  q <- max(1L, ncol(design) - 1L)
  beta_sub <- cf[-1]
  vc_sub <- vc[-1, -1, drop = FALSE]
  global_stat <- if (length(beta_sub) == 0L) {
    NA_real_
  } else {
    as.numeric(crossprod(beta_sub, solve(vc_sub, beta_sub)) / q)
  }
  global_p <- if (is.na(global_stat)) {
    NA_real_
  } else {
    stats::pf(global_stat, q, df_resid, lower.tail = FALSE)
  }

  show_reference <- identical(contrast, "auto") && nlevels(x) == 2L

  out <- data.frame(
    variable = rep(outcome_name, length(levs)),
    label = rep(outcome_label, length(levs)),
    predictor_type = rep("categorical", length(levs)),
    predictor_label = rep(predictor_label, length(levs)),
    level = levs,
    reference = rep(levs[1], length(levs)),
    estimate_type = rep(NA_character_, length(levs)),
    emmean = emmean,
    emmean_se = emmean_se,
    emmean_ci_lower = emmean - crit * emmean_se,
    emmean_ci_upper = emmean + crit * emmean_se,
    estimate = rep(NA_real_, length(levs)),
    estimate_se = rep(NA_real_, length(levs)),
    estimate_ci_lower = rep(NA_real_, length(levs)),
    estimate_ci_upper = rep(NA_real_, length(levs)),
    test_type = c("F", rep(NA_character_, length(levs) - 1L)),
    statistic = c(global_stat, rep(NA_real_, length(levs) - 1L)),
    df1 = c(q, rep(NA_real_, length(levs) - 1L)),
    df2 = c(df_resid, rep(NA_real_, length(levs) - 1L)),
    p.value = c(global_p, rep(NA_real_, length(levs) - 1L)),
    es_type = c("f2", rep(NA_character_, length(levs) - 1L)),
    es_value = c(model_stats$f2, rep(NA_real_, length(levs) - 1L)),
    r2 = c(model_stats$r2, rep(NA_real_, length(levs) - 1L)),
    adj_r2 = c(model_stats$adj_r2, rep(NA_real_, length(levs) - 1L)),
    n = rep(length(y), length(levs)),
    sum_w = rep(if (is.null(weights)) NA_real_ else sum(weights), length(levs)),
    stringsAsFactors = FALSE
  )

  if (isTRUE(show_reference)) {
    coef_names <- names(cf)[-1]
    se <- sqrt(diag(vc))[-1]
    for (i in seq_along(coef_names)) {
      row_idx <- i + 1L
      est <- unname(cf[coef_names[i]])
      se_i <- unname(se[i])
      stat_i <- est / se_i
      out$estimate_type[row_idx] <- "difference"
      out$estimate[row_idx] <- est
      out$estimate_se[row_idx] <- se_i
      out$estimate_ci_lower[row_idx] <- est - crit * se_i
      out$estimate_ci_upper[row_idx] <- est + crit * se_i
      out$test_type[row_idx] <- "t"
      out$statistic[row_idx] <- stat_i
      out$df1[row_idx] <- 1
      out$df2[row_idx] <- df_resid
      out$p.value[row_idx] <- 2 *
        stats::pt(
          abs(stat_i),
          df = df_resid,
          lower.tail = FALSE
        )
    }
  }

  out
}

make_empty_lm_rows <- function(outcome_name, outcome_label, predictor_type) {
  data.frame(
    variable = outcome_name,
    label = outcome_label,
    predictor_type = predictor_type,
    predictor_label = NA_character_,
    level = NA_character_,
    reference = NA_character_,
    estimate_type = NA_character_,
    emmean = NA_real_,
    emmean_se = NA_real_,
    emmean_ci_lower = NA_real_,
    emmean_ci_upper = NA_real_,
    estimate = NA_real_,
    estimate_se = NA_real_,
    estimate_ci_lower = NA_real_,
    estimate_ci_upper = NA_real_,
    test_type = NA_character_,
    statistic = NA_real_,
    df1 = NA_real_,
    df2 = NA_real_,
    p.value = NA_real_,
    es_type = NA_character_,
    es_value = NA_real_,
    r2 = NA_real_,
    adj_r2 = NA_real_,
    n = NA_real_,
    sum_w = NA_real_,
    stringsAsFactors = FALSE
  )
}

is_supported_lm_predictor <- function(x) {
  is.numeric(x) || is.factor(x) || is.character(x) || is.logical(x)
}

coerce_lm_factor <- function(x) {
  if (is.factor(x)) {
    return(x)
  }
  factor(x)
}

detect_weights_column_name <- function(quo, data) {
  if (rlang::quo_is_null(quo)) {
    return(NULL)
  }

  val <- tryCatch(
    rlang::eval_tidy(quo, env = rlang::quo_get_env(quo)),
    error = function(e) NULL
  )
  if (is.character(val) && length(val) == 1L && val %in% names(data)) {
    return(val)
  }

  pos <- tryCatch(
    tidyselect::eval_select(quo, data),
    error = function(e) integer(0)
  )
  if (length(pos) == 1L) {
    return(names(pos))
  }

  NULL
}

compute_lm_vcov <- function(fit, type = "classical") {
  if (identical(type, "classical")) {
    return(stats::vcov(fit))
  }

  x <- stats::model.matrix(fit)
  e <- stats::residuals(fit)
  w <- stats::weights(fit)
  if (is.null(w)) {
    w <- rep(1, length(e))
  }

  xw <- x * sqrt(w)
  xtwx_inv <- tryCatch(
    solve(crossprod(xw)),
    error = function(e2) {
      warning(
        "Robust `vcov = \"",
        type,
        "\"` could not be computed (model matrix is singular); ",
        "falling back to the classical OLS variance, which may contain NA.",
        call. = FALSE
      )
      stats::vcov(fit)
    }
  )
  hat <- rowSums((xw %*% xtwx_inv) * xw)
  n <- nrow(x)
  p <- max(1L, as.integer(round(sum(hat))))
  omega <- (w^2) * (e^2)
  h_adj <- pmax(1 - hat, .Machine$double.eps)
  nh_over_p <- n * hat / p

  scale <- switch(
    type,
    HC0 = rep(1, length(h_adj)),
    HC1 = rep(n / max(1, n - p), length(h_adj)),
    HC2 = 1 / h_adj,
    HC3 = 1 / (h_adj^2),
    HC4 = {
      delta <- pmin(4, nh_over_p)
      1 / (h_adj^delta)
    },
    HC4m = {
      delta <- pmin(1, nh_over_p) + pmin(1.5, nh_over_p)
      1 / (h_adj^delta)
    },
    HC5 = {
      delta <- pmin(nh_over_p, pmax(4, n * 0.7 * max(hat) / p))
      1 / sqrt(h_adj^delta)
    },
    stop("Unknown `vcov` type.", call. = FALSE)
  )

  xtwx_inv %*% crossprod(x, (omega * scale) * x) %*% xtwx_inv
}

compute_lm_model_stats <- function(fit) {
  sm <- summary(fit)
  r2 <- unname(sm$r.squared)
  adj_r2 <- unname(sm$adj.r.squared)
  f2 <- if (is.na(r2) || r2 >= 1) NA_real_ else r2 / (1 - r2)

  list(r2 = r2, adj_r2 = adj_r2, f2 = f2)
}

build_wide_raw_continuous_lm <- function(
  x,
  show_statistic = TRUE,
  show_p_value = TRUE,
  show_n = TRUE,
  show_weighted_n = FALSE,
  effect_size = "none",
  r2_type = "r2",
  ci = TRUE
) {
  vars <- unique(x$variable)
  first_block <- x[x$variable == vars[1], , drop = FALSE]
  by_type <- unique(first_block$predictor_type)[1]
  include_es <- !identical(effect_size, "none")
  include_r2 <- !identical(r2_type, "none")

  out <- data.frame(
    Variable = vapply(
      vars,
      function(v) x$label[match(v, x$variable)],
      character(1)
    ),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  if (identical(by_type, "categorical")) {
    for (lev in first_block$level) {
      out[[paste0("M (", lev, ")")]] <- NA_real_
    }
    if (nrow(first_block) == 2L) {
      out[[get_delta_label_lm(first_block)]] <- NA_real_
      if (isTRUE(ci)) {
        out[["95% CI LL"]] <- NA_real_
        out[["95% CI UL"]] <- NA_real_
      }
    }
  } else {
    out$B <- NA_real_
    if (isTRUE(ci)) {
      out[["95% CI LL"]] <- NA_real_
      out[["95% CI UL"]] <- NA_real_
    }
  }

  test_header <- get_test_header_lm(x, show_statistic, exact = TRUE)
  if (!is.null(test_header)) {
    out[[test_header]] <- NA_real_
  }
  if (show_p_value) {
    out$p <- NA_real_
  }
  if (include_r2) {
    out[[format_r2_header_lm(r2_type)]] <- NA_real_
  }
  if (include_es) {
    out[[format_effect_size_header_lm(effect_size)]] <- NA_real_
  }
  if (show_n) {
    out$n <- NA_real_
  }
  if (show_weighted_n) {
    out[["Weighted n"]] <- NA_real_
  }

  for (i in seq_along(vars)) {
    block <- x[x$variable == vars[i], , drop = FALSE]
    test_row <- get_test_row_index_lm(block)

    if (identical(by_type, "categorical")) {
      for (j in seq_len(nrow(block))) {
        out[i, paste0("M (", block$level[j], ")")] <- block$emmean[j]
      }
      if (nrow(block) == 2L) {
        delta_name <- get_delta_label_lm(block)
        out[[delta_name]][i] <- block$estimate[test_row]
        if (isTRUE(ci)) {
          out[["95% CI LL"]][i] <- block$estimate_ci_lower[test_row]
          out[["95% CI UL"]][i] <- block$estimate_ci_upper[test_row]
        }
      }
    } else {
      out$B[i] <- block$estimate[1]
      if (isTRUE(ci)) {
        out[["95% CI LL"]][i] <- block$estimate_ci_lower[1]
        out[["95% CI UL"]][i] <- block$estimate_ci_upper[1]
      }
    }

    if (!is.null(test_header)) {
      out[[test_header]][i] <- block$statistic[test_row]
    }
    if (show_p_value) {
      out$p[i] <- block$p.value[test_row]
    }
    if (include_r2) {
      out[[format_r2_header_lm(r2_type)]][i] <- get_r2_value_lm(block, r2_type)
    }
    if (include_es) {
      out[[format_effect_size_header_lm(effect_size)]][i] <- block$es_value[1]
    }
    if (show_n) {
      out$n[i] <- block$n[1]
    }
    if (show_weighted_n) {
      out[["Weighted n"]][i] <- block$sum_w[1]
    }
  }

  out
}

build_wide_display_df_continuous_lm <- function(
  x,
  digits = 2L,
  decimal_mark = ".",
  ci_level = 0.95,
  show_statistic = TRUE,
  show_p_value = TRUE,
  show_n = TRUE,
  show_weighted_n = FALSE,
  effect_size = "none",
  r2_type = "r2",
  ci = TRUE,
  fit_digits = 2L,
  effect_size_digits = 2L
) {
  vars <- unique(x$variable)
  first_block <- x[x$variable == vars[1], , drop = FALSE]
  by_type <- unique(first_block$predictor_type)[1]
  ci_ll_name <- paste0(round(ci_level * 100), "% CI LL")
  ci_ul_name <- paste0(round(ci_level * 100), "% CI UL")
  include_es <- !identical(effect_size, "none")
  include_r2 <- !identical(r2_type, "none")
  r2_header <- format_r2_header_lm(r2_type)

  out <- data.frame(
    Variable = vapply(
      vars,
      function(v) x$label[match(v, x$variable)],
      character(1)
    ),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  if (identical(by_type, "categorical")) {
    for (lev in first_block$level) {
      out[[paste0("M (", lev, ")")]] <- ""
    }
    if (nrow(first_block) == 2L) {
      out[[get_delta_label_lm(first_block)]] <- ""
      if (isTRUE(ci)) {
        out[[ci_ll_name]] <- ""
        out[[ci_ul_name]] <- ""
      }
    }
  } else {
    out$B <- ""
    if (isTRUE(ci)) {
      out[[ci_ll_name]] <- ""
      out[[ci_ul_name]] <- ""
    }
  }

  test_header <- get_test_header_lm(x, show_statistic, exact = TRUE)
  if (!is.null(test_header)) {
    out[[test_header]] <- ""
  }
  if (show_p_value) {
    out$p <- ""
  }
  if (include_r2) {
    out[[r2_header]] <- ""
  }
  if (include_es) {
    out[[format_effect_size_header_lm(effect_size)]] <- ""
  }
  if (show_n) {
    out$n <- ""
  }
  if (show_weighted_n) {
    out[["Weighted n"]] <- ""
  }

  for (i in seq_along(vars)) {
    block <- x[x$variable == vars[i], , drop = FALSE]
    test_row <- get_test_row_index_lm(block)
    if (identical(by_type, "categorical")) {
      for (j in seq_len(nrow(block))) {
        out[i, paste0("M (", block$level[j], ")")] <- format_number_lm(
          block$emmean[j],
          digits,
          decimal_mark
        )
      }
      if (nrow(block) == 2L) {
        delta_name <- get_delta_label_lm(block)
        out[[delta_name]][i] <- format_number_lm(
          block$estimate[test_row],
          digits,
          decimal_mark
        )
        if (isTRUE(ci)) {
          out[[ci_ll_name]][i] <- format_number_lm(
            block$estimate_ci_lower[test_row],
            digits,
            decimal_mark
          )
          out[[ci_ul_name]][i] <- format_number_lm(
            block$estimate_ci_upper[test_row],
            digits,
            decimal_mark
          )
        }
      }
    } else {
      out$B[i] <- format_number_lm(block$estimate[1], digits, decimal_mark)
      if (isTRUE(ci)) {
        out[[ci_ll_name]][i] <- format_number_lm(
          block$estimate_ci_lower[1],
          digits,
          decimal_mark
        )
        out[[ci_ul_name]][i] <- format_number_lm(
          block$estimate_ci_upper[1],
          digits,
          decimal_mark
        )
      }
    }

    if (!is.null(test_header)) {
      out[[test_header]][i] <- format_number_lm(
        block$statistic[test_row],
        digits,
        decimal_mark
      )
    }
    if (show_p_value) {
      out$p[i] <- format_p_value_lm(block$p.value[test_row], decimal_mark)
    }
    if (include_es) {
      out[[format_effect_size_header_lm(effect_size)]][i] <- format_number_lm(
        block$es_value[1],
        effect_size_digits,
        decimal_mark
      )
    }
    if (include_r2) {
      out[[r2_header]][i] <- format_number_lm(
        get_r2_value_lm(block, r2_type),
        fit_digits,
        decimal_mark
      )
    }
    if (show_n) {
      out$n[i] <- if (is.na(block$n[1])) {
        ""
      } else {
        as.character(as.integer(block$n[1]))
      }
    }
    if (show_weighted_n) {
      out[["Weighted n"]][i] <- if (is.na(block$sum_w[1])) {
        ""
      } else {
        format_number_lm(block$sum_w[1], digits, decimal_mark)
      }
    }
  }

  out
}

export_continuous_lm_table <- function(
  display_df,
  output,
  ci_level,
  excel_path,
  excel_sheet,
  clipboard_delim,
  word_path
) {
  ci_pct <- paste0(round(ci_level * 100), "%")
  ci_ll <- paste0(ci_pct, " CI LL")
  ci_ul <- paste0(ci_pct, " CI UL")
  has_ci <- all(c(ci_ll, ci_ul) %in% names(display_df))

  if (identical(output, "tinytable")) {
    if (!requireNamespace("tinytable", quietly = TRUE)) {
      stop("Install package 'tinytable'.", call. = FALSE)
    }
    old_tt_opt <- getOption("tinytable_print_output")
    options(tinytable_print_output = "html")
    on.exit(options(tinytable_print_output = old_tt_opt), add = TRUE)

    display_df <- rename_ci_cols_lm(display_df, ci_ll, ci_ul)
    col_keys <- names(display_df)
    nc <- length(col_keys)
    ll_pos <- which(col_keys == "LL")
    ul_pos <- which(col_keys == "UL")

    sub_labels <- rep("", nc)
    if (has_ci) {
      sub_labels[ll_pos] <- "LL"
      sub_labels[ul_pos] <- "UL"
    }
    colnames(display_df) <- sub_labels

    gspec <- list()
    for (j in seq_along(col_keys)) {
      if (has_ci && col_keys[j] %in% c("LL", "UL")) {
        next
      }
      gspec[[col_keys[j]]] <- j
    }
    if (has_ci) {
      gspec[[paste0(ci_pct, " CI")]] <- c(ll_pos, ul_pos)
    }

    tt <- tinytable::tt(display_df)
    tt <- tinytable::group_tt(tt, j = gspec)
    tt <- tinytable::theme_empty(tt)
    tt <- tinytable::style_tt(tt, j = 1, align = "l")
    if (ncol(display_df) > 1L) {
      right_j <- which(col_keys %in% c("n", "Weighted n", "p"))
      center_j <- setdiff(seq_len(nc), c(1L, right_j))
      if (length(center_j) > 0L) {
        tt <- tinytable::style_tt(tt, j = center_j, align = "c")
      }
      if (length(right_j) > 0L) {
        for (rj in right_j) {
          tt <- tinytable::style_tt(tt, j = rj, align = "r")
        }
      }
      spanner_center_j <- setdiff(seq_len(nc), 1L)
      if (length(spanner_center_j) > 0L) {
        tt <- tinytable::style_tt(tt, i = -1, j = spanner_center_j, align = "c")
      }
      tt <- tinytable::style_tt(tt, i = -1, j = 1L, align = "l")
      tt <- tinytable::style_tt(
        tt,
        i = -1,
        j = seq_len(nc),
        line = "t",
        line_width = 0.06
      )
      if (has_ci) {
        tt <- tinytable::style_tt(
          tt,
          i = -1,
          j = c(ll_pos, ul_pos),
          line = "b",
          line_width = 0.06
        )
      }
      tt <- tinytable::style_tt(
        tt,
        i = 0,
        j = seq_len(nc),
        line = "b",
        line_width = 0.06
      )
      tt <- tinytable::style_tt(
        tt,
        i = nrow(display_df),
        j = seq_len(nc),
        line = "b",
        line_width = 0.06
      )
      p_j <- which(col_keys == "p")
      if (length(p_j) == 1L) {
        tt <- tinytable::style_tt(
          tt,
          j = p_j,
          html_css = "white-space: nowrap;"
        )
      }
    }
    return(tt)
  }

  if (identical(output, "gt")) {
    if (!requireNamespace("gt", quietly = TRUE)) {
      stop("Install package 'gt'.", call. = FALSE)
    }

    display_df <- rename_ci_cols_lm(display_df, ci_ll, ci_ul)
    col_keys <- names(display_df)
    tbl <- gt::gt(display_df)

    label_list <- stats::setNames(as.list(rep("", length(col_keys))), col_keys)
    if (has_ci && "LL" %in% col_keys) {
      label_list[["LL"]] <- "LL"
    }
    if (has_ci && "UL" %in% col_keys) {
      label_list[["UL"]] <- "UL"
    }
    tbl <- gt::cols_label(tbl, .list = label_list)

    single_cols <- setdiff(col_keys, c("LL", "UL"))
    for (col in single_cols) {
      tbl <- gt::tab_spanner(
        tbl,
        label = col,
        columns = col,
        id = paste0("spn_", make.names(col))
      )
    }
    if (has_ci) {
      tbl <- gt::tab_spanner(
        tbl,
        label = paste0(ci_pct, " CI"),
        columns = c("LL", "UL")
      )
    }

    tbl <- gt::cols_align(tbl, align = "left", columns = "Variable")
    center_cols <- setdiff(col_keys, c("Variable", "n", "p"))
    if (length(center_cols) > 0L) {
      tbl <- gt::cols_align(tbl, align = "center", columns = center_cols)
    }
    right_cols <- intersect(c("n", "Weighted n", "p"), col_keys)
    if (length(right_cols) > 0L) {
      tbl <- gt::cols_align(tbl, align = "right", columns = right_cols)
    }

    rule <- gt::cell_borders(
      sides = "bottom",
      color = "currentColor",
      weight = gt::px(1)
    )
    rule_top <- gt::cell_borders(
      sides = "top",
      color = "currentColor",
      weight = gt::px(1)
    )
    tbl <- gt::tab_options(
      tbl,
      table.border.top.width = gt::px(0),
      table.border.bottom.width = gt::px(0),
      table_body.border.top.width = gt::px(0),
      table_body.border.bottom.width = gt::px(0),
      table_body.hlines.color = "transparent",
      column_labels.border.top.width = gt::px(0),
      column_labels.border.bottom.width = gt::px(0),
      column_labels.border.lr.color = "transparent"
    )
    tbl <- gt::tab_style(
      tbl,
      style = rule_top,
      locations = gt::cells_column_spanners()
    )
    if (has_ci) {
      tbl <- gt::tab_style(
        tbl,
        style = rule_top,
        locations = gt::cells_column_labels(columns = c("LL", "UL"))
      )
    }
    tbl <- gt::tab_style(
      tbl,
      style = rule,
      locations = gt::cells_column_labels()
    )
    tbl <- gt::tab_style(
      tbl,
      style = rule,
      locations = gt::cells_body(rows = nrow(display_df))
    )
    tbl <- gt::tab_style(
      tbl,
      style = gt::cell_text(align = "left"),
      locations = gt::cells_column_labels(columns = "Variable")
    )
    non_variable_cols <- setdiff(col_keys, "Variable")
    if (length(non_variable_cols) > 0L) {
      tbl <- gt::tab_style(
        tbl,
        style = gt::cell_text(align = "center"),
        locations = gt::cells_column_labels(columns = non_variable_cols)
      )
    }
    tbl <- gt::tab_style(
      tbl,
      style = gt::cell_text(align = "left"),
      locations = gt::cells_column_spanners(spanners = "spn_Variable")
    )

    ci_css_sel <- if (has_ci) {
      paste(
        vapply(
          c("LL", "UL"),
          function(id) sprintf('.gt_table thead tr:last-child th[id="%s"]', id),
          character(1)
        ),
        collapse = ",\n"
      )
    } else {
      ""
    }
    apa_css <- paste(
      ".gt_table thead tr:first-child {",
      "  border-top: 1px solid currentColor !important;",
      "}",
      ".gt_table thead tr.gt_spanner_row {",
      "  border-bottom-style: none !important;",
      "}",
      ".gt_table thead th, .gt_table thead td {",
      "  background-color: transparent !important;",
      "}",
      if (has_ci) paste0(ci_css_sel, " {") else "",
      if (has_ci) "  border-top: 1px solid currentColor !important;" else "",
      if (has_ci) "}" else "",
      ".gt_table thead tr:last-child {",
      "  border-bottom: 1px solid currentColor !important;",
      "}",
      ".gt_table tbody tr:last-child {",
      "  border-bottom: 1px solid currentColor !important;",
      "}",
      ".gt_table tbody tr {",
      "  border-top-style: none !important;",
      "  border-bottom-style: none !important;",
      "}",
      ".gt_table .gt_col_heading, .gt_table .gt_spanner {",
      "  white-space: nowrap !important;",
      "}",
      ".gt_table .gt_row .gt_right, .gt_table .gt_row .gt_center {",
      "  white-space: nowrap !important;",
      "}",
      sep = "\n"
    )
    tbl <- gt::opt_css(tbl, css = apa_css)

    return(tbl)
  }

  if (output %in% c("flextable", "word")) {
    if (!requireNamespace("flextable", quietly = TRUE)) {
      stop("Install package 'flextable'.", call. = FALSE)
    }
    if (
      identical(output, "word") &&
        !requireNamespace("officer", quietly = TRUE)
    ) {
      stop("Install package 'officer'.", call. = FALSE)
    }

    display_df <- rename_ci_cols_lm(display_df, ci_ll, ci_ul)
    col_keys <- names(display_df)
    hdrs <- build_header_rows_lm(col_keys, ci_pct)
    map <- data.frame(
      col_keys = col_keys,
      top = hdrs$top,
      bottom = hdrs$bottom,
      stringsAsFactors = FALSE,
      check.names = FALSE
    )

    ft <- flextable::flextable(display_df)
    ft <- flextable::set_header_df(ft, mapping = map, key = "col_keys")
    ft <- flextable::merge_h(ft, part = "header")

    bd <- spicy_fp_border(color = "black", width = 1)
    ci_j <- which(col_keys %in% c("LL", "UL"))
    left_j <- 1L
    right_j <- which(col_keys %in% c("n", "Weighted n", "p"))
    center_j <- setdiff(seq_along(col_keys), c(left_j, right_j))

    ft <- flextable::align(ft, j = left_j, part = "header", align = "left")
    ft <- flextable::align(ft, j = left_j, part = "body", align = "left")
    if (length(center_j) > 0L) {
      ft <- flextable::align(ft, j = center_j, part = "all", align = "center")
    }
    if (length(right_j) > 0L) {
      ft <- flextable::align(ft, j = right_j, part = "header", align = "center")
      ft <- flextable::align(ft, j = right_j, part = "body", align = "right")
    }

    ft <- flextable::hline_top(ft, part = "header", border = bd)
    if (has_ci) {
      ft <- flextable::hline(
        ft,
        i = 1,
        j = ci_j,
        part = "header",
        border = bd
      )
    }
    ft <- flextable::hline_bottom(ft, part = "header", border = bd)
    ft <- flextable::hline_bottom(ft, part = "body", border = bd)
    if ("p" %in% col_keys) {
      ft <- flextable::compose(
        ft,
        j = which(col_keys == "p"),
        part = "body",
        value = flextable::as_paragraph(
          flextable::as_chunk(display_df[[which(col_keys == "p")]])
        )
      )
    }
    ft <- flextable::autofit(ft)

    if (identical(output, "word")) {
      if (is.null(word_path) || !nzchar(word_path)) {
        stop(
          "`word_path` must be provided for `output = \"word\"`.",
          call. = FALSE
        )
      }
      flextable::save_as_docx(ft, path = word_path)
      return(word_path)
    }

    return(ft)
  }

  if (identical(output, "excel")) {
    if (!requireNamespace("openxlsx2", quietly = TRUE)) {
      stop("Install package 'openxlsx2'.", call. = FALSE)
    }
    if (is.null(excel_path) || !nzchar(excel_path)) {
      stop(
        "`excel_path` must be provided for `output = \"excel\"`.",
        call. = FALSE
      )
    }

    display_df <- rename_ci_cols_lm(display_df, ci_ll, ci_ul)
    col_keys <- names(display_df)
    nc <- length(col_keys)
    hdrs <- build_header_rows_lm(col_keys, ci_pct)
    ci_j <- which(col_keys %in% c("LL", "UL"))

    wb <- openxlsx2::wb_workbook()
    wb <- openxlsx2::wb_add_worksheet(wb, excel_sheet)
    wb <- openxlsx2::wb_add_data(
      wb,
      x = as.data.frame(t(hdrs$top), stringsAsFactors = FALSE),
      start_row = 1,
      col_names = FALSE
    )
    wb <- openxlsx2::wb_add_data(
      wb,
      x = as.data.frame(t(hdrs$bottom), stringsAsFactors = FALSE),
      start_row = 2,
      col_names = FALSE
    )
    wb <- openxlsx2::wb_add_data(
      wb,
      x = display_df,
      start_row = 3,
      col_names = FALSE,
      row_names = FALSE
    )
    if (has_ci) {
      wb <- openxlsx2::wb_merge_cells(
        wb,
        dims = openxlsx2::wb_dims(rows = 1, cols = ci_j)
      )
    }
    last_row <- 2 + nrow(display_df)

    left_cols <- 1L
    right_cols <- which(col_keys %in% c("n", "Weighted n", "p"))
    center_cols <- setdiff(seq_len(nc), c(left_cols, right_cols))
    header_rows <- 1:2
    body_rows <- if (last_row >= 3) 3:last_row else integer(0)

    wb <- openxlsx2::wb_add_cell_style(
      wb,
      dims = openxlsx2::wb_dims(rows = 1:last_row, cols = left_cols),
      horizontal = "left"
    )
    if (length(center_cols) > 0L) {
      wb <- openxlsx2::wb_add_cell_style(
        wb,
        dims = openxlsx2::wb_dims(rows = header_rows, cols = center_cols),
        horizontal = "center",
        vertical = "center"
      )
      if (length(body_rows) > 0L) {
        wb <- openxlsx2::wb_add_cell_style(
          wb,
          dims = openxlsx2::wb_dims(rows = body_rows, cols = center_cols),
          horizontal = "center",
          vertical = "center"
        )
      }
    }
    if (length(right_cols) > 0L) {
      wb <- openxlsx2::wb_add_cell_style(
        wb,
        dims = openxlsx2::wb_dims(rows = header_rows, cols = right_cols),
        horizontal = "center",
        vertical = "center"
      )
    }
    if (length(right_cols) > 0L && length(body_rows) > 0L) {
      wb <- openxlsx2::wb_add_cell_style(
        wb,
        dims = openxlsx2::wb_dims(rows = body_rows, cols = right_cols),
        horizontal = "right"
      )
    }

    wb <- openxlsx2::wb_add_border(
      wb,
      dims = openxlsx2::wb_dims(rows = 1, cols = 1:nc),
      top_border = "thin"
    )
    if (has_ci) {
      wb <- openxlsx2::wb_add_border(
        wb,
        dims = openxlsx2::wb_dims(rows = 1, cols = ci_j),
        bottom_border = "thin"
      )
    }
    wb <- openxlsx2::wb_add_border(
      wb,
      dims = openxlsx2::wb_dims(rows = 2, cols = 1:nc),
      bottom_border = "thin"
    )
    if (nrow(display_df) > 0) {
      wb <- openxlsx2::wb_add_border(
        wb,
        dims = openxlsx2::wb_dims(rows = last_row, cols = 1:nc),
        bottom_border = "thin"
      )
    }
    openxlsx2::wb_save(wb, excel_path, overwrite = TRUE)
    return(excel_path)
  }

  if (identical(output, "clipboard")) {
    if (!requireNamespace("clipr", quietly = TRUE)) {
      stop("Install package 'clipr'.", call. = FALSE)
    }

    display_df <- rename_ci_cols_lm(display_df, ci_ll, ci_ul)
    col_keys <- names(display_df)
    hdrs <- build_header_rows_lm(col_keys, ci_pct)
    clip_mat <- rbind(hdrs$top, hdrs$bottom, as.matrix(display_df))
    lines <- apply(clip_mat, 1, function(r) {
      paste(r, collapse = clipboard_delim)
    })
    clipr::write_clip(paste(lines, collapse = "\n"))
    message("Linear-model table copied to clipboard.")
    return(invisible(display_df))
  }

  stop("Unknown output format.", call. = FALSE)
}

rename_ci_cols_lm <- function(display_df, ci_ll, ci_ul) {
  names(display_df)[names(display_df) == ci_ll] <- "LL"
  names(display_df)[names(display_df) == ci_ul] <- "UL"
  display_df
}

build_header_rows_lm <- function(col_keys, ci_pct) {
  nc <- length(col_keys)
  top <- col_keys
  top[col_keys == "LL"] <- paste0(ci_pct, " CI")
  top[col_keys == "UL"] <- paste0(ci_pct, " CI")
  bottom <- rep("", nc)
  bottom[col_keys == "LL"] <- "LL"
  bottom[col_keys == "UL"] <- "UL"
  list(top = top, bottom = bottom)
}

get_delta_label_lm <- function(block) {
  paste0("\u0394 (", block$level[2], " - ", block$level[1], ")")
}

get_test_row_index_lm <- function(block) {
  if (identical(unique(block$predictor_type)[1], "continuous")) {
    return(1L)
  }
  if (nrow(block) == 2L && any(!is.na(block$estimate))) {
    return(which(!is.na(block$estimate))[1])
  }
  1L
}

get_test_header_lm <- function(block, show_statistic = TRUE, exact = TRUE) {
  if (!isTRUE(show_statistic)) {
    return(NULL)
  }
  predictor_type <- unique(block$predictor_type)
  predictor_type <- predictor_type[!is.na(predictor_type)][1]
  df1_vals <- unique(stats::na.omit(block$df1))
  df2_vals <- unique(stats::na.omit(block$df2))
  same_df1 <- length(df1_vals) == 1L
  same_df2 <- length(df2_vals) == 1L

  if (identical(predictor_type, "continuous")) {
    if (isTRUE(exact) && same_df2) {
      return(paste0("t(", as.integer(round(df2_vals)), ")"))
    }
    return("t")
  }
  if (length(unique(stats::na.omit(block$level))) == 2L) {
    if (isTRUE(exact) && same_df2) {
      return(paste0("t(", as.integer(round(df2_vals)), ")"))
    }
    return("t")
  }
  if (isTRUE(exact) && same_df1 && same_df2) {
    return(
      paste0(
        "F(",
        as.integer(round(df1_vals)),
        ", ",
        as.integer(round(df2_vals)),
        ")"
      )
    )
  }
  "F"
}

format_effect_size_header_lm <- function(effect_size = "f2") {
  switch(
    effect_size,
    f2 = "f\u00B2",
    effect_size
  )
}

format_r2_header_lm <- function(r2_type = "r2") {
  switch(
    r2_type,
    r2 = "R\u00B2",
    adj_r2 = "Adj. R\u00B2",
    r2_type
  )
}

get_r2_value_lm <- function(block, r2_type = "r2") {
  switch(
    r2_type,
    r2 = block$r2[1],
    adj_r2 = block$adj_r2[1],
    NA_real_
  )
}

format_number_lm <- function(x, digits = 2L, decimal_mark = ".") {
  if (length(x) > 1L) {
    return(vapply(
      x,
      format_number_lm,
      character(1),
      digits = digits,
      decimal_mark = decimal_mark
    ))
  }
  if (is.na(x)) {
    return("")
  }
  out <- formatC(x, digits = digits, format = "f")
  if (!identical(decimal_mark, ".")) {
    out <- chartr(".", decimal_mark, out)
  }
  out
}

format_p_value_lm <- function(p, decimal_mark = ".") {
  if (is.na(p)) {
    return("")
  }
  if (p < 0.001) {
    return(if (identical(decimal_mark, ".")) "<.001" else "<,001")
  }
  out <- format_number_lm(p, 3L, decimal_mark)
  out <- sub("^0(?=[\\.,])", "", out, perl = TRUE)
  out <- sub("^-0(?=[\\.,])", "-", out, perl = TRUE)
  out
}

# ---- structure ----

test_that("table_continuous_lm returns expected raw structure", {
  out <- table_continuous_lm(
    iris,
    select = c(Sepal.Length, Petal.Width),
    by = Species,
    output = "long"
  )

  expect_s3_class(out, "data.frame")
  expect_named(
    out,
    c(
      "variable",
      "label",
      "predictor_type",
      "predictor_label",
      "level",
      "reference",
      "estimate_type",
      "emmean",
      "emmean_se",
      "emmean_ci_lower",
      "emmean_ci_upper",
      "estimate",
      "estimate_se",
      "estimate_ci_lower",
      "estimate_ci_upper",
      "test_type",
      "statistic",
      "df1",
      "df2",
      "p.value",
      "es_type",
      "es_value",
      "r2",
      "adj_r2",
      "n",
      "sum_w"
    )
  )
  expect_equal(nrow(out), 6L)
})

test_that("table_continuous_lm default output returns a spicy table", {
  out <- table_continuous_lm(
    iris,
    select = c(Sepal.Length, Petal.Width),
    by = Species
  )

  expect_s3_class(out, "spicy_continuous_lm_table")
  expect_s3_class(out, "spicy_table")
  expect_equal(attr(out, "by_var"), "Species")
  expect_equal(attr(out, "vcov_type"), "classical")
})

# ---- computation ----

test_that("table_continuous_lm numeric predictor slope matches lm", {
  out <- table_continuous_lm(
    iris,
    select = Sepal.Length,
    by = Petal.Width,
    output = "long"
  )
  fit <- lm(Sepal.Length ~ Petal.Width, data = iris)

  expect_equal(out$predictor_type, "continuous")
  expect_equal(out$estimate, unname(coef(fit)[["Petal.Width"]]))
  expect_equal(out$r2, unname(summary(fit)$r.squared))
})

test_that("table_continuous_lm binary factor stores means and auto contrast", {
  df <- iris[iris$Species != "virginica", ]
  df$Species <- droplevels(df$Species)

  out <- table_continuous_lm(
    df,
    select = Sepal.Length,
    by = Species,
    output = "long"
  )

  expect_equal(nrow(out), 2L)
  expect_equal(out$level, c("setosa", "versicolor"))
  expect_true(is.na(out$estimate[1]))
  expect_equal(
    out$estimate[2],
    mean(df$Sepal.Length[df$Species == "versicolor"]) -
      mean(df$Sepal.Length[df$Species == "setosa"])
  )
  expect_false(is.na(out$p.value[1]))
  expect_equal(out$es_type[1], "f2")
})

test_that("table_continuous_lm 3+ level factor reports estimated means", {
  out <- table_continuous_lm(
    iris,
    select = Sepal.Length,
    by = Species,
    output = "long"
  )

  expect_equal(nrow(out), 3L)
  expect_true(all(is.na(out$estimate)))
  expect_equal(
    out$emmean,
    as.vector(tapply(iris$Sepal.Length, iris$Species, mean))
  )
  expect_false(is.na(out$statistic[1]))
  expect_true(all(is.na(out$statistic[-1])))
})

test_that("table_continuous_lm contrast none suppresses binary contrasts", {
  df <- iris[iris$Species != "virginica", ]
  df$Species <- droplevels(df$Species)

  out <- table_continuous_lm(
    df,
    select = Sepal.Length,
    by = Species,
    contrast = "none",
    output = "long"
  )

  expect_true(is.na(out$estimate[1]))
  expect_true(is.na(out$estimate[2]))
  expect_true(all(is.na(out$estimate_type)))
})

test_that("table_continuous_lm weights accept column names", {
  df <- data.frame(
    y = c(1, 2, 3, 10),
    g = factor(c("A", "A", "B", "B")),
    w = c(1, 1, 1, 5)
  )

  out <- table_continuous_lm(
    df,
    select = y,
    by = g,
    weights = w,
    output = "long"
  )

  expect_equal(out$sum_w[1], sum(df$w))
  expect_equal(
    out$emmean[2],
    weighted.mean(df$y[df$g == "B"], df$w[df$g == "B"])
  )
})

test_that("table_continuous_lm can display weighted n separately from n", {
  df <- data.frame(
    y = c(1, 2, 3, 10),
    g = factor(c("A", "A", "B", "B")),
    w = c(1, 1, 1, 5)
  )

  out <- table_continuous_lm(
    df,
    select = y,
    by = g,
    weights = w,
    show_weighted_n = TRUE,
    output = "data.frame"
  )

  expect_true(all(c("n", "Weighted n") %in% names(out)))
  expect_equal(out$n[1], 4)
  expect_equal(out$`Weighted n`[1], sum(df$w))
  expect_lt(match("n", names(out)), match("Weighted n", names(out)))
})

test_that("table_continuous_lm ignores weighted n when weights are absent", {
  expect_warning(
    out <- table_continuous_lm(
      iris,
      select = Sepal.Length,
      by = Species,
      show_weighted_n = TRUE,
      output = "data.frame"
    ),
    "show_weighted_n"
  )

  expect_false("Weighted n" %in% names(out))
})

test_that("table_continuous_lm HC3 changes standard errors on heteroskedastic data", {
  df <- data.frame(
    y = c(1.0, 1.2, 1.1, 5.0, 7.5, 10.0),
    x = c(0, 0, 0, 1, 1, 1)
  )

  classical <- table_continuous_lm(
    df,
    select = y,
    by = x,
    vcov = "classical",
    output = "long"
  )
  robust <- table_continuous_lm(
    df,
    select = y,
    by = x,
    vcov = "HC3",
    output = "long"
  )

  expect_false(isTRUE(all.equal(classical$estimate_se, robust$estimate_se)))
})

test_that("table_continuous_lm HC4, HC4m, and HC5 use leverage-adjusted SE", {
  df <- data.frame(
    y = c(2, 2.2, 2.1, 2.4, 3.1, 8.5),
    x = c(0, 1, 2, 3, 4, 20)
  )

  manual_se <- function(type) {
    fit <- lm(y ~ x, data = df)
    xmat <- model.matrix(fit)
    e <- residuals(fit)
    xtx_inv <- solve(crossprod(xmat))
    hat <- rowSums((xmat %*% xtx_inv) * xmat)
    n <- nrow(xmat)
    p <- max(1L, as.integer(round(sum(hat))))
    h_adj <- pmax(1 - hat, .Machine$double.eps)
    nh_over_p <- n * hat / p

    scale <- switch(
      type,
      HC4 = {
        delta <- pmin(4, nh_over_p)
        1 / h_adj^delta
      },
      HC4m = {
        delta <- pmin(1, nh_over_p) + pmin(1.5, nh_over_p)
        1 / h_adj^delta
      },
      HC5 = {
        delta <- pmin(
          nh_over_p,
          pmax(4, n * 0.7 * max(hat) / p)
        )
        1 / sqrt(h_adj^delta)
      }
    )

    vc <- xtx_inv %*% crossprod(xmat, (e^2 * scale) * xmat) %*% xtx_inv
    sqrt(diag(vc))[2]
  }

  out_hc4 <- table_continuous_lm(
    df,
    select = y,
    by = x,
    vcov = "HC4",
    output = "long"
  )
  out_hc4m <- table_continuous_lm(
    df,
    select = y,
    by = x,
    vcov = "HC4m",
    output = "long"
  )
  out_hc5 <- table_continuous_lm(
    df,
    select = y,
    by = x,
    vcov = "HC5",
    output = "long"
  )

  expect_equal(out_hc4$estimate_se, unname(manual_se("HC4")))
  expect_equal(out_hc4m$estimate_se, unname(manual_se("HC4m")))
  expect_equal(out_hc5$estimate_se, unname(manual_se("HC5")))
})

# ---- display ----

test_that("table_continuous_lm uses dedicated digits for fit and effect size", {
  out <- table_continuous_lm(
    iris,
    select = Sepal.Length,
    by = Species,
    effect_size = "f2",
    digits = 1,
    fit_digits = 4,
    effect_size_digits = 3
  )

  display <- spicy:::build_wide_display_df_continuous_lm(
    out,
    digits = 1L,
    decimal_mark = ".",
    ci_level = 0.95,
    show_statistic = FALSE,
    show_p_value = TRUE,
    show_n = TRUE,
    effect_size = "f2",
    r2_type = "r2",
    ci = TRUE,
    fit_digits = 4L,
    effect_size_digits = 3L
  )

  expect_equal(attr(out, "fit_digits"), 4L)
  expect_equal(attr(out, "effect_size_digits"), 3L)
  expect_match(display$`R²`[1], "^0\\.[0-9]{4}$")
  expect_match(display$`f²`[1], "^[0-9]+\\.[0-9]{3}$")
})

test_that("table_continuous_lm rejects deprecated contrast value", {
  expect_error(
    table_continuous_lm(
      sochealth,
      select = wellbeing_score,
      by = education,
      contrast = "reference",
      output = "long"
    ),
    "should be one of"
  )
})

test_that("table_continuous_lm data.frame output puts >2 categorical levels in columns", {
  out <- table_continuous_lm(
    iris,
    select = c(Sepal.Length, Petal.Width),
    by = Species,
    statistic = TRUE,
    output = "data.frame"
  )

  expect_s3_class(out, "data.frame")
  expect_true("M (setosa)" %in% names(out))
  expect_true("M (versicolor)" %in% names(out))
  expect_true("M (virginica)" %in% names(out))
  expect_false(any(grepl("^Δ", names(out))))
  expect_false(any(grepl("CI", names(out))))
  expect_true("F(2, 147)" %in% names(out))
  expect_true("p" %in% names(out))
  expect_false("f²" %in% names(out))
  expect_true("R²" %in% names(out))
  expect_equal(out$Variable[1], "Sepal.Length")
  expect_true(is.numeric(out$p))
})

test_that("table_continuous_lm data.frame output labels binary contrasts explicitly", {
  df <- iris[iris$Species != "virginica", ]
  df$Species <- droplevels(df$Species)

  out <- table_continuous_lm(
    df,
    select = c(Sepal.Length, Petal.Width),
    by = Species,
    statistic = TRUE,
    output = "data.frame"
  )

  expect_true("M (setosa)" %in% names(out))
  expect_true("M (versicolor)" %in% names(out))
  expect_true("Δ (versicolor - setosa)" %in% names(out))
  expect_true("95% CI LL" %in% names(out))
  expect_true("95% CI UL" %in% names(out))
  expect_true(any(grepl("^t\\([0-9]+\\)$", names(out))))
  expect_true("R²" %in% names(out))
  expect_true(is.numeric(out$`Δ (versicolor - setosa)`))
})

test_that("table_continuous_lm data.frame output honors optional columns", {
  out <- table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    statistic = FALSE,
    p_value = FALSE,
    show_n = FALSE,
    effect_size = "none",
    output = "data.frame"
  )

  expect_false("t" %in% names(out))
  expect_false("p" %in% names(out))
  expect_false("n" %in% names(out))
  expect_false("f²" %in% names(out))
  expect_true("R²" %in% names(out))
})

test_that("table_continuous_lm data.frame can hide CI columns", {
  out <- table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    ci = FALSE,
    output = "data.frame"
  )

  expect_false(any(grepl("CI", names(out))))
})

test_that("table_continuous_lm data.frame uses R2 by default and can show adjusted R2", {
  out_r2 <- table_continuous_lm(
    iris,
    select = c(Sepal.Length, Petal.Width),
    by = Species,
    output = "data.frame"
  )
  out <- table_continuous_lm(
    iris,
    select = c(Sepal.Length, Petal.Width),
    by = Species,
    r2 = "adj_r2",
    output = "data.frame"
  )

  expect_true("R²" %in% names(out_r2))
  expect_false("Adj. R²" %in% names(out_r2))
  expect_true("Adj. R²" %in% names(out))
  expect_false("R²" %in% names(out))
  expect_true(is.numeric(out$`Adj. R²`))
})

test_that("table_continuous_lm rejects deprecated wide output", {
  expect_error(
    table_continuous_lm(
      sochealth,
      select = c(wellbeing_score, bmi),
      by = sex,
      output = "wide"
    ),
    "should be one of"
  )
})

# ---- validation ----

test_that("table_continuous_lm validates select and weights", {
  df <- data.frame(y = letters[1:3], x = 1:3, w = c(1, -1, 1))

  expect_warning(
    out <- table_continuous_lm(df, select = y, by = x, output = "long"),
    "No numeric outcome columns selected"
  )
  expect_equal(ncol(out), 0L)
  expect_error(
    table_continuous_lm(
      transform(df, y = 1:3),
      select = y,
      by = x,
      weights = w
    ),
    "weights"
  )
})

test_that("table_continuous_lm validates core user arguments", {
  df <- data.frame(y = 1:3, x = 1:3)

  expect_error(
    table_continuous_lm(list(y = 1:3), select = y, by = x, output = "long"),
    "data.frame"
  )
  expect_error(
    table_continuous_lm(df, select = y, by = x, ci_level = 1, output = "long"),
    "ci_level"
  )
  expect_error(
    table_continuous_lm(df, select = y, by = x, digits = -1, output = "long"),
    "digits"
  )
  expect_error(
    table_continuous_lm(
      df,
      select = y,
      by = x,
      fit_digits = -1,
      output = "long"
    ),
    "fit_digits"
  )
  expect_error(
    table_continuous_lm(
      df,
      select = y,
      by = x,
      effect_size_digits = -1,
      output = "long"
    ),
    "effect_size_digits"
  )
  expect_error(
    table_continuous_lm(
      df,
      select = y,
      by = x,
      decimal_mark = ";",
      output = "long"
    ),
    "decimal_mark"
  )
  expect_error(
    table_continuous_lm(
      df,
      select = y,
      by = x,
      labels = c("Y label"),
      output = "long"
    ),
    "named character vector"
  )
  expect_error(
    table_continuous_lm(
      df,
      select = y,
      by = x,
      statistic = NA,
      output = "long"
    ),
    "TRUE/FALSE"
  )
})

test_that("table_continuous_lm validates predictor and regex selection", {
  df <- data.frame(
    y = 1:3,
    z = 4:6,
    bad = I(list(1, 2, 3))
  )

  expect_error(
    table_continuous_lm(df, select = y, by = bad, output = "long"),
    "must be numeric, logical, character, or factor"
  )
  expect_error(
    table_continuous_lm(df, select = c("y", "z"), by = y, regex = TRUE),
    "single regex pattern"
  )
  expect_error(
    table_continuous_lm(df, select = unknown, by = y, output = "long"),
    "select"
  )
})

test_that("table_continuous_lm validates weight edge cases", {
  df <- data.frame(y = 1:3, x = 1:3, w = c(1, Inf, 1), z = c(0, 0, 0))

  err1 <- tryCatch(
    {
      table_continuous_lm(df, select = y, by = x, weights = w, output = "long")
      NULL
    },
    error = identity
  )
  err2 <- tryCatch(
    {
      table_continuous_lm(df, select = y, by = x, weights = z, output = "long")
      NULL
    },
    error = identity
  )

  expect_match(conditionMessage(err1), "finite")
  expect_match(conditionMessage(err2), "at least one positive value")
})

test_that("table_continuous_lm uses labels, exclude, regex, and character by", {
  df <- sochealth
  attr(df$wellbeing_score, "label") <- "Wellbeing attr"

  out <- table_continuous_lm(
    df,
    select = "^well|^bmi$",
    by = "sex",
    regex = TRUE,
    exclude = bmi,
    output = "data.frame"
  )

  expect_equal(out$Variable, "Wellbeing attr")
})

test_that("table_continuous_lm reports ignored non-numeric outcomes in verbose mode", {
  expect_message(
    out <- table_continuous_lm(
      sochealth,
      select = c(wellbeing_score, sex),
      by = age,
      verbose = TRUE,
      output = "data.frame"
    ),
    "Ignoring non-numeric selected outcomes: sex"
  )

  expect_equal(out$Variable, "WHO-5 wellbeing index (0-100)")
})

test_that("table_continuous_lm helper functions cover empty-model cases", {
  num_empty <- spicy:::fit_numeric_predictor_lm_rows(
    y = c(1, 2),
    x = c(1, 1),
    weights = NULL,
    outcome_name = "y",
    outcome_label = "Y",
    predictor_label = "X",
    vcov_type = "classical",
    ci_level = 0.95
  )
  cat_empty <- spicy:::fit_categorical_predictor_lm_rows(
    y = c(1, 2),
    x = factor(c("A", "A")),
    weights = NULL,
    outcome_name = "y",
    outcome_label = "Y",
    predictor_label = "G",
    vcov_type = "classical",
    contrast = "auto",
    ci_level = 0.95
  )

  expect_true(all(is.na(num_empty$estimate)))
  expect_equal(num_empty$predictor_type, "continuous")
  expect_true(all(is.na(cat_empty$emmean)))
  expect_equal(cat_empty$predictor_type, "categorical")
})

test_that("table_continuous_lm helper functions handle coercion and weights detection", {
  expect_true(is.factor(spicy:::coerce_lm_factor(c("a", "b"))))
  expect_true(is.factor(spicy:::coerce_lm_factor(factor("a"))))
  expect_true(spicy:::is_supported_lm_predictor(TRUE))
  expect_false(spicy:::is_supported_lm_predictor(list(1, 2)))

  df <- data.frame(y = 1:3, w = c(1, 2, 3))
  expect_equal(
    spicy:::detect_weights_column_name(rlang::quo(w), df),
    "w"
  )
  expect_equal(
    spicy:::detect_weights_column_name(rlang::quo("w"), df),
    "w"
  )
  expect_null(spicy:::detect_weights_column_name(rlang::quo(c(1, 2, 3)), df))
})

test_that("table_continuous_lm internal builders cover numeric displays", {
  out <- table_continuous_lm(
    sochealth,
    select = wellbeing_score,
    by = age,
    statistic = TRUE,
    effect_size = "f2",
    output = "long"
  )

  wide_raw <- spicy:::build_wide_raw_continuous_lm(
    out,
    show_statistic = TRUE,
    show_p_value = TRUE,
    show_n = TRUE,
    effect_size = "f2",
    r2_type = "adj_r2",
    ci = TRUE
  )
  wide_display <- spicy:::build_wide_display_df_continuous_lm(
    out,
    digits = 2L,
    decimal_mark = ".",
    ci_level = 0.95,
    show_statistic = TRUE,
    show_p_value = TRUE,
    show_n = TRUE,
    effect_size = "f2",
    r2_type = "adj_r2",
    ci = TRUE
  )

  expect_true(all(
    c("B", "95% CI LL", "95% CI UL", "p", "Adj. R²", "f²", "n") %in%
      names(wide_raw)
  ))
  expect_lt(match("Adj. R²", names(wide_raw)), match("f²", names(wide_raw)))
  expect_true(any(grepl("^t", names(wide_raw))))
  expect_true(all(
    c("Variable", "B", "95% CI LL", "95% CI UL", "p", "Adj. R²", "f²", "n") %in%
      names(wide_display)
  ))
  expect_lt(
    match("Adj. R²", names(wide_display)),
    match("f²", names(wide_display))
  )
  expect_true(any(grepl("^t", names(wide_display))))
})

test_that("table_continuous_lm internal builders cover binary categorical variants", {
  df <- sochealth
  out <- table_continuous_lm(
    df,
    select = c(wellbeing_score, bmi),
    by = sex,
    statistic = FALSE,
    p_value = FALSE,
    show_n = FALSE,
    effect_size = "none",
    r2 = "none",
    ci = FALSE,
    output = "long"
  )

  wide_raw <- spicy:::build_wide_raw_continuous_lm(
    out,
    show_statistic = FALSE,
    show_p_value = FALSE,
    show_n = FALSE,
    effect_size = "none",
    r2_type = "none",
    ci = FALSE
  )
  wide_display <- spicy:::build_wide_display_df_continuous_lm(
    out,
    digits = 2L,
    decimal_mark = ",",
    ci_level = 0.95,
    show_statistic = FALSE,
    show_p_value = FALSE,
    show_n = FALSE,
    effect_size = "none",
    r2_type = "none",
    ci = FALSE
  )

  expect_true(all(
    c("M (Female)", "M (Male)", "Δ (Male - Female)") %in% names(wide_raw)
  ))
  expect_false(any(
    c("p", "n", "f²", "R²", "Adj. R²", "95% CI LL") %in% names(wide_raw)
  ))
  expect_true(all(
    c("Variable", "M (Female)", "M (Male)", "Δ (Male - Female)") %in%
      names(wide_display)
  ))
  expect_false(any(
    c("p", "n", "f²", "R²", "Adj. R²", "95% CI LL") %in% names(wide_display)
  ))
})

test_that("table_continuous_lm low-level formatting helpers behave as expected", {
  block <- table_continuous_lm(
    iris[iris$Species != "virginica", ],
    select = Sepal.Length,
    by = Species,
    output = "long"
  )

  expect_equal(
    spicy:::rename_ci_cols_lm(
      setNames(
        data.frame(1, 2, check.names = FALSE),
        c("95% CI LL", "95% CI UL")
      ),
      "95% CI LL",
      "95% CI UL"
    ),
    setNames(data.frame(1, 2, check.names = FALSE), c("LL", "UL"))
  )
  hdr <- spicy:::build_header_rows_lm(c("Variable", "LL", "UL"), "95%")
  expect_equal(hdr$top, c("Variable", "95% CI", "95% CI"))
  expect_equal(hdr$bottom, c("", "LL", "UL"))
  expect_match(spicy:::get_delta_label_lm(block), "versicolor - setosa")
  expect_equal(spicy:::get_test_row_index_lm(block), 2L)
  expect_equal(spicy:::get_test_header_lm(block, TRUE, FALSE), "t")
  expect_null(spicy:::get_test_header_lm(block, FALSE, TRUE))
  expect_equal(spicy:::format_effect_size_header_lm("f2"), "f²")
  expect_equal(spicy:::format_r2_header_lm("adj_r2"), "Adj. R²")
  expect_true(is.numeric(spicy:::get_r2_value_lm(block, "r2")))
  expect_equal(spicy:::format_number_lm(c(1.2, NA), 1L, ","), c("1,2", ""))
  expect_equal(spicy:::format_p_value_lm(NA_real_), "")
  expect_equal(spicy:::format_p_value_lm(0.045, ","), ",045")
})

test_that("table_continuous_lm internal covariance helper covers fallback branches", {
  fit <- lm(mpg ~ wt, data = mtcars)
  fit_singular <- lm(mpg ~ wt + I(2 * wt), data = mtcars)

  expect_error(
    spicy:::compute_lm_vcov(fit, "bogus"),
    "Unknown `vcov` type"
  )

  expect_warning(
    vc <- spicy:::compute_lm_vcov(fit_singular, "HC3"),
    "singular"
  )
  expect_true(is.matrix(vc))
  expect_equal(dim(vc), c(3L, 3L))
  expect_true(all(is.na(vc)))
})

test_that("table_continuous_lm export helper validates output-specific paths", {
  display_df <- data.frame(
    Variable = "x",
    `M (A)` = "1.00",
    p = ".050",
    check.names = FALSE
  )

  expect_error(
    spicy:::export_continuous_lm_table(
      display_df,
      output = "excel",
      ci_level = 0.95,
      excel_path = NULL,
      excel_sheet = "Sheet1",
      clipboard_delim = "\t",
      word_path = NULL
    ),
    "excel_path"
  )
  expect_error(
    spicy:::export_continuous_lm_table(
      display_df,
      output = "word",
      ci_level = 0.95,
      excel_path = NULL,
      excel_sheet = "Sheet1",
      clipboard_delim = "\t",
      word_path = NULL
    ),
    "word_path"
  )
  expect_error(
    spicy:::export_continuous_lm_table(
      display_df,
      output = "bogus",
      ci_level = 0.95,
      excel_path = NULL,
      excel_sheet = "Sheet1",
      clipboard_delim = "\t",
      word_path = NULL
    ),
    "Unknown output format"
  )
})

# ---- optional outputs ----

test_that("table_continuous_lm tinytable output works", {
  skip_if_not_installed("tinytable")

  out <- table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    output = "tinytable"
  )

  expect_true(inherits(out, "tinytable"))
})

test_that("table_continuous_lm gt output works", {
  skip_if_not_installed("gt")

  out <- table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    output = "gt"
  )

  expect_s3_class(out, "gt_tbl")
})

test_that("table_continuous_lm gt output includes a CI spanner", {
  skip_if_not_installed("gt")

  out <- table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    statistic = TRUE,
    output = "gt"
  )

  spanners <- out[["_spanners"]]
  labels <- unlist(spanners$spanner_label)
  expect_true(any(grepl("CI", labels)))
})

test_that("table_continuous_lm flextable output works", {
  skip_if_not_installed("flextable")

  out <- table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    output = "flextable"
  )

  expect_s3_class(out, "flextable")
})

test_that("table_continuous_lm flextable has 2-row header with CI spanner", {
  skip_if_not_installed("flextable")

  out <- table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    statistic = TRUE,
    output = "flextable"
  )

  hdr <- out$header$dataset
  expect_equal(nrow(hdr), 2L)
  expect_true(any(grepl("CI", hdr[1, ])))
})

test_that("table_continuous_lm excel output works", {
  skip_if_not_installed("openxlsx2")

  tmp <- tempfile(fileext = ".xlsx")
  on.exit(unlink(tmp), add = TRUE)

  local_mocked_bindings(
    wb_save = function(wb, file, overwrite = TRUE, ...) {
      invisible(wb)
    },
    .package = "openxlsx2"
  )

  out <- table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    output = "excel",
    excel_path = tmp
  )

  expect_equal(out, tmp)
})

test_that("table_continuous_lm word output works", {
  skip_if_not_installed("flextable")
  skip_if_not_installed("officer")

  tmp <- tempfile(fileext = ".docx")
  on.exit(unlink(tmp), add = TRUE)

  out <- table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    output = "word",
    word_path = tmp
  )

  expect_equal(out, tmp)
  expect_true(file.exists(tmp))
})

test_that("table_continuous_lm clipboard output can be exercised with a mocked writer", {
  skip_if_not_installed("clipr")

  captured <- NULL

  local_mocked_bindings(
    write_clip = function(text, ...) {
      captured <<- text
      invisible(text)
    },
    .package = "clipr"
  )

  expect_message(
    out <- table_continuous_lm(
      sochealth,
      select = c(wellbeing_score, bmi),
      by = sex,
      statistic = TRUE,
      output = "clipboard",
      clipboard_delim = ";"
    ),
    "copied to clipboard"
  )

  expect_s3_class(out, "data.frame")
  expect_true(is.character(captured))
  expect_match(
    captured,
    "Variable;M \\(Female\\);M \\(Male\\);Δ \\(Male - Female\\)"
  )
  expect_match(captured, "95% CI;95% CI")
  expect_match(captured, "LL;UL")
  expect_match(captured, "<\\.001|<,001")
})

# Resolve a single-column argument that may be provided as:
# - an unquoted column name (`by = grp`)
# - a single character column name (`by = "grp"`)
# - a character scalar stored in an external object (`by = by_col`)
# - a tidyselect expression (`by = all_of(by_col)`)
resolve_single_column_selection <- function(quo, data, arg) {
  val <- tryCatch(
    rlang::eval_tidy(quo, env = rlang::quo_get_env(quo)),
    error = function(e) NULL
  )

  if (is.character(val)) {
    if (length(val) != 1L || !val %in% names(data)) {
      stop(
        sprintf("`%s` must select exactly one column in `data`.", arg),
        call. = FALSE
      )
    }
    return(val)
  }

  pos <- tryCatch(
    tidyselect::eval_select(quo, data),
    error = function(e) {
      stop(
        sprintf("`%s` must select exactly one column in `data`.", arg),
        call. = FALSE
      )
    }
  )

  if (length(pos) != 1L) {
    stop(
      sprintf("`%s` must select exactly one column in `data`.", arg),
      call. = FALSE
    )
  }

  names(pos)
}

# Resolve a multi-column argument that may be provided as:
# - an unquoted selection (`exclude = c(x, y)`)
# - a character vector (`exclude = c("x", "y")`)
# - a character vector stored in an external object
# - a tidyselect helper (`exclude = starts_with("Sepal")`)
resolve_multi_column_selection <- function(quo, data, arg) {
  if (rlang::quo_is_null(quo)) {
    return(character())
  }

  sentinel <- new.env(parent = emptyenv())
  val <- tryCatch(
    rlang::eval_tidy(quo, env = rlang::quo_get_env(quo)),
    error = function(e) sentinel
  )

  if (is.null(val)) {
    return(character())
  }

  if (is.character(val)) {
    return(val)
  }

  pos <- tryCatch(
    tidyselect::eval_select(quo, data),
    error = function(e) {
      stop(
        sprintf("`%s` must select columns in `data`.", arg),
        call. = FALSE
      )
    }
  )

  names(pos)
}

# Resolve a weights argument that may be provided as:
# - an unquoted numeric column (`weights = w`)
# - a single character column name (`weights = "w"`)
# - a character scalar stored in an external object (`weights = wt_col`)
# - a numeric vector stored in an external object
resolve_weights_argument <- function(quo, data, arg = "weights") {
  if (rlang::quo_is_null(quo)) {
    return(NULL)
  }

  sentinel <- new.env(parent = emptyenv())
  val <- tryCatch(
    rlang::eval_tidy(
      quo,
      data = data,
      env = rlang::quo_get_env(quo)
    ),
    error = function(e) sentinel
  )

  if (identical(val, sentinel)) {
    stop(
      sprintf(
        "`%s` must be NULL, numeric vector, or a single column name.",
        arg
      ),
      call. = FALSE
    )
  }

  if (is.null(val)) {
    return(NULL)
  }

  if (is.character(val)) {
    if (length(val) != 1L || !val %in% names(data)) {
      stop(
        sprintf(
          "When character, `%s` must be a column name in `data`.",
          arg
        ),
        call. = FALSE
      )
    }
    val <- data[[val]]
  }

  if (!is.numeric(val)) {
    stop(
      sprintf(
        "`%s` must be NULL, numeric vector, or a single column name.",
        arg
      ),
      call. = FALSE
    )
  }

  if (length(val) != nrow(data)) {
    stop(
      sprintf("Numeric `%s` must have length `nrow(data)`.", arg),
      call. = FALSE
    )
  }

  unname(val)
}

#' Row-wise Count of Specific or Special Values
#'
#' `count_n()` counts, for each row of a data frame or matrix, how many times one or more values appear across selected columns.
#' It supports type-safe comparison, case-insensitive string matching, and detection of special values such as `NA`, `NaN`, `Inf`, and `-Inf`.
#'
#' This function is particularly useful for summarizing data quality or patterns in row-wise structures,
#' and is designed to work fluently inside `dplyr::mutate()` pipelines.
#'
#' Internally, `count_n()` wraps the stable and dependency-free base function `base_count_n()`, allowing high flexibility and testability.
#'
#' @param data A data frame or matrix. Optional inside `mutate()`.
#' @param select Columns to include. Defaults to `tidyselect::everything()`.
#'   Uses tidyselect helpers like [tidyselect::starts_with()], etc.
#'   If `regex = TRUE`, `select` is treated as a regex string.
#' @param exclude Character vector of column names to exclude after selection.
#'   Defaults to `NULL` (no exclusion).
#' @param count Value(s) to count. Defaults to `NULL`. Ignored if `special` is used.
#'   Multiple values are allowed (e.g., `count = c(1, 2, 3)` or `count = c("yes", "no")`).
#'   R automatically coerces all values in `count` to a common type (e.g., `c(2, "2")` becomes `c("2", "2")`),
#'   so all values are expected to be of the same final type.
#'   If `allow_coercion = FALSE`, matching is type-safe using `identical()`, and the type of `count` must match that of the values in the data.
#' @param special Character vector of special values to count: `"NA"`, `"NaN"`, `"Inf"`, `"-Inf"`, or `"all"`.
#'   Defaults to `NULL`.
#'   `"NA"` uses `is.na()`, and therefore includes both `NA` and `NaN` values.
#'   `"NaN"` uses `is.nan()` to match only actual NaN values.
#' @param allow_coercion Logical. If `TRUE` (the default), values are compared after coercion.
#'   If `FALSE`, uses strict matching via `identical()`.
#' @param ignore_case Logical. If `FALSE` (the default), comparisons are case-sensitive.
#'   If `TRUE`, performs case-insensitive string comparisons.
#' @param regex Logical. If `FALSE` (the default), uses tidyselect helpers.
#'   If `TRUE`, interprets `select` as a regular expression pattern.
#' @param verbose Logical. If `FALSE` (the default), messages are suppressed.
#'   If `TRUE`, prints processing messages.
#'
#' @return A numeric vector of row-wise counts (unnamed).
#'
#' @note
#' This function is inspired by [datawizard::row_count()], but provides additional flexibility:
#'
#' * **Element-wise type-safe matching** using `identical()` when `allow_coercion = FALSE`. This ensures that both the value and its type match exactly, enabling precise comparisons in mixed-type columns.
#' * **Support for multiple values in `count`**, allowing queries like `count = c(2, 3)` or `count = c("yes", "no")` to count any of several values per row.
#' * **Detection of special values** such as `NA`, `NaN`, `Inf`, and `-Inf` through the `special` argument — a feature not available in `row_count()`.
#' * **Tidyverse-native behavior**: can be used inside `mutate()` without explicitly passing a `data` argument.
#'
#' ### Value coercion behavior
#' R automatically coerces mixed-type vectors passed to `count` into a common type.
#' For example, `count = c(2, "2")` becomes `c("2", "2")`, because R converts numeric and character values to a unified type.
#' This means that mixed-type checks are not possible at runtime once `count` is passed to the function.
#' To ensure accurate type-sensitive matching, users should avoid mixing types in `count` explicitly.
#'
#' ### Strict matching mode (`allow_coercion = FALSE`)
#' When strict matching is enabled, each value in `count` must match the type of the target column exactly.
#'
#' For factor columns, this means that `count` must also be a factor. Supplying `count = "b"` (a character string) will not match a factor value, even if the label appears identical.
#'
#' A common and intuitive approach is to use `count = factor("b")`, which works in many cases. However, `identical()` — used internally for strict comparisons — also checks the internal structure of the factor, including the order and content of its levels.
#' As a result, comparisons may still fail if the levels differ, even when the label is the same.
#'
#' To ensure a perfect match (label **and** levels), you can reuse a value taken directly from the data (e.g., `df$x[2]`). This guarantees that both the class and the factor levels align. However, this approach only works reliably if all selected columns have the same factor structure.
#'
#' ### Case-insensitive matching (`ignore_case = TRUE`)
#' When `ignore_case = TRUE`, all values involved in the comparison are converted to lowercase using `tolower()` before matching.
#' This behavior applies to both character and factor columns. Factors are first converted to character internally.
#'
#' Importantly, this case-insensitive mode takes precedence over strict type comparison: values are no longer compared using `identical()`, but rather using lowercase string equality. This enables more flexible matching — for example, `"b"` and `"B"` will match even when `allow_coercion = FALSE`.
#'
#' #### Example: strict vs. case-insensitive matching with factors
#' ```r
#' df <- tibble::tibble(
#'   x = factor(c("a", "b", "c")),
#'   y = factor(c("b", "B", "a"))
#' )
#'
#' # Strict match fails with character input
#' count_n(df, count = "b", allow_coercion = FALSE)
#' #> [1] 0 0 0
#'
#' # Match works only where factor levels match exactly
#' count_n(df, count = factor("b", levels = levels(df$x)), allow_coercion = FALSE)
#' #> [1] 0 1 0
#'
#' # Case-insensitive match succeeds for both "b" and "B"
#' count_n(df, count = "b", ignore_case = TRUE)
#' #> [1] 1 2 0
#' ```
#'
#' Like [datawizard::row_count()], this function also supports regex-based column selection, case-insensitive string comparison, and column exclusion.
#'
#' @examples
#' library(dplyr)
#' library(tibble)
#' library(haven)
#'
#' # Basic usage
#' df <- tibble(
#'   x = c(1, 2, 2, 3, NA),
#'   y = c(2, 2, NA, 3, 2),
#'   z = c("2", "2", "2", "3", "2")
#' )
#' count_n(df, count = 2)
#' count_n(df, count = 2, allow_coercion = FALSE)
#' df |> mutate(num_twos = count_n(count = 2))
#'
#' # Mixed types and special values
#' df <- tibble(
#'   num   = c(1, 2, NA, -Inf, NaN),
#'   char  = c("a", "B", "b", "a", NA),
#'   fact  = factor(c("a", "b", "b", "a", "c")),
#'   date  = as.Date(c("2023-01-01", "2023-01-01", NA, "2023-01-02", "2023-01-01")),
#'   lab   = labelled(c(1, 2, 1, 2, NA), labels = c(No = 1, Yes = 2)),
#'   logic = c(TRUE, FALSE, NA, TRUE, FALSE)
#' )
#' count_n(df, count = 2)
#' count_n(df, count = "b", ignore_case = TRUE)
#' count_n(df, count = "a", select = fact)
#' count_n(df, count = as.Date("2023-01-01"), select = date)
#'
#' # Count special values
#' count_n(df, special = "NA")
#'
#' # Column selection strategies
#' df <- tibble(
#'   score_math    = c(1, 2, 2, 3, NA),
#'   score_science = c(2, 2, NA, 3, 2),
#'   score_lang    = c("2", "2", "2", "3", "2"),
#'   name          = c("Jean", "Marie", "Ali", "Zoe", "Nina")
#' )
#' count_n(df, select = c(score_math, score_science), count = 2)
#' count_n(df, select = starts_with("score_"), exclude = "score_lang", count = 2)
#' count_n(df, select = "^score_", regex = TRUE, count = 2)
#' df |> mutate(nb_two = count_n(count = 2))
#'
#' # Strict type-safe matching with factor columns
#' df <- tibble(
#'   x = factor(c("a", "b", "c")),
#'   y = factor(c("b", "B", "a"))
#' )
#'
#' # Coercion: character "b" matches both x and y
#' count_n(df, count = "b")
#'
#' # Strict match: fails because "b" is character, not factor (returns only 0s)
#' count_n(df, count = "b", allow_coercion = FALSE)
#'
#' # Strict match with factor value: works only where levels match
#' count_n(df, count = factor("b", levels = levels(df$x)), allow_coercion = FALSE)
#'
#' @export
count_n <- function(
  data = NULL,
  select = tidyselect::everything(),
  exclude = NULL,
  count = NULL,
  special = NULL,
  allow_coercion = TRUE,
  ignore_case = FALSE,
  regex = FALSE,
  verbose = FALSE
) {
  if (is.null(data)) {
    data <- dplyr::pick(tidyselect::everything())
  }

  data <- as.data.frame(data)

  col_names <- if (regex) {
    if (missing(select)) {
      select <- ".*"
    }
    if (!is.character(select) || length(select) != 1L || is.na(select)) {
      stop(
        "When `regex = TRUE`, `select` must be a single character pattern.",
        call. = FALSE
      )
    }
    grep(select, names(data), value = TRUE)
  } else {
    sel_quo <- rlang::enquo(select)
    sel_val <- tryCatch(
      rlang::eval_tidy(sel_quo, env = rlang::quo_get_env(sel_quo)),
      error = function(e) NULL
    )
    if (is.character(sel_val)) {
      sel_val
    } else {
      names(tidyselect::eval_select(sel_quo, data))
    }
  }

  if (!is.null(exclude)) {
    col_names <- setdiff(col_names, exclude)
  }

  base_count_n(
    data = data,
    select = col_names,
    count = count,
    special = special,
    allow_coercion = allow_coercion,
    ignore_case = ignore_case,
    verbose = verbose
  )
}

#' @keywords internal
base_count_n <- function(
  data,
  select = names(data),
  count = NULL,
  special = NULL,
  allow_coercion = TRUE,
  ignore_case = FALSE,
  verbose = FALSE
) {
  if (is.null(count) && is.null(special)) {
    stop("You must specify either `count` or `special`.", call. = FALSE)
  }

  if (!is.null(count)) {
    if (length(count) == 1L && is.na(count)) {
      stop(
        "Use `special = \"NA\"` to count missing values, not `count = NA`.",
        call. = FALSE
      )
    }
    has_na <- vapply(count, is.na, logical(1))
    if (any(has_na)) {
      warning(
        "NA values in `count` are ignored. Use `special = \"NA\"` to count missing values.",
        call. = FALSE
      )
      count <- count[!has_na]
    }
  }

  if (!is.null(special) && !is.null(count)) {
    warning(
      "Both `special` and `count` supplied; `count` is ignored.",
      call. = FALSE
    )
  }

  data <- data[, select, drop = FALSE]
  is_list_col <- vapply(data, is.list, logical(1))
  list_cols <- names(data)[is_list_col]
  data <- data[!is_list_col]

  if (verbose && length(list_cols) > 0) {
    message("Ignored list columns: ", paste(list_cols, collapse = ", "))
  }

  if (!is.null(special)) {
    allowed <- c("NA", "NaN", "Inf", "-Inf")
    if ("all" %in% special) {
      special <- allowed
    }
    if (!all(special %in% allowed)) {
      stop(
        "Invalid `special`. Use 'NA', 'NaN', 'Inf', '-Inf', or 'all'.",
        call. = FALSE
      )
    }

    checkers <- list(
      "NA" = is.na,
      "NaN" = function(x) {
        if (is.numeric(x)) is.nan(x) else rep(FALSE, length(x))
      },
      "Inf" = function(x) {
        if (is.numeric(x)) is.infinite(x) & x > 0 else rep(FALSE, length(x))
      },
      "-Inf" = function(x) {
        if (is.numeric(x)) is.infinite(x) & x < 0 else rep(FALSE, length(x))
      }
    )

    logical_list <- lapply(special, function(s) {
      test_fun <- checkers[[s]]
      as.data.frame(lapply(data, test_fun))
    })

    logical_combined <- Reduce(`|`, logical_list)
    result <- rowSums(logical_combined, na.rm = TRUE)
    names(result) <- NULL
    return(result)
  }

  compare_fun <- function(x, values) {
    if (ignore_case) {
      if (is.factor(x)) {
        x <- as.character(x)
      }
      if (is.factor(values)) {
        values <- as.character(values)
      }
      if (is.character(x) && is.character(values)) {
        x <- tolower(x)
        values <- tolower(values)
      }
    }

    if (!allow_coercion) {
      if (identical(class(x), class(values)) && !is.factor(x)) {
        x %in% values
      } else {
        vapply(
          seq_along(x),
          function(i) any(vapply(values, identical, logical(1), x[i])),
          logical(1)
        )
      }
    } else {
      x %in% values
    }
  }

  results <- lapply(data, function(col) {
    tryCatch(
      {
        compare_fun(col, count)
      },
      error = function(e) NULL
    )
  })

  ignored <- names(data)[vapply(results, is.null, logical(1))]
  results <- Filter(Negate(is.null), results)

  if (verbose && length(ignored) > 0) {
    message("Ignored incompatible columns: ", paste(ignored, collapse = ", "))
  }

  if (length(results) == 0) {
    return(rep(0L, nrow(data)))
  }

  result <- rowSums(as.data.frame(results), na.rm = TRUE)
  names(result) <- NULL
  result
}

test_that("count_n basic value counting works", {
  df <- tibble::tibble(
    a = c("a", "b", "b"),
    b = c("b", "c", "b"),
    c = c("c", "b", "b")
  )

  expect_equal(count_n(df, count = "b"), c(1, 2, 3))
  expect_equal(count_n(df, count = c("b", "c")), c(2, 3, 3))
})

test_that("count_n works with ignore_case = TRUE", {
  df <- tibble::tibble(x = c("A", "b", "B"), y = c("a", "B", "b"))
  expect_equal(count_n(df, count = "b", ignore_case = TRUE), c(0, 2, 2))
})

test_that("count_n handles strict comparison with allow_coercion = FALSE", {
  df <- tibble::tibble(x = c(1, 2, 2), y = c("2", 2, 3))
  expect_equal(count_n(df, count = 2, allow_coercion = FALSE), c(0, 1, 1))
})

test_that("count_n handles multiple values with allow_coercion = FALSE", {
  df <- tibble::tibble(x = c(1, 2, 2), y = c("2", 2, 3))

  # Same type (character)
  expect_equal(
    count_n(df, count = c("2", "3"), allow_coercion = FALSE),
    c(1, 1, 1)
  )

  # Same type (numeric)
  expect_equal(count_n(df, count = c(1, 2), allow_coercion = FALSE), c(1, 1, 1))
})

test_that("count_n handles special = NA, NaN, Inf, -Inf", {
  df <- tibble::tibble(
    a = c(NA, 1, NaN, Inf, -Inf),
    b = c(NaN, Inf, NA, NA, -Inf),
    c = 1:5
  )

  expect_equal(count_n(df, special = "NA"), c(2, 0, 2, 1, 0))
  expect_equal(count_n(df, special = "NaN"), c(1, 0, 1, 0, 0))
  expect_equal(count_n(df, special = "Inf"), c(0, 1, 0, 1, 0))
  expect_equal(count_n(df, special = "-Inf"), c(0, 0, 0, 0, 2))
  expect_equal(count_n(df, special = c("NA", "NaN")), c(2, 0, 2, 1, 0))
  expect_equal(count_n(df, special = "all"), c(2, 1, 2, 2, 2))
})

test_that("count_n special works with non-numeric columns", {
  df <- tibble::tibble(
    x = c("a", NA, "b"),
    y = c(1, NaN, 3),
    z = factor(c("x", NA, "y"))
  )

  # NaN should only count in numeric column y, not crash on character/factor
  expect_equal(count_n(df, special = "NaN"), c(0, 1, 0))
  # NA: x=NA, y=NaN (is.na(NaN)==TRUE), z=NA → 3 NAs in row 2
  expect_equal(count_n(df, special = "NA"), c(0, 3, 0))
  expect_equal(count_n(df, special = "all"), c(0, 3, 0))
})

test_that("count_n handles factor variables including ignore_case", {
  df <- tibble::tibble(
    x = factor(c("a", "b", "c")), # levels: a, b, c
    y = factor(c("b", "B", "a")) # levels: a, b, B
  )

  # Default coercion: character "b" matches in both x and y
  expect_equal(count_n(df, count = "b"), c(1, 1, 0))

  # Strict mode: character input fails (character ≠ factor)
  expect_equal(count_n(df, count = "b", allow_coercion = FALSE), c(0, 0, 0))

  # Strict mode: with factor value having matching levels (for x)
  expect_equal(
    count_n(
      df,
      count = factor("b", levels = levels(df$x)),
      allow_coercion = FALSE
    ),
    c(0, 1, 0)
  )

  # Using a value from the data (ensures type + levels)
  expect_equal(count_n(df, count = df$x[2], allow_coercion = FALSE), c(0, 1, 0))

  # Case-insensitive match: works for both x and y after conversion to character
  expect_equal(count_n(df, count = "b", ignore_case = TRUE), c(1, 2, 0))
  expect_equal(count_n(df, count = "B", ignore_case = TRUE), c(1, 2, 0))

  # Case-insensitive + strict: internally uses character comparison, not identical()
  expect_equal(
    count_n(df, count = "b", ignore_case = TRUE, allow_coercion = FALSE),
    c(1, 2, 0)
  )
  expect_equal(
    count_n(
      df,
      count = factor("b", levels = levels(df$x)),
      ignore_case = TRUE,
      allow_coercion = FALSE
    ),
    c(1, 2, 0)
  )
})


test_that("count_n handles labelled variables from haven", {
  skip_if_not_installed("haven")

  df <- tibble::tibble(
    lab = haven::labelled(
      c(1, 2, 1, 2, NA),
      labels = c(No = 1, Yes = 2)
    ),
    other = c(2, 2, 1, 1, 2)
  )

  expect_equal(count_n(df, count = 2), c(1, 2, 0, 1, 1))
  expect_equal(count_n(df, count = 2, allow_coercion = FALSE), c(1, 1, 0, 0, 1))
})

test_that("count_n handles selection and exclusion", {
  df <- tibble::tibble(x1 = "a", x2 = "b", x3 = "b", skip = "b")
  expect_equal(count_n(df, count = "b", select = starts_with("x")), 2)
  expect_equal(
    count_n(df, count = "b", select = everything(), exclude = "skip"),
    2
  )
})

test_that("count_n works with regex selection", {
  df <- tibble::tibble(a_1 = "a", a_2 = "b", b_1 = "b")
  expect_equal(count_n(df, count = "b", select = "^a_", regex = TRUE), 1)
})

test_that("count_n regex mode supports default select and validates pattern", {
  df <- tibble::tibble(a = 1:2, b = c(1, 3))

  expect_equal(count_n(df, count = 1, regex = TRUE), c(2, 0))
  expect_error(
    count_n(df, count = 1, regex = TRUE, select = c("^a", "^b")),
    "single character pattern",
    fixed = TRUE
  )
})

test_that("count_n returns unnamed numeric vector", {
  df <- tibble::tibble(x = c("a", "b", "b"))
  result <- count_n(df, count = "b")
  expect_true(is.numeric(result))
  expect_null(names(result))
})

test_that("count_n works inside mutate()", {
  df <- tibble::tibble(x = c("a", "b", "c"), y = c("b", "b", "b"))
  out <- df |> dplyr::mutate(n_b = count_n(count = "b"))
  expect_equal(out$n_b, c(1, 2, 1))
})

test_that("count_n ignores incompatible columns safely", {
  df <- tibble::tibble(
    x = c("a", "b"),
    y = as.Date(c("2022-01-01", "2022-01-02")),
    z = list(1, 2)
  )
  expect_equal(count_n(df, count = "b"), c(0, 1))
})

test_that("count_n verbose reports list columns", {
  df <- tibble::tibble(x = c("a", "b"), z = list(1, 2))
  expect_message(
    count_n(df, count = "a", verbose = TRUE),
    "Ignored list columns: z"
  )
})

test_that("count_n returns 0 for empty or incompatible selection", {
  df <- tibble::tibble(x = 1:3, y = letters[1:3])
  expect_equal(count_n(df, count = "z", select = "x"), c(0, 0, 0))
  expect_equal(
    count_n(df, count = "z", select = starts_with("not_here")),
    c(0, 0, 0)
  )
})

test_that("count_n throws error if neither count nor special is specified", {
  df <- tibble::tibble(x = 1:3, y = letters[1:3])
  expect_error(
    count_n(df),
    "You must specify either `count` or `special`"
  )
})

test_that("count_n throws error when count = NA", {
  df <- tibble::tibble(x = c(1, NA, 3))
  expect_error(
    count_n(df, count = NA),
    'Use `special = "NA"`',
    fixed = TRUE
  )
})

test_that("count_n throws error for invalid special values", {
  df <- tibble::tibble(x = c(1, 2, 3))
  expect_error(count_n(df, special = "bogus"), "Invalid `special`")
})

test_that("count_n warns and strips NA from count vector", {
  df <- tibble::tibble(x = c(1, 2, NA), y = c(NA, 2, 3))
  expect_warning(
    res <- count_n(df, count = c(NA, 2)),
    "NA values in `count` are ignored"
  )
  expect_equal(res, c(0, 2, 0))
})

test_that("count_n warns when both special and count are supplied", {
  df <- tibble::tibble(x = c(1, NA, 3))
  expect_warning(
    res <- count_n(df, count = 1, special = "NA"),
    "count.*ignored"
  )
  expect_equal(res, c(0, 1, 0))
})

test_that("count_n strict mode uses fast path for same-type atomics", {
  df <- tibble::tibble(x = c(1, 2, 3), y = c(2, 2, 1))
  expect_equal(count_n(df, count = 2, allow_coercion = FALSE), c(1, 2, 0))
  df2 <- tibble::tibble(x = c("a", "b"), y = c("b", "a"))
  expect_equal(count_n(df2, count = "b", allow_coercion = FALSE), c(1, 1))
})

test_that("mean_n works on basic numeric data", {
  df <- tibble::tibble(
    a = c(1, 2, 3),
    b = c(4, NA, 6),
    c = c(NA, NA, 9)
  )

  # Default: all values must be valid
  expect_equal(mean_n(df), c(NA, NA, 6))

  # With min_valid = 1
  expect_equal(mean_n(df, min_valid = 1), c(2.5, 2, 6))

  # With min_valid = 2
  expect_equal(mean_n(df, min_valid = 2), c(2.5, NA, 6))

  # With min_valid as proportion
  expect_equal(mean_n(df, min_valid = 2 / 3), c(2.5, NA, 6))
})

test_that("mean_n handles column selection and exclusion", {
  df <- tibble::tibble(
    var1 = c(10, 20),
    var2 = c(30, 40),
    var3 = c(50, NA),
    other = c("A", "B")
  )

  # Select specific columns
  expect_equal(mean_n(df, select = c(var1, var2)), c(20, 30))

  # Exclude one column
  expect_equal(mean_n(df, exclude = "var3"), c(20, 30))

  # Regex selection
  expect_equal(mean_n(df, select = "^var", regex = TRUE), c(30, NA))
})

test_that("mean_n ignores non-numeric columns", {
  df <- tibble::tibble(
    num1 = c(1, 2),
    char = c("a", "b"),
    num2 = c(3, NA)
  )

  # Should ignore character column silently (with message)
  expect_equal(mean_n(df), c(2, NA))
})

test_that("mean_n returns NA when not enough valid values", {
  df <- tibble::tibble(
    x = c(1, NA, 3),
    y = c(NA, NA, NA),
    z = c(1, 2, 3)
  )

  expect_equal(mean_n(df, min_valid = 2), c(1, NA, 3))
})

test_that("mean_n works on matrices", {
  mat <- matrix(c(1, 2, NA, 4, 5, NA, 7, 8, 9), nrow = 3, byrow = TRUE)
  expect_equal(mean_n(mat, min_valid = 2), c(1.5, 4.5, 8))
})

test_that("mean_n warns when no numeric columns are selected", {
  df <- tibble::tibble(
    a = c(NA, NA),
    b = c(NA, NA)
  )

  expect_warning(
    res <- mean_n(df),
    "No numeric columns selected",
    fixed = TRUE
  )
  expect_true(all(is.na(res)))
})

test_that("mean_n validates min_valid and digits", {
  df <- tibble::tibble(a = c(1, 2), b = c(3, 4))
  expect_error(mean_n(df, min_valid = -1), "non-negative")
  expect_error(mean_n(df, min_valid = "a"), "non-negative")
  expect_error(mean_n(df, digits = -1), "non-negative")
  expect_error(mean_n(df, digits = "a"), "non-negative")
})

test_that("mean_n regex mode supports default select", {
  df <- tibble::tibble(var1 = c(1, 2), var2 = c(3, 4))
  expect_equal(mean_n(df, regex = TRUE), c(2, 3))
})

test_that("mean_n works with character vector select", {
  df <- tibble::tibble(a = c(1, 2), b = c(3, 4), c = c(5, 6))
  items <- c("a", "b")
  expect_equal(mean_n(df, select = items), c(2, 3))
})

test_that("mean_n digits argument rounds result", {
  df <- tibble::tibble(a = c(1, 2), b = c(2, 3), c = c(3, 4))
  res <- mean_n(df, digits = 1)
  expect_equal(res, c(2.0, 3.0))
})

test_that("mean_n verbose prints info messages", {
  df <- tibble::tibble(num = c(1, 2), char = c("a", "b"))
  expect_message(mean_n(df, verbose = TRUE), "Ignored non-numeric")
  expect_message(mean_n(df, verbose = TRUE), "Row means computed")
})

test_that("mean_n regex errors on non-character select", {
  df <- tibble::tibble(a = 1, b = 2)
  expect_error(mean_n(df, select = 1, regex = TRUE), "single character pattern")
})

test_that("mean_n works inside dplyr::mutate", {
  df <- tibble::tibble(x = c(1, 2), y = c(3, 4))
  res <- dplyr::mutate(df, m = mean_n(select = c(x, y)))
  expect_equal(res$m, c(2, 3))
})

test_that("mean_n with exclude drops columns", {
  df <- tibble::tibble(a = c(10, 20), b = c(30, 40), c = c(50, 60))
  expect_equal(mean_n(df, exclude = c("a", "c")), c(30, 40))
})

---
title: "Getting started with spicy"
description: >
  Get started with spicy for descriptive statistics, variable
  inspection, frequency tables, cross-tabulations, association
  measures, categorical and continuous summary tables, and codebooks in
  R. A tidyverse-friendly alternative to SPSS and Stata for survey and
  labelled data workflows.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with spicy}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

build_rich_tables <- identical(Sys.getenv("IN_PKGDOWN"), "true")
```

```{r setup}
library(spicy)
```

spicy is an R package for descriptive statistics and data analysis,
designed for data science and survey research workflows. It covers
variable inspection, frequency tables, cross-tabulations with
chi-squared tests and effect sizes, and publication-ready summary
tables, offering functionality similar to Stata or SPSS but within a
tidyverse-friendly R environment. This vignette walks through the core
workflow using the bundled [`sochealth`](../reference/sochealth.html)
dataset, a simulated social-health survey with 1200 respondents and
24 variables.

## Inspect your data

`varlist()` (or its shortcut `vl()`) gives a compact overview of every
variable in a data frame: name, label, representative values, class,
number of distinct values, valid observations, and missing values.
In RStudio or Positron, calling `varlist()` without arguments opens an
interactive viewer - this is the most common usage in practice. Here we
use `tbl = TRUE` to produce static output for the vignette:

```{r varlist}
varlist(sochealth, tbl = TRUE)
```

You can also select specific columns with tidyselect syntax:

```{r varlist-select}
varlist(sochealth, starts_with("bmi"), income, weight, tbl = TRUE)
```

## Frequency tables

`freq()` produces frequency tables with counts, percentages, and
(optionally) valid and cumulative percentages.

```{r freq}
freq(sochealth, education)
```

Weighted frequencies use the `weights` argument. With `rescale = TRUE`,
the total weighted N matches the unweighted N:

```{r freq-weighted}
freq(sochealth, education, weights = weight, rescale = TRUE)
```

## Cross-tabulations

`cross_tab()` crosses two categorical variables. By default it shows
counts, a chi-squared test, and Cramer's V:

```{r crosstab}
cross_tab(sochealth, smoking, education)
```

Add percentages with `percent`:

```{r crosstab-pct}
cross_tab(sochealth, smoking, education, percent = "col")
```

Group by a third variable with `by`:

```{r crosstab-by}
cross_tab(sochealth, smoking, education, by = sex)
```

When both variables are ordered factors, `cross_tab()` automatically
selects an ordinal measure (Kendall's Tau-b) instead of Cramer's V:

```{r crosstab-ordinal}
cross_tab(sochealth, self_rated_health, education)
```

## Association measures

For a quick overview of all available association statistics, pass a
contingency table to `assoc_measures()`:

```{r assoc-measures}
tbl <- xtabs(~ smoking + education, data = sochealth)
assoc_measures(tbl)
```

Individual functions such as `cramer_v()`, `gamma_gk()`, or
`kendall_tau_b()` return a scalar by default. Pass `detail = TRUE` for
the confidence interval and p-value:

```{r cramer-detail}
cramer_v(tbl, detail = TRUE)
```

## Summary tables

`table_categorical()` covers grouped or one-way summary tables for
categorical variables:

```{r table-categorical-tt, eval = build_rich_tables}
table_categorical(
  sochealth,
  select = c(smoking, physical_activity, dentist_12m),
  by = education,
  output = "tinytable"
)
```

`table_continuous()` summarizes continuous variables, either overall or
by a categorical `by` variable, and can also add group-comparison
tests:

```{r table-continuous}
table_continuous(
  sochealth,
  select = c(bmi, life_sat_health),
  by = education
)
```

`table_continuous_lm()` covers the same reporting territory when you
want to stay in a linear-model framework, for example with robust or
cluster-robust standard errors, case weights, or additive covariate
adjustment:

```{r table-continuous-lm}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  vcov = "HC3"
)
```

For detailed guidance, see the dedicated articles on
`table_categorical()`, `table_continuous()`, `table_continuous_lm()`,
and the final reporting overview for APA-style summary tables.

## Row-wise summaries

`mean_n()`, `sum_n()`, and `count_n()` compute row-wise statistics
across selected columns, with automatic handling of missing values.

```{r mean-n}
sochealth |>
  dplyr::mutate(
    mean_sat  = mean_n(select = starts_with("life_sat")),
    sum_sat   = sum_n(select = starts_with("life_sat"), min_valid = 2),
    n_missing = count_n(select = starts_with("life_sat"), special = "NA")
  ) |>
  dplyr::select(starts_with("life_sat"), mean_sat, sum_sat, n_missing) |>
  head() |>
  as.data.frame()
```

## Learn more

- See `?varlist` to inspect variables, labels, values, and missing
  data.
- See `?freq` for one-way frequency tables (weights, sorting, custom
  missing values, labelled-data display modes).
- See `?cross_tab` for the full list of arguments (weights, simulation,
  association measures).
- See `?assoc_measures` for the complete list of association
  statistics; `?cramer_v` for the canonical entry point.
- See `?table_categorical` for grouped or one-way categorical tables.
- See `?table_continuous` for continuous summaries and group
  comparisons.
- See `?table_continuous_lm` for model-based mean-comparison tables
  with robust / cluster-robust / bootstrap / jackknife SE, case
  weights, or additive covariate adjustment.
- See `?mean_n`, `?sum_n`, `?count_n` for row-wise summaries with
  optional minimum-valid-values rules.
- See `?code_book` to generate an interactive HTML codebook;
  `?label_from_names` to derive variable labels from
  `"code. label"`-style column names (e.g., LimeSurvey exports).

---
title: "Explore variables and build codebooks in R"
description: >
  Explore variables, inspect labels, and build interactive codebooks in R
  with spicy. Learn how to use varlist(), vl(), code_book(), and
  label_from_names() for survey and labelled datasets.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Explore variables and build codebooks in R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

build_rich_tables <- identical(Sys.getenv("IN_PKGDOWN"), "true")
```

```{r setup}
library(spicy)
```

Before you build frequency tables or cross-tabulations, it is often worth
checking how your variables are named, labelled, and coded.

spicy provides a simple workflow for variable exploration and
documentation in R. You can derive labels from imported column names,
inspect variables with `varlist()` or `vl()`, and build an interactive
codebook with `code_book()`.

This vignette focuses on three common tasks:

- clean imported column names and recover variable labels with
  `label_from_names()`
- inspect variables, labels, values, classes, and missing data with
  `varlist()` and `vl()`
- generate an interactive codebook for review or export with
  `code_book()`

These tools are especially useful for survey datasets, labelled data,
and imported files where variable names and labels need to be checked
before analysis.

## Why inspect variables before analysis?

Variable inspection helps catch common problems early: unclear names,
missing labels, unexpected coding, and variables with many missing
values. A quick review of your dataset also makes it easier to choose
which variables to tabulate, summarize, or report later.

## Recover labels from imported column names

Some imported files store both a variable name and a variable label in
the column header. `label_from_names()` splits names of the form
`name<sep>label`, renames the columns, and stores the label as a proper
variable label.

```{r label-from-names}
df <- tibble::tibble(
  "age. Age of respondent" = c(25, 30, 41),
  "edu. Highest education level" = c("Lower", "Upper", "Tertiary"),
  "smoke. Current smoker" = c("No", "Yes", "No")
)

out <- label_from_names(df)
labelled::var_label(out)
```

The default separator `". "` was chosen to match **LimeSurvey CSV
exports** taken with *Export results -> Export format: CSV ->
Headings: Question code & question text*, which produce column names
of the form `"code. question text"`. Pass `sep =` to use any other
literal separator.

## Inspect variables with varlist()

`varlist()` gives a compact summary of each variable, including its
name, label, representative values, class, number of distinct values,
number of valid observations, and missing values.

In RStudio or Positron, the main way to use `varlist()` is
interactively. With its default behavior, it opens a searchable,
sortable variable overview in the Viewer, which makes it easy to scan
labels, look for specific variables, filter what you want to inspect,
and review the structure of a dataset before analysis.

```{r varlist-interactive, eval = FALSE}
varlist(sochealth)
```

If you prefer a shorter call in interactive work, `vl()` is a shortcut
for `varlist()`:

```{r vl-interactive, eval = FALSE}
vl(sochealth)
```

If you want the same summary returned as a tibble, use `tbl = TRUE`:

```{r varlist-all}
varlist(sochealth, tbl = TRUE)
```

If you want the `Values` column to include explicit missing values, use
`include_na = TRUE`:

```{r varlist-include-na}
head(subset(varlist(sochealth, include_na = TRUE, tbl = TRUE), NAs > 0))
```

If you want to display all unique non-missing values in the `Values`
column, use `values = TRUE`. This is especially useful for variables
with a small number of distinct values:

```{r varlist-values}
head(subset(varlist(sochealth, values = TRUE, tbl = TRUE), N_distinct <= 5))
```

For a focused inspection, select only the variables you want to review:

```{r varlist-selected}
varlist(sochealth, smoking, education, income_group, tbl = TRUE)
```

This is often enough to confirm that labels, factor levels, and missing
values look correct before moving on to tabulations.

## Select subsets of variables

`varlist()` supports tidyselect, which makes it easy to inspect a subset
of variables by name pattern or type.

```{r varlist-tidyselect}
varlist(sochealth, starts_with("life_sat"), tbl = TRUE)
```

```{r varlist-numeric}
varlist(sochealth, where(is.numeric), tbl = TRUE)
```

`vl()` also works with tidyselect in the same way:

```{r vl-example}
vl(sochealth, starts_with("bmi"), tbl = TRUE)
```

## Build an interactive codebook

When you want a searchable and exportable overview of the whole dataset or a
selected set of variables, `code_book()` builds an interactive codebook in the
Viewer.

```{r code-book-basic, eval = build_rich_tables}
if (requireNamespace("DT", quietly = TRUE)) {
  code_book(sochealth)
}
```

Use the same tidyselect-style selectors as `varlist()` to build a focused
codebook:

```{r code-book-select, eval = build_rich_tables}
if (requireNamespace("DT", quietly = TRUE)) {
  code_book(
    sochealth,
    starts_with("bmi"),
    values = TRUE,
    title = "BMI codebook",
    filename = "bmi_codebook"
  )
}
```

You can also request a fuller display of values or include missing
values explicitly in the summary:

```{r code-book-values, eval = build_rich_tables}
if (requireNamespace("DT", quietly = TRUE)) {
  code_book(sochealth, values = TRUE, include_na = TRUE)
}
```

This is useful when reviewing a dataset with collaborators or preparing
documentation before analysis.

## When to use varlist() and code_book()

Use `varlist()` when you want a quick summary in a script or a tibble
you can inspect directly.

Use `vl()` when you want the same summary with a shorter call in
interactive work.

Use `code_book()` when you want a searchable, interactive codebook
for review or export.

The two tools share their column structure but differ in one default:
`varlist()` shows only the **observed** factor levels in the
`Values` column (Stata `tab` style), whereas `code_book()` shows
**all declared** levels including unused ones (SPSS `FREQUENCIES`
style, appropriate for schema documentation). Pass `factor_levels =`
explicitly to either function to override the default.

---
title: "Continuous summary tables in R"
description: >
  Build continuous summary tables in R with table_continuous(),
  including grouped descriptives, group-comparison tests, effect sizes,
  and export to console, gt, tinytable, flextable, Excel, or Word.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Continuous summary tables in R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

build_rich_tables <- identical(Sys.getenv("IN_PKGDOWN"), "true")

source("_pkgdown-helpers.R")
```

```{r setup}
library(spicy)
```

`table_continuous()` summarizes continuous variables either overall or
by a categorical grouping variable. It is designed for readable summary
tables in the console and for publication-ready outputs in rendered
documents. When `by` is supplied, it can also add group-comparison
tests, test statistics, and effect sizes.

## Basic usage

Use `select` to choose the continuous variables you want to summarize:

```{r basic}
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health)
)
```

If you omit `select`, `table_continuous()` scans the data frame and
keeps numeric columns:

```{r default-select}
table_continuous(sochealth)
```

## Grouped summaries

Add `by` to summarize the same variables across categories. When
`by` is supplied, a Welch-test *p*-value column is added automatically:

```{r grouped}
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education
)
```

This is the main pattern for reporting continuous variables across
groups such as education, sex, treatment arm, or survey wave. Pass
`p_value = FALSE` to suppress the test column and keep the output
strictly descriptive.

If you want the same outcomes reported in a linear-model workflow,
with heteroskedasticity-consistent or cluster-robust standard errors,
case weights, or additive covariate adjustment, use
`table_continuous_lm()` instead:

```{r lm-companion}
table_continuous_lm(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education,
  vcov = "HC3"
)
```

## Add test statistics and effect sizes

Grouped tables can also report the test statistic and an effect size
alongside the default *p*-value column:

```{r pvalue-effect}
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education,
  statistic = TRUE,
  effect_size = "auto",
  effect_size_ci = TRUE
)
```

Use `test = "student"` for equal-variance parametric tests or
`test = "nonparametric"` for rank-based comparisons:

```{r nonparametric}
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score),
  by = education,
  test = "nonparametric",
  statistic = TRUE,
  effect_size = TRUE
)
```

`effect_size = TRUE` auto-selects the canonical measure for the
chosen `test` and number of groups: Hedges' *g* (parametric, 2
groups), eta-squared (parametric, 3+ groups), rank-biserial *r*
(nonparametric, 2 groups), or epsilon-squared (nonparametric, 3+
groups). To pick a specific measure explicitly, pass its character
name:

```{r effect-size-explicit}
table_continuous(
  sochealth,
  select = wellbeing_score,
  by = sex,
  effect_size = "hedges_g",
  effect_size_ci = TRUE
)
```

Allowed values are `"none"` (default), `"auto"` (= legacy `TRUE`),
`"hedges_g"`, `"eta_sq"`, `"r_rb"`, and `"epsilon_sq"`. Incompatible
explicit choices (e.g. `"eta_sq"` with two groups, or `"hedges_g"`
with `test = "nonparametric"`) trigger an actionable error.

When you need the underlying columns for further processing, use
`output = "data.frame"`:

```{r raw-output}
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score),
  by = education,
  statistic = TRUE,
  effect_size = TRUE,
  output = "data.frame"
)
```

## Selecting variables

`select` supports tidyselect helpers:

```{r tidyselect}
table_continuous(
  sochealth,
  select = starts_with("life_sat"),
  by = sex
)
```

For more programmatic selection, set `regex = TRUE`:

```{r regex}
table_continuous(
  sochealth,
  select = "^life_sat",
  regex = TRUE,
  by = education,
  output = "data.frame"
)
```

Use `exclude` when you want a broad selection with one or two explicit
removals:

```{r exclude}
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health, life_sat_work),
  exclude = "life_sat_work",
  by = sex
)
```

## Labels and output formats

Use `labels` to replace technical variable names with reporting labels:

```{r labels, eval = build_rich_tables}
pkgdown_dark_gt(
  table_continuous(
    sochealth,
    select = c(bmi, wellbeing_score, life_sat_health),
    by = education,
    labels = c(
      bmi = "Body mass index",
      wellbeing_score = "Well-being score",
      life_sat_health = "Satisfaction with health"
    ),
    output = "gt"
  )
)
```

`table_continuous()` supports the same reporting-oriented outputs as
`table_categorical()`:

| `output` value | Returned object |
|:--|:--|
| `"default"` | Styled ASCII console table |
| `"data.frame"` / `"long"` | Plain `data.frame` with the underlying long-format rows (synonyms; pick whichever reads better in your code) |
| `"tinytable"` | Formatted tinytable |
| `"gt"` | Formatted gt table |
| `"flextable"` | Formatted flextable |
| `"excel"` | Written `.xlsx` file |
| `"clipboard"` | Copied text table |
| `"word"` | Written `.docx` file |

For example, `tinytable` works well in Quarto and R Markdown documents:

```{r tinytable, eval = build_rich_tables}
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education,
  output = "tinytable"
)
```

## Export to Excel or Word

Use the same function to export the table directly:

```{r export, eval = FALSE}
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education,
  output = "excel",
  excel_path = "table_continuous.xlsx"
)

table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education,
  output = "word",
  word_path = "table_continuous.docx"
)
```

## Display options

The printed ASCII table and every rendered output share the same
formatting vocabulary as `table_continuous_lm()` and
`table_categorical()`:

- `align = "decimal"` (default) aligns numeric columns on the
  decimal mark, matching SPSS / SAS / LaTeX `siunitx` conventions.
  `"center"`, `"right"`, and the legacy `"auto"` are the
  alternatives.
- `p_digits = 3` (default, the APA standard) drives both the
  displayed precision of the `p` column and the small-*p* threshold
  (`p_digits = 4` -> `.0451` and `<.0001`).
- `show_n = FALSE` and `ci = FALSE` drop the corresponding columns
  (and the CI spanner / borders) structurally from every output;
  the underlying `n` and `ci_lower` / `ci_upper` are always present
  in `output = "data.frame"` / `"long"` for downstream code.

```{r display-opts}
table_continuous(
  sochealth,
  select = wellbeing_score,
  by = sex,
  ci = FALSE,
  show_n = FALSE,
  p_digits = 4
)
```

## Tidying for downstream pipelines

`table_continuous()` returns an object that can be coerced to a plain
`data.frame` / `tbl_df` (stripping the spicy formatting attributes)
or piped into `broom::tidy()` / `broom::glance()` for any downstream
tidyverse stats workflow:

```{r tidy-glance}
out <- table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score),
  by = sex
)

# Long descriptive rows: one per (variable x group) with broom-style
# columns (outcome, label, group, estimate = mean, std.error,
# conf.low / conf.high, n, min, max, sd).
broom::tidy(out)

# One row per outcome with the omnibus test + effect-size summary
# (test_type, statistic, df, df.residual, p.value, es_type, es_value,
# es_ci_lower / es_ci_upper, n_total).
broom::glance(out)

# Or just unbox to a plain data.frame (long-format underlying data)
head(as.data.frame(out))
```

## See also

- See `vignette("table-categorical", package = "spicy")` for grouped
  tables of categorical variables.
- See `vignette("table-continuous-lm", package = "spicy")` for
  model-based continuous summary tables with robust standard errors or
  case weights.
- See `vignette("summary-tables-reporting", package = "spicy")` for a
  cross-function reporting workflow using both summary-table helpers.

---
title: "Categorical summary tables in R"
description: >
  Build categorical summary tables in R with table_categorical(),
  including grouped cross-tabulations, effect sizes, confidence
  intervals, and export to gt, tinytable, flextable, Excel, or Word.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Categorical summary tables in R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

build_rich_tables <- identical(Sys.getenv("IN_PKGDOWN"), "true")

source("_pkgdown-helpers.R")
```

```{r setup}
library(spicy)
```

`table_categorical()` builds publication-ready categorical tables suitable for
APA-style reporting in social science and data science research. With
`by`, it produces grouped cross-tabulation tables with chi-squared
\(p\)-values, effect sizes, confidence intervals, and multi-level
headers. Without `by`, it produces one-way frequency-style tables for
the selected variables. Export to gt, tinytable, flextable, Excel, or
Word. This vignette walks through the main features.

## Basic usage

For grouped tables, provide a data frame, one or more selected
variables, and a grouping variable:

```{r basic}
table_categorical(
  sochealth,
  select = c(smoking, physical_activity, dentist_12m),
  by = education
)
```

The default output is `"default"`, which prints a styled ASCII table to
the console. Use `output = "data.frame"` to get a plain numeric
data frame suitable for further processing.

## One-way tables

Omit `by` to build a frequency-style table for the selected variables:

```{r oneway}
table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  output = "default"
)
```

## Output formats

`table_categorical()` supports several output formats. The table below
summarizes the options:

| Format | Description |
|---|---|
| `"default"` | Styled ASCII table in the console (default) |
| `"data.frame"` | Wide data frame, one row per modality |
| `"long"` | Long data frame, one row per modality x group |
| `"gt"` | Formatted gt table |
| `"tinytable"` | Formatted tinytable |
| `"flextable"` | Formatted flextable |
| `"excel"` | Excel file (requires `excel_path`) |
| `"clipboard"` | Copy to clipboard |
| `"word"` | Word document (requires `word_path`) |

### gt output

The `"gt"` format produces a table with APA-style borders, column
spanners, and proper alignment:

```{r gt, eval = build_rich_tables}
pkgdown_dark_gt(
  table_categorical(
    sochealth,
    select = c(smoking, physical_activity, dentist_12m),
    by = education,
    output = "gt"
  )
)
```

### tinytable output

```{r tinytable, eval = build_rich_tables}
table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  by = sex,
  output = "tinytable"
)
```

### Data frame output

Use `output = "data.frame"` for a wide numeric data frame (one row per
modality), or `output = "long"` for a long format (one row per
modality x group):

```{r data-frame}
table_categorical(
  sochealth,
  select = smoking,
  by = education,
  output = "data.frame"
)
```

## Custom labels

By default, `table_categorical()` uses variable names as row headers.
Use the `labels` argument to provide human-readable labels. Two
forms are accepted (matching `table_continuous()` and
`table_continuous_lm()`):

- A **named character vector** keyed by column name in `data` --
  the recommended form. Only listed columns are relabelled; others
  fall back to the column name.
- A **positional character vector** of the same length as `select`
  -- the legacy spicy < 0.11.0 form, kept for backward
  compatibility.

```{r labels, eval = build_rich_tables}
pkgdown_dark_gt(
  table_categorical(
    sochealth,
    select = c(smoking, physical_activity),
    by = education,
    labels = c(
      smoking           = "Smoking status",
      physical_activity = "Regular physical activity"
    ),
    output = "gt"
  )
)
```

## Association measures and confidence intervals

`table_categorical()` picks the association measure per row variable
based on the variable type (`assoc_measure = "auto"`, the default):

* **2x2** (binary row variable vs. binary `by`) -> `phi`,
* both ordered factors -> Kendall's `tau_b`,
* otherwise -> Cramer's `V`.

When the chosen measures differ across rows, the column header
collapses to `"Effect size"` and an APA-style `Note.` line documents
which measure was used for each variable.

Override with a single string for uniform application, or with a
named vector to mix measures per row:

```{r assoc-measure, eval = build_rich_tables}
# Uniform: same measure for every row variable
table_categorical(
  sochealth,
  select = smoking,
  by = education,
  assoc_measure = "lambda",
  output = "tinytable"
)
```

```{r assoc-measure-named, eval = build_rich_tables}
# Per-row: pick the right measure for each variable.
# `smoking` x `education` is 2x3 (binary x ordered) -> Cramer's V;
# `self_rated_health` x `education` is ordered x ordered -> Tau-b.
# The mixed result collapses the header to "Effect size" and adds an
# APA `Note.` line documenting the per-row measure.
table_categorical(
  sochealth,
  select = c(smoking, self_rated_health),
  by = education,
  assoc_measure = c(
    smoking           = "cramer_v",
    self_rated_health = "tau_b"
  ),
  output = "tinytable"
)
```

Add confidence intervals with `assoc_ci = TRUE`. In rendered formats
(`gt`, `tinytable`, `flextable`, `word`), the CI is shown inline:

```{r ci-rendered, eval = build_rich_tables}
pkgdown_dark_gt(
  table_categorical(
    sochealth,
    select = c(smoking, physical_activity),
    by = education,
    assoc_ci = TRUE,
    output = "gt"
  )
)
```

In data formats (`"data.frame"`, `"long"`, `"excel"`, `"clipboard"`),
separate `CI lower` and `CI upper` columns are added:

```{r ci-data}
table_categorical(
  sochealth,
  select = smoking,
  by = education,
  assoc_ci = TRUE,
  output = "data.frame"
)
```

## Weighted tables

Pass survey weights with the `weights` argument. Use `rescale = TRUE` so
the total weighted N matches the unweighted N:

```{r weighted, eval = build_rich_tables}
pkgdown_dark_gt(
  table_categorical(
    sochealth,
    select = c(smoking, physical_activity),
    by = education,
    weights = "weight",
    rescale = TRUE,
    output = "gt"
  )
)
```

## Handling missing values

By default, rows with missing values are dropped (`drop_na = TRUE`).
Set `drop_na = FALSE` to display them as a "(Missing)" category:

```{r missing, eval = build_rich_tables}
pkgdown_dark_gt(
  table_categorical(
    sochealth,
    select = income_group,
    by = education,
    drop_na = FALSE,
    output = "gt"
  )
)
```

## Filtering and reordering levels

Use `levels_keep` to display only specific modalities. The order you
specify controls the display order, which is useful for placing
"(Missing)" first to highlight missingness:

```{r levels-keep, eval = build_rich_tables}
pkgdown_dark_gt(
  table_categorical(
    sochealth,
    select = income_group,
    by = education,
    drop_na = FALSE,
    levels_keep = c("(Missing)", "Low", "High"),
    output = "gt"
  )
)
```

## Formatting options

Control the number of digits for percentages, p-values, and the
association measure:

```{r formatting, eval = build_rich_tables}
pkgdown_dark_gt(
  table_categorical(
    sochealth,
    select = smoking,
    by = education,
    percent_digits = 2,
    p_digits = 4,
    v_digits = 3,
    output = "gt"
  )
)
```

`p_digits` drives both the displayed precision of the `p` column and
the small-*p* threshold (`p_digits = 3` -> `<.001`, `p_digits = 4`
-> `<.0001`), matching `table_continuous()` and
`table_continuous_lm()`.

## Decimal alignment

By default (`align = "decimal"`) numeric columns are aligned on the
decimal mark, the standard scientific-publication convention used by
SPSS, SAS, LaTeX `siunitx`, and the native primitives of
`gt::cols_align_decimal()` and `tinytable::style_tt(align = "d")`.
Engines without a native primitive (`flextable`, `word`,
`clipboard`, ASCII print) get the alignment via leading / trailing
space padding, with `flextable` / `word` switching the body font to
`Consolas` so character widths match.

Pass `align = "auto"` to revert to the legacy uniform
right-alignment used in spicy < 0.11.0:

```{r align}
table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  by = sex,
  align = "auto"
)
```

`"center"` and `"right"` apply literal alignment.

## Tidying for downstream pipelines

`table_categorical()` returns an object that can be coerced to a
plain `data.frame` / `tbl_df` (stripping the spicy formatting
attributes) or piped into `broom::tidy()` / `broom::glance()` for
any downstream tidyverse-stats workflow:

```{r tidy-glance}
out <- table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  by = sex
)

# One row per (variable x level x group) with broom-style columns
# (outcome, level, group, n, proportion). The synthetic Total
# margin is excluded so each observation is counted once.
broom::tidy(out)

# One row per outcome with the omnibus chi-squared test and the
# chosen association measure (test_type, statistic, df, p.value,
# assoc_type, assoc_value, assoc_ci_lower / assoc_ci_upper, n_total).
broom::glance(out)
```

## Exporting to Excel, Word, or clipboard

For Excel export, provide a file path:

```r
table_categorical(
  sochealth,
  select = c(smoking, physical_activity, dentist_12m),
  by = education,
  output = "excel",
  excel_path = "my_table.xlsx"
)
```

For Word, use `output = "word"`:

```r
table_categorical(
  sochealth,
  select = c(smoking, physical_activity, dentist_12m),
  by = education,
  output = "word",
  word_path = "my_table.docx"
)
```

You can also copy directly to the clipboard for pasting into a
spreadsheet or a text editor:

```r
table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  by = education,
  output = "clipboard"
)
```

---
title: "Publication-ready regression tables"
description: >
  Coefficient tables for `lm` and `glm` fits with APA-aligned
  formatting: classical, heteroskedasticity-consistent and
  cluster-robust variance, four standardisation methods, partial
  effect sizes with noncentral-F CIs, average marginal effects,
  multiple-comparison adjustment, side-by-side and hierarchical
  layouts, and output to console, gt, tinytable, flextable,
  Excel, Word, or clipboard.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Publication-ready regression tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

build_rich_tables <- identical(Sys.getenv("IN_PKGDOWN"), "true")

source("_pkgdown-helpers.R")
```

```{r setup}
library(spicy)
```

`table_regression()` produces a coefficient summary table from
one or several fitted `lm()` or `glm()` models. The output is
publication-ready by default and follows APA Manual 7 (American
Psychological Association 2020, Tables 7.13–7.15) formatting
conventions: paired estimate-and-CI columns, APA p-values without
leading zero, factor levels grouped under their parent variable,
fit statistics at the foot of the table, and a self-documenting
note line that names the variance estimator and any
methodological choice that affected the rendered values.

The function is **fit-first**: you pass already-fitted models,
not raw data and a formula. The same object exports cleanly to a
long broom-canonical frame (`broom::tidy()`) and to a
one-row-per-model glance summary (`broom::glance()`).

`lm` and `glm` are both supported. The *Generalised linear
models* section covers the glm-specific argument semantics;
everything else applies to both. Mixed-effects models are on the
roadmap.

## Basic usage

Pass a fitted `lm()` object. The default rendering returns a
single-model table with `B`, `SE`, `95% CI`, and `p` columns and
a fit-statistics footer (`n`, `R²`, `Adj.R²`):

```{r basic}
fit <- lm(wellbeing_score ~ age + sex + smoking, data = sochealth)
table_regression(fit)
```

Reading the table:

* Each predictor occupies one row; factor predictors are grouped
  under their parent variable name with one row per level. The
  reference level carries the `(ref.)` annotation and an em-dash
  across the statistic columns, so the substantive comparison is
  visible in a single glance (NEJM / BMJ clinical convention).
* The `95% CI` column reports a symmetric Wald interval at the
  level set by `ci_level` (default 0.95). The CI header tracks
  the chosen level — switching to `ci_level = 0.99` re-labels the
  column accordingly.
* The footer line names the variance estimator in plain English
  so the reader can find the inferential regime without leaving
  the table. The estimator switches with `vcov` (covered in
  *Robust variance* below).

## Standardised coefficients

Standardised coefficients (`β`) make predictors with different
natural scales comparable: a one-standard-deviation increase in
`X` predicts a `β`-standard-deviation change in `Y`. APA Manual 7
§7.13 recommends reporting both `B` and `β` so the unstandardised
effect (natural units, interpretable) stays alongside the
standardised effect (comparable across predictors).

`standardized` selects the method. Four are available; the choice
is consequential and well-documented (Cohen, Cohen, West, and
Aiken 2003 §3.4; Gelman 2008):

* `"refit"` — refit the model on z-scored outcome and predictors.
  Gold-standard convention, used by SPSS `REGRESSION` and Stata
  `regress, beta`. Both numeric and dummy-coded predictors enter
  the refit on the same scale.
* `"posthoc"` — algebraic rescaling `β = B × SD(X) / SD(Y)`,
  applied to the original fit. Numerically identical to `"refit"`
  for purely linear-additive Gaussian models; preferred when
  refitting is expensive or when `lm()` was wrapped in a pipeline
  that resists re-execution.
* `"basic"` — algebraic, but factor dummies keep their 0/1 scale
  rather than being z-scored. Useful when factor levels carry
  meaningful base rates that scale-free standardisation would
  obscure.
* `"smart"` — Gelman's (2008) recommendation: numeric predictors
  divided by `2 × SD(X)`; binary predictors centred only. The
  resulting `β` is the predicted change in `Y` for one within-
  sample standard deviation of `X`, on the original `Y` scale,
  which keeps factor and numeric coefficients on roughly
  comparable footing.

When `standardized != "none"`, the `"beta"` token is auto-injected
into `show_columns` immediately after `"b"`:

```{r beta}
fit <- lm(wellbeing_score ~ age + sex + smoking, data = sochealth)
table_regression(fit, standardized = "refit")
```

**Caveat: standardised coefficients with interactions or
transformed terms.** When the model contains a product term, an
`I()`, `poly()`, `log()`, or `splines::ns()` wrapper, the
standardised coefficient of the non-additive term has no
closed-form "one-SD change in X" reading (Aiken and West 1991;
Cohen et al. 2003 §7.7). The function emits a classed
`spicy_caveat` warning at runtime AND prints a method-specific
caveat line in the table footer, so the limitation is exposed at
the point of use without blocking the table.

## Per-coefficient effect sizes

For each predictor, you can request a partial effect-size column.
Three measures are available — Cohen's `f²` (`partial_f2`),
Pearson's partial `η²` (`partial_eta2`), and the Olejnik–Algina
bias-corrected partial `ω²` (`partial_omega2`). Each carries a
confidence interval derived from noncentral-`F` inversion
(Smithson 2003; Steiger 2004), exposed as a separate `<token>_ci`
column. The shortcuts `"all_f2"`, `"all_eta2"`, `"all_omega2"`
expand to the point estimate and its CI pair in one go:

```{r partial}
fit <- lm(wellbeing_score ~ age + sex + smoking + education,
          data = sochealth)
table_regression(
  fit,
  show_columns = c("b", "p", "all_eta2", "all_omega2")
)
```

Methodology notes:

* The partial F-test is computed on a Type-II ANOVA reference
  (`car::Anova`), which respects the principle of marginality and
  is the SAS / SPSS default for unbalanced designs.
* The `ω²` point estimate is bias-corrected via the Olejnik and
  Algina (2003) formula `((F − 1) × df1) / (F × df1 + N − df1)`,
  yielding a less-biased small-sample estimator than partial `η²`.
* The CI bounds come from inverting the noncentrality parameter
  of the F-distribution at the lower and upper confidence levels
  (Steiger 2004 §4). The Steiger inversion always brackets the
  bias-corrected point estimate, even when the lower bound clips
  at zero (common for near-null terms).
* For factor predictors with `k` levels, the partial F-test is
  the joint `(k − 1)` df Wald test, so the same effect-size value
  is broadcast across all non-reference dummy rows; the reference
  row shows an em-dash.

## Average marginal effects (AME)

The `B` coefficient is the model's *structural* effect — the
change in the linear predictor `Xβ` for a one-unit change in
`x`, holding the other predictors constant. The **average
marginal effect (AME)** is the *observable* effect on the
response: the average partial derivative `dE[Y|X] / dx` over the
sample. For a purely linear-additive `lm()` the two coincide. For
models with interactions or transformed terms, and for any
`glm()` with a non-identity link, the AME is the quantity the
substantive reader actually needs.

Add `"ame"` to `show_columns` for the point estimate. Companion
tokens display the SE, CI, and p-value: `"ame_se"`, `"ame_ci"`,
`"ame_p"`. The shortcut `"all_ame"` expands to the full set.

```{r ame-lm}
fit <- lm(wellbeing_score ~ age + sex + smoking, data = sochealth)
table_regression(
  fit,
  show_columns = c("b", "ame", "ame_ci", "ame_p")
)
```

Reading conventions:

* `"p"` always refers to the `B` (or `β`) coefficient, never to
  the AME. Use `"ame_p"` for the AME-specific p-value.
* Placing `"ame"` after `"p"` keeps the "which p belongs to what"
  reading unambiguous — the B-block (`B / SE / p`) is closed
  before the AME-block opens.
* For non-linear models the AME is reported on the **response
  scale**, not the link scale. The *Generalised linear models*
  section below works out a logistic-regression example.

Inference is delegated to
[`marginaleffects::avg_slopes()`][marginaleffects::avg_slopes] and
respects the chosen variance estimator. Under cluster-robust
variance (covered in *Robust variance*), the AME inference shares
the coefficient's t-distribution and Satterthwaite-corrected
degrees of freedom via `clubSandwich::linear_contrast()`, so `B`
and AME are reported on the same inferential footing in the same
table.

## Multiple models side by side

Pass a list of `lm()` fits. The default column layout places each
model in its own panel under a centered **spanner label** showing
the model name; sub-columns (`B / SE / p`) are repeated under the
spanner. When dependent variables differ across models and the
user did not supply labels (`model_labels = ` or named list), the
bare DV name is lifted into the spanner automatically:

```{r multi}
m_wellbeing <- lm(wellbeing_score ~ age + sex + smoking,
                   data = sochealth)
m_bmi       <- lm(bmi             ~ age + sex + smoking,
                   data = sochealth)
table_regression(list(m_wellbeing, m_bmi))
```

When all models share the same DV, the DV appears in the title.
Use a named list — e.g. `list(Crude = m1, Adjusted = m2)` — to
set the spanner labels explicitly; pass `model_labels = c(...)`
to override the names from the list.

## Hierarchical / nested regression

Set `nested = TRUE` to add **in-table change-statistic rows**
(APA Table 7.13 / Stata `esttab` / SPSS Model Summary
convention). Each adjacent pair (M2 vs M1, M3 vs M2, …)
contributes one column of change stats below `R² / Adj.R²`; the
FIRST model column gets em-dashes (no previous model to compare
to).

Hierarchical regression requires every model in the sequence to
share the same analytic sample: identical `nobs` AND identical
response variable. R's listwise deletion otherwise produces
different `n` per model as soon as a predictor introduced by M2
or M3 has missing values that M1 did not see — a silent bias on
the change statistics. We restrict to complete cases on the
union of predictors up front:

```{r sochealth-cc}
sochealth_cc <- sochealth |>
  dplyr::select(wellbeing_score, age, sex, smoking, bmi, region) |>
  na.omit()
```

```{r nested}
m1 <- lm(wellbeing_score ~ age + sex,                 data = sochealth_cc)
m2 <- lm(wellbeing_score ~ age + sex + smoking,       data = sochealth_cc)
m3 <- lm(wellbeing_score ~ age + sex + smoking + bmi, data = sochealth_cc)
table_regression(list(m1, m2, m3), nested = TRUE)
```

Default change tokens auto-injected: `c("r2_change", "f_change",
"p_change")` for `lm` (APA hierarchical-regression standard),
`c("lrt_change", "p_change")` for `glm` (Hosmer & Lemeshow §3.5).
Customise via `show_fit_stats`; the order of tokens controls the
order of the rows. Other change tokens are available:
`"adj_r2_change"`, `"f2_change"`, `"deviance_change"`,
`"aic_change"` / `"aicc_change"` / `"bic_change"`.

Validation is strict: identical `nobs` AND identical response
across all models, otherwise a `spicy_invalid_input` error
explains the listwise-deletion trap and suggests refitting on the
common subset.

## Robust variance

The default `vcov = "classical"` reports the OLS standard error,
valid under homoskedastic, independent errors. Two robust
alternatives are first-class arguments; bootstrap and jackknife
variants are also available via `vcov = "bootstrap"` / `"jackknife"`.

### Heteroskedasticity-consistent (HC)

When error variance plausibly depends on the predictors (a
ubiquitous concern in cross-sectional social-science data), set
`vcov = "HC*"` for sandwich-style standard errors via
[`sandwich::vcovHC()`][sandwich::vcovHC]. The valid types are
`HC0` through `HC5`; `HC3` is the small-sample-friendly default
(Long and Ervin 2000):

```{r hc}
fit <- lm(wellbeing_score ~ age + sex + smoking, data = sochealth)
table_regression(fit, vcov = "HC3")
```

The footer reads "*Std. errors: heteroskedasticity-robust (HC3)*";
the column header for the confidence interval automatically tracks
`ci_level`.

### Cluster-robust (CR)

For clustered observations (repeated measures on the same person,
students within schools, observations within regions),
`vcov = "CR*"` requests cluster-robust variance via
[`clubSandwich::vcovCR()`][clubSandwich::vcovCR], with the cluster
identifier supplied through `cluster`. Three forms are accepted:

* **Formula** — `cluster = ~region`. The variables are looked up
  in `model.frame(fit)` first, then in the model's original
  `data` argument. Recommended: independent of the dataset's
  name, composable for multi-way clustering
  (`cluster = ~region:year`).
* **String** — `cluster = "region"`. Single column name resolved
  the same way. Convenient but cannot express interactions.
* **Vector** — `cluster = df$region`. Atomic vector of length
  `nobs(fit)`. Use this when the cluster key is derived on the
  fly (`cluster = interaction(df$region, df$year)`) or pulled
  from a different dataset with matching row order.

Bare unquoted names (`cluster = region`) are **not** accepted —
they would require non-standard evaluation that breaks under
programmatic use (function wrapping, loops, dynamic column
choice).

```{r cr}
fit <- lm(wellbeing_score ~ age + sex + smoking, data = sochealth_cc)
table_regression(
  fit,
  vcov = "CR2",
  cluster = ~region,
  show_columns = c("b", "se", "ci", "p", "ame", "ame_p")
)
```

`CR2` (the Bell-McCaffrey adjustment) is the recommended default
under few clusters (Pustejovsky and Tipton 2018; Imbens and
Kolesár 2016). Coefficient inference uses
[`clubSandwich::coef_test()`][clubSandwich::coef_test] with
Satterthwaite-corrected degrees of freedom. When AME columns are
requested, the same Satterthwaite framework is applied to the
AME contrast via `clubSandwich::linear_contrast()`, so `B` and
AME share the same t-distribution with the same df — visible in
the footer.

## Multiple-comparison adjustment

`p_adjust` applies a family-wise or false-discovery-rate
adjustment to the p-values via
[`stats::p.adjust()`][stats::p.adjust]. The family is the model's
full coefficient set (intercept and reference rows excluded);
`B` and `AME` are adjusted as independent families within the
same call. Available methods are the same as in
`stats::p.adjust()`: `"none"` (default), `"holm"`, `"hochberg"`,
`"hommel"`, `"bonferroni"`, `"BH"` / `"fdr"`, `"BY"`.

```{r p-adjust}
fit <- lm(wellbeing_score ~ age + sex + smoking + education,
          data = sochealth)
table_regression(fit, p_adjust = "bonferroni")
```

The footer documents the chosen method and the family size; the
SE column is unchanged. `p_adjust` is orthogonal to `vcov`:
combining a robust SE with a family-wise correction is fully
supported, e.g. `vcov = "HC3", p_adjust = "holm"`. The adjustment
is applied **before** `keep` / `drop` filtering, so the family
stays the model's full coefficient set and the displayed adjusted
p-values reflect the right denominator regardless of which
subset is shown.

A methodological note. Adjusting the p-values of every
coefficient of a single regression model is *not* the standard
convention in social-science or clinical reporting (Rothman 1990;
Gelman, Hill, and Yajima 2012; Greenland 2017; Harrell 2015 §5.4;
APA Manual 7 §6.46). Each coefficient tests a scientifically
distinct hypothesis on a distinct predictor, which is not the
situation that family-wise procedures were designed for. The
default `"none"` reflects this consensus.

Adjustment is nonetheless legitimate in three contexts:

1. **Mass screening** with many candidate predictors and no
   prior hypothesis — typically `"BH"` (Benjamini–Hochberg,
   false discovery rate).
2. **Pre-registered multi-endpoint confirmatory designs** —
   typically `"holm"` (Holm's step-down, strong family-wise
   error rate control).
3. **When a reviewer, an editor, or a statistical analysis plan
   explicitly requests it** — apply the requested method and
   document it in the footer.

The argument is exposed under the same *transparency* rule used
for `standardized`: the tool is here, the methodological choice
is yours, and the footer makes the choice visible to the reader.

## Filtering displayed coefficients

`keep` and `drop` accept regular expressions matched against
coefficient names (as returned by [stats::coef()]). They are
mutually exclusive — pick `keep` to whitelist focal predictors,
or `drop` to hide a few control variables. Multiple patterns
combine with logical OR.

```{r keep}
fit <- lm(wellbeing_score ~ age + sex + smoking + bmi + education,
          data = sochealth)
table_regression(fit, keep = c("^smoking", "^bmi$"))
```

```{r drop}
table_regression(fit, drop = "^education")
```

Together with `p_adjust`, this is the article-ready workflow:
adjust on the full model, display only the rows the reader cares
about — `table_regression(fit, p_adjust = "BH", keep = "^treatment")`.

## Generalised linear models (glm)

`table_regression()` accepts any `glm()` fit. Inference defaults
to the `z`-asymptotic Wald regime — the convention used by
`summary.glm()`, Stata's `logit, or`, and SPSS `LOGISTIC
REGRESSION`. The table title becomes family-aware ("Logistic
regression", "Poisson regression", "Probit regression", …) and
the default fit-statistics block swaps in `nobs`,
`pseudo_r2_mcfadden`, `pseudo_r2_nagelkerke`, and `AIC` instead
of `R²` and `Adj.R²`.

We illustrate with a logistic regression of `smoking` on `sex`,
`age`, and `education` from `sochealth`. The `education` variable
is an ordered factor in the source data; we convert it to a plain
factor so contrasts are dummy-coded (one row per level) rather
than the polynomial trends (`.L`, `.Q`, …) R applies by default to
ordered factors. The model mixes a binary factor (`sex`), a
numeric predictor (`age`), and a 3-level factor (`education`) —
enough to exercise every glm-specific feature below.

```{r glm-basic}
sh <- sochealth |>
  dplyr::mutate(education = factor(education, ordered = FALSE))

fit <- glm(smoking ~ sex + age + education, data = sh,
           family = binomial)
table_regression(fit)
```

### Response-scale display: `exponentiate = TRUE`

Set `exponentiate = TRUE` to switch the `B` column to the
response scale and rebrand its header per family and link: `OR`
for `binomial(logit)`, `IRR` for `poisson(log)`, `HR` for
`binomial(cloglog)`, `RR` for `binomial(log)`, `MR` for
`Gamma(log)`, and the generic `exp(B)` otherwise. The standard
error follows the delta-method approximation
`SE_OR = OR × SE_log-odds` (the Stata `logit, or` convention).
The test statistic and the p-value are invariant under any
monotone transformation, so they remain on the link scale and
match the unexponentiated table verbatim:

```{r glm-or}
table_regression(fit, exponentiate = TRUE)
```

### Term-level partial chi-square

`partial_chi2` is the glm analog of `partial_f2`: for each model
term, the partial likelihood-ratio chi-square via
`drop1(test = "LRT")` (SAS PROC LOGISTIC `TYPE3`; Long & Freese
2014 §3.5). Rendered as `value (df)` so factor terms (`k − 1`
df) and numeric terms (1 df) read at a glance:

```{r glm-partial-chi2}
fit2 <- glm(smoking ~ age + education, data = sh, family = binomial)
table_regression(fit2, show_columns = c("b", "partial_chi2", "p"))
```

### Standardised coefficients: `refit` and the new `pseudo`

For `glm`, `standardized = "refit"` z-scores numeric *predictors*
only and refits the model — the response stays on its observed
scale because the link function is fixed. This is the
"x-standardization" convention (Long and Freese 2014 §4.3.4).
The other algebraic methods (`"posthoc"`, `"basic"`, `"smart"`)
apply X-only scaling using the same algebra as in the `lm` case.

`standardized = "pseudo"` (`glm` only) is the Menard (2004, 2011)
*fully* standardised coefficient, scaling by `SD(X) / SD(Y*)`
where `Y*` is the latent variable on the link scale and
`SD(Y*) = sqrt(var(linear-predictor) + var_link)` with
`var_link` = π²/3 for logit, 1 for probit, π²/6 for cloglog.
Defined for binomial families; non-binomial returns NA with a
`spicy_caveat`:

```{r glm-pseudo}
table_regression(fit, standardized = "pseudo")
```

### Average marginal effects: probability units, not log-odds

For `glm`, AME is computed as the average of `dE[Y|X] / dx` over
the observed sample — **on the response scale, not the link
scale**. For logistic regression this means the displayed AME is
in **probability units, not log-odds**: an AME of `-0.15` for
`education = Tertiary` reads "on average, holding sex and age
constant, a respondent with tertiary education is 15 percentage
points less likely to be a current smoker than one with lower
secondary education". This is almost always the quantity a
substantive reader wants from a logistic regression, and it is
the reason AME is increasingly recommended over odds ratios for
communicating logistic effects (Mood 2010; Long and Freese 2014
§5.3).

```{r glm-ame}
table_regression(fit, show_columns = c("b", "p", "ame", "ame_ci", "ame_p"))
```

Point estimate and inference are delegated to
[`marginaleffects::avg_slopes()`][marginaleffects::avg_slopes].
Under cluster-robust variance, the Satterthwaite-df handling
described in the *Robust variance* section applies to `glm` AME
as well.

### Profile-likelihood CIs: `ci_method = "profile"`

The default is `ci_method = "wald"` (symmetric
`estimate ± z × SE`, matching `summary.glm`). For `glm` you can
also request `ci_method = "profile"`, which calls
`MASS::confint.glm()` to compute the profile-likelihood CI —
asymmetric, exact for likelihood-based inference, and the gold
standard under sparse data or near-boundary estimates (Venables
& Ripley *MASS* §7.2). The estimate, SE, statistic, and p-value
all remain Wald; `"profile"` only refines the CI bounds:

```{r glm-profile}
table_regression(fit, ci_method = "profile")
```

### Hierarchical glm (LRT)

`nested = TRUE` on a list of nested glm fits adds a "Model
comparison" block whose default tokens are `c("LRT", "p")` — the
APA-conventional layout for hierarchical logistic regression
(Hosmer & Lemeshow §3.5; Long & Freese 2014 §3.6). The
chi-square statistic comes from `anova(test = "LRT")`.
`r2_change` and `F` are not defined for glm and return em-dashes
if explicitly requested:

```{r glm-nested}
m1 <- glm(smoking ~ sex,                   data = sh, family = binomial)
m2 <- glm(smoking ~ sex + age,             data = sh, family = binomial)
m3 <- glm(smoking ~ sex + age + education, data = sh, family = binomial)
table_regression(list(m1, m2, m3), nested = TRUE)
```

### Gaussian glm caveat

A `glm()` with `family = gaussian` and `link = "identity"` is
mathematically equivalent to `lm()` but lacks the variance-
explained effect-size family (`partial_f2 / η² / ω²`) and the
Satterthwaite-corrected AME path. Following the *transparency
over rejection* rule, spicy accepts the fit and emits a
`spicy_caveat` suggesting a refit with `lm()`.

## Significance stars

Stars are off by default. APA 7 §6.46 explicitly discourages
asterisks-only reporting in favour of exact p-values, and ASA's
post-2019 guidance (Wasserstein, Schirm, and Lazar 2019)
reinforces the same point. Set `stars = TRUE` for the APA preset
(`*** p < .001, ** p < .01, * p < .05`) or pass a named numeric
vector for custom thresholds:

```{r stars}
fit <- lm(wellbeing_score ~ age + sex + smoking, data = sochealth)
table_regression(fit, stars = TRUE)
```

Stars suffix the `B` column (or `β` when standardisation is
requested); the threshold mapping is auto-documented in the
footer. The `p` column itself remains unstarred so the numeric
value stays readable.

## Display knobs

`labels` accepts a named character vector keyed by either formula
term labels (`"sex"`) or coefficient names (`"sexMale"`), so
individual contrast rows can be relabelled. `intercept_position`
switches the intercept to the bottom (Stata convention).
`reference_style = "annotation"` lifts the reference level into
the factor header (`"sex: [ref: Female]"`) and drops the explicit
reference row, for compact tables. `decimal_mark = ","` switches
to European decimal notation and automatically changes the CI
separator to `";"` to avoid ambiguity:

```{r polish}
fit <- lm(wellbeing_score ~ age + sex + education, data = sochealth)
table_regression(
  fit,
  labels = c(
    age            = "Age (years)",
    sex            = "Sex",
    education      = "Education"
  ),
  reference_style = "annotation",
  intercept_position = "last",
  decimal_mark = ","
)
```

## Output formats

The default print method draws an ASCII table to the console.
The same content is available as a plain `data.frame`
(`output = "data.frame"`), a long broom-style data.frame
(`output = "long"`), or as a rich-format table for `gt`,
`tinytable`, `flextable`, `excel`, `word`, or `clipboard`:

```{r data-frame}
out <- table_regression(
  lm(wellbeing_score ~ age + sex + smoking, data = sochealth),
  output = "data.frame"
)
str(out)
```

```{r long}
out <- table_regression(
  lm(wellbeing_score ~ age + sex + smoking, data = sochealth),
  output = "long"
)
out[, c("model_id", "term", "estimate_type", "estimate",
        "std.error", "p.value")]
```

```{r gt-output, eval = build_rich_tables}
pkgdown_dark_gt(
  table_regression(
    lm(wellbeing_score ~ age + sex + smoking, data = sochealth),
    output = "gt"
  )
)
```

## broom integration

`broom::tidy()` returns a long tibble with one row per
`(model_id, term, estimate_type)` and broom-canonical column
names (`estimate`, `std.error`, `conf.low`, `conf.high`,
`statistic`, `p.value`). `broom::glance()` returns one row per
`(model_id, outcome)` with model-level statistics; `df.residual`
is preserved as a numeric so cluster-robust Satterthwaite df
flows through verbatim:

```{r broom}
fit <- lm(wellbeing_score ~ age + sex + smoking, data = sochealth)
out <- table_regression(fit, standardized = "refit")

broom::tidy(out)
broom::glance(out)
```

The long format is the right entry point when the table is one
step in a larger pipeline — saving to disk for the manuscript
appendix, faceting by subgroup, or feeding a downstream
post-estimation analysis. The `tidy()` output keeps the
`estimate_type` column so the same data frame can hold rows for
B, β, AME, and per-coefficient effect-size estimates without
ambiguity.

## See also

- `vignette("table-continuous-lm", package = "spicy")` for the
  one-predictor-by-many-outcomes counterpart (estimated marginal
  means, contrast or slope, four effect-size families).
- `vignette("summary-tables-reporting", package = "spicy")` for
  an end-to-end reporting workflow combining the spicy summary-
  table helpers.

## References

Aiken, L. S., and West, S. G. (1991).
*Multiple Regression: Testing and Interpreting Interactions*.
Sage.

American Psychological Association (2020).
*Publication Manual of the American Psychological Association*
(7th ed.). Section 6.46.

Cohen, J., Cohen, P., West, S. G., and Aiken, L. S. (2003).
*Applied Multiple Regression / Correlation Analysis for the
Behavioral Sciences* (3rd ed.). Lawrence Erlbaum.

Gelman, A. (2008). Scaling regression inputs by dividing by two
standard deviations. *Statistics in Medicine*, 27(15), 2865–2873.

Gelman, A., Hill, J., and Yajima, M. (2012). Why we (usually)
don't have to worry about multiple comparisons. *Journal of
Research on Educational Effectiveness*, 5(2), 189–211.

Greenland, S. (2017). For and against methodologies: Some
perspectives on recent causal and statistical inference debates.
*European Journal of Epidemiology*, 32(1), 3–20.

Harrell, F. E. (2015). *Regression Modeling Strategies* (2nd
ed.). Springer. Section 5.4 on multiplicity.

Hosmer, D. W., Lemeshow, S., and Sturdivant, R. X. (2013).
*Applied Logistic Regression* (3rd ed.). Wiley.

Imbens, G. W., and Kolesár, M. (2016). Robust standard errors in
small samples: Some practical advice. *Review of Economics and
Statistics*, 98(4), 701–712.

Long, J. S., and Ervin, L. H. (2000). Using heteroscedasticity
consistent standard errors in the linear regression model.
*The American Statistician*, 54(3), 217–224.

Long, J. S., and Freese, J. (2014). *Regression Models for
Categorical Dependent Variables Using Stata* (3rd ed.). Stata
Press.

McFadden, D. (1974). Conditional logit analysis of qualitative
choice behavior. In P. Zarembka (Ed.), *Frontiers in
Econometrics* (pp. 105–142). Academic Press.

Menard, S. (2004). Six approaches to calculating standardized
logistic regression coefficients. *The American Statistician*,
58(3), 218–223.

Menard, S. (2011). Standards for standardized logistic
regression coefficients. *Social Forces*, 89(4), 1409–1428.

Mood, C. (2010). Logistic regression: Why we cannot do what we
think we can do, and what we can do about it. *European
Sociological Review*, 26(1), 67–82.

Nagelkerke, N. J. D. (1991). A note on a general definition of
the coefficient of determination. *Biometrika*, 78(3), 691–692.

Olejnik, S., and Algina, J. (2003). Generalized eta and omega
squared statistics: Measures of effect size for some common
research designs. *Psychological Methods*, 8(4), 434–447.

Pustejovsky, J. E., and Tipton, E. (2018). Small-sample methods
for cluster-robust variance estimation and hypothesis testing in
fixed effects models. *Journal of Business & Economic
Statistics*, 36(4), 672–683.

Rothman, K. J. (1990). No adjustments are needed for multiple
comparisons. *Epidemiology*, 1(1), 43–46.

Smithson, M. (2003). *Confidence Intervals*. Sage.

Steiger, J. H. (2004). Beyond the F test: Effect size confidence
intervals and tests of close fit in the analysis of variance and
contrast analysis. *Psychological Methods*, 9(2), 164–182.

Tjur, T. (2009). Coefficients of determination in logistic
regression models — A new proposal: The coefficient of
discrimination. *The American Statistician*, 63(4), 366–372.

Venables, W. N., and Ripley, B. D. (2002). *Modern Applied
Statistics with S* (4th ed.). Springer. Section 7.2 on profile
likelihood.

Wasserstein, R. L., Schirm, A. L., and Lazar, N. A. (2019).
Moving to a world beyond "p < 0.05". *The American Statistician*,
73(sup1), 1–19.

---
title: "Model-based continuous summary tables in R"
description: >
  Build model-based summary tables for continuous outcomes in R with
  table_continuous_lm(): estimated marginal means, robust /
  cluster-robust / bootstrap / jackknife standard errors, case
  weights, additive covariate adjustment (G-computation or
  equal-weight), four effect-size families (f2, Cohen's d, Hedges'
  g, Hays' omega2) with noncentral CIs, and APA-style output to
  console, gt, tinytable, flextable, Excel, Word, or clipboard.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Model-based continuous summary tables in R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

build_rich_tables <- identical(Sys.getenv("IN_PKGDOWN"), "true")

source("_pkgdown-helpers.R")
```

```{r setup}
library(spicy)
```

`table_continuous_lm()` is the model-based companion to
`table_continuous()`. It fits one linear model per selected
continuous outcome --
`lm(outcome ~ by + covariate1 + covariate2 + ...)` if covariates are
supplied, otherwise `lm(outcome ~ by)` -- and returns a compact
reporting table. This is the right choice when you want to stay in
a linear-model workflow with heteroskedasticity-consistent or
cluster-robust standard errors, bootstrap / jackknife variance,
case weights, additive covariate adjustment, or one of four
effect-size families (`f2`, Cohen's *d*, Hedges' *g*, Hays'
\(\omega^2\)) with noncentral CIs.

## Basic usage

Use `select` for one or more continuous outcomes and `by` for the
single predictor:

```{r basic}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi, life_sat_health),
  by = sex
)
```

For categorical predictors, the table reports estimated means by level.
When the predictor is dichotomous, it can also show a single mean
difference and confidence interval.

## Robust standard errors

Use `vcov = "HC*"` when you want heteroskedasticity-consistent standard
errors and tests:

```{r robust}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  vcov = "HC3",
  statistic = TRUE
)
```

The `HC*` family is computed via [sandwich::vcovHC()] and includes
`"HC0"`, `"HC1"`, `"HC2"`, `"HC3"` (default for small / moderate
samples), `"HC4"`, `"HC4m"`, and `"HC5"`.

## Cluster-robust standard errors

When observations are not independent (repeated measurements per
individual, students nested in classes, panel data), classical and
`HC*` standard errors are biased downward. Use the `CR*` family
together with `cluster = id_var` to get cluster-robust inference,
dispatched to `clubSandwich`:

```{r cluster, eval = requireNamespace("clubSandwich", quietly = TRUE)}
table_continuous_lm(
  sleep,
  select = extra,
  by = group,
  vcov = "CR2",
  cluster = ID,
  statistic = TRUE
)
```

`"CR2"` is the recommended default (Bell & McCaffrey 2002;
Pustejovsky & Tipton 2018). It produces fractional Satterthwaite
degrees of freedom, rendered in the displayed test header as e.g.
`t(8.7)` or `F(2, 12.4)`. `"CR1"` matches Stata's
`vce(cluster id)` default.

## Bootstrap and jackknife

For situations where the residual distribution is non-standard or
the sample is small, `vcov = "bootstrap"` and `vcov = "jackknife"`
provide resampling-based variance estimators in pure base R (no
dependency added):

```{r bootstrap, eval = FALSE}
table_continuous_lm(
  sochealth,
  select = wellbeing_score,
  by = sex,
  vcov = "bootstrap",
  boot_n = 1000  # default
)
```

When `cluster` is supplied, bootstrap switches to a cluster
bootstrap (Cameron, Gelbach & Miller 2008) and jackknife to
leave-one-cluster-out (Quenouille 1956). Both estimators use
asymptotic inference: `z` for single-coefficient contrasts and
`chi^2(q)` for the global Wald test on `k > 2` categorical
predictors, rendered in the displayed test header.

## Case weights

Use `weights` when you want weighted estimated means or slopes in the
same model-based table:

```{r weights}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = education,
  weights = weight,
  show_weighted_n = TRUE
)
```

This is often the most natural summary-table function when your
reporting workflow already relies on weighted linear models.

## Numeric predictors

If `by` is numeric, `table_continuous_lm()` reports the slope rather
than group means:

```{r numeric-by}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = age,
  vcov = "HC3"
)
```

When you need the underlying returned data for further processing, use
`output = "data.frame"` for the wide raw table or `output = "long"` for
the analytic long table.

## Covariate adjustment

When the comparison of `by`-group means is potentially confounded by
other variables, supply them through `covariates`. Each per-outcome
linear model becomes
`lm(outcome ~ by + covariate1 + covariate2 + ...)`, and the reported
`Δ` (categorical `by`) or slope (numeric `by`), its CI, *p*, R² and
effect size are all **adjusted for** the covariates. The default
ASCII print emits an APA-style footer naming the adjustment so the
reader knows which estimand they are reading.

```{r adjustment-basic}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  covariates = c(age, education),
  vcov = "HC3"
)
```

`covariates` accepts the full tidyselect vocabulary
(`c(age, sex)`, `tidyselect::starts_with("life_sat_")`,
`tidyselect::all_of(cov_names)`) or a literal character vector. A
covariate that also appears in `select` is silently auto-excluded
from the outcome list (mirroring how `by` / `weights` are excluded
from `select`); a covariate equal to `by` is rejected up front.

### Two estimands for the adjusted marginal means

Two methodologically distinct conventions exist for computing the
covariate-adjusted per-group means displayed in the table (the
`M (level)` columns of the printed output, and the `emmean` /
`emmean_se` / `emmean_ci_lower` / `emmean_ci_upper` columns of the
`output = "long"` data frame) when `by` is categorical.
`table_continuous_lm()` exposes the choice via `adjustment`; both
are valid but produce different numerical answers when at least one
factor covariate has non-uniform observed proportions.

* `adjustment = "proportional"` (the **default**) — G-computation /
  population-weighted standardisation. For each focal level of `by`,
  predictions are evaluated at every observation in the data with
  `by` replaced by that level (covariates kept at their observed
  values), then averaged. This is the convention of Stata's
  `margins` and `marginaleffects::avg_predictions(variables)`.
  Interpretation: *"What is the predicted mean in this population
  if everyone had `by = lvl`, holding their actual covariate
  profile?"*. Best for causal / counterfactual reporting.

* `adjustment = "balanced"` — equal-weight averaging on a synthetic
  grid. Factor-covariate level combinations are crossed and weighted
  uniformly (`1 / k`); numeric covariates are fixed at their sample
  mean. This is the convention of `emmeans::emmeans()` (default),
  SPSS UNIANOVA EMMEANS, and SAS LSMEANS. Interpretation:
  *"What is the predicted mean assuming a balanced covariate
  design?"*. Best for descriptive / ANOVA-style APA reporting.

The two methods coincide trivially when `covariates` is `NULL`, and
also when all covariates are numeric or logical (no factor levels
to expand over). They diverge only when categorical covariate
proportions are skewed in the observed data.

```{r adjustment-balanced}
# Same model, balanced (emmeans-style) marginal means:
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  covariates = c(age, education),
  adjustment = "balanced",
  vcov = "HC3"
)
```

### Effect sizes under adjustment

Adding covariates changes which effect sizes remain meaningful:

* `effect_size = "f2"` and `"omega2"` are reported as **partial**
  *f²* / partial *ω²* via `stats::drop1()` restricted to the focal
  term — the correct generalisation of the bivariate effect size to
  a covariate-adjusted model.
* `effect_size = "d"` and `"g"` raise `spicy_unsupported`: Cohen's
  *d* and Hedges' *g* have no defined extension under adjustment
  (the pooled SD has no canonical analogue). The error message
  points to `f2` / `omega2` as the partial-F generalisations.

Both effect-size variants and their noncentral CIs (when
`effect_size_ci = TRUE`) are computed from the same fitted model as
the displayed coefficients and remain invariant to `vcov` (HC* /
CR* / bootstrap / jackknife only affect the contrast SE).

### Scope: additive only in v1

`covariates` accepts additive terms only. Formula syntax with
interactions or transforms (`covariates = ~ age * sex`,
`covariates = ~ I(age^2)`) raises `spicy_unsupported` with a
migration hint. Interactions and polynomials are reserved for a
future spicy release. To fit a fully-specified model right now,
do so with `lm()` directly and inspect via `summary()` or
`table_regression()` (which accepts interaction / polynomial / `I()`
terms natively).

## Effect sizes

`table_continuous_lm()` supports four effect sizes via the
`effect_size` argument. All are derived from the **same fitted model**
as the displayed coefficients and `R²`, so the table stays internally
consistent — and all are **invariant to `vcov`** (the choice of
classical or `HC*` standard errors changes the contrast SE, CI, and
test statistic but not the standardized effect size itself).

Cohen's *d* and Hedges' *g* are the conventions for two-group
comparisons (Cohen 1988; Hedges and Olkin 1985; APA 2020) and require
`by` to have exactly two non-empty levels:

```{r es-d}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = smoking,
  effect_size = "d"
)
```

Hedges' *g* applies the small-sample correction
`J = 1 - 3/(4·df_resid - 1)`. It is generally preferred over raw *d*
in published reports (Goulet-Pelletier and Cousineau 2018):

```{r es-g}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = smoking,
  effect_size = "g"
)
```

For categorical predictors with three or more levels (or numeric
predictors), use Hays' `omega²` for a less biased estimate of the
population variance explained (Hays 1963; Olejnik and Algina 2003;
Lakens 2013):

```{r es-omega2}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = education,
  effect_size = "omega2"
)
```

Cohen's `f²` (= `R² / (1 - R²)`) is the effect size familiar from
power-analysis frameworks (e.g. G*Power) and is defined for any
predictor type:

```{r es-f2}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = age,
  effect_size = "f2"
)
```

## Confidence intervals for effect sizes

Setting `effect_size_ci = TRUE` adds a confidence interval for the
effect size. The implementation uses the modern noncentral-distribution
inversion approach — the consensus standard in commercial statistical
software (Stata `esize` / `estat esize`, SAS `PROC TTEST` and
`PROC GLM EFFECTSIZE`) and in mainstream R packages (`effectsize`,
`MOTE`, `TOSTER`, `effsize`):

- Noncentral *t* inversion for `"d"` and `"g"` (Steiger and Fouladi
  1997; Goulet-Pelletier and Cousineau 2018; Cousineau and
  Goulet-Pelletier 2021), which has nominal coverage across sample
  sizes — unlike the older Hedges-Olkin normal approximation that is
  biased for small samples.
- Noncentral *F* inversion for `"omega2"` and `"f2"`
  (Steiger 2004; Smithson 2003).

In the printed and rendered outputs, the effect size is displayed with
the bracketed CI (article-ready):

```{r es-ci}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = smoking,
  effect_size = "g",
  effect_size_ci = TRUE
)
```

The CI level follows `ci_level` (default `0.95`). For programmatic
access, the wide raw output exposes separate numeric columns, and the
long output always includes them:

```{r es-ci-raw}
table_continuous_lm(
  sochealth,
  select = wellbeing_score,
  by = smoking,
  effect_size = "g",
  effect_size_ci = TRUE,
  output = "data.frame"
)
```

```{r es-ci-long}
out <- table_continuous_lm(
  sochealth,
  select = wellbeing_score,
  by = smoking,
  effect_size = "g",
  effect_size_ci = TRUE,
  output = "long"
)
out[, c("variable", "es_type", "es_value", "es_ci_lower", "es_ci_upper")]
```

When `weights` is supplied, all four effect sizes (and their CIs) are
computed from the weighted least-squares fit, keeping them consistent
with the weighted contrast and its CI (DuMouchel and Duncan 1983).
This is the right convention for case-weighted reporting; for
propensity-score balance assessment (Austin and Stuart 2015) or
complex-survey designs, dedicated packages (`cobalt::bal.tab()` and
`survey`) are more appropriate.

## Article-style polish

Pretty outcome labels and a comma decimal separator (useful for
European reporting):

```{r polish}
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  labels = c(
    wellbeing_score = "WHO-5 wellbeing (0-100)",
    bmi = "Body-mass index (kg/m²)"
  ),
  effect_size = "g",
  effect_size_ci = TRUE,
  decimal_mark = ","
)
```

## Publication-ready output

The function supports the same output formats as the other summary-table
helpers, including `tinytable`, `gt`, `flextable`, `excel`, `word`, and
`clipboard`.

```{r gt-output, eval = build_rich_tables}
pkgdown_dark_gt(
  table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi, life_sat_health),
    by = sex,
    vcov = "HC3",
    statistic = TRUE,
    output = "gt"
  )
)
```

## Tidying for downstream pipelines

`table_continuous_lm()` returns an object that can be coerced to a
plain `data.frame` / `tbl_df` (stripping the spicy formatting
attributes) or piped into `broom::tidy()` / `broom::glance()` for
any downstream tidyverse-stats workflow:

```{r tidy-glance}
out <- table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  effect_size = "g",
  effect_size_ci = TRUE
)

# One row per estimated parameter: emmean per level, contrast for
# binary predictors, slope for numeric predictors.
broom::tidy(out)

# One row per outcome with model-level statistics: r.squared,
# adj.r.squared, F / t, df, p.value, nobs, weighted_n, plus the
# effect-size summary.
broom::glance(out)
```

## See also

- See `vignette("table-continuous", package = "spicy")` for descriptive
  continuous summary tables with classical group-comparison tests.
- See `vignette("summary-tables-reporting", package = "spicy")` for a
  cross-function reporting workflow using the summary-table helpers.

---
title: "Summary tables for APA-style reporting"
description: >
  Learn when to use table_categorical(), table_continuous(), and
  table_continuous_lm() for APA-style reporting in R, how their shared
  arguments fit together, and which output format to choose for
  console, Quarto, Word, or Excel workflows.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Summary tables for APA-style reporting}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

build_rich_tables <- identical(Sys.getenv("IN_PKGDOWN"), "true")

source("_pkgdown-helpers.R")
```

```{r setup}
library(spicy)
```

`table_categorical()`, `table_continuous()`, and
`table_continuous_lm()` share the same reporting grammar: choose
variables with `select`, optionally split the table with `by`, apply
readable labels, and pick an output format that matches your reporting
workflow. This vignette focuses on that shared logic rather than
repeating every function-specific option.

## Choose the right function

Use the function that matches the type of variables you want to report:

| Function | Use for | Optional `by` | Typical additions |
|:--|:--|:--|:--|
| `table_categorical()` | Factors, labelled categorical variables, grouped frequency-style summaries | Yes | Chi-squared test, association measure, confidence interval |
| `table_continuous()` | Numeric or continuous variables | Yes | Group-comparison test, statistic, effect size |
| `table_continuous_lm()` | Continuous outcomes in a linear-model framework | No, requires a single predictor | Robust / cluster-robust / bootstrap / jackknife SE, case weights, additive covariate adjustment, four effect-size families with noncentral CIs |

In practice:

- use `table_categorical()` for smoking status, education, or activity;
- use `table_continuous()` for BMI, income, or scale scores;
- use `table_continuous_lm()` when the same outcomes should be reported
  through simple weighted or robust linear models;
- keep `by` for the grouping variable you want to compare across.

## A shared interface

Both functions use the same core arguments:

```{r grammar-categorical}
table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  by = education,
  labels = c("Smoking status", "Regular physical activity"),
  output = "tinytable"
)
```

```{r grammar-continuous}
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education,
  labels = c(
    bmi = "Body mass index",
    wellbeing_score = "Well-being score",
    life_sat_health = "Satisfaction with health"
  ),
  output = "tinytable"
)
```

```{r grammar-continuous-lm}
table_continuous_lm(
  sochealth,
  select = c(bmi, wellbeing_score, life_sat_health),
  by = education,
  weights = weight
)
```

The same argument pattern works in both cases:

- `select` chooses the reported variables;
- `by` defines the grouping structure;
- `labels` cleans up the row labels;
- `output` decides how the result is rendered or exported.

For model-based continuous tables, the same pattern applies, but `by`
must be a single predictor because one linear model is fit per outcome.

## A practical reporting sequence

A common report contains both table types, often with the same grouping
variable. For example, you might first summarize categorical health
behaviors, then summarize continuous well-being indicators.

### Categorical variables

```{r report-categorical, eval = build_rich_tables}
pkgdown_dark_gt(
  table_categorical(
    sochealth,
    select = c(smoking, physical_activity, dentist_12m),
    by = education,
    labels = c(
      "Smoking status",
      "Regular physical activity",
      "Visited a dentist in the last 12 months"
    ),
    output = "gt"
  )
)
```

### Continuous variables

```{r report-continuous, eval = build_rich_tables}
pkgdown_dark_gt(
  table_continuous(
    sochealth,
    select = c(bmi, wellbeing_score, life_sat_health),
    by = education,
    labels = c(
      bmi = "Body mass index",
      wellbeing_score = "Well-being score",
      life_sat_health = "Satisfaction with health"
    ),
    p_value = TRUE,
    effect_size = TRUE,
    output = "gt"
  )
)
```

This keeps the reporting structure consistent while still using the
function that fits each variable type.

### Model-based continuous variables

```{r report-continuous-lm, eval = build_rich_tables}
pkgdown_dark_gt(
  table_continuous_lm(
    sochealth,
    select = c(bmi, wellbeing_score, life_sat_health),
    by = sex,
    vcov = "HC3",
    statistic = TRUE,
    output = "gt"
  )
)
```

This is the better summary-table path when the article is already
organized around simple linear models, weighted analyses, or robust
standard errors.

## Choose the output format

All three functions support the same reporting formats:

| Output | Best use |
|:--|:--|
| `"default"` | Quick console review in plain ASCII |
| `"tinytable"` | Quarto or R Markdown documents |
| `"gt"` | HTML output with styled reporting tables |
| `"flextable"` | Office-first workflows; also renders in HTML |
| `"excel"` | Spreadsheet handoff or downstream editing |
| `"word"` | Direct `.docx` export |
| `"clipboard"` | Fast pasting into another application |

Pick the output based on where the table is going, not on the analysis
itself. The underlying selection and grouping pattern stays the same.

If you want an object that fits naturally into Word and PowerPoint
workflows but can also be rendered in HTML documents, `flextable` is a
good choice:

```{r output-flextable, eval = FALSE}
if (requireNamespace("flextable", quietly = TRUE)) {
  table_continuous(
    sochealth,
    select = c(bmi, wellbeing_score, life_sat_health),
    by = education,
    output = "flextable"
  )
}
```

## Post-process the returned table object

Both summary-table helpers return regular `gt`, `tinytable`, or
`flextable` objects, so you can keep styling them with the native
package API.

Use `gt::` functions when you want to keep the `gt` workflow:

```{r postprocess-gt, eval = build_rich_tables}
tab <- pkgdown_dark_gt(table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  by = education,
  labels = c("Smoking status", "Regular physical activity"),
  output = "gt"
))

tab |>
  gt::tab_header(
    title = "Health behaviors by education",
    subtitle = "Categorical summary table"
  ) |>
  gt::tab_source_note(
    gt::md("*Percentages are computed within each education group.*")
  )
```

Use `tinytable::` functions when you want lightweight table-specific
styling:

```{r postprocess-tinytable, eval = build_rich_tables}
tab <- table_categorical(
  sochealth,
  select = c(smoking, physical_activity),
  by = education,
  labels = c("Smoking status", "Regular physical activity"),
  output = "tinytable"
)

tab |>
  tinytable::style_tt(
    i = 2:3,
    j = 2:5,
    background = "red",
    color = "white",
    bold = TRUE
  )
```

Use `flextable::` functions when you want to keep working toward Office
or HTML document output. The example is shown as code here because the
dark pkgdown theme is not a reliable preview of the final `flextable`
HTML rendering:

```{r postprocess-flextable, eval = FALSE}
if (requireNamespace("flextable", quietly = TRUE)) {
  tab <- table_continuous(
    sochealth,
    select = c(bmi, wellbeing_score),
    by = education,
    output = "flextable"
  )

  tab |>
    flextable::theme_booktabs() |>
    flextable::autofit() |>
    flextable::fontsize(size = 10, part = "all")
}
```

## Keep the detailed options in the function-specific articles

The dedicated articles go deeper into each function:

- `table_categorical()` covers missing values, level filtering,
  association measures, and one-way frequency-style tables.
- `table_continuous()` covers grouped descriptive statistics,
  parametric and nonparametric tests, and effect sizes.
- `table_continuous_lm()` covers estimated marginal means or slopes
  from linear models, robust / cluster-robust / bootstrap / jackknife
  variance, case weights, additive covariate adjustment
  (G-computation or equal-weight), and four effect-size families
  with noncentral CIs.

Use this vignette as the final reporting overview, then consult the
function-specific articles when you need the detailed controls.

---
title: "Cramer's V, Phi, and association measures for contingency tables in R"
description: >
  Calculate Cramer's V, Phi, Goodman-Kruskal Gamma, Kendall's Tau-b,
  Somers' D, and other effect sizes for contingency tables in R, with
  confidence intervals and p-values.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Cramer's V, Phi, and association measures for contingency tables in R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(spicy)
```

spicy provides a full suite of effect size and association measures for
contingency tables, covering both nominal and ordinal variables. This
vignette explains which measure to use depending on the measurement
level of your variables, and how to obtain confidence intervals and
p-values for chi-squared-based and rank-based statistics.

## Choosing the right measure

The table below summarizes the recommended measures by variable type.

| Variable types | Recommended measure | Function |
|---|---|---|
| Nominal x Nominal | Cramer's V | `cramer_v()` |
| Nominal x Nominal | Contingency Coefficient | `contingency_coef()` |
| Nominal x Nominal (2x2) | Phi | `phi()` |
| Ordinal x Ordinal | Kendall's Tau-b | `kendall_tau_b()` |
| Ordinal x Ordinal (rectangular) | Kendall's Tau-c | `kendall_tau_c()` |
| Ordinal x Ordinal | Goodman-Kruskal Gamma | `gamma_gk()` |
| Ordinal x Ordinal (asymmetric) | Somers' D | `somers_d()` |
| Nominal (asymmetric, PRE) | Lambda | `lambda_gk()` |
| Nominal (asymmetric, PRE) | Goodman-Kruskal Tau | `goodman_kruskal_tau()` |
| Nominal (asymmetric, PRE) | Uncertainty Coefficient | `uncertainty_coef()` |
| 2x2 table | Yule's Q | `yule_q()` |

PRE = Proportional Reduction in Error. These measures quantify how much
knowing one variable reduces prediction error for the other.

All functions accept a contingency table (class `table`, typically from
`xtabs()` or `table()`).

## Quick overview with assoc_measures()

`assoc_measures()` computes all available measures at once:

```{r assoc-all}
tbl <- xtabs(~ smoking + education, data = sochealth)
assoc_measures(tbl)
```

This is useful for exploratory analysis. For reporting, pick the measure
that matches your variable types.

## Nominal variables

### Cramer's V

Cramer's V measures the strength of association between two nominal
variables. It ranges from 0 (no association) to 1 (perfect association).

```{r cramer}
tbl <- xtabs(~ smoking + education, data = sochealth)
cramer_v(tbl)
```

Pass `detail = TRUE` for the confidence interval and p-value. The
p-value tests the null hypothesis of no association using the Pearson
chi-squared test.

```{r cramer-detail}
cramer_v(tbl, detail = TRUE)
```

### Phi coefficient

For 2x2 tables, Phi is equivalent to Cramer's V. spicy implements Phi
as \(\sqrt{\chi^2 / n}\), matching the DescTools and SPSS conventions
-- the value is always non-negative and reports only the *strength*
of association, not its direction. To recover the signed direction,
compute the Pearson correlation between the two binary variables
explicitly (e.g., `cor(x, y)` after coding both 0/1). The p-value
tests H0: no association (Pearson chi-squared test).

```{r phi}
tbl_22 <- xtabs(~ smoking + physical_activity, data = sochealth)
phi(tbl_22, detail = TRUE)
```

### Contingency coefficient

The contingency coefficient is an alternative to Cramer's V. Its upper
bound depends on the table dimensions, which makes it harder to compare
across tables of different sizes. The p-value tests H0: no association
(Pearson chi-squared test).

```{r contingency}
contingency_coef(tbl, detail = TRUE)
```

## Ordinal variables

When both variables are ordinal (ordered factors), measures that account
for the ordering are more appropriate than Cramer's V.

### Goodman-Kruskal Gamma

Gamma ranges from -1 to +1. It ignores tied pairs, which makes it
sensitive to the direction of association but tends to overestimate
strength when there are many ties.

```{r gamma}
tbl_ord <- xtabs(~ self_rated_health + education, data = sochealth)
gamma_gk(tbl_ord, detail = TRUE)
```

A positive value means that higher values on one variable tend to occur
with higher values on the other. The p-value tests H0: Gamma = 0 using
a Wald z-test.

### Kendall's Tau-b

Tau-b adjusts for ties and ranges from -1 to +1. It is generally
preferred over Gamma for square or near-square tables. The p-value
tests H0: Tau-b = 0 (Wald z-test).

```{r tau-b}
kendall_tau_b(tbl_ord, detail = TRUE)
```

### Kendall's Tau-c

Tau-c is similar to Tau-b but adjusts for rectangular tables where the
number of rows and columns differ. The p-value tests H0: Tau-c = 0
(Wald z-test).

```{r tau-c}
kendall_tau_c(tbl_ord, detail = TRUE)
```

### Somers' D

Somers' D is an asymmetric measure: it distinguishes between a dependent
and an independent variable. By default, the row variable is treated as
dependent (D(R|C)). The p-value tests H0: D = 0 (Wald z-test).

```{r somers}
somers_d(tbl_ord, detail = TRUE)
```

## Asymmetric (PRE) measures

These measures answer a specific question: how much does knowing the
column variable reduce our error in predicting the row variable (or vice
versa)?

### Lambda

Lambda measures the proportional reduction in classification error. It
can equal zero even when the variables are associated, if the modal
category does not change across columns. The p-value tests H0:
Lambda = 0 (Wald z-test).

```{r lambda}
tbl <- xtabs(~ self_rated_health + education, data = sochealth)
lambda_gk(tbl, detail = TRUE)
```

### Goodman-Kruskal Tau

Tau measures the proportional reduction in error when predicting the row
variable from the column variable, using the full distribution (not just
the mode). The p-value tests H0: Tau = 0 (Wald z-test).

```{r gk-tau}
goodman_kruskal_tau(tbl, detail = TRUE)
```

### Uncertainty coefficient

The uncertainty coefficient (Theil's U) is based on entropy. It measures
how much knowing one variable reduces uncertainty about the other. The
p-value tests H0: U = 0 (Wald z-test).

```{r uncertainty}
uncertainty_coef(tbl, detail = TRUE)
```

## Yule's Q

Yule's Q is defined for 2x2 tables only. It ranges from -1 to +1 and
is equivalent to Gamma for 2x2 tables. The p-value tests H0: Q = 0
(Wald z-test).

```{r yule}
tbl_22 <- xtabs(~ smoking + physical_activity, data = sochealth)
yule_q(tbl_22, detail = TRUE)
```

## Automatic selection in cross_tab()

`cross_tab()` can automatically select an appropriate measure via
`assoc_measure = "auto"` (the default). When both variables are ordered
factors, it picks Kendall's Tau-b; otherwise it uses Cramer's V.

```{r cross-tab-auto}
# Nominal: Cramer's V
cross_tab(sochealth, smoking, education)

# Ordinal: Kendall's Tau-b (automatic)
cross_tab(sochealth, self_rated_health, education)
```

You can override the automatic choice:

```{r cross-tab-override}
cross_tab(sochealth, self_rated_health, education, assoc_measure = "gamma")
```

## Confidence intervals

All functions support confidence intervals via `detail = TRUE`. The
confidence level defaults to 95% and can be changed with `conf_level`:

```{r ci-level}
cramer_v(tbl, detail = TRUE, conf_level = 0.99)
```

To get only the estimate and p-value (no CI), pass `conf_level = NULL`:

```{r ci-null}
cramer_v(tbl, detail = TRUE, conf_level = NULL)
```

## Controlling decimal places

When `detail = FALSE` (the default), functions return a plain numeric
scalar, so R's own formatting rules apply. When `detail = TRUE`, the
result uses a custom print method that defaults to 3 decimal places.
Pass `digits` to change the precision of the estimate, SE, and CI
columns. The p-value follows APA-style formatting independent of
`digits`: 3 decimal places with the leading zero stripped (`.045`)
or `<.001` below the threshold:

```{r digits}
cramer_v(tbl, detail = TRUE, digits = 4)
```

The same `digits` argument works for `assoc_measures()`:

```{r digits-table}
assoc_measures(tbl, digits = 2)
```

You can also store a result and re-display it with a different
precision without recalculating:

```{r digits-print}
res <- cramer_v(tbl, detail = TRUE)
print(res, digits = 5)
```

---
title: "Frequency tables and cross-tabulations in R"
description: >
  Create frequency tables and cross-tabulations with chi-squared tests,
  effect sizes, and weighted counts in R using the spicy package.
  Covers sorting, percentages, grouping, and labelled data.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Frequency tables and cross-tabulations in R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(spicy)
```

`freq()` and `cross_tab()` are the core tabulation functions in spicy.
They handle factors, labelled variables (from haven or labelled),
weights, and missing values out of the box. This vignette covers the
main options using the bundled `sochealth` dataset.

## Frequency tables with freq()

### Basic usage

Pass a data frame and a variable name to get counts and percentages:

```{r freq-basic}
freq(sochealth, education)
```

### Sorting

Sort by frequency with `sort = "-"` (decreasing) or `sort = "+"`
(increasing). Sort alphabetically with `sort = "name+"` or
`sort = "name-"`:

```{r freq-sort}
freq(sochealth, education, sort = "-")
```

Sort alphabetically:

```{r freq-sort-name}
freq(sochealth, education, sort = "name+")
```

### Cumulative percentages

Add cumulative columns with `cum = TRUE`:

```{r freq-cum}
freq(sochealth, smoking, cum = TRUE)
```

### Weighted frequencies

Supply a weight variable with `weights`. By default, `rescale = TRUE`
adjusts the weighted total to match the unweighted sample size:

```{r freq-weighted}
freq(sochealth, education, weights = weight)
```

Set `rescale = FALSE` to keep the raw weighted counts:

```{r freq-weighted-raw}
freq(sochealth, education, weights = weight, rescale = FALSE)
```

### Labelled variables

When a variable has value labels (e.g., imported from SPSS or Stata
with haven), `freq()` shows them by default with the `[code] label`
format. Control this with `labelled_levels`:

```{r freq-labelled}
# Create a labelled version of the smoking variable
sh <- sochealth
sh$smoking_lbl <- labelled::labelled(
  ifelse(sh$smoking == "Yes", 1L, 0L),
  labels = c("Non-smoker" = 0L, "Current smoker" = 1L)
)

# Default: [code] label
freq(sh, smoking_lbl)

# Labels only (no codes)
freq(sh, smoking_lbl, labelled_levels = "labels")

# Codes only (no labels)
freq(sh, smoking_lbl, labelled_levels = "values")
```

### Custom missing values

Treat specific values as missing with `na_val`:

```{r freq-naval}
freq(sochealth, income_group, na_val = "High")
```

## Cross-tabulations with cross_tab()

### Basic two-way table

Cross two variables to get a contingency table with a chi-squared test
and effect size:

```{r cross-basic}
cross_tab(sochealth, smoking, education)
```

### Row and column percentages

Use `percent = "row"` or `percent = "col"` to display percentages
instead of raw counts:

```{r cross-col}
cross_tab(sochealth, smoking, education, percent = "col")
```

```{r cross-row}
cross_tab(sochealth, smoking, education, percent = "row")
```

### Grouping with by

Stratify the table by a third variable:

```{r cross-by}
cross_tab(sochealth, smoking, education, by = sex)
```

For more than one grouping variable, use `interaction()`:

```{r cross-by-interaction}
cross_tab(sochealth, smoking, education,
          by = interaction(sex, age_group))
```

### Ordinal variables

When both variables are ordered factors, `cross_tab()` automatically
switches from Cramer's V to Kendall's Tau-b:

```{r cross-ordinal}
cross_tab(sochealth, self_rated_health, education)
```

You can override the automatic selection with `assoc_measure`:

```{r cross-assoc}
cross_tab(sochealth, self_rated_health, education, assoc_measure = "gamma")
```

### Confidence intervals for effect sizes

Add a 95% confidence interval for the association measure with
`assoc_ci = TRUE`:

```{r cross-ci}
cross_tab(sochealth, smoking, education, assoc_ci = TRUE)
```

### Weighted cross-tabulations

Weights are supplied via `weights`, just like in `freq()`. The
default differs, however: `cross_tab()` defaults to
`rescale = FALSE` (raw weighted counts), whereas `freq()` defaults
to `rescale = TRUE`. Without rescaling, the table shows raw
weighted counts:

```{r cross-weighted-raw}
cross_tab(sochealth, smoking, education, weights = weight)
```

Pass `rescale = TRUE` to make the weighted total match the
unweighted sample size:

```{r cross-weighted}
cross_tab(sochealth, smoking, education, weights = weight, rescale = TRUE)
```

### Monte Carlo simulation

When expected cell counts are small, use simulated p-values:

```{r cross-simulate}
cross_tab(sochealth, smoking, education,
          simulate_p = TRUE, simulate_B = 5000)
```

### Data frame output

Set `styled = FALSE` to get a plain data frame for further processing:

```{r cross-df}
cross_tab(sochealth, smoking, education,
          percent = "col", styled = FALSE)
```

## Setting global defaults

You can set package-wide defaults with `options()` so you don't have to
repeat arguments:

```{r options, eval = FALSE}
options(
  spicy.percent   = "column",
  spicy.simulate_p = TRUE,
  spicy.rescale   = TRUE
)
```

## Learn more

- `vignette("association-measures")` - choosing the right effect size
  for your contingency table.
- `vignette("table-categorical")` - building publication-ready
  categorical tables.
- `?freq` and `?cross_tab` for the full argument reference.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freq.R
\name{freq}
\alias{freq}
\title{Frequency Table}
\usage{
freq(
  data,
  x = NULL,
  weights = NULL,
  digits = 1,
  valid = TRUE,
  cum = FALSE,
  sort = "",
  na_val = NULL,
  labelled_levels = c("prefixed", "labels", "values"),
  factor_levels = c("observed", "all"),
  rescale = TRUE,
  styled = TRUE,
  ...
)
}
\arguments{
\item{data}{A \code{data.frame}, vector, or factor. If a data frame is provided,
specify the target variable \code{x}. If both \code{data} and \code{x} are supplied as
vectors, \code{data} is ignored with a warning.}

\item{x}{A variable from \code{data} (unquoted).}

\item{weights}{Optional numeric vector of weights (same length as \code{x}).
The variable may be referenced as a bare name when it belongs to \code{data},
or as a qualified expression like \code{other$w} (evaluated in the calling
environment), which always takes precedence over \code{data} lookup. \code{NA}
weights are treated as zero with a warning; see \code{Details}.}

\item{digits}{Number of decimal digits to display for percentages (default: \code{1}).}

\item{valid}{Logical. If \code{TRUE} (default), display valid percentages
(excluding missing values).}

\item{cum}{Logical. If \code{FALSE} (the default), cumulative percentages are omitted.
If \code{TRUE}, adds cumulative percentages.}

\item{sort}{Sorting method for values:
\itemize{
\item \code{""} - no sorting (default)
\item \code{"+"} - increasing frequency
\item \code{"-"} - decreasing frequency
\item \code{"name+"} - alphabetical A-Z
\item \code{"name-"} - alphabetical Z-A
}}

\item{na_val}{Atomic vector of numeric or character values to be treated as missing (\code{NA}).

For \emph{labelled} variables (from \strong{haven} or \strong{labelled}), this argument
must refer to the underlying coded values, not the visible labels.

Example:

\if{html}{\out{<div class="sourceCode">}}\preformatted{x <- labelled(c(1, 2, 3, 1, 2, 3), c("Low" = 1, "Medium" = 2, "High" = 3))
freq(x, na_val = 1) # Treat all "Low" as missing
}\if{html}{\out{</div>}}}

\item{labelled_levels}{For \code{labelled} variables, defines how labels and
values are displayed:
\itemize{
\item \code{"prefixed"} or \code{"p"} - show labels as \verb{[value] label} (default)
\item \code{"labels"} or \code{"l"} - show only labels
\item \code{"values"} or \code{"v"} - show only numeric codes
}}

\item{factor_levels}{Character. Controls how factor and labelled values
are displayed in the frequency table. \code{"observed"} (the default;
matches Stata's \code{tab}) shows only levels present in the data.
\code{"all"} (matches SPSS \code{FREQUENCIES} and \code{\link[=code_book]{code_book()}}'s default)
keeps every declared level, including unused ones, which appear
with \code{n = 0}.}

\item{rescale}{Logical. If \code{TRUE} (default), rescale weights so that their
total equals the unweighted sample size (\code{length(weights)}). See
\code{Details} for the interaction with \code{NA} weights.}

\item{styled}{Logical. If \code{TRUE} (default), print the formatted spicy table.
If \code{FALSE}, return a plain \code{data.frame} with frequency values.}

\item{...}{Additional arguments passed to \code{\link[=print.spicy_freq_table]{print.spicy_freq_table()}}.}
}
\value{
With \code{styled = FALSE}, a plain \code{data.frame} with no extra attributes
and columns:
\itemize{
\item \code{value} - unique values or factor levels
\item \code{n} - frequency count (weighted if applicable)
\item \code{prop} - proportion of total
\item \code{valid_prop} - proportion of valid responses (if \code{valid = TRUE})
\item \code{cum_prop}, \code{cum_valid_prop} - cumulative percentages (if \code{cum = TRUE})
}

With \code{styled = TRUE} (default), prints the formatted table to the
console and invisibly returns a \code{spicy_freq_table} object: the same
\code{data.frame} carrying rendering metadata as attributes (\code{digits},
\code{data_name}, \code{var_name}, \code{var_label}, \code{class_name}, \code{n_total},
\code{n_valid}, \code{weighted}, \code{rescaled}, \code{weight_var}) used by
\code{\link[=print.spicy_freq_table]{print.spicy_freq_table()}}.
}
\description{
Creates a frequency table for a vector or variable from a data frame, with
options for weighting, sorting, handling \emph{labelled} data, defining custom
missing values, and displaying cumulative percentages.

When \code{styled = TRUE}, the function prints a spicy-formatted ASCII table
using \code{\link[=print.spicy_freq_table]{print.spicy_freq_table()}} and \code{\link[=spicy_print_table]{spicy_print_table()}}; otherwise, it
returns a \code{data.frame} containing frequencies and proportions.
}
\details{
This function is designed to mimic common frequency procedures from
statistical software such as SPSS or Stata, while integrating the
flexibility of R's data structures.

It automatically detects the type of input (\code{vector}, \code{factor}, or
\code{labelled}) and applies appropriate transformations, including:
\itemize{
\item Handling of labelled variables via \strong{labelled} or \strong{haven}
\item Optional recoding of specific values as missing (\code{na_val})
\item Optional weighting with a rescaling mechanism
\item Support for cumulative percentages (\code{cum = TRUE})
\item Multiple display modes for labels via \code{labelled_levels}
\item Schema-vs-observed level display via \code{factor_levels}
}

For factor and labelled inputs, the \code{factor_levels} argument
controls whether declared-but-unobserved levels appear in the
output. The default \code{"observed"} drops them (Stata \code{tab} behavior);
\code{"all"} keeps them with \code{n = 0}, matching SPSS \code{FREQUENCIES} and
\code{\link[=code_book]{code_book()}}'s default. For schema-level inspection without
computing frequencies, use \code{\link[=varlist]{varlist()}} or \code{\link[=code_book]{code_book()}} with
\code{factor_levels = "all"}.

When weighting is applied (\code{weights}), the frequencies and percentages are
computed proportionally to the weights. The argument \code{rescale = TRUE}
normalizes weights so their sum equals the unweighted sample size
(\code{length(weights)}).

Missing values in \code{weights} are treated as zero (with a warning), so
the corresponding rows contribute nothing to any cell. With
\code{rescale = TRUE}, the remaining weights are normalized so the total
weighted N still equals \code{length(weights)} — the implicit share of the
zeroed rows is redistributed over the others, mirroring Stata's
\code{pweight} semantics. With \code{rescale = FALSE}, the total weighted N is
the actual sum of non-\code{NA} weights.
}
\examples{
# Frequency table with labelled ordered factor
freq(sochealth, education)
freq(sochealth, self_rated_health, sort = "-")

library(labelled)

# Simple numeric vector
x <- c(1, 2, 2, 3, 3, 3, NA)
freq(x)

# Plain vector with a sentinel value recoded as missing
freq(c(1, 2, 3, 99, 99), na_val = 99)

# Labelled variable (haven-style)
x_lbl <- labelled(
  c(1, 2, 3, 1, 2, 3, 1, 2, NA),
  labels = c("Low" = 1, "Medium" = 2, "High" = 3)
)
var_label(x_lbl) <- "Satisfaction level"

# Treat value 1 ("Low") as missing
freq(x_lbl, na_val = 1)

# Display only labels, add cumulative \%
freq(x_lbl, labelled_levels = "labels", cum = TRUE)

# Display values only, sorted descending
freq(x_lbl, labelled_levels = "values", sort = "-")

# Show all declared factor levels, including unused ones (SPSS-style).
# The default "observed" mirrors Stata's `tab` and drops unused levels.
f <- factor(c("Yes", "No", "Yes"), levels = c("Yes", "No", "Maybe"))
freq(f, factor_levels = "all")

# With weighting
df <- data.frame(
  sex = factor(c("Male", "Female", "Female", "Male", NA, "Female")),
  weight = c(12, 8, 10, 15, 7, 9)
)

# Weighted frequencies (normalized)
freq(df, sex, weights = weight, rescale = TRUE)

# Weighted frequencies (without rescaling)
freq(df, sex, weights = weight, rescale = FALSE)

# Base R style, with weights and cumulative percentages
freq(df$sex, weights = df$weight, cum = TRUE)

# Piped version (tidy syntax) and sort alphabetically descending ("name-")
df |> freq(sex, sort = "name-")

# Non-styled return (for programmatic use)
f <- freq(df, sex, styled = FALSE)
head(f)

}
\seealso{
\code{\link[=print.spicy_freq_table]{print.spicy_freq_table()}} for formatted printing.
\code{\link[=spicy_print_table]{spicy_print_table()}} for the underlying ASCII rendering engine.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{somers_d}
\alias{somers_d}
\title{Somers' D}
\usage{
somers_d(
  x,
  direction = c("row", "column", "symmetric"),
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
)
}
\arguments{
\item{x}{A contingency table (of class \code{table}).}

\item{direction}{Direction of prediction:
\code{"row"} (default, column predicts row),
\code{"column"} (row predicts column),
or \code{"symmetric"} (average of both directions).}

\item{detail}{Logical. If \code{FALSE} (default), return the estimate
as a numeric scalar. If \code{TRUE}, return a named numeric vector
including confidence interval and p-value.}

\item{conf_level}{A number between 0 and 1 giving the confidence
level (default \code{0.95}). Only used when \code{detail = TRUE}. Set
to \code{NULL} to omit the confidence interval.}

\item{digits}{Number of decimal places used when printing the
result (default \code{3}). Only affects the \code{detail = TRUE} output.}

\item{.include_se}{Internal parameter; do not use.}
}
\value{
Same structure as \code{\link[=cramer_v]{cramer_v()}}: a scalar when
\code{detail = FALSE}, a named vector when \code{detail = TRUE}.
The p-value tests H0: D = 0 (Wald z-test).
}
\description{
\code{somers_d()} computes Somers' D for a two-way contingency
table of ordinal variables.
}
\details{
Somers' D is an asymmetric ordinal measure defined as
\eqn{d = (C - D) / (C + D + T)}, where \eqn{T} is the
number of pairs tied on the independent variable.
The symmetric version is the harmonic mean of the two
asymmetric values.
Standard error formulas follow the DescTools implementations
(Signorell et al., 2024); see \code{\link[=cramer_v]{cramer_v()}} for full references.
}
\examples{
tab <- table(sochealth$education, sochealth$self_rated_health)
somers_d(tab, direction = "row")
somers_d(tab, direction = "column", detail = TRUE)

}
\seealso{
\code{\link[=kendall_tau_b]{kendall_tau_b()}}, \code{\link[=gamma_gk]{gamma_gk()}}, \code{\link[=assoc_measures]{assoc_measures()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{uncertainty_coef}
\alias{uncertainty_coef}
\title{Uncertainty Coefficient}
\usage{
uncertainty_coef(
  x,
  direction = c("symmetric", "row", "column"),
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
)
}
\arguments{
\item{x}{A contingency table (of class \code{table}).}

\item{direction}{Direction of prediction:
\code{"symmetric"} (default), \code{"row"} (column predicts row),
or \code{"column"} (row predicts column).}

\item{detail}{Logical. If \code{FALSE} (default), return the estimate
as a numeric scalar. If \code{TRUE}, return a named numeric vector
including confidence interval and p-value.}

\item{conf_level}{A number between 0 and 1 giving the confidence
level (default \code{0.95}). Only used when \code{detail = TRUE}. Set
to \code{NULL} to omit the confidence interval.}

\item{digits}{Number of decimal places used when printing the
result (default \code{3}). Only affects the \code{detail = TRUE} output.}

\item{.include_se}{Internal parameter; do not use.}
}
\value{
Same structure as \code{\link[=cramer_v]{cramer_v()}}: a scalar when
\code{detail = FALSE}, a named vector when \code{detail = TRUE}.
The p-value tests H0: U = 0 (Wald z-test).
}
\description{
\code{uncertainty_coef()} computes the Uncertainty Coefficient
(Theil's U) for a two-way contingency table, based on
information entropy.
}
\details{
The uncertainty coefficient measures association using
Shannon entropy.
For \code{direction = "row"}:
\eqn{U = (H_X + H_Y - H_{XY}) / H_X}, where \eqn{H_X},
\eqn{H_Y} are the marginal entropies and \eqn{H_{XY}} is
the joint entropy.
The symmetric version is
\eqn{U = 2 (H_X + H_Y - H_{XY}) / (H_X + H_Y)}.
Standard error formulas follow the DescTools implementations
(Signorell et al., 2024); see \code{\link[=cramer_v]{cramer_v()}} for full references.
}
\examples{
tab <- table(sochealth$smoking, sochealth$education)
uncertainty_coef(tab)
uncertainty_coef(tab, direction = "row", detail = TRUE)

}
\seealso{
\code{\link[=lambda_gk]{lambda_gk()}}, \code{\link[=goodman_kruskal_tau]{goodman_kruskal_tau()}}, \code{\link[=assoc_measures]{assoc_measures()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/copy_clipboard.R
\name{copy_clipboard}
\alias{copy_clipboard}
\title{Copy data to the clipboard}
\usage{
copy_clipboard(
  x,
  row.names.as.col = FALSE,
  row.names = TRUE,
  col.names = TRUE,
  show_message = TRUE,
  quiet = FALSE,
  ...
)
}
\arguments{
\item{x}{A data frame, matrix, 2D array, 3D array, table, or atomic vector to be copied.}

\item{row.names.as.col}{Logical or character. If \code{FALSE} (the default), row names are not added as a column. If \code{TRUE}, a column named \code{"rownames"} is prepended. If a character string is supplied, it is used as the column name for row names.}

\item{row.names}{Logical. If \code{TRUE} (the default), includes row names in the clipboard output. If \code{FALSE}, row names are omitted.}

\item{col.names}{Logical. If \code{TRUE} (the default), includes column names in the clipboard output. If \code{FALSE}, column names are omitted.}

\item{show_message}{Logical. If \code{TRUE} (the default), displays a success message after copying. If \code{FALSE}, no success message is printed.}

\item{quiet}{Logical. If \code{FALSE} (the default), messages are shown. If \code{TRUE}, suppresses all messages, including success, coercion notices, and warnings.}

\item{...}{Additional arguments passed to \code{\link[clipr:write_clip]{clipr::write_clip()}}.}
}
\value{
Invisibly returns the object \code{x}. The main purpose is the side effect of copying data to the clipboard.
}
\description{
\code{copy_clipboard()} copies a data frame, matrix, array (2D or higher), table or vector to the clipboard.
You can paste the result into a text editor (e.g. Notepad++, Sublime Text), a spreadsheet (e.g. Excel, LibreOffice Calc), or a word processor (e.g. Word).
}
\details{
Note: Objects that are not data frames or 2D matrices (e.g. atomic vectors, arrays, tables) are automatically converted to character
when copied to the clipboard, as required by \code{\link[clipr:write_clip]{clipr::write_clip()}}. The original object in R remains unchanged.

For multidimensional arrays (e.g. 3D arrays), the entire array is flattened into a 1D character vector, with each element on a new line.
To preserve a tabular structure, you should extract a 2D slice before copying. For example: \code{copy_clipboard(my_array[, , 1])}.
}
\examples{
\donttest{
if (clipr::clipr_available()) {
  # Data frame
  copy_clipboard(sochealth)

  # Data frame with row names as column
  copy_clipboard(head(sochealth), row.names.as.col = "id")

  # Matrix
  mat <- matrix(1:6, nrow = 2)
  copy_clipboard(mat)

  # Table
  tbl <- table(sochealth$education)
  copy_clipboard(tbl)

  # Array (3D) — flattened to character
  arr <- array(1:8, dim = c(2, 2, 2))
  copy_clipboard(arr)

  # Recommended: copy 2D slice for tabular layout
  copy_clipboard(arr[, , 1])

  # Numeric vector
  copy_clipboard(c(3.14, 2.71, 1.618))

  # Character vector
  copy_clipboard(c("apple", "banana", "cherry"))

  # Quiet mode (no messages shown)
  copy_clipboard(sochealth, quiet = TRUE)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{gamma_gk}
\alias{gamma_gk}
\title{Goodman-Kruskal Gamma}
\usage{
gamma_gk(
  x,
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
)
}
\arguments{
\item{x}{A contingency table (of class \code{table}).}

\item{detail}{Logical. If \code{FALSE} (default), return the estimate
as a numeric scalar. If \code{TRUE}, return a named numeric vector
including confidence interval and p-value.}

\item{conf_level}{A number between 0 and 1 giving the confidence
level (default \code{0.95}). Only used when \code{detail = TRUE}. Set
to \code{NULL} to omit the confidence interval.}

\item{digits}{Number of decimal places used when printing the
result (default \code{3}). Only affects the \code{detail = TRUE} output.}

\item{.include_se}{Internal parameter; do not use.}
}
\value{
Same structure as \code{\link[=cramer_v]{cramer_v()}}: a scalar when
\code{detail = FALSE}, a named vector when \code{detail = TRUE}.
The p-value tests H0: gamma = 0 (Wald z-test).
}
\description{
\code{gamma_gk()} computes the Goodman-Kruskal Gamma statistic for a
two-way contingency table of ordinal variables.
}
\details{
Gamma is computed as \eqn{\gamma = (C - D) / (C + D)}, where
\eqn{C} and \eqn{D} are the numbers of concordant and
discordant pairs. It ignores tied pairs, making it appropriate
for ordinal variables with many ties.
Standard error formulas follow the DescTools implementations
(Signorell et al., 2024); see \code{\link[=cramer_v]{cramer_v()}} for full references.
}
\examples{
tab <- table(sochealth$education, sochealth$self_rated_health)
gamma_gk(tab)
gamma_gk(tab, detail = TRUE)

}
\seealso{
\code{\link[=kendall_tau_b]{kendall_tau_b()}}, \code{\link[=kendall_tau_c]{kendall_tau_c()}}, \code{\link[=somers_d]{somers_d()}},
\code{\link[=assoc_measures]{assoc_measures()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freq_print.R
\name{print.spicy_freq_table}
\alias{print.spicy_freq_table}
\title{Styled print method for \code{freq()} tables}
\usage{
\method{print}{spicy_freq_table}(x, ...)
}
\arguments{
\item{x}{A \code{data.frame} returned by \code{\link[=freq]{freq()}} with attached attributes:
\itemize{
\item \code{"digits"}: number of decimal digits to display
\item \code{"data_name"}: name of the source dataset
\item \code{"var_name"}: name of the variable
\item \code{"var_label"}: variable label, if defined
\item \code{"class_name"}: original class of the variable
\item \code{"weighted"}, \code{"rescaled"}, \code{"weight_var"}: weighting metadata
}}

\item{...}{Additional arguments (ignored, required for S3 method compatibility)}
}
\value{
Invisibly returns \code{x} after printing the formatted table.
}
\description{
Internal print method used by \code{\link[=freq]{freq()}} to display a styled, spicy-formatted
frequency table in the console.
It formats valid, missing, and total rows; handles cumulative and valid
percentages; and appends a labeled footer including metadata such as
variable label, class, dataset name, and weighting information.
}
\details{
This function is part of the \emph{spicy table rendering engine}.
It is automatically called when printing the result of \code{\link[=freq]{freq()}} with
\code{styled = TRUE}.
The output uses \code{\link[=spicy_print_table]{spicy_print_table()}} internally to render a colorized ASCII
table with consistent alignment and separators.

The printed table includes:
\itemize{
\item Valid and missing value sections (if applicable)
\item Optional cumulative and valid percentages
\item A final 'Total' row shown in the \strong{Category} column
\item A footer summarizing metadata (variable label, data source, weights)
}
}
\section{Output structure}{

The printed table includes the following columns:
\itemize{
\item \strong{Category}: Sections such as "Valid", "Missing", and "Total"
\item \strong{Values}: Observed categories or levels
\item \strong{Freq.}: Frequency count (weighted if applicable)
\item \strong{Percent}: Percentage of total
\item \strong{Valid Percent}: Percentage among valid values (optional)
\item \strong{Cum. Percent}: Cumulative percentage (optional)
\item \strong{Cum. Valid Percent}: Cumulative valid percentage (optional)
}
}

\examples{
# Example using labelled data
library(labelled)
x <- labelled(
  c(1, 2, 3, 1, 2, 3, 1, 2, NA),
  labels = c("Low" = 1, "Medium" = 2, "High" = 3)
)
var_label(x) <- "Satisfaction level"
# Capture result without printing, then print explicitly
df <- spicy::freq(x, styled = FALSE)
print(df) # dispatches to print.spicy_freq_table()

}
\seealso{
\code{\link[=freq]{freq()}} for the main frequency table generator.
\code{\link[=spicy_print_table]{spicy_print_table()}} for the generic ASCII table renderer.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{sochealth}
\alias{sochealth}
\title{Simulated social-health survey}
\format{
A tibble with 1200 rows and 24 variables:
\describe{
\item{sex}{Factor. Sex of the respondent.}
\item{age}{Numeric. Age in years (25--75).}
\item{age_group}{Ordered factor. Age group
(25--34, 35--49, 50--64, 65--75).}
\item{education}{Ordered factor. Highest education level
(Lower secondary, Upper secondary, Tertiary).}
\item{social_class}{Ordered factor. Subjective social class
(Lower, Working, Lower middle, Middle, Upper middle).}
\item{region}{Factor. Region of residence (6 regions).}
\item{employment_status}{Factor. Employment status
(Employed, Student, Unemployed, Inactive).}
\item{income_group}{Ordered factor. Household income group
(Low, Lower middle, Upper middle, High). Contains missing
values.}
\item{income}{Numeric. Monthly household income in CHF.}
\item{smoking}{Factor. Current smoker (No, Yes). Contains
missing values.}
\item{physical_activity}{Factor. Regular physical activity
(No, Yes).}
\item{dentist_12m}{Factor. Dentist visit in the last 12 months
(No, Yes).}
\item{self_rated_health}{Ordered factor. Self-rated health
(Poor, Fair, Good, Very good). Contains missing values.}
\item{wellbeing_score}{Numeric. WHO-5 wellbeing index
(0--100).}
\item{bmi}{Numeric. Body mass index. Contains missing values.}
\item{bmi_category}{Ordered factor. BMI category
(Normal weight, Overweight, Obesity). Contains missing values.}
\item{institutional_trust}{Ordered factor. Trust in institutions
(Very low, Low, High, Very high).}
\item{political_position}{Numeric. Political position on a
0 (left) to 10 (right) scale. Contains missing values.}
\item{life_sat_health}{Integer. Satisfaction with own health
(1--5 Likert scale). Contains missing values.}
\item{life_sat_work}{Integer. Satisfaction with work or main
activity (1--5 Likert scale). Contains missing values.}
\item{life_sat_relationships}{Integer. Satisfaction with
personal relationships (1--5 Likert scale). Contains missing
values.}
\item{life_sat_standard}{Integer. Satisfaction with standard
of living (1--5 Likert scale). Contains missing values.}
\item{response_date}{POSIXct. Date and time of survey response
(September--November 2024).}
\item{weight}{Numeric. Survey design weight.}
}
}
\source{
Simulated data for illustration purposes.
}
\usage{
sochealth
}
\description{
A simulated dataset of 1200 respondents from a fictional
social-health survey, designed to illustrate the main features of
the spicy package: variable labels, ordered factors, survey
weights, association measures, and APA-style reporting.
}
\details{
All variables carry labels (accessible via \code{\link[labelled:var_label]{labelled::var_label()}}
and displayed by \code{\link[=varlist]{varlist()}}). Several ordered factors are included
so that \code{\link[=cross_tab]{cross_tab()}} can demonstrate automatic ordinal measure
selection.
}
\examples{
data(sochealth)
varlist(sochealth)
freq(sochealth, education)
cross_tab(sochealth, education, self_rated_health)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{print.spicy_assoc_table}
\alias{print.spicy_assoc_table}
\title{Print an association measures summary table}
\usage{
\method{print}{spicy_assoc_table}(x, digits = attr(x, "digits") \%||\% 3L, ...)
}
\arguments{
\item{x}{A \code{spicy_assoc_table} object.}

\item{digits}{Number of decimal places for estimates, SE, and
confidence intervals. Defaults to 3. The p-value is always
formatted separately (\verb{< 0.001} or three decimal places).}

\item{...}{Ignored.}
}
\value{
\code{x}, invisibly.
}
\description{
Formats a \code{spicy_assoc_table} data frame (returned by
\code{\link[=assoc_measures]{assoc_measures()}}) with fixed decimal places, aligned columns,
and \verb{< 0.001} notation for small p-values.
}
\seealso{
\code{\link[=assoc_measures]{assoc_measures()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{lambda_gk}
\alias{lambda_gk}
\title{Goodman-Kruskal's Lambda}
\usage{
lambda_gk(
  x,
  direction = c("symmetric", "row", "column"),
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
)
}
\arguments{
\item{x}{A contingency table (of class \code{table}).}

\item{direction}{Direction of prediction:
\code{"symmetric"} (default), \code{"row"} (column predicts row),
or \code{"column"} (row predicts column).}

\item{detail}{Logical. If \code{FALSE} (default), return the estimate
as a numeric scalar. If \code{TRUE}, return a named numeric vector
including confidence interval and p-value.}

\item{conf_level}{A number between 0 and 1 giving the confidence
level (default \code{0.95}). Only used when \code{detail = TRUE}. Set
to \code{NULL} to omit the confidence interval.}

\item{digits}{Number of decimal places used when printing the
result (default \code{3}). Only affects the \code{detail = TRUE} output.}

\item{.include_se}{Internal parameter; do not use.}
}
\value{
Same structure as \code{\link[=cramer_v]{cramer_v()}}: a scalar when
\code{detail = FALSE}, a named vector when \code{detail = TRUE}.
The p-value tests H0: lambda = 0 (Wald z-test).
}
\description{
\code{lambda_gk()} computes Goodman-Kruskal's Lambda, a proportional
reduction in error (PRE) measure for nominal variables.
}
\details{
Lambda measures how much prediction error is reduced when
the independent variable is used to predict the dependent
variable. It ranges from 0 (no reduction) to 1 (perfect
prediction). Lambda can equal zero even when variables
are associated if the modal category dominates in every
column (or row).
Standard error formulas follow the DescTools implementations
(Signorell et al., 2024); see \code{\link[=cramer_v]{cramer_v()}} for full references.
}
\examples{
tab <- table(sochealth$smoking, sochealth$education)
lambda_gk(tab)
lambda_gk(tab, direction = "row")
lambda_gk(tab, direction = "column", detail = TRUE)

}
\seealso{
\code{\link[=goodman_kruskal_tau]{goodman_kruskal_tau()}}, \code{\link[=uncertainty_coef]{uncertainty_coef()}},
\code{\link[=assoc_measures]{assoc_measures()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{phi}
\alias{phi}
\title{Phi coefficient}
\usage{
phi(x, detail = FALSE, conf_level = 0.95, digits = 3L, .include_se = FALSE)
}
\arguments{
\item{x}{A contingency table (of class \code{table}).}

\item{detail}{Logical. If \code{FALSE} (default), return the estimate
as a numeric scalar. If \code{TRUE}, return a named numeric vector
including confidence interval and p-value.}

\item{conf_level}{A number between 0 and 1 giving the confidence
level (default \code{0.95}). Only used when \code{detail = TRUE}. Set
to \code{NULL} to omit the confidence interval.}

\item{digits}{Number of decimal places used when printing the
result (default \code{3}). Only affects the \code{detail = TRUE} output.}

\item{.include_se}{Internal parameter; do not use.}
}
\value{
Same structure as \code{\link[=cramer_v]{cramer_v()}}: a scalar when
\code{detail = FALSE}, a named vector when \code{detail = TRUE}.
The p-value tests the null hypothesis of no association
(Pearson chi-squared test).
}
\description{
\code{phi()} computes the phi coefficient for a 2x2 contingency table.
}
\details{
The phi coefficient is \eqn{\phi = \sqrt{\chi^2 / n}}.
It is equivalent to Cramer's V for 2x2 tables and equals the
Pearson correlation between the two binary variables.
The confidence interval uses the Fisher z-transformation.
Standard error formulas follow the DescTools implementations
(Signorell et al., 2024); see \code{\link[=cramer_v]{cramer_v()}} for full references.
}
\examples{
tab <- table(sochealth$smoking, sochealth$sex)
phi(tab)
phi(tab, detail = TRUE)

}
\seealso{
\code{\link[=cramer_v]{cramer_v()}}, \code{\link[=yule_q]{yule_q()}}, \code{\link[=assoc_measures]{assoc_measures()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_categorical.R
\name{table_categorical}
\alias{table_categorical}
\title{Categorical summary table}
\usage{
table_categorical(
  data,
  select,
  by = NULL,
  labels = NULL,
  levels_keep = NULL,
  include_total = TRUE,
  drop_na = TRUE,
  weights = NULL,
  rescale = FALSE,
  correct = FALSE,
  simulate_p = FALSE,
  simulate_B = 2000,
  percent_digits = 1,
  p_digits = 3,
  v_digits = 2,
  assoc_measure = "auto",
  assoc_ci = FALSE,
  decimal_mark = ".",
  output = c("default", "data.frame", "long", "tinytable", "gt", "flextable", "excel",
    "clipboard", "word"),
  indent_text = "  ",
  indent_text_excel_clipboard = strrep(" ", 6),
  add_multilevel_header = TRUE,
  blank_na_wide = FALSE,
  excel_path = NULL,
  excel_sheet = "Categorical",
  clipboard_delim = "\\t",
  word_path = NULL
)
}
\arguments{
\item{data}{A data frame.}

\item{select}{Columns to include as row variables. Supports tidyselect
syntax and character vectors of column names.}

\item{by}{Optional grouping column used for columns/groups. Accepts an
unquoted column name or a single character column name.}

\item{labels}{An optional character vector of display labels for the
variables named in \code{select} (must be the same length and in the same
order). When \code{NULL} (the default), column names are used as-is.}

\item{levels_keep}{Optional character vector of levels to keep/order for row
modalities. If \code{NULL}, all observed levels are kept.}

\item{include_total}{Logical. If \code{TRUE} (the default), includes a \code{Total} group
when available.}

\item{drop_na}{Logical. If \code{TRUE} (the default), removes rows with \code{NA} in the
row/group variable before each cross-tabulation. If \code{FALSE}, missing values
are displayed as a dedicated \code{"(Missing)"} level.}

\item{weights}{Optional weights. Either \code{NULL} (the default), a numeric vector
of length \code{nrow(data)}, or a single column in \code{data} supplied as an
unquoted name or a character string.}

\item{rescale}{Logical. If \code{FALSE} (the default), weights are used as-is.
If \code{TRUE}, rescales weights so total weighted N matches raw N.
Passed to \code{spicy::cross_tab()}.}

\item{correct}{Logical. If \code{FALSE} (the default), no continuity correction is
applied. If \code{TRUE}, applies Yates correction in 2x2 chi-squared contexts.
Passed to \code{spicy::cross_tab()}.}

\item{simulate_p}{Logical. If \code{FALSE} (the default), uses asymptotic p-values.
If \code{TRUE}, uses Monte Carlo simulation. Passed to \code{spicy::cross_tab()}.}

\item{simulate_B}{Integer. Number of Monte Carlo replicates when
\code{simulate_p = TRUE}. Defaults to \code{2000}.}

\item{percent_digits}{Number of digits for percentages in report outputs.
Defaults to \code{1}.}

\item{p_digits}{Number of digits for p-values (except \verb{< .001}).
Defaults to \code{3}.}

\item{v_digits}{Number of digits for the association measure. Defaults
to \code{2}.}

\item{assoc_measure}{Passed to \code{\link[=cross_tab]{cross_tab()}}. Which association measure
to report (\code{"auto"}, \code{"cramer_v"}, \code{"phi"}, \code{"gamma"}, \code{"tau_b"},
\code{"tau_c"}, \code{"somers_d"}, \code{"lambda"}, \code{"none"}). Defaults to \code{"auto"}.}

\item{assoc_ci}{Passed to \code{\link[=cross_tab]{cross_tab()}}. If \code{TRUE}, includes the
confidence interval of the association measure. In data formats
(\code{"data.frame"}, \code{"long"}, \code{"excel"}, \code{"clipboard"}), two extra
columns \verb{CI lower} and \verb{CI upper} are added.
In rendered formats (\code{"gt"}, \code{"tinytable"}, \code{"flextable"}, \code{"word"}),
the CI is shown inline (e.g., \code{.14 [.08, .19]}).
Defaults to \code{FALSE}.}

\item{decimal_mark}{Decimal separator (\code{"."} or \code{","}). Defaults to \code{"."}.}

\item{output}{Output format. One of:
\itemize{
\item \code{"default"} (a printed ASCII table, returned invisibly)
\item \code{"data.frame"} (a wide numeric \code{data.frame})
\item \code{"long"} (a long numeric \code{data.frame})
\item \code{"tinytable"} (requires \code{tinytable})
\item \code{"gt"} (requires \code{gt})
\item \code{"flextable"} (requires \code{flextable})
\item \code{"excel"} (requires \code{openxlsx2})
\item \code{"clipboard"} (requires \code{clipr})
\item \code{"word"} (requires \code{flextable} and \code{officer})
}}

\item{indent_text}{Prefix used for modality labels in report table building.
Defaults to \code{"  "} (two spaces).}

\item{indent_text_excel_clipboard}{Stronger indentation used in Excel and
clipboard exports. Defaults to six non-breaking spaces.}

\item{add_multilevel_header}{Logical. If \code{TRUE} (the default), merges top
headers in Excel export.}

\item{blank_na_wide}{Logical. If \code{FALSE} (the default), \code{NA} values are kept
as-is in wide raw output. If \code{TRUE}, replaces them with empty strings.}

\item{excel_path}{Path for \code{output = "excel"}. Defaults to \code{NULL}.}

\item{excel_sheet}{Sheet name for Excel export. Defaults to \code{"Categorical"}.}

\item{clipboard_delim}{Delimiter for clipboard text export. Defaults to \code{"\\t"}.}

\item{word_path}{Path for \code{output = "word"} or optional save path when
\code{output = "flextable"}. Defaults to \code{NULL}.}
}
\value{
Depends on \code{output}:
\itemize{
\item \code{"default"}: prints a styled ASCII table and returns the
underlying \code{data.frame} invisibly (S3 class
\code{"spicy_categorical_table"}).
\item \code{"data.frame"}: a wide \code{data.frame} with one row per
variable--level combination.
When \code{by} is used, the columns are \code{Variable}, \code{Level}, and one
pair of \code{n} / \verb{\\\%} columns per group level (plus \code{Total} when
\code{include_total = TRUE}), followed by \code{Chi2}, \code{df}, \code{p}, and the
association measure column.
When \code{by = NULL}, the columns are \code{Variable}, \code{Level}, \code{n}, \verb{\\\%}.
\item \code{"long"}: a long \code{data.frame} with columns \code{variable},
\code{level}, \code{group}, \code{n}, \code{percent} (and \code{chi2}, \code{df}, \code{p},
association measure columns when \code{by} is used).
\item \code{"tinytable"}: a \code{tinytable} object.
\item \code{"gt"}: a \code{gt_tbl} object.
\item \code{"flextable"}: a \code{flextable} object.
\item \code{"excel"} / \code{"word"}: writes to disk and returns the file
path invisibly.
\item \code{"clipboard"}: copies the table and returns the display
\code{data.frame} invisibly.
}
}
\description{
Builds a publication-ready frequency or cross-tabulation table for one
or many categorical variables selected with tidyselect syntax.

With \code{by}, produces grouped cross-tabulation summaries (using
\code{\link[=cross_tab]{cross_tab()}} internally) with Chi-squared \emph{p}-values and optional
association measures.
Without \code{by}, produces one-way frequency-style summaries.

Multiple output formats are available via \code{output}: a printed ASCII
table (\code{"default"}), a wide or long numeric \code{data.frame}
(\code{"data.frame"}, \code{"long"}), or publication-ready tables
(\code{"tinytable"}, \code{"gt"}, \code{"flextable"}, \code{"excel"}, \code{"clipboard"},
\code{"word"}).
}
\details{
When \code{by} is used, each selected variable is cross-tabulated against
the grouping variable with \code{\link[=cross_tab]{cross_tab()}}. Chi-squared statistics,
\emph{p}-values, and the chosen association measure are reported for each
variable.

Optional output engines require suggested packages:
\itemize{
\item \pkg{tinytable} for \code{output = "tinytable"}
\item \pkg{gt} for \code{output = "gt"}
\item \pkg{flextable} for \code{output = "flextable"}
\item \pkg{flextable} + \pkg{officer} for \code{output = "word"}
\item \pkg{openxlsx2} for \code{output = "excel"}
\item \pkg{clipr} for \code{output = "clipboard"}
}
}
\examples{
# Long numeric output
table_categorical(
  data = sochealth,
  select = c(smoking, physical_activity),
  by = education,
  labels = c("Current smoker", "Physical activity"),
  output = "long"
)

# ASCII console output (default)
table_categorical(
  data = sochealth,
  select = c(smoking, physical_activity),
  by = sex
)

# One-way frequency-style table
table_categorical(
  data = sochealth,
  select = c(smoking, physical_activity)
)

# Wide numeric data.frame
table_categorical(
  data = sochealth,
  select = c(smoking, physical_activity),
  by = education,
  labels = c("Current smoker", "Physical activity"),
  output = "data.frame"
)

# Weighted example
table_categorical(
  data = sochealth,
  select = c(smoking, physical_activity),
  by = education,
  labels = c("Current smoker", "Physical activity"),
  weights = "weight",
  rescale = TRUE,
  simulate_p = FALSE,
  output = "long"
)

\donttest{
# Optional output: tinytable
if (requireNamespace("tinytable", quietly = TRUE)) {
  table_categorical(
    data = sochealth,
    select = c(smoking, physical_activity),
    by = sex,
    labels = c("Current smoker", "Physical activity"),
    output = "tinytable"
  )
}

# Optional output: Excel
if (requireNamespace("openxlsx2", quietly = TRUE)) {
  table_categorical(
    data = sochealth,
    select = c(smoking, physical_activity),
    by = education,
    labels = c("Current smoker", "Physical activity"),
    output = "excel",
    excel_path = tempfile(fileext = ".xlsx")
  )
}
}

}
\seealso{
\code{\link[=table_continuous]{table_continuous()}} for continuous variables;
\code{\link[=cross_tab]{cross_tab()}} for two-way cross-tabulations; \code{\link[=freq]{freq()}} for one-way
frequency tables.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cross_tab.R
\name{print.spicy_cross_table}
\alias{print.spicy_cross_table}
\title{Print method for spicy_cross_table objects}
\usage{
\method{print}{spicy_cross_table}(x, digits = NULL, ...)
}
\arguments{
\item{x}{A \code{spicy_cross_table} object.}

\item{digits}{Optional integer; number of decimal places to display.
Defaults to the value stored in the object.}

\item{...}{Additional arguments passed to internal formatting functions.}
}
\description{
Prints a formatted SPSS-like crosstable created by \code{\link[=cross_tab]{cross_tab()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{cramer_v}
\alias{cramer_v}
\title{Cramer's V}
\usage{
cramer_v(
  x,
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
)
}
\arguments{
\item{x}{A contingency table (of class \code{table}).}

\item{detail}{Logical. If \code{FALSE} (default), return the estimate
as a numeric scalar. If \code{TRUE}, return a named numeric vector
including confidence interval and p-value.}

\item{conf_level}{A number between 0 and 1 giving the confidence
level (default \code{0.95}). Only used when \code{detail = TRUE}. Set
to \code{NULL} to omit the confidence interval.}

\item{digits}{Number of decimal places used when printing the
result (default \code{3}). Only affects the \code{detail = TRUE} output.}

\item{.include_se}{Internal parameter; do not use.}
}
\value{
When \code{detail = FALSE}: a single numeric value (the
estimate).
When \code{detail = TRUE} and \code{conf_level} is non-\code{NULL}:
\code{c(estimate, ci_lower, ci_upper, p_value)}.
When \code{detail = TRUE} and \code{conf_level = NULL}:
\code{c(estimate, p_value)}.
The p-value tests the null hypothesis of no association
(Pearson chi-squared test).
}
\description{
\code{cramer_v()} computes Cramer's V for a two-way contingency table,
measuring the strength of association between two categorical variables.
}
\details{
Cramer's V is computed as
\eqn{V = \sqrt{\chi^2 / (n \cdot (k - 1))}}, where \eqn{\chi^2}
is the Pearson chi-squared statistic, \eqn{n} is the total count,
and \eqn{k = \min(r, c)}.
The confidence interval uses the Fisher z-transformation.
Standard error formulas follow the DescTools implementations
(Signorell et al., 2024).
}
\examples{
tab <- table(sochealth$smoking, sochealth$education)
cramer_v(tab)
cramer_v(tab, detail = TRUE)
cramer_v(tab, detail = TRUE, conf_level = NULL)

}
\references{
Agresti, A. (2002). \emph{Categorical Data Analysis} (2nd ed.). Wiley.

Liebetrau, A. M. (1983). \emph{Measures of Association}. Sage.

Signorell, A. et al. (2024). \emph{DescTools: Tools for Descriptive
Statistics}. R package.
}
\seealso{
\code{\link[=phi]{phi()}}, \code{\link[=contingency_coef]{contingency_coef()}}, \code{\link[=assoc_measures]{assoc_measures()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tables_ascii.R
\name{spicy_print_table}
\alias{spicy_print_table}
\title{Print a spicy-formatted ASCII table}
\usage{
spicy_print_table(
  x,
  title = attr(x, "title"),
  note = attr(x, "note"),
  padding = c("compact", "normal", "wide"),
  first_column_line = TRUE,
  row_total_line = TRUE,
  column_total_line = TRUE,
  bottom_line = FALSE,
  lines_color = "darkgrey",
  align_left_cols = NULL,
  align_center_cols = integer(0),
  group_sep_rows = integer(0),
  ...
)
}
\arguments{
\item{x}{A \code{spicy_table} or \code{data.frame} to be printed.}

\item{title}{Optional title displayed above the table. Defaults to the
\code{"title"} attribute of \code{x} if present.}

\item{note}{Optional note displayed below the table. Defaults to the \code{"note"}
attribute of \code{x} if present.}

\item{padding}{Character string controlling horizontal spacing between columns:
\itemize{
\item \code{"compact"} - minimal spacing
\item \code{"normal"} - moderate spacing (default)
\item \code{"wide"} - extra spacing (for wide displays)
}}

\item{first_column_line}{Logical. If \code{TRUE} (the default), adds a vertical separator
after the first column.}

\item{row_total_line, column_total_line, bottom_line}{Logical flags controlling
the presence of horizontal lines before total rows/columns or at the bottom
of the table.
Both \code{row_total_line} and \code{column_total_line} default to \code{TRUE};
\code{bottom_line} defaults to \code{FALSE}.}

\item{lines_color}{Character. Color for table separators. Defaults to \code{"darkgrey"}.
Only applied if the output supports ANSI colors (see \code{\link[crayon:has_color]{crayon::has_color()}}).}

\item{align_left_cols}{Integer vector of column indices to left-align.
If \code{NULL} (the default), alignment is auto-detected based on \code{x}:
\itemize{
\item For \code{freq} tables -> \code{c(1, 2)}
\item For \code{cross} tables -> \code{1}
}}

\item{align_center_cols}{Integer vector of column indices to
center-align. Defaults to \code{integer(0)}.}

\item{group_sep_rows}{Integer vector of row indices before which a
light dashed separator line is drawn. Defaults to \code{integer(0)}.}

\item{...}{Additional arguments passed to \code{\link[=build_ascii_table]{build_ascii_table()}}.}
}
\value{
Invisibly returns \code{x}, after printing the formatted ASCII table to the console.
}
\description{
User-facing helper that prints a visually aligned, spicy-styled ASCII table
created by functions such as \code{\link[=freq]{freq()}} or \code{\link[=cross_tab]{cross_tab()}}.
It automatically adjusts column alignment, spacing, and separators for
improved readability in console outputs.

This function wraps the internal renderer \code{\link[=build_ascii_table]{build_ascii_table()}}, adding
optional titles, notes, and automatic alignment rules depending on the type
of table.
}
\details{
\code{spicy_print_table()} detects whether the table represents frequencies
(\code{freq}-style) or cross-tabulations (\code{cross}-style) and adjusts formatting
accordingly:
\itemize{
\item For \strong{frequency tables}, the first two columns (\emph{Category} and \emph{Values})
are left-aligned.
\item For \strong{cross tables}, only the first column (row variable) is left-aligned.
}

The function supports Unicode line-drawing characters and colored separators
using the \strong{crayon} package, with graceful fallback to monochrome output when
color is not supported.
If the table exceeds the console width, it is split into stacked horizontal
panels while repeating the left-most identifier columns.
}
\examples{
# Simple demonstration
df <- data.frame(
  Category = c("Valid", "", "Missing", "Total"),
  Values = c("Yes", "No", "NA", ""),
  Freq. = c(12, 8, 1, 21),
  Percent = c(57.1, 38.1, 4.8, 100.0)
)

spicy_print_table(df,
  title = "Frequency table: Example",
  note = "Class: data.frame\nData: demo"
)

}
\seealso{
\code{\link[=build_ascii_table]{build_ascii_table()}} for the underlying text rendering engine.
\code{\link[=print.spicy_freq_table]{print.spicy_freq_table()}} for the specialized printing method used by \code{\link[=freq]{freq()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{contingency_coef}
\alias{contingency_coef}
\title{Pearson's contingency coefficient}
\usage{
contingency_coef(
  x,
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
)
}
\arguments{
\item{x}{A contingency table (of class \code{table}).}

\item{detail}{Logical. If \code{FALSE} (default), return the estimate
as a numeric scalar. If \code{TRUE}, return a named numeric vector
including confidence interval and p-value.}

\item{conf_level}{A number between 0 and 1 giving the confidence
level (default \code{0.95}). Only used when \code{detail = TRUE}. Set
to \code{NULL} to omit the confidence interval.}

\item{digits}{Number of decimal places used when printing the
result (default \code{3}). Only affects the \code{detail = TRUE} output.}

\item{.include_se}{Internal parameter; do not use.}
}
\value{
Same structure as \code{\link[=cramer_v]{cramer_v()}}: a scalar when
\code{detail = FALSE}, a named vector when \code{detail = TRUE}.
The p-value tests the null hypothesis of no association
(Pearson chi-squared test). CI values are \code{NA} because no
standard asymptotic SE exists for C.
}
\description{
\code{contingency_coef()} computes Pearson's contingency coefficient C
for a two-way contingency table.
}
\details{
The contingency coefficient is
\eqn{C = \sqrt{\chi^2 / (\chi^2 + n)}}.
It ranges from 0 (independence) to a maximum that depends on
the table dimensions. No standard asymptotic standard error exists,
so the confidence interval is not computed.
}
\examples{
tab <- table(sochealth$smoking, sochealth$education)
contingency_coef(tab)

}
\seealso{
\code{\link[=cramer_v]{cramer_v()}}, \code{\link[=assoc_measures]{assoc_measures()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/count_n.R
\name{count_n}
\alias{count_n}
\title{Row-wise Count of Specific or Special Values}
\usage{
count_n(
  data = NULL,
  select = tidyselect::everything(),
  exclude = NULL,
  count = NULL,
  special = NULL,
  allow_coercion = TRUE,
  ignore_case = FALSE,
  regex = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{data}{A data frame or matrix. Optional inside \code{mutate()}.}

\item{select}{Columns to include. Defaults to \code{tidyselect::everything()}.
Uses tidyselect helpers like \code{\link[tidyselect:starts_with]{tidyselect::starts_with()}}, etc.
If \code{regex = TRUE}, \code{select} is treated as a regex string.}

\item{exclude}{Character vector of column names to exclude after selection.
Defaults to \code{NULL} (no exclusion).}

\item{count}{Value(s) to count. Defaults to \code{NULL}. Ignored if \code{special} is used.
Multiple values are allowed (e.g., \code{count = c(1, 2, 3)} or \code{count = c("yes", "no")}).
R automatically coerces all values in \code{count} to a common type (e.g., \code{c(2, "2")} becomes \code{c("2", "2")}),
so all values are expected to be of the same final type.
If \code{allow_coercion = FALSE}, matching is type-safe using \code{identical()}, and the type of \code{count} must match that of the values in the data.}

\item{special}{Character vector of special values to count: \code{"NA"}, \code{"NaN"}, \code{"Inf"}, \code{"-Inf"}, or \code{"all"}.
Defaults to \code{NULL}.
\code{"NA"} uses \code{is.na()}, and therefore includes both \code{NA} and \code{NaN} values.
\code{"NaN"} uses \code{is.nan()} to match only actual NaN values.}

\item{allow_coercion}{Logical. If \code{TRUE} (the default), values are compared after coercion.
If \code{FALSE}, uses strict matching via \code{identical()}.}

\item{ignore_case}{Logical. If \code{FALSE} (the default), comparisons are case-sensitive.
If \code{TRUE}, performs case-insensitive string comparisons.}

\item{regex}{Logical. If \code{FALSE} (the default), uses tidyselect helpers.
If \code{TRUE}, interprets \code{select} as a regular expression pattern.}

\item{verbose}{Logical. If \code{FALSE} (the default), messages are suppressed.
If \code{TRUE}, prints processing messages.}
}
\value{
A numeric vector of row-wise counts (unnamed).
}
\description{
\code{count_n()} counts, for each row of a data frame or matrix, how many times one or more values appear across selected columns.
It supports type-safe comparison, case-insensitive string matching, and detection of special values such as \code{NA}, \code{NaN}, \code{Inf}, and \code{-Inf}.
}
\details{
This function is particularly useful for summarizing data quality or patterns in row-wise structures,
and is designed to work fluently inside \code{dplyr::mutate()} pipelines.

Internally, \code{count_n()} wraps the stable and dependency-free base function \code{base_count_n()}, allowing high flexibility and testability.
}
\note{
This function is inspired by \code{\link[datawizard:row_count]{datawizard::row_count()}}, but provides additional flexibility:
\itemize{
\item \strong{Element-wise type-safe matching} using \code{identical()} when \code{allow_coercion = FALSE}. This ensures that both the value and its type match exactly, enabling precise comparisons in mixed-type columns.
\item \strong{Support for multiple values in \code{count}}, allowing queries like \code{count = c(2, 3)} or \code{count = c("yes", "no")} to count any of several values per row.
\item \strong{Detection of special values} such as \code{NA}, \code{NaN}, \code{Inf}, and \code{-Inf} through the \code{special} argument — a feature not available in \code{row_count()}.
\item \strong{Tidyverse-native behavior}: can be used inside \code{mutate()} without explicitly passing a \code{data} argument.
}
\subsection{Value coercion behavior}{

R automatically coerces mixed-type vectors passed to \code{count} into a common type.
For example, \code{count = c(2, "2")} becomes \code{c("2", "2")}, because R converts numeric and character values to a unified type.
This means that mixed-type checks are not possible at runtime once \code{count} is passed to the function.
To ensure accurate type-sensitive matching, users should avoid mixing types in \code{count} explicitly.
}

\subsection{Strict matching mode (\code{allow_coercion = FALSE})}{

When strict matching is enabled, each value in \code{count} must match the type of the target column exactly.

For factor columns, this means that \code{count} must also be a factor. Supplying \code{count = "b"} (a character string) will not match a factor value, even if the label appears identical.

A common and intuitive approach is to use \code{count = factor("b")}, which works in many cases. However, \code{identical()} — used internally for strict comparisons — also checks the internal structure of the factor, including the order and content of its levels.
As a result, comparisons may still fail if the levels differ, even when the label is the same.

To ensure a perfect match (label \strong{and} levels), you can reuse a value taken directly from the data (e.g., \code{df$x[2]}). This guarantees that both the class and the factor levels align. However, this approach only works reliably if all selected columns have the same factor structure.
}

\subsection{Case-insensitive matching (\code{ignore_case = TRUE})}{

When \code{ignore_case = TRUE}, all values involved in the comparison are converted to lowercase using \code{tolower()} before matching.
This behavior applies to both character and factor columns. Factors are first converted to character internally.

Importantly, this case-insensitive mode takes precedence over strict type comparison: values are no longer compared using \code{identical()}, but rather using lowercase string equality. This enables more flexible matching — for example, \code{"b"} and \code{"B"} will match even when \code{allow_coercion = FALSE}.
\subsection{Example: strict vs. case-insensitive matching with factors}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{df <- tibble::tibble(
  x = factor(c("a", "b", "c")),
  y = factor(c("b", "B", "a"))
)

# Strict match fails with character input
count_n(df, count = "b", allow_coercion = FALSE)
#> [1] 0 0 0

# Match works only where factor levels match exactly
count_n(df, count = factor("b", levels = levels(df$x)), allow_coercion = FALSE)
#> [1] 0 1 0

# Case-insensitive match succeeds for both "b" and "B"
count_n(df, count = "b", ignore_case = TRUE)
#> [1] 1 2 0
}\if{html}{\out{</div>}}

Like \code{\link[datawizard:row_count]{datawizard::row_count()}}, this function also supports regex-based column selection, case-insensitive string comparison, and column exclusion.
}

}
}
\examples{
library(dplyr)
library(tibble)
library(haven)

# Basic usage
df <- tibble(
  x = c(1, 2, 2, 3, NA),
  y = c(2, 2, NA, 3, 2),
  z = c("2", "2", "2", "3", "2")
)
count_n(df, count = 2)
count_n(df, count = 2, allow_coercion = FALSE)
df |> mutate(num_twos = count_n(count = 2))

# Mixed types and special values
df <- tibble(
  num   = c(1, 2, NA, -Inf, NaN),
  char  = c("a", "B", "b", "a", NA),
  fact  = factor(c("a", "b", "b", "a", "c")),
  date  = as.Date(c("2023-01-01", "2023-01-01", NA, "2023-01-02", "2023-01-01")),
  lab   = labelled(c(1, 2, 1, 2, NA), labels = c(No = 1, Yes = 2)),
  logic = c(TRUE, FALSE, NA, TRUE, FALSE)
)
count_n(df, count = 2)
count_n(df, count = "b", ignore_case = TRUE)
count_n(df, count = "a", select = fact)
count_n(df, count = as.Date("2023-01-01"), select = date)

# Count special values
count_n(df, special = "NA")

# Column selection strategies
df <- tibble(
  score_math    = c(1, 2, 2, 3, NA),
  score_science = c(2, 2, NA, 3, 2),
  score_lang    = c("2", "2", "2", "3", "2"),
  name          = c("Jean", "Marie", "Ali", "Zoe", "Nina")
)
count_n(df, select = c(score_math, score_science), count = 2)
count_n(df, select = starts_with("score_"), exclude = "score_lang", count = 2)
count_n(df, select = "^score_", regex = TRUE, count = 2)
df |> mutate(nb_two = count_n(count = 2))

# Strict type-safe matching with factor columns
df <- tibble(
  x = factor(c("a", "b", "c")),
  y = factor(c("b", "B", "a"))
)

# Coercion: character "b" matches both x and y
count_n(df, count = "b")

# Strict match: fails because "b" is character, not factor (returns only 0s)
count_n(df, count = "b", allow_coercion = FALSE)

# Strict match with factor value: works only where levels match
count_n(df, count = factor("b", levels = levels(df$x)), allow_coercion = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_continuous_lm.R
\name{table_continuous_lm}
\alias{table_continuous_lm}
\title{Continuous-outcome linear-model table}
\usage{
table_continuous_lm(
  data,
  select = tidyselect::everything(),
  by,
  exclude = NULL,
  regex = FALSE,
  weights = NULL,
  vcov = c("classical", "HC0", "HC1", "HC2", "HC3", "HC4", "HC4m", "HC5"),
  contrast = c("auto", "none"),
  statistic = FALSE,
  p_value = TRUE,
  show_n = TRUE,
  show_weighted_n = FALSE,
  effect_size = c("none", "f2"),
  r2 = c("r2", "adj_r2", "none"),
  ci = TRUE,
  labels = NULL,
  ci_level = 0.95,
  digits = 2,
  fit_digits = 2,
  effect_size_digits = 2,
  decimal_mark = ".",
  output = c("default", "data.frame", "long", "tinytable", "gt", "flextable", "excel",
    "clipboard", "word"),
  excel_path = NULL,
  excel_sheet = "Linear models",
  clipboard_delim = "\\t",
  word_path = NULL,
  verbose = FALSE
)
}
\arguments{
\item{data}{A \code{data.frame}.}

\item{select}{Outcome columns to include. If \code{regex = FALSE}, use tidyselect
syntax or a character vector of column names (default:
\code{tidyselect::everything()}). If \code{regex = TRUE}, provide a regular expression
pattern (character string).}

\item{by}{A single predictor column. Accepts an unquoted column name or a
single character column name. The predictor can be numeric, logical,
character, or factor.}

\item{exclude}{Columns to exclude from \code{select}. Supports tidyselect syntax
and character vectors of column names.}

\item{regex}{Logical. If \code{FALSE} (the default), uses tidyselect helpers. If
\code{TRUE}, the \code{select} argument is treated as a regular expression.}

\item{weights}{Optional case weights. Accepts \code{NULL}, an unquoted numeric
column name, a single character column name, or a numeric vector of length
\code{nrow(data)}.}

\item{vcov}{Variance estimator used for uncertainty estimates. One of
\code{"classical"} (default), \code{"HC0"}, \code{"HC1"}, \code{"HC2"}, \code{"HC3"}, \code{"HC4"},
\code{"HC4m"}, or \code{"HC5"}.}

\item{contrast}{Contrast display for categorical predictors. One of:
\itemize{
\item \code{"auto"}: show a single reference contrast only when \code{by} has exactly
two levels.
\item \code{"none"}: suppress difference columns for categorical predictors.
}}

\item{statistic}{Logical. If \code{TRUE}, includes a test-statistic column in the
wide and rendered outputs. Defaults to \code{FALSE}.}

\item{p_value}{Logical. If \code{TRUE}, includes a \code{p} column in the wide and
rendered outputs. Defaults to \code{TRUE}.}

\item{show_n}{Logical. If \code{TRUE}, includes an unweighted \code{n} column in the
wide and rendered outputs. Defaults to \code{TRUE}.}

\item{show_weighted_n}{Logical. If \code{TRUE} and \code{weights} is supplied,
includes a \verb{Weighted n} column equal to the sum of case weights in the
analytic sample. Defaults to \code{FALSE}.}

\item{effect_size}{Character. Effect-size column to include in the wide and
rendered outputs. One of \code{"none"} (the default) or \code{"f2"}.}

\item{r2}{Character. Fit statistic to include in the wide and rendered
outputs. One of \code{"r2"} (the default), \code{"adj_r2"}, or \code{"none"}.}

\item{ci}{Logical. If \code{TRUE}, includes contrast confidence-interval columns
in the wide and rendered outputs when a single contrast is shown.
Defaults to \code{TRUE}.}

\item{labels}{An optional named character vector of outcome labels. Names
must match column names in \code{data}. When \code{NULL} (the default), labels are
auto-detected from variable attributes; if none are found, the column name
is used.}

\item{ci_level}{Confidence level for coefficient and model-based mean
intervals (default: \code{0.95}). Must be between 0 and 1 exclusive.}

\item{digits}{Number of decimal places for descriptive values, regression
coefficients, and test statistics (default: \code{2}).}

\item{fit_digits}{Number of decimal places for model-fit columns (\code{R2} or
adjusted \code{R2}) in wide and rendered outputs (default: \code{2}).}

\item{effect_size_digits}{Number of decimal places for effect-size columns
(\code{f2}) in wide and rendered outputs (default: \code{2}).}

\item{decimal_mark}{Character used as decimal separator. Either \code{"."}
(default) or \code{","}.}

\item{output}{Output format. One of:
\itemize{
\item \code{"default"}: a printed ASCII table, returned invisibly
\item \code{"data.frame"}: a plain wide \code{data.frame}
\item \code{"long"}: a raw long \code{data.frame}
\item \code{"tinytable"} (requires \code{tinytable})
\item \code{"gt"} (requires \code{gt})
\item \code{"flextable"} (requires \code{flextable})
\item \code{"excel"} (requires \code{openxlsx2})
\item \code{"clipboard"} (requires \code{clipr})
\item \code{"word"} (requires \code{flextable} and \code{officer})
}}

\item{excel_path}{File path for \code{output = "excel"}.}

\item{excel_sheet}{Sheet name for \code{output = "excel"} (default:
\code{"Linear models"}).}

\item{clipboard_delim}{Delimiter for \code{output = "clipboard"} (default:
\code{"\\t"}).}

\item{word_path}{File path for \code{output = "word"}.}

\item{verbose}{Logical. If \code{TRUE}, prints messages about ignored
non-numeric selected outcomes (default: \code{FALSE}).}
}
\value{
Depends on \code{output}:
\itemize{
\item \code{"default"}: prints a styled ASCII table and returns the
underlying long \code{data.frame} invisibly, with class
\code{"spicy_continuous_lm_table"}.
\item \code{"data.frame"}: a plain wide \code{data.frame} with one row per outcome
and numeric columns for means, optional contrasts, optional test
statistics, model fit, and sample size.
\item \code{"long"}: a raw \code{data.frame} with one block per outcome and columns
describing estimated means, contrasts, test statistics, model fit, and
sample size.
\item \code{"tinytable"}: a \code{tinytable} object.
\item \code{"gt"}: a \code{gt_tbl} object.
\item \code{"flextable"}: a \code{flextable} object.
\item \code{"excel"} / \code{"word"}: writes to disk and returns the file path.
\item \code{"clipboard"}: copies the wide table and returns it invisibly.
}

If no numeric outcome columns remain after applying \code{select}, \code{exclude},
and \code{regex}, the function emits a warning and returns an empty
\code{data.frame()} regardless of \code{output}.
}
\description{
Builds APA-style summary tables from a series of simple linear models for
one or many continuous outcomes selected with tidyselect syntax.

A single predictor is supplied with \code{by}, and each selected numeric
outcome is fit as \code{lm(outcome ~ by, ...)}. When \code{by} is categorical, the
function returns a model-based mean-comparison table with fitted means by
level derived from the linear model, plus an optional single difference for
dichotomous predictors. When \code{by} is numeric, the table reports the slope
and its confidence interval.

Multiple output formats are available via \code{output}: a printed ASCII table
(\code{"default"}), a plain wide \code{data.frame} (\code{"data.frame"}), a raw long
\code{data.frame} (\code{"long"}), or rendered outputs (\code{"tinytable"}, \code{"gt"},
\code{"flextable"}, \code{"excel"}, \code{"clipboard"}, \code{"word"}).
}
\details{
\code{table_continuous_lm()} is designed for article-style bivariate reporting:
a single predictor supplied with \code{by}, and one simple model per selected
continuous outcome. The model fit is always \code{lm(outcome ~ by, ...)}.

For categorical predictors, the reported means are model-based fitted means
for each level of \code{by}, and the reported contrasts are derived from the same
fitted linear model. For an unweighted \code{lm(y ~ factor)} with classical
variance, these fitted means coincide numerically with the empirical
subgroup means; the "model-based" qualifier matters because (a) under
\code{weights}, the means become weighted least-squares estimates, (b) their
confidence intervals are derived from the model \code{vcov} (classical or \verb{HC*})
rather than from a separate empirical standard error, and (c) the reported
tests, \code{p}-values, and effect sizes all come from the same fitted model,
so means, contrasts, and inference stay internally consistent.

Compared with \code{\link[=table_continuous]{table_continuous()}}, this function is the model-based
companion for users who want to report bivariate mean comparisons in a
linear-model framework. In practice, it is the better choice when you
want heteroskedasticity-consistent standard errors (\code{vcov = "HC*"}),
model fit statistics, or case weights via \code{lm(..., weights = ...)}.
Because the function exists to report a fitted model, its inferential
output is part of the default display: \code{p_value = TRUE} is on by default,
and the model \code{R2} is shown unless explicitly disabled with \code{r2 = "none"}.
Set \code{p_value = FALSE} or \code{r2 = "none"} to suppress them.

Effect size is reported as Cohen's \code{f2}, computed from the model \code{R2} as
\code{R2 / (1 - R2)}. When \code{vcov != "classical"}, standard errors, confidence
intervals, and test statistics use the requested heteroskedasticity-
consistent estimator, while \code{R2} and adjusted \code{R2} remain the ordinary
least-squares fit statistics.

When \code{weights} is supplied, \code{table_continuous_lm()} fits weighted linear
models using \code{lm(..., weights = ...)} and reports weighted model-based
means or slopes accordingly. This is appropriate for case-weighted
analyses and weighted article tables, but it is not a substitute for a
full complex-survey design workflow.

In wide and rendered outputs, \code{n} always reports the unweighted analytic
sample size used for each outcome. When \code{show_weighted_n = TRUE}, an
additional \verb{Weighted n} column reports the sum of case weights for the same
analytic sample.

For dichotomous categorical predictors, the wide outputs report means in
reference-level order and labels the contrast column explicitly as
\code{Delta (level2 - level1)}. For categorical predictors with more than two
levels, no single contrast or contrast confidence interval is shown in the
wide outputs; instead, the table reports level-specific means plus the
overall \code{F} test when \code{statistic = TRUE} (or \code{F(df1, df2)} when the degrees
of freedom are constant across outcomes).

Optional output engines require suggested packages:
\itemize{
\item \pkg{tinytable} for \code{output = "tinytable"}
\item \pkg{gt} for \code{output = "gt"}
\item \pkg{flextable} for \code{output = "flextable"}
\item \pkg{flextable} + \pkg{officer} for \code{output = "word"}
\item \pkg{openxlsx2} for \code{output = "excel"}
\item \pkg{clipr} for \code{output = "clipboard"}
}
}
\examples{
table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex
)

table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  output = "data.frame"
)

table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = age,
  vcov = "HC3",
  ci = FALSE
)

table_continuous_lm(
  sochealth,
  select = c(wellbeing_score, bmi),
  by = sex,
  weights = weight,
  statistic = TRUE,
  effect_size = "f2",
  show_weighted_n = TRUE
)

\donttest{
if (requireNamespace("tinytable", quietly = TRUE)) {
  table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    output = "tinytable"
  )
}

if (requireNamespace("gt", quietly = TRUE)) {
  table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    output = "gt"
  )
}

if (requireNamespace("flextable", quietly = TRUE)) {
  table_continuous_lm(
    sochealth,
    select = c(wellbeing_score, bmi),
    by = sex,
    output = "flextable"
  )
}
}

}
\seealso{
\code{\link[=table_continuous]{table_continuous()}}, \code{\link[=table_categorical]{table_categorical()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spicy-package.R
\docType{package}
\name{spicy-package}
\alias{spicy}
\alias{spicy-package}
\title{spicy: Descriptive Statistics, Summary Tables, and Data Management Tools}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Provides tools for descriptive data analysis, variable inspection, data management, and tabulation workflows in 'R'. Summarizes variable metadata, labels, classes, missing values, and representative values, with support for readable frequency tables, cross-tabulations, association measures for contingency tables (Cramer's V, Phi, Goodman-Kruskal Gamma, Kendall's Tau-b, Somers' D, and others), categorical and continuous summary tables, and model-based bivariate tables for continuous outcomes, including APA-style reporting outputs. Includes helpers for interactive codebooks, variable label extraction, clipboard export, and row-wise descriptive summaries. Designed to make descriptive analysis and reporting faster, clearer, and easier to work with in practice.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/amaltawfik/spicy/}
  \item \url{https://amaltawfik.github.io/spicy/}
  \item Report bugs at \url{https://github.com/amaltawfik/spicy/issues}
}

}
\author{
\strong{Maintainer}: Amal Tawfik \email{amal.tawfik@hesav.ch} (\href{https://orcid.org/0009-0006-2422-1555}{ORCID}) (\href{https://ror.org/04j47fz63}{ROR}) [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_from_names.R
\name{label_from_names}
\alias{label_from_names}
\title{Derive variable labels from column names \verb{name<sep>label}}
\usage{
label_from_names(df, sep = ". ")
}
\arguments{
\item{df}{A \code{data.frame} or tibble with column names of the form
\verb{"name<sep>label"} (e.g. "name. label"). (by default from LimeSurvey).}

\item{sep}{Character string used as separator between name and label.
Default is \code{". "} (LimeSurvey's default), but any literal string can be used.}
}
\value{
A base \code{tibble} with column names equal to the \emph{names} (before \code{sep})
and \code{var_label} attributes equal to the \emph{labels} (after \code{sep}).
}
\description{
Splits each column name at the \strong{first} occurrence of \code{sep}, renames
the column to the part before \code{sep} (the \emph{name}), and assigns the part
after \code{sep} as a \code{\link[labelled:var_label]{labelled::var_label()}}. This works even if the label
itself contains the separator.
}
\details{
This function is especially useful for \strong{LimeSurvey CSV exports} when using
\emph{Export results} → \emph{Export format: CSV} → \emph{Headings: Question code & question text},
where column names look like \code{"code. question text"}. In this case the
default separator is \code{". "}.
}
\examples{
# Example with LimeSurvey-style column names
df <- data.frame(
  "age. Age of respondent" = c(25, 30),
  "score. Total score. Manually computed." = c(12, 14),
  check.names = FALSE
)

# sep = ". " by default (LimeSurvey)
out <- label_from_names(df)
labelled::var_label(out)

# Example with a custom separator ("|")
df2 <- data.frame(
  "id|Identifier" = 1:3,
  "score|Total score" = c(10, 20, 30),
  check.names = FALSE
)
out2 <- label_from_names(df2, sep = "|")
labelled::var_label(out2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_continuous_print.R
\name{print.spicy_continuous_table}
\alias{print.spicy_continuous_table}
\title{Print method for continuous summary tables}
\usage{
\method{print}{spicy_continuous_table}(x, ...)
}
\arguments{
\item{x}{A \code{data.frame} of class \code{"spicy_continuous_table"} as returned
by \code{\link[=table_continuous]{table_continuous()}}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Formats and prints a \code{spicy_continuous_table} object as a styled ASCII
table using \code{\link[=spicy_print_table]{spicy_print_table()}}.
}
\seealso{
\code{\link[=table_continuous]{table_continuous()}}, \code{\link[=spicy_print_table]{spicy_print_table()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_continuous.R
\name{table_continuous}
\alias{table_continuous}
\title{Continuous summary table}
\usage{
table_continuous(
  data,
  select = tidyselect::everything(),
  by = NULL,
  exclude = NULL,
  regex = FALSE,
  test = c("welch", "student", "nonparametric"),
  p_value = NULL,
  statistic = FALSE,
  effect_size = FALSE,
  effect_size_ci = FALSE,
  labels = NULL,
  ci_level = 0.95,
  digits = 2,
  effect_size_digits = 2,
  decimal_mark = ".",
  output = c("default", "data.frame", "tinytable", "gt", "flextable", "excel",
    "clipboard", "word"),
  excel_path = NULL,
  excel_sheet = "Descriptives",
  clipboard_delim = "\\t",
  word_path = NULL,
  verbose = FALSE
)
}
\arguments{
\item{data}{A \code{data.frame}.}

\item{select}{Columns to include. If \code{regex = FALSE}, use tidyselect
syntax or a character vector of column names (default:
\code{tidyselect::everything()}). If \code{regex = TRUE}, provide a regular
expression pattern (character string).}

\item{by}{Optional grouping column. Accepts an unquoted column name or
a single character column name. The column does not need to be
numeric.}

\item{exclude}{Columns to exclude. Supports tidyselect syntax and
character vectors of column names.}

\item{regex}{Logical. If \code{FALSE} (the default), uses tidyselect
helpers. If \code{TRUE}, the \code{select} argument is treated as a regular
expression.}

\item{test}{Character. Statistical test to use when comparing groups.
One of \code{"welch"} (default), \code{"student"}, or \code{"nonparametric"}.
\itemize{
\item \code{"welch"}: Welch \emph{t}-test (2 groups) or Welch one-way ANOVA
(3+ groups). Does not assume equal variances.
\item \code{"student"}: Student \emph{t}-test (2 groups) or classic one-way
ANOVA (3+ groups). Assumes equal variances.
\item \code{"nonparametric"}: Wilcoxon rank-sum / Mann--Whitney \emph{U}
(2 groups) or Kruskal--Wallis \emph{H} (3+ groups).
}

Used whenever \code{by} is supplied (since \code{p_value} defaults to \code{TRUE}
in that case) or when \code{statistic = TRUE} / \code{effect_size = TRUE}.
Ignored when \code{by} is not used, or when all three display toggles
are turned off.}

\item{p_value}{Logical or \code{NULL}. If \code{TRUE} and \code{by} is used, adds a
\emph{p}-value column from the test specified by \code{test}. When \code{NULL} (the
default), the \emph{p}-value is shown automatically whenever \code{by} is
supplied, and hidden otherwise. Pass \code{p_value = FALSE} to suppress
the column explicitly. Ignored when \code{by} is not used.}

\item{statistic}{Logical. If \code{TRUE} and \code{by} is used, the test
statistic is shown in an additional column (e.g.,
\code{t(df) = ...}, \code{F(df1, df2) = ...}, \code{W = ...}, or \code{H(df) = ...}).
Both \code{p_value} and \code{statistic} are independent; either or both
can be enabled. Defaults to \code{FALSE}. Ignored when \code{by} is not
used.}

\item{effect_size}{Logical. If \code{TRUE} and \code{by} is used, adds an
effect-size column ("ES"). The measure is chosen automatically:
\itemize{
\item Hedges' \emph{g} (bias-corrected) - 2 groups, parametric (CI via
Hedges & Olkin approximation).
\item Eta-squared (\eqn{\eta^2}) - 3+ groups, parametric (CI via
noncentral \emph{F} distribution).
\item Rank-biserial \emph{r} (\code{r_rb}) - 2 groups, nonparametric (CI via
Fisher \emph{z}-transform).
\item Epsilon-squared (\eqn{\varepsilon^2}) - 3+ groups,
nonparametric (CI via percentile bootstrap, 2 000 replicates).
}

Defaults to \code{FALSE}. Ignored when \code{by} is not used.}

\item{effect_size_ci}{Logical. If \code{TRUE}, appends the confidence
interval of the effect size in brackets (e.g.,
\code{g = 0.45 [0.22, 0.68]}). Implies \code{effect_size = TRUE}.
Defaults to \code{FALSE}.}

\item{labels}{An optional named character vector of variable labels.
Names must match column names in \code{data}. When \code{NULL} (the default),
labels are auto-detected from variable attributes (e.g., haven
labels); if none are found, the column name is used.}

\item{ci_level}{Confidence level for the mean confidence interval
(default: \code{0.95}). Must be between 0 and 1 exclusive.}

\item{digits}{Number of decimal places for descriptive values and test
statistics (default: \code{2}).}

\item{effect_size_digits}{Number of decimal places for effect-size values
in formatted displays (default: \code{2}).}

\item{decimal_mark}{Character used as decimal separator.
Either \code{"."} (default) or \code{","}.}

\item{output}{Output format. One of:
\itemize{
\item \code{"default"} (a printed ASCII table, returned invisibly)
\item \code{"data.frame"} (a plain numeric \code{data.frame})
\item \code{"tinytable"} (requires \code{tinytable})
\item \code{"gt"} (requires \code{gt})
\item \code{"flextable"} (requires \code{flextable})
\item \code{"excel"} (requires \code{openxlsx2})
\item \code{"clipboard"} (requires \code{clipr})
\item \code{"word"} (requires \code{flextable} and \code{officer})
}}

\item{excel_path}{File path for \code{output = "excel"}.}

\item{excel_sheet}{Sheet name for \code{output = "excel"}
(default: \code{"Descriptives"}).}

\item{clipboard_delim}{Delimiter for \code{output = "clipboard"}
(default: \code{"\\t"}).}

\item{word_path}{File path for \code{output = "word"}.}

\item{verbose}{Logical. If \code{TRUE}, prints messages about excluded
non-numeric columns (default: \code{FALSE}).}
}
\value{
Depends on \code{output}:
\itemize{
\item \code{"default"}: prints a styled ASCII table and returns the
underlying \code{data.frame} invisibly (S3 class
\code{"spicy_continuous_table"}).
\item \code{"data.frame"}: a plain \code{data.frame} with columns
\code{variable}, \code{label}, \code{group} (when \code{by} is used), \code{mean}, \code{sd},
\code{min}, \code{max}, \code{ci_lower}, \code{ci_upper}, \code{n}.
When \code{by} is used together with \code{p_value = TRUE},
\code{statistic = TRUE}, or \code{effect_size = TRUE}, additional columns
are appended (populated on the first row of each variable block
only):
\itemize{
\item \code{test_type} -- test identifier (e.g., \code{"welch_t"},
\code{"welch_anova"}, \code{"student_t"}, \code{"anova"}, \code{"wilcoxon"},
\code{"kruskal"}).
\item \code{statistic}, \code{df1}, \code{df2}, \code{p.value} -- test results.
\item \code{es_type} -- effect-size identifier (\code{"hedges_g"},
\code{"eta_sq"}, \code{"r_rb"}, or \code{"epsilon_sq"}), when
\code{effect_size = TRUE}.
\item \code{es_value}, \code{es_ci_lower}, \code{es_ci_upper} -- effect-size
estimate and confidence interval bounds.
}
\item \code{"tinytable"}: a \code{tinytable} object.
\item \code{"gt"}: a \code{gt_tbl} object.
\item \code{"flextable"}: a \code{flextable} object.
\item \code{"excel"} / \code{"word"}: writes to disk and returns the file
path invisibly.
\item \code{"clipboard"}: copies the table and returns the display
\code{data.frame} invisibly.
}
}
\description{
Computes descriptive statistics (mean, SD, min, max, confidence interval
of the mean, \emph{n}) for one or many continuous variables selected with
tidyselect syntax.

With \code{by}, produces grouped summaries and reports a group-comparison
\emph{p}-value by default (Welch test; change via \code{test}). Additional
inferential output is opt-in: test statistics (\code{statistic}) and
effect sizes (\code{effect_size} / \code{effect_size_ci}). Set \code{p_value = FALSE}
to suppress the \emph{p}-value column. Without \code{by}, produces one-way
descriptive summaries.

Multiple output formats are available via \code{output}: a printed ASCII
table (\code{"default"}), a plain numeric \code{data.frame} (\code{"data.frame"}), or
publication-ready tables (\code{"tinytable"}, \code{"gt"}, \code{"flextable"},
\code{"excel"}, \code{"clipboard"}, \code{"word"}).
}
\details{
Non-numeric columns are silently dropped (set \code{verbose = TRUE} to see
which columns were excluded). When a single constant column is passed,
SD and CI are shown as \code{"--"} in the ASCII table.

Optional output engines require suggested packages:
\itemize{
\item \pkg{tinytable} for \code{output = "tinytable"}
\item \pkg{gt} for \code{output = "gt"}
\item \pkg{flextable} for \code{output = "flextable"}
\item \pkg{flextable} + \pkg{officer} for \code{output = "word"}
\item \pkg{openxlsx2} for \code{output = "excel"}
\item \pkg{clipr} for \code{output = "clipboard"}
}
}
\examples{
table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score)
)

table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score),
  output = "data.frame"
)

table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score),
  by = education
)

table_continuous(
  sochealth,
  select = c(bmi, wellbeing_score),
  by = education,
  statistic = TRUE
)

table_continuous(
  sochealth,
  select = wellbeing_score,
  by = education,
  effect_size_ci = TRUE,
  effect_size_digits = 3
)

table_continuous(
  sochealth,
  select = wellbeing_score,
  by = sex,
  effect_size_ci = TRUE
)

table_continuous(
  sochealth,
  select = "^life_sat",
  regex = TRUE
)

table_continuous(
  sochealth,
  select = c(bmi, life_sat_health),
  labels = c(
    bmi = "Body mass index",
    life_sat_health = "Satisfaction with health"
  )
)

\donttest{
if (requireNamespace("tinytable", quietly = TRUE)) {
  table_continuous(
    sochealth,
    select = starts_with("life_sat"),
    by = education,
    output = "tinytable"
  )
}

if (requireNamespace("gt", quietly = TRUE)) {
  table_continuous(
    sochealth,
    select = starts_with("life_sat"),
    by = education,
    output = "gt"
  )
}

if (requireNamespace("flextable", quietly = TRUE)) {
  table_continuous(
    sochealth,
    select = starts_with("life_sat"),
    by = education,
    output = "flextable"
  )
}
}

}
\seealso{
\code{\link[=table_categorical]{table_categorical()}} for categorical variables;
\code{\link[=freq]{freq()}} for one-way frequency tables; \code{\link[=cross_tab]{cross_tab()}} for two-way
cross-tabulations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/varlist.R
\name{varlist}
\alias{varlist}
\alias{vl}
\title{Generate a comprehensive summary of the variables}
\usage{
varlist(
  x,
  ...,
  values = FALSE,
  tbl = FALSE,
  include_na = FALSE,
  factor_levels = c("observed", "all")
)

vl(
  x,
  ...,
  values = FALSE,
  tbl = FALSE,
  include_na = FALSE,
  factor_levels = c("observed", "all")
)
}
\arguments{
\item{x}{A data frame, or a transformation of one.}

\item{...}{Optional tidyselect-style column selectors (e.g.
\code{starts_with("var")}, \code{where(is.numeric)}, etc.). Columns can be selected
or reordered, but renaming selections is not supported.}

\item{values}{Logical. If \code{FALSE} (the default), displays a compact summary
of the variable's values. For numeric, character, date/time, labelled, and
factor variables, all unique non-missing values are shown when there are
at most four; otherwise the first three values, an ellipsis (\code{...}), and
the last value are shown. Values are sorted when appropriate (e.g.,
numeric, character, date).
For factors, \code{factor_levels} controls whether observed or all declared
levels are shown; level order is preserved.
For labelled variables, prefixed labels are displayed via
\code{labelled::to_factor(levels = "prefixed")}.
If \code{TRUE}, all unique non-missing values are displayed.}

\item{tbl}{Logical. If \code{FALSE} (the default), opens the summary in the Viewer
if the session is interactive. If \code{TRUE}, returns a tibble.}

\item{include_na}{Logical. If \code{TRUE}, unique missing value markers
(\verb{<NA>}, \verb{<NaN>}) are explicitly appended at the end of the \code{Values}
summary when present in the variable. This applies to all variable types.
Literal strings \code{"NA"}, \code{"NaN"}, and \code{""} are quoted to distinguish them
from missing markers. If \code{FALSE} (the default), missing values are omitted
from \code{Values} but still counted in the \code{NAs} column.}

\item{factor_levels}{Character. Controls how factor values are displayed
in \code{Values}. \code{"observed"} (the default; \code{\link[=code_book]{code_book()}} uses \code{"all"})
shows only levels present in the data, preserving factor level order.
\code{"all"} shows all declared levels, including unused levels.}
}
\value{
A tibble with one row per selected variable, containing the following
columns:
\itemize{
\item \code{Variable}: variable names
\item \code{Label}: variable labels (if available via the \code{label} attribute)
\item \code{Values}: a summary of the variable's values, depending on the \code{values}
and \code{include_na} arguments. If \code{values = FALSE}, a compact summary is
shown: all unique values when there are at most four, otherwise
3 + ... + last. If \code{values = TRUE}, all unique non-missing values are
displayed. For labelled variables, \strong{prefixed labels} are displayed using
\code{labelled::to_factor(levels = "prefixed")}.
For factors, levels are displayed according to \code{factor_levels}.
Matrix and array columns are summarized by their dimensions.
Missing value markers (\verb{<NA>}, \verb{<NaN>}) are optionally appended at the
end (controlled via \code{include_na}). Literal strings \code{"NA"}, \code{"NaN"}, and
\code{""} are quoted to distinguish them from missing markers.
\item \code{Class}: the class of each variable (possibly multiple, e.g.
\verb{"labelled", "numeric"})
\item \code{N_distinct}: number of distinct non-missing values
\item \code{N_valid}: number of non-missing observations
\item \code{NAs}: number of missing observations
}

If \code{tbl = TRUE}, the tibble is returned. If \code{tbl = FALSE} and the session is
interactive, the summary is displayed in the Viewer pane and the function
returns invisibly. In non-interactive sessions, a message is displayed and
the function returns invisibly.
}
\description{
\code{varlist()} lists the variables of a data frame and extracts essential
metadata, including variable names, labels, summary values, classes, number
of distinct values, number of valid (non-missing) observations, and number
of missing values.

\code{vl()} is a convenient shorthand for \code{varlist()} that offers identical
functionality with a shorter name.
}
\details{
The function can also apply tidyselect-style variable selectors to select or
reorder columns dynamically.

If used interactively (e.g. in RStudio or Positron), the summary is
displayed in the Viewer pane with a contextual title like \code{vl: sochealth}.
If the data frame has been transformed or subsetted, the title will display
an asterisk (\code{*}), e.g. \verb{vl: sochealth*}. Anonymous or ambiguous calls use
\verb{vl: <data>}.

For factor variables, \code{varlist()} defaults to displaying only the levels
observed in the data (\code{factor_levels = "observed"}) — a reflection of what
is actually present. By contrast, \code{\link[=code_book]{code_book()}} defaults to \code{"all"} to
document the declared schema, including unused levels. Pass \code{factor_levels}
explicitly to override either default.
}
\examples{
varlist(sochealth, tbl = TRUE)
sochealth |> varlist(tbl = TRUE)
varlist(sochealth, where(is.numeric), values = TRUE, tbl = TRUE)
varlist(
  sochealth,
  starts_with("bmi"),
  values = TRUE,
  include_na = TRUE,
  tbl = TRUE
)

df <- data.frame(
  group = factor(c("A", "B", NA), levels = c("A", "B", "C"))
)
varlist(
  df,
  values = TRUE,
  include_na = TRUE,
  factor_levels = "all",
  tbl = TRUE
)

vl(sochealth, tbl = TRUE)
sochealth |> vl(tbl = TRUE)
vl(sochealth, starts_with("bmi"), tbl = TRUE)
vl(sochealth, where(is.numeric), values = TRUE, tbl = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/code_book.R
\name{code_book}
\alias{code_book}
\title{Generate an interactive variable codebook}
\usage{
code_book(
  x,
  ...,
  values = FALSE,
  include_na = FALSE,
  title = "Codebook",
  filename = NULL,
  factor_levels = c("all", "observed")
)
}
\arguments{
\item{x}{A data frame or tibble.}

\item{...}{Optional tidyselect-style column selectors (e.g.
\code{starts_with("var")}, \code{where(is.numeric)}, etc.). Columns can be selected
or reordered, but renaming selections is not supported.}

\item{values}{Logical. If \code{FALSE} (the default), displays a compact
summary of the variable's values. For numeric, character, date/time,
labelled, and factor variables, all unique non-missing values are shown
when there are at most four; otherwise the first three values, an ellipsis
(\code{...}), and the last value are shown. Values are sorted when appropriate
(e.g., numeric, character, date).
For factors, \code{factor_levels} controls whether observed or all declared
levels are shown; level order is preserved.
For labelled variables, prefixed labels are displayed via
\code{labelled::to_factor(levels = "prefixed")}.
If \code{TRUE}, all unique non-missing values are displayed.}

\item{include_na}{Logical. If \code{TRUE}, unique missing value markers
(\verb{<NA>}, \verb{<NaN>}) are appended at the end of the \code{Values} summary when
present in the variable. This applies to all variable types. Literal
strings \code{"NA"}, \code{"NaN"}, and \code{""} are quoted to distinguish them from
missing markers. If \code{FALSE} (the default), missing values are omitted from
\code{Values} but still counted in the \code{NAs} column.}

\item{title}{Optional character string displayed as the table caption.
Defaults to \code{"Codebook"}. Set to \code{NULL} to remove the title
completely. When \code{filename = NULL}, the title is also used as the base for
export filenames after conversion to a portable ASCII name.}

\item{filename}{Optional character string used as the base for exported CSV,
Excel, and PDF filenames. If \code{NULL} (the default), a portable filename is
derived from \code{title}, falling back to \code{"Codebook"} when needed. File
extensions are added by the browser/export engine.}

\item{factor_levels}{Character. Controls how factor values are displayed
in \code{Values}. \code{"all"} (the default; \code{\link[=varlist]{varlist()}} uses \code{"observed"}) shows
all declared levels, including unused levels. \code{"observed"} shows only
levels present in the data, preserving factor level order.}
}
\value{
A \code{DT::datatable} object.
}
\description{
\code{code_book()} creates an interactive and exportable codebook summarizing
selected variables of a data frame. It builds upon \code{\link[=varlist]{varlist()}} to provide
an overview of variable names, labels, classes, and representative values in
a sortable, searchable table.

The output is displayed as an interactive \code{DT::datatable()} in the Viewer pane
(for example in RStudio or Positron), allowing searching, sorting, and export
(copy, print, CSV, Excel, PDF) directly.
}
\details{
\itemize{
\item The interactive \code{datatable} supports column sorting, global searching, and
client-side export to various formats.
\item Variable selection uses the same tidyselect interface as \code{\link[=varlist]{varlist()}}.
\item By default, factor variables document all declared levels, including
unused levels — appropriate for a schema-oriented codebook. This differs
from \code{\link[=varlist]{varlist()}}, which defaults to \code{"observed"} to summarize observed
data only. Pass \code{factor_levels = "observed"} to mirror \code{\link[=varlist]{varlist()}}'s
default.
\item All exports occur client-side through the Viewer or Tab.
}
}
\section{Dependencies}{

Requires the following package:
\itemize{
\item \strong{DT}
}
}

\examples{
\dontrun{
if (requireNamespace("DT", quietly = TRUE)) {
  code_book(sochealth)
  code_book(sochealth, starts_with("bmi"))
  code_book(sochealth, starts_with("bmi"), values = TRUE, include_na = TRUE)

  factors <- data.frame(
    group = factor(c("A", "B", NA), levels = c("A", "B", "C"))
  )
  code_book(
    factors,
    values = TRUE,
    include_na = TRUE,
    factor_levels = "observed"
  )

  code_book(
    sochealth,
    starts_with("bmi"),
    title = "BMI codebook",
    filename = "bmi_codebook"
  )
}
}

}
\seealso{
\code{\link[=varlist]{varlist()}} for generating the underlying variable summaries.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spicy_tables.R
\name{spicy_tables}
\alias{spicy_tables}
\title{Spicy Table Engine: Frequency and Cross-tabulation Rendering}
\description{
The \emph{spicy table engine} provides a cohesive set of tools for creating and
printing formatted ASCII tables in R, designed for descriptive statistics.

Functions in this family include:
\itemize{
\item \code{\link[=freq]{freq()}} — frequency tables with support for weights, labelled data, and cumulative percentages
\item \code{\link[=spicy_print_table]{spicy_print_table()}} — general-purpose ASCII table printer
\item \code{\link[=build_ascii_table]{build_ascii_table()}} — internal rendering engine for column alignment and formatting
}
}
\details{
All functions in this family share a common philosophy:
\itemize{
\item Console-friendly display with Unicode box-drawing characters
\item Consistent alignment and spacing across outputs
\item Automatic detection of variable type (\code{factor}, \code{labelled}, \code{numeric})
\item Optional integration of variable labels and weighting information
}
}
\section{Core functions}{

\itemize{
\item \strong{\code{freq()}} — Main entry point for generating frequency tables.
\item \strong{\code{spicy_print_table()}} — Applies formatting and optional titles or notes.
\item \strong{\code{build_ascii_table()}} — Internal engine handling padding, alignment, and box rules.
}
}

\section{Output styling}{

The spicy table engine supports multiple padding options via \code{padding}:
\code{"compact"} (default), \code{"normal"}, and \code{"wide"}.
Horizontal and vertical rules can be customized, and colors are supported
when the terminal allows ANSI color output (via the \strong{crayon} package).
}

\seealso{
\code{\link[=print.spicy_freq_table]{print.spicy_freq_table()}} for the specialized frequency display method.
\code{\link[labelled:to_factor]{labelled::to_factor()}} and \code{\link[dplyr:pull]{dplyr::pull()}} for data transformations.
}
\concept{spicy tables}
\keyword{descriptive}
\keyword{frequency}
\keyword{internal}
\keyword{spicy}
\keyword{tables}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{assoc_measures}
\alias{assoc_measures}
\title{Association measures summary table}
\usage{
assoc_measures(
  x,
  type = c("all", "nominal", "ordinal"),
  conf_level = 0.95,
  digits = 3L
)
}
\arguments{
\item{x}{A contingency table (of class \code{table}).}

\item{type}{Which family of measures to compute:
\code{"all"} (default), \code{"nominal"}, or \code{"ordinal"}.}

\item{conf_level}{A number between 0 and 1 giving the confidence
level (default \code{0.95}). Set to \code{NULL} to omit the confidence
interval.}

\item{digits}{Number of decimal places used when printing the
result (default \code{3}).}
}
\value{
A data frame with columns \code{measure}, \code{estimate}, \code{se},
\code{ci_lower}, \code{ci_upper}, and \code{p_value}. For nominal measures
(Cramer's V, Phi, Contingency Coef.), the p-value comes from
the Pearson chi-squared test of independence. For all other
measures, it is a Wald z-test of H0: measure = 0.
}
\description{
\code{assoc_measures()} computes a range of association measures for a
two-way contingency table and returns them in a tidy data frame.
}
\details{
\code{type = "all"} (the default) returns all nominal and ordinal
measures. Use \code{type = "nominal"} or \code{type = "ordinal"} to
restrict the output to a single family.

The nominal family includes \code{\link[=cramer_v]{cramer_v()}}, \code{\link[=contingency_coef]{contingency_coef()}},
\code{\link[=lambda_gk]{lambda_gk()}}, \code{\link[=goodman_kruskal_tau]{goodman_kruskal_tau()}}, \code{\link[=uncertainty_coef]{uncertainty_coef()}},
and (for 2x2 tables) \code{\link[=phi]{phi()}} and \code{\link[=yule_q]{yule_q()}}.

The ordinal family includes \code{\link[=gamma_gk]{gamma_gk()}}, \code{\link[=kendall_tau_b]{kendall_tau_b()}},
\code{\link[=kendall_tau_c]{kendall_tau_c()}}, and \code{\link[=somers_d]{somers_d()}}.

Standard error formulas follow the DescTools implementations
(Signorell et al., 2024).
}
\examples{
tab <- table(sochealth$smoking, sochealth$education)
assoc_measures(tab)
assoc_measures(tab, type = "nominal")
assoc_measures(tab, type = "ordinal")

}
\references{
Agresti, A. (2002). \emph{Categorical Data Analysis} (2nd ed.). Wiley.

Liebetrau, A. M. (1983). \emph{Measures of Association}. Sage.

Signorell, A. et al. (2024). \emph{DescTools: Tools for Descriptive
Statistics}. R package.
}
\seealso{
\code{\link[=cramer_v]{cramer_v()}}, \code{\link[=gamma_gk]{gamma_gk()}}, \code{\link[=kendall_tau_b]{kendall_tau_b()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{yule_q}
\alias{yule_q}
\title{Yule's Q}
\usage{
yule_q(x, detail = FALSE, conf_level = 0.95, digits = 3L, .include_se = FALSE)
}
\arguments{
\item{x}{A contingency table (of class \code{table}).}

\item{detail}{Logical. If \code{FALSE} (default), return the estimate
as a numeric scalar. If \code{TRUE}, return a named numeric vector
including confidence interval and p-value.}

\item{conf_level}{A number between 0 and 1 giving the confidence
level (default \code{0.95}). Only used when \code{detail = TRUE}. Set
to \code{NULL} to omit the confidence interval.}

\item{digits}{Number of decimal places used when printing the
result (default \code{3}). Only affects the \code{detail = TRUE} output.}

\item{.include_se}{Internal parameter; do not use.}
}
\value{
Same structure as \code{\link[=cramer_v]{cramer_v()}}: a scalar when
\code{detail = FALSE}, a named vector when \code{detail = TRUE}.
The p-value tests H0: Q = 0 (Wald z-test).
}
\description{
\code{yule_q()} computes Yule's Q coefficient of association for a 2x2
contingency table.
}
\details{
For a 2x2 table with cells \eqn{a, b, c, d}, Yule's Q is
\eqn{Q = (ad - bc) / (ad + bc)}.
It is equivalent to the Goodman-Kruskal Gamma for 2x2 tables.
The asymptotic standard error is
\eqn{SE = 0.5 (1 - Q^2) \sqrt{1/a + 1/b + 1/c + 1/d}}.
Standard error formulas follow the DescTools implementations
(Signorell et al., 2024); see \code{\link[=cramer_v]{cramer_v()}} for full references.
}
\examples{
tab <- table(sochealth$smoking, sochealth$sex)
yule_q(tab)

}
\seealso{
\code{\link[=phi]{phi()}}, \code{\link[=gamma_gk]{gamma_gk()}}, \code{\link[=assoc_measures]{assoc_measures()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_categorical_print.R
\name{print.spicy_categorical_table}
\alias{print.spicy_categorical_table}
\title{Print method for categorical summary tables}
\usage{
\method{print}{spicy_categorical_table}(x, ...)
}
\arguments{
\item{x}{A \code{data.frame} of class \code{"spicy_categorical_table"} as returned by
\code{\link[=table_categorical]{table_categorical()}} with \code{output = "default"} and \code{styled = TRUE}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Formats and prints a \code{spicy_categorical_table} object as a styled ASCII table using
\code{\link[=spicy_print_table]{spicy_print_table()}}.
}
\seealso{
\code{\link[=table_categorical]{table_categorical()}}, \code{\link[=spicy_print_table]{spicy_print_table()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cross_tab.R
\name{print.spicy_cross_table_list}
\alias{print.spicy_cross_table_list}
\title{Internal print method for lists of cross-tab tables}
\usage{
\method{print}{spicy_cross_table_list}(x, ...)
}
\arguments{
\item{x}{A \code{spicy_cross_table_list} object.}

\item{...}{Additional arguments passed to individual print methods.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Prints each element of a \code{spicy_cross_table_list} object on its own,
inserting a blank line between tables.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cross_tab.R
\name{cross_tab}
\alias{cross_tab}
\title{Cross-tabulation}
\usage{
cross_tab(
  data,
  x,
  y = NULL,
  by = NULL,
  weights = NULL,
  rescale = FALSE,
  percent = c("none", "column", "row"),
  include_stats = TRUE,
  assoc_measure = c("auto", "cramer_v", "phi", "gamma", "tau_b", "tau_c", "somers_d",
    "lambda", "none"),
  assoc_ci = FALSE,
  correct = FALSE,
  simulate_p = FALSE,
  simulate_B = 2000,
  digits = NULL,
  styled = TRUE,
  show_n = TRUE
)
}
\arguments{
\item{data}{A data frame. Alternatively, a vector when using the
vector-based interface.}

\item{x}{Row variable (unquoted).}

\item{y}{Column variable (unquoted). Mandatory; for one-way tables, use \code{\link[=freq]{freq()}}.}

\item{by}{Optional grouping variable or expression. Can be a single variable
or a combination of multiple variables (e.g. \code{interaction(vs, am)}).}

\item{weights}{Optional numeric weights.}

\item{rescale}{Logical. If \code{FALSE} (the default), weights are used as-is.
If \code{TRUE}, rescales weights so total weighted N matches raw N.}

\item{percent}{One of \code{"none"} (the default), \code{"row"}, \code{"column"}.
Unique abbreviations are accepted (e.g. \code{"n"}, \code{"r"}, \code{"c"}).}

\item{include_stats}{Logical. If \code{TRUE} (the default), computes Chi-squared
and an association measure (see \code{assoc_measure}).}

\item{assoc_measure}{Character. Which association measure to report.
\code{"auto"} (default) selects Kendall's Tau-b when both variables are
ordered factors and Cramer's V otherwise. Other choices:
\code{"cramer_v"}, \code{"phi"}, \code{"gamma"}, \code{"tau_b"}, \code{"tau_c"},
\code{"somers_d"}, \code{"lambda"}, \code{"none"}.}

\item{assoc_ci}{Logical. If \code{TRUE}, includes the 95 percent confidence
interval of the association measure in the note. Defaults to \code{FALSE}.}

\item{correct}{Logical. If \code{FALSE} (the default), no continuity correction is
applied. If \code{TRUE}, applies Yates correction (only for 2x2 tables).}

\item{simulate_p}{Logical. If \code{FALSE} (the default), uses asymptotic p-values.
If \code{TRUE}, uses Monte Carlo simulation.}

\item{simulate_B}{Integer. Number of replicates for Monte Carlo simulation.
Defaults to \code{2000}.}

\item{digits}{Number of decimals. Defaults to \code{1} for percentages, \code{0} for counts.}

\item{styled}{Logical. If \code{TRUE} (the default), returns a \code{spicy_cross_table} object
(for formatted printing). If \code{FALSE}, returns a plain \code{data.frame}.}

\item{show_n}{Logical. If \code{TRUE} (the default), adds marginal N totals when
\code{percent != "none"}.}
}
\value{
A \code{data.frame}, list of data.frames, or \code{spicy_cross_table} object.
When \code{by} is used, returns a \code{spicy_cross_table_list}.
}
\description{
Computes a two-way cross-tabulation with optional weights, grouping
(including combinations of multiple variables), percentage displays,
and inferential statistics.

\code{cross_tab()} produces weighted or unweighted contingency tables with
row or column percentages, optional grouping via \code{by}, and associated
Chi-squared tests with an association measure and diagnostic information.

Both \code{x} and \code{y} variables are required. For one-way frequency tables,
use \code{\link[=freq]{freq()}} instead.
}
\section{Global Options}{


The function recognizes the following global options that modify its default behavior:
\itemize{
\item \strong{\code{options(spicy.percent = "column")}}
Sets the default percentage mode for all calls to \code{cross_tab()}.
Valid values are \code{"none"}, \code{"row"}, and \code{"column"}.
Equivalent to setting \code{percent = "column"} (or another choice) in each call.
\item \strong{\code{options(spicy.simulate_p = TRUE)}}
Enables Monte Carlo simulation for all Chi-squared tests by default.
Equivalent to setting \code{simulate_p = TRUE} in every call.
\item \strong{\code{options(spicy.rescale = TRUE)}}
Automatically rescales weights so that total weighted N equals the raw N.
Equivalent to setting \code{rescale = TRUE} in each call.
}

These options are convenient for users who wish to enforce consistent behavior
across multiple calls to \code{cross_tab()} and other spicy table functions.
They can be disabled or reset by setting them to \code{NULL}:
\code{options(spicy.percent = NULL, spicy.simulate_p = NULL, spicy.rescale = NULL)}.

Example:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{options(spicy.simulate_p = TRUE, spicy.rescale = TRUE)
cross_tab(sochealth, smoking, education, weights = weight)
}\if{html}{\out{</div>}}
}

\examples{
# Basic crosstab
cross_tab(sochealth, smoking, education)

# Column percentages
cross_tab(sochealth, smoking, education, percent = "column")

# Weighted (rescaled)
cross_tab(sochealth, smoking, education, weights = weight, rescale = TRUE)

# Grouped by sex
cross_tab(sochealth, smoking, education, by = sex)

# Grouped by combination of variables
cross_tab(sochealth, smoking, education, by = interaction(sex, age_group))

# Ordinal variables: auto-selects Kendall's Tau-b
cross_tab(sochealth, education, self_rated_health)

# 2x2 table with Yates correction
cross_tab(sochealth, smoking, physical_activity, correct = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{goodman_kruskal_tau}
\alias{goodman_kruskal_tau}
\title{Goodman-Kruskal's Tau}
\usage{
goodman_kruskal_tau(
  x,
  direction = c("row", "column"),
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
)
}
\arguments{
\item{x}{A contingency table (of class \code{table}).}

\item{direction}{Direction of prediction:
\code{"row"} (default, column predicts row) or
\code{"column"} (row predicts column).}

\item{detail}{Logical. If \code{FALSE} (default), return the estimate
as a numeric scalar. If \code{TRUE}, return a named numeric vector
including confidence interval and p-value.}

\item{conf_level}{A number between 0 and 1 giving the confidence
level (default \code{0.95}). Only used when \code{detail = TRUE}. Set
to \code{NULL} to omit the confidence interval.}

\item{digits}{Number of decimal places used when printing the
result (default \code{3}). Only affects the \code{detail = TRUE} output.}

\item{.include_se}{Internal parameter; do not use.}
}
\value{
Same structure as \code{\link[=cramer_v]{cramer_v()}}: a scalar when
\code{detail = FALSE}, a named vector when \code{detail = TRUE}.
The p-value tests H0: tau = 0 (Wald z-test).
}
\description{
\code{goodman_kruskal_tau()} computes Goodman-Kruskal's Tau, a
proportional reduction in error (PRE) measure for nominal
variables.
}
\details{
Unlike \code{\link[=lambda_gk]{lambda_gk()}}, Goodman-Kruskal's Tau uses all cell
frequencies rather than only the modal categories, making it
more sensitive to association patterns where lambda may be zero.
Standard error formulas follow the DescTools implementations
(Signorell et al., 2024); see \code{\link[=cramer_v]{cramer_v()}} for full references.
}
\examples{
tab <- table(sochealth$smoking, sochealth$education)
goodman_kruskal_tau(tab)
goodman_kruskal_tau(tab, direction = "column", detail = TRUE)

}
\seealso{
\code{\link[=lambda_gk]{lambda_gk()}}, \code{\link[=uncertainty_coef]{uncertainty_coef()}}, \code{\link[=assoc_measures]{assoc_measures()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tables_ascii.R
\name{build_ascii_table}
\alias{build_ascii_table}
\title{Build a formatted ASCII table}
\usage{
build_ascii_table(
  x,
  padding = c("compact", "normal", "wide"),
  first_column_line = TRUE,
  row_total_line = TRUE,
  column_total_line = TRUE,
  bottom_line = FALSE,
  lines_color = "darkgrey",
  align_left_cols = c(1L, 2L),
  align_center_cols = integer(0),
  group_sep_rows = integer(0),
  ...
)
}
\arguments{
\item{x}{A \code{data.frame} or \code{spicy_table} object containing the table to format.
Typically, this includes columns such as \emph{Category}, \emph{Values}, \emph{Freq.}, \emph{Percent}, etc.}

\item{padding}{Character string controlling horizontal spacing between columns:
\itemize{
\item \code{"compact"} — minimal spacing
\item \code{"normal"} — moderate spacing (default)
\item \code{"wide"} — extra spacing (for large displays or wide content)
}}

\item{first_column_line}{Logical. If \code{TRUE} (the default), a vertical separator
is drawn after the first column (useful for separating categories from data).}

\item{row_total_line, column_total_line}{Logical. Control horizontal rules
before total rows or columns. Both default to \code{TRUE}.}

\item{bottom_line}{Logical. If \code{FALSE} (the default), no closing line is drawn.
If \code{TRUE}, draws a closing line at the bottom of the table.}

\item{lines_color}{Character. Color used for table separators. Defaults to \code{"darkgrey"}.
The color is applied only when ANSI color support is available
(see \code{\link[crayon:has_color]{crayon::has_color()}}).}

\item{align_left_cols}{Integer vector of column indices to left-align.
Defaults to \code{c(1, 2)} for frequency tables (Category + Values).}

\item{align_center_cols}{Integer vector of column indices to
center-align. Defaults to \code{integer(0)} (no centered columns).
Columns not in \code{align_left_cols} or \code{align_center_cols} are
right-aligned.}

\item{group_sep_rows}{Integer vector of row indices before which a
light dashed separator line is drawn. Defaults to \code{integer(0)}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A single character string containing the full ASCII-formatted table,
suitable for direct printing with \code{cat()}.
}
\description{
Low-level internal function that constructs a visually aligned ASCII table
from a \code{data.frame}.
It supports Unicode characters, ANSI colors, dynamic width adjustment,
left/right alignment, and spacing control.
This function is primarily used internally by higher-level wrappers such as
\code{\link[=spicy_print_table]{spicy_print_table()}} or \code{\link[=print.spicy_freq_table]{print.spicy_freq_table()}}.
}
\details{
\code{build_ascii_table()} is the rendering engine that produces the aligned text
layout of \strong{spicy-formatted tables}.
It automatically detects cell widths (including colored text), inserts Unicode
separators, and applies padding for different display modes (\code{"compact"},
\code{"normal"}, \code{"wide"}).

For most users, this function should not be called directly. Instead, use
\code{\link[=spicy_print_table]{spicy_print_table()}} which adds headers, notes, and alignment logic
automatically.
}
\examples{
# Internal usage example (for developers)
df <- data.frame(
  Category = c("Valid", "", "Missing", "Total"),
  Values = c("Yes", "No", "NA", ""),
  Freq. = c(12, 8, 1, 21),
  Percent = c(57.1, 38.1, 4.8, 100.0)
)

cat(build_ascii_table(df, padding = "compact"))

}
\seealso{
\code{\link[=spicy_print_table]{spicy_print_table()}} for a user-facing wrapper that adds titles and notes.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mean_n.R
\name{mean_n}
\alias{mean_n}
\title{Row Means with Optional Minimum Valid Values}
\usage{
mean_n(
  data = NULL,
  select = tidyselect::everything(),
  exclude = NULL,
  min_valid = NULL,
  digits = NULL,
  regex = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{data}{A \code{data.frame} or \code{matrix}.}

\item{select}{Columns to include. If \code{regex = FALSE}, use tidyselect syntax (default: \code{tidyselect::everything()}).
If \code{regex = TRUE}, provide a regular expression pattern (character string).}

\item{exclude}{Columns to exclude (default: \code{NULL}).}

\item{min_valid}{Minimum number of valid (non-NA) values required per row.
If a proportion, it's applied to the number of selected columns.
Defaults to \code{NULL} (all values must be valid).}

\item{digits}{Optional number of decimal places to round the result.
Defaults to \code{NULL} (no rounding).}

\item{regex}{Logical. If \code{FALSE} (the default), uses tidyselect helpers.
If \code{TRUE}, the \code{select} argument is treated as a regular expression.}

\item{verbose}{Logical. If \code{FALSE} (the default), messages are suppressed.
If \code{TRUE}, prints a message about non-numeric columns excluded.}
}
\value{
A numeric vector of row-wise means.
}
\description{
\code{mean_n()} computes row means from a \code{data.frame} or \code{matrix}, handling missing values (\code{NA}s) automatically.
Row-wise means are calculated across selected numeric columns, with an optional condition on the minimum number (or proportion) of valid (non-missing) values required for a row to be included.
Non-numeric columns are excluded automatically and reported.
}
\examples{
library(dplyr)

# Create a simple numeric data frame
df <- tibble(
  var1 = c(10, NA, 30, 40, 50),
  var2 = c(5, NA, 15, NA, 25),
  var3 = c(NA, 30, 20, 50, 10)
)

# Compute row-wise mean (all values must be valid by default)
mean_n(df)

# Require at least 2 valid (non-NA) values per row
mean_n(df, min_valid = 2)

# Require at least 50\% valid (non-NA) values per row
mean_n(df, min_valid = 0.5)

# Round the result to 1 decimal
mean_n(df, digits = 1)

# Select specific columns
mean_n(df, select = c(var1, var2))

# Select specific columns using a pipe
df |>
  select(var1, var2) |>
  mean_n()

# Exclude a column
mean_n(df, exclude = "var3")

# Select columns ending with "1"
mean_n(df, select = ends_with("1"))

# Use with native pipe
df |> mean_n(select = starts_with("var"))

# Use inside dplyr::mutate()
df |> mutate(mean_score = mean_n(min_valid = 2))

# Select columns directly inside mutate()
df |> mutate(mean_score = mean_n(select = c(var1, var2), min_valid = 1))

# Select columns before mutate
df |>
  select(var1, var2) |>
  mutate(mean_score = mean_n(min_valid = 1))

# Show verbose processing info
df |> mutate(mean_score = mean_n(min_valid = 2, digits = 1, verbose = TRUE))

# Add character and grouping columns
df_mixed <- mutate(df,
  name = letters[1:5],
  group = c("A", "A", "B", "B", "A")
)
df_mixed

# Non-numeric columns are ignored
mean_n(df_mixed)

# Use within mutate() on mixed data
df_mixed |> mutate(mean_score = mean_n(select = starts_with("var")))

# Use everything() but exclude non-numeric columns manually
mean_n(df_mixed, select = everything(), exclude = "group")

# Select columns using regex
mean_n(df_mixed, select = "^var", regex = TRUE)
mean_n(df_mixed, select = "ar", regex = TRUE)

# Apply to a subset of rows (first 3)
df_mixed[1:3, ] |> mean_n(select = starts_with("var"))

# Store the result in a new column
df_mixed$mean_score <- mean_n(df_mixed, select = starts_with("var"))
df_mixed

# With a numeric matrix
mat <- matrix(c(1, 2, NA, 4, 5, NA, 7, 8, 9), nrow = 3, byrow = TRUE)
mat
mat |> mean_n(min_valid = 2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assoc.R
\name{kendall_tau_b}
\alias{kendall_tau_b}
\title{Kendall's Tau-b}
\usage{
kendall_tau_b(
  x,
  detail = FALSE,
  conf_level = 0.95,
  digits = 3L,
  .include_se = FALSE
)
}
\arguments{
\item{x}{A contingency table (of class \code{table}).}

\item{detail}{Logical. If \code{FALSE} (default), return the estimate
as a numeric scalar. If \code{TRUE}, return a named numeric vector
including confidence interval and p-value.}

\item{conf_level}{A number between 0 and 1 giving the confidence
level (default \code{0.95}). Only used when \code{detail = TRUE}. Set
to \code{NULL} to omit the confidence interval.}

\item{digits}{Number of decimal places used when printing the
result (default \code{3}). Only affects the \code{detail = TRUE} output.}

\item{.include_se}{Internal parameter; do not use.}
}
\value{
Same structure as \code{\link[=cramer_v]{cramer_v()}}: a scalar when
\code{detail = FALSE}, a named vector when \code{detail = TRUE}.
The p-value tests H0: tau-b = 0 (Wald z-test).
}
\description{
\code{kendall_tau_b()} computes Kendall's Tau-b for a two-way
contingency table of ordinal variables.
}
\details{
Kendall's Tau-b is computed as
\eqn{\tau_b = (C - D) / \sqrt{(n_0 - n_1)(n_0 - n_2)}},
where \eqn{n_0 = n(n-1)/2}, \eqn{n_1} is the number of
pairs tied on the row variable, and \eqn{n_2} is the number
tied on the column variable. Tau-b corrects for ties and is
appropriate for square tables.
Standard error formulas follow the DescTools implementations
(Signorell et al., 2024); see \code{\link[=cramer_v]{cramer_v()}} for full references.
}
\examples{
tab <- table(sochealth$education, sochealth$self_rated_health)
kendall_tau_b(tab)

}
\seealso{
\code{\link[=kendall_tau_c]{kendall_tau_c()}}, \code{\link[=gamma_gk]{gamma_gk()}}, \code{\link[=somers_d]{somers_d()}},
\code{\link[=assoc_measures]{assoc_measures()}}
}
